{% extends theme('base.html') %}
{% import 'macro/flask.html' as flask %}
{% import 'macro/form.html' as fui %}
{% import 'partial/login_field.html' as login_field %}

{% block container %}
    <div class="ui container" hx-boost="false">
        <div class="content">
            <div class="ui clearing segment">
                {% block header %}
                    {% if logo_url %}
                        <a href="{{ url_for('core.account.index') }}">
                            <img class="ui tiny centered image" src="{{ logo_url }}" alt="{{ website_name }}">
                        </a>
                    {% else %}
                        <i class="massive sign in icon image ui"></i>
                    {% endif %}

                    <h2 class="ui center aligned header">
                        <div class="content">
                            {{ _("Sign in at %(website)s", website=website_name) }}
                        </div>
                        <div class="sub header">{% trans %}Manage your information and your authorizations{% endtrans %}</div>
                    </h2>
                {% endblock %}

                {% block messages %}
                    {{ flask.messages() }}
                {% endblock %}

                {% call fui.render_form(form) %}
                    {% block fields %}
                        {% block login_field scoped %}{{ login_field.render_field(form.login, class="autofocus") }}{% endblock %}
                    {% endblock %}

                    {% block buttons %}
                        <div class="ui right aligned container">
                            <div class="ui stackable buttons">
                                {% if features.has_registration %}
                                    <a type="button" class="ui right floated button" href="{{ url_for('core.account.join') }}">{{ _("Create an account") }}</a>
                                {% endif %}
                                {% if features.has_smtp and features.has_password_recovery %}
                                    <a type="button" class="ui right floated button" href="{{ url_for('core.auth.forgotten') }}">{{ _("Forgotten password") }}</a>
                                {% endif %}
                                <button type="submit" name="answer" class="ui right floated primary button" hx-boost="false">{{ _("Continue") }}</button>
                            </div>
                        </div>
                    {% endblock %}
                {% endcall %}
            </div>
        </div>
    </div>
{% endblock %}
