{% extends theme('base.html') %}
{% import 'macro/form.html' as fui %}

{% block content %}
    <div class="loginform">
        {% block header %}
            <h3 class="ui top attached header">
                {% if logo_url %}
                    <img class="ui image" src="{{ logo_url }}">
                {% endif %}
                <div class="content">
                    {% trans %}Password reset{% endtrans %}
                </div>
            </h3>
        {% endblock %}

        <div class="ui attached clearing segment">
            {{ fui.render_form(form, _("Password reset"), action=url_for("core.auth.reset", user=user, hash=hash)) }}
        </div>
    </div>
{% endblock %}
