{% extends theme('base.html') %}
{% import "macro/table.html" as table %}

{%- block title -%}
    {%- trans %}Clients{% endtrans -%}
{%- endblock -%}

{% block submenu %}
    <a class="item" href="{{ url_for('core.admin.mail_index') }}">
        <i class="user mail icon"></i>
        {% trans %}Emails{% endtrans %}
    </a>
    <a class="active item" href="{{ url_for('oidc.clients.index') }}">
        <i class="th list icon"></i>
        {% trans %}Clients{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.clients.add') }}">
        <i class="plus icon"></i>
        {% trans %}Add a client{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.tokens.index') }}">
        <i class="key icon"></i>
        {% trans %}Tokens{% endtrans %}
    </a>
    <a class="item" href="{{ url_for('oidc.authorizations.index') }}">
        <i class="user secret icon"></i>
        {% trans %}Codes{% endtrans %}
    </a>
{% endblock %}

{% block content %}
    <div class="ui top attached segment">
        <h2 class="ui center aligned header">
            <div class="content">
                {% trans %}Clients{% endtrans %}
            </div>
        </h2>

        {{ table.search(table_form, "table.clients") }}
    </div>
    {% include "partial/client_list.html" %}
{% endblock %}
