from canaille.app.i18n import lazy_gettext as _

SCOPE_DETAILS = {
    "profile": (
        "id card outline",
        _("Info about yourself, such as your name."),
    ),
    "email": ("at", _("Your e-mail address.")),
    "address": ("envelope open outline", _("Your postal address.")),
    "phone": ("phone", _("Your phone number.")),
    "groups": ("users", _("Groups you belong to.")),
}
