/*
 * # Fomantic UI - 2.9.3
 * https://github.com/fomantic/Fomantic-UI
 * https://fomantic-ui.com/
 *
 * Copyright 2023 Contributors
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */
/*!
 * # Fomantic-UI 2.9.3 - Reset
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Reset
*******************************/

/* Border-Box */

*,
*::before,
*::after {
  box-sizing: inherit;
}

html {
  box-sizing: border-box;
}

/* iPad Input Shadows */

input[type="text"],
input[type="email"],
input[type="search"],
input[type="password"] {
  -webkit-appearance: none;
  -moz-appearance: none;
  /* mobile firefox too! */
}

/*! normalize.css v8.0.1 | MIT License | github.com/necolas/normalize.css */

/* Document
   ========================================================================== */

/**
 * 1. Correct the line height in all browsers.
 * 2. Prevent adjustments of font size after orientation changes in iOS.
 */

html {
  line-height: 1.15;
  /* 1 */
  -webkit-text-size-adjust: 100%;
  /* 2 */
}

/* Sections
   ========================================================================== */

/**
 * Remove the margin in all browsers.
 */

body {
  margin: 0;
}

/**
 * Render the `main` element consistently in IE.
 */

main {
  display: block;
}

/**
 * Correct the font size and margin on `h1` elements within `section` and
 * `article` contexts in Chrome, Firefox, and Safari.
 */

h1 {
  font-size: 2em;
  margin: 0.67em 0;
}

/* Grouping content
   ========================================================================== */

/**
 * 1. Add the correct box sizing in Firefox.
 * 2. Show the overflow in Edge and IE.
 */

hr {
  box-sizing: content-box;
  /* 1 */
  height: 0;
  /* 1 */
  overflow: visible;
  /* 2 */
}

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */

pre {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */
}

/* Text-level semantics
   ========================================================================== */

/**
 * Remove the gray background on active links in IE 10.
 */

a {
  background-color: transparent;
}

/**
 * 1. Remove the bottom border in Chrome 57-
 * 2. Add the correct text decoration in Chrome, Edge, IE, Opera, and Safari.
 */

abbr[title] {
  border-bottom: none;
  /* 1 */
  text-decoration: underline;
  /* 2 */
  -webkit-text-decoration: underline dotted;
  text-decoration: underline dotted;
  /* 2 */
}

/**
 * Add the correct font weight in Chrome, Edge, and Safari.
 */

b,
strong {
  font-weight: bolder;
}

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */

code,
kbd,
samp {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */
}

/**
 * Add the correct font size in all browsers.
 */

small {
  font-size: 80%;
}

/**
 * Prevent `sub` and `sup` elements from affecting the line height in
 * all browsers.
 */

sub,
sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline;
}

sub {
  bottom: -0.25em;
}

sup {
  top: -0.5em;
}

/* Embedded content
   ========================================================================== */

/**
 * Remove the border on images inside links in IE 10.
 */

img {
  border-style: none;
}

/* Forms
   ========================================================================== */

/**
 * 1. Change the font styles in all browsers.
 * 2. Remove the margin in Firefox and Safari.
 */

button,
input,
optgroup,
select,
textarea {
  font-family: inherit;
  /* 1 */
  font-size: 100%;
  /* 1 */
  line-height: 1.15;
  /* 1 */
  margin: 0;
  /* 2 */
}

/**
 * Show the overflow in IE.
 * 1. Show the overflow in Edge.
 */

button,
input {
  overflow: visible;
  /* 1 */
}

/**
 * Remove the inheritance of text transform in Edge, Firefox, and IE.
 * 1. Remove the inheritance of text transform in Firefox.
 */

button,
select {
  text-transform: none;
  /* 1 */
}

/**
 * Correct the inability to style clickable types in iOS and Safari.
 */

button,
[type="button"],
[type="reset"],
[type="submit"] {
  -webkit-appearance: button;
}

/**
 * Remove the inner border and padding in Firefox.
 */

button::-moz-focus-inner,
[type="button"]::-moz-focus-inner,
[type="reset"]::-moz-focus-inner,
[type="submit"]::-moz-focus-inner {
  border-style: none;
  padding: 0;
}

/**
 * Restore the focus styles unset by the previous rule.
 */

button:-moz-focusring,
[type="button"]:-moz-focusring,
[type="reset"]:-moz-focusring,
[type="submit"]:-moz-focusring {
  outline: 1px dotted ButtonText;
}

/**
 * Correct the padding in Firefox.
 */

fieldset {
  padding: 0.35em 0.75em 0.625em;
}

/**
 * 1. Correct the text wrapping in Edge and IE.
 * 2. Correct the color inheritance from `fieldset` elements in IE.
 * 3. Remove the padding so developers are not caught out when they zero out
 *    `fieldset` elements in all browsers.
 */

legend {
  box-sizing: border-box;
  /* 1 */
  color: inherit;
  /* 2 */
  display: table;
  /* 1 */
  max-width: 100%;
  /* 1 */
  padding: 0;
  /* 3 */
  white-space: normal;
  /* 1 */
}

/**
 * Add the correct vertical alignment in Chrome, Firefox, and Opera.
 */

progress {
  vertical-align: baseline;
}

/**
 * Remove the default vertical scrollbar in IE 10+.
 */

textarea {
  overflow: auto;
}

/**
 * 1. Add the correct box sizing in IE 10.
 * 2. Remove the padding in IE 10.
 */

[type="checkbox"],
[type="radio"] {
  box-sizing: border-box;
  /* 1 */
  padding: 0;
  /* 2 */
}

/**
 * Correct the cursor style of increment and decrement buttons in Chrome.
 */

[type="number"]::-webkit-inner-spin-button,
[type="number"]::-webkit-outer-spin-button {
  height: auto;
}

/**
 * 1. Correct the odd appearance in Chrome and Safari.
 * 2. Correct the outline style in Safari.
 */

[type="search"] {
  -webkit-appearance: textfield;
  /* 1 */
  outline-offset: -2px;
  /* 2 */
}

/**
 * Remove the inner padding in Chrome and Safari on macOS.
 */

[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none;
}

/**
 * 1. Correct the inability to style clickable types in iOS and Safari.
 * 2. Change font properties to `inherit` in Safari.
 */

::-webkit-file-upload-button {
  -webkit-appearance: button;
  /* 1 */
  font: inherit;
  /* 2 */
}

/* Interactive
   ========================================================================== */

/*
 * Add the correct display in Edge, IE 10+, and Firefox.
 */

details {
  display: block;
}

/*
 * Add the correct display in all browsers.
 */

summary {
  display: list-item;
}

/* Misc
   ========================================================================== */

/**
 * Add the correct display in IE 10+.
 */

template {
  display: none;
}

/**
 * Add the correct display in IE 10.
 */

[hidden] {
  display: none;
}

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Site
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Page
*******************************/

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/LatoLatin-Regular.woff2") format("woff2") , url("./themes/default/assets/fonts/LatoLatin-Regular.woff") format("woff");
  font-style: normal;
  font-weight: normal;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0000, U+000D, U+0020-007E, U+00A0-017F, U+0192, U+0218-021B, U+0237, U+02C6-02C7, U+02C9, U+02D8-02DD, U+0394, U+03A9, U+03BC, U+03C0, U+1E80-1E85, U+2010, U+2013-2014, U+2018-201A, U+201C-201E, U+2020-2022, U+2026, U+2030, U+2039-203A, U+2044, U+20A3-20A4, U+20A7, U+20AC, U+2113, U+2122, U+2126, U+212E, U+2202, U+2206, U+220F, U+2211-2212, U+2215, U+2219-221A, U+221E, U+222B, U+2248, U+2260, U+2264-2265, U+25CA, U+F8FF, U+FB00-FB04;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/LatoLatin-Bold.woff2") format("woff2") , url("./themes/default/assets/fonts/LatoLatin-Bold.woff") format("woff");
  font-style: normal;
  font-weight: bold;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0000, U+000D, U+0020-007E, U+00A0-017F, U+0192, U+0218-021B, U+0237, U+02C6-02C7, U+02C9, U+02D8-02DD, U+0394, U+03A9, U+03BC, U+03C0, U+1E80-1E85, U+2010, U+2013-2014, U+2018-201A, U+201C-201E, U+2020-2022, U+2026, U+2030, U+2039-203A, U+2044, U+20A3-20A4, U+20A7, U+20AC, U+2113, U+2122, U+2126, U+212E, U+2202, U+2206, U+220F, U+2211-2212, U+2215, U+2219-221A, U+221E, U+222B, U+2248, U+2260, U+2264-2265, U+25CA, U+F8FF, U+FB00-FB04;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/LatoLatin-Italic.woff2") format("woff2") , url("./themes/default/assets/fonts/LatoLatin-Italic.woff") format("woff");
  font-style: italic;
  font-weight: normal;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0000, U+000D, U+0020-007E, U+00A0-017F, U+0192, U+0218-021B, U+0237, U+02C6-02C7, U+02C9, U+02D8-02DD, U+0394, U+03A9, U+03BC, U+03C0, U+1E80-1E85, U+2010, U+2013-2014, U+2018-201A, U+201C-201E, U+2020-2022, U+2026, U+2030, U+2039-203A, U+2044, U+20A3-20A4, U+20A7, U+20AC, U+2113, U+2122, U+2126, U+212E, U+2202, U+2206, U+220F, U+2211-2212, U+2215, U+2219-221A, U+221E, U+222B, U+2248, U+2260, U+2264-2265, U+25CA, U+F8FF, U+FB00-FB04;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/LatoLatin-BoldItalic.woff2") format("woff2") , url("./themes/default/assets/fonts/LatoLatin-BoldItalic.woff") format("woff");
  font-style: italic;
  font-weight: bold;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0000, U+000D, U+0020-007E, U+00A0-017F, U+0192, U+0218-021B, U+0237, U+02C6-02C7, U+02C9, U+02D8-02DD, U+0394, U+03A9, U+03BC, U+03C0, U+1E80-1E85, U+2010, U+2013-2014, U+2018-201A, U+201C-201E, U+2020-2022, U+2026, U+2030, U+2039-203A, U+2044, U+20A3-20A4, U+20A7, U+20AC, U+2113, U+2122, U+2126, U+212E, U+2202, U+2206, U+220F, U+2211-2212, U+2215, U+2219-221A, U+221E, U+222B, U+2248, U+2260, U+2264-2265, U+25CA, U+F8FF, U+FB00-FB04;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/Lato-Regular.woff2") format("woff2") , url("./themes/default/assets/fonts/Lato-Regular.woff") format("woff");
  font-style: normal;
  font-weight: normal;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0180-0191, U+0193-0217, U+021C-0236, U+0238-02C5, U+02C8, U+02CA-02D7, U+02DE-036F, U+0374-0375, U+037A-037E, U+0384-038A, U+038C, U+038E-0393, U+0395-03A1, U+03A3-03A8, U+03AA-03BB, U+03BD-03BF, U+03C1-03CE, U+03D0-0486, U+0488-0513, U+1D00-1DCA, U+1DFE-1E7F, U+1E86-1E9B, U+1E9E, U+1EA0-1EF9, U+1F00-1F15, U+1F18-1F1D, U+1F20-1F45, U+1F48-1F4D, U+1F50-1F57, U+1F59, U+1F5B, U+1F5D, U+1F5F-1F7D, U+1F80-1FB4, U+1FB6-1FC4, U+1FC6-1FD3, U+1FD6-1FDB, U+1FDD-1FEF, U+1FF2-1FF4, U+1FF6-1FFE, U+2000-200F, U+2012, U+2015-2017, U+201B, U+201F, U+202F, U+2034, U+203C-203E, U+205E-205F, U+2070-2071, U+2074-2094, U+20A0-20A2, U+20A5-20A6, U+20A8-20AB, U+20AD-20B5, U+20B8-20BA, U+20DD, U+2105, U+2116-2117, U+2120, U+2132, U+214D-214E, U+2153-215F, U+2183-2184, U+2190-2199, U+21A8, U+221F, U+2229, U+2261, U+2302, U+2310, U+2320-2321, U+2460-2473, U+24EA-24F4, U+24FF-2500, U+2502, U+250C, U+2510, U+2514, U+2518, U+2C60-2C6C, U+2C74-2C77;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/Lato-Bold.woff2") format("woff2") , url("./themes/default/assets/fonts/Lato-Bold.woff") format("woff");
  font-style: normal;
  font-weight: bold;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0180-0191, U+0193-0217, U+021C-0236, U+0238-02C5, U+02C8, U+02CA-02D7, U+02DE-036F, U+0374-0375, U+037A-037E, U+0384-038A, U+038C, U+038E-0393, U+0395-03A1, U+03A3-03A8, U+03AA-03BB, U+03BD-03BF, U+03C1-03CE, U+03D0-0486, U+0488-0513, U+1D00-1DCA, U+1DFE-1E7F, U+1E86-1E9B, U+1E9E, U+1EA0-1EF9, U+1F00-1F15, U+1F18-1F1D, U+1F20-1F45, U+1F48-1F4D, U+1F50-1F57, U+1F59, U+1F5B, U+1F5D, U+1F5F-1F7D, U+1F80-1FB4, U+1FB6-1FC4, U+1FC6-1FD3, U+1FD6-1FDB, U+1FDD-1FEF, U+1FF2-1FF4, U+1FF6-1FFE, U+2000-200F, U+2012, U+2015-2017, U+201B, U+201F, U+202F, U+2034, U+203C-203E, U+205E-205F, U+2070-2071, U+2074-2094, U+20A0-20A2, U+20A5-20A6, U+20A8-20AB, U+20AD-20B5, U+20B8-20BA, U+20DD, U+2105, U+2116-2117, U+2120, U+2132, U+214D-214E, U+2153-215F, U+2183-2184, U+2190-2199, U+21A8, U+221F, U+2229, U+2261, U+2302, U+2310, U+2320-2321, U+2460-2473, U+24EA-24F4, U+24FF-2500, U+2502, U+250C, U+2510, U+2514, U+2518, U+2C60-2C6C, U+2C74-2C77;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/Lato-Italic.woff2") format("woff2") , url("./themes/default/assets/fonts/Lato-Italic.woff") format("woff");
  font-style: italic;
  font-weight: normal;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0180-0191, U+0193-0217, U+021C-0236, U+0238-02C5, U+02C8, U+02CA-02D7, U+02DE-036F, U+0374-0375, U+037A-037E, U+0384-038A, U+038C, U+038E-0393, U+0395-03A1, U+03A3-03A8, U+03AA-03BB, U+03BD-03BF, U+03C1-03CE, U+03D0-0486, U+0488-0513, U+1D00-1DCA, U+1DFE-1E7F, U+1E86-1E9B, U+1E9E, U+1EA0-1EF9, U+1F00-1F15, U+1F18-1F1D, U+1F20-1F45, U+1F48-1F4D, U+1F50-1F57, U+1F59, U+1F5B, U+1F5D, U+1F5F-1F7D, U+1F80-1FB4, U+1FB6-1FC4, U+1FC6-1FD3, U+1FD6-1FDB, U+1FDD-1FEF, U+1FF2-1FF4, U+1FF6-1FFE, U+2000-200F, U+2012, U+2015-2017, U+201B, U+201F, U+202F, U+2034, U+203C-203E, U+205E-205F, U+2070-2071, U+2074-2094, U+20A0-20A2, U+20A5-20A6, U+20A8-20AB, U+20AD-20B5, U+20B8-20BA, U+20DD, U+2105, U+2116-2117, U+2120, U+2132, U+214D-214E, U+2153-215F, U+2183-2184, U+2190-2199, U+21A8, U+221F, U+2229, U+2261, U+2302, U+2310, U+2320-2321, U+2460-2473, U+24EA-24F4, U+24FF-2500, U+2502, U+250C, U+2510, U+2514, U+2518, U+2C60-2C6C, U+2C74-2C77;
}

@font-face {
  font-family: "Lato";
  src: url("./themes/default/assets/fonts/Lato-BoldItalic.woff2") format("woff2") , url("./themes/default/assets/fonts/Lato-BoldItalic.woff") format("woff");
  font-style: italic;
  font-weight: bold;
  font-display: swap;
  text-rendering: optimizeLegibility;
  unicode-range: U+0180-0191, U+0193-0217, U+021C-0236, U+0238-02C5, U+02C8, U+02CA-02D7, U+02DE-036F, U+0374-0375, U+037A-037E, U+0384-038A, U+038C, U+038E-0393, U+0395-03A1, U+03A3-03A8, U+03AA-03BB, U+03BD-03BF, U+03C1-03CE, U+03D0-0486, U+0488-0513, U+1D00-1DCA, U+1DFE-1E7F, U+1E86-1E9B, U+1E9E, U+1EA0-1EF9, U+1F00-1F15, U+1F18-1F1D, U+1F20-1F45, U+1F48-1F4D, U+1F50-1F57, U+1F59, U+1F5B, U+1F5D, U+1F5F-1F7D, U+1F80-1FB4, U+1FB6-1FC4, U+1FC6-1FD3, U+1FD6-1FDB, U+1FDD-1FEF, U+1FF2-1FF4, U+1FF6-1FFE, U+2000-200F, U+2012, U+2015-2017, U+201B, U+201F, U+202F, U+2034, U+203C-203E, U+205E-205F, U+2070-2071, U+2074-2094, U+20A0-20A2, U+20A5-20A6, U+20A8-20AB, U+20AD-20B5, U+20B8-20BA, U+20DD, U+2105, U+2116-2117, U+2120, U+2132, U+214D-214E, U+2153-215F, U+2183-2184, U+2190-2199, U+21A8, U+221F, U+2229, U+2261, U+2302, U+2310, U+2320-2321, U+2460-2473, U+24EA-24F4, U+24FF-2500, U+2502, U+250C, U+2510, U+2514, U+2518, U+2C60-2C6C, U+2C74-2C77;
}

html,
body {
  height: 100%;
}

html {
  font-size: 14px;
}

body {
  margin: 0;
  padding: 0;
  overflow-x: hidden;
  min-width: 320px;
  background: #fff;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 14px;
  line-height: 1.4285em;
  color: rgba(0, 0, 0, 0.87);
}

/*******************************
             Headers
*******************************/

h1,
h2,
h3,
h4,
h5 {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  line-height: 1.28571429em;
  margin: calc(2rem - 0.1428571428571429em) 0 1rem;
  font-weight: bold;
  padding: 0;
}

h1 {
  min-height: 1rem;
  font-size: 2rem;
}

h2 {
  font-size: 1.71428571rem;
}

h3 {
  font-size: 1.28571429rem;
}

h4 {
  font-size: 1.07142857rem;
}

h5 {
  font-size: 1rem;
}

h1:first-child,
h2:first-child,
h3:first-child,
h4:first-child,
h5:first-child {
  margin-top: 0;
}

h1:last-child,
h2:last-child,
h3:last-child,
h4:last-child,
h5:last-child {
  margin-bottom: 0;
}

/*******************************
             Text
*******************************/

p {
  margin: 0 0 1em;
  line-height: 1.4285em;
}

p:first-child {
  margin-top: 0;
}

p:last-child {
  margin-bottom: 0;
}

/* -------------------
        Links
-------------------- */

a {
  color: #4183c4;
  text-decoration: none;
}

a:hover {
  color: #1e70bf;
  text-decoration: none;
}

/*******************************
         Scrollbars
*******************************/

/* Force Simple Scrollbars */

body ::-webkit-scrollbar {
  -webkit-appearance: none;
  width: 10px;
  height: 10px;
}

body ::-webkit-scrollbar-track {
  background: rgba(0, 0, 0, 0.1);
  border-radius: 0;
}

body ::-webkit-scrollbar-thumb {
  cursor: pointer;
  border-radius: 5px;
  background: rgba(0, 0, 0, 0.25);
  -webkit-transition: color 0.2s ease;
  transition: color 0.2s ease;
}

body ::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(0, 0, 0, 0.15);
}

body ::-webkit-scrollbar-thumb:hover {
  background: rgba(128, 135, 139, 0.8);
}

body .ui {
  /* IE11 */
  scrollbar-face-color: #bfbfbf;
  scrollbar-shadow-color: #bfbfbf;
  scrollbar-track-color: #e6e6e6;
  scrollbar-arrow-color: #e6e6e6;
  /* firefox: first color thumb, second track */
  scrollbar-color: rgba(0, 0, 0, 0.25) rgba(0, 0, 0, 0.1);
  scrollbar-width: thin;
}

/* Inverted UI */

body .ui.inverted:not(.dimmer)::-webkit-scrollbar-track {
  background: rgba(255, 255, 255, 0.1);
}

body .ui.inverted:not(.dimmer)::-webkit-scrollbar-thumb {
  background: rgba(255, 255, 255, 0.25);
}

body .ui.inverted:not(.dimmer)::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(255, 255, 255, 0.15);
}

body .ui.inverted:not(.dimmer)::-webkit-scrollbar-thumb:hover {
  background: rgba(255, 255, 255, 0.35);
}

body .ui.inverted:not(.dimmer) {
  /* IE11 */
  scrollbar-face-color: #656565;
  scrollbar-shadow-color: #656565;
  scrollbar-track-color: #323232;
  scrollbar-arrow-color: #323232;
  /* firefox: first color thumb, second track */
  scrollbar-color: rgba(255, 255, 255, 0.25) rgba(255, 255, 255, 0.1);
}

/*******************************
          Highlighting
*******************************/

/* Site */

::selection {
  background-color: #cce2ff;
  color: rgba(0, 0, 0, 0.87);
}

/* Form */

textarea::selection,
input::selection {
  background-color: rgba(100, 100, 100, 0.4);
  color: rgba(0, 0, 0, 0.87);
}

/*******************************
        Global Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Button
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Button
*******************************/

.ui.button {
  cursor: pointer;
  display: inline-block;
  min-height: 1em;
  outline: none;
  border: none;
  vertical-align: baseline;
  background: #e0e1e2 none;
  color: rgba(0, 0, 0, 0.6);
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  margin: 0 0.25em 0 0;
  padding: 0.78571429em 1.5em 0.78571429em;
  text-transform: none;
  text-shadow: none;
  font-weight: bold;
  line-height: 1em;
  font-style: normal;
  text-align: center;
  text-decoration: none;
  border-radius: 0.28571429rem;
  box-shadow: 0 0 0 1px transparent inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  transition: opacity 0.1s ease, background-color 0.1s ease, color 0.1s ease, box-shadow 0.1s ease, background 0.1s ease;
  will-change: auto;
  -webkit-tap-highlight-color: transparent;
}

/*******************************
            States
*******************************/

/* --------------
      Hover
--------------- */

.ui.button:hover {
  background-color: #cacbcd;
  background-image: none;
  box-shadow: 0 0 0 1px transparent inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
  color: rgba(0, 0, 0, 0.8);
}

.ui.button:hover .icon {
  opacity: 0.85;
}

/* --------------
      Focus
--------------- */

.ui.button:focus {
  background-color: #cacbcd;
  color: rgba(0, 0, 0, 0.8);
  background-image: none;
  box-shadow: "";
}

.ui.button:focus .icon {
  opacity: 0.85;
}

/* --------------
      Down
--------------- */

.ui.button:active,
.ui.active.button:active {
  background-color: #babbbc;
  background-image: "";
  color: rgba(0, 0, 0, 0.9);
  box-shadow: none;
}

/* --------------
     Active
--------------- */

.ui.active.button {
  background-color: #c0c1c2;
  background-image: none;
  box-shadow: 0 0 0 1px transparent inset;
  color: rgba(0, 0, 0, 0.95);
}

.ui.active.button:hover {
  background-color: #c0c1c2;
  background-image: none;
  color: rgba(0, 0, 0, 0.95);
}

.ui.active.button:active {
  background-color: #c0c1c2;
  background-image: none;
}

/* --------------
        Loading
    --------------- */

/* Specificity hack */

.ui.ui.ui.ui.ui.ui.loading.button {
  position: relative;
  cursor: default;
  text-shadow: none !important;
  color: transparent;
  opacity: 1;
  pointer-events: auto;
  transition: all 0s linear, opacity 0.1s ease;
}

.ui.loading.button::before {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.15);
}

.ui.loading.button::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  border-radius: 500rem;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid currentColor;
  color: #fff;
  box-shadow: 0 0 0 1px transparent;
}

.ui.ui.ui.loading.button .label {
  background-color: transparent;
  border-color: transparent;
  color: transparent;
}

.ui.labeled.icon.loading.button .icon {
  background-color: transparent;
  box-shadow: none;
}

.ui.basic.loading.button:not(.inverted)::before {
  border-color: rgba(0, 0, 0, 0.1);
}

.ui.basic.loading.button:not(.inverted)::after {
  border-color: #767676;
}

/* -------------------
          Disabled
    -------------------- */

.ui.buttons .disabled.button:not(.basic),
.ui.disabled.button,
.ui.button:disabled,
.ui.disabled.button:hover,
.ui.disabled.active.button {
  cursor: default;
  opacity: 0.45 !important;
  background-image: none;
  box-shadow: none;
  pointer-events: none !important;
}

/* Basic Group With Disabled */

.ui.basic.buttons .ui.disabled.button {
  border-color: rgba(34, 36, 38, 0.5);
}

/*******************************
             Types
*******************************/

/* -------------------
           Animated
    -------------------- */

.ui.animated.button {
  position: relative;
  overflow: hidden;
  padding-right: 0 !important;
  vertical-align: middle;
  z-index: 1;
}

.ui.animated.button .content {
  will-change: transform, opacity;
}

.ui.animated.button .visible.content {
  position: relative;
  margin-right: 1.5em;
}

.ui.animated.button .hidden.content {
  position: absolute;
  width: 100%;
}

/* Horizontal */

.ui.animated.button .visible.content,
.ui.animated.button .hidden.content {
  transition: right 0.3s ease 0s;
}

.ui.animated.button .visible.content {
  left: auto;
  right: 0;
}

.ui.animated.button .hidden.content {
  top: 50%;
  left: auto;
  right: -100%;
  margin-top: -0.5em;
}

.ui.animated.button:focus .visible.content,
.ui.animated.button:hover .visible.content {
  left: auto;
  right: 200%;
}

.ui.animated.button:focus .hidden.content,
.ui.animated.button:hover .hidden.content {
  left: auto;
  right: 0;
}

/* Vertical */

.ui.vertical.animated.button .visible.content,
.ui.vertical.animated.button .hidden.content {
  transition: top 0.3s ease, transform 0.3s ease;
}

.ui.vertical.animated.button .visible.content {
  transform: translateY(0);
  right: auto;
}

.ui.vertical.animated.button .hidden.content {
  top: -50%;
  left: 0;
  right: auto;
}

.ui.vertical.animated.button:focus .visible.content,
.ui.vertical.animated.button:hover .visible.content {
  transform: translateY(200%);
  right: auto;
}

.ui.vertical.animated.button:focus .hidden.content,
.ui.vertical.animated.button:hover .hidden.content {
  top: 50%;
  right: auto;
}

/* Fade */

.ui.fade.animated.button .visible.content,
.ui.fade.animated.button .hidden.content {
  transition: opacity 0.3s ease, transform 0.3s ease;
}

.ui.fade.animated.button .visible.content {
  left: auto;
  right: auto;
  opacity: 1;
  transform: scale(1);
}

.ui.fade.animated.button .hidden.content {
  opacity: 0;
  left: 0;
  right: auto;
  transform: scale(1.5);
}

.ui.fade.animated.button:focus .visible.content,
.ui.fade.animated.button:hover .visible.content {
  left: auto;
  right: auto;
  opacity: 0;
  transform: scale(0.75);
}

.ui.fade.animated.button:focus .hidden.content,
.ui.fade.animated.button:hover .hidden.content {
  left: 0;
  right: auto;
  opacity: 1;
  transform: scale(1);
}

/* -------------------
           Inverted
    -------------------- */

.ui.inverted.button {
  box-shadow: 0 0 0 2px #fff inset;
  background: transparent none;
  color: #fff;
  text-shadow: none !important;
}

/* Group */

.ui.inverted.buttons .button {
  margin: 0 0 0 -2px;
}

.ui.inverted.buttons .button:first-child {
  margin-left: 0;
}

.ui.inverted.vertical.buttons .button {
  margin: 0 0 -2px 0;
}

.ui.inverted.vertical.buttons .button:first-child {
  margin-top: 0;
}

/* States */

/* Hover */

.ui.inverted.button:hover {
  background: #fff;
  box-shadow: 0 0 0 2px #fff inset;
  color: rgba(0, 0, 0, 0.8);
}

/* Active / Focus */

.ui.inverted.button:focus,
.ui.inverted.button.active {
  background: #fff;
  box-shadow: 0 0 0 2px #fff inset;
  color: rgba(0, 0, 0, 0.8);
}

/* Active Focus */

.ui.inverted.button.active:focus {
  background: #dcddde;
  box-shadow: 0 0 0 2px #dcddde inset;
  color: rgba(0, 0, 0, 0.8);
}

/* -------------------
        Labeled Button
    -------------------- */

.ui.labeled.button:not(.icon) {
  display: inline-flex;
  flex-direction: row;
  background: none;
  padding: 0 !important;
  border: none;
  box-shadow: none;
}

.ui.labeled.button > .button {
  margin: 0;
}

.ui.labeled.button > .label {
  display: flex;
  align-items: center;
  margin: 0 0 0 -1px !important;
  font-size: 1em;
  padding: "";
  border-color: rgba(34, 36, 38, 0.15);
}

/* Tag */

.ui.labeled.button > .tag.label::before {
  width: 1.85em;
  height: 1.85em;
}

/* Right */

.ui.labeled.button:not([class*="left labeled"]) > .button {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.labeled.button:not([class*="left labeled"]) > .label {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

/* Left Side */

.ui[class*="left labeled"].button > .button {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui[class*="left labeled"].button > .label {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

/* -------------------
           Social
    -------------------- */

/* Facebook */

.ui.facebook.button {
  background-color: #3b5998;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.facebook.button:hover {
  background-color: #304d8a;
  color: #fff;
  text-shadow: none;
}

.ui.facebook.button:active {
  background-color: #2d4373;
  color: #fff;
  text-shadow: none;
}

/* Twitter */

.ui.twitter.button {
  background-color: #1da1f2;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.twitter.button:hover {
  background-color: #0298f3;
  color: #fff;
  text-shadow: none;
}

.ui.twitter.button:active {
  background-color: #0c85d0;
  color: #fff;
  text-shadow: none;
}

/* Google Plus */

.ui.google.plus.button {
  background-color: #dd4b39;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.google.plus.button:hover {
  background-color: #e0321c;
  color: #fff;
  text-shadow: none;
}

.ui.google.plus.button:active {
  background-color: #c23321;
  color: #fff;
  text-shadow: none;
}

/* Linked In */

.ui.linkedin.button {
  background-color: #0077b5;
  color: #fff;
  text-shadow: none;
}

.ui.linkedin.button:hover {
  background-color: #00669c;
  color: #fff;
  text-shadow: none;
}

.ui.linkedin.button:active {
  background-color: #005582;
  color: #fff;
  text-shadow: none;
}

/* YouTube */

.ui.youtube.button {
  background-color: #f00;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.youtube.button:hover {
  background-color: #e60000;
  color: #fff;
  text-shadow: none;
}

.ui.youtube.button:active {
  background-color: #cc0000;
  color: #fff;
  text-shadow: none;
}

/* Instagram */

.ui.instagram.button {
  background-color: #49769c;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.instagram.button:hover {
  background-color: #3d698e;
  color: #fff;
  text-shadow: none;
}

.ui.instagram.button:active {
  background-color: #395c79;
  color: #fff;
  text-shadow: none;
}

/* Pinterest */

.ui.pinterest.button {
  background-color: #bd081c;
  color: #fff;
  text-shadow: none;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.pinterest.button:hover {
  background-color: #ac0013;
  color: #fff;
  text-shadow: none;
}

.ui.pinterest.button:active {
  background-color: #8c0615;
  color: #fff;
  text-shadow: none;
}

/* VK */

.ui.vk.button {
  background-color: #45668e;
  color: #fff;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.vk.button:hover {
  background-color: #395980;
  color: #fff;
}

.ui.vk.button:active {
  background-color: #344d6c;
  color: #fff;
}

/* WhatsApp */

.ui.whatsapp.button {
  background-color: #25d366;
  color: #fff;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.whatsapp.button:hover {
  background-color: #19c55a;
  color: #fff;
}

.ui.whatsapp.button:active {
  background-color: #1da851;
  color: #fff;
}

/* Telegram */

.ui.telegram.button {
  background-color: #08c;
  color: #fff;
  background-image: none;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.telegram.button:hover {
  background-color: #0077b3;
  color: #fff;
}

.ui.telegram.button:active {
  background-color: #006699;
  color: #fff;
}

/* --------------
     Icon
--------------- */

.ui.button > .icon:not(.button) {
  height: auto;
  opacity: 0.8;
  transition: opacity 0.1s ease;
  color: "";
}

.ui.button:not(.icon) > .icon:not(.button):not(.dropdown),
.ui.button:not(.icon) > .icons:not(.button):not(.dropdown) {
  margin: 0 0.42857143em 0 -0.21428571em;
  vertical-align: baseline;
}

.ui.button:not(.icon) > .icons:not(.button):not(.dropdown) > .icon {
  vertical-align: baseline;
}

.ui.button:not(.icon) > .right.icon:not(.button):not(.dropdown) {
  margin: 0 -0.21428571em 0 0.42857143em;
}

/*******************************
           Variations
*******************************/

/* --------------
       Stackable
    --------------- */

/* Tablet Or Below */

@media only screen and (max-width: 767.98px) {
  .ui.stackable.buttons {
    flex-direction: column;
    width: 100%;
  }

  .ui.stackable.buttons .button:first-child {
    border-bottom-left-radius: 0;
    border-top-right-radius: 0.28571429rem;
  }

  .ui.stackable.buttons .button:last-child {
    border-bottom-left-radius: 0.28571429rem;
    border-top-right-radius: 0;
  }

  .ui.stackable.buttons .button:only-child {
    border-radius: 0.28571429rem;
  }
}

/* -------------------
           Floated
    -------------------- */

.ui[class*="left floated"].buttons,
.ui[class*="left floated"].button {
  float: left;
  margin-left: 0;
  margin-right: 0.25em;
}

.ui[class*="right floated"].buttons,
.ui[class*="right floated"].button {
  float: right;
  margin-right: 0;
  margin-left: 0.25em;
}

/* -------------------
           Compact
    -------------------- */

.ui.compact.buttons .button,
.ui.compact.button {
  padding: 0.58928571em 1.125em 0.58928571em;
}

.ui.compact.icon.buttons .button,
.ui.compact.icon.button {
  padding: 0.58928571em 0.58928571em 0.58928571em;
}

.ui.compact.labeled.icon.buttons .button,
.ui.compact.labeled.icon.button {
  padding: 0.58928571em 3.69642857em 0.58928571em;
}

.ui.compact.labeled.icon.buttons .button > .icon,
.ui.compact.labeled.icon.button > .icon {
  padding: 0.58928571em 0;
}

/* -------------------
        Sizes
-------------------- */

.ui.buttons .button,
.ui.buttons .or,
.ui.button {
  font-size: 1rem;
}

.ui.mini.buttons .dropdown,
.ui.mini.buttons .dropdown .menu > .item,
.ui.mini.buttons .button,
.ui.mini.buttons .or,
.ui.ui.ui.ui.mini.button {
  font-size: 0.78571429rem;
}

.ui.tiny.buttons .dropdown,
.ui.tiny.buttons .dropdown .menu > .item,
.ui.tiny.buttons .button,
.ui.tiny.buttons .or,
.ui.ui.ui.ui.tiny.button {
  font-size: 0.85714286rem;
}

.ui.small.buttons .dropdown,
.ui.small.buttons .dropdown .menu > .item,
.ui.small.buttons .button,
.ui.small.buttons .or,
.ui.ui.ui.ui.small.button {
  font-size: 0.92857143rem;
}

.ui.large.buttons .dropdown,
.ui.large.buttons .dropdown .menu > .item,
.ui.large.buttons .button,
.ui.large.buttons .or,
.ui.ui.ui.ui.large.button {
  font-size: 1.14285714rem;
}

.ui.big.buttons .dropdown,
.ui.big.buttons .dropdown .menu > .item,
.ui.big.buttons .button,
.ui.big.buttons .or,
.ui.ui.ui.ui.big.button {
  font-size: 1.28571429rem;
}

.ui.huge.buttons .dropdown,
.ui.huge.buttons .dropdown .menu > .item,
.ui.huge.buttons .button,
.ui.huge.buttons .or,
.ui.ui.ui.ui.huge.button {
  font-size: 1.42857143rem;
}

.ui.massive.buttons .dropdown,
.ui.massive.buttons .dropdown .menu > .item,
.ui.massive.buttons .button,
.ui.massive.buttons .or,
.ui.ui.ui.ui.massive.button {
  font-size: 1.71428571rem;
}

/* --------------
        Icon Only
    --------------- */

.ui.icon.buttons .button,
.ui.icon.button:not(.animated):not(.compact):not(.labeled) {
  padding: 0.78571429em 0.78571429em 0.78571429em;
}

.ui.animated.icon.button > .content > .icon,
.ui.icon.buttons .button > .icon,
.ui.icon.button > .icon {
  opacity: 0.9;
  margin: 0 !important;
  vertical-align: top;
}

.ui.animated.button > .content > .icon {
  vertical-align: top;
}

/* -------------------
            Basic
    -------------------- */

.ui.basic.buttons .button,
.ui.basic.button {
  background: transparent none;
  color: rgba(0, 0, 0, 0.6);
  font-weight: normal;
  border-radius: 0.28571429rem;
  text-transform: none;
  text-shadow: none !important;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons {
  box-shadow: none;
  border: 1px solid rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
  border-right: none;
}

.ui.basic.buttons .button {
  border-radius: 0;
}

.ui.basic.buttons .button:hover,
.ui.basic.button:hover {
  background: #fff;
  color: rgba(0, 0, 0, 0.8);
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.35) inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons .button:focus,
.ui.basic.button:focus {
  background: #fff;
  color: rgba(0, 0, 0, 0.8);
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.35) inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons .button:active,
.ui.basic.button:active {
  background: #f8f8f8;
  color: rgba(0, 0, 0, 0.9);
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.15) inset, 0 1px 4px 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons .active.button,
.ui.basic.active.button {
  background: rgba(0, 0, 0, 0.05);
  box-shadow: "";
  color: rgba(0, 0, 0, 0.95);
}

.ui.basic.buttons .active.button:hover,
.ui.basic.active.button:hover {
  background-color: rgba(0, 0, 0, 0.05);
}

.ui.basic.buttons .button:hover {
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.35) inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons .button:active {
  box-shadow: 0 0 0 1px rgba(0, 0, 0, 0.15) inset, 0 1px 4px 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.basic.buttons .active.button {
  box-shadow: "";
}

/* Standard Basic Inverted */

.ui.basic.inverted.buttons .button,
.ui.basic.inverted.button {
  background-color: transparent;
  color: #f9fafb;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
}

.ui.basic.inverted.buttons .button:hover,
.ui.basic.inverted.button:hover {
  color: #fff;
  box-shadow: 0 0 0 2px #ffffff inset;
}

.ui.basic.inverted.buttons .button:focus,
.ui.basic.inverted.button:focus {
  color: #fff;
  box-shadow: 0 0 0 2px #ffffff inset;
}

.ui.basic.inverted.buttons .button:active,
.ui.basic.inverted.button:active {
  background-color: rgba(255, 255, 255, 0.08);
  color: #fff;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.9) inset;
}

.ui.basic.inverted.buttons .active.button,
.ui.basic.inverted.active.button {
  background-color: rgba(255, 255, 255, 0.08);
  color: #fff;
  text-shadow: none;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.7) inset;
}

.ui.basic.inverted.buttons .active.button:hover,
.ui.basic.inverted.active.button:hover {
  background-color: rgba(255, 255, 255, 0.15);
  box-shadow: 0 0 0 2px #ffffff inset;
}

/* Basic Group */

.ui.basic.buttons:not(.inverted) .button:not(.basic) {
  border-right: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: none;
}

.ui.basic.vertical.buttons .button {
  border-left: none;
  border-left-width: 0;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.basic.vertical.buttons:not(.spaced) .button:first-child {
  border-top: none;
}

/* -------------------
           Tertiary
    -------------------- */

/* Overline Mixin */

.ui.tertiary.button {
  transition: color 0.1s ease !important;
  border-radius: 0;
  margin: 0.28571429em 0.25em 0.28571429em 0 !important;
  padding: 0.5em 0.5em !important;
  box-shadow: none;
  color: rgba(0, 0, 0, 0.6);
  background: none;
}

.ui.tertiary.button:hover {
  box-shadow: inset 0 -0.2em 0 #666666;
  color: #333333;
  background: none;
}

.ui.tertiary.button:focus {
  box-shadow: inset 0 -0.2em 0 #666666;
  color: #333333;
  background: none;
}

.ui.tertiary.button:active {
  box-shadow: inset 0 -0.2em 0 #999999;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
  color: #666666;
  background: none;
}

/* --------------
      Labeled Icon
    --------------- */

.ui.labeled.icon.buttons .button,
.ui.labeled.icon.button {
  position: relative;
  padding-left: 4.07142857em !important;
  padding-right: 1.5em !important;
}

/* Left Labeled */

.ui.labeled.icon.buttons > .button > .icon,
.ui.labeled.icon.button > .icon {
  position: absolute;
  top: 0;
  left: 0;
  height: 100%;
  line-height: 1;
  border-radius: 0;
  border-top-left-radius: inherit;
  border-bottom-left-radius: inherit;
  text-align: center;
  animation: none;
  padding: 0.78571429em 0;
  margin: 0;
  width: 2.57142857em;
  background-color: rgba(0, 0, 0, 0.05);
  color: "";
  box-shadow: -1px 0 0 0 transparent inset;
}

/* Right Labeled */

.ui[class*="right labeled"].icon.button {
  padding-right: 4.07142857em !important;
  padding-left: 1.5em !important;
}

.ui[class*="right labeled"].icon.button > .icon {
  left: auto;
  right: 0;
  border-radius: 0;
  border-top-right-radius: inherit;
  border-bottom-right-radius: inherit;
  box-shadow: 1px 0 0 0 transparent inset;
}

.ui.labeled.icon.buttons > .button > .icon::before,
.ui.labeled.icon.button > .icon::before,
.ui.labeled.icon.buttons > .button > .icon::after,
.ui.labeled.icon.button > .icon::after {
  display: block;
  position: relative;
  width: 100%;
  top: 0;
  text-align: center;
}

.ui.labeled.icon.buttons .button > .icon {
  border-radius: 0;
}

.ui.labeled.icon.buttons .button:first-child > .icon {
  border-top-left-radius: 0.28571429rem;
  border-bottom-left-radius: 0.28571429rem;
}

.ui.labeled.icon.buttons .button:last-child > .icon {
  border-top-right-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.vertical.labeled.icon.buttons .button:first-child > .icon {
  border-radius: 0;
  border-top-left-radius: 0.28571429rem;
}

.ui.vertical.labeled.icon.buttons .button:last-child > .icon {
  border-radius: 0;
  border-bottom-left-radius: 0.28571429rem;
}

/* Loading Icon in Labeled Button */

.ui.labeled.icon.button > .loading.icon::before {
  animation: loader 2s linear infinite;
}

/* --------------
         Toggle
    --------------- */

/* Toggle (Modifies active state to give affordances) */

.ui.toggle.buttons .active.button,
.ui.buttons .button.toggle.active,
.ui.button.toggle.active {
  background-color: #21ba45;
  box-shadow: none;
  text-shadow: none;
  color: #fff;
}

.ui.button.toggle.active:hover {
  background-color: #16ab39;
  text-shadow: none;
  color: #fff;
}

/* --------------
        Circular
    --------------- */

.ui.circular.button {
  border-radius: 10em;
}

.ui.circular.button > .icon {
  width: 1em;
  vertical-align: baseline;
}

/* -------------------
          Or Buttons
    -------------------- */

.ui.buttons .or {
  position: relative;
  width: 0.3em;
  height: 2.57142857em;
  z-index: 3;
}

.ui.buttons .or::before {
  position: absolute;
  text-align: center;
  border-radius: 500rem;
  content: "or";
  top: 50%;
  left: 50%;
  background-color: #fff;
  text-shadow: none;
  margin-top: -0.89285714em;
  margin-left: -0.89285714em;
  width: 1.78571429em;
  height: 1.78571429em;
  line-height: 1.78571429em;
  color: rgba(0, 0, 0, 0.4);
  font-style: normal;
  font-weight: bold;
  box-shadow: 0 0 0 1px transparent inset;
}

.ui.buttons .or[data-text]::before {
  content: attr(data-text);
}

/* Fluid Or */

.ui.fluid.buttons .or {
  width: 0 !important;
}

.ui.fluid.buttons .or::after {
  display: none;
}

/* -------------------
           Attached
    -------------------- */

/* Singular */

.ui.attached.button {
  position: relative;
  display: block;
  margin: 0;
  border-radius: 0;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15);
}

/* Top / Bottom */

.ui.attached.top.button {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.attached.bottom.button {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* Left / Right */

.ui.left.attached.button {
  display: inline-block;
  border-left: none;
  text-align: right;
  padding-right: 0.75em;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui.right.attached.button {
  display: inline-block;
  text-align: left;
  padding-left: 0.75em;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

/* Plural */

.ui.attached.buttons {
  position: relative;
  display: flex;
  border-radius: 0;
  width: auto !important;
  z-index: auto;
  margin-left: -1px;
  margin-right: -1px;
}

.ui.attached.buttons .button {
  margin: 0;
}

.ui.attached.buttons .button:first-child {
  border-radius: 0;
}

.ui.attached.buttons .button:last-child {
  border-radius: 0;
}

/* Top / Bottom */

.ui[class*="top attached"].buttons {
  margin-bottom: 0;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui[class*="top attached"].buttons .button:first-child {
  border-radius: 0.28571429rem 0 0 0;
}

.ui[class*="top attached"].buttons .button:last-child {
  border-radius: 0 0.28571429rem 0 0;
}

.ui[class*="bottom attached"].buttons {
  margin-top: -1px;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui[class*="bottom attached"].buttons .button:first-child {
  border-radius: 0 0 0 0.28571429rem;
}

.ui[class*="bottom attached"].buttons .button:last-child {
  border-radius: 0 0 0.28571429rem 0;
}

/* Left / Right */

.ui[class*="left attached"].buttons {
  display: inline-flex;
  margin-right: 0;
  margin-left: -1px;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui[class*="left attached"].buttons .button:first-child {
  margin-left: -1px;
  border-radius: 0 0.28571429rem 0 0;
}

.ui[class*="left attached"].buttons .button:last-child {
  margin-left: -1px;
  border-radius: 0 0 0.28571429rem 0;
}

.ui[class*="right attached"].buttons {
  display: inline-flex;
  margin-left: 0;
  margin-right: -1px;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui[class*="right attached"].buttons .button:first-child {
  margin-left: -1px;
  border-radius: 0.28571429rem 0 0 0;
}

.ui[class*="right attached"].buttons .button:last-child {
  margin-left: -1px;
  border-radius: 0 0 0 0.28571429rem;
}

/* -------------------
            Fluid
    -------------------- */

.ui.fluid.buttons,
.ui.fluid.button {
  width: 100%;
}

.ui.fluid.button {
  display: block;
}

.ui.two.buttons {
  width: 100%;
}

.ui.two.buttons > .button {
  width: 50%;
}

.ui.three.buttons {
  width: 100%;
}

.ui.three.buttons > .button {
  width: 33.333%;
}

.ui.four.buttons {
  width: 100%;
}

.ui.four.buttons > .button {
  width: 25%;
}

.ui.five.buttons {
  width: 100%;
}

.ui.five.buttons > .button {
  width: 20%;
}

.ui.six.buttons {
  width: 100%;
}

.ui.six.buttons > .button {
  width: 16.666%;
}

.ui.seven.buttons {
  width: 100%;
}

.ui.seven.buttons > .button {
  width: 14.285%;
}

.ui.eight.buttons {
  width: 100%;
}

.ui.eight.buttons > .button {
  width: 12.5%;
}

.ui.nine.buttons {
  width: 100%;
}

.ui.nine.buttons > .button {
  width: 11.11%;
}

.ui.ten.buttons {
  width: 100%;
}

.ui.ten.buttons > .button {
  width: 10%;
}

.ui.eleven.buttons {
  width: 100%;
}

.ui.eleven.buttons > .button {
  width: 9.09%;
}

.ui.twelve.buttons {
  width: 100%;
}

.ui.twelve.buttons > .button {
  width: 8.3333%;
}

/* Fluid Vertical Buttons */

.ui.fluid.vertical.buttons,
.ui.fluid.vertical.buttons > .button {
  display: flex;
  width: auto;
  justify-content: center;
}

.ui.two.vertical.buttons > .button {
  height: 50%;
}

.ui.three.vertical.buttons > .button {
  height: 33.333%;
}

.ui.four.vertical.buttons > .button {
  height: 25%;
}

.ui.five.vertical.buttons > .button {
  height: 20%;
}

.ui.six.vertical.buttons > .button {
  height: 16.666%;
}

.ui.seven.vertical.buttons > .button {
  height: 14.285%;
}

.ui.eight.vertical.buttons > .button {
  height: 12.5%;
}

.ui.nine.vertical.buttons > .button {
  height: 11.11%;
}

.ui.ten.vertical.buttons > .button {
  height: 10%;
}

.ui.eleven.vertical.buttons > .button {
  height: 9.09%;
}

.ui.twelve.vertical.buttons > .button {
  height: 8.3333%;
}

/* -------------------
       Colors
-------------------- */

.ui.primary.buttons .button,
.ui.primary.button {
  background-color: #2185d0;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.primary.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.primary.buttons .button:hover,
.ui.primary.button:hover {
  background-color: #1678c2;
  color: #fff;
  text-shadow: none;
}

.ui.primary.buttons .button:focus,
.ui.primary.button:focus {
  background-color: #0d71bb;
  color: #fff;
  text-shadow: none;
}

.ui.primary.buttons .button:active,
.ui.primary.button:active {
  background-color: #1a69a4;
  color: #fff;
  text-shadow: none;
}

.ui.primary.buttons .active.button,
.ui.primary.buttons .active.button:active,
.ui.primary.active.button,
.ui.primary.button .active.button:active {
  background-color: #1279c6;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.primary.buttons .button,
.ui.basic.primary.button {
  background: transparent;
  box-shadow: 0 0 0 1px #2185d0 inset;
  color: #2185d0;
}

.ui.basic.primary.buttons .button:hover,
.ui.basic.primary.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #1678c2 inset;
  color: #1678c2;
}

.ui.basic.primary.buttons .button:focus,
.ui.basic.primary.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #0d71bb inset;
  color: #1678c2;
}

.ui.basic.primary.buttons .active.button,
.ui.basic.primary.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #1279c6 inset;
  color: #1a69a4;
}

.ui.basic.primary.buttons .button:active,
.ui.basic.primary.button:active {
  box-shadow: 0 0 0 1px #1a69a4 inset;
  color: #1a69a4;
}

/* Inverted */

.ui.inverted.primary.buttons .button,
.ui.inverted.primary.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #54c8ff inset;
  color: #54c8ff;
}

.ui.inverted.primary.buttons .button:hover,
.ui.inverted.primary.button:hover,
.ui.inverted.primary.buttons .button:focus,
.ui.inverted.primary.button:focus,
.ui.inverted.primary.buttons .button.active,
.ui.inverted.primary.button.active,
.ui.inverted.primary.buttons .button:active,
.ui.inverted.primary.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.primary.buttons .button:hover,
.ui.inverted.primary.button:hover {
  background-color: #21b8ff;
}

.ui.inverted.primary.buttons .button:focus,
.ui.inverted.primary.button:focus {
  background-color: #2bbbff;
}

.ui.inverted.primary.buttons .active.button,
.ui.inverted.primary.active.button {
  background-color: #3ac0ff;
}

.ui.inverted.primary.buttons .button:active,
.ui.inverted.primary.button:active {
  background-color: #21b8ff;
}

/* Inverted Basic */

.ui.inverted.primary.basic.buttons .button,
.ui.inverted.primary.buttons .basic.button,
.ui.inverted.primary.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.primary.basic.buttons .button:hover,
.ui.inverted.primary.buttons .basic.button:hover,
.ui.inverted.primary.basic.button:hover {
  box-shadow: 0 0 0 2px #21b8ff inset;
  color: #54c8ff;
}

.ui.inverted.primary.basic.buttons .button:focus,
.ui.inverted.primary.basic.buttons .button:focus,
.ui.inverted.primary.basic.button:focus {
  box-shadow: 0 0 0 2px #2bbbff inset;
  color: #54c8ff;
}

.ui.inverted.primary.basic.buttons .active.button,
.ui.inverted.primary.buttons .basic.active.button,
.ui.inverted.primary.basic.active.button {
  box-shadow: 0 0 0 2px #3ac0ff inset;
  color: #54c8ff;
}

.ui.inverted.primary.basic.buttons .button:active,
.ui.inverted.primary.buttons .basic.button:active,
.ui.inverted.primary.basic.button:active {
  box-shadow: 0 0 0 2px #21b8ff inset;
  color: #54c8ff;
}

/* Tertiary */

.ui.tertiary.primary.buttons .button,
.ui.tertiary.primary.buttons .tertiary.button,
.ui.tertiary.primary.button {
  background: transparent;
  box-shadow: none;
  color: #2185d0;
}

.ui.tertiary.primary.buttons .button:hover,
.ui.tertiary.primary.buttons button:hover,
.ui.tertiary.primary.button:hover {
  box-shadow: inset 0 -0.2em 0 #2b75ac;
  color: #2b75ac;
}

.ui.tertiary.primary.buttons .button:focus,
.ui.tertiary.primary.buttons .tertiary.button:focus,
.ui.tertiary.primary.button:focus {
  box-shadow: inset 0 -0.2em 0 #216ea7;
  color: #216ea7;
}

.ui.tertiary.primary.buttons .active.button,
.ui.tertiary.primary.buttons .tertiary.active.button,
.ui.tertiary.primary.active.button,
.ui.tertiary.primary.buttons .button:active,
.ui.tertiary.primary.buttons .tertiary.button:active,
.ui.tertiary.primary.button:active {
  box-shadow: inset 0 -0.2em 0 #007bd8;
  color: #1279c6;
}

.ui.secondary.buttons .button,
.ui.secondary.button {
  background-color: #1b1c1d;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.secondary.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.secondary.buttons .button:hover,
.ui.secondary.button:hover {
  background-color: #27292a;
  color: #fff;
  text-shadow: none;
}

.ui.secondary.buttons .button:focus,
.ui.secondary.button:focus {
  background-color: #2e3032;
  color: #fff;
  text-shadow: none;
}

.ui.secondary.buttons .button:active,
.ui.secondary.button:active {
  background-color: #343637;
  color: #fff;
  text-shadow: none;
}

.ui.secondary.buttons .active.button,
.ui.secondary.buttons .active.button:active,
.ui.secondary.active.button,
.ui.secondary.button .active.button:active {
  background-color: #27292a;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.secondary.buttons .button,
.ui.basic.secondary.button {
  background: transparent;
  box-shadow: 0 0 0 1px #1b1c1d inset;
  color: #1b1c1d;
}

.ui.basic.secondary.buttons .button:hover,
.ui.basic.secondary.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #27292a inset;
  color: #27292a;
}

.ui.basic.secondary.buttons .button:focus,
.ui.basic.secondary.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #2e3032 inset;
  color: #27292a;
}

.ui.basic.secondary.buttons .active.button,
.ui.basic.secondary.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #27292a inset;
  color: #343637;
}

.ui.basic.secondary.buttons .button:active,
.ui.basic.secondary.button:active {
  box-shadow: 0 0 0 1px #343637 inset;
  color: #343637;
}

/* Inverted */

.ui.inverted.secondary.buttons .button,
.ui.inverted.secondary.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #545454 inset;
  color: #545454;
}

.ui.inverted.secondary.buttons .button:hover,
.ui.inverted.secondary.button:hover,
.ui.inverted.secondary.buttons .button:focus,
.ui.inverted.secondary.button:focus,
.ui.inverted.secondary.buttons .button.active,
.ui.inverted.secondary.button.active,
.ui.inverted.secondary.buttons .button:active,
.ui.inverted.secondary.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.secondary.buttons .button:hover,
.ui.inverted.secondary.button:hover {
  background-color: #6e6e6e;
}

.ui.inverted.secondary.buttons .button:focus,
.ui.inverted.secondary.button:focus {
  background-color: #686868;
}

.ui.inverted.secondary.buttons .active.button,
.ui.inverted.secondary.active.button {
  background-color: #616161;
}

.ui.inverted.secondary.buttons .button:active,
.ui.inverted.secondary.button:active {
  background-color: #6e6e6e;
}

/* Inverted Basic */

.ui.inverted.secondary.basic.buttons .button,
.ui.inverted.secondary.buttons .basic.button,
.ui.inverted.secondary.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.secondary.basic.buttons .button:hover,
.ui.inverted.secondary.buttons .basic.button:hover,
.ui.inverted.secondary.basic.button:hover {
  box-shadow: 0 0 0 2px #6e6e6e inset;
  color: #545454;
}

.ui.inverted.secondary.basic.buttons .button:focus,
.ui.inverted.secondary.basic.buttons .button:focus,
.ui.inverted.secondary.basic.button:focus {
  box-shadow: 0 0 0 2px #686868 inset;
  color: #545454;
}

.ui.inverted.secondary.basic.buttons .active.button,
.ui.inverted.secondary.buttons .basic.active.button,
.ui.inverted.secondary.basic.active.button {
  box-shadow: 0 0 0 2px #616161 inset;
  color: #545454;
}

.ui.inverted.secondary.basic.buttons .button:active,
.ui.inverted.secondary.buttons .basic.button:active,
.ui.inverted.secondary.basic.button:active {
  box-shadow: 0 0 0 2px #6e6e6e inset;
  color: #545454;
}

/* Tertiary */

.ui.tertiary.secondary.buttons .button,
.ui.tertiary.secondary.buttons .tertiary.button,
.ui.tertiary.secondary.button {
  background: transparent;
  box-shadow: none;
  color: #1b1c1d;
}

.ui.tertiary.secondary.buttons .button:hover,
.ui.tertiary.secondary.buttons button:hover,
.ui.tertiary.secondary.button:hover {
  box-shadow: inset 0 -0.2em 0 #292929;
  color: #292929;
}

.ui.tertiary.secondary.buttons .button:focus,
.ui.tertiary.secondary.buttons .tertiary.button:focus,
.ui.tertiary.secondary.button:focus {
  box-shadow: inset 0 -0.2em 0 #303030;
  color: #303030;
}

.ui.tertiary.secondary.buttons .active.button,
.ui.tertiary.secondary.buttons .tertiary.active.button,
.ui.tertiary.secondary.active.button,
.ui.tertiary.secondary.buttons .button:active,
.ui.tertiary.secondary.buttons .tertiary.button:active,
.ui.tertiary.secondary.button:active {
  box-shadow: inset 0 -0.2em 0 #1f2933;
  color: #27292a;
}

.ui.red.buttons .button,
.ui.red.button {
  background-color: #db2828;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.red.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.red.buttons .button:hover,
.ui.red.button:hover {
  background-color: #d01919;
  color: #fff;
  text-shadow: none;
}

.ui.red.buttons .button:focus,
.ui.red.button:focus {
  background-color: #ca1010;
  color: #fff;
  text-shadow: none;
}

.ui.red.buttons .button:active,
.ui.red.button:active {
  background-color: #b21e1e;
  color: #fff;
  text-shadow: none;
}

.ui.red.buttons .active.button,
.ui.red.buttons .active.button:active,
.ui.red.active.button,
.ui.red.button .active.button:active {
  background-color: #d41515;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.red.buttons .button,
.ui.basic.red.button {
  background: transparent;
  box-shadow: 0 0 0 1px #db2828 inset;
  color: #db2828;
}

.ui.basic.red.buttons .button:hover,
.ui.basic.red.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #d01919 inset;
  color: #d01919;
}

.ui.basic.red.buttons .button:focus,
.ui.basic.red.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #ca1010 inset;
  color: #d01919;
}

.ui.basic.red.buttons .active.button,
.ui.basic.red.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #d41515 inset;
  color: #b21e1e;
}

.ui.basic.red.buttons .button:active,
.ui.basic.red.button:active {
  box-shadow: 0 0 0 1px #b21e1e inset;
  color: #b21e1e;
}

/* Inverted */

.ui.inverted.red.buttons .button,
.ui.inverted.red.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #ff695e inset;
  color: #ff695e;
}

.ui.inverted.red.buttons .button:hover,
.ui.inverted.red.button:hover,
.ui.inverted.red.buttons .button:focus,
.ui.inverted.red.button:focus,
.ui.inverted.red.buttons .button.active,
.ui.inverted.red.button.active,
.ui.inverted.red.buttons .button:active,
.ui.inverted.red.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.red.buttons .button:hover,
.ui.inverted.red.button:hover {
  background-color: #ff392b;
}

.ui.inverted.red.buttons .button:focus,
.ui.inverted.red.button:focus {
  background-color: #ff4335;
}

.ui.inverted.red.buttons .active.button,
.ui.inverted.red.active.button {
  background-color: #ff5144;
}

.ui.inverted.red.buttons .button:active,
.ui.inverted.red.button:active {
  background-color: #ff392b;
}

/* Inverted Basic */

.ui.inverted.red.basic.buttons .button,
.ui.inverted.red.buttons .basic.button,
.ui.inverted.red.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.red.basic.buttons .button:hover,
.ui.inverted.red.buttons .basic.button:hover,
.ui.inverted.red.basic.button:hover {
  box-shadow: 0 0 0 2px #ff392b inset;
  color: #ff695e;
}

.ui.inverted.red.basic.buttons .button:focus,
.ui.inverted.red.basic.buttons .button:focus,
.ui.inverted.red.basic.button:focus {
  box-shadow: 0 0 0 2px #ff4335 inset;
  color: #ff695e;
}

.ui.inverted.red.basic.buttons .active.button,
.ui.inverted.red.buttons .basic.active.button,
.ui.inverted.red.basic.active.button {
  box-shadow: 0 0 0 2px #ff5144 inset;
  color: #ff695e;
}

.ui.inverted.red.basic.buttons .button:active,
.ui.inverted.red.buttons .basic.button:active,
.ui.inverted.red.basic.button:active {
  box-shadow: 0 0 0 2px #ff392b inset;
  color: #ff695e;
}

/* Tertiary */

.ui.tertiary.red.buttons .button,
.ui.tertiary.red.buttons .tertiary.button,
.ui.tertiary.red.button {
  background: transparent;
  box-shadow: none;
  color: #db2828;
}

.ui.tertiary.red.buttons .button:hover,
.ui.tertiary.red.buttons button:hover,
.ui.tertiary.red.button:hover {
  box-shadow: inset 0 -0.2em 0 #b93131;
  color: #b93131;
}

.ui.tertiary.red.buttons .button:focus,
.ui.tertiary.red.buttons .tertiary.button:focus,
.ui.tertiary.red.button:focus {
  box-shadow: inset 0 -0.2em 0 #b52626;
  color: #b52626;
}

.ui.tertiary.red.buttons .active.button,
.ui.tertiary.red.buttons .tertiary.active.button,
.ui.tertiary.red.active.button,
.ui.tertiary.red.buttons .button:active,
.ui.tertiary.red.buttons .tertiary.button:active,
.ui.tertiary.red.button:active {
  box-shadow: inset 0 -0.2em 0 #ea0000;
  color: #d41515;
}

.ui.orange.buttons .button,
.ui.orange.button {
  background-color: #f2711c;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.orange.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.orange.buttons .button:hover,
.ui.orange.button:hover {
  background-color: #f26202;
  color: #fff;
  text-shadow: none;
}

.ui.orange.buttons .button:focus,
.ui.orange.button:focus {
  background-color: #e55b00;
  color: #fff;
  text-shadow: none;
}

.ui.orange.buttons .button:active,
.ui.orange.button:active {
  background-color: #cf590c;
  color: #fff;
  text-shadow: none;
}

.ui.orange.buttons .active.button,
.ui.orange.buttons .active.button:active,
.ui.orange.active.button,
.ui.orange.button .active.button:active {
  background-color: #f56100;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.orange.buttons .button,
.ui.basic.orange.button {
  background: transparent;
  box-shadow: 0 0 0 1px #f2711c inset;
  color: #f2711c;
}

.ui.basic.orange.buttons .button:hover,
.ui.basic.orange.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #f26202 inset;
  color: #f26202;
}

.ui.basic.orange.buttons .button:focus,
.ui.basic.orange.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #e55b00 inset;
  color: #f26202;
}

.ui.basic.orange.buttons .active.button,
.ui.basic.orange.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #f56100 inset;
  color: #cf590c;
}

.ui.basic.orange.buttons .button:active,
.ui.basic.orange.button:active {
  box-shadow: 0 0 0 1px #cf590c inset;
  color: #cf590c;
}

/* Inverted */

.ui.inverted.orange.buttons .button,
.ui.inverted.orange.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #ff851b inset;
  color: #ff851b;
}

.ui.inverted.orange.buttons .button:hover,
.ui.inverted.orange.button:hover,
.ui.inverted.orange.buttons .button:focus,
.ui.inverted.orange.button:focus,
.ui.inverted.orange.buttons .button.active,
.ui.inverted.orange.button.active,
.ui.inverted.orange.buttons .button:active,
.ui.inverted.orange.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.orange.buttons .button:hover,
.ui.inverted.orange.button:hover {
  background-color: #e76b00;
}

.ui.inverted.orange.buttons .button:focus,
.ui.inverted.orange.button:focus {
  background-color: #f17000;
}

.ui.inverted.orange.buttons .active.button,
.ui.inverted.orange.active.button {
  background-color: #ff7701;
}

.ui.inverted.orange.buttons .button:active,
.ui.inverted.orange.button:active {
  background-color: #e76b00;
}

/* Inverted Basic */

.ui.inverted.orange.basic.buttons .button,
.ui.inverted.orange.buttons .basic.button,
.ui.inverted.orange.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.orange.basic.buttons .button:hover,
.ui.inverted.orange.buttons .basic.button:hover,
.ui.inverted.orange.basic.button:hover {
  box-shadow: 0 0 0 2px #e76b00 inset;
  color: #ff851b;
}

.ui.inverted.orange.basic.buttons .button:focus,
.ui.inverted.orange.basic.buttons .button:focus,
.ui.inverted.orange.basic.button:focus {
  box-shadow: 0 0 0 2px #f17000 inset;
  color: #ff851b;
}

.ui.inverted.orange.basic.buttons .active.button,
.ui.inverted.orange.buttons .basic.active.button,
.ui.inverted.orange.basic.active.button {
  box-shadow: 0 0 0 2px #ff7701 inset;
  color: #ff851b;
}

.ui.inverted.orange.basic.buttons .button:active,
.ui.inverted.orange.buttons .basic.button:active,
.ui.inverted.orange.basic.button:active {
  box-shadow: 0 0 0 2px #e76b00 inset;
  color: #ff851b;
}

/* Tertiary */

.ui.tertiary.orange.buttons .button,
.ui.tertiary.orange.buttons .tertiary.button,
.ui.tertiary.orange.button {
  background: transparent;
  box-shadow: none;
  color: #f2711c;
}

.ui.tertiary.orange.buttons .button:hover,
.ui.tertiary.orange.buttons button:hover,
.ui.tertiary.orange.button:hover {
  box-shadow: inset 0 -0.2em 0 #da671b;
  color: #da671b;
}

.ui.tertiary.orange.buttons .button:focus,
.ui.tertiary.orange.buttons .tertiary.button:focus,
.ui.tertiary.orange.button:focus {
  box-shadow: inset 0 -0.2em 0 #ce6017;
  color: #ce6017;
}

.ui.tertiary.orange.buttons .active.button,
.ui.tertiary.orange.buttons .tertiary.active.button,
.ui.tertiary.orange.active.button,
.ui.tertiary.orange.buttons .button:active,
.ui.tertiary.orange.buttons .tertiary.button:active,
.ui.tertiary.orange.button:active {
  box-shadow: inset 0 -0.2em 0 #f56100;
  color: #f56100;
}

.ui.yellow.buttons .button,
.ui.yellow.button {
  background-color: #fbbd08;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.yellow.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.yellow.buttons .button:hover,
.ui.yellow.button:hover {
  background-color: #eaae00;
  color: #fff;
  text-shadow: none;
}

.ui.yellow.buttons .button:focus,
.ui.yellow.button:focus {
  background-color: #daa300;
  color: #fff;
  text-shadow: none;
}

.ui.yellow.buttons .button:active,
.ui.yellow.button:active {
  background-color: #cd9903;
  color: #fff;
  text-shadow: none;
}

.ui.yellow.buttons .active.button,
.ui.yellow.buttons .active.button:active,
.ui.yellow.active.button,
.ui.yellow.button .active.button:active {
  background-color: #eaae00;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.yellow.buttons .button,
.ui.basic.yellow.button {
  background: transparent;
  box-shadow: 0 0 0 1px #fbbd08 inset;
  color: #fbbd08;
}

.ui.basic.yellow.buttons .button:hover,
.ui.basic.yellow.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #eaae00 inset;
  color: #eaae00;
}

.ui.basic.yellow.buttons .button:focus,
.ui.basic.yellow.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #daa300 inset;
  color: #eaae00;
}

.ui.basic.yellow.buttons .active.button,
.ui.basic.yellow.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #eaae00 inset;
  color: #cd9903;
}

.ui.basic.yellow.buttons .button:active,
.ui.basic.yellow.button:active {
  box-shadow: 0 0 0 1px #cd9903 inset;
  color: #cd9903;
}

/* Inverted */

.ui.inverted.yellow.buttons .button,
.ui.inverted.yellow.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #ffe21f inset;
  color: #ffe21f;
}

.ui.inverted.yellow.buttons .button:hover,
.ui.inverted.yellow.button:hover,
.ui.inverted.yellow.buttons .button:focus,
.ui.inverted.yellow.button:focus,
.ui.inverted.yellow.buttons .button.active,
.ui.inverted.yellow.button.active,
.ui.inverted.yellow.buttons .button:active,
.ui.inverted.yellow.button:active {
  box-shadow: none;
  color: rgba(0, 0, 0, 0.6);
}

.ui.inverted.yellow.buttons .button:hover,
.ui.inverted.yellow.button:hover {
  background-color: #ebcd00;
}

.ui.inverted.yellow.buttons .button:focus,
.ui.inverted.yellow.button:focus {
  background-color: #f5d500;
}

.ui.inverted.yellow.buttons .active.button,
.ui.inverted.yellow.active.button {
  background-color: #ffdf05;
}

.ui.inverted.yellow.buttons .button:active,
.ui.inverted.yellow.button:active {
  background-color: #ebcd00;
}

/* Inverted Basic */

.ui.inverted.yellow.basic.buttons .button,
.ui.inverted.yellow.buttons .basic.button,
.ui.inverted.yellow.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.yellow.basic.buttons .button:hover,
.ui.inverted.yellow.buttons .basic.button:hover,
.ui.inverted.yellow.basic.button:hover {
  box-shadow: 0 0 0 2px #ebcd00 inset;
  color: #ffe21f;
}

.ui.inverted.yellow.basic.buttons .button:focus,
.ui.inverted.yellow.basic.buttons .button:focus,
.ui.inverted.yellow.basic.button:focus {
  box-shadow: 0 0 0 2px #f5d500 inset;
  color: #ffe21f;
}

.ui.inverted.yellow.basic.buttons .active.button,
.ui.inverted.yellow.buttons .basic.active.button,
.ui.inverted.yellow.basic.active.button {
  box-shadow: 0 0 0 2px #ffdf05 inset;
  color: #ffe21f;
}

.ui.inverted.yellow.basic.buttons .button:active,
.ui.inverted.yellow.buttons .basic.button:active,
.ui.inverted.yellow.basic.button:active {
  box-shadow: 0 0 0 2px #ebcd00 inset;
  color: #ffe21f;
}

/* Tertiary */

.ui.tertiary.yellow.buttons .button,
.ui.tertiary.yellow.buttons .tertiary.button,
.ui.tertiary.yellow.button {
  background: transparent;
  box-shadow: none;
  color: #fbbd08;
}

.ui.tertiary.yellow.buttons .button:hover,
.ui.tertiary.yellow.buttons button:hover,
.ui.tertiary.yellow.button:hover {
  box-shadow: inset 0 -0.2em 0 #d2a217;
  color: #d2a217;
}

.ui.tertiary.yellow.buttons .button:focus,
.ui.tertiary.yellow.buttons .tertiary.button:focus,
.ui.tertiary.yellow.button:focus {
  box-shadow: inset 0 -0.2em 0 #c49816;
  color: #c49816;
}

.ui.tertiary.yellow.buttons .active.button,
.ui.tertiary.yellow.buttons .tertiary.active.button,
.ui.tertiary.yellow.active.button,
.ui.tertiary.yellow.buttons .button:active,
.ui.tertiary.yellow.buttons .tertiary.button:active,
.ui.tertiary.yellow.button:active {
  box-shadow: inset 0 -0.2em 0 #eaae00;
  color: #eaae00;
}

.ui.olive.buttons .button,
.ui.olive.button {
  background-color: #b5cc18;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.olive.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.olive.buttons .button:hover,
.ui.olive.button:hover {
  background-color: #a7bd0d;
  color: #fff;
  text-shadow: none;
}

.ui.olive.buttons .button:focus,
.ui.olive.button:focus {
  background-color: #a0b605;
  color: #fff;
  text-shadow: none;
}

.ui.olive.buttons .button:active,
.ui.olive.button:active {
  background-color: #8d9e13;
  color: #fff;
  text-shadow: none;
}

.ui.olive.buttons .active.button,
.ui.olive.buttons .active.button:active,
.ui.olive.active.button,
.ui.olive.button .active.button:active {
  background-color: #aac109;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.olive.buttons .button,
.ui.basic.olive.button {
  background: transparent;
  box-shadow: 0 0 0 1px #b5cc18 inset;
  color: #b5cc18;
}

.ui.basic.olive.buttons .button:hover,
.ui.basic.olive.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #a7bd0d inset;
  color: #a7bd0d;
}

.ui.basic.olive.buttons .button:focus,
.ui.basic.olive.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #a0b605 inset;
  color: #a7bd0d;
}

.ui.basic.olive.buttons .active.button,
.ui.basic.olive.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #aac109 inset;
  color: #8d9e13;
}

.ui.basic.olive.buttons .button:active,
.ui.basic.olive.button:active {
  box-shadow: 0 0 0 1px #8d9e13 inset;
  color: #8d9e13;
}

/* Inverted */

.ui.inverted.olive.buttons .button,
.ui.inverted.olive.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #d9e778 inset;
  color: #d9e778;
}

.ui.inverted.olive.buttons .button:hover,
.ui.inverted.olive.button:hover,
.ui.inverted.olive.buttons .button:focus,
.ui.inverted.olive.button:focus,
.ui.inverted.olive.buttons .button.active,
.ui.inverted.olive.button.active,
.ui.inverted.olive.buttons .button:active,
.ui.inverted.olive.button:active {
  box-shadow: none;
  color: rgba(0, 0, 0, 0.6);
}

.ui.inverted.olive.buttons .button:hover,
.ui.inverted.olive.button:hover {
  background-color: #d2e745;
}

.ui.inverted.olive.buttons .button:focus,
.ui.inverted.olive.button:focus {
  background-color: #daef47;
}

.ui.inverted.olive.buttons .active.button,
.ui.inverted.olive.active.button {
  background-color: #daed59;
}

.ui.inverted.olive.buttons .button:active,
.ui.inverted.olive.button:active {
  background-color: #cddf4d;
}

/* Inverted Basic */

.ui.inverted.olive.basic.buttons .button,
.ui.inverted.olive.buttons .basic.button,
.ui.inverted.olive.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.olive.basic.buttons .button:hover,
.ui.inverted.olive.buttons .basic.button:hover,
.ui.inverted.olive.basic.button:hover {
  box-shadow: 0 0 0 2px #d2e745 inset;
  color: #d9e778;
}

.ui.inverted.olive.basic.buttons .button:focus,
.ui.inverted.olive.basic.buttons .button:focus,
.ui.inverted.olive.basic.button:focus {
  box-shadow: 0 0 0 2px #daef47 inset;
  color: #d9e778;
}

.ui.inverted.olive.basic.buttons .active.button,
.ui.inverted.olive.buttons .basic.active.button,
.ui.inverted.olive.basic.active.button {
  box-shadow: 0 0 0 2px #daed59 inset;
  color: #d9e778;
}

.ui.inverted.olive.basic.buttons .button:active,
.ui.inverted.olive.buttons .basic.button:active,
.ui.inverted.olive.basic.button:active {
  box-shadow: 0 0 0 2px #cddf4d inset;
  color: #d9e778;
}

/* Tertiary */

.ui.tertiary.olive.buttons .button,
.ui.tertiary.olive.buttons .tertiary.button,
.ui.tertiary.olive.button {
  background: transparent;
  box-shadow: none;
  color: #b5cc18;
}

.ui.tertiary.olive.buttons .button:hover,
.ui.tertiary.olive.buttons button:hover,
.ui.tertiary.olive.button:hover {
  box-shadow: inset 0 -0.2em 0 #98a922;
  color: #98a922;
}

.ui.tertiary.olive.buttons .button:focus,
.ui.tertiary.olive.buttons .tertiary.button:focus,
.ui.tertiary.olive.button:focus {
  box-shadow: inset 0 -0.2em 0 #92a418;
  color: #92a418;
}

.ui.tertiary.olive.buttons .active.button,
.ui.tertiary.olive.buttons .tertiary.active.button,
.ui.tertiary.olive.active.button,
.ui.tertiary.olive.buttons .button:active,
.ui.tertiary.olive.buttons .tertiary.button:active,
.ui.tertiary.olive.button:active {
  box-shadow: inset 0 -0.2em 0 #b1cb00;
  color: #aac109;
}

.ui.green.buttons .button,
.ui.green.button {
  background-color: #21ba45;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.green.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.green.buttons .button:hover,
.ui.green.button:hover {
  background-color: #16ab39;
  color: #fff;
  text-shadow: none;
}

.ui.green.buttons .button:focus,
.ui.green.button:focus {
  background-color: #0ea432;
  color: #fff;
  text-shadow: none;
}

.ui.green.buttons .button:active,
.ui.green.button:active {
  background-color: #198f35;
  color: #fff;
  text-shadow: none;
}

.ui.green.buttons .active.button,
.ui.green.buttons .active.button:active,
.ui.green.active.button,
.ui.green.button .active.button:active {
  background-color: #13ae38;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.green.buttons .button,
.ui.basic.green.button {
  background: transparent;
  box-shadow: 0 0 0 1px #21ba45 inset;
  color: #21ba45;
}

.ui.basic.green.buttons .button:hover,
.ui.basic.green.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #16ab39 inset;
  color: #16ab39;
}

.ui.basic.green.buttons .button:focus,
.ui.basic.green.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #0ea432 inset;
  color: #16ab39;
}

.ui.basic.green.buttons .active.button,
.ui.basic.green.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #13ae38 inset;
  color: #198f35;
}

.ui.basic.green.buttons .button:active,
.ui.basic.green.button:active {
  box-shadow: 0 0 0 1px #198f35 inset;
  color: #198f35;
}

/* Inverted */

.ui.inverted.green.buttons .button,
.ui.inverted.green.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #2ecc40 inset;
  color: #2ecc40;
}

.ui.inverted.green.buttons .button:hover,
.ui.inverted.green.button:hover,
.ui.inverted.green.buttons .button:focus,
.ui.inverted.green.button:focus,
.ui.inverted.green.buttons .button.active,
.ui.inverted.green.button.active,
.ui.inverted.green.buttons .button:active,
.ui.inverted.green.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.green.buttons .button:hover,
.ui.inverted.green.button:hover {
  background-color: #1ea92e;
}

.ui.inverted.green.buttons .button:focus,
.ui.inverted.green.button:focus {
  background-color: #19b82b;
}

.ui.inverted.green.buttons .active.button,
.ui.inverted.green.active.button {
  background-color: #1fc231;
}

.ui.inverted.green.buttons .button:active,
.ui.inverted.green.button:active {
  background-color: #25a233;
}

/* Inverted Basic */

.ui.inverted.green.basic.buttons .button,
.ui.inverted.green.buttons .basic.button,
.ui.inverted.green.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.green.basic.buttons .button:hover,
.ui.inverted.green.buttons .basic.button:hover,
.ui.inverted.green.basic.button:hover {
  box-shadow: 0 0 0 2px #1ea92e inset;
  color: #2ecc40;
}

.ui.inverted.green.basic.buttons .button:focus,
.ui.inverted.green.basic.buttons .button:focus,
.ui.inverted.green.basic.button:focus {
  box-shadow: 0 0 0 2px #19b82b inset;
  color: #2ecc40;
}

.ui.inverted.green.basic.buttons .active.button,
.ui.inverted.green.buttons .basic.active.button,
.ui.inverted.green.basic.active.button {
  box-shadow: 0 0 0 2px #1fc231 inset;
  color: #2ecc40;
}

.ui.inverted.green.basic.buttons .button:active,
.ui.inverted.green.buttons .basic.button:active,
.ui.inverted.green.basic.button:active {
  box-shadow: 0 0 0 2px #25a233 inset;
  color: #2ecc40;
}

/* Tertiary */

.ui.tertiary.green.buttons .button,
.ui.tertiary.green.buttons .tertiary.button,
.ui.tertiary.green.button {
  background: transparent;
  box-shadow: none;
  color: #21ba45;
}

.ui.tertiary.green.buttons .button:hover,
.ui.tertiary.green.buttons button:hover,
.ui.tertiary.green.button:hover {
  box-shadow: inset 0 -0.2em 0 #2a9844;
  color: #2a9844;
}

.ui.tertiary.green.buttons .button:focus,
.ui.tertiary.green.buttons .tertiary.button:focus,
.ui.tertiary.green.button:focus {
  box-shadow: inset 0 -0.2em 0 #20923b;
  color: #20923b;
}

.ui.tertiary.green.buttons .active.button,
.ui.tertiary.green.buttons .tertiary.active.button,
.ui.tertiary.green.active.button,
.ui.tertiary.green.buttons .button:active,
.ui.tertiary.green.buttons .tertiary.button:active,
.ui.tertiary.green.button:active {
  box-shadow: inset 0 -0.2em 0 #00c22e;
  color: #13ae38;
}

.ui.teal.buttons .button,
.ui.teal.button {
  background-color: #00b5ad;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.teal.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.teal.buttons .button:hover,
.ui.teal.button:hover {
  background-color: #009c95;
  color: #fff;
  text-shadow: none;
}

.ui.teal.buttons .button:focus,
.ui.teal.button:focus {
  background-color: #008c86;
  color: #fff;
  text-shadow: none;
}

.ui.teal.buttons .button:active,
.ui.teal.button:active {
  background-color: #00827c;
  color: #fff;
  text-shadow: none;
}

.ui.teal.buttons .active.button,
.ui.teal.buttons .active.button:active,
.ui.teal.active.button,
.ui.teal.button .active.button:active {
  background-color: #009c95;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.teal.buttons .button,
.ui.basic.teal.button {
  background: transparent;
  box-shadow: 0 0 0 1px #00b5ad inset;
  color: #00b5ad;
}

.ui.basic.teal.buttons .button:hover,
.ui.basic.teal.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #009c95 inset;
  color: #009c95;
}

.ui.basic.teal.buttons .button:focus,
.ui.basic.teal.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #008c86 inset;
  color: #009c95;
}

.ui.basic.teal.buttons .active.button,
.ui.basic.teal.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #009c95 inset;
  color: #00827c;
}

.ui.basic.teal.buttons .button:active,
.ui.basic.teal.button:active {
  box-shadow: 0 0 0 1px #00827c inset;
  color: #00827c;
}

/* Inverted */

.ui.inverted.teal.buttons .button,
.ui.inverted.teal.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #6dffff inset;
  color: #6dffff;
}

.ui.inverted.teal.buttons .button:hover,
.ui.inverted.teal.button:hover,
.ui.inverted.teal.buttons .button:focus,
.ui.inverted.teal.button:focus,
.ui.inverted.teal.buttons .button.active,
.ui.inverted.teal.button.active,
.ui.inverted.teal.buttons .button:active,
.ui.inverted.teal.button:active {
  box-shadow: none;
  color: rgba(0, 0, 0, 0.6);
}

.ui.inverted.teal.buttons .button:hover,
.ui.inverted.teal.button:hover {
  background-color: #3affff;
}

.ui.inverted.teal.buttons .button:focus,
.ui.inverted.teal.button:focus {
  background-color: #44ffff;
}

.ui.inverted.teal.buttons .active.button,
.ui.inverted.teal.active.button {
  background-color: #54ffff;
}

.ui.inverted.teal.buttons .button:active,
.ui.inverted.teal.button:active {
  background-color: #3affff;
}

/* Inverted Basic */

.ui.inverted.teal.basic.buttons .button,
.ui.inverted.teal.buttons .basic.button,
.ui.inverted.teal.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.teal.basic.buttons .button:hover,
.ui.inverted.teal.buttons .basic.button:hover,
.ui.inverted.teal.basic.button:hover {
  box-shadow: 0 0 0 2px #3affff inset;
  color: #6dffff;
}

.ui.inverted.teal.basic.buttons .button:focus,
.ui.inverted.teal.basic.buttons .button:focus,
.ui.inverted.teal.basic.button:focus {
  box-shadow: 0 0 0 2px #44ffff inset;
  color: #6dffff;
}

.ui.inverted.teal.basic.buttons .active.button,
.ui.inverted.teal.buttons .basic.active.button,
.ui.inverted.teal.basic.active.button {
  box-shadow: 0 0 0 2px #54ffff inset;
  color: #6dffff;
}

.ui.inverted.teal.basic.buttons .button:active,
.ui.inverted.teal.buttons .basic.button:active,
.ui.inverted.teal.basic.button:active {
  box-shadow: 0 0 0 2px #3affff inset;
  color: #6dffff;
}

/* Tertiary */

.ui.tertiary.teal.buttons .button,
.ui.tertiary.teal.buttons .tertiary.button,
.ui.tertiary.teal.button {
  background: transparent;
  box-shadow: none;
  color: #00b5ad;
}

.ui.tertiary.teal.buttons .button:hover,
.ui.tertiary.teal.buttons button:hover,
.ui.tertiary.teal.button:hover {
  box-shadow: inset 0 -0.2em 0 #108c86;
  color: #108c86;
}

.ui.tertiary.teal.buttons .button:focus,
.ui.tertiary.teal.buttons .tertiary.button:focus,
.ui.tertiary.teal.button:focus {
  box-shadow: inset 0 -0.2em 0 #0e7e79;
  color: #0e7e79;
}

.ui.tertiary.teal.buttons .active.button,
.ui.tertiary.teal.buttons .tertiary.active.button,
.ui.tertiary.teal.active.button,
.ui.tertiary.teal.buttons .button:active,
.ui.tertiary.teal.buttons .tertiary.button:active,
.ui.tertiary.teal.button:active {
  box-shadow: inset 0 -0.2em 0 #009c95;
  color: #009c95;
}

.ui.blue.buttons .button,
.ui.blue.button {
  background-color: #2185d0;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.blue.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.blue.buttons .button:hover,
.ui.blue.button:hover {
  background-color: #1678c2;
  color: #fff;
  text-shadow: none;
}

.ui.blue.buttons .button:focus,
.ui.blue.button:focus {
  background-color: #0d71bb;
  color: #fff;
  text-shadow: none;
}

.ui.blue.buttons .button:active,
.ui.blue.button:active {
  background-color: #1a69a4;
  color: #fff;
  text-shadow: none;
}

.ui.blue.buttons .active.button,
.ui.blue.buttons .active.button:active,
.ui.blue.active.button,
.ui.blue.button .active.button:active {
  background-color: #1279c6;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.blue.buttons .button,
.ui.basic.blue.button {
  background: transparent;
  box-shadow: 0 0 0 1px #2185d0 inset;
  color: #2185d0;
}

.ui.basic.blue.buttons .button:hover,
.ui.basic.blue.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #1678c2 inset;
  color: #1678c2;
}

.ui.basic.blue.buttons .button:focus,
.ui.basic.blue.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #0d71bb inset;
  color: #1678c2;
}

.ui.basic.blue.buttons .active.button,
.ui.basic.blue.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #1279c6 inset;
  color: #1a69a4;
}

.ui.basic.blue.buttons .button:active,
.ui.basic.blue.button:active {
  box-shadow: 0 0 0 1px #1a69a4 inset;
  color: #1a69a4;
}

/* Inverted */

.ui.inverted.blue.buttons .button,
.ui.inverted.blue.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #54c8ff inset;
  color: #54c8ff;
}

.ui.inverted.blue.buttons .button:hover,
.ui.inverted.blue.button:hover,
.ui.inverted.blue.buttons .button:focus,
.ui.inverted.blue.button:focus,
.ui.inverted.blue.buttons .button.active,
.ui.inverted.blue.button.active,
.ui.inverted.blue.buttons .button:active,
.ui.inverted.blue.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.blue.buttons .button:hover,
.ui.inverted.blue.button:hover {
  background-color: #21b8ff;
}

.ui.inverted.blue.buttons .button:focus,
.ui.inverted.blue.button:focus {
  background-color: #2bbbff;
}

.ui.inverted.blue.buttons .active.button,
.ui.inverted.blue.active.button {
  background-color: #3ac0ff;
}

.ui.inverted.blue.buttons .button:active,
.ui.inverted.blue.button:active {
  background-color: #21b8ff;
}

/* Inverted Basic */

.ui.inverted.blue.basic.buttons .button,
.ui.inverted.blue.buttons .basic.button,
.ui.inverted.blue.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.blue.basic.buttons .button:hover,
.ui.inverted.blue.buttons .basic.button:hover,
.ui.inverted.blue.basic.button:hover {
  box-shadow: 0 0 0 2px #21b8ff inset;
  color: #54c8ff;
}

.ui.inverted.blue.basic.buttons .button:focus,
.ui.inverted.blue.basic.buttons .button:focus,
.ui.inverted.blue.basic.button:focus {
  box-shadow: 0 0 0 2px #2bbbff inset;
  color: #54c8ff;
}

.ui.inverted.blue.basic.buttons .active.button,
.ui.inverted.blue.buttons .basic.active.button,
.ui.inverted.blue.basic.active.button {
  box-shadow: 0 0 0 2px #3ac0ff inset;
  color: #54c8ff;
}

.ui.inverted.blue.basic.buttons .button:active,
.ui.inverted.blue.buttons .basic.button:active,
.ui.inverted.blue.basic.button:active {
  box-shadow: 0 0 0 2px #21b8ff inset;
  color: #54c8ff;
}

/* Tertiary */

.ui.tertiary.blue.buttons .button,
.ui.tertiary.blue.buttons .tertiary.button,
.ui.tertiary.blue.button {
  background: transparent;
  box-shadow: none;
  color: #2185d0;
}

.ui.tertiary.blue.buttons .button:hover,
.ui.tertiary.blue.buttons button:hover,
.ui.tertiary.blue.button:hover {
  box-shadow: inset 0 -0.2em 0 #2b75ac;
  color: #2b75ac;
}

.ui.tertiary.blue.buttons .button:focus,
.ui.tertiary.blue.buttons .tertiary.button:focus,
.ui.tertiary.blue.button:focus {
  box-shadow: inset 0 -0.2em 0 #216ea7;
  color: #216ea7;
}

.ui.tertiary.blue.buttons .active.button,
.ui.tertiary.blue.buttons .tertiary.active.button,
.ui.tertiary.blue.active.button,
.ui.tertiary.blue.buttons .button:active,
.ui.tertiary.blue.buttons .tertiary.button:active,
.ui.tertiary.blue.button:active {
  box-shadow: inset 0 -0.2em 0 #007bd8;
  color: #1279c6;
}

.ui.violet.buttons .button,
.ui.violet.button {
  background-color: #6435c9;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.violet.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.violet.buttons .button:hover,
.ui.violet.button:hover {
  background-color: #5829bb;
  color: #fff;
  text-shadow: none;
}

.ui.violet.buttons .button:focus,
.ui.violet.button:focus {
  background-color: #4f20b5;
  color: #fff;
  text-shadow: none;
}

.ui.violet.buttons .button:active,
.ui.violet.button:active {
  background-color: #502aa1;
  color: #fff;
  text-shadow: none;
}

.ui.violet.buttons .active.button,
.ui.violet.buttons .active.button:active,
.ui.violet.active.button,
.ui.violet.button .active.button:active {
  background-color: #5626bf;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.violet.buttons .button,
.ui.basic.violet.button {
  background: transparent;
  box-shadow: 0 0 0 1px #6435c9 inset;
  color: #6435c9;
}

.ui.basic.violet.buttons .button:hover,
.ui.basic.violet.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #5829bb inset;
  color: #5829bb;
}

.ui.basic.violet.buttons .button:focus,
.ui.basic.violet.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #4f20b5 inset;
  color: #5829bb;
}

.ui.basic.violet.buttons .active.button,
.ui.basic.violet.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #5626bf inset;
  color: #502aa1;
}

.ui.basic.violet.buttons .button:active,
.ui.basic.violet.button:active {
  box-shadow: 0 0 0 1px #502aa1 inset;
  color: #502aa1;
}

/* Inverted */

.ui.inverted.violet.buttons .button,
.ui.inverted.violet.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #a291fb inset;
  color: #a291fb;
}

.ui.inverted.violet.buttons .button:hover,
.ui.inverted.violet.button:hover,
.ui.inverted.violet.buttons .button:focus,
.ui.inverted.violet.button:focus,
.ui.inverted.violet.buttons .button.active,
.ui.inverted.violet.button.active,
.ui.inverted.violet.buttons .button:active,
.ui.inverted.violet.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.violet.buttons .button:hover,
.ui.inverted.violet.button:hover {
  background-color: #745aff;
}

.ui.inverted.violet.buttons .button:focus,
.ui.inverted.violet.button:focus {
  background-color: #7d64ff;
}

.ui.inverted.violet.buttons .active.button,
.ui.inverted.violet.active.button {
  background-color: #8a73ff;
}

.ui.inverted.violet.buttons .button:active,
.ui.inverted.violet.button:active {
  background-color: #7860f9;
}

/* Inverted Basic */

.ui.inverted.violet.basic.buttons .button,
.ui.inverted.violet.buttons .basic.button,
.ui.inverted.violet.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.violet.basic.buttons .button:hover,
.ui.inverted.violet.buttons .basic.button:hover,
.ui.inverted.violet.basic.button:hover {
  box-shadow: 0 0 0 2px #745aff inset;
  color: #a291fb;
}

.ui.inverted.violet.basic.buttons .button:focus,
.ui.inverted.violet.basic.buttons .button:focus,
.ui.inverted.violet.basic.button:focus {
  box-shadow: 0 0 0 2px #7d64ff inset;
  color: #a291fb;
}

.ui.inverted.violet.basic.buttons .active.button,
.ui.inverted.violet.buttons .basic.active.button,
.ui.inverted.violet.basic.active.button {
  box-shadow: 0 0 0 2px #8a73ff inset;
  color: #a291fb;
}

.ui.inverted.violet.basic.buttons .button:active,
.ui.inverted.violet.buttons .basic.button:active,
.ui.inverted.violet.basic.button:active {
  box-shadow: 0 0 0 2px #7860f9 inset;
  color: #a291fb;
}

/* Tertiary */

.ui.tertiary.violet.buttons .button,
.ui.tertiary.violet.buttons .tertiary.button,
.ui.tertiary.violet.button {
  background: transparent;
  box-shadow: none;
  color: #6435c9;
}

.ui.tertiary.violet.buttons .button:hover,
.ui.tertiary.violet.buttons button:hover,
.ui.tertiary.violet.button:hover {
  box-shadow: inset 0 -0.2em 0 #6040a5;
  color: #6040a5;
}

.ui.tertiary.violet.buttons .button:focus,
.ui.tertiary.violet.buttons .tertiary.button:focus,
.ui.tertiary.violet.button:focus {
  box-shadow: inset 0 -0.2em 0 #5735a0;
  color: #5735a0;
}

.ui.tertiary.violet.buttons .active.button,
.ui.tertiary.violet.buttons .tertiary.active.button,
.ui.tertiary.violet.active.button,
.ui.tertiary.violet.buttons .button:active,
.ui.tertiary.violet.buttons .tertiary.button:active,
.ui.tertiary.violet.button:active {
  box-shadow: inset 0 -0.2em 0 #4e0fd6;
  color: #5626bf;
}

.ui.purple.buttons .button,
.ui.purple.button {
  background-color: #a333c8;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.purple.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.purple.buttons .button:hover,
.ui.purple.button:hover {
  background-color: #9627ba;
  color: #fff;
  text-shadow: none;
}

.ui.purple.buttons .button:focus,
.ui.purple.button:focus {
  background-color: #8f1eb4;
  color: #fff;
  text-shadow: none;
}

.ui.purple.buttons .button:active,
.ui.purple.button:active {
  background-color: #82299f;
  color: #fff;
  text-shadow: none;
}

.ui.purple.buttons .active.button,
.ui.purple.buttons .active.button:active,
.ui.purple.active.button,
.ui.purple.button .active.button:active {
  background-color: #9724be;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.purple.buttons .button,
.ui.basic.purple.button {
  background: transparent;
  box-shadow: 0 0 0 1px #a333c8 inset;
  color: #a333c8;
}

.ui.basic.purple.buttons .button:hover,
.ui.basic.purple.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #9627ba inset;
  color: #9627ba;
}

.ui.basic.purple.buttons .button:focus,
.ui.basic.purple.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #8f1eb4 inset;
  color: #9627ba;
}

.ui.basic.purple.buttons .active.button,
.ui.basic.purple.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #9724be inset;
  color: #82299f;
}

.ui.basic.purple.buttons .button:active,
.ui.basic.purple.button:active {
  box-shadow: 0 0 0 1px #82299f inset;
  color: #82299f;
}

/* Inverted */

.ui.inverted.purple.buttons .button,
.ui.inverted.purple.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #dc73ff inset;
  color: #dc73ff;
}

.ui.inverted.purple.buttons .button:hover,
.ui.inverted.purple.button:hover,
.ui.inverted.purple.buttons .button:focus,
.ui.inverted.purple.button:focus,
.ui.inverted.purple.buttons .button.active,
.ui.inverted.purple.button.active,
.ui.inverted.purple.buttons .button:active,
.ui.inverted.purple.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.purple.buttons .button:hover,
.ui.inverted.purple.button:hover {
  background-color: #cf40ff;
}

.ui.inverted.purple.buttons .button:focus,
.ui.inverted.purple.button:focus {
  background-color: #d24aff;
}

.ui.inverted.purple.buttons .active.button,
.ui.inverted.purple.active.button {
  background-color: #d65aff;
}

.ui.inverted.purple.buttons .button:active,
.ui.inverted.purple.button:active {
  background-color: #cf40ff;
}

/* Inverted Basic */

.ui.inverted.purple.basic.buttons .button,
.ui.inverted.purple.buttons .basic.button,
.ui.inverted.purple.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.purple.basic.buttons .button:hover,
.ui.inverted.purple.buttons .basic.button:hover,
.ui.inverted.purple.basic.button:hover {
  box-shadow: 0 0 0 2px #cf40ff inset;
  color: #dc73ff;
}

.ui.inverted.purple.basic.buttons .button:focus,
.ui.inverted.purple.basic.buttons .button:focus,
.ui.inverted.purple.basic.button:focus {
  box-shadow: 0 0 0 2px #d24aff inset;
  color: #dc73ff;
}

.ui.inverted.purple.basic.buttons .active.button,
.ui.inverted.purple.buttons .basic.active.button,
.ui.inverted.purple.basic.active.button {
  box-shadow: 0 0 0 2px #d65aff inset;
  color: #dc73ff;
}

.ui.inverted.purple.basic.buttons .button:active,
.ui.inverted.purple.buttons .basic.button:active,
.ui.inverted.purple.basic.button:active {
  box-shadow: 0 0 0 2px #cf40ff inset;
  color: #dc73ff;
}

/* Tertiary */

.ui.tertiary.purple.buttons .button,
.ui.tertiary.purple.buttons .tertiary.button,
.ui.tertiary.purple.button {
  background: transparent;
  box-shadow: none;
  color: #a333c8;
}

.ui.tertiary.purple.buttons .button:hover,
.ui.tertiary.purple.buttons button:hover,
.ui.tertiary.purple.button:hover {
  box-shadow: inset 0 -0.2em 0 #8a3ea4;
  color: #8a3ea4;
}

.ui.tertiary.purple.buttons .button:focus,
.ui.tertiary.purple.buttons .tertiary.button:focus,
.ui.tertiary.purple.button:focus {
  box-shadow: inset 0 -0.2em 0 #84339f;
  color: #84339f;
}

.ui.tertiary.purple.buttons .active.button,
.ui.tertiary.purple.buttons .tertiary.active.button,
.ui.tertiary.purple.active.button,
.ui.tertiary.purple.buttons .button:active,
.ui.tertiary.purple.buttons .tertiary.button:active,
.ui.tertiary.purple.button:active {
  box-shadow: inset 0 -0.2em 0 #a30dd4;
  color: #9724be;
}

.ui.pink.buttons .button,
.ui.pink.button {
  background-color: #e03997;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.pink.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.pink.buttons .button:hover,
.ui.pink.button:hover {
  background-color: #e61a8d;
  color: #fff;
  text-shadow: none;
}

.ui.pink.buttons .button:focus,
.ui.pink.button:focus {
  background-color: #e10f85;
  color: #fff;
  text-shadow: none;
}

.ui.pink.buttons .button:active,
.ui.pink.button:active {
  background-color: #c71f7e;
  color: #fff;
  text-shadow: none;
}

.ui.pink.buttons .active.button,
.ui.pink.buttons .active.button:active,
.ui.pink.active.button,
.ui.pink.button .active.button:active {
  background-color: #ea158d;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.pink.buttons .button,
.ui.basic.pink.button {
  background: transparent;
  box-shadow: 0 0 0 1px #e03997 inset;
  color: #e03997;
}

.ui.basic.pink.buttons .button:hover,
.ui.basic.pink.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #e61a8d inset;
  color: #e61a8d;
}

.ui.basic.pink.buttons .button:focus,
.ui.basic.pink.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #e10f85 inset;
  color: #e61a8d;
}

.ui.basic.pink.buttons .active.button,
.ui.basic.pink.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #ea158d inset;
  color: #c71f7e;
}

.ui.basic.pink.buttons .button:active,
.ui.basic.pink.button:active {
  box-shadow: 0 0 0 1px #c71f7e inset;
  color: #c71f7e;
}

/* Inverted */

.ui.inverted.pink.buttons .button,
.ui.inverted.pink.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #ff8edf inset;
  color: #ff8edf;
}

.ui.inverted.pink.buttons .button:hover,
.ui.inverted.pink.button:hover,
.ui.inverted.pink.buttons .button:focus,
.ui.inverted.pink.button:focus,
.ui.inverted.pink.buttons .button.active,
.ui.inverted.pink.button.active,
.ui.inverted.pink.buttons .button:active,
.ui.inverted.pink.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.pink.buttons .button:hover,
.ui.inverted.pink.button:hover {
  background-color: #ff5bd1;
}

.ui.inverted.pink.buttons .button:focus,
.ui.inverted.pink.button:focus {
  background-color: #ff65d3;
}

.ui.inverted.pink.buttons .active.button,
.ui.inverted.pink.active.button {
  background-color: #ff74d8;
}

.ui.inverted.pink.buttons .button:active,
.ui.inverted.pink.button:active {
  background-color: #ff5bd1;
}

/* Inverted Basic */

.ui.inverted.pink.basic.buttons .button,
.ui.inverted.pink.buttons .basic.button,
.ui.inverted.pink.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.pink.basic.buttons .button:hover,
.ui.inverted.pink.buttons .basic.button:hover,
.ui.inverted.pink.basic.button:hover {
  box-shadow: 0 0 0 2px #ff5bd1 inset;
  color: #ff8edf;
}

.ui.inverted.pink.basic.buttons .button:focus,
.ui.inverted.pink.basic.buttons .button:focus,
.ui.inverted.pink.basic.button:focus {
  box-shadow: 0 0 0 2px #ff65d3 inset;
  color: #ff8edf;
}

.ui.inverted.pink.basic.buttons .active.button,
.ui.inverted.pink.buttons .basic.active.button,
.ui.inverted.pink.basic.active.button {
  box-shadow: 0 0 0 2px #ff74d8 inset;
  color: #ff8edf;
}

.ui.inverted.pink.basic.buttons .button:active,
.ui.inverted.pink.buttons .basic.button:active,
.ui.inverted.pink.basic.button:active {
  box-shadow: 0 0 0 2px #ff5bd1 inset;
  color: #ff8edf;
}

/* Tertiary */

.ui.tertiary.pink.buttons .button,
.ui.tertiary.pink.buttons .tertiary.button,
.ui.tertiary.pink.button {
  background: transparent;
  box-shadow: none;
  color: #e03997;
}

.ui.tertiary.pink.buttons .button:hover,
.ui.tertiary.pink.buttons button:hover,
.ui.tertiary.pink.button:hover {
  box-shadow: inset 0 -0.2em 0 #cc3389;
  color: #cc3389;
}

.ui.tertiary.pink.buttons .button:focus,
.ui.tertiary.pink.buttons .tertiary.button:focus,
.ui.tertiary.pink.button:focus {
  box-shadow: inset 0 -0.2em 0 #c92782;
  color: #c92782;
}

.ui.tertiary.pink.buttons .active.button,
.ui.tertiary.pink.buttons .tertiary.active.button,
.ui.tertiary.pink.active.button,
.ui.tertiary.pink.buttons .button:active,
.ui.tertiary.pink.buttons .tertiary.button:active,
.ui.tertiary.pink.button:active {
  box-shadow: inset 0 -0.2em 0 #ff0090;
  color: #ea158d;
}

.ui.brown.buttons .button,
.ui.brown.button {
  background-color: #a5673f;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.brown.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.brown.buttons .button:hover,
.ui.brown.button:hover {
  background-color: #975b33;
  color: #fff;
  text-shadow: none;
}

.ui.brown.buttons .button:focus,
.ui.brown.button:focus {
  background-color: #90532b;
  color: #fff;
  text-shadow: none;
}

.ui.brown.buttons .button:active,
.ui.brown.button:active {
  background-color: #805031;
  color: #fff;
  text-shadow: none;
}

.ui.brown.buttons .active.button,
.ui.brown.buttons .active.button:active,
.ui.brown.active.button,
.ui.brown.button .active.button:active {
  background-color: #995a31;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.brown.buttons .button,
.ui.basic.brown.button {
  background: transparent;
  box-shadow: 0 0 0 1px #a5673f inset;
  color: #a5673f;
}

.ui.basic.brown.buttons .button:hover,
.ui.basic.brown.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #975b33 inset;
  color: #975b33;
}

.ui.basic.brown.buttons .button:focus,
.ui.basic.brown.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #90532b inset;
  color: #975b33;
}

.ui.basic.brown.buttons .active.button,
.ui.basic.brown.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #995a31 inset;
  color: #805031;
}

.ui.basic.brown.buttons .button:active,
.ui.basic.brown.button:active {
  box-shadow: 0 0 0 1px #805031 inset;
  color: #805031;
}

/* Inverted */

.ui.inverted.brown.buttons .button,
.ui.inverted.brown.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #d67c1c inset;
  color: #d67c1c;
}

.ui.inverted.brown.buttons .button:hover,
.ui.inverted.brown.button:hover,
.ui.inverted.brown.buttons .button:focus,
.ui.inverted.brown.button:focus,
.ui.inverted.brown.buttons .button.active,
.ui.inverted.brown.button.active,
.ui.inverted.brown.buttons .button:active,
.ui.inverted.brown.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.brown.buttons .button:hover,
.ui.inverted.brown.button:hover {
  background-color: #b0620f;
}

.ui.inverted.brown.buttons .button:focus,
.ui.inverted.brown.button:focus {
  background-color: #c16808;
}

.ui.inverted.brown.buttons .active.button,
.ui.inverted.brown.active.button {
  background-color: #cc6f0d;
}

.ui.inverted.brown.buttons .button:active,
.ui.inverted.brown.button:active {
  background-color: #a96216;
}

/* Inverted Basic */

.ui.inverted.brown.basic.buttons .button,
.ui.inverted.brown.buttons .basic.button,
.ui.inverted.brown.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.brown.basic.buttons .button:hover,
.ui.inverted.brown.buttons .basic.button:hover,
.ui.inverted.brown.basic.button:hover {
  box-shadow: 0 0 0 2px #b0620f inset;
  color: #d67c1c;
}

.ui.inverted.brown.basic.buttons .button:focus,
.ui.inverted.brown.basic.buttons .button:focus,
.ui.inverted.brown.basic.button:focus {
  box-shadow: 0 0 0 2px #c16808 inset;
  color: #d67c1c;
}

.ui.inverted.brown.basic.buttons .active.button,
.ui.inverted.brown.buttons .basic.active.button,
.ui.inverted.brown.basic.active.button {
  box-shadow: 0 0 0 2px #cc6f0d inset;
  color: #d67c1c;
}

.ui.inverted.brown.basic.buttons .button:active,
.ui.inverted.brown.buttons .basic.button:active,
.ui.inverted.brown.basic.button:active {
  box-shadow: 0 0 0 2px #a96216 inset;
  color: #d67c1c;
}

/* Tertiary */

.ui.tertiary.brown.buttons .button,
.ui.tertiary.brown.buttons .tertiary.button,
.ui.tertiary.brown.button {
  background: transparent;
  box-shadow: none;
  color: #a5673f;
}

.ui.tertiary.brown.buttons .button:hover,
.ui.tertiary.brown.buttons button:hover,
.ui.tertiary.brown.button:hover {
  box-shadow: inset 0 -0.2em 0 #835f48;
  color: #835f48;
}

.ui.tertiary.brown.buttons .button:focus,
.ui.tertiary.brown.buttons .tertiary.button:focus,
.ui.tertiary.brown.button:focus {
  box-shadow: inset 0 -0.2em 0 #7d573e;
  color: #7d573e;
}

.ui.tertiary.brown.buttons .active.button,
.ui.tertiary.brown.buttons .tertiary.active.button,
.ui.tertiary.brown.active.button,
.ui.tertiary.brown.buttons .button:active,
.ui.tertiary.brown.buttons .tertiary.button:active,
.ui.tertiary.brown.button:active {
  box-shadow: inset 0 -0.2em 0 #ae561d;
  color: #995a31;
}

.ui.grey.buttons .button,
.ui.grey.button {
  background-color: #767676;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.grey.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.grey.buttons .button:hover,
.ui.grey.button:hover {
  background-color: #838383;
  color: #fff;
  text-shadow: none;
}

.ui.grey.buttons .button:focus,
.ui.grey.button:focus {
  background-color: #8a8a8a;
  color: #fff;
  text-shadow: none;
}

.ui.grey.buttons .button:active,
.ui.grey.button:active {
  background-color: #909090;
  color: #fff;
  text-shadow: none;
}

.ui.grey.buttons .active.button,
.ui.grey.buttons .active.button:active,
.ui.grey.active.button,
.ui.grey.button .active.button:active {
  background-color: #696969;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.grey.buttons .button,
.ui.basic.grey.button {
  background: transparent;
  box-shadow: 0 0 0 1px #767676 inset;
  color: #767676;
}

.ui.basic.grey.buttons .button:hover,
.ui.basic.grey.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #838383 inset;
  color: #838383;
}

.ui.basic.grey.buttons .button:focus,
.ui.basic.grey.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #8a8a8a inset;
  color: #838383;
}

.ui.basic.grey.buttons .active.button,
.ui.basic.grey.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #696969 inset;
  color: #909090;
}

.ui.basic.grey.buttons .button:active,
.ui.basic.grey.button:active {
  box-shadow: 0 0 0 1px #909090 inset;
  color: #909090;
}

/* Inverted */

.ui.inverted.grey.buttons .button,
.ui.inverted.grey.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #d4d4d5 inset;
  color: #fff;
}

.ui.inverted.grey.buttons .button:hover,
.ui.inverted.grey.button:hover,
.ui.inverted.grey.buttons .button:focus,
.ui.inverted.grey.button:focus,
.ui.inverted.grey.buttons .button.active,
.ui.inverted.grey.button.active,
.ui.inverted.grey.buttons .button:active,
.ui.inverted.grey.button:active {
  box-shadow: none;
  color: rgba(0, 0, 0, 0.6);
}

.ui.inverted.grey.buttons .button:hover,
.ui.inverted.grey.button:hover {
  background-color: #c2c4c5;
}

.ui.inverted.grey.buttons .button:focus,
.ui.inverted.grey.button:focus {
  background-color: #c7c9cb;
}

.ui.inverted.grey.buttons .active.button,
.ui.inverted.grey.active.button {
  background-color: #cfd0d2;
}

.ui.inverted.grey.buttons .button:active,
.ui.inverted.grey.button:active {
  background-color: #c2c4c5;
}

/* Inverted Basic */

.ui.inverted.grey.basic.buttons .button,
.ui.inverted.grey.buttons .basic.button,
.ui.inverted.grey.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.grey.basic.buttons .button:hover,
.ui.inverted.grey.buttons .basic.button:hover,
.ui.inverted.grey.basic.button:hover {
  box-shadow: 0 0 0 2px #c2c4c5 inset;
  color: #fff;
}

.ui.inverted.grey.basic.buttons .button:focus,
.ui.inverted.grey.basic.buttons .button:focus,
.ui.inverted.grey.basic.button:focus {
  box-shadow: 0 0 0 2px #c7c9cb inset;
  color: #dcddde;
}

.ui.inverted.grey.basic.buttons .active.button,
.ui.inverted.grey.buttons .basic.active.button,
.ui.inverted.grey.basic.active.button {
  box-shadow: 0 0 0 2px #cfd0d2 inset;
  color: #fff;
}

.ui.inverted.grey.basic.buttons .button:active,
.ui.inverted.grey.buttons .basic.button:active,
.ui.inverted.grey.basic.button:active {
  box-shadow: 0 0 0 2px #c2c4c5 inset;
  color: #fff;
}

/* Tertiary */

.ui.tertiary.grey.buttons .button,
.ui.tertiary.grey.buttons .tertiary.button,
.ui.tertiary.grey.button {
  background: transparent;
  box-shadow: none;
  color: #767676;
}

.ui.tertiary.grey.buttons .button:hover,
.ui.tertiary.grey.buttons button:hover,
.ui.tertiary.grey.button:hover {
  box-shadow: inset 0 -0.2em 0 #909090;
  color: #909090;
}

.ui.tertiary.grey.buttons .button:focus,
.ui.tertiary.grey.buttons .tertiary.button:focus,
.ui.tertiary.grey.button:focus {
  box-shadow: inset 0 -0.2em 0 #9f9f9f;
  color: #9f9f9f;
}

.ui.tertiary.grey.buttons .active.button,
.ui.tertiary.grey.buttons .tertiary.active.button,
.ui.tertiary.grey.active.button,
.ui.tertiary.grey.buttons .button:active,
.ui.tertiary.grey.buttons .tertiary.button:active,
.ui.tertiary.grey.button:active {
  box-shadow: inset 0 -0.2em 0 #5d5d5d;
  color: #696969;
}

.ui.black.buttons .button,
.ui.black.button {
  background-color: #1b1c1d;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.black.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.black.buttons .button:hover,
.ui.black.button:hover {
  background-color: #27292a;
  color: #fff;
  text-shadow: none;
}

.ui.black.buttons .button:focus,
.ui.black.button:focus {
  background-color: #2f3032;
  color: #fff;
  text-shadow: none;
}

.ui.black.buttons .button:active,
.ui.black.button:active {
  background-color: #343637;
  color: #fff;
  text-shadow: none;
}

.ui.black.buttons .active.button,
.ui.black.buttons .active.button:active,
.ui.black.active.button,
.ui.black.button .active.button:active {
  background-color: #0f0f10;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.black.buttons .button,
.ui.basic.black.button {
  background: transparent;
  box-shadow: 0 0 0 1px #1b1c1d inset;
  color: #1b1c1d;
}

.ui.basic.black.buttons .button:hover,
.ui.basic.black.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #27292a inset;
  color: #27292a;
}

.ui.basic.black.buttons .button:focus,
.ui.basic.black.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #2f3032 inset;
  color: #27292a;
}

.ui.basic.black.buttons .active.button,
.ui.basic.black.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #0f0f10 inset;
  color: #343637;
}

.ui.basic.black.buttons .button:active,
.ui.basic.black.button:active {
  box-shadow: 0 0 0 1px #343637 inset;
  color: #343637;
}

/* Inverted */

.ui.inverted.black.buttons .button,
.ui.inverted.black.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px #d4d4d5 inset;
  color: #fff;
}

.ui.inverted.black.buttons .button:hover,
.ui.inverted.black.button:hover,
.ui.inverted.black.buttons .button:focus,
.ui.inverted.black.button:focus,
.ui.inverted.black.buttons .button.active,
.ui.inverted.black.button.active,
.ui.inverted.black.buttons .button:active,
.ui.inverted.black.button:active {
  box-shadow: none;
  color: #fff;
}

.ui.inverted.black.buttons .button:hover,
.ui.inverted.black.button:hover {
  background-color: #000000;
}

.ui.inverted.black.buttons .button:focus,
.ui.inverted.black.button:focus {
  background-color: #000000;
}

.ui.inverted.black.buttons .active.button,
.ui.inverted.black.active.button {
  background-color: #000000;
}

.ui.inverted.black.buttons .button:active,
.ui.inverted.black.button:active {
  background-color: #000000;
}

/* Inverted Basic */

.ui.inverted.black.basic.buttons .button,
.ui.inverted.black.buttons .basic.button,
.ui.inverted.black.basic.button {
  background-color: transparent;
  box-shadow: 0 0 0 2px rgba(255, 255, 255, 0.5) inset;
  color: #fff;
}

.ui.inverted.black.basic.buttons .button:hover,
.ui.inverted.black.buttons .basic.button:hover,
.ui.inverted.black.basic.button:hover {
  box-shadow: 0 0 0 2px #000000 inset;
  color: #fff;
}

.ui.inverted.black.basic.buttons .button:focus,
.ui.inverted.black.basic.buttons .button:focus,
.ui.inverted.black.basic.button:focus {
  box-shadow: 0 0 0 2px #000000 inset;
  color: #545454;
}

.ui.inverted.black.basic.buttons .active.button,
.ui.inverted.black.buttons .basic.active.button,
.ui.inverted.black.basic.active.button {
  box-shadow: 0 0 0 2px #000000 inset;
  color: #fff;
}

.ui.inverted.black.basic.buttons .button:active,
.ui.inverted.black.buttons .basic.button:active,
.ui.inverted.black.basic.button:active {
  box-shadow: 0 0 0 2px #000000 inset;
  color: #fff;
}

/* Tertiary */

.ui.tertiary.black.buttons .button,
.ui.tertiary.black.buttons .tertiary.button,
.ui.tertiary.black.button {
  background: transparent;
  box-shadow: none;
  color: #1b1c1d;
}

.ui.tertiary.black.buttons .button:hover,
.ui.tertiary.black.buttons button:hover,
.ui.tertiary.black.button:hover {
  box-shadow: inset 0 -0.2em 0 #8b8f93;
  color: #8b8f93;
}

.ui.tertiary.black.buttons .button:focus,
.ui.tertiary.black.buttons .tertiary.button:focus,
.ui.tertiary.black.button:focus {
  box-shadow: inset 0 -0.2em 0 #93969a;
  color: #93969a;
}

.ui.tertiary.black.buttons .active.button,
.ui.tertiary.black.buttons .tertiary.active.button,
.ui.tertiary.black.active.button,
.ui.tertiary.black.buttons .button:active,
.ui.tertiary.black.buttons .tertiary.button:active,
.ui.tertiary.black.button:active {
  box-shadow: inset 0 -0.2em 0 #404245;
  color: #0f0f10;
}

/* Standard */

.ui.positive.buttons .button,
.ui.positive.button {
  background-color: #21ba45;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.positive.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.positive.buttons .button:hover,
.ui.positive.button:hover {
  background-color: #16ab39;
  color: #fff;
  text-shadow: none;
}

.ui.positive.buttons .button:focus,
.ui.positive.button:focus {
  background-color: #0ea432;
  color: #fff;
  text-shadow: none;
}

.ui.positive.buttons .button:active,
.ui.positive.button:active {
  background-color: #198f35;
  color: #fff;
  text-shadow: none;
}

.ui.positive.buttons .active.button,
.ui.positive.buttons .active.button:active,
.ui.positive.active.button,
.ui.positive.button .active.button:active {
  background-color: #13ae38;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.positive.buttons .button,
.ui.basic.positive.button {
  background: transparent;
  box-shadow: 0 0 0 1px #21ba45 inset;
  color: #21ba45;
}

.ui.basic.positive.buttons .button:hover,
.ui.basic.positive.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #16ab39 inset;
  color: #16ab39;
}

.ui.basic.positive.buttons .button:focus,
.ui.basic.positive.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #0ea432 inset;
  color: #16ab39;
}

.ui.basic.positive.buttons .active.button,
.ui.basic.positive.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #13ae38 inset;
  color: #198f35;
}

.ui.basic.positive.buttons .button:active,
.ui.basic.positive.button:active {
  box-shadow: 0 0 0 1px #198f35 inset;
  color: #198f35;
}

.ui.buttons:not(.vertical) > .basic.positive.button:not(:first-child) {
  margin-left: -1px;
}

/* Standard */

.ui.negative.buttons .button,
.ui.negative.button {
  background-color: #db2828;
  color: #fff;
  text-shadow: none;
  background-image: none;
}

.ui.negative.button {
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.negative.buttons .button:hover,
.ui.negative.button:hover {
  background-color: #d01919;
  color: #fff;
  text-shadow: none;
}

.ui.negative.buttons .button:focus,
.ui.negative.button:focus {
  background-color: #ca1010;
  color: #fff;
  text-shadow: none;
}

.ui.negative.buttons .button:active,
.ui.negative.button:active {
  background-color: #b21e1e;
  color: #fff;
  text-shadow: none;
}

.ui.negative.buttons .active.button,
.ui.negative.buttons .active.button:active,
.ui.negative.active.button,
.ui.negative.button .active.button:active {
  background-color: #d41515;
  color: #fff;
  text-shadow: none;
}

/* Basic */

.ui.basic.negative.buttons .button,
.ui.basic.negative.button {
  background: transparent;
  box-shadow: 0 0 0 1px #db2828 inset;
  color: #db2828;
}

.ui.basic.negative.buttons .button:hover,
.ui.basic.negative.button:hover {
  background: transparent;
  box-shadow: 0 0 0 1px #d01919 inset;
  color: #d01919;
}

.ui.basic.negative.buttons .button:focus,
.ui.basic.negative.button:focus {
  background: transparent;
  box-shadow: 0 0 0 1px #ca1010 inset;
  color: #d01919;
}

.ui.basic.negative.buttons .active.button,
.ui.basic.negative.active.button {
  background: transparent;
  box-shadow: 0 0 0 1px #d41515 inset;
  color: #b21e1e;
}

.ui.basic.negative.buttons .button:active,
.ui.basic.negative.button:active {
  box-shadow: 0 0 0 1px #b21e1e inset;
  color: #b21e1e;
}

.ui.buttons:not(.vertical) > .basic.negative.button:not(:first-child) {
  margin-left: -1px;
}

/*******************************
                Groups
    *******************************/

.ui.buttons {
  display: inline-flex;
  flex-direction: row;
  font-size: 0;
  vertical-align: baseline;
  margin: 0 0.25em 0 0;
}

.ui.buttons:not(.basic):not(.inverted) {
  box-shadow: none;
}

/* Clearfix */

.ui.buttons::after {
  content: ".";
  display: block;
  height: 0;
  clear: both;
  visibility: hidden;
}

/* Standard Group */

.ui.buttons .button {
  flex: 1 0 auto;
  border-radius: 0;
  margin: 0;
}

.ui.buttons:not(.basic):not(.inverted) > .button:not(.basic):not(.inverted) {
  box-shadow: 0 0 0 1px transparent inset, 0 0 0 0 rgba(34, 36, 38, 0.15) inset;
}

.ui.buttons .button:first-child {
  border-left: none;
  margin-left: 0;
  border-top-left-radius: 0.28571429rem;
  border-bottom-left-radius: 0.28571429rem;
}

.ui.buttons .button:last-child {
  border-top-right-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

/* Vertical Style */

.ui.vertical.buttons {
  display: inline-flex;
  flex-direction: column;
}

.ui.vertical.buttons .button {
  display: block;
  float: none;
  width: 100%;
  margin: 0;
  border-radius: 0;
}

.ui.vertical.buttons .button:not(.basic) {
  box-shadow: none;
}

.ui.vertical.buttons .button:first-child {
  border-top-left-radius: 0.28571429rem;
  border-top-right-radius: 0.28571429rem;
}

.ui.vertical.buttons .button:last-child {
  margin-bottom: 0;
  border-bottom-left-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.vertical.buttons .button:only-child {
  border-radius: 0.28571429rem;
}

.ui.vertical.buttons .basic.button:not(:first-child) {
  border-top: none;
}

.ui.wrapping.buttons {
  flex-wrap: wrap;
}

.ui.wrapping.basic.buttons {
  border-bottom: none;
}

.ui.wrapping.basic.buttons .button {
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.wrapping.basic.buttons .button:hover {
  background: transparent !important;
}

.ui.compact.wrapping.buttons .button {
  flex: none;
}

.ui.wrapped.buttons:not(.spaced) {
  border-top-right-radius: 0;
}

.ui.wrapped.buttons:not(.spaced) .button:first-child {
  border-radius: 0.28571429rem 0 0 0;
}

.ui.wrapped.buttons:not(.spaced) .button:last-child {
  border-radius: 0 0 0.28571429rem 0;
}

.ui.wrapped[class*="top attached"].buttons {
  border-radius: 0.28571429rem 0 0 0;
}

.ui.wrapped[class*="top attached"].buttons .button:last-child {
  border-radius: 0;
}

.ui.wrapped[class*="bottom attached"].buttons {
  border-radius: 0 0 0 0.28571429rem;
}

.ui.wrapped[class*="bottom attached"].buttons .button:first-child {
  border-radius: 0;
}

.ui.spaced.buttons .ui.button {
  margin-bottom: 1em;
  margin-right: 1em;
  border-radius: 0.28571429rem;
}

.ui.spaced.basic.buttons {
  border: none;
}

.ui.spaced.basic.buttons .button {
  border: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.spaced.basic.buttons .button.basic {
  border: none;
}

.ui.spaced.basic.vertical.buttons .button:first-child {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.spaced.basic.wrapping.buttons .button:not(.basic) {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  border-right: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.spaced.basic.wrapping.buttons .button:not(.basic):first-child {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.spaced.basic.inverted.buttons .basic.button {
  margin-bottom: calc(1em + 1px);
  margin-top: 1px;
}

.ui.basic.buttons:not(.vertical).inverted .button,
.ui.basic.buttons:not(.vertical) .basic.button {
  margin-left: -1px;
  border-right: none;
}

.ui.basic.buttons:not(.vertical):not(.spaced):not(.inverted) .basic.button {
  margin-top: -1px;
  border-bottom: none;
}

.ui.inverted.basic.buttons:not(.spaced) .button,
.ui.attached.basic.buttons:not(.wrapping):not(.inverted) .basic.button {
  margin-bottom: -1px;
}

.ui.basic.buttons:not(.vertical):not(.attached):not(.spaced):not(.wrapping):not(.inverted) .basic.button {
  margin-bottom: -1px;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Container
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Container
*******************************/

/* All Sizes */

.ui.container {
  display: block;
  max-width: 100%;
}

/* Mobile */

@media only screen and (max-width: 767.98px) {
  .ui.ui.ui.container:not(.fluid) {
    width: auto;
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.ui.ui.grid.container {
    width: auto;
  }

  .ui.ui.ui.relaxed.grid.container {
    width: auto;
  }

  .ui.ui.ui.very.relaxed.grid.container {
    width: auto;
  }
}

/* Tablet */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.ui.ui.container:not(.fluid) {
    width: 723px;
    margin-left: auto;
    margin-right: auto;
  }

  .ui.ui.ui.grid.container {
    width: calc(723px + 2rem);
  }

  .ui.ui.ui.relaxed.grid.container {
    width: calc(723px + 3rem);
  }

  .ui.ui.ui.very.relaxed.grid.container {
    width: calc(723px + 5rem);
  }
}

/* Small Monitor */

@media only screen and (min-width: 992px) and (max-width: 1199.98px) {
  .ui.ui.ui.container:not(.fluid) {
    width: 933px;
    margin-left: auto;
    margin-right: auto;
  }

  .ui.ui.ui.grid.container {
    width: calc(933px + 2rem);
  }

  .ui.ui.ui.relaxed.grid.container {
    width: calc(933px + 3rem);
  }

  .ui.ui.ui.very.relaxed.grid.container {
    width: calc(933px + 5rem);
  }
}

/* Large Monitor */

@media only screen and (min-width: 1200px) {
  .ui.ui.ui.container:not(.fluid) {
    width: 1127px;
    margin-left: auto;
    margin-right: auto;
  }

  .ui.ui.ui.grid.container {
    width: calc(1127px + 2rem);
  }

  .ui.ui.ui.relaxed.grid.container {
    width: calc(1127px + 3rem);
  }

  .ui.ui.ui.very.relaxed.grid.container {
    width: calc(1127px + 5rem);
  }
}

/*******************************
             Types
*******************************/

/* Text Container */

.ui.text.container {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  max-width: 700px;
  line-height: 1.5;
  font-size: 1.14285714rem;
}

/* Wide Container */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.ui.ui.wide.container {
    width: 867.6px;
  }
}

@media only screen and (min-width: 992px) and (max-width: 1199.98px) {
  .ui.ui.ui.wide.container {
    width: 1119.6px;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.ui.ui.wide.container {
    width: 1352.4px;
  }
}

/* Fluid */

.ui.fluid.container {
  width: 100%;
}

/*******************************
           Variations
*******************************/

.ui[class*="left aligned"].container {
  text-align: left;
}

.ui[class*="center aligned"].container {
  text-align: center;
}

.ui[class*="right aligned"].container {
  text-align: right;
}

.ui.justified.container {
  text-align: justify;
  -webkit-hyphens: auto;
  -ms-hyphens: auto;
  hyphens: auto;
}

/* --------------
      Scrolling
    --------------- */

.ui.scrolling.container {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

@media only screen and (max-width: 767.98px) {
  .ui.scrolling.container.short {
    max-height: 11.25em;
  }

  .ui.scrolling.container.short.resizable {
    height: 11.25em;
  }

  .ui.scrolling.container[class*="very short"] {
    max-height: 7.5em;
  }

  .ui.scrolling.container[class*="very short"].resizable {
    height: 7.5em;
  }

  .ui.scrolling.container {
    max-height: 15em;
  }

  .ui.scrolling.container.long {
    max-height: 30em;
  }

  .ui.scrolling.container.long.resizable {
    height: 30em;
  }

  .ui.scrolling.container[class*="very long"] {
    max-height: 45em;
  }

  .ui.scrolling.container[class*="very long"].resizable {
    height: 45em;
  }
}

@media only screen and (min-width: 768px) {
  .ui.scrolling.container.short {
    max-height: 13.5em;
  }

  .ui.scrolling.container.short.resizable {
    height: 13.5em;
  }

  .ui.scrolling.container[class*="very short"] {
    max-height: 9em;
  }

  .ui.scrolling.container[class*="very short"].resizable {
    height: 9em;
  }

  .ui.scrolling.container {
    max-height: 18em;
  }

  .ui.scrolling.container.resizable {
    height: 18em;
  }

  .ui.scrolling.container.long {
    max-height: 36em;
  }

  .ui.scrolling.container.long.resizable {
    height: 36em;
  }

  .ui.scrolling.container[class*="very long"] {
    max-height: 54em;
  }

  .ui.scrolling.container[class*="very long"].resizable {
    height: 54em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.scrolling.container.short {
    max-height: 18em;
  }

  .ui.scrolling.container.short.resizable {
    height: 18em;
  }

  .ui.scrolling.container[class*="very short"] {
    max-height: 12em;
  }

  .ui.scrolling.container[class*="very short"].resizable {
    height: 12em;
  }

  .ui.scrolling.container {
    max-height: 24em;
  }

  .ui.scrolling.container.resizable {
    height: 24em;
  }

  .ui.scrolling.container.long {
    max-height: 48em;
  }

  .ui.scrolling.container.long.resizable {
    height: 48em;
  }

  .ui.scrolling.container[class*="very long"] {
    max-height: 72em;
  }

  .ui.scrolling.container[class*="very long"].resizable {
    height: 72em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.scrolling.container.short {
    max-height: 22.5em;
  }

  .ui.scrolling.container.short.resizable {
    height: 22.5em;
  }

  .ui.scrolling.container[class*="very short"] {
    max-height: 15em;
  }

  .ui.scrolling.container[class*="very short"].resizable {
    height: 15em;
  }

  .ui.scrolling.container {
    max-height: 30em;
  }

  .ui.scrolling.container.resizable {
    height: 30em;
  }

  .ui.scrolling.container.long {
    max-height: 60em;
  }

  .ui.scrolling.container.long.resizable {
    height: 60em;
  }

  .ui.scrolling.container[class*="very long"] {
    max-height: 90em;
  }

  .ui.scrolling.container[class*="very long"].resizable {
    height: 90em;
  }
}

.ui.resizable.scrolling.container {
  resize: vertical;
  max-height: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Divider
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Divider
*******************************/

.ui.divider {
  margin: 1rem 0;
  line-height: 1;
  height: 0;
  font-weight: bold;
  text-transform: uppercase;
  letter-spacing: 0.05em;
  color: rgba(0, 0, 0, 0.85);
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}

/* --------------
      Basic
--------------- */

.ui.divider:not(.vertical):not(.horizontal) {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  border-bottom: 1px solid rgba(255, 255, 255, 0.1);
}

/* --------------
    Coupling
--------------- */

/* Allow divider between each column row */

.ui.grid > .column + .divider,
.ui.grid > .row > .column + .divider {
  left: auto;
}

/* --------------
       Horizontal
    --------------- */

.ui.horizontal.divider {
  display: table;
  white-space: nowrap;
  height: auto;
  margin: "";
  line-height: 1;
  text-align: center;
}

.ui.horizontal.divider::before,
.ui.horizontal.divider::after {
  content: "";
  display: table-cell;
  position: relative;
  top: 50%;
  width: 50%;
  background-repeat: no-repeat;
}

.ui.horizontal.divider::before {
  background-position: right 1em top 50%;
}

.ui.horizontal.divider::after {
  background-position: left 1em top 50%;
}

/* --------------
        Vertical
    --------------- */

.ui.vertical.divider {
  position: absolute;
  z-index: 2;
  top: 50%;
  left: 50%;
  margin: 0;
  padding: 0;
  width: auto;
  height: 50%;
  line-height: 0;
  text-align: center;
  transform: translateX(-50%);
}

.ui.vertical.divider::before,
.ui.vertical.divider::after {
  position: absolute;
  left: 50%;
  content: "";
  z-index: 3;
  border-left: 1px solid rgba(34, 36, 38, 0.15);
  border-right: 1px solid rgba(255, 255, 255, 0.1);
  width: 0;
  height: calc(100% - 1rem);
}

.ui.vertical.divider::before {
  top: -100%;
}

.ui.vertical.divider::after {
  top: auto;
  bottom: 0;
}

/* Inside grid */

@media only screen and (max-width: 767.98px) {
  .ui.stackable.grid .ui.vertical.divider,
  .ui.grid .stackable.row .ui.vertical.divider {
    display: table;
    white-space: nowrap;
    height: auto;
    margin: "";
    overflow: hidden;
    line-height: 1;
    text-align: center;
    position: static;
    top: 0;
    left: 0;
    transform: none;
  }

  .ui.stackable.grid .ui.vertical.divider::before,
  .ui.grid .stackable.row .ui.vertical.divider::before,
  .ui.stackable.grid .ui.vertical.divider::after,
  .ui.grid .stackable.row .ui.vertical.divider::after {
    left: 0;
    border-left: none;
    border-right: none;
    content: "";
    display: table-cell;
    position: relative;
    top: 50%;
    width: 50%;
    background-repeat: no-repeat;
  }

  .ui.stackable.grid .ui.vertical.divider::before,
  .ui.grid .stackable.row .ui.vertical.divider::before {
    background-position: right 1em top 50%;
  }

  .ui.stackable.grid .ui.vertical.divider::after,
  .ui.grid .stackable.row .ui.vertical.divider::after {
    background-position: left 1em top 50%;
  }
}

/* --------------
          Icon
    --------------- */

.ui.divider > .icon {
  margin: 0;
  font-size: 1rem;
  height: 1em;
  vertical-align: middle;
}

/* --------------
         Header
    --------------- */

.ui.horizontal.divider[class*="left aligned"]::before {
  display: none;
}

.ui.horizontal.divider[class*="left aligned"]::after {
  width: 100%;
}

.ui.horizontal.divider[class*="right aligned"]::before {
  width: 100%;
}

.ui.horizontal.divider[class*="right aligned"]::after {
  display: none;
}

/*******************************
          Variations
*******************************/

/* --------------
        Hidden
    --------------- */

.ui.hidden.divider {
  border-color: transparent !important;
}

.ui.hidden.divider::before,
.ui.hidden.divider::after {
  display: none;
}

/* --------------
    Inverted
--------------- */

.ui.divider.inverted,
.ui.vertical.inverted.divider,
.ui.horizontal.inverted.divider {
  color: #fff;
}

.ui.divider.inverted,
.ui.divider.inverted::after,
.ui.divider.inverted::before {
  border-color: rgba(34, 36, 38, 0.15) rgba(255, 255, 255, 0.15) rgba(255, 255, 255, 0.15) rgba(34, 36, 38, 0.15) !important;
}

/* --------------
    Fitted
--------------- */

.ui.fitted.divider {
  margin: 0;
}

/* --------------
        Clearing
    --------------- */

.ui.clearing.divider {
  clear: both;
}

/* --------------
        Section
    --------------- */

.ui.section.divider {
  margin-top: 2rem;
  margin-bottom: 2rem;
}

/* --------------
     Sizes
--------------- */

.ui.divider {
  font-size: 1rem;
}

.ui.mini.divider {
  font-size: 0.78571429rem;
}

.ui.tiny.divider {
  font-size: 0.85714286rem;
}

.ui.small.divider {
  font-size: 0.92857143rem;
}

.ui.large.divider {
  font-size: 1.14285714rem;
}

.ui.big.divider {
  font-size: 1.28571429rem;
}

.ui.huge.divider {
  font-size: 1.42857143rem;
}

.ui.massive.divider {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

.ui.horizontal.divider::before,
.ui.horizontal.divider::after {
  background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAABaAAAAACCAYAAACuTHuKAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyFpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNS1jMDE0IDc5LjE1MTQ4MSwgMjAxMy8wMy8xMy0xMjowOToxNSAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENDIChXaW5kb3dzKSIgeG1wTU06SW5zdGFuY2VJRD0ieG1wLmlpZDo1OThBRDY4OUNDMTYxMUU0OUE3NUVGOEJDMzMzMjE2NyIgeG1wTU06RG9jdW1lbnRJRD0ieG1wLmRpZDo1OThBRDY4QUNDMTYxMUU0OUE3NUVGOEJDMzMzMjE2NyI+IDx4bXBNTTpEZXJpdmVkRnJvbSBzdFJlZjppbnN0YW5jZUlEPSJ4bXAuaWlkOjU5OEFENjg3Q0MxNjExRTQ5QTc1RUY4QkMzMzMyMTY3IiBzdFJlZjpkb2N1bWVudElEPSJ4bXAuZGlkOjU5OEFENjg4Q0MxNjExRTQ5QTc1RUY4QkMzMzMyMTY3Ii8+IDwvcmRmOkRlc2NyaXB0aW9uPiA8L3JkZjpSREY+IDwveDp4bXBtZXRhPiA8P3hwYWNrZXQgZW5kPSJyIj8+VU513gAAADVJREFUeNrs0DENACAQBDBIWLGBJQby/mUcJn5sJXQmOQMAAAAAAJqt+2prAAAAAACg2xdgANk6BEVuJgyMAAAAAElFTkSuQmCC");
}

@media only screen and (max-width: 767px) {
  .ui.stackable.grid .ui.vertical.divider::before,
  .ui.grid .stackable.row .ui.vertical.divider::before,
  .ui.stackable.grid .ui.vertical.divider::after,
  .ui.grid .stackable.row .ui.vertical.divider::after {
    background-image: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAABaAAAAACCAYAAACuTHuKAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyFpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNS1jMDE0IDc5LjE1MTQ4MSwgMjAxMy8wMy8xMy0xMjowOToxNSAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENDIChXaW5kb3dzKSIgeG1wTU06SW5zdGFuY2VJRD0ieG1wLmlpZDo1OThBRDY4OUNDMTYxMUU0OUE3NUVGOEJDMzMzMjE2NyIgeG1wTU06RG9jdW1lbnRJRD0ieG1wLmRpZDo1OThBRDY4QUNDMTYxMUU0OUE3NUVGOEJDMzMzMjE2NyI+IDx4bXBNTTpEZXJpdmVkRnJvbSBzdFJlZjppbnN0YW5jZUlEPSJ4bXAuaWlkOjU5OEFENjg3Q0MxNjExRTQ5QTc1RUY4QkMzMzMyMTY3IiBzdFJlZjpkb2N1bWVudElEPSJ4bXAuZGlkOjU5OEFENjg4Q0MxNjExRTQ5QTc1RUY4QkMzMzMyMTY3Ii8+IDwvcmRmOkRlc2NyaXB0aW9uPiA8L3JkZjpSREY+IDwveDp4bXBtZXRhPiA8P3hwYWNrZXQgZW5kPSJyIj8+VU513gAAADVJREFUeNrs0DENACAQBDBIWLGBJQby/mUcJn5sJXQmOQMAAAAAAJqt+2prAAAAAACg2xdgANk6BEVuJgyMAAAAAElFTkSuQmCC");
  }
}

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Emoji
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Emoji
*******************************/

em[data-emoji] {
  opacity: 1;
  speak: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

em[data-emoji]::before {
  content: "\00A0\00A0\00A0\00A0\00A0\00A0\00A0";
  display: inline-block;
  line-height: 1em;
  background-repeat: no-repeat;
  background-position: center center;
}

/*******************************
             States
*******************************/

em[data-emoji].disabled {
  opacity: 0.45;
}

/*******************************
           Variations
*******************************/

em[data-emoji].loading::before {
  animation: loader 2s linear infinite;
}

/* -------------------
             Link
    -------------------- */

em[data-emoji].link:not(.disabled) {
  cursor: pointer;
}

/* rtl:begin:ignore */

em[data-emoji].small {
  font-size: 1.5em;
  vertical-align: middle;
}

em[data-emoji].medium {
  font-size: 3em;
  vertical-align: middle;
}

em[data-emoji].large {
  font-size: 6em;
  vertical-align: middle;
}

em[data-emoji].big {
  font-size: 7.5em;
  vertical-align: middle;
}

em[data-emoji=":interrobang:"]::before,
em[data-emoji="interrobang"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2049.svg");
}

em[data-emoji=":tm:"]::before,
em[data-emoji="tm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2122.svg");
}

em[data-emoji=":information_source:"]::before,
em[data-emoji="information_source"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2139.svg");
}

em[data-emoji=":left_right_arrow:"]::before,
em[data-emoji="left_right_arrow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2194.svg");
}

em[data-emoji=":arrow_up_down:"]::before,
em[data-emoji="arrow_up_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2195.svg");
}

em[data-emoji=":arrow_upper_left:"]::before,
em[data-emoji="arrow_upper_left"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2196.svg");
}

em[data-emoji=":arrow_upper_right:"]::before,
em[data-emoji="arrow_upper_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2197.svg");
}

em[data-emoji=":arrow_lower_right:"]::before,
em[data-emoji="arrow_lower_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2198.svg");
}

em[data-emoji=":arrow_lower_left:"]::before,
em[data-emoji="arrow_lower_left"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2199.svg");
}

em[data-emoji=":keyboard:"]::before,
em[data-emoji="keyboard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2328.svg");
}

em[data-emoji=":sunny:"]::before,
em[data-emoji="sunny"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2600.svg");
}

em[data-emoji=":cloud:"]::before,
em[data-emoji="cloud"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2601.svg");
}

em[data-emoji=":umbrella2:"]::before,
em[data-emoji="umbrella2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2602.svg");
}

em[data-emoji=":snowman2:"]::before,
em[data-emoji="snowman2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2603.svg");
}

em[data-emoji=":comet:"]::before,
em[data-emoji="comet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2604.svg");
}

em[data-emoji=":ballot_box_with_check:"]::before,
em[data-emoji="ballot_box_with_check"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2611.svg");
}

em[data-emoji=":umbrella:"]::before,
em[data-emoji="umbrella"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2614.svg");
}

em[data-emoji=":coffee:"]::before,
em[data-emoji="coffee"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2615.svg");
}

em[data-emoji=":shamrock:"]::before,
em[data-emoji="shamrock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2618.svg");
}

em[data-emoji=":skull_crossbones:"]::before,
em[data-emoji="skull_crossbones"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2620.svg");
}

em[data-emoji=":radioactive:"]::before,
em[data-emoji="radioactive"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2622.svg");
}

em[data-emoji=":biohazard:"]::before,
em[data-emoji="biohazard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2623.svg");
}

em[data-emoji=":orthodox_cross:"]::before,
em[data-emoji="orthodox_cross"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2626.svg");
}

em[data-emoji=":wheel_of_dharma:"]::before,
em[data-emoji="wheel_of_dharma"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2638.svg");
}

em[data-emoji=":frowning2:"]::before,
em[data-emoji="frowning2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2639.svg");
}

em[data-emoji=":female_sign:"]::before,
em[data-emoji="female_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2640.svg");
}

em[data-emoji=":male_sign:"]::before,
em[data-emoji="male_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2642.svg");
}

em[data-emoji=":aries:"]::before,
em[data-emoji="aries"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2648.svg");
}

em[data-emoji=":taurus:"]::before,
em[data-emoji="taurus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2649.svg");
}

em[data-emoji=":sagittarius:"]::before,
em[data-emoji="sagittarius"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2650.svg");
}

em[data-emoji=":capricorn:"]::before,
em[data-emoji="capricorn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2651.svg");
}

em[data-emoji=":aquarius:"]::before,
em[data-emoji="aquarius"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2652.svg");
}

em[data-emoji=":pisces:"]::before,
em[data-emoji="pisces"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2653.svg");
}

em[data-emoji=":spades:"]::before,
em[data-emoji="spades"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2660.svg");
}

em[data-emoji=":clubs:"]::before,
em[data-emoji="clubs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2663.svg");
}

em[data-emoji=":hearts:"]::before,
em[data-emoji="hearts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2665.svg");
}

em[data-emoji=":diamonds:"]::before,
em[data-emoji="diamonds"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2666.svg");
}

em[data-emoji=":hotsprings:"]::before,
em[data-emoji="hotsprings"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2668.svg");
}

em[data-emoji=":hammer_pick:"]::before,
em[data-emoji="hammer_pick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2692.svg");
}

em[data-emoji=":anchor:"]::before,
em[data-emoji="anchor"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2693.svg");
}

em[data-emoji=":crossed_swords:"]::before,
em[data-emoji="crossed_swords"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2694.svg");
}

em[data-emoji=":medical_symbol:"]::before,
em[data-emoji="medical_symbol"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2695.svg");
}

em[data-emoji=":scales:"]::before,
em[data-emoji="scales"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2696.svg");
}

em[data-emoji=":alembic:"]::before,
em[data-emoji="alembic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2697.svg");
}

em[data-emoji=":gear:"]::before,
em[data-emoji="gear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2699.svg");
}

em[data-emoji=":scissors:"]::before,
em[data-emoji="scissors"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2702.svg");
}

em[data-emoji=":white_check_mark:"]::before,
em[data-emoji="white_check_mark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2705.svg");
}

em[data-emoji=":airplane:"]::before,
em[data-emoji="airplane"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2708.svg");
}

em[data-emoji=":envelope:"]::before,
em[data-emoji="envelope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2709.svg");
}

em[data-emoji=":black_nib:"]::before,
em[data-emoji="black_nib"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2712.svg");
}

em[data-emoji=":heavy_check_mark:"]::before,
em[data-emoji="heavy_check_mark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2714.svg");
}

em[data-emoji=":heavy_multiplication_x:"]::before,
em[data-emoji="heavy_multiplication_x"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2716.svg");
}

em[data-emoji=":star_of_david:"]::before,
em[data-emoji="star_of_david"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2721.svg");
}

em[data-emoji=":sparkles:"]::before,
em[data-emoji="sparkles"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2728.svg");
}

em[data-emoji=":eight_spoked_asterisk:"]::before,
em[data-emoji="eight_spoked_asterisk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2733.svg");
}

em[data-emoji=":eight_pointed_black_star:"]::before,
em[data-emoji="eight_pointed_black_star"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2734.svg");
}

em[data-emoji=":snowflake:"]::before,
em[data-emoji="snowflake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2744.svg");
}

em[data-emoji=":sparkle:"]::before,
em[data-emoji="sparkle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2747.svg");
}

em[data-emoji=":question:"]::before,
em[data-emoji="question"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2753.svg");
}

em[data-emoji=":grey_question:"]::before,
em[data-emoji="grey_question"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2754.svg");
}

em[data-emoji=":grey_exclamation:"]::before,
em[data-emoji="grey_exclamation"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2755.svg");
}

em[data-emoji=":exclamation:"]::before,
em[data-emoji="exclamation"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2757.svg");
}

em[data-emoji=":heart_exclamation:"]::before,
em[data-emoji="heart_exclamation"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2763.svg");
}

em[data-emoji=":heart:"]::before,
em[data-emoji="heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2764.svg");
}

em[data-emoji=":heavy_plus_sign:"]::before,
em[data-emoji="heavy_plus_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2795.svg");
}

em[data-emoji=":heavy_minus_sign:"]::before,
em[data-emoji="heavy_minus_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2796.svg");
}

em[data-emoji=":heavy_division_sign:"]::before,
em[data-emoji="heavy_division_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2797.svg");
}

em[data-emoji=":arrow_heading_up:"]::before,
em[data-emoji="arrow_heading_up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2934.svg");
}

em[data-emoji=":arrow_heading_down:"]::before,
em[data-emoji="arrow_heading_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2935.svg");
}

em[data-emoji=":wavy_dash:"]::before,
em[data-emoji="wavy_dash"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/3030.svg");
}

em[data-emoji=":congratulations:"]::before,
em[data-emoji="congratulations"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/3297.svg");
}

em[data-emoji=":secret:"]::before,
em[data-emoji="secret"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/3299.svg");
}

em[data-emoji=":grinning:"]::before,
em[data-emoji="grinning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f600.svg");
}

em[data-emoji=":smiley:"]::before,
em[data-emoji="smiley"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f603.svg");
}

em[data-emoji=":smile:"]::before,
em[data-emoji="smile"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f604.svg");
}

em[data-emoji=":grin:"]::before,
em[data-emoji="grin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f601.svg");
}

em[data-emoji=":laughing:"]::before,
em[data-emoji="laughing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f606.svg");
}

em[data-emoji=":face_holding_back_tears:"]::before,
em[data-emoji="face_holding_back_tears"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f979.svg");
}

em[data-emoji=":sweat_smile:"]::before,
em[data-emoji="sweat_smile"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f605.svg");
}

em[data-emoji=":joy:"]::before,
em[data-emoji="joy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f602.svg");
}

em[data-emoji=":rofl:"]::before,
em[data-emoji="rofl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f923.svg");
}

em[data-emoji=":smiling_face_with_tear:"]::before,
em[data-emoji="smiling_face_with_tear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f972.svg");
}

em[data-emoji=":relaxed:"]::before,
em[data-emoji="relaxed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/263a.svg");
}

em[data-emoji=":blush:"]::before,
em[data-emoji="blush"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60a.svg");
}

em[data-emoji=":innocent:"]::before,
em[data-emoji="innocent"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f607.svg");
}

em[data-emoji=":slight_smile:"]::before,
em[data-emoji="slight_smile"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f642.svg");
}

em[data-emoji=":upside_down:"]::before,
em[data-emoji="upside_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f643.svg");
}

em[data-emoji=":wink:"]::before,
em[data-emoji="wink"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f609.svg");
}

em[data-emoji=":relieved:"]::before,
em[data-emoji="relieved"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60c.svg");
}

em[data-emoji=":heart_eyes:"]::before,
em[data-emoji="heart_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60d.svg");
}

em[data-emoji=":smiling_face_with_3_hearts:"]::before,
em[data-emoji="smiling_face_with_3_hearts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f970.svg");
}

em[data-emoji=":kissing_heart:"]::before,
em[data-emoji="kissing_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f618.svg");
}

em[data-emoji=":kissing:"]::before,
em[data-emoji="kissing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f617.svg");
}

em[data-emoji=":kissing_smiling_eyes:"]::before,
em[data-emoji="kissing_smiling_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f619.svg");
}

em[data-emoji=":kissing_closed_eyes:"]::before,
em[data-emoji="kissing_closed_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61a.svg");
}

em[data-emoji=":yum:"]::before,
em[data-emoji="yum"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60b.svg");
}

em[data-emoji=":stuck_out_tongue:"]::before,
em[data-emoji="stuck_out_tongue"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61b.svg");
}

em[data-emoji=":stuck_out_tongue_closed_eyes:"]::before,
em[data-emoji="stuck_out_tongue_closed_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61d.svg");
}

em[data-emoji=":stuck_out_tongue_winking_eye:"]::before,
em[data-emoji="stuck_out_tongue_winking_eye"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61c.svg");
}

em[data-emoji=":zany_face:"]::before,
em[data-emoji="zany_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92a.svg");
}

em[data-emoji=":face_with_raised_eyebrow:"]::before,
em[data-emoji="face_with_raised_eyebrow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f928.svg");
}

em[data-emoji=":face_with_monocle:"]::before,
em[data-emoji="face_with_monocle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d0.svg");
}

em[data-emoji=":nerd:"]::before,
em[data-emoji="nerd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f913.svg");
}

em[data-emoji=":sunglasses:"]::before,
em[data-emoji="sunglasses"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60e.svg");
}

em[data-emoji=":disguised_face:"]::before,
em[data-emoji="disguised_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f978.svg");
}

em[data-emoji=":star_struck:"]::before,
em[data-emoji="star_struck"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f929.svg");
}

em[data-emoji=":partying_face:"]::before,
em[data-emoji="partying_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f973.svg");
}

em[data-emoji=":smirk:"]::before,
em[data-emoji="smirk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f60f.svg");
}

em[data-emoji=":unamused:"]::before,
em[data-emoji="unamused"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f612.svg");
}

em[data-emoji=":disappointed:"]::before,
em[data-emoji="disappointed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61e.svg");
}

em[data-emoji=":pensive:"]::before,
em[data-emoji="pensive"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f614.svg");
}

em[data-emoji=":worried:"]::before,
em[data-emoji="worried"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f61f.svg");
}

em[data-emoji=":confused:"]::before,
em[data-emoji="confused"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f615.svg");
}

em[data-emoji=":slight_frown:"]::before,
em[data-emoji="slight_frown"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f641.svg");
}

em[data-emoji=":persevere:"]::before,
em[data-emoji="persevere"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f623.svg");
}

em[data-emoji=":confounded:"]::before,
em[data-emoji="confounded"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f616.svg");
}

em[data-emoji=":tired_face:"]::before,
em[data-emoji="tired_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62b.svg");
}

em[data-emoji=":weary:"]::before,
em[data-emoji="weary"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f629.svg");
}

em[data-emoji=":pleading_face:"]::before,
em[data-emoji="pleading_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97a.svg");
}

em[data-emoji=":cry:"]::before,
em[data-emoji="cry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f622.svg");
}

em[data-emoji=":sob:"]::before,
em[data-emoji="sob"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62d.svg");
}

em[data-emoji=":triumph:"]::before,
em[data-emoji="triumph"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f624.svg");
}

em[data-emoji=":angry:"]::before,
em[data-emoji="angry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f620.svg");
}

em[data-emoji=":rage:"]::before,
em[data-emoji="rage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f621.svg");
}

em[data-emoji=":face_with_symbols_over_mouth:"]::before,
em[data-emoji="face_with_symbols_over_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92c.svg");
}

em[data-emoji=":exploding_head:"]::before,
em[data-emoji="exploding_head"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92f.svg");
}

em[data-emoji=":flushed:"]::before,
em[data-emoji="flushed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f633.svg");
}

em[data-emoji=":hot_face:"]::before,
em[data-emoji="hot_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f975.svg");
}

em[data-emoji=":cold_face:"]::before,
em[data-emoji="cold_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f976.svg");
}

em[data-emoji=":face_in_clouds:"]::before,
em[data-emoji="face_in_clouds"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f636-200d-1f32b-fe0f.svg");
}

em[data-emoji=":scream:"]::before,
em[data-emoji="scream"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f631.svg");
}

em[data-emoji=":fearful:"]::before,
em[data-emoji="fearful"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f628.svg");
}

em[data-emoji=":cold_sweat:"]::before,
em[data-emoji="cold_sweat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f630.svg");
}

em[data-emoji=":disappointed_relieved:"]::before,
em[data-emoji="disappointed_relieved"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f625.svg");
}

em[data-emoji=":sweat:"]::before,
em[data-emoji="sweat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f613.svg");
}

em[data-emoji=":hugging:"]::before,
em[data-emoji="hugging"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f917.svg");
}

em[data-emoji=":thinking:"]::before,
em[data-emoji="thinking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f914.svg");
}

em[data-emoji=":face_with_peeking_eye:"]::before,
em[data-emoji="face_with_peeking_eye"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae3.svg");
}

em[data-emoji=":face_with_hand_over_mouth:"]::before,
em[data-emoji="face_with_hand_over_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92d.svg");
}

em[data-emoji=":face_with_open_eyes_and_hand_over_mouth:"]::before,
em[data-emoji="face_with_open_eyes_and_hand_over_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae2.svg");
}

em[data-emoji=":saluting_face:"]::before,
em[data-emoji="saluting_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae1.svg");
}

em[data-emoji=":shushing_face:"]::before,
em[data-emoji="shushing_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92b.svg");
}

em[data-emoji=":melting_face:"]::before,
em[data-emoji="melting_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae0.svg");
}

em[data-emoji=":lying_face:"]::before,
em[data-emoji="lying_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f925.svg");
}

em[data-emoji=":no_mouth:"]::before,
em[data-emoji="no_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f636.svg");
}

em[data-emoji=":dotted_line_face:"]::before,
em[data-emoji="dotted_line_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae5.svg");
}

em[data-emoji=":neutral_face:"]::before,
em[data-emoji="neutral_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f610.svg");
}

em[data-emoji=":face_with_diagonal_mouth:"]::before,
em[data-emoji="face_with_diagonal_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae4.svg");
}

em[data-emoji=":expressionless:"]::before,
em[data-emoji="expressionless"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f611.svg");
}

em[data-emoji=":grimacing:"]::before,
em[data-emoji="grimacing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62c.svg");
}

em[data-emoji=":rolling_eyes:"]::before,
em[data-emoji="rolling_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f644.svg");
}

em[data-emoji=":hushed:"]::before,
em[data-emoji="hushed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62f.svg");
}

em[data-emoji=":frowning:"]::before,
em[data-emoji="frowning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f626.svg");
}

em[data-emoji=":anguished:"]::before,
em[data-emoji="anguished"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f627.svg");
}

em[data-emoji=":open_mouth:"]::before,
em[data-emoji="open_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62e.svg");
}

em[data-emoji=":astonished:"]::before,
em[data-emoji="astonished"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f632.svg");
}

em[data-emoji=":yawning_face:"]::before,
em[data-emoji="yawning_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f971.svg");
}

em[data-emoji=":sleeping:"]::before,
em[data-emoji="sleeping"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f634.svg");
}

em[data-emoji=":drooling_face:"]::before,
em[data-emoji="drooling_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f924.svg");
}

em[data-emoji=":sleepy:"]::before,
em[data-emoji="sleepy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62a.svg");
}

em[data-emoji=":face_exhaling:"]::before,
em[data-emoji="face_exhaling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f62e-200d-1f4a8.svg");
}

em[data-emoji=":dizzy_face:"]::before,
em[data-emoji="dizzy_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f635.svg");
}

em[data-emoji=":face_with_spiral_eyes:"]::before,
em[data-emoji="face_with_spiral_eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f635-200d-1f4ab.svg");
}

em[data-emoji=":zipper_mouth:"]::before,
em[data-emoji="zipper_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f910.svg");
}

em[data-emoji=":woozy_face:"]::before,
em[data-emoji="woozy_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f974.svg");
}

em[data-emoji=":nauseated_face:"]::before,
em[data-emoji="nauseated_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f922.svg");
}

em[data-emoji=":face_vomiting:"]::before,
em[data-emoji="face_vomiting"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f92e.svg");
}

em[data-emoji=":sneezing_face:"]::before,
em[data-emoji="sneezing_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f927.svg");
}

em[data-emoji=":mask:"]::before,
em[data-emoji="mask"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f637.svg");
}

em[data-emoji=":thermometer_face:"]::before,
em[data-emoji="thermometer_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f912.svg");
}

em[data-emoji=":head_bandage:"]::before,
em[data-emoji="head_bandage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f915.svg");
}

em[data-emoji=":money_mouth:"]::before,
em[data-emoji="money_mouth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f911.svg");
}

em[data-emoji=":cowboy:"]::before,
em[data-emoji="cowboy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f920.svg");
}

em[data-emoji=":smiling_imp:"]::before,
em[data-emoji="smiling_imp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f608.svg");
}

em[data-emoji=":imp:"]::before,
em[data-emoji="imp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47f.svg");
}

em[data-emoji=":japanese_ogre:"]::before,
em[data-emoji="japanese_ogre"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f479.svg");
}

em[data-emoji=":japanese_goblin:"]::before,
em[data-emoji="japanese_goblin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47a.svg");
}

em[data-emoji=":clown:"]::before,
em[data-emoji="clown"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f921.svg");
}

em[data-emoji=":poop:"]::before,
em[data-emoji="poop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a9.svg");
}

em[data-emoji=":ghost:"]::before,
em[data-emoji="ghost"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47b.svg");
}

em[data-emoji=":skull:"]::before,
em[data-emoji="skull"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f480.svg");
}

em[data-emoji=":alien:"]::before,
em[data-emoji="alien"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47d.svg");
}

em[data-emoji=":space_invader:"]::before,
em[data-emoji="space_invader"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47e.svg");
}

em[data-emoji=":robot:"]::before,
em[data-emoji="robot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f916.svg");
}

em[data-emoji=":jack_o_lantern:"]::before,
em[data-emoji="jack_o_lantern"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f383.svg");
}

em[data-emoji=":smiley_cat:"]::before,
em[data-emoji="smiley_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63a.svg");
}

em[data-emoji=":smile_cat:"]::before,
em[data-emoji="smile_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f638.svg");
}

em[data-emoji=":joy_cat:"]::before,
em[data-emoji="joy_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f639.svg");
}

em[data-emoji=":heart_eyes_cat:"]::before,
em[data-emoji="heart_eyes_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63b.svg");
}

em[data-emoji=":smirk_cat:"]::before,
em[data-emoji="smirk_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63c.svg");
}

em[data-emoji=":kissing_cat:"]::before,
em[data-emoji="kissing_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63d.svg");
}

em[data-emoji=":scream_cat:"]::before,
em[data-emoji="scream_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f640.svg");
}

em[data-emoji=":crying_cat_face:"]::before,
em[data-emoji="crying_cat_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63f.svg");
}

em[data-emoji=":pouting_cat:"]::before,
em[data-emoji="pouting_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f63e.svg");
}

em[data-emoji=":heart_hands:"]::before,
em[data-emoji="heart_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6.svg");
}

em[data-emoji=":heart_hands_tone1:"]::before,
em[data-emoji="heart_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6-1f3fb.svg");
}

em[data-emoji=":heart_hands_tone2:"]::before,
em[data-emoji="heart_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6-1f3fc.svg");
}

em[data-emoji=":heart_hands_tone3:"]::before,
em[data-emoji="heart_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6-1f3fd.svg");
}

em[data-emoji=":heart_hands_tone4:"]::before,
em[data-emoji="heart_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6-1f3fe.svg");
}

em[data-emoji=":heart_hands_tone5:"]::before,
em[data-emoji="heart_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf6-1f3ff.svg");
}

em[data-emoji=":palms_up_together:"]::before,
em[data-emoji="palms_up_together"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932.svg");
}

em[data-emoji=":palms_up_together_tone1:"]::before,
em[data-emoji="palms_up_together_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932-1f3fb.svg");
}

em[data-emoji=":palms_up_together_tone2:"]::before,
em[data-emoji="palms_up_together_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932-1f3fc.svg");
}

em[data-emoji=":palms_up_together_tone3:"]::before,
em[data-emoji="palms_up_together_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932-1f3fd.svg");
}

em[data-emoji=":palms_up_together_tone4:"]::before,
em[data-emoji="palms_up_together_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932-1f3fe.svg");
}

em[data-emoji=":palms_up_together_tone5:"]::before,
em[data-emoji="palms_up_together_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f932-1f3ff.svg");
}

em[data-emoji=":open_hands:"]::before,
em[data-emoji="open_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450.svg");
}

em[data-emoji=":open_hands_tone1:"]::before,
em[data-emoji="open_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450-1f3fb.svg");
}

em[data-emoji=":open_hands_tone2:"]::before,
em[data-emoji="open_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450-1f3fc.svg");
}

em[data-emoji=":open_hands_tone3:"]::before,
em[data-emoji="open_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450-1f3fd.svg");
}

em[data-emoji=":open_hands_tone4:"]::before,
em[data-emoji="open_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450-1f3fe.svg");
}

em[data-emoji=":open_hands_tone5:"]::before,
em[data-emoji="open_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f450-1f3ff.svg");
}

em[data-emoji=":raised_hands:"]::before,
em[data-emoji="raised_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c.svg");
}

em[data-emoji=":raised_hands_tone1:"]::before,
em[data-emoji="raised_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c-1f3fb.svg");
}

em[data-emoji=":raised_hands_tone2:"]::before,
em[data-emoji="raised_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c-1f3fc.svg");
}

em[data-emoji=":raised_hands_tone3:"]::before,
em[data-emoji="raised_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c-1f3fd.svg");
}

em[data-emoji=":raised_hands_tone4:"]::before,
em[data-emoji="raised_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c-1f3fe.svg");
}

em[data-emoji=":raised_hands_tone5:"]::before,
em[data-emoji="raised_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64c-1f3ff.svg");
}

em[data-emoji=":clap:"]::before,
em[data-emoji="clap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f.svg");
}

em[data-emoji=":clap_tone1:"]::before,
em[data-emoji="clap_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f-1f3fb.svg");
}

em[data-emoji=":clap_tone2:"]::before,
em[data-emoji="clap_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f-1f3fc.svg");
}

em[data-emoji=":clap_tone3:"]::before,
em[data-emoji="clap_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f-1f3fd.svg");
}

em[data-emoji=":clap_tone4:"]::before,
em[data-emoji="clap_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f-1f3fe.svg");
}

em[data-emoji=":clap_tone5:"]::before,
em[data-emoji="clap_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44f-1f3ff.svg");
}

em[data-emoji=":handshake:"]::before,
em[data-emoji="handshake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d.svg");
}

em[data-emoji=":handshake_tone1:"]::before,
em[data-emoji="handshake_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d-1f3fb.svg");
}

em[data-emoji=":handshake_tone1_tone2:"]::before,
em[data-emoji="handshake_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fb-200d-1faf2-1f3fc.svg");
}

em[data-emoji=":handshake_tone1_tone3:"]::before,
em[data-emoji="handshake_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fb-200d-1faf2-1f3fd.svg");
}

em[data-emoji=":handshake_tone1_tone4:"]::before,
em[data-emoji="handshake_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fb-200d-1faf2-1f3fe.svg");
}

em[data-emoji=":handshake_tone1_tone5:"]::before,
em[data-emoji="handshake_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fb-200d-1faf2-1f3ff.svg");
}

em[data-emoji=":handshake_tone2_tone1:"]::before,
em[data-emoji="handshake_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fc-200d-1faf2-1f3fb.svg");
}

em[data-emoji=":handshake_tone2:"]::before,
em[data-emoji="handshake_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d-1f3fc.svg");
}

em[data-emoji=":handshake_tone2_tone3:"]::before,
em[data-emoji="handshake_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fc-200d-1faf2-1f3fd.svg");
}

em[data-emoji=":handshake_tone2_tone4:"]::before,
em[data-emoji="handshake_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fc-200d-1faf2-1f3fe.svg");
}

em[data-emoji=":handshake_tone2_tone5:"]::before,
em[data-emoji="handshake_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fc-200d-1faf2-1f3ff.svg");
}

em[data-emoji=":handshake_tone3_tone1:"]::before,
em[data-emoji="handshake_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fd-200d-1faf2-1f3fb.svg");
}

em[data-emoji=":handshake_tone3_tone2:"]::before,
em[data-emoji="handshake_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fd-200d-1faf2-1f3fc.svg");
}

em[data-emoji=":handshake_tone3:"]::before,
em[data-emoji="handshake_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d-1f3fd.svg");
}

em[data-emoji=":handshake_tone3_tone4:"]::before,
em[data-emoji="handshake_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fd-200d-1faf2-1f3fe.svg");
}

em[data-emoji=":handshake_tone3_tone5:"]::before,
em[data-emoji="handshake_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fd-200d-1faf2-1f3ff.svg");
}

em[data-emoji=":handshake_tone4_tone1:"]::before,
em[data-emoji="handshake_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fe-200d-1faf2-1f3fb.svg");
}

em[data-emoji=":handshake_tone4_tone2:"]::before,
em[data-emoji="handshake_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fe-200d-1faf2-1f3fc.svg");
}

em[data-emoji=":handshake_tone4_tone3:"]::before,
em[data-emoji="handshake_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fe-200d-1faf2-1f3fd.svg");
}

em[data-emoji=":handshake_tone4:"]::before,
em[data-emoji="handshake_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d-1f3fe.svg");
}

em[data-emoji=":handshake_tone4_tone5:"]::before,
em[data-emoji="handshake_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fe-200d-1faf2-1f3ff.svg");
}

em[data-emoji=":handshake_tone5_tone1:"]::before,
em[data-emoji="handshake_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3ff-200d-1faf2-1f3fb.svg");
}

em[data-emoji=":handshake_tone5_tone2:"]::before,
em[data-emoji="handshake_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3ff-200d-1faf2-1f3fc.svg");
}

em[data-emoji=":handshake_tone5_tone3:"]::before,
em[data-emoji="handshake_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3ff-200d-1faf2-1f3fd.svg");
}

em[data-emoji=":handshake_tone5_tone4:"]::before,
em[data-emoji="handshake_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3ff-200d-1faf2-1f3fe.svg");
}

em[data-emoji=":handshake_tone5:"]::before,
em[data-emoji="handshake_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91d-1f3ff.svg");
}

em[data-emoji=":thumbsup:"]::before,
em[data-emoji="thumbsup"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d.svg");
}

em[data-emoji=":thumbsup_tone1:"]::before,
em[data-emoji="thumbsup_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d-1f3fb.svg");
}

em[data-emoji=":thumbsup_tone2:"]::before,
em[data-emoji="thumbsup_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d-1f3fc.svg");
}

em[data-emoji=":thumbsup_tone3:"]::before,
em[data-emoji="thumbsup_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d-1f3fd.svg");
}

em[data-emoji=":thumbsup_tone4:"]::before,
em[data-emoji="thumbsup_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d-1f3fe.svg");
}

em[data-emoji=":thumbsup_tone5:"]::before,
em[data-emoji="thumbsup_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44d-1f3ff.svg");
}

em[data-emoji=":thumbsdown:"]::before,
em[data-emoji="thumbsdown"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e.svg");
}

em[data-emoji=":thumbsdown_tone1:"]::before,
em[data-emoji="thumbsdown_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e-1f3fb.svg");
}

em[data-emoji=":thumbsdown_tone2:"]::before,
em[data-emoji="thumbsdown_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e-1f3fc.svg");
}

em[data-emoji=":thumbsdown_tone3:"]::before,
em[data-emoji="thumbsdown_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e-1f3fd.svg");
}

em[data-emoji=":thumbsdown_tone4:"]::before,
em[data-emoji="thumbsdown_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e-1f3fe.svg");
}

em[data-emoji=":thumbsdown_tone5:"]::before,
em[data-emoji="thumbsdown_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44e-1f3ff.svg");
}

em[data-emoji=":punch:"]::before,
em[data-emoji="punch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a.svg");
}

em[data-emoji=":punch_tone1:"]::before,
em[data-emoji="punch_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a-1f3fb.svg");
}

em[data-emoji=":punch_tone2:"]::before,
em[data-emoji="punch_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a-1f3fc.svg");
}

em[data-emoji=":punch_tone3:"]::before,
em[data-emoji="punch_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a-1f3fd.svg");
}

em[data-emoji=":punch_tone4:"]::before,
em[data-emoji="punch_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a-1f3fe.svg");
}

em[data-emoji=":punch_tone5:"]::before,
em[data-emoji="punch_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44a-1f3ff.svg");
}

em[data-emoji=":fist:"]::before,
em[data-emoji="fist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a.svg");
}

em[data-emoji=":fist_tone1:"]::before,
em[data-emoji="fist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a-1f3fb.svg");
}

em[data-emoji=":fist_tone2:"]::before,
em[data-emoji="fist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a-1f3fc.svg");
}

em[data-emoji=":fist_tone3:"]::before,
em[data-emoji="fist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a-1f3fd.svg");
}

em[data-emoji=":fist_tone4:"]::before,
em[data-emoji="fist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a-1f3fe.svg");
}

em[data-emoji=":fist_tone5:"]::before,
em[data-emoji="fist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270a-1f3ff.svg");
}

em[data-emoji=":left_facing_fist:"]::before,
em[data-emoji="left_facing_fist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b.svg");
}

em[data-emoji=":left_facing_fist_tone1:"]::before,
em[data-emoji="left_facing_fist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b-1f3fb.svg");
}

em[data-emoji=":left_facing_fist_tone2:"]::before,
em[data-emoji="left_facing_fist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b-1f3fc.svg");
}

em[data-emoji=":left_facing_fist_tone3:"]::before,
em[data-emoji="left_facing_fist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b-1f3fd.svg");
}

em[data-emoji=":left_facing_fist_tone4:"]::before,
em[data-emoji="left_facing_fist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b-1f3fe.svg");
}

em[data-emoji=":left_facing_fist_tone5:"]::before,
em[data-emoji="left_facing_fist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91b-1f3ff.svg");
}

em[data-emoji=":right_facing_fist:"]::before,
em[data-emoji="right_facing_fist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c.svg");
}

em[data-emoji=":right_facing_fist_tone1:"]::before,
em[data-emoji="right_facing_fist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c-1f3fb.svg");
}

em[data-emoji=":right_facing_fist_tone2:"]::before,
em[data-emoji="right_facing_fist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c-1f3fc.svg");
}

em[data-emoji=":right_facing_fist_tone3:"]::before,
em[data-emoji="right_facing_fist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c-1f3fd.svg");
}

em[data-emoji=":right_facing_fist_tone4:"]::before,
em[data-emoji="right_facing_fist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c-1f3fe.svg");
}

em[data-emoji=":right_facing_fist_tone5:"]::before,
em[data-emoji="right_facing_fist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91c-1f3ff.svg");
}

em[data-emoji=":fingers_crossed:"]::before,
em[data-emoji="fingers_crossed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e.svg");
}

em[data-emoji=":fingers_crossed_tone1:"]::before,
em[data-emoji="fingers_crossed_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e-1f3fb.svg");
}

em[data-emoji=":fingers_crossed_tone2:"]::before,
em[data-emoji="fingers_crossed_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e-1f3fc.svg");
}

em[data-emoji=":fingers_crossed_tone3:"]::before,
em[data-emoji="fingers_crossed_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e-1f3fd.svg");
}

em[data-emoji=":fingers_crossed_tone4:"]::before,
em[data-emoji="fingers_crossed_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e-1f3fe.svg");
}

em[data-emoji=":fingers_crossed_tone5:"]::before,
em[data-emoji="fingers_crossed_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91e-1f3ff.svg");
}

em[data-emoji=":v:"]::before,
em[data-emoji="v"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c.svg");
}

em[data-emoji=":v_tone1:"]::before,
em[data-emoji="v_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c-1f3fb.svg");
}

em[data-emoji=":v_tone2:"]::before,
em[data-emoji="v_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c-1f3fc.svg");
}

em[data-emoji=":v_tone3:"]::before,
em[data-emoji="v_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c-1f3fd.svg");
}

em[data-emoji=":v_tone4:"]::before,
em[data-emoji="v_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c-1f3fe.svg");
}

em[data-emoji=":v_tone5:"]::before,
em[data-emoji="v_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270c-1f3ff.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed_tone1:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0-1f3fb.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed_tone2:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0-1f3fc.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed_tone3:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0-1f3fd.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed_tone4:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0-1f3fe.svg");
}

em[data-emoji=":hand_with_index_finger_and_thumb_crossed_tone5:"]::before,
em[data-emoji="hand_with_index_finger_and_thumb_crossed_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf0-1f3ff.svg");
}

em[data-emoji=":love_you_gesture:"]::before,
em[data-emoji="love_you_gesture"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f.svg");
}

em[data-emoji=":love_you_gesture_tone1:"]::before,
em[data-emoji="love_you_gesture_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f-1f3fb.svg");
}

em[data-emoji=":love_you_gesture_tone2:"]::before,
em[data-emoji="love_you_gesture_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f-1f3fc.svg");
}

em[data-emoji=":love_you_gesture_tone3:"]::before,
em[data-emoji="love_you_gesture_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f-1f3fd.svg");
}

em[data-emoji=":love_you_gesture_tone4:"]::before,
em[data-emoji="love_you_gesture_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f-1f3fe.svg");
}

em[data-emoji=":love_you_gesture_tone5:"]::before,
em[data-emoji="love_you_gesture_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91f-1f3ff.svg");
}

em[data-emoji=":metal:"]::before,
em[data-emoji="metal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918.svg");
}

em[data-emoji=":metal_tone1:"]::before,
em[data-emoji="metal_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918-1f3fb.svg");
}

em[data-emoji=":metal_tone2:"]::before,
em[data-emoji="metal_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918-1f3fc.svg");
}

em[data-emoji=":metal_tone3:"]::before,
em[data-emoji="metal_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918-1f3fd.svg");
}

em[data-emoji=":metal_tone4:"]::before,
em[data-emoji="metal_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918-1f3fe.svg");
}

em[data-emoji=":metal_tone5:"]::before,
em[data-emoji="metal_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f918-1f3ff.svg");
}

em[data-emoji=":ok_hand:"]::before,
em[data-emoji="ok_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c.svg");
}

em[data-emoji=":ok_hand_tone1:"]::before,
em[data-emoji="ok_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c-1f3fb.svg");
}

em[data-emoji=":ok_hand_tone2:"]::before,
em[data-emoji="ok_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c-1f3fc.svg");
}

em[data-emoji=":ok_hand_tone3:"]::before,
em[data-emoji="ok_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c-1f3fd.svg");
}

em[data-emoji=":ok_hand_tone4:"]::before,
em[data-emoji="ok_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c-1f3fe.svg");
}

em[data-emoji=":ok_hand_tone5:"]::before,
em[data-emoji="ok_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44c-1f3ff.svg");
}

em[data-emoji=":pinched_fingers:"]::before,
em[data-emoji="pinched_fingers"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c.svg");
}

em[data-emoji=":pinched_fingers_tone2:"]::before,
em[data-emoji="pinched_fingers_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c-1f3fc.svg");
}

em[data-emoji=":pinched_fingers_tone1:"]::before,
em[data-emoji="pinched_fingers_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c-1f3fb.svg");
}

em[data-emoji=":pinched_fingers_tone3:"]::before,
em[data-emoji="pinched_fingers_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c-1f3fd.svg");
}

em[data-emoji=":pinched_fingers_tone4:"]::before,
em[data-emoji="pinched_fingers_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c-1f3fe.svg");
}

em[data-emoji=":pinched_fingers_tone5:"]::before,
em[data-emoji="pinched_fingers_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90c-1f3ff.svg");
}

em[data-emoji=":pinching_hand:"]::before,
em[data-emoji="pinching_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f.svg");
}

em[data-emoji=":pinching_hand_tone1:"]::before,
em[data-emoji="pinching_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f-1f3fb.svg");
}

em[data-emoji=":pinching_hand_tone2:"]::before,
em[data-emoji="pinching_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f-1f3fc.svg");
}

em[data-emoji=":pinching_hand_tone3:"]::before,
em[data-emoji="pinching_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f-1f3fd.svg");
}

em[data-emoji=":pinching_hand_tone4:"]::before,
em[data-emoji="pinching_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f-1f3fe.svg");
}

em[data-emoji=":pinching_hand_tone5:"]::before,
em[data-emoji="pinching_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90f-1f3ff.svg");
}

em[data-emoji=":palm_down_hand:"]::before,
em[data-emoji="palm_down_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3.svg");
}

em[data-emoji=":palm_down_hand_tone1:"]::before,
em[data-emoji="palm_down_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3-1f3fb.svg");
}

em[data-emoji=":palm_down_hand_tone2:"]::before,
em[data-emoji="palm_down_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3-1f3fc.svg");
}

em[data-emoji=":palm_down_hand_tone3:"]::before,
em[data-emoji="palm_down_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3-1f3fd.svg");
}

em[data-emoji=":palm_down_hand_tone4:"]::before,
em[data-emoji="palm_down_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3-1f3fe.svg");
}

em[data-emoji=":palm_down_hand_tone5:"]::before,
em[data-emoji="palm_down_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf3-1f3ff.svg");
}

em[data-emoji=":palm_up_hand:"]::before,
em[data-emoji="palm_up_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4.svg");
}

em[data-emoji=":palm_up_hand_tone1:"]::before,
em[data-emoji="palm_up_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4-1f3fb.svg");
}

em[data-emoji=":palm_up_hand_tone2:"]::before,
em[data-emoji="palm_up_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4-1f3fc.svg");
}

em[data-emoji=":palm_up_hand_tone3:"]::before,
em[data-emoji="palm_up_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4-1f3fd.svg");
}

em[data-emoji=":palm_up_hand_tone4:"]::before,
em[data-emoji="palm_up_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4-1f3fe.svg");
}

em[data-emoji=":palm_up_hand_tone5:"]::before,
em[data-emoji="palm_up_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf4-1f3ff.svg");
}

em[data-emoji=":point_left:"]::before,
em[data-emoji="point_left"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448.svg");
}

em[data-emoji=":point_left_tone1:"]::before,
em[data-emoji="point_left_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448-1f3fb.svg");
}

em[data-emoji=":point_left_tone2:"]::before,
em[data-emoji="point_left_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448-1f3fc.svg");
}

em[data-emoji=":point_left_tone3:"]::before,
em[data-emoji="point_left_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448-1f3fd.svg");
}

em[data-emoji=":point_left_tone4:"]::before,
em[data-emoji="point_left_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448-1f3fe.svg");
}

em[data-emoji=":point_left_tone5:"]::before,
em[data-emoji="point_left_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f448-1f3ff.svg");
}

em[data-emoji=":point_right:"]::before,
em[data-emoji="point_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449.svg");
}

em[data-emoji=":point_right_tone1:"]::before,
em[data-emoji="point_right_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449-1f3fb.svg");
}

em[data-emoji=":point_right_tone2:"]::before,
em[data-emoji="point_right_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449-1f3fc.svg");
}

em[data-emoji=":point_right_tone3:"]::before,
em[data-emoji="point_right_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449-1f3fd.svg");
}

em[data-emoji=":point_right_tone4:"]::before,
em[data-emoji="point_right_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449-1f3fe.svg");
}

em[data-emoji=":point_right_tone5:"]::before,
em[data-emoji="point_right_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f449-1f3ff.svg");
}

em[data-emoji=":point_up_2:"]::before,
em[data-emoji="point_up_2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446.svg");
}

em[data-emoji=":point_up_2_tone1:"]::before,
em[data-emoji="point_up_2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446-1f3fb.svg");
}

em[data-emoji=":point_up_2_tone2:"]::before,
em[data-emoji="point_up_2_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446-1f3fc.svg");
}

em[data-emoji=":point_up_2_tone3:"]::before,
em[data-emoji="point_up_2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446-1f3fd.svg");
}

em[data-emoji=":point_up_2_tone4:"]::before,
em[data-emoji="point_up_2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446-1f3fe.svg");
}

em[data-emoji=":point_up_2_tone5:"]::before,
em[data-emoji="point_up_2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f446-1f3ff.svg");
}

em[data-emoji=":point_down:"]::before,
em[data-emoji="point_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447.svg");
}

em[data-emoji=":point_down_tone1:"]::before,
em[data-emoji="point_down_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447-1f3fb.svg");
}

em[data-emoji=":point_down_tone2:"]::before,
em[data-emoji="point_down_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447-1f3fc.svg");
}

em[data-emoji=":point_down_tone3:"]::before,
em[data-emoji="point_down_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447-1f3fd.svg");
}

em[data-emoji=":point_down_tone4:"]::before,
em[data-emoji="point_down_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447-1f3fe.svg");
}

em[data-emoji=":point_down_tone5:"]::before,
em[data-emoji="point_down_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f447-1f3ff.svg");
}

em[data-emoji=":point_up:"]::before,
em[data-emoji="point_up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d.svg");
}

em[data-emoji=":point_up_tone1:"]::before,
em[data-emoji="point_up_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d-1f3fb.svg");
}

em[data-emoji=":point_up_tone2:"]::before,
em[data-emoji="point_up_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d-1f3fc.svg");
}

em[data-emoji=":point_up_tone3:"]::before,
em[data-emoji="point_up_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d-1f3fd.svg");
}

em[data-emoji=":point_up_tone4:"]::before,
em[data-emoji="point_up_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d-1f3fe.svg");
}

em[data-emoji=":point_up_tone5:"]::before,
em[data-emoji="point_up_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/261d-1f3ff.svg");
}

em[data-emoji=":raised_hand:"]::before,
em[data-emoji="raised_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b.svg");
}

em[data-emoji=":raised_hand_tone1:"]::before,
em[data-emoji="raised_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b-1f3fb.svg");
}

em[data-emoji=":raised_hand_tone2:"]::before,
em[data-emoji="raised_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b-1f3fc.svg");
}

em[data-emoji=":raised_hand_tone3:"]::before,
em[data-emoji="raised_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b-1f3fd.svg");
}

em[data-emoji=":raised_hand_tone4:"]::before,
em[data-emoji="raised_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b-1f3fe.svg");
}

em[data-emoji=":raised_hand_tone5:"]::before,
em[data-emoji="raised_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270b-1f3ff.svg");
}

em[data-emoji=":raised_back_of_hand:"]::before,
em[data-emoji="raised_back_of_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a.svg");
}

em[data-emoji=":raised_back_of_hand_tone1:"]::before,
em[data-emoji="raised_back_of_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a-1f3fb.svg");
}

em[data-emoji=":raised_back_of_hand_tone2:"]::before,
em[data-emoji="raised_back_of_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a-1f3fc.svg");
}

em[data-emoji=":raised_back_of_hand_tone3:"]::before,
em[data-emoji="raised_back_of_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a-1f3fd.svg");
}

em[data-emoji=":raised_back_of_hand_tone4:"]::before,
em[data-emoji="raised_back_of_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a-1f3fe.svg");
}

em[data-emoji=":raised_back_of_hand_tone5:"]::before,
em[data-emoji="raised_back_of_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f91a-1f3ff.svg");
}

em[data-emoji=":hand_splayed:"]::before,
em[data-emoji="hand_splayed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590.svg");
}

em[data-emoji=":hand_splayed_tone1:"]::before,
em[data-emoji="hand_splayed_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590-1f3fb.svg");
}

em[data-emoji=":hand_splayed_tone2:"]::before,
em[data-emoji="hand_splayed_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590-1f3fc.svg");
}

em[data-emoji=":hand_splayed_tone3:"]::before,
em[data-emoji="hand_splayed_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590-1f3fd.svg");
}

em[data-emoji=":hand_splayed_tone4:"]::before,
em[data-emoji="hand_splayed_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590-1f3fe.svg");
}

em[data-emoji=":hand_splayed_tone5:"]::before,
em[data-emoji="hand_splayed_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f590-1f3ff.svg");
}

em[data-emoji=":vulcan:"]::before,
em[data-emoji="vulcan"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596.svg");
}

em[data-emoji=":vulcan_tone1:"]::before,
em[data-emoji="vulcan_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596-1f3fb.svg");
}

em[data-emoji=":vulcan_tone2:"]::before,
em[data-emoji="vulcan_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596-1f3fc.svg");
}

em[data-emoji=":vulcan_tone3:"]::before,
em[data-emoji="vulcan_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596-1f3fd.svg");
}

em[data-emoji=":vulcan_tone4:"]::before,
em[data-emoji="vulcan_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596-1f3fe.svg");
}

em[data-emoji=":vulcan_tone5:"]::before,
em[data-emoji="vulcan_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f596-1f3ff.svg");
}

em[data-emoji=":wave:"]::before,
em[data-emoji="wave"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b.svg");
}

em[data-emoji=":wave_tone1:"]::before,
em[data-emoji="wave_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b-1f3fb.svg");
}

em[data-emoji=":wave_tone2:"]::before,
em[data-emoji="wave_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b-1f3fc.svg");
}

em[data-emoji=":wave_tone3:"]::before,
em[data-emoji="wave_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b-1f3fd.svg");
}

em[data-emoji=":wave_tone4:"]::before,
em[data-emoji="wave_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b-1f3fe.svg");
}

em[data-emoji=":wave_tone5:"]::before,
em[data-emoji="wave_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f44b-1f3ff.svg");
}

em[data-emoji=":call_me:"]::before,
em[data-emoji="call_me"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919.svg");
}

em[data-emoji=":call_me_tone1:"]::before,
em[data-emoji="call_me_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919-1f3fb.svg");
}

em[data-emoji=":call_me_tone2:"]::before,
em[data-emoji="call_me_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919-1f3fc.svg");
}

em[data-emoji=":call_me_tone3:"]::before,
em[data-emoji="call_me_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919-1f3fd.svg");
}

em[data-emoji=":call_me_tone4:"]::before,
em[data-emoji="call_me_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919-1f3fe.svg");
}

em[data-emoji=":call_me_tone5:"]::before,
em[data-emoji="call_me_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f919-1f3ff.svg");
}

em[data-emoji=":leftwards_hand:"]::before,
em[data-emoji="leftwards_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2.svg");
}

em[data-emoji=":leftwards_hand_tone1:"]::before,
em[data-emoji="leftwards_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2-1f3fb.svg");
}

em[data-emoji=":leftwards_hand_tone2:"]::before,
em[data-emoji="leftwards_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2-1f3fc.svg");
}

em[data-emoji=":leftwards_hand_tone3:"]::before,
em[data-emoji="leftwards_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2-1f3fd.svg");
}

em[data-emoji=":leftwards_hand_tone4:"]::before,
em[data-emoji="leftwards_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2-1f3fe.svg");
}

em[data-emoji=":leftwards_hand_tone5:"]::before,
em[data-emoji="leftwards_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf2-1f3ff.svg");
}

em[data-emoji=":rightwards_hand:"]::before,
em[data-emoji="rightwards_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1.svg");
}

em[data-emoji=":rightwards_hand_tone1:"]::before,
em[data-emoji="rightwards_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fb.svg");
}

em[data-emoji=":rightwards_hand_tone2:"]::before,
em[data-emoji="rightwards_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fc.svg");
}

em[data-emoji=":rightwards_hand_tone3:"]::before,
em[data-emoji="rightwards_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fd.svg");
}

em[data-emoji=":rightwards_hand_tone4:"]::before,
em[data-emoji="rightwards_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3fe.svg");
}

em[data-emoji=":rightwards_hand_tone5:"]::before,
em[data-emoji="rightwards_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf1-1f3ff.svg");
}

em[data-emoji=":muscle:"]::before,
em[data-emoji="muscle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa.svg");
}

em[data-emoji=":muscle_tone1:"]::before,
em[data-emoji="muscle_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa-1f3fb.svg");
}

em[data-emoji=":muscle_tone2:"]::before,
em[data-emoji="muscle_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa-1f3fc.svg");
}

em[data-emoji=":muscle_tone3:"]::before,
em[data-emoji="muscle_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa-1f3fd.svg");
}

em[data-emoji=":muscle_tone4:"]::before,
em[data-emoji="muscle_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa-1f3fe.svg");
}

em[data-emoji=":muscle_tone5:"]::before,
em[data-emoji="muscle_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4aa-1f3ff.svg");
}

em[data-emoji=":mechanical_arm:"]::before,
em[data-emoji="mechanical_arm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9be.svg");
}

em[data-emoji=":middle_finger:"]::before,
em[data-emoji="middle_finger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595.svg");
}

em[data-emoji=":middle_finger_tone1:"]::before,
em[data-emoji="middle_finger_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595-1f3fb.svg");
}

em[data-emoji=":middle_finger_tone2:"]::before,
em[data-emoji="middle_finger_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595-1f3fc.svg");
}

em[data-emoji=":middle_finger_tone3:"]::before,
em[data-emoji="middle_finger_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595-1f3fd.svg");
}

em[data-emoji=":middle_finger_tone4:"]::before,
em[data-emoji="middle_finger_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595-1f3fe.svg");
}

em[data-emoji=":middle_finger_tone5:"]::before,
em[data-emoji="middle_finger_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f595-1f3ff.svg");
}

em[data-emoji=":writing_hand:"]::before,
em[data-emoji="writing_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d.svg");
}

em[data-emoji=":writing_hand_tone1:"]::before,
em[data-emoji="writing_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d-1f3fb.svg");
}

em[data-emoji=":writing_hand_tone2:"]::before,
em[data-emoji="writing_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d-1f3fc.svg");
}

em[data-emoji=":writing_hand_tone3:"]::before,
em[data-emoji="writing_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d-1f3fd.svg");
}

em[data-emoji=":writing_hand_tone4:"]::before,
em[data-emoji="writing_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d-1f3fe.svg");
}

em[data-emoji=":writing_hand_tone5:"]::before,
em[data-emoji="writing_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270d-1f3ff.svg");
}

em[data-emoji=":pray:"]::before,
em[data-emoji="pray"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f.svg");
}

em[data-emoji=":pray_tone1:"]::before,
em[data-emoji="pray_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f-1f3fb.svg");
}

em[data-emoji=":pray_tone2:"]::before,
em[data-emoji="pray_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f-1f3fc.svg");
}

em[data-emoji=":pray_tone3:"]::before,
em[data-emoji="pray_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f-1f3fd.svg");
}

em[data-emoji=":pray_tone4:"]::before,
em[data-emoji="pray_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f-1f3fe.svg");
}

em[data-emoji=":pray_tone5:"]::before,
em[data-emoji="pray_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64f-1f3ff.svg");
}

em[data-emoji=":index_pointing_at_the_viewer:"]::before,
em[data-emoji="index_pointing_at_the_viewer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5.svg");
}

em[data-emoji=":index_pointing_at_the_viewer_tone1:"]::before,
em[data-emoji="index_pointing_at_the_viewer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5-1f3fb.svg");
}

em[data-emoji=":index_pointing_at_the_viewer_tone2:"]::before,
em[data-emoji="index_pointing_at_the_viewer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5-1f3fc.svg");
}

em[data-emoji=":index_pointing_at_the_viewer_tone3:"]::before,
em[data-emoji="index_pointing_at_the_viewer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5-1f3fd.svg");
}

em[data-emoji=":index_pointing_at_the_viewer_tone4:"]::before,
em[data-emoji="index_pointing_at_the_viewer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5-1f3fe.svg");
}

em[data-emoji=":index_pointing_at_the_viewer_tone5:"]::before,
em[data-emoji="index_pointing_at_the_viewer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faf5-1f3ff.svg");
}

em[data-emoji=":foot:"]::before,
em[data-emoji="foot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6.svg");
}

em[data-emoji=":foot_tone1:"]::before,
em[data-emoji="foot_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6-1f3fb.svg");
}

em[data-emoji=":foot_tone2:"]::before,
em[data-emoji="foot_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6-1f3fc.svg");
}

em[data-emoji=":foot_tone3:"]::before,
em[data-emoji="foot_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6-1f3fd.svg");
}

em[data-emoji=":foot_tone4:"]::before,
em[data-emoji="foot_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6-1f3fe.svg");
}

em[data-emoji=":foot_tone5:"]::before,
em[data-emoji="foot_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b6-1f3ff.svg");
}

em[data-emoji=":leg:"]::before,
em[data-emoji="leg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5.svg");
}

em[data-emoji=":leg_tone1:"]::before,
em[data-emoji="leg_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5-1f3fb.svg");
}

em[data-emoji=":leg_tone2:"]::before,
em[data-emoji="leg_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5-1f3fc.svg");
}

em[data-emoji=":leg_tone3:"]::before,
em[data-emoji="leg_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5-1f3fd.svg");
}

em[data-emoji=":leg_tone4:"]::before,
em[data-emoji="leg_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5-1f3fe.svg");
}

em[data-emoji=":leg_tone5:"]::before,
em[data-emoji="leg_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b5-1f3ff.svg");
}

em[data-emoji=":mechanical_leg:"]::before,
em[data-emoji="mechanical_leg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bf.svg");
}

em[data-emoji=":lipstick:"]::before,
em[data-emoji="lipstick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f484.svg");
}

em[data-emoji=":kiss:"]::before,
em[data-emoji="kiss"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48b.svg");
}

em[data-emoji=":lips:"]::before,
em[data-emoji="lips"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f444.svg");
}

em[data-emoji=":biting_lip:"]::before,
em[data-emoji="biting_lip"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae6.svg");
}

em[data-emoji=":tooth:"]::before,
em[data-emoji="tooth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b7.svg");
}

em[data-emoji=":tongue:"]::before,
em[data-emoji="tongue"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f445.svg");
}

em[data-emoji=":ear:"]::before,
em[data-emoji="ear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442.svg");
}

em[data-emoji=":ear_tone1:"]::before,
em[data-emoji="ear_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442-1f3fb.svg");
}

em[data-emoji=":ear_tone2:"]::before,
em[data-emoji="ear_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442-1f3fc.svg");
}

em[data-emoji=":ear_tone3:"]::before,
em[data-emoji="ear_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442-1f3fd.svg");
}

em[data-emoji=":ear_tone4:"]::before,
em[data-emoji="ear_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442-1f3fe.svg");
}

em[data-emoji=":ear_tone5:"]::before,
em[data-emoji="ear_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f442-1f3ff.svg");
}

em[data-emoji=":ear_with_hearing_aid:"]::before,
em[data-emoji="ear_with_hearing_aid"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb.svg");
}

em[data-emoji=":ear_with_hearing_aid_tone1:"]::before,
em[data-emoji="ear_with_hearing_aid_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb-1f3fb.svg");
}

em[data-emoji=":ear_with_hearing_aid_tone2:"]::before,
em[data-emoji="ear_with_hearing_aid_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb-1f3fc.svg");
}

em[data-emoji=":ear_with_hearing_aid_tone3:"]::before,
em[data-emoji="ear_with_hearing_aid_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb-1f3fd.svg");
}

em[data-emoji=":ear_with_hearing_aid_tone4:"]::before,
em[data-emoji="ear_with_hearing_aid_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb-1f3fe.svg");
}

em[data-emoji=":ear_with_hearing_aid_tone5:"]::before,
em[data-emoji="ear_with_hearing_aid_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bb-1f3ff.svg");
}

em[data-emoji=":nose:"]::before,
em[data-emoji="nose"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443.svg");
}

em[data-emoji=":nose_tone1:"]::before,
em[data-emoji="nose_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443-1f3fb.svg");
}

em[data-emoji=":nose_tone2:"]::before,
em[data-emoji="nose_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443-1f3fc.svg");
}

em[data-emoji=":nose_tone3:"]::before,
em[data-emoji="nose_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443-1f3fd.svg");
}

em[data-emoji=":nose_tone4:"]::before,
em[data-emoji="nose_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443-1f3fe.svg");
}

em[data-emoji=":nose_tone5:"]::before,
em[data-emoji="nose_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f443-1f3ff.svg");
}

em[data-emoji=":footprints:"]::before,
em[data-emoji="footprints"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f463.svg");
}

em[data-emoji=":eye:"]::before,
em[data-emoji="eye"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f441.svg");
}

em[data-emoji=":eyes:"]::before,
em[data-emoji="eyes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f440.svg");
}

em[data-emoji=":anatomical_heart:"]::before,
em[data-emoji="anatomical_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac0.svg");
}

em[data-emoji=":lungs:"]::before,
em[data-emoji="lungs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac1.svg");
}

em[data-emoji=":brain:"]::before,
em[data-emoji="brain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e0.svg");
}

em[data-emoji=":speaking_head:"]::before,
em[data-emoji="speaking_head"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5e3.svg");
}

em[data-emoji=":bust_in_silhouette:"]::before,
em[data-emoji="bust_in_silhouette"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f464.svg");
}

em[data-emoji=":busts_in_silhouette:"]::before,
em[data-emoji="busts_in_silhouette"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f465.svg");
}

em[data-emoji=":people_hugging:"]::before,
em[data-emoji="people_hugging"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac2.svg");
}

em[data-emoji=":baby:"]::before,
em[data-emoji="baby"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476.svg");
}

em[data-emoji=":baby_tone1:"]::before,
em[data-emoji="baby_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476-1f3fb.svg");
}

em[data-emoji=":baby_tone2:"]::before,
em[data-emoji="baby_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476-1f3fc.svg");
}

em[data-emoji=":baby_tone3:"]::before,
em[data-emoji="baby_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476-1f3fd.svg");
}

em[data-emoji=":baby_tone4:"]::before,
em[data-emoji="baby_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476-1f3fe.svg");
}

em[data-emoji=":baby_tone5:"]::before,
em[data-emoji="baby_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f476-1f3ff.svg");
}

em[data-emoji=":child:"]::before,
em[data-emoji="child"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2.svg");
}

em[data-emoji=":child_tone1:"]::before,
em[data-emoji="child_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2-1f3fb.svg");
}

em[data-emoji=":child_tone2:"]::before,
em[data-emoji="child_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2-1f3fc.svg");
}

em[data-emoji=":child_tone3:"]::before,
em[data-emoji="child_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2-1f3fd.svg");
}

em[data-emoji=":child_tone4:"]::before,
em[data-emoji="child_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2-1f3fe.svg");
}

em[data-emoji=":child_tone5:"]::before,
em[data-emoji="child_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d2-1f3ff.svg");
}

em[data-emoji=":girl:"]::before,
em[data-emoji="girl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467.svg");
}

em[data-emoji=":girl_tone1:"]::before,
em[data-emoji="girl_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467-1f3fb.svg");
}

em[data-emoji=":girl_tone2:"]::before,
em[data-emoji="girl_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467-1f3fc.svg");
}

em[data-emoji=":girl_tone3:"]::before,
em[data-emoji="girl_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467-1f3fd.svg");
}

em[data-emoji=":girl_tone4:"]::before,
em[data-emoji="girl_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467-1f3fe.svg");
}

em[data-emoji=":girl_tone5:"]::before,
em[data-emoji="girl_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f467-1f3ff.svg");
}

em[data-emoji=":boy:"]::before,
em[data-emoji="boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466.svg");
}

em[data-emoji=":boy_tone1:"]::before,
em[data-emoji="boy_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466-1f3fb.svg");
}

em[data-emoji=":boy_tone2:"]::before,
em[data-emoji="boy_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466-1f3fc.svg");
}

em[data-emoji=":boy_tone3:"]::before,
em[data-emoji="boy_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466-1f3fd.svg");
}

em[data-emoji=":boy_tone4:"]::before,
em[data-emoji="boy_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466-1f3fe.svg");
}

em[data-emoji=":boy_tone5:"]::before,
em[data-emoji="boy_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f466-1f3ff.svg");
}

em[data-emoji=":adult:"]::before,
em[data-emoji="adult"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1.svg");
}

em[data-emoji=":adult_tone1:"]::before,
em[data-emoji="adult_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb.svg");
}

em[data-emoji=":adult_tone2:"]::before,
em[data-emoji="adult_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc.svg");
}

em[data-emoji=":adult_tone3:"]::before,
em[data-emoji="adult_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd.svg");
}

em[data-emoji=":adult_tone4:"]::before,
em[data-emoji="adult_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe.svg");
}

em[data-emoji=":adult_tone5:"]::before,
em[data-emoji="adult_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff.svg");
}

em[data-emoji=":woman:"]::before,
em[data-emoji="woman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469.svg");
}

em[data-emoji=":woman_tone1:"]::before,
em[data-emoji="woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb.svg");
}

em[data-emoji=":woman_tone2:"]::before,
em[data-emoji="woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc.svg");
}

em[data-emoji=":woman_tone3:"]::before,
em[data-emoji="woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd.svg");
}

em[data-emoji=":woman_tone4:"]::before,
em[data-emoji="woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe.svg");
}

em[data-emoji=":woman_tone5:"]::before,
em[data-emoji="woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff.svg");
}

em[data-emoji=":man:"]::before,
em[data-emoji="man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468.svg");
}

em[data-emoji=":man_tone1:"]::before,
em[data-emoji="man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb.svg");
}

em[data-emoji=":man_tone2:"]::before,
em[data-emoji="man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc.svg");
}

em[data-emoji=":man_tone3:"]::before,
em[data-emoji="man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd.svg");
}

em[data-emoji=":man_tone4:"]::before,
em[data-emoji="man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe.svg");
}

em[data-emoji=":man_tone5:"]::before,
em[data-emoji="man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff.svg");
}

em[data-emoji=":person_curly_hair:"]::before,
em[data-emoji="person_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9b1.svg");
}

em[data-emoji=":person_tone1_curly_hair:"]::before,
em[data-emoji="person_tone1_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9b1.svg");
}

em[data-emoji=":person_tone2_curly_hair:"]::before,
em[data-emoji="person_tone2_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9b1.svg");
}

em[data-emoji=":person_tone3_curly_hair:"]::before,
em[data-emoji="person_tone3_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9b1.svg");
}

em[data-emoji=":person_tone4_curly_hair:"]::before,
em[data-emoji="person_tone4_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9b1.svg");
}

em[data-emoji=":person_tone5_curly_hair:"]::before,
em[data-emoji="person_tone5_curly_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired:"]::before,
em[data-emoji="woman_curly_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired_tone1:"]::before,
em[data-emoji="woman_curly_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired_tone2:"]::before,
em[data-emoji="woman_curly_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired_tone3:"]::before,
em[data-emoji="woman_curly_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired_tone4:"]::before,
em[data-emoji="woman_curly_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9b1.svg");
}

em[data-emoji=":woman_curly_haired_tone5:"]::before,
em[data-emoji="woman_curly_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired:"]::before,
em[data-emoji="man_curly_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired_tone1:"]::before,
em[data-emoji="man_curly_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired_tone2:"]::before,
em[data-emoji="man_curly_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired_tone3:"]::before,
em[data-emoji="man_curly_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired_tone4:"]::before,
em[data-emoji="man_curly_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9b1.svg");
}

em[data-emoji=":man_curly_haired_tone5:"]::before,
em[data-emoji="man_curly_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9b1.svg");
}

em[data-emoji=":person_red_hair:"]::before,
em[data-emoji="person_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9b0.svg");
}

em[data-emoji=":person_tone1_red_hair:"]::before,
em[data-emoji="person_tone1_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9b0.svg");
}

em[data-emoji=":person_tone2_red_hair:"]::before,
em[data-emoji="person_tone2_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9b0.svg");
}

em[data-emoji=":person_tone3_red_hair:"]::before,
em[data-emoji="person_tone3_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9b0.svg");
}

em[data-emoji=":person_tone4_red_hair:"]::before,
em[data-emoji="person_tone4_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9b0.svg");
}

em[data-emoji=":person_tone5_red_hair:"]::before,
em[data-emoji="person_tone5_red_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired:"]::before,
em[data-emoji="woman_red_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired_tone1:"]::before,
em[data-emoji="woman_red_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired_tone2:"]::before,
em[data-emoji="woman_red_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired_tone3:"]::before,
em[data-emoji="woman_red_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired_tone4:"]::before,
em[data-emoji="woman_red_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9b0.svg");
}

em[data-emoji=":woman_red_haired_tone5:"]::before,
em[data-emoji="woman_red_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired:"]::before,
em[data-emoji="man_red_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired_tone1:"]::before,
em[data-emoji="man_red_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired_tone2:"]::before,
em[data-emoji="man_red_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired_tone3:"]::before,
em[data-emoji="man_red_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired_tone4:"]::before,
em[data-emoji="man_red_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9b0.svg");
}

em[data-emoji=":man_red_haired_tone5:"]::before,
em[data-emoji="man_red_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9b0.svg");
}

em[data-emoji=":blond_haired_person:"]::before,
em[data-emoji="blond_haired_person"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471.svg");
}

em[data-emoji=":blond_haired_person_tone1:"]::before,
em[data-emoji="blond_haired_person_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fb.svg");
}

em[data-emoji=":blond_haired_person_tone2:"]::before,
em[data-emoji="blond_haired_person_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fc.svg");
}

em[data-emoji=":blond_haired_person_tone3:"]::before,
em[data-emoji="blond_haired_person_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fd.svg");
}

em[data-emoji=":blond_haired_person_tone4:"]::before,
em[data-emoji="blond_haired_person_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fe.svg");
}

em[data-emoji=":blond_haired_person_tone5:"]::before,
em[data-emoji="blond_haired_person_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3ff.svg");
}

em[data-emoji=":blond-haired_woman:"]::before,
em[data-emoji="blond-haired_woman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_woman_tone1:"]::before,
em[data-emoji="blond-haired_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_woman_tone2:"]::before,
em[data-emoji="blond-haired_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_woman_tone3:"]::before,
em[data-emoji="blond-haired_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_woman_tone4:"]::before,
em[data-emoji="blond-haired_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_woman_tone5:"]::before,
em[data-emoji="blond-haired_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":blond-haired_man:"]::before,
em[data-emoji="blond-haired_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-200d-2642-fe0f.svg");
}

em[data-emoji=":blond-haired_man_tone1:"]::before,
em[data-emoji="blond-haired_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":blond-haired_man_tone2:"]::before,
em[data-emoji="blond-haired_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":blond-haired_man_tone3:"]::before,
em[data-emoji="blond-haired_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":blond-haired_man_tone4:"]::before,
em[data-emoji="blond-haired_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":blond-haired_man_tone5:"]::before,
em[data-emoji="blond-haired_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f471-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_white_hair:"]::before,
em[data-emoji="person_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9b3.svg");
}

em[data-emoji=":person_tone1_white_hair:"]::before,
em[data-emoji="person_tone1_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9b3.svg");
}

em[data-emoji=":person_tone2_white_hair:"]::before,
em[data-emoji="person_tone2_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9b3.svg");
}

em[data-emoji=":person_tone3_white_hair:"]::before,
em[data-emoji="person_tone3_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9b3.svg");
}

em[data-emoji=":person_tone4_white_hair:"]::before,
em[data-emoji="person_tone4_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9b3.svg");
}

em[data-emoji=":person_tone5_white_hair:"]::before,
em[data-emoji="person_tone5_white_hair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired:"]::before,
em[data-emoji="woman_white_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired_tone1:"]::before,
em[data-emoji="woman_white_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired_tone2:"]::before,
em[data-emoji="woman_white_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired_tone3:"]::before,
em[data-emoji="woman_white_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired_tone4:"]::before,
em[data-emoji="woman_white_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9b3.svg");
}

em[data-emoji=":woman_white_haired_tone5:"]::before,
em[data-emoji="woman_white_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired:"]::before,
em[data-emoji="man_white_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired_tone1:"]::before,
em[data-emoji="man_white_haired_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired_tone2:"]::before,
em[data-emoji="man_white_haired_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired_tone3:"]::before,
em[data-emoji="man_white_haired_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired_tone4:"]::before,
em[data-emoji="man_white_haired_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9b3.svg");
}

em[data-emoji=":man_white_haired_tone5:"]::before,
em[data-emoji="man_white_haired_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9b3.svg");
}

em[data-emoji=":person_bald:"]::before,
em[data-emoji="person_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9b2.svg");
}

em[data-emoji=":person_tone1_bald:"]::before,
em[data-emoji="person_tone1_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9b2.svg");
}

em[data-emoji=":person_tone2_bald:"]::before,
em[data-emoji="person_tone2_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9b2.svg");
}

em[data-emoji=":person_tone3_bald:"]::before,
em[data-emoji="person_tone3_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9b2.svg");
}

em[data-emoji=":person_tone4_bald:"]::before,
em[data-emoji="person_tone4_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9b2.svg");
}

em[data-emoji=":person_tone5_bald:"]::before,
em[data-emoji="person_tone5_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald:"]::before,
em[data-emoji="woman_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald_tone1:"]::before,
em[data-emoji="woman_bald_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald_tone2:"]::before,
em[data-emoji="woman_bald_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald_tone3:"]::before,
em[data-emoji="woman_bald_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald_tone4:"]::before,
em[data-emoji="woman_bald_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9b2.svg");
}

em[data-emoji=":woman_bald_tone5:"]::before,
em[data-emoji="woman_bald_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9b2.svg");
}

em[data-emoji=":man_bald:"]::before,
em[data-emoji="man_bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9b2.svg");
}

em[data-emoji=":man_bald_tone1:"]::before,
em[data-emoji="man_bald_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9b2.svg");
}

em[data-emoji=":man_bald_tone2:"]::before,
em[data-emoji="man_bald_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9b2.svg");
}

em[data-emoji=":man_bald_tone3:"]::before,
em[data-emoji="man_bald_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9b2.svg");
}

em[data-emoji=":man_bald_tone4:"]::before,
em[data-emoji="man_bald_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9b2.svg");
}

em[data-emoji=":man_bald_tone5:"]::before,
em[data-emoji="man_bald_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9b2.svg");
}

em[data-emoji=":bearded_person:"]::before,
em[data-emoji="bearded_person"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4.svg");
}

em[data-emoji=":bearded_person_tone1:"]::before,
em[data-emoji="bearded_person_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fb.svg");
}

em[data-emoji=":bearded_person_tone2:"]::before,
em[data-emoji="bearded_person_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fc.svg");
}

em[data-emoji=":bearded_person_tone3:"]::before,
em[data-emoji="bearded_person_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fd.svg");
}

em[data-emoji=":bearded_person_tone4:"]::before,
em[data-emoji="bearded_person_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fe.svg");
}

em[data-emoji=":bearded_person_tone5:"]::before,
em[data-emoji="bearded_person_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3ff.svg");
}

em[data-emoji=":woman_beard:"]::before,
em[data-emoji="woman_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tone1_beard:"]::before,
em[data-emoji="woman_tone1_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tone2_beard:"]::before,
em[data-emoji="woman_tone2_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tone3_beard:"]::before,
em[data-emoji="woman_tone3_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tone4_beard:"]::before,
em[data-emoji="woman_tone4_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tone5_beard:"]::before,
em[data-emoji="woman_tone5_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_beard:"]::before,
em[data-emoji="man_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tone1_beard:"]::before,
em[data-emoji="man_tone1_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tone2_beard:"]::before,
em[data-emoji="man_tone2_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tone3_beard:"]::before,
em[data-emoji="man_tone3_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tone4_beard:"]::before,
em[data-emoji="man_tone4_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tone5_beard:"]::before,
em[data-emoji="man_tone5_beard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d4-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":older_adult:"]::before,
em[data-emoji="older_adult"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3.svg");
}

em[data-emoji=":older_adult_tone1:"]::before,
em[data-emoji="older_adult_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3-1f3fb.svg");
}

em[data-emoji=":older_adult_tone2:"]::before,
em[data-emoji="older_adult_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3-1f3fc.svg");
}

em[data-emoji=":older_adult_tone3:"]::before,
em[data-emoji="older_adult_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3-1f3fd.svg");
}

em[data-emoji=":older_adult_tone4:"]::before,
em[data-emoji="older_adult_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3-1f3fe.svg");
}

em[data-emoji=":older_adult_tone5:"]::before,
em[data-emoji="older_adult_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d3-1f3ff.svg");
}

em[data-emoji=":older_woman:"]::before,
em[data-emoji="older_woman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475.svg");
}

em[data-emoji=":older_woman_tone1:"]::before,
em[data-emoji="older_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475-1f3fb.svg");
}

em[data-emoji=":older_woman_tone2:"]::before,
em[data-emoji="older_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475-1f3fc.svg");
}

em[data-emoji=":older_woman_tone3:"]::before,
em[data-emoji="older_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475-1f3fd.svg");
}

em[data-emoji=":older_woman_tone4:"]::before,
em[data-emoji="older_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475-1f3fe.svg");
}

em[data-emoji=":older_woman_tone5:"]::before,
em[data-emoji="older_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f475-1f3ff.svg");
}

em[data-emoji=":older_man:"]::before,
em[data-emoji="older_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474.svg");
}

em[data-emoji=":older_man_tone1:"]::before,
em[data-emoji="older_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474-1f3fb.svg");
}

em[data-emoji=":older_man_tone2:"]::before,
em[data-emoji="older_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474-1f3fc.svg");
}

em[data-emoji=":older_man_tone3:"]::before,
em[data-emoji="older_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474-1f3fd.svg");
}

em[data-emoji=":older_man_tone4:"]::before,
em[data-emoji="older_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474-1f3fe.svg");
}

em[data-emoji=":older_man_tone5:"]::before,
em[data-emoji="older_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f474-1f3ff.svg");
}

em[data-emoji=":man_with_chinese_cap:"]::before,
em[data-emoji="man_with_chinese_cap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472.svg");
}

em[data-emoji=":man_with_chinese_cap_tone1:"]::before,
em[data-emoji="man_with_chinese_cap_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472-1f3fb.svg");
}

em[data-emoji=":man_with_chinese_cap_tone2:"]::before,
em[data-emoji="man_with_chinese_cap_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472-1f3fc.svg");
}

em[data-emoji=":man_with_chinese_cap_tone3:"]::before,
em[data-emoji="man_with_chinese_cap_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472-1f3fd.svg");
}

em[data-emoji=":man_with_chinese_cap_tone4:"]::before,
em[data-emoji="man_with_chinese_cap_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472-1f3fe.svg");
}

em[data-emoji=":man_with_chinese_cap_tone5:"]::before,
em[data-emoji="man_with_chinese_cap_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f472-1f3ff.svg");
}

em[data-emoji=":person_wearing_turban:"]::before,
em[data-emoji="person_wearing_turban"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473.svg");
}

em[data-emoji=":person_wearing_turban_tone1:"]::before,
em[data-emoji="person_wearing_turban_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fb.svg");
}

em[data-emoji=":person_wearing_turban_tone2:"]::before,
em[data-emoji="person_wearing_turban_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fc.svg");
}

em[data-emoji=":person_wearing_turban_tone3:"]::before,
em[data-emoji="person_wearing_turban_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fd.svg");
}

em[data-emoji=":person_wearing_turban_tone4:"]::before,
em[data-emoji="person_wearing_turban_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fe.svg");
}

em[data-emoji=":person_wearing_turban_tone5:"]::before,
em[data-emoji="person_wearing_turban_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3ff.svg");
}

em[data-emoji=":woman_wearing_turban:"]::before,
em[data-emoji="woman_wearing_turban"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_wearing_turban_tone1:"]::before,
em[data-emoji="woman_wearing_turban_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_wearing_turban_tone2:"]::before,
em[data-emoji="woman_wearing_turban_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_wearing_turban_tone3:"]::before,
em[data-emoji="woman_wearing_turban_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_wearing_turban_tone4:"]::before,
em[data-emoji="woman_wearing_turban_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_wearing_turban_tone5:"]::before,
em[data-emoji="woman_wearing_turban_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_wearing_turban:"]::before,
em[data-emoji="man_wearing_turban"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-200d-2642-fe0f.svg");
}

em[data-emoji=":man_wearing_turban_tone1:"]::before,
em[data-emoji="man_wearing_turban_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_wearing_turban_tone2:"]::before,
em[data-emoji="man_wearing_turban_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_wearing_turban_tone3:"]::before,
em[data-emoji="man_wearing_turban_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_wearing_turban_tone4:"]::before,
em[data-emoji="man_wearing_turban_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_wearing_turban_tone5:"]::before,
em[data-emoji="man_wearing_turban_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f473-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":woman_with_headscarf:"]::before,
em[data-emoji="woman_with_headscarf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5.svg");
}

em[data-emoji=":woman_with_headscarf_tone1:"]::before,
em[data-emoji="woman_with_headscarf_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5-1f3fb.svg");
}

em[data-emoji=":woman_with_headscarf_tone2:"]::before,
em[data-emoji="woman_with_headscarf_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5-1f3fc.svg");
}

em[data-emoji=":woman_with_headscarf_tone3:"]::before,
em[data-emoji="woman_with_headscarf_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5-1f3fd.svg");
}

em[data-emoji=":woman_with_headscarf_tone4:"]::before,
em[data-emoji="woman_with_headscarf_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5-1f3fe.svg");
}

em[data-emoji=":woman_with_headscarf_tone5:"]::before,
em[data-emoji="woman_with_headscarf_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d5-1f3ff.svg");
}

em[data-emoji=":police_officer:"]::before,
em[data-emoji="police_officer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e.svg");
}

em[data-emoji=":police_officer_tone1:"]::before,
em[data-emoji="police_officer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fb.svg");
}

em[data-emoji=":police_officer_tone2:"]::before,
em[data-emoji="police_officer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fc.svg");
}

em[data-emoji=":police_officer_tone3:"]::before,
em[data-emoji="police_officer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fd.svg");
}

em[data-emoji=":police_officer_tone4:"]::before,
em[data-emoji="police_officer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fe.svg");
}

em[data-emoji=":police_officer_tone5:"]::before,
em[data-emoji="police_officer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3ff.svg");
}

em[data-emoji=":woman_police_officer:"]::before,
em[data-emoji="woman_police_officer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_police_officer_tone1:"]::before,
em[data-emoji="woman_police_officer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_police_officer_tone2:"]::before,
em[data-emoji="woman_police_officer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_police_officer_tone3:"]::before,
em[data-emoji="woman_police_officer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_police_officer_tone4:"]::before,
em[data-emoji="woman_police_officer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_police_officer_tone5:"]::before,
em[data-emoji="woman_police_officer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_police_officer:"]::before,
em[data-emoji="man_police_officer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-200d-2642-fe0f.svg");
}

em[data-emoji=":man_police_officer_tone1:"]::before,
em[data-emoji="man_police_officer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_police_officer_tone2:"]::before,
em[data-emoji="man_police_officer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_police_officer_tone3:"]::before,
em[data-emoji="man_police_officer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_police_officer_tone4:"]::before,
em[data-emoji="man_police_officer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_police_officer_tone5:"]::before,
em[data-emoji="man_police_officer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46e-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":construction_worker:"]::before,
em[data-emoji="construction_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477.svg");
}

em[data-emoji=":construction_worker_tone1:"]::before,
em[data-emoji="construction_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fb.svg");
}

em[data-emoji=":construction_worker_tone2:"]::before,
em[data-emoji="construction_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fc.svg");
}

em[data-emoji=":construction_worker_tone3:"]::before,
em[data-emoji="construction_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fd.svg");
}

em[data-emoji=":construction_worker_tone4:"]::before,
em[data-emoji="construction_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fe.svg");
}

em[data-emoji=":construction_worker_tone5:"]::before,
em[data-emoji="construction_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3ff.svg");
}

em[data-emoji=":woman_construction_worker:"]::before,
em[data-emoji="woman_construction_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_construction_worker_tone1:"]::before,
em[data-emoji="woman_construction_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_construction_worker_tone2:"]::before,
em[data-emoji="woman_construction_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_construction_worker_tone3:"]::before,
em[data-emoji="woman_construction_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_construction_worker_tone4:"]::before,
em[data-emoji="woman_construction_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_construction_worker_tone5:"]::before,
em[data-emoji="woman_construction_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_construction_worker:"]::before,
em[data-emoji="man_construction_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-200d-2642-fe0f.svg");
}

em[data-emoji=":man_construction_worker_tone1:"]::before,
em[data-emoji="man_construction_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_construction_worker_tone2:"]::before,
em[data-emoji="man_construction_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_construction_worker_tone3:"]::before,
em[data-emoji="man_construction_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_construction_worker_tone4:"]::before,
em[data-emoji="man_construction_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_construction_worker_tone5:"]::before,
em[data-emoji="man_construction_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f477-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":guard:"]::before,
em[data-emoji="guard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482.svg");
}

em[data-emoji=":guard_tone1:"]::before,
em[data-emoji="guard_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fb.svg");
}

em[data-emoji=":guard_tone2:"]::before,
em[data-emoji="guard_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fc.svg");
}

em[data-emoji=":guard_tone3:"]::before,
em[data-emoji="guard_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fd.svg");
}

em[data-emoji=":guard_tone4:"]::before,
em[data-emoji="guard_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fe.svg");
}

em[data-emoji=":guard_tone5:"]::before,
em[data-emoji="guard_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3ff.svg");
}

em[data-emoji=":woman_guard:"]::before,
em[data-emoji="woman_guard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_guard_tone1:"]::before,
em[data-emoji="woman_guard_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_guard_tone2:"]::before,
em[data-emoji="woman_guard_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_guard_tone3:"]::before,
em[data-emoji="woman_guard_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_guard_tone4:"]::before,
em[data-emoji="woman_guard_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_guard_tone5:"]::before,
em[data-emoji="woman_guard_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_guard:"]::before,
em[data-emoji="man_guard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-200d-2642-fe0f.svg");
}

em[data-emoji=":man_guard_tone1:"]::before,
em[data-emoji="man_guard_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_guard_tone2:"]::before,
em[data-emoji="man_guard_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_guard_tone3:"]::before,
em[data-emoji="man_guard_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_guard_tone4:"]::before,
em[data-emoji="man_guard_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_guard_tone5:"]::before,
em[data-emoji="man_guard_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f482-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":detective:"]::before,
em[data-emoji="detective"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575.svg");
}

em[data-emoji=":detective_tone1:"]::before,
em[data-emoji="detective_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fb.svg");
}

em[data-emoji=":detective_tone2:"]::before,
em[data-emoji="detective_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fc.svg");
}

em[data-emoji=":detective_tone3:"]::before,
em[data-emoji="detective_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fd.svg");
}

em[data-emoji=":detective_tone4:"]::before,
em[data-emoji="detective_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fe.svg");
}

em[data-emoji=":detective_tone5:"]::before,
em[data-emoji="detective_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3ff.svg");
}

em[data-emoji=":woman_detective:"]::before,
em[data-emoji="woman_detective"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-fe0f-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_detective_tone1:"]::before,
em[data-emoji="woman_detective_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_detective_tone2:"]::before,
em[data-emoji="woman_detective_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_detective_tone3:"]::before,
em[data-emoji="woman_detective_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_detective_tone4:"]::before,
em[data-emoji="woman_detective_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_detective_tone5:"]::before,
em[data-emoji="woman_detective_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_detective:"]::before,
em[data-emoji="man_detective"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-fe0f-200d-2642-fe0f.svg");
}

em[data-emoji=":man_detective_tone1:"]::before,
em[data-emoji="man_detective_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_detective_tone2:"]::before,
em[data-emoji="man_detective_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_detective_tone3:"]::before,
em[data-emoji="man_detective_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_detective_tone4:"]::before,
em[data-emoji="man_detective_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_detective_tone5:"]::before,
em[data-emoji="man_detective_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f575-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":health_worker:"]::before,
em[data-emoji="health_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-2695-fe0f.svg");
}

em[data-emoji=":health_worker_tone1:"]::before,
em[data-emoji="health_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2695-fe0f.svg");
}

em[data-emoji=":health_worker_tone2:"]::before,
em[data-emoji="health_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2695-fe0f.svg");
}

em[data-emoji=":health_worker_tone3:"]::before,
em[data-emoji="health_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2695-fe0f.svg");
}

em[data-emoji=":health_worker_tone4:"]::before,
em[data-emoji="health_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2695-fe0f.svg");
}

em[data-emoji=":health_worker_tone5:"]::before,
em[data-emoji="health_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker:"]::before,
em[data-emoji="woman_health_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker_tone1:"]::before,
em[data-emoji="woman_health_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker_tone2:"]::before,
em[data-emoji="woman_health_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker_tone3:"]::before,
em[data-emoji="woman_health_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker_tone4:"]::before,
em[data-emoji="woman_health_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2695-fe0f.svg");
}

em[data-emoji=":woman_health_worker_tone5:"]::before,
em[data-emoji="woman_health_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker:"]::before,
em[data-emoji="man_health_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker_tone1:"]::before,
em[data-emoji="man_health_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker_tone2:"]::before,
em[data-emoji="man_health_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker_tone3:"]::before,
em[data-emoji="man_health_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker_tone4:"]::before,
em[data-emoji="man_health_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2695-fe0f.svg");
}

em[data-emoji=":man_health_worker_tone5:"]::before,
em[data-emoji="man_health_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2695-fe0f.svg");
}

em[data-emoji=":farmer:"]::before,
em[data-emoji="farmer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f33e.svg");
}

em[data-emoji=":farmer_tone1:"]::before,
em[data-emoji="farmer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f33e.svg");
}

em[data-emoji=":farmer_tone2:"]::before,
em[data-emoji="farmer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f33e.svg");
}

em[data-emoji=":farmer_tone3:"]::before,
em[data-emoji="farmer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f33e.svg");
}

em[data-emoji=":farmer_tone4:"]::before,
em[data-emoji="farmer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f33e.svg");
}

em[data-emoji=":farmer_tone5:"]::before,
em[data-emoji="farmer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer:"]::before,
em[data-emoji="woman_farmer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer_tone1:"]::before,
em[data-emoji="woman_farmer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer_tone2:"]::before,
em[data-emoji="woman_farmer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer_tone3:"]::before,
em[data-emoji="woman_farmer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer_tone4:"]::before,
em[data-emoji="woman_farmer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f33e.svg");
}

em[data-emoji=":woman_farmer_tone5:"]::before,
em[data-emoji="woman_farmer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f33e.svg");
}

em[data-emoji=":man_farmer:"]::before,
em[data-emoji="man_farmer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f33e.svg");
}

em[data-emoji=":man_farmer_tone1:"]::before,
em[data-emoji="man_farmer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f33e.svg");
}

em[data-emoji=":man_farmer_tone2:"]::before,
em[data-emoji="man_farmer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f33e.svg");
}

em[data-emoji=":man_farmer_tone3:"]::before,
em[data-emoji="man_farmer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f33e.svg");
}

em[data-emoji=":man_farmer_tone4:"]::before,
em[data-emoji="man_farmer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f33e.svg");
}

em[data-emoji=":man_farmer_tone5:"]::before,
em[data-emoji="man_farmer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f33e.svg");
}

em[data-emoji=":cook:"]::before,
em[data-emoji="cook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f373.svg");
}

em[data-emoji=":cook_tone1:"]::before,
em[data-emoji="cook_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f373.svg");
}

em[data-emoji=":cook_tone2:"]::before,
em[data-emoji="cook_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f373.svg");
}

em[data-emoji=":cook_tone3:"]::before,
em[data-emoji="cook_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f373.svg");
}

em[data-emoji=":cook_tone4:"]::before,
em[data-emoji="cook_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f373.svg");
}

em[data-emoji=":cook_tone5:"]::before,
em[data-emoji="cook_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f373.svg");
}

em[data-emoji=":woman_cook:"]::before,
em[data-emoji="woman_cook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f373.svg");
}

em[data-emoji=":woman_cook_tone1:"]::before,
em[data-emoji="woman_cook_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f373.svg");
}

em[data-emoji=":woman_cook_tone2:"]::before,
em[data-emoji="woman_cook_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f373.svg");
}

em[data-emoji=":woman_cook_tone3:"]::before,
em[data-emoji="woman_cook_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f373.svg");
}

em[data-emoji=":woman_cook_tone4:"]::before,
em[data-emoji="woman_cook_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f373.svg");
}

em[data-emoji=":woman_cook_tone5:"]::before,
em[data-emoji="woman_cook_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f373.svg");
}

em[data-emoji=":man_cook:"]::before,
em[data-emoji="man_cook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f373.svg");
}

em[data-emoji=":man_cook_tone1:"]::before,
em[data-emoji="man_cook_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f373.svg");
}

em[data-emoji=":man_cook_tone2:"]::before,
em[data-emoji="man_cook_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f373.svg");
}

em[data-emoji=":man_cook_tone3:"]::before,
em[data-emoji="man_cook_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f373.svg");
}

em[data-emoji=":man_cook_tone4:"]::before,
em[data-emoji="man_cook_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f373.svg");
}

em[data-emoji=":man_cook_tone5:"]::before,
em[data-emoji="man_cook_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f373.svg");
}

em[data-emoji=":student:"]::before,
em[data-emoji="student"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f393.svg");
}

em[data-emoji=":student_tone1:"]::before,
em[data-emoji="student_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f393.svg");
}

em[data-emoji=":student_tone2:"]::before,
em[data-emoji="student_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f393.svg");
}

em[data-emoji=":student_tone3:"]::before,
em[data-emoji="student_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f393.svg");
}

em[data-emoji=":student_tone4:"]::before,
em[data-emoji="student_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f393.svg");
}

em[data-emoji=":student_tone5:"]::before,
em[data-emoji="student_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f393.svg");
}

em[data-emoji=":woman_student:"]::before,
em[data-emoji="woman_student"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f393.svg");
}

em[data-emoji=":woman_student_tone1:"]::before,
em[data-emoji="woman_student_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f393.svg");
}

em[data-emoji=":woman_student_tone2:"]::before,
em[data-emoji="woman_student_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f393.svg");
}

em[data-emoji=":woman_student_tone3:"]::before,
em[data-emoji="woman_student_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f393.svg");
}

em[data-emoji=":woman_student_tone4:"]::before,
em[data-emoji="woman_student_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f393.svg");
}

em[data-emoji=":woman_student_tone5:"]::before,
em[data-emoji="woman_student_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f393.svg");
}

em[data-emoji=":man_student:"]::before,
em[data-emoji="man_student"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f393.svg");
}

em[data-emoji=":man_student_tone1:"]::before,
em[data-emoji="man_student_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f393.svg");
}

em[data-emoji=":man_student_tone2:"]::before,
em[data-emoji="man_student_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f393.svg");
}

em[data-emoji=":man_student_tone3:"]::before,
em[data-emoji="man_student_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f393.svg");
}

em[data-emoji=":man_student_tone4:"]::before,
em[data-emoji="man_student_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f393.svg");
}

em[data-emoji=":man_student_tone5:"]::before,
em[data-emoji="man_student_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f393.svg");
}

em[data-emoji=":singer:"]::before,
em[data-emoji="singer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f3a4.svg");
}

em[data-emoji=":singer_tone1:"]::before,
em[data-emoji="singer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f3a4.svg");
}

em[data-emoji=":singer_tone2:"]::before,
em[data-emoji="singer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f3a4.svg");
}

em[data-emoji=":singer_tone3:"]::before,
em[data-emoji="singer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f3a4.svg");
}

em[data-emoji=":singer_tone4:"]::before,
em[data-emoji="singer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f3a4.svg");
}

em[data-emoji=":singer_tone5:"]::before,
em[data-emoji="singer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer:"]::before,
em[data-emoji="woman_singer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer_tone1:"]::before,
em[data-emoji="woman_singer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer_tone2:"]::before,
em[data-emoji="woman_singer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer_tone3:"]::before,
em[data-emoji="woman_singer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer_tone4:"]::before,
em[data-emoji="woman_singer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f3a4.svg");
}

em[data-emoji=":woman_singer_tone5:"]::before,
em[data-emoji="woman_singer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f3a4.svg");
}

em[data-emoji=":man_singer:"]::before,
em[data-emoji="man_singer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f3a4.svg");
}

em[data-emoji=":man_singer_tone1:"]::before,
em[data-emoji="man_singer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f3a4.svg");
}

em[data-emoji=":man_singer_tone2:"]::before,
em[data-emoji="man_singer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f3a4.svg");
}

em[data-emoji=":man_singer_tone3:"]::before,
em[data-emoji="man_singer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f3a4.svg");
}

em[data-emoji=":man_singer_tone4:"]::before,
em[data-emoji="man_singer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f3a4.svg");
}

em[data-emoji=":man_singer_tone5:"]::before,
em[data-emoji="man_singer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f3a4.svg");
}

em[data-emoji=":teacher:"]::before,
em[data-emoji="teacher"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f3eb.svg");
}

em[data-emoji=":teacher_tone1:"]::before,
em[data-emoji="teacher_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f3eb.svg");
}

em[data-emoji=":teacher_tone2:"]::before,
em[data-emoji="teacher_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f3eb.svg");
}

em[data-emoji=":teacher_tone3:"]::before,
em[data-emoji="teacher_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f3eb.svg");
}

em[data-emoji=":teacher_tone4:"]::before,
em[data-emoji="teacher_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f3eb.svg");
}

em[data-emoji=":teacher_tone5:"]::before,
em[data-emoji="teacher_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher:"]::before,
em[data-emoji="woman_teacher"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher_tone1:"]::before,
em[data-emoji="woman_teacher_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher_tone2:"]::before,
em[data-emoji="woman_teacher_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher_tone3:"]::before,
em[data-emoji="woman_teacher_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher_tone4:"]::before,
em[data-emoji="woman_teacher_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f3eb.svg");
}

em[data-emoji=":woman_teacher_tone5:"]::before,
em[data-emoji="woman_teacher_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher:"]::before,
em[data-emoji="man_teacher"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher_tone1:"]::before,
em[data-emoji="man_teacher_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher_tone2:"]::before,
em[data-emoji="man_teacher_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher_tone3:"]::before,
em[data-emoji="man_teacher_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher_tone4:"]::before,
em[data-emoji="man_teacher_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f3eb.svg");
}

em[data-emoji=":man_teacher_tone5:"]::before,
em[data-emoji="man_teacher_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f3eb.svg");
}

em[data-emoji=":factory_worker:"]::before,
em[data-emoji="factory_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f3ed.svg");
}

em[data-emoji=":factory_worker_tone1:"]::before,
em[data-emoji="factory_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f3ed.svg");
}

em[data-emoji=":factory_worker_tone2:"]::before,
em[data-emoji="factory_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f3ed.svg");
}

em[data-emoji=":factory_worker_tone3:"]::before,
em[data-emoji="factory_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f3ed.svg");
}

em[data-emoji=":factory_worker_tone4:"]::before,
em[data-emoji="factory_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f3ed.svg");
}

em[data-emoji=":factory_worker_tone5:"]::before,
em[data-emoji="factory_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker:"]::before,
em[data-emoji="woman_factory_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker_tone1:"]::before,
em[data-emoji="woman_factory_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker_tone2:"]::before,
em[data-emoji="woman_factory_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker_tone3:"]::before,
em[data-emoji="woman_factory_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker_tone4:"]::before,
em[data-emoji="woman_factory_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f3ed.svg");
}

em[data-emoji=":woman_factory_worker_tone5:"]::before,
em[data-emoji="woman_factory_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker:"]::before,
em[data-emoji="man_factory_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker_tone1:"]::before,
em[data-emoji="man_factory_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker_tone2:"]::before,
em[data-emoji="man_factory_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker_tone3:"]::before,
em[data-emoji="man_factory_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker_tone4:"]::before,
em[data-emoji="man_factory_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f3ed.svg");
}

em[data-emoji=":man_factory_worker_tone5:"]::before,
em[data-emoji="man_factory_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f3ed.svg");
}

em[data-emoji=":technologist:"]::before,
em[data-emoji="technologist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f4bb.svg");
}

em[data-emoji=":technologist_tone1:"]::before,
em[data-emoji="technologist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f4bb.svg");
}

em[data-emoji=":technologist_tone2:"]::before,
em[data-emoji="technologist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f4bb.svg");
}

em[data-emoji=":technologist_tone3:"]::before,
em[data-emoji="technologist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f4bb.svg");
}

em[data-emoji=":technologist_tone4:"]::before,
em[data-emoji="technologist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f4bb.svg");
}

em[data-emoji=":technologist_tone5:"]::before,
em[data-emoji="technologist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist:"]::before,
em[data-emoji="woman_technologist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist_tone1:"]::before,
em[data-emoji="woman_technologist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist_tone2:"]::before,
em[data-emoji="woman_technologist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist_tone3:"]::before,
em[data-emoji="woman_technologist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist_tone4:"]::before,
em[data-emoji="woman_technologist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f4bb.svg");
}

em[data-emoji=":woman_technologist_tone5:"]::before,
em[data-emoji="woman_technologist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist:"]::before,
em[data-emoji="man_technologist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist_tone1:"]::before,
em[data-emoji="man_technologist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist_tone2:"]::before,
em[data-emoji="man_technologist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist_tone3:"]::before,
em[data-emoji="man_technologist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist_tone4:"]::before,
em[data-emoji="man_technologist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f4bb.svg");
}

em[data-emoji=":man_technologist_tone5:"]::before,
em[data-emoji="man_technologist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f4bb.svg");
}

em[data-emoji=":office_worker:"]::before,
em[data-emoji="office_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f4bc.svg");
}

em[data-emoji=":office_worker_tone1:"]::before,
em[data-emoji="office_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f4bc.svg");
}

em[data-emoji=":office_worker_tone2:"]::before,
em[data-emoji="office_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f4bc.svg");
}

em[data-emoji=":office_worker_tone3:"]::before,
em[data-emoji="office_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f4bc.svg");
}

em[data-emoji=":office_worker_tone4:"]::before,
em[data-emoji="office_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f4bc.svg");
}

em[data-emoji=":office_worker_tone5:"]::before,
em[data-emoji="office_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker:"]::before,
em[data-emoji="woman_office_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker_tone1:"]::before,
em[data-emoji="woman_office_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker_tone2:"]::before,
em[data-emoji="woman_office_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker_tone3:"]::before,
em[data-emoji="woman_office_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker_tone4:"]::before,
em[data-emoji="woman_office_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f4bc.svg");
}

em[data-emoji=":woman_office_worker_tone5:"]::before,
em[data-emoji="woman_office_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker:"]::before,
em[data-emoji="man_office_worker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker_tone1:"]::before,
em[data-emoji="man_office_worker_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker_tone2:"]::before,
em[data-emoji="man_office_worker_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker_tone3:"]::before,
em[data-emoji="man_office_worker_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker_tone4:"]::before,
em[data-emoji="man_office_worker_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f4bc.svg");
}

em[data-emoji=":man_office_worker_tone5:"]::before,
em[data-emoji="man_office_worker_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f4bc.svg");
}

em[data-emoji=":mechanic:"]::before,
em[data-emoji="mechanic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f527.svg");
}

em[data-emoji=":mechanic_tone1:"]::before,
em[data-emoji="mechanic_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f527.svg");
}

em[data-emoji=":mechanic_tone2:"]::before,
em[data-emoji="mechanic_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f527.svg");
}

em[data-emoji=":mechanic_tone3:"]::before,
em[data-emoji="mechanic_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f527.svg");
}

em[data-emoji=":mechanic_tone4:"]::before,
em[data-emoji="mechanic_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f527.svg");
}

em[data-emoji=":mechanic_tone5:"]::before,
em[data-emoji="mechanic_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic:"]::before,
em[data-emoji="woman_mechanic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic_tone1:"]::before,
em[data-emoji="woman_mechanic_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic_tone2:"]::before,
em[data-emoji="woman_mechanic_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic_tone3:"]::before,
em[data-emoji="woman_mechanic_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic_tone4:"]::before,
em[data-emoji="woman_mechanic_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f527.svg");
}

em[data-emoji=":woman_mechanic_tone5:"]::before,
em[data-emoji="woman_mechanic_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f527.svg");
}

em[data-emoji=":man_mechanic:"]::before,
em[data-emoji="man_mechanic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f527.svg");
}

em[data-emoji=":man_mechanic_tone1:"]::before,
em[data-emoji="man_mechanic_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f527.svg");
}

em[data-emoji=":man_mechanic_tone2:"]::before,
em[data-emoji="man_mechanic_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f527.svg");
}

em[data-emoji=":man_mechanic_tone3:"]::before,
em[data-emoji="man_mechanic_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f527.svg");
}

em[data-emoji=":man_mechanic_tone4:"]::before,
em[data-emoji="man_mechanic_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f527.svg");
}

em[data-emoji=":man_mechanic_tone5:"]::before,
em[data-emoji="man_mechanic_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f527.svg");
}

em[data-emoji=":scientist:"]::before,
em[data-emoji="scientist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f52c.svg");
}

em[data-emoji=":scientist_tone1:"]::before,
em[data-emoji="scientist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f52c.svg");
}

em[data-emoji=":scientist_tone2:"]::before,
em[data-emoji="scientist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f52c.svg");
}

em[data-emoji=":scientist_tone3:"]::before,
em[data-emoji="scientist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f52c.svg");
}

em[data-emoji=":scientist_tone4:"]::before,
em[data-emoji="scientist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f52c.svg");
}

em[data-emoji=":scientist_tone5:"]::before,
em[data-emoji="scientist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist:"]::before,
em[data-emoji="woman_scientist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist_tone1:"]::before,
em[data-emoji="woman_scientist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist_tone2:"]::before,
em[data-emoji="woman_scientist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist_tone3:"]::before,
em[data-emoji="woman_scientist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist_tone4:"]::before,
em[data-emoji="woman_scientist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f52c.svg");
}

em[data-emoji=":woman_scientist_tone5:"]::before,
em[data-emoji="woman_scientist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f52c.svg");
}

em[data-emoji=":man_scientist:"]::before,
em[data-emoji="man_scientist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f52c.svg");
}

em[data-emoji=":man_scientist_tone1:"]::before,
em[data-emoji="man_scientist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f52c.svg");
}

em[data-emoji=":man_scientist_tone2:"]::before,
em[data-emoji="man_scientist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f52c.svg");
}

em[data-emoji=":man_scientist_tone3:"]::before,
em[data-emoji="man_scientist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f52c.svg");
}

em[data-emoji=":man_scientist_tone4:"]::before,
em[data-emoji="man_scientist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f52c.svg");
}

em[data-emoji=":man_scientist_tone5:"]::before,
em[data-emoji="man_scientist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f52c.svg");
}

em[data-emoji=":artist:"]::before,
em[data-emoji="artist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f3a8.svg");
}

em[data-emoji=":artist_tone1:"]::before,
em[data-emoji="artist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f3a8.svg");
}

em[data-emoji=":artist_tone2:"]::before,
em[data-emoji="artist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f3a8.svg");
}

em[data-emoji=":artist_tone3:"]::before,
em[data-emoji="artist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f3a8.svg");
}

em[data-emoji=":artist_tone4:"]::before,
em[data-emoji="artist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f3a8.svg");
}

em[data-emoji=":artist_tone5:"]::before,
em[data-emoji="artist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist:"]::before,
em[data-emoji="woman_artist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist_tone1:"]::before,
em[data-emoji="woman_artist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist_tone2:"]::before,
em[data-emoji="woman_artist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist_tone3:"]::before,
em[data-emoji="woman_artist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist_tone4:"]::before,
em[data-emoji="woman_artist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f3a8.svg");
}

em[data-emoji=":woman_artist_tone5:"]::before,
em[data-emoji="woman_artist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f3a8.svg");
}

em[data-emoji=":man_artist:"]::before,
em[data-emoji="man_artist"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f3a8.svg");
}

em[data-emoji=":man_artist_tone1:"]::before,
em[data-emoji="man_artist_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f3a8.svg");
}

em[data-emoji=":man_artist_tone2:"]::before,
em[data-emoji="man_artist_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f3a8.svg");
}

em[data-emoji=":man_artist_tone3:"]::before,
em[data-emoji="man_artist_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f3a8.svg");
}

em[data-emoji=":man_artist_tone4:"]::before,
em[data-emoji="man_artist_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f3a8.svg");
}

em[data-emoji=":man_artist_tone5:"]::before,
em[data-emoji="man_artist_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f3a8.svg");
}

em[data-emoji=":firefighter:"]::before,
em[data-emoji="firefighter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f692.svg");
}

em[data-emoji=":firefighter_tone1:"]::before,
em[data-emoji="firefighter_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f692.svg");
}

em[data-emoji=":firefighter_tone2:"]::before,
em[data-emoji="firefighter_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f692.svg");
}

em[data-emoji=":firefighter_tone3:"]::before,
em[data-emoji="firefighter_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f692.svg");
}

em[data-emoji=":firefighter_tone4:"]::before,
em[data-emoji="firefighter_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f692.svg");
}

em[data-emoji=":firefighter_tone5:"]::before,
em[data-emoji="firefighter_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter:"]::before,
em[data-emoji="woman_firefighter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter_tone1:"]::before,
em[data-emoji="woman_firefighter_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter_tone2:"]::before,
em[data-emoji="woman_firefighter_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter_tone3:"]::before,
em[data-emoji="woman_firefighter_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter_tone4:"]::before,
em[data-emoji="woman_firefighter_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f692.svg");
}

em[data-emoji=":woman_firefighter_tone5:"]::before,
em[data-emoji="woman_firefighter_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f692.svg");
}

em[data-emoji=":man_firefighter:"]::before,
em[data-emoji="man_firefighter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f692.svg");
}

em[data-emoji=":man_firefighter_tone1:"]::before,
em[data-emoji="man_firefighter_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f692.svg");
}

em[data-emoji=":man_firefighter_tone2:"]::before,
em[data-emoji="man_firefighter_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f692.svg");
}

em[data-emoji=":man_firefighter_tone3:"]::before,
em[data-emoji="man_firefighter_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f692.svg");
}

em[data-emoji=":man_firefighter_tone4:"]::before,
em[data-emoji="man_firefighter_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f692.svg");
}

em[data-emoji=":man_firefighter_tone5:"]::before,
em[data-emoji="man_firefighter_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f692.svg");
}

em[data-emoji=":pilot:"]::before,
em[data-emoji="pilot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-2708-fe0f.svg");
}

em[data-emoji=":pilot_tone1:"]::before,
em[data-emoji="pilot_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2708-fe0f.svg");
}

em[data-emoji=":pilot_tone2:"]::before,
em[data-emoji="pilot_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2708-fe0f.svg");
}

em[data-emoji=":pilot_tone3:"]::before,
em[data-emoji="pilot_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2708-fe0f.svg");
}

em[data-emoji=":pilot_tone4:"]::before,
em[data-emoji="pilot_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2708-fe0f.svg");
}

em[data-emoji=":pilot_tone5:"]::before,
em[data-emoji="pilot_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot:"]::before,
em[data-emoji="woman_pilot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot_tone1:"]::before,
em[data-emoji="woman_pilot_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot_tone2:"]::before,
em[data-emoji="woman_pilot_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot_tone3:"]::before,
em[data-emoji="woman_pilot_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot_tone4:"]::before,
em[data-emoji="woman_pilot_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2708-fe0f.svg");
}

em[data-emoji=":woman_pilot_tone5:"]::before,
em[data-emoji="woman_pilot_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot:"]::before,
em[data-emoji="man_pilot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot_tone1:"]::before,
em[data-emoji="man_pilot_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot_tone2:"]::before,
em[data-emoji="man_pilot_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot_tone3:"]::before,
em[data-emoji="man_pilot_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot_tone4:"]::before,
em[data-emoji="man_pilot_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2708-fe0f.svg");
}

em[data-emoji=":man_pilot_tone5:"]::before,
em[data-emoji="man_pilot_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2708-fe0f.svg");
}

em[data-emoji=":astronaut:"]::before,
em[data-emoji="astronaut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f680.svg");
}

em[data-emoji=":astronaut_tone1:"]::before,
em[data-emoji="astronaut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f680.svg");
}

em[data-emoji=":astronaut_tone2:"]::before,
em[data-emoji="astronaut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f680.svg");
}

em[data-emoji=":astronaut_tone3:"]::before,
em[data-emoji="astronaut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f680.svg");
}

em[data-emoji=":astronaut_tone4:"]::before,
em[data-emoji="astronaut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f680.svg");
}

em[data-emoji=":astronaut_tone5:"]::before,
em[data-emoji="astronaut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut:"]::before,
em[data-emoji="woman_astronaut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut_tone1:"]::before,
em[data-emoji="woman_astronaut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut_tone2:"]::before,
em[data-emoji="woman_astronaut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut_tone3:"]::before,
em[data-emoji="woman_astronaut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut_tone4:"]::before,
em[data-emoji="woman_astronaut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f680.svg");
}

em[data-emoji=":woman_astronaut_tone5:"]::before,
em[data-emoji="woman_astronaut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f680.svg");
}

em[data-emoji=":man_astronaut:"]::before,
em[data-emoji="man_astronaut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f680.svg");
}

em[data-emoji=":man_astronaut_tone1:"]::before,
em[data-emoji="man_astronaut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f680.svg");
}

em[data-emoji=":man_astronaut_tone2:"]::before,
em[data-emoji="man_astronaut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f680.svg");
}

em[data-emoji=":man_astronaut_tone3:"]::before,
em[data-emoji="man_astronaut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f680.svg");
}

em[data-emoji=":man_astronaut_tone4:"]::before,
em[data-emoji="man_astronaut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f680.svg");
}

em[data-emoji=":man_astronaut_tone5:"]::before,
em[data-emoji="man_astronaut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f680.svg");
}

em[data-emoji=":judge:"]::before,
em[data-emoji="judge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-2696-fe0f.svg");
}

em[data-emoji=":judge_tone1:"]::before,
em[data-emoji="judge_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2696-fe0f.svg");
}

em[data-emoji=":judge_tone2:"]::before,
em[data-emoji="judge_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2696-fe0f.svg");
}

em[data-emoji=":judge_tone3:"]::before,
em[data-emoji="judge_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2696-fe0f.svg");
}

em[data-emoji=":judge_tone4:"]::before,
em[data-emoji="judge_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2696-fe0f.svg");
}

em[data-emoji=":judge_tone5:"]::before,
em[data-emoji="judge_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge:"]::before,
em[data-emoji="woman_judge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge_tone1:"]::before,
em[data-emoji="woman_judge_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge_tone2:"]::before,
em[data-emoji="woman_judge_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge_tone3:"]::before,
em[data-emoji="woman_judge_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge_tone4:"]::before,
em[data-emoji="woman_judge_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2696-fe0f.svg");
}

em[data-emoji=":woman_judge_tone5:"]::before,
em[data-emoji="woman_judge_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge:"]::before,
em[data-emoji="man_judge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge_tone1:"]::before,
em[data-emoji="man_judge_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge_tone2:"]::before,
em[data-emoji="man_judge_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge_tone3:"]::before,
em[data-emoji="man_judge_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge_tone4:"]::before,
em[data-emoji="man_judge_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2696-fe0f.svg");
}

em[data-emoji=":man_judge_tone5:"]::before,
em[data-emoji="man_judge_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2696-fe0f.svg");
}

em[data-emoji=":person_with_veil:"]::before,
em[data-emoji="person_with_veil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470.svg");
}

em[data-emoji=":person_with_veil_tone1:"]::before,
em[data-emoji="person_with_veil_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fb.svg");
}

em[data-emoji=":person_with_veil_tone2:"]::before,
em[data-emoji="person_with_veil_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fc.svg");
}

em[data-emoji=":person_with_veil_tone3:"]::before,
em[data-emoji="person_with_veil_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fd.svg");
}

em[data-emoji=":person_with_veil_tone4:"]::before,
em[data-emoji="person_with_veil_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fe.svg");
}

em[data-emoji=":person_with_veil_tone5:"]::before,
em[data-emoji="person_with_veil_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3ff.svg");
}

em[data-emoji=":woman_with_veil:"]::before,
em[data-emoji="woman_with_veil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_with_veil_tone1:"]::before,
em[data-emoji="woman_with_veil_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_with_veil_tone2:"]::before,
em[data-emoji="woman_with_veil_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_with_veil_tone3:"]::before,
em[data-emoji="woman_with_veil_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_with_veil_tone4:"]::before,
em[data-emoji="woman_with_veil_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_with_veil_tone5:"]::before,
em[data-emoji="woman_with_veil_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_with_veil:"]::before,
em[data-emoji="man_with_veil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-200d-2642-fe0f.svg");
}

em[data-emoji=":man_with_veil_tone1:"]::before,
em[data-emoji="man_with_veil_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_with_veil_tone2:"]::before,
em[data-emoji="man_with_veil_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_with_veil_tone3:"]::before,
em[data-emoji="man_with_veil_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_with_veil_tone4:"]::before,
em[data-emoji="man_with_veil_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_with_veil_tone5:"]::before,
em[data-emoji="man_with_veil_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f470-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_in_tuxedo:"]::before,
em[data-emoji="person_in_tuxedo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935.svg");
}

em[data-emoji=":person_in_tuxedo_tone1:"]::before,
em[data-emoji="person_in_tuxedo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fb.svg");
}

em[data-emoji=":person_in_tuxedo_tone2:"]::before,
em[data-emoji="person_in_tuxedo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fc.svg");
}

em[data-emoji=":person_in_tuxedo_tone3:"]::before,
em[data-emoji="person_in_tuxedo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fd.svg");
}

em[data-emoji=":person_in_tuxedo_tone4:"]::before,
em[data-emoji="person_in_tuxedo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fe.svg");
}

em[data-emoji=":person_in_tuxedo_tone5:"]::before,
em[data-emoji="person_in_tuxedo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3ff.svg");
}

em[data-emoji=":woman_in_tuxedo:"]::before,
em[data-emoji="woman_in_tuxedo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_tuxedo_tone1:"]::before,
em[data-emoji="woman_in_tuxedo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_tuxedo_tone2:"]::before,
em[data-emoji="woman_in_tuxedo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_tuxedo_tone3:"]::before,
em[data-emoji="woman_in_tuxedo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_tuxedo_tone4:"]::before,
em[data-emoji="woman_in_tuxedo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_tuxedo_tone5:"]::before,
em[data-emoji="woman_in_tuxedo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo:"]::before,
em[data-emoji="man_in_tuxedo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo_tone1:"]::before,
em[data-emoji="man_in_tuxedo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo_tone2:"]::before,
em[data-emoji="man_in_tuxedo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo_tone3:"]::before,
em[data-emoji="man_in_tuxedo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo_tone4:"]::before,
em[data-emoji="man_in_tuxedo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_tuxedo_tone5:"]::before,
em[data-emoji="man_in_tuxedo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f935-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_with_crown:"]::before,
em[data-emoji="person_with_crown"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5.svg");
}

em[data-emoji=":person_with_crown_tone1:"]::before,
em[data-emoji="person_with_crown_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5-1f3fb.svg");
}

em[data-emoji=":person_with_crown_tone2:"]::before,
em[data-emoji="person_with_crown_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5-1f3fc.svg");
}

em[data-emoji=":person_with_crown_tone3:"]::before,
em[data-emoji="person_with_crown_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5-1f3fd.svg");
}

em[data-emoji=":person_with_crown_tone4:"]::before,
em[data-emoji="person_with_crown_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5-1f3fe.svg");
}

em[data-emoji=":person_with_crown_tone5:"]::before,
em[data-emoji="person_with_crown_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac5-1f3ff.svg");
}

em[data-emoji=":princess:"]::before,
em[data-emoji="princess"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478.svg");
}

em[data-emoji=":princess_tone1:"]::before,
em[data-emoji="princess_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478-1f3fb.svg");
}

em[data-emoji=":princess_tone2:"]::before,
em[data-emoji="princess_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478-1f3fc.svg");
}

em[data-emoji=":princess_tone3:"]::before,
em[data-emoji="princess_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478-1f3fd.svg");
}

em[data-emoji=":princess_tone4:"]::before,
em[data-emoji="princess_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478-1f3fe.svg");
}

em[data-emoji=":princess_tone5:"]::before,
em[data-emoji="princess_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f478-1f3ff.svg");
}

em[data-emoji=":prince:"]::before,
em[data-emoji="prince"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934.svg");
}

em[data-emoji=":prince_tone1:"]::before,
em[data-emoji="prince_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934-1f3fb.svg");
}

em[data-emoji=":prince_tone2:"]::before,
em[data-emoji="prince_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934-1f3fc.svg");
}

em[data-emoji=":prince_tone3:"]::before,
em[data-emoji="prince_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934-1f3fd.svg");
}

em[data-emoji=":prince_tone4:"]::before,
em[data-emoji="prince_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934-1f3fe.svg");
}

em[data-emoji=":prince_tone5:"]::before,
em[data-emoji="prince_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f934-1f3ff.svg");
}

em[data-emoji=":superhero:"]::before,
em[data-emoji="superhero"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8.svg");
}

em[data-emoji=":superhero_tone1:"]::before,
em[data-emoji="superhero_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fb.svg");
}

em[data-emoji=":superhero_tone2:"]::before,
em[data-emoji="superhero_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fc.svg");
}

em[data-emoji=":superhero_tone3:"]::before,
em[data-emoji="superhero_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fd.svg");
}

em[data-emoji=":superhero_tone4:"]::before,
em[data-emoji="superhero_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fe.svg");
}

em[data-emoji=":superhero_tone5:"]::before,
em[data-emoji="superhero_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3ff.svg");
}

em[data-emoji=":woman_superhero:"]::before,
em[data-emoji="woman_superhero"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_superhero_tone1:"]::before,
em[data-emoji="woman_superhero_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_superhero_tone2:"]::before,
em[data-emoji="woman_superhero_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_superhero_tone3:"]::before,
em[data-emoji="woman_superhero_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_superhero_tone4:"]::before,
em[data-emoji="woman_superhero_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_superhero_tone5:"]::before,
em[data-emoji="woman_superhero_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_superhero:"]::before,
em[data-emoji="man_superhero"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-200d-2642-fe0f.svg");
}

em[data-emoji=":man_superhero_tone1:"]::before,
em[data-emoji="man_superhero_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_superhero_tone2:"]::before,
em[data-emoji="man_superhero_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_superhero_tone3:"]::before,
em[data-emoji="man_superhero_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_superhero_tone4:"]::before,
em[data-emoji="man_superhero_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_superhero_tone5:"]::before,
em[data-emoji="man_superhero_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b8-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":supervillain:"]::before,
em[data-emoji="supervillain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9.svg");
}

em[data-emoji=":supervillain_tone1:"]::before,
em[data-emoji="supervillain_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fb.svg");
}

em[data-emoji=":supervillain_tone2:"]::before,
em[data-emoji="supervillain_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fc.svg");
}

em[data-emoji=":supervillain_tone3:"]::before,
em[data-emoji="supervillain_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fd.svg");
}

em[data-emoji=":supervillain_tone4:"]::before,
em[data-emoji="supervillain_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fe.svg");
}

em[data-emoji=":supervillain_tone5:"]::before,
em[data-emoji="supervillain_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3ff.svg");
}

em[data-emoji=":woman_supervillain:"]::before,
em[data-emoji="woman_supervillain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_supervillain_tone1:"]::before,
em[data-emoji="woman_supervillain_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_supervillain_tone2:"]::before,
em[data-emoji="woman_supervillain_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_supervillain_tone3:"]::before,
em[data-emoji="woman_supervillain_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_supervillain_tone4:"]::before,
em[data-emoji="woman_supervillain_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_supervillain_tone5:"]::before,
em[data-emoji="woman_supervillain_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_supervillain:"]::before,
em[data-emoji="man_supervillain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-200d-2642-fe0f.svg");
}

em[data-emoji=":man_supervillain_tone1:"]::before,
em[data-emoji="man_supervillain_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_supervillain_tone2:"]::before,
em[data-emoji="man_supervillain_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_supervillain_tone3:"]::before,
em[data-emoji="man_supervillain_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_supervillain_tone4:"]::before,
em[data-emoji="man_supervillain_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_supervillain_tone5:"]::before,
em[data-emoji="man_supervillain_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b9-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":ninja:"]::before,
em[data-emoji="ninja"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977.svg");
}

em[data-emoji=":ninja_tone1:"]::before,
em[data-emoji="ninja_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977-1f3fb.svg");
}

em[data-emoji=":ninja_tone2:"]::before,
em[data-emoji="ninja_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977-1f3fc.svg");
}

em[data-emoji=":ninja_tone3:"]::before,
em[data-emoji="ninja_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977-1f3fd.svg");
}

em[data-emoji=":ninja_tone4:"]::before,
em[data-emoji="ninja_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977-1f3fe.svg");
}

em[data-emoji=":ninja_tone5:"]::before,
em[data-emoji="ninja_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f977-1f3ff.svg");
}

em[data-emoji=":mx_claus:"]::before,
em[data-emoji="mx_claus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f384.svg");
}

em[data-emoji=":mx_claus_tone1:"]::before,
em[data-emoji="mx_claus_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f384.svg");
}

em[data-emoji=":mx_claus_tone2:"]::before,
em[data-emoji="mx_claus_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f384.svg");
}

em[data-emoji=":mx_claus_tone3:"]::before,
em[data-emoji="mx_claus_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f384.svg");
}

em[data-emoji=":mx_claus_tone4:"]::before,
em[data-emoji="mx_claus_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f384.svg");
}

em[data-emoji=":mx_claus_tone5:"]::before,
em[data-emoji="mx_claus_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f384.svg");
}

em[data-emoji=":mrs_claus:"]::before,
em[data-emoji="mrs_claus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936.svg");
}

em[data-emoji=":mrs_claus_tone1:"]::before,
em[data-emoji="mrs_claus_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936-1f3fb.svg");
}

em[data-emoji=":mrs_claus_tone2:"]::before,
em[data-emoji="mrs_claus_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936-1f3fc.svg");
}

em[data-emoji=":mrs_claus_tone3:"]::before,
em[data-emoji="mrs_claus_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936-1f3fd.svg");
}

em[data-emoji=":mrs_claus_tone4:"]::before,
em[data-emoji="mrs_claus_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936-1f3fe.svg");
}

em[data-emoji=":mrs_claus_tone5:"]::before,
em[data-emoji="mrs_claus_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f936-1f3ff.svg");
}

em[data-emoji=":santa:"]::before,
em[data-emoji="santa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385.svg");
}

em[data-emoji=":santa_tone1:"]::before,
em[data-emoji="santa_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385-1f3fb.svg");
}

em[data-emoji=":santa_tone2:"]::before,
em[data-emoji="santa_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385-1f3fc.svg");
}

em[data-emoji=":santa_tone3:"]::before,
em[data-emoji="santa_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385-1f3fd.svg");
}

em[data-emoji=":santa_tone4:"]::before,
em[data-emoji="santa_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385-1f3fe.svg");
}

em[data-emoji=":santa_tone5:"]::before,
em[data-emoji="santa_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f385-1f3ff.svg");
}

em[data-emoji=":mage:"]::before,
em[data-emoji="mage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9.svg");
}

em[data-emoji=":mage_tone1:"]::before,
em[data-emoji="mage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fb.svg");
}

em[data-emoji=":mage_tone2:"]::before,
em[data-emoji="mage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fc.svg");
}

em[data-emoji=":mage_tone3:"]::before,
em[data-emoji="mage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fd.svg");
}

em[data-emoji=":mage_tone4:"]::before,
em[data-emoji="mage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fe.svg");
}

em[data-emoji=":mage_tone5:"]::before,
em[data-emoji="mage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3ff.svg");
}

em[data-emoji=":woman_mage:"]::before,
em[data-emoji="woman_mage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mage_tone1:"]::before,
em[data-emoji="woman_mage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mage_tone2:"]::before,
em[data-emoji="woman_mage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mage_tone3:"]::before,
em[data-emoji="woman_mage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mage_tone4:"]::before,
em[data-emoji="woman_mage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mage_tone5:"]::before,
em[data-emoji="woman_mage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_mage:"]::before,
em[data-emoji="man_mage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mage_tone1:"]::before,
em[data-emoji="man_mage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mage_tone2:"]::before,
em[data-emoji="man_mage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mage_tone3:"]::before,
em[data-emoji="man_mage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mage_tone4:"]::before,
em[data-emoji="man_mage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mage_tone5:"]::before,
em[data-emoji="man_mage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d9-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":elf:"]::before,
em[data-emoji="elf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd.svg");
}

em[data-emoji=":elf_tone1:"]::before,
em[data-emoji="elf_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fb.svg");
}

em[data-emoji=":elf_tone2:"]::before,
em[data-emoji="elf_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fc.svg");
}

em[data-emoji=":elf_tone3:"]::before,
em[data-emoji="elf_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fd.svg");
}

em[data-emoji=":elf_tone4:"]::before,
em[data-emoji="elf_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fe.svg");
}

em[data-emoji=":elf_tone5:"]::before,
em[data-emoji="elf_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3ff.svg");
}

em[data-emoji=":woman_elf:"]::before,
em[data-emoji="woman_elf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_elf_tone1:"]::before,
em[data-emoji="woman_elf_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_elf_tone2:"]::before,
em[data-emoji="woman_elf_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_elf_tone3:"]::before,
em[data-emoji="woman_elf_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_elf_tone4:"]::before,
em[data-emoji="woman_elf_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_elf_tone5:"]::before,
em[data-emoji="woman_elf_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_elf:"]::before,
em[data-emoji="man_elf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_elf_tone1:"]::before,
em[data-emoji="man_elf_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_elf_tone2:"]::before,
em[data-emoji="man_elf_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_elf_tone3:"]::before,
em[data-emoji="man_elf_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_elf_tone4:"]::before,
em[data-emoji="man_elf_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_elf_tone5:"]::before,
em[data-emoji="man_elf_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dd-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":troll:"]::before,
em[data-emoji="troll"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cc.svg");
}

em[data-emoji=":vampire:"]::before,
em[data-emoji="vampire"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db.svg");
}

em[data-emoji=":vampire_tone1:"]::before,
em[data-emoji="vampire_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fb.svg");
}

em[data-emoji=":vampire_tone2:"]::before,
em[data-emoji="vampire_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fc.svg");
}

em[data-emoji=":vampire_tone3:"]::before,
em[data-emoji="vampire_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fd.svg");
}

em[data-emoji=":vampire_tone4:"]::before,
em[data-emoji="vampire_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fe.svg");
}

em[data-emoji=":vampire_tone5:"]::before,
em[data-emoji="vampire_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3ff.svg");
}

em[data-emoji=":woman_vampire:"]::before,
em[data-emoji="woman_vampire"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_vampire_tone1:"]::before,
em[data-emoji="woman_vampire_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_vampire_tone2:"]::before,
em[data-emoji="woman_vampire_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_vampire_tone3:"]::before,
em[data-emoji="woman_vampire_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_vampire_tone4:"]::before,
em[data-emoji="woman_vampire_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_vampire_tone5:"]::before,
em[data-emoji="woman_vampire_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_vampire:"]::before,
em[data-emoji="man_vampire"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-200d-2642-fe0f.svg");
}

em[data-emoji=":man_vampire_tone1:"]::before,
em[data-emoji="man_vampire_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_vampire_tone2:"]::before,
em[data-emoji="man_vampire_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_vampire_tone3:"]::before,
em[data-emoji="man_vampire_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_vampire_tone4:"]::before,
em[data-emoji="man_vampire_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_vampire_tone5:"]::before,
em[data-emoji="man_vampire_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9db-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":zombie:"]::before,
em[data-emoji="zombie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9df.svg");
}

em[data-emoji=":woman_zombie:"]::before,
em[data-emoji="woman_zombie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9df-200d-2640-fe0f.svg");
}

em[data-emoji=":man_zombie:"]::before,
em[data-emoji="man_zombie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9df-200d-2642-fe0f.svg");
}

em[data-emoji=":genie:"]::before,
em[data-emoji="genie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9de.svg");
}

em[data-emoji=":woman_genie:"]::before,
em[data-emoji="woman_genie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9de-200d-2640-fe0f.svg");
}

em[data-emoji=":man_genie:"]::before,
em[data-emoji="man_genie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9de-200d-2642-fe0f.svg");
}

em[data-emoji=":merperson:"]::before,
em[data-emoji="merperson"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc.svg");
}

em[data-emoji=":merperson_tone1:"]::before,
em[data-emoji="merperson_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fb.svg");
}

em[data-emoji=":merperson_tone2:"]::before,
em[data-emoji="merperson_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fc.svg");
}

em[data-emoji=":merperson_tone3:"]::before,
em[data-emoji="merperson_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fd.svg");
}

em[data-emoji=":merperson_tone4:"]::before,
em[data-emoji="merperson_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fe.svg");
}

em[data-emoji=":merperson_tone5:"]::before,
em[data-emoji="merperson_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3ff.svg");
}

em[data-emoji=":mermaid:"]::before,
em[data-emoji="mermaid"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-200d-2640-fe0f.svg");
}

em[data-emoji=":mermaid_tone1:"]::before,
em[data-emoji="mermaid_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":mermaid_tone2:"]::before,
em[data-emoji="mermaid_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":mermaid_tone3:"]::before,
em[data-emoji="mermaid_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":mermaid_tone4:"]::before,
em[data-emoji="mermaid_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":mermaid_tone5:"]::before,
em[data-emoji="mermaid_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":merman:"]::before,
em[data-emoji="merman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-200d-2642-fe0f.svg");
}

em[data-emoji=":merman_tone1:"]::before,
em[data-emoji="merman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":merman_tone2:"]::before,
em[data-emoji="merman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":merman_tone3:"]::before,
em[data-emoji="merman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":merman_tone4:"]::before,
em[data-emoji="merman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":merman_tone5:"]::before,
em[data-emoji="merman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9dc-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":fairy:"]::before,
em[data-emoji="fairy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da.svg");
}

em[data-emoji=":fairy_tone1:"]::before,
em[data-emoji="fairy_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fb.svg");
}

em[data-emoji=":fairy_tone2:"]::before,
em[data-emoji="fairy_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fc.svg");
}

em[data-emoji=":fairy_tone3:"]::before,
em[data-emoji="fairy_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fd.svg");
}

em[data-emoji=":fairy_tone4:"]::before,
em[data-emoji="fairy_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fe.svg");
}

em[data-emoji=":fairy_tone5:"]::before,
em[data-emoji="fairy_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3ff.svg");
}

em[data-emoji=":woman_fairy:"]::before,
em[data-emoji="woman_fairy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_fairy_tone1:"]::before,
em[data-emoji="woman_fairy_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_fairy_tone2:"]::before,
em[data-emoji="woman_fairy_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_fairy_tone3:"]::before,
em[data-emoji="woman_fairy_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_fairy_tone4:"]::before,
em[data-emoji="woman_fairy_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_fairy_tone5:"]::before,
em[data-emoji="woman_fairy_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_fairy:"]::before,
em[data-emoji="man_fairy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-200d-2642-fe0f.svg");
}

em[data-emoji=":man_fairy_tone1:"]::before,
em[data-emoji="man_fairy_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_fairy_tone2:"]::before,
em[data-emoji="man_fairy_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_fairy_tone3:"]::before,
em[data-emoji="man_fairy_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_fairy_tone4:"]::before,
em[data-emoji="man_fairy_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_fairy_tone5:"]::before,
em[data-emoji="man_fairy_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9da-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":angel:"]::before,
em[data-emoji="angel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c.svg");
}

em[data-emoji=":angel_tone1:"]::before,
em[data-emoji="angel_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c-1f3fb.svg");
}

em[data-emoji=":angel_tone2:"]::before,
em[data-emoji="angel_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c-1f3fc.svg");
}

em[data-emoji=":angel_tone3:"]::before,
em[data-emoji="angel_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c-1f3fd.svg");
}

em[data-emoji=":angel_tone4:"]::before,
em[data-emoji="angel_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c-1f3fe.svg");
}

em[data-emoji=":angel_tone5:"]::before,
em[data-emoji="angel_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f47c-1f3ff.svg");
}

em[data-emoji=":pregnant_person:"]::before,
em[data-emoji="pregnant_person"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4.svg");
}

em[data-emoji=":pregnant_person_tone1:"]::before,
em[data-emoji="pregnant_person_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4-1f3fb.svg");
}

em[data-emoji=":pregnant_person_tone2:"]::before,
em[data-emoji="pregnant_person_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4-1f3fc.svg");
}

em[data-emoji=":pregnant_person_tone3:"]::before,
em[data-emoji="pregnant_person_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4-1f3fd.svg");
}

em[data-emoji=":pregnant_person_tone4:"]::before,
em[data-emoji="pregnant_person_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4-1f3fe.svg");
}

em[data-emoji=":pregnant_person_tone5:"]::before,
em[data-emoji="pregnant_person_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac4-1f3ff.svg");
}

em[data-emoji=":pregnant_woman:"]::before,
em[data-emoji="pregnant_woman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930.svg");
}

em[data-emoji=":pregnant_woman_tone1:"]::before,
em[data-emoji="pregnant_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930-1f3fb.svg");
}

em[data-emoji=":pregnant_woman_tone2:"]::before,
em[data-emoji="pregnant_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930-1f3fc.svg");
}

em[data-emoji=":pregnant_woman_tone3:"]::before,
em[data-emoji="pregnant_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930-1f3fd.svg");
}

em[data-emoji=":pregnant_woman_tone4:"]::before,
em[data-emoji="pregnant_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930-1f3fe.svg");
}

em[data-emoji=":pregnant_woman_tone5:"]::before,
em[data-emoji="pregnant_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f930-1f3ff.svg");
}

em[data-emoji=":pregnant_man:"]::before,
em[data-emoji="pregnant_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3.svg");
}

em[data-emoji=":pregnant_man_tone1:"]::before,
em[data-emoji="pregnant_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3-1f3fb.svg");
}

em[data-emoji=":pregnant_man_tone2:"]::before,
em[data-emoji="pregnant_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3-1f3fc.svg");
}

em[data-emoji=":pregnant_man_tone3:"]::before,
em[data-emoji="pregnant_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3-1f3fd.svg");
}

em[data-emoji=":pregnant_man_tone4:"]::before,
em[data-emoji="pregnant_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3-1f3fe.svg");
}

em[data-emoji=":pregnant_man_tone5:"]::before,
em[data-emoji="pregnant_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fac3-1f3ff.svg");
}

em[data-emoji=":breast_feeding:"]::before,
em[data-emoji="breast_feeding"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931.svg");
}

em[data-emoji=":breast_feeding_tone1:"]::before,
em[data-emoji="breast_feeding_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931-1f3fb.svg");
}

em[data-emoji=":breast_feeding_tone2:"]::before,
em[data-emoji="breast_feeding_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931-1f3fc.svg");
}

em[data-emoji=":breast_feeding_tone3:"]::before,
em[data-emoji="breast_feeding_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931-1f3fd.svg");
}

em[data-emoji=":breast_feeding_tone4:"]::before,
em[data-emoji="breast_feeding_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931-1f3fe.svg");
}

em[data-emoji=":breast_feeding_tone5:"]::before,
em[data-emoji="breast_feeding_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f931-1f3ff.svg");
}

em[data-emoji=":person_feeding_baby:"]::before,
em[data-emoji="person_feeding_baby"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f37c.svg");
}

em[data-emoji=":person_feeding_baby_tone1:"]::before,
em[data-emoji="person_feeding_baby_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f37c.svg");
}

em[data-emoji=":person_feeding_baby_tone2:"]::before,
em[data-emoji="person_feeding_baby_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f37c.svg");
}

em[data-emoji=":person_feeding_baby_tone3:"]::before,
em[data-emoji="person_feeding_baby_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f37c.svg");
}

em[data-emoji=":person_feeding_baby_tone4:"]::before,
em[data-emoji="person_feeding_baby_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f37c.svg");
}

em[data-emoji=":person_feeding_baby_tone5:"]::before,
em[data-emoji="person_feeding_baby_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby:"]::before,
em[data-emoji="woman_feeding_baby"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby_tone1:"]::before,
em[data-emoji="woman_feeding_baby_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby_tone2:"]::before,
em[data-emoji="woman_feeding_baby_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby_tone3:"]::before,
em[data-emoji="woman_feeding_baby_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby_tone4:"]::before,
em[data-emoji="woman_feeding_baby_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f37c.svg");
}

em[data-emoji=":woman_feeding_baby_tone5:"]::before,
em[data-emoji="woman_feeding_baby_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby:"]::before,
em[data-emoji="man_feeding_baby"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby_tone1:"]::before,
em[data-emoji="man_feeding_baby_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby_tone2:"]::before,
em[data-emoji="man_feeding_baby_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby_tone3:"]::before,
em[data-emoji="man_feeding_baby_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby_tone4:"]::before,
em[data-emoji="man_feeding_baby_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f37c.svg");
}

em[data-emoji=":man_feeding_baby_tone5:"]::before,
em[data-emoji="man_feeding_baby_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f37c.svg");
}

em[data-emoji=":person_bowing:"]::before,
em[data-emoji="person_bowing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647.svg");
}

em[data-emoji=":person_bowing_tone1:"]::before,
em[data-emoji="person_bowing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fb.svg");
}

em[data-emoji=":person_bowing_tone2:"]::before,
em[data-emoji="person_bowing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fc.svg");
}

em[data-emoji=":person_bowing_tone3:"]::before,
em[data-emoji="person_bowing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fd.svg");
}

em[data-emoji=":person_bowing_tone4:"]::before,
em[data-emoji="person_bowing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fe.svg");
}

em[data-emoji=":person_bowing_tone5:"]::before,
em[data-emoji="person_bowing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3ff.svg");
}

em[data-emoji=":woman_bowing:"]::before,
em[data-emoji="woman_bowing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bowing_tone1:"]::before,
em[data-emoji="woman_bowing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bowing_tone2:"]::before,
em[data-emoji="woman_bowing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bowing_tone3:"]::before,
em[data-emoji="woman_bowing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bowing_tone4:"]::before,
em[data-emoji="woman_bowing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bowing_tone5:"]::before,
em[data-emoji="woman_bowing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_bowing:"]::before,
em[data-emoji="man_bowing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bowing_tone1:"]::before,
em[data-emoji="man_bowing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bowing_tone2:"]::before,
em[data-emoji="man_bowing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bowing_tone3:"]::before,
em[data-emoji="man_bowing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bowing_tone4:"]::before,
em[data-emoji="man_bowing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bowing_tone5:"]::before,
em[data-emoji="man_bowing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f647-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_tipping_hand:"]::before,
em[data-emoji="person_tipping_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481.svg");
}

em[data-emoji=":person_tipping_hand_tone1:"]::before,
em[data-emoji="person_tipping_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fb.svg");
}

em[data-emoji=":person_tipping_hand_tone2:"]::before,
em[data-emoji="person_tipping_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fc.svg");
}

em[data-emoji=":person_tipping_hand_tone3:"]::before,
em[data-emoji="person_tipping_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fd.svg");
}

em[data-emoji=":person_tipping_hand_tone4:"]::before,
em[data-emoji="person_tipping_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fe.svg");
}

em[data-emoji=":person_tipping_hand_tone5:"]::before,
em[data-emoji="person_tipping_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3ff.svg");
}

em[data-emoji=":woman_tipping_hand:"]::before,
em[data-emoji="woman_tipping_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tipping_hand_tone1:"]::before,
em[data-emoji="woman_tipping_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tipping_hand_tone2:"]::before,
em[data-emoji="woman_tipping_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tipping_hand_tone3:"]::before,
em[data-emoji="woman_tipping_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tipping_hand_tone4:"]::before,
em[data-emoji="woman_tipping_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_tipping_hand_tone5:"]::before,
em[data-emoji="woman_tipping_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_tipping_hand:"]::before,
em[data-emoji="man_tipping_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tipping_hand_tone1:"]::before,
em[data-emoji="man_tipping_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tipping_hand_tone2:"]::before,
em[data-emoji="man_tipping_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tipping_hand_tone3:"]::before,
em[data-emoji="man_tipping_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tipping_hand_tone4:"]::before,
em[data-emoji="man_tipping_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_tipping_hand_tone5:"]::before,
em[data-emoji="man_tipping_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f481-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_gesturing_no:"]::before,
em[data-emoji="person_gesturing_no"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645.svg");
}

em[data-emoji=":person_gesturing_no_tone1:"]::before,
em[data-emoji="person_gesturing_no_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fb.svg");
}

em[data-emoji=":person_gesturing_no_tone2:"]::before,
em[data-emoji="person_gesturing_no_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fc.svg");
}

em[data-emoji=":person_gesturing_no_tone3:"]::before,
em[data-emoji="person_gesturing_no_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fd.svg");
}

em[data-emoji=":person_gesturing_no_tone4:"]::before,
em[data-emoji="person_gesturing_no_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fe.svg");
}

em[data-emoji=":person_gesturing_no_tone5:"]::before,
em[data-emoji="person_gesturing_no_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3ff.svg");
}

em[data-emoji=":woman_gesturing_no:"]::before,
em[data-emoji="woman_gesturing_no"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_no_tone1:"]::before,
em[data-emoji="woman_gesturing_no_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_no_tone2:"]::before,
em[data-emoji="woman_gesturing_no_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_no_tone3:"]::before,
em[data-emoji="woman_gesturing_no_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_no_tone4:"]::before,
em[data-emoji="woman_gesturing_no_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_no_tone5:"]::before,
em[data-emoji="woman_gesturing_no_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_gesturing_no:"]::before,
em[data-emoji="man_gesturing_no"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_no_tone1:"]::before,
em[data-emoji="man_gesturing_no_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_no_tone2:"]::before,
em[data-emoji="man_gesturing_no_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_no_tone3:"]::before,
em[data-emoji="man_gesturing_no_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_no_tone4:"]::before,
em[data-emoji="man_gesturing_no_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_no_tone5:"]::before,
em[data-emoji="man_gesturing_no_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f645-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_gesturing_ok:"]::before,
em[data-emoji="person_gesturing_ok"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646.svg");
}

em[data-emoji=":person_gesturing_ok_tone1:"]::before,
em[data-emoji="person_gesturing_ok_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fb.svg");
}

em[data-emoji=":person_gesturing_ok_tone2:"]::before,
em[data-emoji="person_gesturing_ok_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fc.svg");
}

em[data-emoji=":person_gesturing_ok_tone3:"]::before,
em[data-emoji="person_gesturing_ok_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fd.svg");
}

em[data-emoji=":person_gesturing_ok_tone4:"]::before,
em[data-emoji="person_gesturing_ok_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fe.svg");
}

em[data-emoji=":person_gesturing_ok_tone5:"]::before,
em[data-emoji="person_gesturing_ok_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3ff.svg");
}

em[data-emoji=":woman_gesturing_ok:"]::before,
em[data-emoji="woman_gesturing_ok"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_ok_tone1:"]::before,
em[data-emoji="woman_gesturing_ok_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_ok_tone2:"]::before,
em[data-emoji="woman_gesturing_ok_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_ok_tone3:"]::before,
em[data-emoji="woman_gesturing_ok_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_ok_tone4:"]::before,
em[data-emoji="woman_gesturing_ok_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_gesturing_ok_tone5:"]::before,
em[data-emoji="woman_gesturing_ok_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok:"]::before,
em[data-emoji="man_gesturing_ok"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok_tone1:"]::before,
em[data-emoji="man_gesturing_ok_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok_tone2:"]::before,
em[data-emoji="man_gesturing_ok_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok_tone3:"]::before,
em[data-emoji="man_gesturing_ok_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok_tone4:"]::before,
em[data-emoji="man_gesturing_ok_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_gesturing_ok_tone5:"]::before,
em[data-emoji="man_gesturing_ok_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f646-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_raising_hand:"]::before,
em[data-emoji="person_raising_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b.svg");
}

em[data-emoji=":person_raising_hand_tone1:"]::before,
em[data-emoji="person_raising_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fb.svg");
}

em[data-emoji=":person_raising_hand_tone2:"]::before,
em[data-emoji="person_raising_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fc.svg");
}

em[data-emoji=":person_raising_hand_tone3:"]::before,
em[data-emoji="person_raising_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fd.svg");
}

em[data-emoji=":person_raising_hand_tone4:"]::before,
em[data-emoji="person_raising_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fe.svg");
}

em[data-emoji=":person_raising_hand_tone5:"]::before,
em[data-emoji="person_raising_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3ff.svg");
}

em[data-emoji=":woman_raising_hand:"]::before,
em[data-emoji="woman_raising_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_raising_hand_tone1:"]::before,
em[data-emoji="woman_raising_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_raising_hand_tone2:"]::before,
em[data-emoji="woman_raising_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_raising_hand_tone3:"]::before,
em[data-emoji="woman_raising_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_raising_hand_tone4:"]::before,
em[data-emoji="woman_raising_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_raising_hand_tone5:"]::before,
em[data-emoji="woman_raising_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_raising_hand:"]::before,
em[data-emoji="man_raising_hand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-200d-2642-fe0f.svg");
}

em[data-emoji=":man_raising_hand_tone1:"]::before,
em[data-emoji="man_raising_hand_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_raising_hand_tone2:"]::before,
em[data-emoji="man_raising_hand_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_raising_hand_tone3:"]::before,
em[data-emoji="man_raising_hand_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_raising_hand_tone4:"]::before,
em[data-emoji="man_raising_hand_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_raising_hand_tone5:"]::before,
em[data-emoji="man_raising_hand_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64b-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_person:"]::before,
em[data-emoji="deaf_person"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf.svg");
}

em[data-emoji=":deaf_person_tone1:"]::before,
em[data-emoji="deaf_person_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fb.svg");
}

em[data-emoji=":deaf_person_tone2:"]::before,
em[data-emoji="deaf_person_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fc.svg");
}

em[data-emoji=":deaf_person_tone3:"]::before,
em[data-emoji="deaf_person_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fd.svg");
}

em[data-emoji=":deaf_person_tone4:"]::before,
em[data-emoji="deaf_person_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fe.svg");
}

em[data-emoji=":deaf_person_tone5:"]::before,
em[data-emoji="deaf_person_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3ff.svg");
}

em[data-emoji=":deaf_woman:"]::before,
em[data-emoji="deaf_woman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_woman_tone1:"]::before,
em[data-emoji="deaf_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_woman_tone2:"]::before,
em[data-emoji="deaf_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_woman_tone3:"]::before,
em[data-emoji="deaf_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_woman_tone4:"]::before,
em[data-emoji="deaf_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_woman_tone5:"]::before,
em[data-emoji="deaf_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":deaf_man:"]::before,
em[data-emoji="deaf_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_man_tone1:"]::before,
em[data-emoji="deaf_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_man_tone2:"]::before,
em[data-emoji="deaf_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_man_tone3:"]::before,
em[data-emoji="deaf_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_man_tone4:"]::before,
em[data-emoji="deaf_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":deaf_man_tone5:"]::before,
em[data-emoji="deaf_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cf-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_facepalming:"]::before,
em[data-emoji="person_facepalming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926.svg");
}

em[data-emoji=":person_facepalming_tone1:"]::before,
em[data-emoji="person_facepalming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fb.svg");
}

em[data-emoji=":person_facepalming_tone2:"]::before,
em[data-emoji="person_facepalming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fc.svg");
}

em[data-emoji=":person_facepalming_tone3:"]::before,
em[data-emoji="person_facepalming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fd.svg");
}

em[data-emoji=":person_facepalming_tone4:"]::before,
em[data-emoji="person_facepalming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fe.svg");
}

em[data-emoji=":person_facepalming_tone5:"]::before,
em[data-emoji="person_facepalming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3ff.svg");
}

em[data-emoji=":woman_facepalming:"]::before,
em[data-emoji="woman_facepalming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_facepalming_tone1:"]::before,
em[data-emoji="woman_facepalming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_facepalming_tone2:"]::before,
em[data-emoji="woman_facepalming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_facepalming_tone3:"]::before,
em[data-emoji="woman_facepalming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_facepalming_tone4:"]::before,
em[data-emoji="woman_facepalming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_facepalming_tone5:"]::before,
em[data-emoji="woman_facepalming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_facepalming:"]::before,
em[data-emoji="man_facepalming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-200d-2642-fe0f.svg");
}

em[data-emoji=":man_facepalming_tone1:"]::before,
em[data-emoji="man_facepalming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_facepalming_tone2:"]::before,
em[data-emoji="man_facepalming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_facepalming_tone3:"]::before,
em[data-emoji="man_facepalming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_facepalming_tone4:"]::before,
em[data-emoji="man_facepalming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_facepalming_tone5:"]::before,
em[data-emoji="man_facepalming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f926-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_shrugging:"]::before,
em[data-emoji="person_shrugging"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937.svg");
}

em[data-emoji=":person_shrugging_tone1:"]::before,
em[data-emoji="person_shrugging_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fb.svg");
}

em[data-emoji=":person_shrugging_tone2:"]::before,
em[data-emoji="person_shrugging_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fc.svg");
}

em[data-emoji=":person_shrugging_tone3:"]::before,
em[data-emoji="person_shrugging_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fd.svg");
}

em[data-emoji=":person_shrugging_tone4:"]::before,
em[data-emoji="person_shrugging_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fe.svg");
}

em[data-emoji=":person_shrugging_tone5:"]::before,
em[data-emoji="person_shrugging_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3ff.svg");
}

em[data-emoji=":woman_shrugging:"]::before,
em[data-emoji="woman_shrugging"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_shrugging_tone1:"]::before,
em[data-emoji="woman_shrugging_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_shrugging_tone2:"]::before,
em[data-emoji="woman_shrugging_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_shrugging_tone3:"]::before,
em[data-emoji="woman_shrugging_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_shrugging_tone4:"]::before,
em[data-emoji="woman_shrugging_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_shrugging_tone5:"]::before,
em[data-emoji="woman_shrugging_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_shrugging:"]::before,
em[data-emoji="man_shrugging"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-200d-2642-fe0f.svg");
}

em[data-emoji=":man_shrugging_tone1:"]::before,
em[data-emoji="man_shrugging_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_shrugging_tone2:"]::before,
em[data-emoji="man_shrugging_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_shrugging_tone3:"]::before,
em[data-emoji="man_shrugging_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_shrugging_tone4:"]::before,
em[data-emoji="man_shrugging_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_shrugging_tone5:"]::before,
em[data-emoji="man_shrugging_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f937-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_pouting:"]::before,
em[data-emoji="person_pouting"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e.svg");
}

em[data-emoji=":person_pouting_tone1:"]::before,
em[data-emoji="person_pouting_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fb.svg");
}

em[data-emoji=":person_pouting_tone2:"]::before,
em[data-emoji="person_pouting_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fc.svg");
}

em[data-emoji=":person_pouting_tone3:"]::before,
em[data-emoji="person_pouting_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fd.svg");
}

em[data-emoji=":person_pouting_tone4:"]::before,
em[data-emoji="person_pouting_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fe.svg");
}

em[data-emoji=":person_pouting_tone5:"]::before,
em[data-emoji="person_pouting_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3ff.svg");
}

em[data-emoji=":woman_pouting:"]::before,
em[data-emoji="woman_pouting"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_pouting_tone1:"]::before,
em[data-emoji="woman_pouting_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_pouting_tone2:"]::before,
em[data-emoji="woman_pouting_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_pouting_tone3:"]::before,
em[data-emoji="woman_pouting_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_pouting_tone4:"]::before,
em[data-emoji="woman_pouting_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_pouting_tone5:"]::before,
em[data-emoji="woman_pouting_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_pouting:"]::before,
em[data-emoji="man_pouting"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-200d-2642-fe0f.svg");
}

em[data-emoji=":man_pouting_tone1:"]::before,
em[data-emoji="man_pouting_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_pouting_tone2:"]::before,
em[data-emoji="man_pouting_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_pouting_tone3:"]::before,
em[data-emoji="man_pouting_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_pouting_tone4:"]::before,
em[data-emoji="man_pouting_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_pouting_tone5:"]::before,
em[data-emoji="man_pouting_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64e-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_frowning:"]::before,
em[data-emoji="person_frowning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d.svg");
}

em[data-emoji=":person_frowning_tone1:"]::before,
em[data-emoji="person_frowning_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fb.svg");
}

em[data-emoji=":person_frowning_tone2:"]::before,
em[data-emoji="person_frowning_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fc.svg");
}

em[data-emoji=":person_frowning_tone3:"]::before,
em[data-emoji="person_frowning_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fd.svg");
}

em[data-emoji=":person_frowning_tone4:"]::before,
em[data-emoji="person_frowning_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fe.svg");
}

em[data-emoji=":person_frowning_tone5:"]::before,
em[data-emoji="person_frowning_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3ff.svg");
}

em[data-emoji=":woman_frowning:"]::before,
em[data-emoji="woman_frowning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_frowning_tone1:"]::before,
em[data-emoji="woman_frowning_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_frowning_tone2:"]::before,
em[data-emoji="woman_frowning_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_frowning_tone3:"]::before,
em[data-emoji="woman_frowning_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_frowning_tone4:"]::before,
em[data-emoji="woman_frowning_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_frowning_tone5:"]::before,
em[data-emoji="woman_frowning_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_frowning:"]::before,
em[data-emoji="man_frowning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-200d-2642-fe0f.svg");
}

em[data-emoji=":man_frowning_tone1:"]::before,
em[data-emoji="man_frowning_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_frowning_tone2:"]::before,
em[data-emoji="man_frowning_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_frowning_tone3:"]::before,
em[data-emoji="man_frowning_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_frowning_tone4:"]::before,
em[data-emoji="man_frowning_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_frowning_tone5:"]::before,
em[data-emoji="man_frowning_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64d-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_getting_haircut:"]::before,
em[data-emoji="person_getting_haircut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487.svg");
}

em[data-emoji=":person_getting_haircut_tone1:"]::before,
em[data-emoji="person_getting_haircut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fb.svg");
}

em[data-emoji=":person_getting_haircut_tone2:"]::before,
em[data-emoji="person_getting_haircut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fc.svg");
}

em[data-emoji=":person_getting_haircut_tone3:"]::before,
em[data-emoji="person_getting_haircut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fd.svg");
}

em[data-emoji=":person_getting_haircut_tone4:"]::before,
em[data-emoji="person_getting_haircut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fe.svg");
}

em[data-emoji=":person_getting_haircut_tone5:"]::before,
em[data-emoji="person_getting_haircut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3ff.svg");
}

em[data-emoji=":woman_getting_haircut:"]::before,
em[data-emoji="woman_getting_haircut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_haircut_tone1:"]::before,
em[data-emoji="woman_getting_haircut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_haircut_tone2:"]::before,
em[data-emoji="woman_getting_haircut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_haircut_tone3:"]::before,
em[data-emoji="woman_getting_haircut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_haircut_tone4:"]::before,
em[data-emoji="woman_getting_haircut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_haircut_tone5:"]::before,
em[data-emoji="woman_getting_haircut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_getting_haircut:"]::before,
em[data-emoji="man_getting_haircut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_haircut_tone1:"]::before,
em[data-emoji="man_getting_haircut_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_haircut_tone2:"]::before,
em[data-emoji="man_getting_haircut_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_haircut_tone3:"]::before,
em[data-emoji="man_getting_haircut_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_haircut_tone4:"]::before,
em[data-emoji="man_getting_haircut_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_haircut_tone5:"]::before,
em[data-emoji="man_getting_haircut_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f487-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_getting_massage:"]::before,
em[data-emoji="person_getting_massage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486.svg");
}

em[data-emoji=":person_getting_massage_tone1:"]::before,
em[data-emoji="person_getting_massage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fb.svg");
}

em[data-emoji=":person_getting_massage_tone2:"]::before,
em[data-emoji="person_getting_massage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fc.svg");
}

em[data-emoji=":person_getting_massage_tone3:"]::before,
em[data-emoji="person_getting_massage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fd.svg");
}

em[data-emoji=":person_getting_massage_tone4:"]::before,
em[data-emoji="person_getting_massage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fe.svg");
}

em[data-emoji=":person_getting_massage_tone5:"]::before,
em[data-emoji="person_getting_massage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3ff.svg");
}

em[data-emoji=":woman_getting_face_massage:"]::before,
em[data-emoji="woman_getting_face_massage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_face_massage_tone1:"]::before,
em[data-emoji="woman_getting_face_massage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_face_massage_tone2:"]::before,
em[data-emoji="woman_getting_face_massage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_face_massage_tone3:"]::before,
em[data-emoji="woman_getting_face_massage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_face_massage_tone4:"]::before,
em[data-emoji="woman_getting_face_massage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_getting_face_massage_tone5:"]::before,
em[data-emoji="woman_getting_face_massage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage:"]::before,
em[data-emoji="man_getting_face_massage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage_tone1:"]::before,
em[data-emoji="man_getting_face_massage_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage_tone2:"]::before,
em[data-emoji="man_getting_face_massage_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage_tone3:"]::before,
em[data-emoji="man_getting_face_massage_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage_tone4:"]::before,
em[data-emoji="man_getting_face_massage_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_getting_face_massage_tone5:"]::before,
em[data-emoji="man_getting_face_massage_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f486-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_in_steamy_room:"]::before,
em[data-emoji="person_in_steamy_room"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6.svg");
}

em[data-emoji=":person_in_steamy_room_tone1:"]::before,
em[data-emoji="person_in_steamy_room_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fb.svg");
}

em[data-emoji=":person_in_steamy_room_tone2:"]::before,
em[data-emoji="person_in_steamy_room_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fc.svg");
}

em[data-emoji=":person_in_steamy_room_tone3:"]::before,
em[data-emoji="person_in_steamy_room_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fd.svg");
}

em[data-emoji=":person_in_steamy_room_tone4:"]::before,
em[data-emoji="person_in_steamy_room_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fe.svg");
}

em[data-emoji=":person_in_steamy_room_tone5:"]::before,
em[data-emoji="person_in_steamy_room_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3ff.svg");
}

em[data-emoji=":woman_in_steamy_room:"]::before,
em[data-emoji="woman_in_steamy_room"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_steamy_room_tone1:"]::before,
em[data-emoji="woman_in_steamy_room_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_steamy_room_tone2:"]::before,
em[data-emoji="woman_in_steamy_room_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_steamy_room_tone3:"]::before,
em[data-emoji="woman_in_steamy_room_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_steamy_room_tone4:"]::before,
em[data-emoji="woman_in_steamy_room_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_steamy_room_tone5:"]::before,
em[data-emoji="woman_in_steamy_room_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room:"]::before,
em[data-emoji="man_in_steamy_room"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room_tone1:"]::before,
em[data-emoji="man_in_steamy_room_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room_tone2:"]::before,
em[data-emoji="man_in_steamy_room_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room_tone3:"]::before,
em[data-emoji="man_in_steamy_room_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room_tone4:"]::before,
em[data-emoji="man_in_steamy_room_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_steamy_room_tone5:"]::before,
em[data-emoji="man_in_steamy_room_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d6-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":nail_care:"]::before,
em[data-emoji="nail_care"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485.svg");
}

em[data-emoji=":nail_care_tone1:"]::before,
em[data-emoji="nail_care_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485-1f3fb.svg");
}

em[data-emoji=":nail_care_tone2:"]::before,
em[data-emoji="nail_care_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485-1f3fc.svg");
}

em[data-emoji=":nail_care_tone3:"]::before,
em[data-emoji="nail_care_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485-1f3fd.svg");
}

em[data-emoji=":nail_care_tone4:"]::before,
em[data-emoji="nail_care_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485-1f3fe.svg");
}

em[data-emoji=":nail_care_tone5:"]::before,
em[data-emoji="nail_care_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f485-1f3ff.svg");
}

em[data-emoji=":selfie:"]::before,
em[data-emoji="selfie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933.svg");
}

em[data-emoji=":selfie_tone1:"]::before,
em[data-emoji="selfie_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933-1f3fb.svg");
}

em[data-emoji=":selfie_tone2:"]::before,
em[data-emoji="selfie_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933-1f3fc.svg");
}

em[data-emoji=":selfie_tone3:"]::before,
em[data-emoji="selfie_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933-1f3fd.svg");
}

em[data-emoji=":selfie_tone4:"]::before,
em[data-emoji="selfie_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933-1f3fe.svg");
}

em[data-emoji=":selfie_tone5:"]::before,
em[data-emoji="selfie_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f933-1f3ff.svg");
}

em[data-emoji=":dancer:"]::before,
em[data-emoji="dancer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483.svg");
}

em[data-emoji=":dancer_tone1:"]::before,
em[data-emoji="dancer_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483-1f3fb.svg");
}

em[data-emoji=":dancer_tone2:"]::before,
em[data-emoji="dancer_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483-1f3fc.svg");
}

em[data-emoji=":dancer_tone3:"]::before,
em[data-emoji="dancer_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483-1f3fd.svg");
}

em[data-emoji=":dancer_tone4:"]::before,
em[data-emoji="dancer_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483-1f3fe.svg");
}

em[data-emoji=":dancer_tone5:"]::before,
em[data-emoji="dancer_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f483-1f3ff.svg");
}

em[data-emoji=":man_dancing:"]::before,
em[data-emoji="man_dancing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a.svg");
}

em[data-emoji=":man_dancing_tone1:"]::before,
em[data-emoji="man_dancing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a-1f3fb.svg");
}

em[data-emoji=":man_dancing_tone2:"]::before,
em[data-emoji="man_dancing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a-1f3fc.svg");
}

em[data-emoji=":man_dancing_tone3:"]::before,
em[data-emoji="man_dancing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a-1f3fd.svg");
}

em[data-emoji=":man_dancing_tone5:"]::before,
em[data-emoji="man_dancing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a-1f3ff.svg");
}

em[data-emoji=":man_dancing_tone4:"]::before,
em[data-emoji="man_dancing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f57a-1f3fe.svg");
}

em[data-emoji=":people_with_bunny_ears_partying:"]::before,
em[data-emoji="people_with_bunny_ears_partying"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46f.svg");
}

em[data-emoji=":women_with_bunny_ears_partying:"]::before,
em[data-emoji="women_with_bunny_ears_partying"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46f-200d-2640-fe0f.svg");
}

em[data-emoji=":men_with_bunny_ears_partying:"]::before,
em[data-emoji="men_with_bunny_ears_partying"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46f-200d-2642-fe0f.svg");
}

em[data-emoji=":levitate:"]::before,
em[data-emoji="levitate"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574.svg");
}

em[data-emoji=":levitate_tone1:"]::before,
em[data-emoji="levitate_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574-1f3fb.svg");
}

em[data-emoji=":levitate_tone2:"]::before,
em[data-emoji="levitate_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574-1f3fc.svg");
}

em[data-emoji=":levitate_tone3:"]::before,
em[data-emoji="levitate_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574-1f3fd.svg");
}

em[data-emoji=":levitate_tone4:"]::before,
em[data-emoji="levitate_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574-1f3fe.svg");
}

em[data-emoji=":levitate_tone5:"]::before,
em[data-emoji="levitate_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f574-1f3ff.svg");
}

em[data-emoji=":person_in_manual_wheelchair:"]::before,
em[data-emoji="person_in_manual_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9bd.svg");
}

em[data-emoji=":person_in_manual_wheelchair_tone1:"]::before,
em[data-emoji="person_in_manual_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9bd.svg");
}

em[data-emoji=":person_in_manual_wheelchair_tone2:"]::before,
em[data-emoji="person_in_manual_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9bd.svg");
}

em[data-emoji=":person_in_manual_wheelchair_tone3:"]::before,
em[data-emoji="person_in_manual_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9bd.svg");
}

em[data-emoji=":person_in_manual_wheelchair_tone4:"]::before,
em[data-emoji="person_in_manual_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9bd.svg");
}

em[data-emoji=":person_in_manual_wheelchair_tone5:"]::before,
em[data-emoji="person_in_manual_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair:"]::before,
em[data-emoji="woman_in_manual_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair_tone1:"]::before,
em[data-emoji="woman_in_manual_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair_tone2:"]::before,
em[data-emoji="woman_in_manual_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair_tone3:"]::before,
em[data-emoji="woman_in_manual_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair_tone4:"]::before,
em[data-emoji="woman_in_manual_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9bd.svg");
}

em[data-emoji=":woman_in_manual_wheelchair_tone5:"]::before,
em[data-emoji="woman_in_manual_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair:"]::before,
em[data-emoji="man_in_manual_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair_tone1:"]::before,
em[data-emoji="man_in_manual_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair_tone2:"]::before,
em[data-emoji="man_in_manual_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair_tone3:"]::before,
em[data-emoji="man_in_manual_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair_tone4:"]::before,
em[data-emoji="man_in_manual_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9bd.svg");
}

em[data-emoji=":man_in_manual_wheelchair_tone5:"]::before,
em[data-emoji="man_in_manual_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9bd.svg");
}

em[data-emoji=":person_in_motorized_wheelchair:"]::before,
em[data-emoji="person_in_motorized_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9bc.svg");
}

em[data-emoji=":person_in_motorized_wheelchair_tone1:"]::before,
em[data-emoji="person_in_motorized_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9bc.svg");
}

em[data-emoji=":person_in_motorized_wheelchair_tone2:"]::before,
em[data-emoji="person_in_motorized_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9bc.svg");
}

em[data-emoji=":person_in_motorized_wheelchair_tone3:"]::before,
em[data-emoji="person_in_motorized_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9bc.svg");
}

em[data-emoji=":person_in_motorized_wheelchair_tone4:"]::before,
em[data-emoji="person_in_motorized_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9bc.svg");
}

em[data-emoji=":person_in_motorized_wheelchair_tone5:"]::before,
em[data-emoji="person_in_motorized_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair:"]::before,
em[data-emoji="woman_in_motorized_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair_tone1:"]::before,
em[data-emoji="woman_in_motorized_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair_tone2:"]::before,
em[data-emoji="woman_in_motorized_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair_tone3:"]::before,
em[data-emoji="woman_in_motorized_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair_tone4:"]::before,
em[data-emoji="woman_in_motorized_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9bc.svg");
}

em[data-emoji=":woman_in_motorized_wheelchair_tone5:"]::before,
em[data-emoji="woman_in_motorized_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair:"]::before,
em[data-emoji="man_in_motorized_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair_tone1:"]::before,
em[data-emoji="man_in_motorized_wheelchair_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair_tone2:"]::before,
em[data-emoji="man_in_motorized_wheelchair_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair_tone3:"]::before,
em[data-emoji="man_in_motorized_wheelchair_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair_tone4:"]::before,
em[data-emoji="man_in_motorized_wheelchair_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9bc.svg");
}

em[data-emoji=":man_in_motorized_wheelchair_tone5:"]::before,
em[data-emoji="man_in_motorized_wheelchair_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9bc.svg");
}

em[data-emoji=":person_walking:"]::before,
em[data-emoji="person_walking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6.svg");
}

em[data-emoji=":person_walking_tone1:"]::before,
em[data-emoji="person_walking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fb.svg");
}

em[data-emoji=":person_walking_tone2:"]::before,
em[data-emoji="person_walking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fc.svg");
}

em[data-emoji=":person_walking_tone3:"]::before,
em[data-emoji="person_walking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fd.svg");
}

em[data-emoji=":person_walking_tone4:"]::before,
em[data-emoji="person_walking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fe.svg");
}

em[data-emoji=":person_walking_tone5:"]::before,
em[data-emoji="person_walking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3ff.svg");
}

em[data-emoji=":woman_walking:"]::before,
em[data-emoji="woman_walking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_walking_tone1:"]::before,
em[data-emoji="woman_walking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_walking_tone2:"]::before,
em[data-emoji="woman_walking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_walking_tone3:"]::before,
em[data-emoji="woman_walking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_walking_tone4:"]::before,
em[data-emoji="woman_walking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_walking_tone5:"]::before,
em[data-emoji="woman_walking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_walking:"]::before,
em[data-emoji="man_walking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-200d-2642-fe0f.svg");
}

em[data-emoji=":man_walking_tone1:"]::before,
em[data-emoji="man_walking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_walking_tone2:"]::before,
em[data-emoji="man_walking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_walking_tone3:"]::before,
em[data-emoji="man_walking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_walking_tone4:"]::before,
em[data-emoji="man_walking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_walking_tone5:"]::before,
em[data-emoji="man_walking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b6-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_with_probing_cane:"]::before,
em[data-emoji="person_with_probing_cane"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f9af.svg");
}

em[data-emoji=":person_with_probing_cane_tone1:"]::before,
em[data-emoji="person_with_probing_cane_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f9af.svg");
}

em[data-emoji=":person_with_probing_cane_tone2:"]::before,
em[data-emoji="person_with_probing_cane_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f9af.svg");
}

em[data-emoji=":person_with_probing_cane_tone3:"]::before,
em[data-emoji="person_with_probing_cane_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f9af.svg");
}

em[data-emoji=":person_with_probing_cane_tone4:"]::before,
em[data-emoji="person_with_probing_cane_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f9af.svg");
}

em[data-emoji=":person_with_probing_cane_tone5:"]::before,
em[data-emoji="person_with_probing_cane_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane:"]::before,
em[data-emoji="woman_with_probing_cane"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane_tone1:"]::before,
em[data-emoji="woman_with_probing_cane_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane_tone2:"]::before,
em[data-emoji="woman_with_probing_cane_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane_tone3:"]::before,
em[data-emoji="woman_with_probing_cane_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane_tone4:"]::before,
em[data-emoji="woman_with_probing_cane_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f9af.svg");
}

em[data-emoji=":woman_with_probing_cane_tone5:"]::before,
em[data-emoji="woman_with_probing_cane_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane:"]::before,
em[data-emoji="man_with_probing_cane"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane_tone1:"]::before,
em[data-emoji="man_with_probing_cane_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane_tone2:"]::before,
em[data-emoji="man_with_probing_cane_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane_tone3:"]::before,
em[data-emoji="man_with_probing_cane_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane_tone4:"]::before,
em[data-emoji="man_with_probing_cane_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f9af.svg");
}

em[data-emoji=":man_with_probing_cane_tone5:"]::before,
em[data-emoji="man_with_probing_cane_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f9af.svg");
}

em[data-emoji=":person_kneeling:"]::before,
em[data-emoji="person_kneeling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce.svg");
}

em[data-emoji=":person_kneeling_tone1:"]::before,
em[data-emoji="person_kneeling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fb.svg");
}

em[data-emoji=":person_kneeling_tone2:"]::before,
em[data-emoji="person_kneeling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fc.svg");
}

em[data-emoji=":person_kneeling_tone3:"]::before,
em[data-emoji="person_kneeling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fd.svg");
}

em[data-emoji=":person_kneeling_tone4:"]::before,
em[data-emoji="person_kneeling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fe.svg");
}

em[data-emoji=":person_kneeling_tone5:"]::before,
em[data-emoji="person_kneeling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3ff.svg");
}

em[data-emoji=":woman_kneeling:"]::before,
em[data-emoji="woman_kneeling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_kneeling_tone1:"]::before,
em[data-emoji="woman_kneeling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_kneeling_tone2:"]::before,
em[data-emoji="woman_kneeling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_kneeling_tone3:"]::before,
em[data-emoji="woman_kneeling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_kneeling_tone4:"]::before,
em[data-emoji="woman_kneeling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_kneeling_tone5:"]::before,
em[data-emoji="woman_kneeling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_kneeling:"]::before,
em[data-emoji="man_kneeling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-200d-2642-fe0f.svg");
}

em[data-emoji=":man_kneeling_tone1:"]::before,
em[data-emoji="man_kneeling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_kneeling_tone2:"]::before,
em[data-emoji="man_kneeling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_kneeling_tone3:"]::before,
em[data-emoji="man_kneeling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_kneeling_tone4:"]::before,
em[data-emoji="man_kneeling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_kneeling_tone5:"]::before,
em[data-emoji="man_kneeling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ce-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_running:"]::before,
em[data-emoji="person_running"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3.svg");
}

em[data-emoji=":person_running_tone1:"]::before,
em[data-emoji="person_running_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fb.svg");
}

em[data-emoji=":person_running_tone2:"]::before,
em[data-emoji="person_running_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fc.svg");
}

em[data-emoji=":person_running_tone3:"]::before,
em[data-emoji="person_running_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fd.svg");
}

em[data-emoji=":person_running_tone4:"]::before,
em[data-emoji="person_running_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fe.svg");
}

em[data-emoji=":person_running_tone5:"]::before,
em[data-emoji="person_running_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3ff.svg");
}

em[data-emoji=":woman_running:"]::before,
em[data-emoji="woman_running"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_running_tone1:"]::before,
em[data-emoji="woman_running_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_running_tone2:"]::before,
em[data-emoji="woman_running_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_running_tone3:"]::before,
em[data-emoji="woman_running_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_running_tone4:"]::before,
em[data-emoji="woman_running_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_running_tone5:"]::before,
em[data-emoji="woman_running_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_running:"]::before,
em[data-emoji="man_running"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-200d-2642-fe0f.svg");
}

em[data-emoji=":man_running_tone1:"]::before,
em[data-emoji="man_running_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_running_tone2:"]::before,
em[data-emoji="man_running_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_running_tone3:"]::before,
em[data-emoji="man_running_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_running_tone4:"]::before,
em[data-emoji="man_running_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_running_tone5:"]::before,
em[data-emoji="man_running_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c3-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_standing:"]::before,
em[data-emoji="person_standing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd.svg");
}

em[data-emoji=":person_standing_tone1:"]::before,
em[data-emoji="person_standing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fb.svg");
}

em[data-emoji=":person_standing_tone2:"]::before,
em[data-emoji="person_standing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fc.svg");
}

em[data-emoji=":person_standing_tone3:"]::before,
em[data-emoji="person_standing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fd.svg");
}

em[data-emoji=":person_standing_tone4:"]::before,
em[data-emoji="person_standing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fe.svg");
}

em[data-emoji=":person_standing_tone5:"]::before,
em[data-emoji="person_standing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3ff.svg");
}

em[data-emoji=":woman_standing:"]::before,
em[data-emoji="woman_standing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_standing_tone1:"]::before,
em[data-emoji="woman_standing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_standing_tone2:"]::before,
em[data-emoji="woman_standing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_standing_tone3:"]::before,
em[data-emoji="woman_standing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_standing_tone4:"]::before,
em[data-emoji="woman_standing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_standing_tone5:"]::before,
em[data-emoji="woman_standing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_standing:"]::before,
em[data-emoji="man_standing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_standing_tone1:"]::before,
em[data-emoji="man_standing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_standing_tone2:"]::before,
em[data-emoji="man_standing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_standing_tone3:"]::before,
em[data-emoji="man_standing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_standing_tone4:"]::before,
em[data-emoji="man_standing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_standing_tone5:"]::before,
em[data-emoji="man_standing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cd-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":people_holding_hands:"]::before,
em[data-emoji="people_holding_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-200d-1f91d-200d-1f9d1.svg");
}

em[data-emoji=":people_holding_hands_tone1:"]::before,
em[data-emoji="people_holding_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f91d-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":people_holding_hands_tone1_tone2:"]::before,
em[data-emoji="people_holding_hands_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f91d-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":people_holding_hands_tone1_tone3:"]::before,
em[data-emoji="people_holding_hands_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f91d-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":people_holding_hands_tone1_tone4:"]::before,
em[data-emoji="people_holding_hands_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f91d-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":people_holding_hands_tone1_tone5:"]::before,
em[data-emoji="people_holding_hands_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-1f91d-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":people_holding_hands_tone2_tone1:"]::before,
em[data-emoji="people_holding_hands_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f91d-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":people_holding_hands_tone2:"]::before,
em[data-emoji="people_holding_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f91d-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":people_holding_hands_tone2_tone3:"]::before,
em[data-emoji="people_holding_hands_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f91d-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":people_holding_hands_tone2_tone4:"]::before,
em[data-emoji="people_holding_hands_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f91d-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":people_holding_hands_tone2_tone5:"]::before,
em[data-emoji="people_holding_hands_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-1f91d-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":people_holding_hands_tone3_tone1:"]::before,
em[data-emoji="people_holding_hands_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f91d-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":people_holding_hands_tone3_tone2:"]::before,
em[data-emoji="people_holding_hands_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f91d-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":people_holding_hands_tone3:"]::before,
em[data-emoji="people_holding_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f91d-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":people_holding_hands_tone3_tone4:"]::before,
em[data-emoji="people_holding_hands_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f91d-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":people_holding_hands_tone3_tone5:"]::before,
em[data-emoji="people_holding_hands_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-1f91d-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":people_holding_hands_tone4_tone1:"]::before,
em[data-emoji="people_holding_hands_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f91d-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":people_holding_hands_tone4_tone2:"]::before,
em[data-emoji="people_holding_hands_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f91d-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":people_holding_hands_tone4_tone3:"]::before,
em[data-emoji="people_holding_hands_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f91d-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":people_holding_hands_tone4:"]::before,
em[data-emoji="people_holding_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f91d-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":people_holding_hands_tone4_tone5:"]::before,
em[data-emoji="people_holding_hands_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-1f91d-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":people_holding_hands_tone5_tone1:"]::before,
em[data-emoji="people_holding_hands_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f91d-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":people_holding_hands_tone5_tone2:"]::before,
em[data-emoji="people_holding_hands_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f91d-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":people_holding_hands_tone5_tone3:"]::before,
em[data-emoji="people_holding_hands_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f91d-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":people_holding_hands_tone5_tone4:"]::before,
em[data-emoji="people_holding_hands_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f91d-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":people_holding_hands_tone5:"]::before,
em[data-emoji="people_holding_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-1f91d-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":couple:"]::before,
em[data-emoji="couple"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone1:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b-1f3fb.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone1_tone2:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone1_tone3:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone1_tone4:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone1_tone5:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone2_tone1:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone2:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b-1f3fc.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone2_tone3:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone2_tone4:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone2_tone5:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone3_tone1:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone3_tone2:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone3:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b-1f3fd.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone3_tone4:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone3_tone5:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone4_tone1:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone4_tone2:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone4_tone3:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone4:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b-1f3fe.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone4_tone5:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone5_tone1:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone5_tone2:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone5_tone3:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone5_tone4:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":woman_and_man_holding_hands_tone5:"]::before,
em[data-emoji="woman_and_man_holding_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46b-1f3ff.svg");
}

em[data-emoji=":two_women_holding_hands:"]::before,
em[data-emoji="two_women_holding_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d.svg");
}

em[data-emoji=":women_holding_hands_tone1:"]::before,
em[data-emoji="women_holding_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d-1f3fb.svg");
}

em[data-emoji=":women_holding_hands_tone1_tone2:"]::before,
em[data-emoji="women_holding_hands_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f469-1f3fc.svg");
}

em[data-emoji=":women_holding_hands_tone1_tone3:"]::before,
em[data-emoji="women_holding_hands_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f469-1f3fd.svg");
}

em[data-emoji=":women_holding_hands_tone1_tone4:"]::before,
em[data-emoji="women_holding_hands_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f469-1f3fe.svg");
}

em[data-emoji=":women_holding_hands_tone1_tone5:"]::before,
em[data-emoji="women_holding_hands_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-1f91d-200d-1f469-1f3ff.svg");
}

em[data-emoji=":women_holding_hands_tone2_tone1:"]::before,
em[data-emoji="women_holding_hands_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f469-1f3fb.svg");
}

em[data-emoji=":women_holding_hands_tone2:"]::before,
em[data-emoji="women_holding_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d-1f3fc.svg");
}

em[data-emoji=":women_holding_hands_tone2_tone3:"]::before,
em[data-emoji="women_holding_hands_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f469-1f3fd.svg");
}

em[data-emoji=":women_holding_hands_tone2_tone4:"]::before,
em[data-emoji="women_holding_hands_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f469-1f3fe.svg");
}

em[data-emoji=":women_holding_hands_tone2_tone5:"]::before,
em[data-emoji="women_holding_hands_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-1f91d-200d-1f469-1f3ff.svg");
}

em[data-emoji=":women_holding_hands_tone3_tone1:"]::before,
em[data-emoji="women_holding_hands_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f469-1f3fb.svg");
}

em[data-emoji=":women_holding_hands_tone3_tone2:"]::before,
em[data-emoji="women_holding_hands_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f469-1f3fc.svg");
}

em[data-emoji=":women_holding_hands_tone3:"]::before,
em[data-emoji="women_holding_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d-1f3fd.svg");
}

em[data-emoji=":women_holding_hands_tone3_tone4:"]::before,
em[data-emoji="women_holding_hands_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f469-1f3fe.svg");
}

em[data-emoji=":women_holding_hands_tone3_tone5:"]::before,
em[data-emoji="women_holding_hands_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-1f91d-200d-1f469-1f3ff.svg");
}

em[data-emoji=":women_holding_hands_tone4_tone1:"]::before,
em[data-emoji="women_holding_hands_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f469-1f3fb.svg");
}

em[data-emoji=":women_holding_hands_tone4_tone2:"]::before,
em[data-emoji="women_holding_hands_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f469-1f3fc.svg");
}

em[data-emoji=":women_holding_hands_tone4_tone3:"]::before,
em[data-emoji="women_holding_hands_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f469-1f3fd.svg");
}

em[data-emoji=":women_holding_hands_tone4:"]::before,
em[data-emoji="women_holding_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d-1f3fe.svg");
}

em[data-emoji=":women_holding_hands_tone4_tone5:"]::before,
em[data-emoji="women_holding_hands_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-1f91d-200d-1f469-1f3ff.svg");
}

em[data-emoji=":women_holding_hands_tone5_tone1:"]::before,
em[data-emoji="women_holding_hands_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f469-1f3fb.svg");
}

em[data-emoji=":women_holding_hands_tone5_tone2:"]::before,
em[data-emoji="women_holding_hands_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f469-1f3fc.svg");
}

em[data-emoji=":women_holding_hands_tone5_tone3:"]::before,
em[data-emoji="women_holding_hands_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f469-1f3fd.svg");
}

em[data-emoji=":women_holding_hands_tone5_tone4:"]::before,
em[data-emoji="women_holding_hands_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-1f91d-200d-1f469-1f3fe.svg");
}

em[data-emoji=":women_holding_hands_tone5:"]::before,
em[data-emoji="women_holding_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46d-1f3ff.svg");
}

em[data-emoji=":two_men_holding_hands:"]::before,
em[data-emoji="two_men_holding_hands"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c.svg");
}

em[data-emoji=":men_holding_hands_tone1:"]::before,
em[data-emoji="men_holding_hands_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c-1f3fb.svg");
}

em[data-emoji=":men_holding_hands_tone1_tone2:"]::before,
em[data-emoji="men_holding_hands_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":men_holding_hands_tone1_tone3:"]::before,
em[data-emoji="men_holding_hands_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":men_holding_hands_tone1_tone4:"]::before,
em[data-emoji="men_holding_hands_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":men_holding_hands_tone1_tone5:"]::before,
em[data-emoji="men_holding_hands_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":men_holding_hands_tone2_tone1:"]::before,
em[data-emoji="men_holding_hands_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":men_holding_hands_tone2:"]::before,
em[data-emoji="men_holding_hands_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c-1f3fc.svg");
}

em[data-emoji=":men_holding_hands_tone2_tone3:"]::before,
em[data-emoji="men_holding_hands_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":men_holding_hands_tone2_tone4:"]::before,
em[data-emoji="men_holding_hands_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":men_holding_hands_tone2_tone5:"]::before,
em[data-emoji="men_holding_hands_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":men_holding_hands_tone3_tone1:"]::before,
em[data-emoji="men_holding_hands_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":men_holding_hands_tone3_tone2:"]::before,
em[data-emoji="men_holding_hands_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":men_holding_hands_tone3:"]::before,
em[data-emoji="men_holding_hands_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c-1f3fd.svg");
}

em[data-emoji=":men_holding_hands_tone3_tone4:"]::before,
em[data-emoji="men_holding_hands_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":men_holding_hands_tone3_tone5:"]::before,
em[data-emoji="men_holding_hands_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":men_holding_hands_tone4_tone1:"]::before,
em[data-emoji="men_holding_hands_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":men_holding_hands_tone4_tone2:"]::before,
em[data-emoji="men_holding_hands_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":men_holding_hands_tone4_tone3:"]::before,
em[data-emoji="men_holding_hands_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":men_holding_hands_tone4:"]::before,
em[data-emoji="men_holding_hands_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c-1f3fe.svg");
}

em[data-emoji=":men_holding_hands_tone4_tone5:"]::before,
em[data-emoji="men_holding_hands_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-1f91d-200d-1f468-1f3ff.svg");
}

em[data-emoji=":men_holding_hands_tone5_tone1:"]::before,
em[data-emoji="men_holding_hands_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f91d-200d-1f468-1f3fb.svg");
}

em[data-emoji=":men_holding_hands_tone5_tone2:"]::before,
em[data-emoji="men_holding_hands_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f91d-200d-1f468-1f3fc.svg");
}

em[data-emoji=":men_holding_hands_tone5_tone3:"]::before,
em[data-emoji="men_holding_hands_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f91d-200d-1f468-1f3fd.svg");
}

em[data-emoji=":men_holding_hands_tone5_tone4:"]::before,
em[data-emoji="men_holding_hands_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-1f91d-200d-1f468-1f3fe.svg");
}

em[data-emoji=":men_holding_hands_tone5:"]::before,
em[data-emoji="men_holding_hands_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46c-1f3ff.svg");
}

em[data-emoji=":couple_with_heart:"]::before,
em[data-emoji="couple_with_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491.svg");
}

em[data-emoji=":couple_with_heart_tone1:"]::before,
em[data-emoji="couple_with_heart_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone1_tone2:"]::before,
em[data-emoji="couple_with_heart_person_person_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone1_tone3:"]::before,
em[data-emoji="couple_with_heart_person_person_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone1_tone4:"]::before,
em[data-emoji="couple_with_heart_person_person_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone1_tone5:"]::before,
em[data-emoji="couple_with_heart_person_person_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone2_tone1:"]::before,
em[data-emoji="couple_with_heart_person_person_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_tone2:"]::before,
em[data-emoji="couple_with_heart_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone2_tone3:"]::before,
em[data-emoji="couple_with_heart_person_person_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone2_tone4:"]::before,
em[data-emoji="couple_with_heart_person_person_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone2_tone5:"]::before,
em[data-emoji="couple_with_heart_person_person_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone3_tone1:"]::before,
em[data-emoji="couple_with_heart_person_person_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone3_tone2:"]::before,
em[data-emoji="couple_with_heart_person_person_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_tone3:"]::before,
em[data-emoji="couple_with_heart_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone3_tone4:"]::before,
em[data-emoji="couple_with_heart_person_person_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone3_tone5:"]::before,
em[data-emoji="couple_with_heart_person_person_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone4_tone1:"]::before,
em[data-emoji="couple_with_heart_person_person_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone4_tone2:"]::before,
em[data-emoji="couple_with_heart_person_person_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone4_tone3:"]::before,
em[data-emoji="couple_with_heart_person_person_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_tone4:"]::before,
em[data-emoji="couple_with_heart_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone4_tone5:"]::before,
em[data-emoji="couple_with_heart_person_person_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone5_tone1:"]::before,
em[data-emoji="couple_with_heart_person_person_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone5_tone2:"]::before,
em[data-emoji="couple_with_heart_person_person_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone5_tone3:"]::before,
em[data-emoji="couple_with_heart_person_person_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_person_person_tone5_tone4:"]::before,
em[data-emoji="couple_with_heart_person_person_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_tone5:"]::before,
em[data-emoji="couple_with_heart_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f491-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_man:"]::before,
em[data-emoji="couple_with_heart_woman_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2764-fe0f-200d-1f468.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone1_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone1_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone1_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone1_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone2_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone2_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone2_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone2_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone3_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone3_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone3_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone3_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone4_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone4_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone4_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone4_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone5_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone5_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone5_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone5_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_man_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_ww:"]::before,
em[data-emoji="couple_ww"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2764-fe0f-200d-1f469.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f469-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone1_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f469-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone1_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f469-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone1_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f469-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone1_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f469-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone2_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f469-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f469-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone2_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f469-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone2_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f469-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone2_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f469-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone3_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f469-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone3_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f469-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f469-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone3_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f469-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone3_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f469-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone4_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f469-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone4_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f469-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone4_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f469-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f469-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone4_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f469-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone5_tone1:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f469-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone5_tone2:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f469-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone5_tone3:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f469-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone5_tone4:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f469-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_woman_woman_tone5:"]::before,
em[data-emoji="couple_with_heart_woman_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f469-1f3ff.svg");
}

em[data-emoji=":couple_mm:"]::before,
em[data-emoji="couple_mm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-2764-fe0f-200d-1f468.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone1:"]::before,
em[data-emoji="couple_with_heart_man_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone1_tone2:"]::before,
em[data-emoji="couple_with_heart_man_man_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone1_tone3:"]::before,
em[data-emoji="couple_with_heart_man_man_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone1_tone4:"]::before,
em[data-emoji="couple_with_heart_man_man_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone1_tone5:"]::before,
em[data-emoji="couple_with_heart_man_man_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone2_tone1:"]::before,
em[data-emoji="couple_with_heart_man_man_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone2:"]::before,
em[data-emoji="couple_with_heart_man_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone2_tone3:"]::before,
em[data-emoji="couple_with_heart_man_man_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone2_tone4:"]::before,
em[data-emoji="couple_with_heart_man_man_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone2_tone5:"]::before,
em[data-emoji="couple_with_heart_man_man_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone3_tone1:"]::before,
em[data-emoji="couple_with_heart_man_man_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone3_tone2:"]::before,
em[data-emoji="couple_with_heart_man_man_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone3:"]::before,
em[data-emoji="couple_with_heart_man_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone3_tone4:"]::before,
em[data-emoji="couple_with_heart_man_man_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone3_tone5:"]::before,
em[data-emoji="couple_with_heart_man_man_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone4_tone1:"]::before,
em[data-emoji="couple_with_heart_man_man_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone4_tone2:"]::before,
em[data-emoji="couple_with_heart_man_man_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone4_tone3:"]::before,
em[data-emoji="couple_with_heart_man_man_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone4:"]::before,
em[data-emoji="couple_with_heart_man_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone4_tone5:"]::before,
em[data-emoji="couple_with_heart_man_man_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone5_tone1:"]::before,
em[data-emoji="couple_with_heart_man_man_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f468-1f3fb.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone5_tone2:"]::before,
em[data-emoji="couple_with_heart_man_man_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f468-1f3fc.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone5_tone3:"]::before,
em[data-emoji="couple_with_heart_man_man_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f468-1f3fd.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone5_tone4:"]::before,
em[data-emoji="couple_with_heart_man_man_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f468-1f3fe.svg");
}

em[data-emoji=":couple_with_heart_man_man_tone5:"]::before,
em[data-emoji="couple_with_heart_man_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f468-1f3ff.svg");
}

em[data-emoji=":couplekiss:"]::before,
em[data-emoji="couplekiss"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f.svg");
}

em[data-emoji=":kiss_tone1:"]::before,
em[data-emoji="kiss_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f-1f3fb.svg");
}

em[data-emoji=":kiss_person_person_tone1_tone2:"]::before,
em[data-emoji="kiss_person_person_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":kiss_person_person_tone1_tone3:"]::before,
em[data-emoji="kiss_person_person_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":kiss_person_person_tone1_tone4:"]::before,
em[data-emoji="kiss_person_person_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":kiss_person_person_tone1_tone5:"]::before,
em[data-emoji="kiss_person_person_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":kiss_person_person_tone2_tone1:"]::before,
em[data-emoji="kiss_person_person_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":kiss_tone2:"]::before,
em[data-emoji="kiss_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f-1f3fc.svg");
}

em[data-emoji=":kiss_person_person_tone2_tone3:"]::before,
em[data-emoji="kiss_person_person_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":kiss_person_person_tone2_tone4:"]::before,
em[data-emoji="kiss_person_person_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":kiss_person_person_tone2_tone5:"]::before,
em[data-emoji="kiss_person_person_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":kiss_person_person_tone3_tone1:"]::before,
em[data-emoji="kiss_person_person_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":kiss_person_person_tone3_tone2:"]::before,
em[data-emoji="kiss_person_person_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":kiss_tone3:"]::before,
em[data-emoji="kiss_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f-1f3fd.svg");
}

em[data-emoji=":kiss_person_person_tone3_tone4:"]::before,
em[data-emoji="kiss_person_person_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":kiss_person_person_tone3_tone5:"]::before,
em[data-emoji="kiss_person_person_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":kiss_person_person_tone4_tone1:"]::before,
em[data-emoji="kiss_person_person_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":kiss_person_person_tone4_tone2:"]::before,
em[data-emoji="kiss_person_person_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":kiss_person_person_tone4_tone3:"]::before,
em[data-emoji="kiss_person_person_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":kiss_tone4:"]::before,
em[data-emoji="kiss_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f-1f3fe.svg");
}

em[data-emoji=":kiss_person_person_tone4_tone5:"]::before,
em[data-emoji="kiss_person_person_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3ff.svg");
}

em[data-emoji=":kiss_person_person_tone5_tone1:"]::before,
em[data-emoji="kiss_person_person_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fb.svg");
}

em[data-emoji=":kiss_person_person_tone5_tone2:"]::before,
em[data-emoji="kiss_person_person_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fc.svg");
}

em[data-emoji=":kiss_person_person_tone5_tone3:"]::before,
em[data-emoji="kiss_person_person_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fd.svg");
}

em[data-emoji=":kiss_person_person_tone5_tone4:"]::before,
em[data-emoji="kiss_person_person_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d1-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f9d1-1f3fe.svg");
}

em[data-emoji=":kiss_tone5:"]::before,
em[data-emoji="kiss_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48f-1f3ff.svg");
}

em[data-emoji=":kiss_woman_man:"]::before,
em[data-emoji="kiss_woman_man"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2764-fe0f-200d-1f48b-200d-1f468.svg");
}

em[data-emoji=":kiss_woman_man_tone1:"]::before,
em[data-emoji="kiss_woman_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_woman_man_tone1_tone2:"]::before,
em[data-emoji="kiss_woman_man_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_woman_man_tone1_tone3:"]::before,
em[data-emoji="kiss_woman_man_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_woman_man_tone1_tone4:"]::before,
em[data-emoji="kiss_woman_man_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_woman_man_tone1_tone5:"]::before,
em[data-emoji="kiss_woman_man_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_woman_man_tone2_tone1:"]::before,
em[data-emoji="kiss_woman_man_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_woman_man_tone2:"]::before,
em[data-emoji="kiss_woman_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_woman_man_tone2_tone3:"]::before,
em[data-emoji="kiss_woman_man_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_woman_man_tone2_tone4:"]::before,
em[data-emoji="kiss_woman_man_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_woman_man_tone2_tone5:"]::before,
em[data-emoji="kiss_woman_man_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_woman_man_tone3_tone1:"]::before,
em[data-emoji="kiss_woman_man_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_woman_man_tone3_tone2:"]::before,
em[data-emoji="kiss_woman_man_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_woman_man_tone3:"]::before,
em[data-emoji="kiss_woman_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_woman_man_tone3_tone4:"]::before,
em[data-emoji="kiss_woman_man_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_woman_man_tone3_tone5:"]::before,
em[data-emoji="kiss_woman_man_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_woman_man_tone4_tone1:"]::before,
em[data-emoji="kiss_woman_man_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_woman_man_tone4_tone2:"]::before,
em[data-emoji="kiss_woman_man_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_woman_man_tone4_tone3:"]::before,
em[data-emoji="kiss_woman_man_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_woman_man_tone4:"]::before,
em[data-emoji="kiss_woman_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_woman_man_tone4_tone5:"]::before,
em[data-emoji="kiss_woman_man_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_woman_man_tone5_tone1:"]::before,
em[data-emoji="kiss_woman_man_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_woman_man_tone5_tone2:"]::before,
em[data-emoji="kiss_woman_man_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_woman_man_tone5_tone3:"]::before,
em[data-emoji="kiss_woman_man_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_woman_man_tone5_tone4:"]::before,
em[data-emoji="kiss_woman_man_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_woman_man_tone5:"]::before,
em[data-emoji="kiss_woman_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_ww:"]::before,
em[data-emoji="kiss_ww"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-2764-fe0f-200d-1f48b-200d-1f469.svg");
}

em[data-emoji=":kiss_woman_woman_tone1:"]::before,
em[data-emoji="kiss_woman_woman_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fb.svg");
}

em[data-emoji=":kiss_woman_woman_tone1_tone2:"]::before,
em[data-emoji="kiss_woman_woman_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fc.svg");
}

em[data-emoji=":kiss_woman_woman_tone1_tone3:"]::before,
em[data-emoji="kiss_woman_woman_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fd.svg");
}

em[data-emoji=":kiss_woman_woman_tone1_tone4:"]::before,
em[data-emoji="kiss_woman_woman_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fe.svg");
}

em[data-emoji=":kiss_woman_woman_tone1_tone5:"]::before,
em[data-emoji="kiss_woman_woman_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3ff.svg");
}

em[data-emoji=":kiss_woman_woman_tone2_tone1:"]::before,
em[data-emoji="kiss_woman_woman_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fb.svg");
}

em[data-emoji=":kiss_woman_woman_tone2:"]::before,
em[data-emoji="kiss_woman_woman_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fc.svg");
}

em[data-emoji=":kiss_woman_woman_tone2_tone3:"]::before,
em[data-emoji="kiss_woman_woman_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fd.svg");
}

em[data-emoji=":kiss_woman_woman_tone2_tone4:"]::before,
em[data-emoji="kiss_woman_woman_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fe.svg");
}

em[data-emoji=":kiss_woman_woman_tone2_tone5:"]::before,
em[data-emoji="kiss_woman_woman_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3ff.svg");
}

em[data-emoji=":kiss_woman_woman_tone3_tone1:"]::before,
em[data-emoji="kiss_woman_woman_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fb.svg");
}

em[data-emoji=":kiss_woman_woman_tone3_tone2:"]::before,
em[data-emoji="kiss_woman_woman_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fc.svg");
}

em[data-emoji=":kiss_woman_woman_tone3:"]::before,
em[data-emoji="kiss_woman_woman_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fd.svg");
}

em[data-emoji=":kiss_woman_woman_tone3_tone4:"]::before,
em[data-emoji="kiss_woman_woman_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fe.svg");
}

em[data-emoji=":kiss_woman_woman_tone3_tone5:"]::before,
em[data-emoji="kiss_woman_woman_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3ff.svg");
}

em[data-emoji=":kiss_woman_woman_tone4_tone1:"]::before,
em[data-emoji="kiss_woman_woman_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fb.svg");
}

em[data-emoji=":kiss_woman_woman_tone4_tone2:"]::before,
em[data-emoji="kiss_woman_woman_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fc.svg");
}

em[data-emoji=":kiss_woman_woman_tone4_tone3:"]::before,
em[data-emoji="kiss_woman_woman_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fd.svg");
}

em[data-emoji=":kiss_woman_woman_tone4:"]::before,
em[data-emoji="kiss_woman_woman_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fe.svg");
}

em[data-emoji=":kiss_woman_woman_tone4_tone5:"]::before,
em[data-emoji="kiss_woman_woman_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3ff.svg");
}

em[data-emoji=":kiss_woman_woman_tone5_tone1:"]::before,
em[data-emoji="kiss_woman_woman_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fb.svg");
}

em[data-emoji=":kiss_woman_woman_tone5_tone2:"]::before,
em[data-emoji="kiss_woman_woman_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fc.svg");
}

em[data-emoji=":kiss_woman_woman_tone5_tone3:"]::before,
em[data-emoji="kiss_woman_woman_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fd.svg");
}

em[data-emoji=":kiss_woman_woman_tone5_tone4:"]::before,
em[data-emoji="kiss_woman_woman_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3fe.svg");
}

em[data-emoji=":kiss_woman_woman_tone5:"]::before,
em[data-emoji="kiss_woman_woman_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f469-1f3ff.svg");
}

em[data-emoji=":kiss_mm:"]::before,
em[data-emoji="kiss_mm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-2764-fe0f-200d-1f48b-200d-1f468.svg");
}

em[data-emoji=":kiss_man_man_tone1:"]::before,
em[data-emoji="kiss_man_man_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_man_man_tone1_tone2:"]::before,
em[data-emoji="kiss_man_man_tone1_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_man_man_tone1_tone3:"]::before,
em[data-emoji="kiss_man_man_tone1_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_man_man_tone1_tone4:"]::before,
em[data-emoji="kiss_man_man_tone1_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_man_man_tone1_tone5:"]::before,
em[data-emoji="kiss_man_man_tone1_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fb-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_man_man_tone2_tone1:"]::before,
em[data-emoji="kiss_man_man_tone2_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_man_man_tone2:"]::before,
em[data-emoji="kiss_man_man_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_man_man_tone2_tone3:"]::before,
em[data-emoji="kiss_man_man_tone2_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_man_man_tone2_tone4:"]::before,
em[data-emoji="kiss_man_man_tone2_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_man_man_tone2_tone5:"]::before,
em[data-emoji="kiss_man_man_tone2_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fc-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_man_man_tone3_tone1:"]::before,
em[data-emoji="kiss_man_man_tone3_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_man_man_tone3_tone2:"]::before,
em[data-emoji="kiss_man_man_tone3_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_man_man_tone3:"]::before,
em[data-emoji="kiss_man_man_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_man_man_tone3_tone4:"]::before,
em[data-emoji="kiss_man_man_tone3_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_man_man_tone3_tone5:"]::before,
em[data-emoji="kiss_man_man_tone3_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fd-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_man_man_tone4_tone1:"]::before,
em[data-emoji="kiss_man_man_tone4_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_man_man_tone4_tone2:"]::before,
em[data-emoji="kiss_man_man_tone4_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_man_man_tone4_tone3:"]::before,
em[data-emoji="kiss_man_man_tone4_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_man_man_tone4:"]::before,
em[data-emoji="kiss_man_man_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_man_man_tone4_tone5:"]::before,
em[data-emoji="kiss_man_man_tone4_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3fe-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":kiss_man_man_tone5_tone1:"]::before,
em[data-emoji="kiss_man_man_tone5_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fb.svg");
}

em[data-emoji=":kiss_man_man_tone5_tone2:"]::before,
em[data-emoji="kiss_man_man_tone5_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fc.svg");
}

em[data-emoji=":kiss_man_man_tone5_tone3:"]::before,
em[data-emoji="kiss_man_man_tone5_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fd.svg");
}

em[data-emoji=":kiss_man_man_tone5_tone4:"]::before,
em[data-emoji="kiss_man_man_tone5_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3fe.svg");
}

em[data-emoji=":kiss_man_man_tone5:"]::before,
em[data-emoji="kiss_man_man_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-1f3ff-200d-2764-fe0f-200d-1f48b-200d-1f468-1f3ff.svg");
}

em[data-emoji=":family:"]::before,
em[data-emoji="family"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f46a.svg");
}

em[data-emoji=":family_man_woman_boy:"]::before,
em[data-emoji="family_man_woman_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f469-200d-1f466.svg");
}

em[data-emoji=":family_mwg:"]::before,
em[data-emoji="family_mwg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f469-200d-1f467.svg");
}

em[data-emoji=":family_mwgb:"]::before,
em[data-emoji="family_mwgb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f469-200d-1f467-200d-1f466.svg");
}

em[data-emoji=":family_mwbb:"]::before,
em[data-emoji="family_mwbb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f469-200d-1f466-200d-1f466.svg");
}

em[data-emoji=":family_mwgg:"]::before,
em[data-emoji="family_mwgg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f469-200d-1f467-200d-1f467.svg");
}

em[data-emoji=":family_wwb:"]::before,
em[data-emoji="family_wwb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f469-200d-1f466.svg");
}

em[data-emoji=":family_wwg:"]::before,
em[data-emoji="family_wwg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f469-200d-1f467.svg");
}

em[data-emoji=":family_wwgb:"]::before,
em[data-emoji="family_wwgb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f469-200d-1f467-200d-1f466.svg");
}

em[data-emoji=":family_wwbb:"]::before,
em[data-emoji="family_wwbb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f469-200d-1f466-200d-1f466.svg");
}

em[data-emoji=":family_wwgg:"]::before,
em[data-emoji="family_wwgg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f469-200d-1f467-200d-1f467.svg");
}

em[data-emoji=":family_mmb:"]::before,
em[data-emoji="family_mmb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f468-200d-1f466.svg");
}

em[data-emoji=":family_mmg:"]::before,
em[data-emoji="family_mmg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f468-200d-1f467.svg");
}

em[data-emoji=":family_mmgb:"]::before,
em[data-emoji="family_mmgb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f468-200d-1f467-200d-1f466.svg");
}

em[data-emoji=":family_mmbb:"]::before,
em[data-emoji="family_mmbb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f468-200d-1f466-200d-1f466.svg");
}

em[data-emoji=":family_mmgg:"]::before,
em[data-emoji="family_mmgg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f468-200d-1f467-200d-1f467.svg");
}

em[data-emoji=":family_woman_boy:"]::before,
em[data-emoji="family_woman_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f466.svg");
}

em[data-emoji=":family_woman_girl:"]::before,
em[data-emoji="family_woman_girl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f467.svg");
}

em[data-emoji=":family_woman_girl_boy:"]::before,
em[data-emoji="family_woman_girl_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f467-200d-1f466.svg");
}

em[data-emoji=":family_woman_boy_boy:"]::before,
em[data-emoji="family_woman_boy_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f466-200d-1f466.svg");
}

em[data-emoji=":family_woman_girl_girl:"]::before,
em[data-emoji="family_woman_girl_girl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f469-200d-1f467-200d-1f467.svg");
}

em[data-emoji=":family_man_boy:"]::before,
em[data-emoji="family_man_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f466.svg");
}

em[data-emoji=":family_man_girl:"]::before,
em[data-emoji="family_man_girl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f467.svg");
}

em[data-emoji=":family_man_girl_boy:"]::before,
em[data-emoji="family_man_girl_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f467-200d-1f466.svg");
}

em[data-emoji=":family_man_boy_boy:"]::before,
em[data-emoji="family_man_boy_boy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f466-200d-1f466.svg");
}

em[data-emoji=":family_man_girl_girl:"]::before,
em[data-emoji="family_man_girl_girl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f468-200d-1f467-200d-1f467.svg");
}

em[data-emoji=":knot:"]::before,
em[data-emoji="knot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa2.svg");
}

em[data-emoji=":yarn:"]::before,
em[data-emoji="yarn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f6.svg");
}

em[data-emoji=":thread:"]::before,
em[data-emoji="thread"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f5.svg");
}

em[data-emoji=":sewing_needle:"]::before,
em[data-emoji="sewing_needle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa1.svg");
}

em[data-emoji=":coat:"]::before,
em[data-emoji="coat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e5.svg");
}

em[data-emoji=":lab_coat:"]::before,
em[data-emoji="lab_coat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97c.svg");
}

em[data-emoji=":safety_vest:"]::before,
em[data-emoji="safety_vest"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ba.svg");
}

em[data-emoji=":womans_clothes:"]::before,
em[data-emoji="womans_clothes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45a.svg");
}

em[data-emoji=":shirt:"]::before,
em[data-emoji="shirt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f455.svg");
}

em[data-emoji=":jeans:"]::before,
em[data-emoji="jeans"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f456.svg");
}

em[data-emoji=":briefs:"]::before,
em[data-emoji="briefs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa72.svg");
}

em[data-emoji=":shorts:"]::before,
em[data-emoji="shorts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa73.svg");
}

em[data-emoji=":necktie:"]::before,
em[data-emoji="necktie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f454.svg");
}

em[data-emoji=":dress:"]::before,
em[data-emoji="dress"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f457.svg");
}

em[data-emoji=":bikini:"]::before,
em[data-emoji="bikini"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f459.svg");
}

em[data-emoji=":one_piece_swimsuit:"]::before,
em[data-emoji="one_piece_swimsuit"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa71.svg");
}

em[data-emoji=":kimono:"]::before,
em[data-emoji="kimono"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f458.svg");
}

em[data-emoji=":sari:"]::before,
em[data-emoji="sari"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97b.svg");
}

em[data-emoji=":thong_sandal:"]::before,
em[data-emoji="thong_sandal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa74.svg");
}

em[data-emoji=":womans_flat_shoe:"]::before,
em[data-emoji="womans_flat_shoe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97f.svg");
}

em[data-emoji=":high_heel:"]::before,
em[data-emoji="high_heel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f460.svg");
}

em[data-emoji=":sandal:"]::before,
em[data-emoji="sandal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f461.svg");
}

em[data-emoji=":boot:"]::before,
em[data-emoji="boot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f462.svg");
}

em[data-emoji=":mans_shoe:"]::before,
em[data-emoji="mans_shoe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45e.svg");
}

em[data-emoji=":athletic_shoe:"]::before,
em[data-emoji="athletic_shoe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45f.svg");
}

em[data-emoji=":hiking_boot:"]::before,
em[data-emoji="hiking_boot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97e.svg");
}

em[data-emoji=":socks:"]::before,
em[data-emoji="socks"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e6.svg");
}

em[data-emoji=":gloves:"]::before,
em[data-emoji="gloves"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e4.svg");
}

em[data-emoji=":scarf:"]::before,
em[data-emoji="scarf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e3.svg");
}

em[data-emoji=":tophat:"]::before,
em[data-emoji="tophat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a9.svg");
}

em[data-emoji=":billed_cap:"]::before,
em[data-emoji="billed_cap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e2.svg");
}

em[data-emoji=":womans_hat:"]::before,
em[data-emoji="womans_hat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f452.svg");
}

em[data-emoji=":mortar_board:"]::before,
em[data-emoji="mortar_board"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f393.svg");
}

em[data-emoji=":helmet_with_cross:"]::before,
em[data-emoji="helmet_with_cross"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26d1.svg");
}

em[data-emoji=":military_helmet:"]::before,
em[data-emoji="military_helmet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa96.svg");
}

em[data-emoji=":crown:"]::before,
em[data-emoji="crown"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f451.svg");
}

em[data-emoji=":ring:"]::before,
em[data-emoji="ring"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48d.svg");
}

em[data-emoji=":pouch:"]::before,
em[data-emoji="pouch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45d.svg");
}

em[data-emoji=":purse:"]::before,
em[data-emoji="purse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45b.svg");
}

em[data-emoji=":handbag:"]::before,
em[data-emoji="handbag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f45c.svg");
}

em[data-emoji=":briefcase:"]::before,
em[data-emoji="briefcase"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4bc.svg");
}

em[data-emoji=":school_satchel:"]::before,
em[data-emoji="school_satchel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f392.svg");
}

em[data-emoji=":luggage:"]::before,
em[data-emoji="luggage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f3.svg");
}

em[data-emoji=":eyeglasses:"]::before,
em[data-emoji="eyeglasses"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f453.svg");
}

em[data-emoji=":dark_sunglasses:"]::before,
em[data-emoji="dark_sunglasses"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f576.svg");
}

em[data-emoji=":goggles:"]::before,
em[data-emoji="goggles"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f97d.svg");
}

em[data-emoji=":closed_umbrella:"]::before,
em[data-emoji="closed_umbrella"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f302.svg");
}

em[data-emoji=":curly_haired:"]::before,
em[data-emoji="curly_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b1.svg");
}

em[data-emoji=":red_haired:"]::before,
em[data-emoji="red_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b0.svg");
}

em[data-emoji=":white_haired:"]::before,
em[data-emoji="white_haired"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b3.svg");
}

em[data-emoji=":bald:"]::before,
em[data-emoji="bald"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b2.svg");
}

em[data-emoji=":dog:"]::before,
em[data-emoji="dog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f436.svg");
}

em[data-emoji=":cat:"]::before,
em[data-emoji="cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f431.svg");
}

em[data-emoji=":mouse:"]::before,
em[data-emoji="mouse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42d.svg");
}

em[data-emoji=":hamster:"]::before,
em[data-emoji="hamster"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f439.svg");
}

em[data-emoji=":rabbit:"]::before,
em[data-emoji="rabbit"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f430.svg");
}

em[data-emoji=":fox:"]::before,
em[data-emoji="fox"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98a.svg");
}

em[data-emoji=":bear:"]::before,
em[data-emoji="bear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43b.svg");
}

em[data-emoji=":panda_face:"]::before,
em[data-emoji="panda_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43c.svg");
}

em[data-emoji=":polar_bear:"]::before,
em[data-emoji="polar_bear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43b-200d-2744-fe0f.svg");
}

em[data-emoji=":koala:"]::before,
em[data-emoji="koala"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f428.svg");
}

em[data-emoji=":tiger:"]::before,
em[data-emoji="tiger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42f.svg");
}

em[data-emoji=":lion_face:"]::before,
em[data-emoji="lion_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f981.svg");
}

em[data-emoji=":cow:"]::before,
em[data-emoji="cow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42e.svg");
}

em[data-emoji=":pig:"]::before,
em[data-emoji="pig"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f437.svg");
}

em[data-emoji=":pig_nose:"]::before,
em[data-emoji="pig_nose"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43d.svg");
}

em[data-emoji=":frog:"]::before,
em[data-emoji="frog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f438.svg");
}

em[data-emoji=":monkey_face:"]::before,
em[data-emoji="monkey_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f435.svg");
}

em[data-emoji=":see_no_evil:"]::before,
em[data-emoji="see_no_evil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f648.svg");
}

em[data-emoji=":hear_no_evil:"]::before,
em[data-emoji="hear_no_evil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f649.svg");
}

em[data-emoji=":speak_no_evil:"]::before,
em[data-emoji="speak_no_evil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f64a.svg");
}

em[data-emoji=":monkey:"]::before,
em[data-emoji="monkey"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f412.svg");
}

em[data-emoji=":chicken:"]::before,
em[data-emoji="chicken"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f414.svg");
}

em[data-emoji=":penguin:"]::before,
em[data-emoji="penguin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f427.svg");
}

em[data-emoji=":bird:"]::before,
em[data-emoji="bird"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f426.svg");
}

em[data-emoji=":baby_chick:"]::before,
em[data-emoji="baby_chick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f424.svg");
}

em[data-emoji=":hatching_chick:"]::before,
em[data-emoji="hatching_chick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f423.svg");
}

em[data-emoji=":hatched_chick:"]::before,
em[data-emoji="hatched_chick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f425.svg");
}

em[data-emoji=":duck:"]::before,
em[data-emoji="duck"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f986.svg");
}

em[data-emoji=":eagle:"]::before,
em[data-emoji="eagle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f985.svg");
}

em[data-emoji=":owl:"]::before,
em[data-emoji="owl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f989.svg");
}

em[data-emoji=":bat:"]::before,
em[data-emoji="bat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f987.svg");
}

em[data-emoji=":wolf:"]::before,
em[data-emoji="wolf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43a.svg");
}

em[data-emoji=":boar:"]::before,
em[data-emoji="boar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f417.svg");
}

em[data-emoji=":horse:"]::before,
em[data-emoji="horse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f434.svg");
}

em[data-emoji=":unicorn:"]::before,
em[data-emoji="unicorn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f984.svg");
}

em[data-emoji=":bee:"]::before,
em[data-emoji="bee"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41d.svg");
}

em[data-emoji=":bug:"]::before,
em[data-emoji="bug"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41b.svg");
}

em[data-emoji=":butterfly:"]::before,
em[data-emoji="butterfly"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98b.svg");
}

em[data-emoji=":snail:"]::before,
em[data-emoji="snail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40c.svg");
}

em[data-emoji=":worm:"]::before,
em[data-emoji="worm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab1.svg");
}

em[data-emoji=":lady_beetle:"]::before,
em[data-emoji="lady_beetle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41e.svg");
}

em[data-emoji=":ant:"]::before,
em[data-emoji="ant"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41c.svg");
}

em[data-emoji=":fly:"]::before,
em[data-emoji="fly"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab0.svg");
}

em[data-emoji=":mosquito:"]::before,
em[data-emoji="mosquito"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99f.svg");
}

em[data-emoji=":cockroach:"]::before,
em[data-emoji="cockroach"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab3.svg");
}

em[data-emoji=":beetle:"]::before,
em[data-emoji="beetle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab2.svg");
}

em[data-emoji=":cricket:"]::before,
em[data-emoji="cricket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f997.svg");
}

em[data-emoji=":spider:"]::before,
em[data-emoji="spider"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f577.svg");
}

em[data-emoji=":spider_web:"]::before,
em[data-emoji="spider_web"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f578.svg");
}

em[data-emoji=":scorpion:"]::before,
em[data-emoji="scorpion"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f982.svg");
}

em[data-emoji=":turtle:"]::before,
em[data-emoji="turtle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f422.svg");
}

em[data-emoji=":snake:"]::before,
em[data-emoji="snake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40d.svg");
}

em[data-emoji=":lizard:"]::before,
em[data-emoji="lizard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98e.svg");
}

em[data-emoji=":t_rex:"]::before,
em[data-emoji="t_rex"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f996.svg");
}

em[data-emoji=":sauropod:"]::before,
em[data-emoji="sauropod"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f995.svg");
}

em[data-emoji=":octopus:"]::before,
em[data-emoji="octopus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f419.svg");
}

em[data-emoji=":squid:"]::before,
em[data-emoji="squid"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f991.svg");
}

em[data-emoji=":shrimp:"]::before,
em[data-emoji="shrimp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f990.svg");
}

em[data-emoji=":lobster:"]::before,
em[data-emoji="lobster"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99e.svg");
}

em[data-emoji=":crab:"]::before,
em[data-emoji="crab"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f980.svg");
}

em[data-emoji=":blowfish:"]::before,
em[data-emoji="blowfish"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f421.svg");
}

em[data-emoji=":tropical_fish:"]::before,
em[data-emoji="tropical_fish"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f420.svg");
}

em[data-emoji=":fish:"]::before,
em[data-emoji="fish"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41f.svg");
}

em[data-emoji=":seal:"]::before,
em[data-emoji="seal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ad.svg");
}

em[data-emoji=":dolphin:"]::before,
em[data-emoji="dolphin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42c.svg");
}

em[data-emoji=":whale:"]::before,
em[data-emoji="whale"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f433.svg");
}

em[data-emoji=":whale2:"]::before,
em[data-emoji="whale2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40b.svg");
}

em[data-emoji=":shark:"]::before,
em[data-emoji="shark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f988.svg");
}

em[data-emoji=":crocodile:"]::before,
em[data-emoji="crocodile"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40a.svg");
}

em[data-emoji=":tiger2:"]::before,
em[data-emoji="tiger2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f405.svg");
}

em[data-emoji=":leopard:"]::before,
em[data-emoji="leopard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f406.svg");
}

em[data-emoji=":zebra:"]::before,
em[data-emoji="zebra"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f993.svg");
}

em[data-emoji=":gorilla:"]::before,
em[data-emoji="gorilla"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98d.svg");
}

em[data-emoji=":orangutan:"]::before,
em[data-emoji="orangutan"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a7.svg");
}

em[data-emoji=":elephant:"]::before,
em[data-emoji="elephant"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f418.svg");
}

em[data-emoji=":mammoth:"]::before,
em[data-emoji="mammoth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a3.svg");
}

em[data-emoji=":bison:"]::before,
em[data-emoji="bison"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ac.svg");
}

em[data-emoji=":hippopotamus:"]::before,
em[data-emoji="hippopotamus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99b.svg");
}

em[data-emoji=":rhino:"]::before,
em[data-emoji="rhino"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98f.svg");
}

em[data-emoji=":dromedary_camel:"]::before,
em[data-emoji="dromedary_camel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42a.svg");
}

em[data-emoji=":camel:"]::before,
em[data-emoji="camel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f42b.svg");
}

em[data-emoji=":giraffe:"]::before,
em[data-emoji="giraffe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f992.svg");
}

em[data-emoji=":kangaroo:"]::before,
em[data-emoji="kangaroo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f998.svg");
}

em[data-emoji=":water_buffalo:"]::before,
em[data-emoji="water_buffalo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f403.svg");
}

em[data-emoji=":ox:"]::before,
em[data-emoji="ox"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f402.svg");
}

em[data-emoji=":cow2:"]::before,
em[data-emoji="cow2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f404.svg");
}

em[data-emoji=":racehorse:"]::before,
em[data-emoji="racehorse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40e.svg");
}

em[data-emoji=":pig2:"]::before,
em[data-emoji="pig2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f416.svg");
}

em[data-emoji=":ram:"]::before,
em[data-emoji="ram"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f40f.svg");
}

em[data-emoji=":sheep:"]::before,
em[data-emoji="sheep"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f411.svg");
}

em[data-emoji=":llama:"]::before,
em[data-emoji="llama"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f999.svg");
}

em[data-emoji=":goat:"]::before,
em[data-emoji="goat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f410.svg");
}

em[data-emoji=":deer:"]::before,
em[data-emoji="deer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f98c.svg");
}

em[data-emoji=":dog2:"]::before,
em[data-emoji="dog2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f415.svg");
}

em[data-emoji=":poodle:"]::before,
em[data-emoji="poodle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f429.svg");
}

em[data-emoji=":guide_dog:"]::before,
em[data-emoji="guide_dog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ae.svg");
}

em[data-emoji=":service_dog:"]::before,
em[data-emoji="service_dog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f415-200d-1f9ba.svg");
}

em[data-emoji=":cat2:"]::before,
em[data-emoji="cat2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f408.svg");
}

em[data-emoji=":black_cat:"]::before,
em[data-emoji="black_cat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f408-200d-2b1b.svg");
}

em[data-emoji=":feather:"]::before,
em[data-emoji="feather"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab6.svg");
}

em[data-emoji=":rooster:"]::before,
em[data-emoji="rooster"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f413.svg");
}

em[data-emoji=":turkey:"]::before,
em[data-emoji="turkey"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f983.svg");
}

em[data-emoji=":dodo:"]::before,
em[data-emoji="dodo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a4.svg");
}

em[data-emoji=":peacock:"]::before,
em[data-emoji="peacock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99a.svg");
}

em[data-emoji=":parrot:"]::before,
em[data-emoji="parrot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99c.svg");
}

em[data-emoji=":swan:"]::before,
em[data-emoji="swan"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a2.svg");
}

em[data-emoji=":flamingo:"]::before,
em[data-emoji="flamingo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a9.svg");
}

em[data-emoji=":dove:"]::before,
em[data-emoji="dove"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f54a.svg");
}

em[data-emoji=":rabbit2:"]::before,
em[data-emoji="rabbit2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f407.svg");
}

em[data-emoji=":raccoon:"]::before,
em[data-emoji="raccoon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f99d.svg");
}

em[data-emoji=":skunk:"]::before,
em[data-emoji="skunk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a8.svg");
}

em[data-emoji=":badger:"]::before,
em[data-emoji="badger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a1.svg");
}

em[data-emoji=":beaver:"]::before,
em[data-emoji="beaver"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ab.svg");
}

em[data-emoji=":otter:"]::before,
em[data-emoji="otter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a6.svg");
}

em[data-emoji=":sloth:"]::before,
em[data-emoji="sloth"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a5.svg");
}

em[data-emoji=":mouse2:"]::before,
em[data-emoji="mouse2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f401.svg");
}

em[data-emoji=":rat:"]::before,
em[data-emoji="rat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f400.svg");
}

em[data-emoji=":chipmunk:"]::before,
em[data-emoji="chipmunk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43f.svg");
}

em[data-emoji=":hedgehog:"]::before,
em[data-emoji="hedgehog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f994.svg");
}

em[data-emoji=":feet:"]::before,
em[data-emoji="feet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f43e.svg");
}

em[data-emoji=":dragon:"]::before,
em[data-emoji="dragon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f409.svg");
}

em[data-emoji=":dragon_face:"]::before,
em[data-emoji="dragon_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f432.svg");
}

em[data-emoji=":cactus:"]::before,
em[data-emoji="cactus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f335.svg");
}

em[data-emoji=":christmas_tree:"]::before,
em[data-emoji="christmas_tree"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f384.svg");
}

em[data-emoji=":evergreen_tree:"]::before,
em[data-emoji="evergreen_tree"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f332.svg");
}

em[data-emoji=":deciduous_tree:"]::before,
em[data-emoji="deciduous_tree"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f333.svg");
}

em[data-emoji=":palm_tree:"]::before,
em[data-emoji="palm_tree"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f334.svg");
}

em[data-emoji=":seedling:"]::before,
em[data-emoji="seedling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f331.svg");
}

em[data-emoji=":herb:"]::before,
em[data-emoji="herb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33f.svg");
}

em[data-emoji=":four_leaf_clover:"]::before,
em[data-emoji="four_leaf_clover"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f340.svg");
}

em[data-emoji=":bamboo:"]::before,
em[data-emoji="bamboo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38d.svg");
}

em[data-emoji=":tanabata_tree:"]::before,
em[data-emoji="tanabata_tree"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38b.svg");
}

em[data-emoji=":leaves:"]::before,
em[data-emoji="leaves"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f343.svg");
}

em[data-emoji=":fallen_leaf:"]::before,
em[data-emoji="fallen_leaf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f342.svg");
}

em[data-emoji=":maple_leaf:"]::before,
em[data-emoji="maple_leaf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f341.svg");
}

em[data-emoji=":empty_nest:"]::before,
em[data-emoji="empty_nest"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab9.svg");
}

em[data-emoji=":nest_with_eggs:"]::before,
em[data-emoji="nest_with_eggs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faba.svg");
}

em[data-emoji=":mushroom:"]::before,
em[data-emoji="mushroom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f344.svg");
}

em[data-emoji=":shell:"]::before,
em[data-emoji="shell"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f41a.svg");
}

em[data-emoji=":coral:"]::before,
em[data-emoji="coral"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab8.svg");
}

em[data-emoji=":rock:"]::before,
em[data-emoji="rock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa8.svg");
}

em[data-emoji=":wood:"]::before,
em[data-emoji="wood"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab5.svg");
}

em[data-emoji=":ear_of_rice:"]::before,
em[data-emoji="ear_of_rice"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33e.svg");
}

em[data-emoji=":potted_plant:"]::before,
em[data-emoji="potted_plant"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab4.svg");
}

em[data-emoji=":bouquet:"]::before,
em[data-emoji="bouquet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f490.svg");
}

em[data-emoji=":tulip:"]::before,
em[data-emoji="tulip"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f337.svg");
}

em[data-emoji=":rose:"]::before,
em[data-emoji="rose"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f339.svg");
}

em[data-emoji=":wilted_rose:"]::before,
em[data-emoji="wilted_rose"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f940.svg");
}

em[data-emoji=":lotus:"]::before,
em[data-emoji="lotus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fab7.svg");
}

em[data-emoji=":hibiscus:"]::before,
em[data-emoji="hibiscus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33a.svg");
}

em[data-emoji=":cherry_blossom:"]::before,
em[data-emoji="cherry_blossom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f338.svg");
}

em[data-emoji=":blossom:"]::before,
em[data-emoji="blossom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33c.svg");
}

em[data-emoji=":sunflower:"]::before,
em[data-emoji="sunflower"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33b.svg");
}

em[data-emoji=":sun_with_face:"]::before,
em[data-emoji="sun_with_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31e.svg");
}

em[data-emoji=":full_moon_with_face:"]::before,
em[data-emoji="full_moon_with_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31d.svg");
}

em[data-emoji=":first_quarter_moon_with_face:"]::before,
em[data-emoji="first_quarter_moon_with_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31b.svg");
}

em[data-emoji=":last_quarter_moon_with_face:"]::before,
em[data-emoji="last_quarter_moon_with_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31c.svg");
}

em[data-emoji=":new_moon_with_face:"]::before,
em[data-emoji="new_moon_with_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31a.svg");
}

em[data-emoji=":full_moon:"]::before,
em[data-emoji="full_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f315.svg");
}

em[data-emoji=":waning_gibbous_moon:"]::before,
em[data-emoji="waning_gibbous_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f316.svg");
}

em[data-emoji=":last_quarter_moon:"]::before,
em[data-emoji="last_quarter_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f317.svg");
}

em[data-emoji=":waning_crescent_moon:"]::before,
em[data-emoji="waning_crescent_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f318.svg");
}

em[data-emoji=":new_moon:"]::before,
em[data-emoji="new_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f311.svg");
}

em[data-emoji=":waxing_crescent_moon:"]::before,
em[data-emoji="waxing_crescent_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f312.svg");
}

em[data-emoji=":first_quarter_moon:"]::before,
em[data-emoji="first_quarter_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f313.svg");
}

em[data-emoji=":waxing_gibbous_moon:"]::before,
em[data-emoji="waxing_gibbous_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f314.svg");
}

em[data-emoji=":crescent_moon:"]::before,
em[data-emoji="crescent_moon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f319.svg");
}

em[data-emoji=":earth_americas:"]::before,
em[data-emoji="earth_americas"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30e.svg");
}

em[data-emoji=":earth_africa:"]::before,
em[data-emoji="earth_africa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30d.svg");
}

em[data-emoji=":earth_asia:"]::before,
em[data-emoji="earth_asia"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30f.svg");
}

em[data-emoji=":ringed_planet:"]::before,
em[data-emoji="ringed_planet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa90.svg");
}

em[data-emoji=":dizzy:"]::before,
em[data-emoji="dizzy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ab.svg");
}

em[data-emoji=":star:"]::before,
em[data-emoji="star"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b50.svg");
}

em[data-emoji=":star2:"]::before,
em[data-emoji="star2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f31f.svg");
}

em[data-emoji=":zap:"]::before,
em[data-emoji="zap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26a1.svg");
}

em[data-emoji=":boom:"]::before,
em[data-emoji="boom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a5.svg");
}

em[data-emoji=":fire:"]::before,
em[data-emoji="fire"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f525.svg");
}

em[data-emoji=":cloud_tornado:"]::before,
em[data-emoji="cloud_tornado"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32a.svg");
}

em[data-emoji=":rainbow:"]::before,
em[data-emoji="rainbow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f308.svg");
}

em[data-emoji=":white_sun_small_cloud:"]::before,
em[data-emoji="white_sun_small_cloud"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f324.svg");
}

em[data-emoji=":partly_sunny:"]::before,
em[data-emoji="partly_sunny"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26c5.svg");
}

em[data-emoji=":white_sun_cloud:"]::before,
em[data-emoji="white_sun_cloud"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f325.svg");
}

em[data-emoji=":white_sun_rain_cloud:"]::before,
em[data-emoji="white_sun_rain_cloud"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f326.svg");
}

em[data-emoji=":cloud_rain:"]::before,
em[data-emoji="cloud_rain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f327.svg");
}

em[data-emoji=":thunder_cloud_rain:"]::before,
em[data-emoji="thunder_cloud_rain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26c8.svg");
}

em[data-emoji=":cloud_lightning:"]::before,
em[data-emoji="cloud_lightning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f329.svg");
}

em[data-emoji=":cloud_snow:"]::before,
em[data-emoji="cloud_snow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f328.svg");
}

em[data-emoji=":snowman:"]::before,
em[data-emoji="snowman"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26c4.svg");
}

em[data-emoji=":wind_blowing_face:"]::before,
em[data-emoji="wind_blowing_face"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32c.svg");
}

em[data-emoji=":dash:"]::before,
em[data-emoji="dash"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a8.svg");
}

em[data-emoji=":droplet:"]::before,
em[data-emoji="droplet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a7.svg");
}

em[data-emoji=":sweat_drops:"]::before,
em[data-emoji="sweat_drops"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a6.svg");
}

em[data-emoji=":bubbles:"]::before,
em[data-emoji="bubbles"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fae7.svg");
}

em[data-emoji=":ocean:"]::before,
em[data-emoji="ocean"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30a.svg");
}

em[data-emoji=":fog:"]::before,
em[data-emoji="fog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32b.svg");
}

em[data-emoji=":green_apple:"]::before,
em[data-emoji="green_apple"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34f.svg");
}

em[data-emoji=":apple:"]::before,
em[data-emoji="apple"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34e.svg");
}

em[data-emoji=":pear:"]::before,
em[data-emoji="pear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f350.svg");
}

em[data-emoji=":tangerine:"]::before,
em[data-emoji="tangerine"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34a.svg");
}

em[data-emoji=":lemon:"]::before,
em[data-emoji="lemon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34b.svg");
}

em[data-emoji=":banana:"]::before,
em[data-emoji="banana"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34c.svg");
}

em[data-emoji=":watermelon:"]::before,
em[data-emoji="watermelon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f349.svg");
}

em[data-emoji=":grapes:"]::before,
em[data-emoji="grapes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f347.svg");
}

em[data-emoji=":blueberries:"]::before,
em[data-emoji="blueberries"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad0.svg");
}

em[data-emoji=":strawberry:"]::before,
em[data-emoji="strawberry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f353.svg");
}

em[data-emoji=":melon:"]::before,
em[data-emoji="melon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f348.svg");
}

em[data-emoji=":cherries:"]::before,
em[data-emoji="cherries"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f352.svg");
}

em[data-emoji=":peach:"]::before,
em[data-emoji="peach"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f351.svg");
}

em[data-emoji=":mango:"]::before,
em[data-emoji="mango"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96d.svg");
}

em[data-emoji=":pineapple:"]::before,
em[data-emoji="pineapple"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f34d.svg");
}

em[data-emoji=":coconut:"]::before,
em[data-emoji="coconut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f965.svg");
}

em[data-emoji=":kiwi:"]::before,
em[data-emoji="kiwi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95d.svg");
}

em[data-emoji=":tomato:"]::before,
em[data-emoji="tomato"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f345.svg");
}

em[data-emoji=":eggplant:"]::before,
em[data-emoji="eggplant"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f346.svg");
}

em[data-emoji=":avocado:"]::before,
em[data-emoji="avocado"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f951.svg");
}

em[data-emoji=":olive:"]::before,
em[data-emoji="olive"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad2.svg");
}

em[data-emoji=":broccoli:"]::before,
em[data-emoji="broccoli"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f966.svg");
}

em[data-emoji=":leafy_green:"]::before,
em[data-emoji="leafy_green"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96c.svg");
}

em[data-emoji=":bell_pepper:"]::before,
em[data-emoji="bell_pepper"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad1.svg");
}

em[data-emoji=":cucumber:"]::before,
em[data-emoji="cucumber"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f952.svg");
}

em[data-emoji=":hot_pepper:"]::before,
em[data-emoji="hot_pepper"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f336.svg");
}

em[data-emoji=":corn:"]::before,
em[data-emoji="corn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f33d.svg");
}

em[data-emoji=":carrot:"]::before,
em[data-emoji="carrot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f955.svg");
}

em[data-emoji=":garlic:"]::before,
em[data-emoji="garlic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c4.svg");
}

em[data-emoji=":onion:"]::before,
em[data-emoji="onion"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c5.svg");
}

em[data-emoji=":potato:"]::before,
em[data-emoji="potato"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f954.svg");
}

em[data-emoji=":sweet_potato:"]::before,
em[data-emoji="sweet_potato"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f360.svg");
}

em[data-emoji=":croissant:"]::before,
em[data-emoji="croissant"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f950.svg");
}

em[data-emoji=":bagel:"]::before,
em[data-emoji="bagel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96f.svg");
}

em[data-emoji=":bread:"]::before,
em[data-emoji="bread"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35e.svg");
}

em[data-emoji=":french_bread:"]::before,
em[data-emoji="french_bread"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f956.svg");
}

em[data-emoji=":flatbread:"]::before,
em[data-emoji="flatbread"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad3.svg");
}

em[data-emoji=":pretzel:"]::before,
em[data-emoji="pretzel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f968.svg");
}

em[data-emoji=":cheese:"]::before,
em[data-emoji="cheese"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c0.svg");
}

em[data-emoji=":egg:"]::before,
em[data-emoji="egg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95a.svg");
}

em[data-emoji=":cooking:"]::before,
em[data-emoji="cooking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f373.svg");
}

em[data-emoji=":butter:"]::before,
em[data-emoji="butter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c8.svg");
}

em[data-emoji=":pancakes:"]::before,
em[data-emoji="pancakes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95e.svg");
}

em[data-emoji=":waffle:"]::before,
em[data-emoji="waffle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c7.svg");
}

em[data-emoji=":bacon:"]::before,
em[data-emoji="bacon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f953.svg");
}

em[data-emoji=":cut_of_meat:"]::before,
em[data-emoji="cut_of_meat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f969.svg");
}

em[data-emoji=":poultry_leg:"]::before,
em[data-emoji="poultry_leg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f357.svg");
}

em[data-emoji=":meat_on_bone:"]::before,
em[data-emoji="meat_on_bone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f356.svg");
}

em[data-emoji=":bone:"]::before,
em[data-emoji="bone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9b4.svg");
}

em[data-emoji=":hotdog:"]::before,
em[data-emoji="hotdog"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32d.svg");
}

em[data-emoji=":hamburger:"]::before,
em[data-emoji="hamburger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f354.svg");
}

em[data-emoji=":fries:"]::before,
em[data-emoji="fries"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35f.svg");
}

em[data-emoji=":pizza:"]::before,
em[data-emoji="pizza"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f355.svg");
}

em[data-emoji=":sandwich:"]::before,
em[data-emoji="sandwich"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96a.svg");
}

em[data-emoji=":stuffed_flatbread:"]::before,
em[data-emoji="stuffed_flatbread"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f959.svg");
}

em[data-emoji=":falafel:"]::before,
em[data-emoji="falafel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c6.svg");
}

em[data-emoji=":taco:"]::before,
em[data-emoji="taco"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32e.svg");
}

em[data-emoji=":burrito:"]::before,
em[data-emoji="burrito"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f32f.svg");
}

em[data-emoji=":tamale:"]::before,
em[data-emoji="tamale"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad4.svg");
}

em[data-emoji=":salad:"]::before,
em[data-emoji="salad"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f957.svg");
}

em[data-emoji=":shallow_pan_of_food:"]::before,
em[data-emoji="shallow_pan_of_food"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f958.svg");
}

em[data-emoji=":fondue:"]::before,
em[data-emoji="fondue"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad5.svg");
}

em[data-emoji=":canned_food:"]::before,
em[data-emoji="canned_food"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96b.svg");
}

em[data-emoji=":jar:"]::before,
em[data-emoji="jar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad9.svg");
}

em[data-emoji=":spaghetti:"]::before,
em[data-emoji="spaghetti"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35d.svg");
}

em[data-emoji=":ramen:"]::before,
em[data-emoji="ramen"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35c.svg");
}

em[data-emoji=":stew:"]::before,
em[data-emoji="stew"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f372.svg");
}

em[data-emoji=":curry:"]::before,
em[data-emoji="curry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35b.svg");
}

em[data-emoji=":sushi:"]::before,
em[data-emoji="sushi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f363.svg");
}

em[data-emoji=":bento:"]::before,
em[data-emoji="bento"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f371.svg");
}

em[data-emoji=":dumpling:"]::before,
em[data-emoji="dumpling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95f.svg");
}

em[data-emoji=":oyster:"]::before,
em[data-emoji="oyster"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9aa.svg");
}

em[data-emoji=":fried_shrimp:"]::before,
em[data-emoji="fried_shrimp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f364.svg");
}

em[data-emoji=":rice_ball:"]::before,
em[data-emoji="rice_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f359.svg");
}

em[data-emoji=":rice:"]::before,
em[data-emoji="rice"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f35a.svg");
}

em[data-emoji=":rice_cracker:"]::before,
em[data-emoji="rice_cracker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f358.svg");
}

em[data-emoji=":fish_cake:"]::before,
em[data-emoji="fish_cake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f365.svg");
}

em[data-emoji=":fortune_cookie:"]::before,
em[data-emoji="fortune_cookie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f960.svg");
}

em[data-emoji=":moon_cake:"]::before,
em[data-emoji="moon_cake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f96e.svg");
}

em[data-emoji=":oden:"]::before,
em[data-emoji="oden"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f362.svg");
}

em[data-emoji=":dango:"]::before,
em[data-emoji="dango"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f361.svg");
}

em[data-emoji=":shaved_ice:"]::before,
em[data-emoji="shaved_ice"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f367.svg");
}

em[data-emoji=":ice_cream:"]::before,
em[data-emoji="ice_cream"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f368.svg");
}

em[data-emoji=":icecream:"]::before,
em[data-emoji="icecream"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f366.svg");
}

em[data-emoji=":pie:"]::before,
em[data-emoji="pie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f967.svg");
}

em[data-emoji=":cupcake:"]::before,
em[data-emoji="cupcake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c1.svg");
}

em[data-emoji=":cake:"]::before,
em[data-emoji="cake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f370.svg");
}

em[data-emoji=":birthday:"]::before,
em[data-emoji="birthday"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f382.svg");
}

em[data-emoji=":custard:"]::before,
em[data-emoji="custard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36e.svg");
}

em[data-emoji=":lollipop:"]::before,
em[data-emoji="lollipop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36d.svg");
}

em[data-emoji=":candy:"]::before,
em[data-emoji="candy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36c.svg");
}

em[data-emoji=":chocolate_bar:"]::before,
em[data-emoji="chocolate_bar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36b.svg");
}

em[data-emoji=":popcorn:"]::before,
em[data-emoji="popcorn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37f.svg");
}

em[data-emoji=":doughnut:"]::before,
em[data-emoji="doughnut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f369.svg");
}

em[data-emoji=":cookie:"]::before,
em[data-emoji="cookie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36a.svg");
}

em[data-emoji=":chestnut:"]::before,
em[data-emoji="chestnut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f330.svg");
}

em[data-emoji=":peanuts:"]::before,
em[data-emoji="peanuts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95c.svg");
}

em[data-emoji=":beans:"]::before,
em[data-emoji="beans"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad8.svg");
}

em[data-emoji=":honey_pot:"]::before,
em[data-emoji="honey_pot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f36f.svg");
}

em[data-emoji=":milk:"]::before,
em[data-emoji="milk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f95b.svg");
}

em[data-emoji=":pouring_liquid:"]::before,
em[data-emoji="pouring_liquid"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad7.svg");
}

em[data-emoji=":baby_bottle:"]::before,
em[data-emoji="baby_bottle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37c.svg");
}

em[data-emoji=":teapot:"]::before,
em[data-emoji="teapot"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fad6.svg");
}

em[data-emoji=":tea:"]::before,
em[data-emoji="tea"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f375.svg");
}

em[data-emoji=":mate:"]::before,
em[data-emoji="mate"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c9.svg");
}

em[data-emoji=":beverage_box:"]::before,
em[data-emoji="beverage_box"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c3.svg");
}

em[data-emoji=":cup_with_straw:"]::before,
em[data-emoji="cup_with_straw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f964.svg");
}

em[data-emoji=":bubble_tea:"]::before,
em[data-emoji="bubble_tea"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9cb.svg");
}

em[data-emoji=":sake:"]::before,
em[data-emoji="sake"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f376.svg");
}

em[data-emoji=":beer:"]::before,
em[data-emoji="beer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37a.svg");
}

em[data-emoji=":beers:"]::before,
em[data-emoji="beers"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37b.svg");
}

em[data-emoji=":champagne_glass:"]::before,
em[data-emoji="champagne_glass"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f942.svg");
}

em[data-emoji=":wine_glass:"]::before,
em[data-emoji="wine_glass"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f377.svg");
}

em[data-emoji=":tumbler_glass:"]::before,
em[data-emoji="tumbler_glass"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f943.svg");
}

em[data-emoji=":cocktail:"]::before,
em[data-emoji="cocktail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f378.svg");
}

em[data-emoji=":tropical_drink:"]::before,
em[data-emoji="tropical_drink"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f379.svg");
}

em[data-emoji=":champagne:"]::before,
em[data-emoji="champagne"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37e.svg");
}

em[data-emoji=":ice_cube:"]::before,
em[data-emoji="ice_cube"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ca.svg");
}

em[data-emoji=":spoon:"]::before,
em[data-emoji="spoon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f944.svg");
}

em[data-emoji=":fork_and_knife:"]::before,
em[data-emoji="fork_and_knife"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f374.svg");
}

em[data-emoji=":fork_knife_plate:"]::before,
em[data-emoji="fork_knife_plate"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f37d.svg");
}

em[data-emoji=":bowl_with_spoon:"]::before,
em[data-emoji="bowl_with_spoon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f963.svg");
}

em[data-emoji=":takeout_box:"]::before,
em[data-emoji="takeout_box"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f961.svg");
}

em[data-emoji=":chopsticks:"]::before,
em[data-emoji="chopsticks"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f962.svg");
}

em[data-emoji=":salt:"]::before,
em[data-emoji="salt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9c2.svg");
}

em[data-emoji=":soccer:"]::before,
em[data-emoji="soccer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26bd.svg");
}

em[data-emoji=":basketball:"]::before,
em[data-emoji="basketball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c0.svg");
}

em[data-emoji=":football:"]::before,
em[data-emoji="football"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c8.svg");
}

em[data-emoji=":baseball:"]::before,
em[data-emoji="baseball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26be.svg");
}

em[data-emoji=":softball:"]::before,
em[data-emoji="softball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94e.svg");
}

em[data-emoji=":tennis:"]::before,
em[data-emoji="tennis"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3be.svg");
}

em[data-emoji=":volleyball:"]::before,
em[data-emoji="volleyball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d0.svg");
}

em[data-emoji=":rugby_football:"]::before,
em[data-emoji="rugby_football"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c9.svg");
}

em[data-emoji=":flying_disc:"]::before,
em[data-emoji="flying_disc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94f.svg");
}

em[data-emoji=":8ball:"]::before,
em[data-emoji="8ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b1.svg");
}

em[data-emoji=":yo_yo:"]::before,
em[data-emoji="yo_yo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa80.svg");
}

em[data-emoji=":ping_pong:"]::before,
em[data-emoji="ping_pong"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d3.svg");
}

em[data-emoji=":badminton:"]::before,
em[data-emoji="badminton"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f8.svg");
}

em[data-emoji=":hockey:"]::before,
em[data-emoji="hockey"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d2.svg");
}

em[data-emoji=":field_hockey:"]::before,
em[data-emoji="field_hockey"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d1.svg");
}

em[data-emoji=":lacrosse:"]::before,
em[data-emoji="lacrosse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94d.svg");
}

em[data-emoji=":cricket_game:"]::before,
em[data-emoji="cricket_game"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cf.svg");
}

em[data-emoji=":boomerang:"]::before,
em[data-emoji="boomerang"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa83.svg");
}

em[data-emoji=":goal:"]::before,
em[data-emoji="goal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f945.svg");
}

em[data-emoji=":golf:"]::before,
em[data-emoji="golf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f3.svg");
}

em[data-emoji=":kite:"]::before,
em[data-emoji="kite"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa81.svg");
}

em[data-emoji=":playground_slide:"]::before,
em[data-emoji="playground_slide"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6dd.svg");
}

em[data-emoji=":bow_and_arrow:"]::before,
em[data-emoji="bow_and_arrow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f9.svg");
}

em[data-emoji=":fishing_pole_and_fish:"]::before,
em[data-emoji="fishing_pole_and_fish"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a3.svg");
}

em[data-emoji=":diving_mask:"]::before,
em[data-emoji="diving_mask"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93f.svg");
}

em[data-emoji=":boxing_glove:"]::before,
em[data-emoji="boxing_glove"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94a.svg");
}

em[data-emoji=":martial_arts_uniform:"]::before,
em[data-emoji="martial_arts_uniform"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94b.svg");
}

em[data-emoji=":running_shirt_with_sash:"]::before,
em[data-emoji="running_shirt_with_sash"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3bd.svg");
}

em[data-emoji=":skateboard:"]::before,
em[data-emoji="skateboard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f9.svg");
}

em[data-emoji=":roller_skate:"]::before,
em[data-emoji="roller_skate"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6fc.svg");
}

em[data-emoji=":sled:"]::before,
em[data-emoji="sled"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f7.svg");
}

em[data-emoji=":ice_skate:"]::before,
em[data-emoji="ice_skate"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f8.svg");
}

em[data-emoji=":curling_stone:"]::before,
em[data-emoji="curling_stone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f94c.svg");
}

em[data-emoji=":ski:"]::before,
em[data-emoji="ski"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3bf.svg");
}

em[data-emoji=":skier:"]::before,
em[data-emoji="skier"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f7.svg");
}

em[data-emoji=":snowboarder:"]::before,
em[data-emoji="snowboarder"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2.svg");
}

em[data-emoji=":snowboarder_tone1:"]::before,
em[data-emoji="snowboarder_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2-1f3fb.svg");
}

em[data-emoji=":snowboarder_tone2:"]::before,
em[data-emoji="snowboarder_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2-1f3fc.svg");
}

em[data-emoji=":snowboarder_tone3:"]::before,
em[data-emoji="snowboarder_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2-1f3fd.svg");
}

em[data-emoji=":snowboarder_tone4:"]::before,
em[data-emoji="snowboarder_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2-1f3fe.svg");
}

em[data-emoji=":snowboarder_tone5:"]::before,
em[data-emoji="snowboarder_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c2-1f3ff.svg");
}

em[data-emoji=":parachute:"]::before,
em[data-emoji="parachute"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa82.svg");
}

em[data-emoji=":person_lifting_weights:"]::before,
em[data-emoji="person_lifting_weights"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb.svg");
}

em[data-emoji=":person_lifting_weights_tone1:"]::before,
em[data-emoji="person_lifting_weights_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fb.svg");
}

em[data-emoji=":person_lifting_weights_tone2:"]::before,
em[data-emoji="person_lifting_weights_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fc.svg");
}

em[data-emoji=":person_lifting_weights_tone3:"]::before,
em[data-emoji="person_lifting_weights_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fd.svg");
}

em[data-emoji=":person_lifting_weights_tone4:"]::before,
em[data-emoji="person_lifting_weights_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fe.svg");
}

em[data-emoji=":person_lifting_weights_tone5:"]::before,
em[data-emoji="person_lifting_weights_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3ff.svg");
}

em[data-emoji=":woman_lifting_weights:"]::before,
em[data-emoji="woman_lifting_weights"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-fe0f-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_lifting_weights_tone1:"]::before,
em[data-emoji="woman_lifting_weights_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_lifting_weights_tone2:"]::before,
em[data-emoji="woman_lifting_weights_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_lifting_weights_tone3:"]::before,
em[data-emoji="woman_lifting_weights_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_lifting_weights_tone4:"]::before,
em[data-emoji="woman_lifting_weights_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_lifting_weights_tone5:"]::before,
em[data-emoji="woman_lifting_weights_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_lifting_weights:"]::before,
em[data-emoji="man_lifting_weights"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-fe0f-200d-2642-fe0f.svg");
}

em[data-emoji=":man_lifting_weights_tone1:"]::before,
em[data-emoji="man_lifting_weights_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_lifting_weights_tone2:"]::before,
em[data-emoji="man_lifting_weights_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_lifting_weights_tone3:"]::before,
em[data-emoji="man_lifting_weights_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_lifting_weights_tone4:"]::before,
em[data-emoji="man_lifting_weights_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_lifting_weights_tone5:"]::before,
em[data-emoji="man_lifting_weights_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cb-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":people_wrestling:"]::before,
em[data-emoji="people_wrestling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93c.svg");
}

em[data-emoji=":women_wrestling:"]::before,
em[data-emoji="women_wrestling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93c-200d-2640-fe0f.svg");
}

em[data-emoji=":men_wrestling:"]::before,
em[data-emoji="men_wrestling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93c-200d-2642-fe0f.svg");
}

em[data-emoji=":person_doing_cartwheel:"]::before,
em[data-emoji="person_doing_cartwheel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938.svg");
}

em[data-emoji=":person_doing_cartwheel_tone1:"]::before,
em[data-emoji="person_doing_cartwheel_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fb.svg");
}

em[data-emoji=":person_doing_cartwheel_tone2:"]::before,
em[data-emoji="person_doing_cartwheel_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fc.svg");
}

em[data-emoji=":person_doing_cartwheel_tone3:"]::before,
em[data-emoji="person_doing_cartwheel_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fd.svg");
}

em[data-emoji=":person_doing_cartwheel_tone4:"]::before,
em[data-emoji="person_doing_cartwheel_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fe.svg");
}

em[data-emoji=":person_doing_cartwheel_tone5:"]::before,
em[data-emoji="person_doing_cartwheel_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3ff.svg");
}

em[data-emoji=":woman_cartwheeling:"]::before,
em[data-emoji="woman_cartwheeling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_cartwheeling_tone1:"]::before,
em[data-emoji="woman_cartwheeling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_cartwheeling_tone2:"]::before,
em[data-emoji="woman_cartwheeling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_cartwheeling_tone3:"]::before,
em[data-emoji="woman_cartwheeling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_cartwheeling_tone4:"]::before,
em[data-emoji="woman_cartwheeling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_cartwheeling_tone5:"]::before,
em[data-emoji="woman_cartwheeling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_cartwheeling:"]::before,
em[data-emoji="man_cartwheeling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-200d-2642-fe0f.svg");
}

em[data-emoji=":man_cartwheeling_tone1:"]::before,
em[data-emoji="man_cartwheeling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_cartwheeling_tone2:"]::before,
em[data-emoji="man_cartwheeling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_cartwheeling_tone3:"]::before,
em[data-emoji="man_cartwheeling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_cartwheeling_tone4:"]::before,
em[data-emoji="man_cartwheeling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_cartwheeling_tone5:"]::before,
em[data-emoji="man_cartwheeling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f938-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_bouncing_ball:"]::before,
em[data-emoji="person_bouncing_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9.svg");
}

em[data-emoji=":person_bouncing_ball_tone1:"]::before,
em[data-emoji="person_bouncing_ball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fb.svg");
}

em[data-emoji=":person_bouncing_ball_tone2:"]::before,
em[data-emoji="person_bouncing_ball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fc.svg");
}

em[data-emoji=":person_bouncing_ball_tone3:"]::before,
em[data-emoji="person_bouncing_ball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fd.svg");
}

em[data-emoji=":person_bouncing_ball_tone4:"]::before,
em[data-emoji="person_bouncing_ball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fe.svg");
}

em[data-emoji=":person_bouncing_ball_tone5:"]::before,
em[data-emoji="person_bouncing_ball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3ff.svg");
}

em[data-emoji=":woman_bouncing_ball:"]::before,
em[data-emoji="woman_bouncing_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-fe0f-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bouncing_ball_tone1:"]::before,
em[data-emoji="woman_bouncing_ball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bouncing_ball_tone2:"]::before,
em[data-emoji="woman_bouncing_ball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bouncing_ball_tone3:"]::before,
em[data-emoji="woman_bouncing_ball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bouncing_ball_tone4:"]::before,
em[data-emoji="woman_bouncing_ball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_bouncing_ball_tone5:"]::before,
em[data-emoji="woman_bouncing_ball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball:"]::before,
em[data-emoji="man_bouncing_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-fe0f-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball_tone1:"]::before,
em[data-emoji="man_bouncing_ball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball_tone2:"]::before,
em[data-emoji="man_bouncing_ball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball_tone3:"]::before,
em[data-emoji="man_bouncing_ball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball_tone4:"]::before,
em[data-emoji="man_bouncing_ball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_bouncing_ball_tone5:"]::before,
em[data-emoji="man_bouncing_ball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f9-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_fencing:"]::before,
em[data-emoji="person_fencing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93a.svg");
}

em[data-emoji=":person_playing_handball:"]::before,
em[data-emoji="person_playing_handball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e.svg");
}

em[data-emoji=":person_playing_handball_tone1:"]::before,
em[data-emoji="person_playing_handball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fb.svg");
}

em[data-emoji=":person_playing_handball_tone2:"]::before,
em[data-emoji="person_playing_handball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fc.svg");
}

em[data-emoji=":person_playing_handball_tone3:"]::before,
em[data-emoji="person_playing_handball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fd.svg");
}

em[data-emoji=":person_playing_handball_tone4:"]::before,
em[data-emoji="person_playing_handball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fe.svg");
}

em[data-emoji=":person_playing_handball_tone5:"]::before,
em[data-emoji="person_playing_handball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3ff.svg");
}

em[data-emoji=":woman_playing_handball:"]::before,
em[data-emoji="woman_playing_handball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_handball_tone1:"]::before,
em[data-emoji="woman_playing_handball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_handball_tone2:"]::before,
em[data-emoji="woman_playing_handball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_handball_tone3:"]::before,
em[data-emoji="woman_playing_handball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_handball_tone4:"]::before,
em[data-emoji="woman_playing_handball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_handball_tone5:"]::before,
em[data-emoji="woman_playing_handball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_playing_handball:"]::before,
em[data-emoji="man_playing_handball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_handball_tone1:"]::before,
em[data-emoji="man_playing_handball_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_handball_tone2:"]::before,
em[data-emoji="man_playing_handball_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_handball_tone3:"]::before,
em[data-emoji="man_playing_handball_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_handball_tone4:"]::before,
em[data-emoji="man_playing_handball_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_handball_tone5:"]::before,
em[data-emoji="man_playing_handball_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93e-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_golfing:"]::before,
em[data-emoji="person_golfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc.svg");
}

em[data-emoji=":person_golfing_tone1:"]::before,
em[data-emoji="person_golfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fb.svg");
}

em[data-emoji=":person_golfing_tone2:"]::before,
em[data-emoji="person_golfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fc.svg");
}

em[data-emoji=":person_golfing_tone3:"]::before,
em[data-emoji="person_golfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fd.svg");
}

em[data-emoji=":person_golfing_tone4:"]::before,
em[data-emoji="person_golfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fe.svg");
}

em[data-emoji=":person_golfing_tone5:"]::before,
em[data-emoji="person_golfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3ff.svg");
}

em[data-emoji=":woman_golfing:"]::before,
em[data-emoji="woman_golfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-fe0f-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_golfing_tone1:"]::before,
em[data-emoji="woman_golfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_golfing_tone2:"]::before,
em[data-emoji="woman_golfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_golfing_tone3:"]::before,
em[data-emoji="woman_golfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_golfing_tone4:"]::before,
em[data-emoji="woman_golfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_golfing_tone5:"]::before,
em[data-emoji="woman_golfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_golfing:"]::before,
em[data-emoji="man_golfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-fe0f-200d-2642-fe0f.svg");
}

em[data-emoji=":man_golfing_tone1:"]::before,
em[data-emoji="man_golfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_golfing_tone2:"]::before,
em[data-emoji="man_golfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_golfing_tone3:"]::before,
em[data-emoji="man_golfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_golfing_tone4:"]::before,
em[data-emoji="man_golfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_golfing_tone5:"]::before,
em[data-emoji="man_golfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cc-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":horse_racing:"]::before,
em[data-emoji="horse_racing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7.svg");
}

em[data-emoji=":horse_racing_tone1:"]::before,
em[data-emoji="horse_racing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7-1f3fb.svg");
}

em[data-emoji=":horse_racing_tone2:"]::before,
em[data-emoji="horse_racing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7-1f3fc.svg");
}

em[data-emoji=":horse_racing_tone3:"]::before,
em[data-emoji="horse_racing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7-1f3fd.svg");
}

em[data-emoji=":horse_racing_tone4:"]::before,
em[data-emoji="horse_racing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7-1f3fe.svg");
}

em[data-emoji=":horse_racing_tone5:"]::before,
em[data-emoji="horse_racing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c7-1f3ff.svg");
}

em[data-emoji=":person_in_lotus_position:"]::before,
em[data-emoji="person_in_lotus_position"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8.svg");
}

em[data-emoji=":person_in_lotus_position_tone1:"]::before,
em[data-emoji="person_in_lotus_position_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fb.svg");
}

em[data-emoji=":person_in_lotus_position_tone2:"]::before,
em[data-emoji="person_in_lotus_position_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fc.svg");
}

em[data-emoji=":person_in_lotus_position_tone3:"]::before,
em[data-emoji="person_in_lotus_position_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fd.svg");
}

em[data-emoji=":person_in_lotus_position_tone4:"]::before,
em[data-emoji="person_in_lotus_position_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fe.svg");
}

em[data-emoji=":person_in_lotus_position_tone5:"]::before,
em[data-emoji="person_in_lotus_position_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3ff.svg");
}

em[data-emoji=":woman_in_lotus_position:"]::before,
em[data-emoji="woman_in_lotus_position"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_lotus_position_tone1:"]::before,
em[data-emoji="woman_in_lotus_position_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_lotus_position_tone2:"]::before,
em[data-emoji="woman_in_lotus_position_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_lotus_position_tone3:"]::before,
em[data-emoji="woman_in_lotus_position_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_lotus_position_tone4:"]::before,
em[data-emoji="woman_in_lotus_position_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_in_lotus_position_tone5:"]::before,
em[data-emoji="woman_in_lotus_position_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position:"]::before,
em[data-emoji="man_in_lotus_position"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position_tone1:"]::before,
em[data-emoji="man_in_lotus_position_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position_tone2:"]::before,
em[data-emoji="man_in_lotus_position_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position_tone3:"]::before,
em[data-emoji="man_in_lotus_position_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position_tone4:"]::before,
em[data-emoji="man_in_lotus_position_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_in_lotus_position_tone5:"]::before,
em[data-emoji="man_in_lotus_position_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d8-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_surfing:"]::before,
em[data-emoji="person_surfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4.svg");
}

em[data-emoji=":person_surfing_tone1:"]::before,
em[data-emoji="person_surfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fb.svg");
}

em[data-emoji=":person_surfing_tone2:"]::before,
em[data-emoji="person_surfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fc.svg");
}

em[data-emoji=":person_surfing_tone3:"]::before,
em[data-emoji="person_surfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fd.svg");
}

em[data-emoji=":person_surfing_tone4:"]::before,
em[data-emoji="person_surfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fe.svg");
}

em[data-emoji=":person_surfing_tone5:"]::before,
em[data-emoji="person_surfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3ff.svg");
}

em[data-emoji=":woman_surfing:"]::before,
em[data-emoji="woman_surfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_surfing_tone1:"]::before,
em[data-emoji="woman_surfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_surfing_tone2:"]::before,
em[data-emoji="woman_surfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_surfing_tone3:"]::before,
em[data-emoji="woman_surfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_surfing_tone4:"]::before,
em[data-emoji="woman_surfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_surfing_tone5:"]::before,
em[data-emoji="woman_surfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_surfing:"]::before,
em[data-emoji="man_surfing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-200d-2642-fe0f.svg");
}

em[data-emoji=":man_surfing_tone1:"]::before,
em[data-emoji="man_surfing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_surfing_tone2:"]::before,
em[data-emoji="man_surfing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_surfing_tone3:"]::before,
em[data-emoji="man_surfing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_surfing_tone4:"]::before,
em[data-emoji="man_surfing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_surfing_tone5:"]::before,
em[data-emoji="man_surfing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c4-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_swimming:"]::before,
em[data-emoji="person_swimming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca.svg");
}

em[data-emoji=":person_swimming_tone1:"]::before,
em[data-emoji="person_swimming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fb.svg");
}

em[data-emoji=":person_swimming_tone2:"]::before,
em[data-emoji="person_swimming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fc.svg");
}

em[data-emoji=":person_swimming_tone3:"]::before,
em[data-emoji="person_swimming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fd.svg");
}

em[data-emoji=":person_swimming_tone4:"]::before,
em[data-emoji="person_swimming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fe.svg");
}

em[data-emoji=":person_swimming_tone5:"]::before,
em[data-emoji="person_swimming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3ff.svg");
}

em[data-emoji=":woman_swimming:"]::before,
em[data-emoji="woman_swimming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_swimming_tone1:"]::before,
em[data-emoji="woman_swimming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_swimming_tone2:"]::before,
em[data-emoji="woman_swimming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_swimming_tone3:"]::before,
em[data-emoji="woman_swimming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_swimming_tone4:"]::before,
em[data-emoji="woman_swimming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_swimming_tone5:"]::before,
em[data-emoji="woman_swimming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_swimming:"]::before,
em[data-emoji="man_swimming"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-200d-2642-fe0f.svg");
}

em[data-emoji=":man_swimming_tone1:"]::before,
em[data-emoji="man_swimming_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_swimming_tone2:"]::before,
em[data-emoji="man_swimming_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_swimming_tone3:"]::before,
em[data-emoji="man_swimming_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_swimming_tone4:"]::before,
em[data-emoji="man_swimming_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_swimming_tone5:"]::before,
em[data-emoji="man_swimming_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ca-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_playing_water_polo:"]::before,
em[data-emoji="person_playing_water_polo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d.svg");
}

em[data-emoji=":person_playing_water_polo_tone1:"]::before,
em[data-emoji="person_playing_water_polo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fb.svg");
}

em[data-emoji=":person_playing_water_polo_tone2:"]::before,
em[data-emoji="person_playing_water_polo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fc.svg");
}

em[data-emoji=":person_playing_water_polo_tone3:"]::before,
em[data-emoji="person_playing_water_polo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fd.svg");
}

em[data-emoji=":person_playing_water_polo_tone4:"]::before,
em[data-emoji="person_playing_water_polo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fe.svg");
}

em[data-emoji=":person_playing_water_polo_tone5:"]::before,
em[data-emoji="person_playing_water_polo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3ff.svg");
}

em[data-emoji=":woman_playing_water_polo:"]::before,
em[data-emoji="woman_playing_water_polo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_water_polo_tone1:"]::before,
em[data-emoji="woman_playing_water_polo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_water_polo_tone2:"]::before,
em[data-emoji="woman_playing_water_polo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_water_polo_tone3:"]::before,
em[data-emoji="woman_playing_water_polo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_water_polo_tone4:"]::before,
em[data-emoji="woman_playing_water_polo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_playing_water_polo_tone5:"]::before,
em[data-emoji="woman_playing_water_polo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo:"]::before,
em[data-emoji="man_playing_water_polo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo_tone1:"]::before,
em[data-emoji="man_playing_water_polo_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo_tone2:"]::before,
em[data-emoji="man_playing_water_polo_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo_tone3:"]::before,
em[data-emoji="man_playing_water_polo_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo_tone4:"]::before,
em[data-emoji="man_playing_water_polo_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_playing_water_polo_tone5:"]::before,
em[data-emoji="man_playing_water_polo_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f93d-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_rowing_boat:"]::before,
em[data-emoji="person_rowing_boat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3.svg");
}

em[data-emoji=":person_rowing_boat_tone1:"]::before,
em[data-emoji="person_rowing_boat_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fb.svg");
}

em[data-emoji=":person_rowing_boat_tone2:"]::before,
em[data-emoji="person_rowing_boat_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fc.svg");
}

em[data-emoji=":person_rowing_boat_tone3:"]::before,
em[data-emoji="person_rowing_boat_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fd.svg");
}

em[data-emoji=":person_rowing_boat_tone4:"]::before,
em[data-emoji="person_rowing_boat_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fe.svg");
}

em[data-emoji=":person_rowing_boat_tone5:"]::before,
em[data-emoji="person_rowing_boat_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3ff.svg");
}

em[data-emoji=":woman_rowing_boat:"]::before,
em[data-emoji="woman_rowing_boat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_rowing_boat_tone1:"]::before,
em[data-emoji="woman_rowing_boat_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_rowing_boat_tone2:"]::before,
em[data-emoji="woman_rowing_boat_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_rowing_boat_tone3:"]::before,
em[data-emoji="woman_rowing_boat_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_rowing_boat_tone4:"]::before,
em[data-emoji="woman_rowing_boat_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_rowing_boat_tone5:"]::before,
em[data-emoji="woman_rowing_boat_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_rowing_boat:"]::before,
em[data-emoji="man_rowing_boat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-200d-2642-fe0f.svg");
}

em[data-emoji=":man_rowing_boat_tone1:"]::before,
em[data-emoji="man_rowing_boat_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_rowing_boat_tone2:"]::before,
em[data-emoji="man_rowing_boat_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_rowing_boat_tone3:"]::before,
em[data-emoji="man_rowing_boat_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_rowing_boat_tone4:"]::before,
em[data-emoji="man_rowing_boat_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_rowing_boat_tone5:"]::before,
em[data-emoji="man_rowing_boat_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a3-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_climbing:"]::before,
em[data-emoji="person_climbing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7.svg");
}

em[data-emoji=":person_climbing_tone1:"]::before,
em[data-emoji="person_climbing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fb.svg");
}

em[data-emoji=":person_climbing_tone2:"]::before,
em[data-emoji="person_climbing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fc.svg");
}

em[data-emoji=":person_climbing_tone3:"]::before,
em[data-emoji="person_climbing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fd.svg");
}

em[data-emoji=":person_climbing_tone4:"]::before,
em[data-emoji="person_climbing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fe.svg");
}

em[data-emoji=":person_climbing_tone5:"]::before,
em[data-emoji="person_climbing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3ff.svg");
}

em[data-emoji=":woman_climbing:"]::before,
em[data-emoji="woman_climbing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_climbing_tone1:"]::before,
em[data-emoji="woman_climbing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_climbing_tone2:"]::before,
em[data-emoji="woman_climbing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_climbing_tone3:"]::before,
em[data-emoji="woman_climbing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_climbing_tone4:"]::before,
em[data-emoji="woman_climbing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_climbing_tone5:"]::before,
em[data-emoji="woman_climbing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_climbing:"]::before,
em[data-emoji="man_climbing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-200d-2642-fe0f.svg");
}

em[data-emoji=":man_climbing_tone1:"]::before,
em[data-emoji="man_climbing_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_climbing_tone2:"]::before,
em[data-emoji="man_climbing_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_climbing_tone3:"]::before,
em[data-emoji="man_climbing_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_climbing_tone4:"]::before,
em[data-emoji="man_climbing_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_climbing_tone5:"]::before,
em[data-emoji="man_climbing_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9d7-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_mountain_biking:"]::before,
em[data-emoji="person_mountain_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5.svg");
}

em[data-emoji=":person_mountain_biking_tone1:"]::before,
em[data-emoji="person_mountain_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fb.svg");
}

em[data-emoji=":person_mountain_biking_tone2:"]::before,
em[data-emoji="person_mountain_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fc.svg");
}

em[data-emoji=":person_mountain_biking_tone3:"]::before,
em[data-emoji="person_mountain_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fd.svg");
}

em[data-emoji=":person_mountain_biking_tone4:"]::before,
em[data-emoji="person_mountain_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fe.svg");
}

em[data-emoji=":person_mountain_biking_tone5:"]::before,
em[data-emoji="person_mountain_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3ff.svg");
}

em[data-emoji=":woman_mountain_biking:"]::before,
em[data-emoji="woman_mountain_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mountain_biking_tone1:"]::before,
em[data-emoji="woman_mountain_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mountain_biking_tone2:"]::before,
em[data-emoji="woman_mountain_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mountain_biking_tone3:"]::before,
em[data-emoji="woman_mountain_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mountain_biking_tone4:"]::before,
em[data-emoji="woman_mountain_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_mountain_biking_tone5:"]::before,
em[data-emoji="woman_mountain_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_mountain_biking:"]::before,
em[data-emoji="man_mountain_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mountain_biking_tone1:"]::before,
em[data-emoji="man_mountain_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mountain_biking_tone2:"]::before,
em[data-emoji="man_mountain_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mountain_biking_tone3:"]::before,
em[data-emoji="man_mountain_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mountain_biking_tone4:"]::before,
em[data-emoji="man_mountain_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_mountain_biking_tone5:"]::before,
em[data-emoji="man_mountain_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b5-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":person_biking:"]::before,
em[data-emoji="person_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4.svg");
}

em[data-emoji=":person_biking_tone1:"]::before,
em[data-emoji="person_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fb.svg");
}

em[data-emoji=":person_biking_tone2:"]::before,
em[data-emoji="person_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fc.svg");
}

em[data-emoji=":person_biking_tone3:"]::before,
em[data-emoji="person_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fd.svg");
}

em[data-emoji=":person_biking_tone4:"]::before,
em[data-emoji="person_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fe.svg");
}

em[data-emoji=":person_biking_tone5:"]::before,
em[data-emoji="person_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3ff.svg");
}

em[data-emoji=":woman_biking:"]::before,
em[data-emoji="woman_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_biking_tone1:"]::before,
em[data-emoji="woman_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_biking_tone2:"]::before,
em[data-emoji="woman_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_biking_tone3:"]::before,
em[data-emoji="woman_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_biking_tone4:"]::before,
em[data-emoji="woman_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_biking_tone5:"]::before,
em[data-emoji="woman_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_biking:"]::before,
em[data-emoji="man_biking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-200d-2642-fe0f.svg");
}

em[data-emoji=":man_biking_tone1:"]::before,
em[data-emoji="man_biking_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_biking_tone2:"]::before,
em[data-emoji="man_biking_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_biking_tone3:"]::before,
em[data-emoji="man_biking_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_biking_tone4:"]::before,
em[data-emoji="man_biking_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_biking_tone5:"]::before,
em[data-emoji="man_biking_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b4-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":trophy:"]::before,
em[data-emoji="trophy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c6.svg");
}

em[data-emoji=":first_place:"]::before,
em[data-emoji="first_place"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f947.svg");
}

em[data-emoji=":second_place:"]::before,
em[data-emoji="second_place"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f948.svg");
}

em[data-emoji=":third_place:"]::before,
em[data-emoji="third_place"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f949.svg");
}

em[data-emoji=":medal:"]::before,
em[data-emoji="medal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c5.svg");
}

em[data-emoji=":military_medal:"]::before,
em[data-emoji="military_medal"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f396.svg");
}

em[data-emoji=":rosette:"]::before,
em[data-emoji="rosette"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f5.svg");
}

em[data-emoji=":reminder_ribbon:"]::before,
em[data-emoji="reminder_ribbon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f397.svg");
}

em[data-emoji=":ticket:"]::before,
em[data-emoji="ticket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ab.svg");
}

em[data-emoji=":tickets:"]::before,
em[data-emoji="tickets"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f39f.svg");
}

em[data-emoji=":circus_tent:"]::before,
em[data-emoji="circus_tent"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3aa.svg");
}

em[data-emoji=":person_juggling:"]::before,
em[data-emoji="person_juggling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939.svg");
}

em[data-emoji=":person_juggling_tone1:"]::before,
em[data-emoji="person_juggling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fb.svg");
}

em[data-emoji=":person_juggling_tone2:"]::before,
em[data-emoji="person_juggling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fc.svg");
}

em[data-emoji=":person_juggling_tone3:"]::before,
em[data-emoji="person_juggling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fd.svg");
}

em[data-emoji=":person_juggling_tone4:"]::before,
em[data-emoji="person_juggling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fe.svg");
}

em[data-emoji=":person_juggling_tone5:"]::before,
em[data-emoji="person_juggling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3ff.svg");
}

em[data-emoji=":woman_juggling:"]::before,
em[data-emoji="woman_juggling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_juggling_tone1:"]::before,
em[data-emoji="woman_juggling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fb-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_juggling_tone2:"]::before,
em[data-emoji="woman_juggling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fc-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_juggling_tone3:"]::before,
em[data-emoji="woman_juggling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fd-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_juggling_tone4:"]::before,
em[data-emoji="woman_juggling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fe-200d-2640-fe0f.svg");
}

em[data-emoji=":woman_juggling_tone5:"]::before,
em[data-emoji="woman_juggling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3ff-200d-2640-fe0f.svg");
}

em[data-emoji=":man_juggling:"]::before,
em[data-emoji="man_juggling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-200d-2642-fe0f.svg");
}

em[data-emoji=":man_juggling_tone1:"]::before,
em[data-emoji="man_juggling_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fb-200d-2642-fe0f.svg");
}

em[data-emoji=":man_juggling_tone2:"]::before,
em[data-emoji="man_juggling_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fc-200d-2642-fe0f.svg");
}

em[data-emoji=":man_juggling_tone3:"]::before,
em[data-emoji="man_juggling_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fd-200d-2642-fe0f.svg");
}

em[data-emoji=":man_juggling_tone4:"]::before,
em[data-emoji="man_juggling_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3fe-200d-2642-fe0f.svg");
}

em[data-emoji=":man_juggling_tone5:"]::before,
em[data-emoji="man_juggling_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f939-1f3ff-200d-2642-fe0f.svg");
}

em[data-emoji=":performing_arts:"]::before,
em[data-emoji="performing_arts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ad.svg");
}

em[data-emoji=":ballet_shoes:"]::before,
em[data-emoji="ballet_shoes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa70.svg");
}

em[data-emoji=":art:"]::before,
em[data-emoji="art"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a8.svg");
}

em[data-emoji=":clapper:"]::before,
em[data-emoji="clapper"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ac.svg");
}

em[data-emoji=":microphone:"]::before,
em[data-emoji="microphone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a4.svg");
}

em[data-emoji=":headphones:"]::before,
em[data-emoji="headphones"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a7.svg");
}

em[data-emoji=":musical_score:"]::before,
em[data-emoji="musical_score"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3bc.svg");
}

em[data-emoji=":musical_keyboard:"]::before,
em[data-emoji="musical_keyboard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b9.svg");
}

em[data-emoji=":drum:"]::before,
em[data-emoji="drum"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f941.svg");
}

em[data-emoji=":long_drum:"]::before,
em[data-emoji="long_drum"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa98.svg");
}

em[data-emoji=":saxophone:"]::before,
em[data-emoji="saxophone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b7.svg");
}

em[data-emoji=":trumpet:"]::before,
em[data-emoji="trumpet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ba.svg");
}

em[data-emoji=":accordion:"]::before,
em[data-emoji="accordion"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa97.svg");
}

em[data-emoji=":guitar:"]::before,
em[data-emoji="guitar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b8.svg");
}

em[data-emoji=":banjo:"]::before,
em[data-emoji="banjo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa95.svg");
}

em[data-emoji=":violin:"]::before,
em[data-emoji="violin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3bb.svg");
}

em[data-emoji=":game_die:"]::before,
em[data-emoji="game_die"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b2.svg");
}

em[data-emoji=":chess_pawn:"]::before,
em[data-emoji="chess_pawn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/265f.svg");
}

em[data-emoji=":dart:"]::before,
em[data-emoji="dart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3af.svg");
}

em[data-emoji=":bowling:"]::before,
em[data-emoji="bowling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b3.svg");
}

em[data-emoji=":video_game:"]::before,
em[data-emoji="video_game"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ae.svg");
}

em[data-emoji=":slot_machine:"]::before,
em[data-emoji="slot_machine"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b0.svg");
}

em[data-emoji=":jigsaw:"]::before,
em[data-emoji="jigsaw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e9.svg");
}

em[data-emoji=":red_car:"]::before,
em[data-emoji="red_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f697.svg");
}

em[data-emoji=":taxi:"]::before,
em[data-emoji="taxi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f695.svg");
}

em[data-emoji=":blue_car:"]::before,
em[data-emoji="blue_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f699.svg");
}

em[data-emoji=":pickup_truck:"]::before,
em[data-emoji="pickup_truck"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6fb.svg");
}

em[data-emoji=":bus:"]::before,
em[data-emoji="bus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68c.svg");
}

em[data-emoji=":trolleybus:"]::before,
em[data-emoji="trolleybus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68e.svg");
}

em[data-emoji=":race_car:"]::before,
em[data-emoji="race_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ce.svg");
}

em[data-emoji=":police_car:"]::before,
em[data-emoji="police_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f693.svg");
}

em[data-emoji=":ambulance:"]::before,
em[data-emoji="ambulance"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f691.svg");
}

em[data-emoji=":fire_engine:"]::before,
em[data-emoji="fire_engine"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f692.svg");
}

em[data-emoji=":minibus:"]::before,
em[data-emoji="minibus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f690.svg");
}

em[data-emoji=":truck:"]::before,
em[data-emoji="truck"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69a.svg");
}

em[data-emoji=":articulated_lorry:"]::before,
em[data-emoji="articulated_lorry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69b.svg");
}

em[data-emoji=":tractor:"]::before,
em[data-emoji="tractor"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69c.svg");
}

em[data-emoji=":probing_cane:"]::before,
em[data-emoji="probing_cane"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9af.svg");
}

em[data-emoji=":crutch:"]::before,
em[data-emoji="crutch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa7c.svg");
}

em[data-emoji=":manual_wheelchair:"]::before,
em[data-emoji="manual_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bd.svg");
}

em[data-emoji=":motorized_wheelchair:"]::before,
em[data-emoji="motorized_wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9bc.svg");
}

em[data-emoji=":scooter:"]::before,
em[data-emoji="scooter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f4.svg");
}

em[data-emoji=":bike:"]::before,
em[data-emoji="bike"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b2.svg");
}

em[data-emoji=":motor_scooter:"]::before,
em[data-emoji="motor_scooter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f5.svg");
}

em[data-emoji=":motorcycle:"]::before,
em[data-emoji="motorcycle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3cd.svg");
}

em[data-emoji=":auto_rickshaw:"]::before,
em[data-emoji="auto_rickshaw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6fa.svg");
}

em[data-emoji=":wheel:"]::before,
em[data-emoji="wheel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6de.svg");
}

em[data-emoji=":rotating_light:"]::before,
em[data-emoji="rotating_light"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a8.svg");
}

em[data-emoji=":oncoming_police_car:"]::before,
em[data-emoji="oncoming_police_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f694.svg");
}

em[data-emoji=":oncoming_bus:"]::before,
em[data-emoji="oncoming_bus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68d.svg");
}

em[data-emoji=":oncoming_automobile:"]::before,
em[data-emoji="oncoming_automobile"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f698.svg");
}

em[data-emoji=":oncoming_taxi:"]::before,
em[data-emoji="oncoming_taxi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f696.svg");
}

em[data-emoji=":aerial_tramway:"]::before,
em[data-emoji="aerial_tramway"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a1.svg");
}

em[data-emoji=":mountain_cableway:"]::before,
em[data-emoji="mountain_cableway"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a0.svg");
}

em[data-emoji=":suspension_railway:"]::before,
em[data-emoji="suspension_railway"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69f.svg");
}

em[data-emoji=":railway_car:"]::before,
em[data-emoji="railway_car"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f683.svg");
}

em[data-emoji=":train:"]::before,
em[data-emoji="train"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68b.svg");
}

em[data-emoji=":mountain_railway:"]::before,
em[data-emoji="mountain_railway"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69e.svg");
}

em[data-emoji=":monorail:"]::before,
em[data-emoji="monorail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f69d.svg");
}

em[data-emoji=":bullettrain_side:"]::before,
em[data-emoji="bullettrain_side"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f684.svg");
}

em[data-emoji=":bullettrain_front:"]::before,
em[data-emoji="bullettrain_front"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f685.svg");
}

em[data-emoji=":light_rail:"]::before,
em[data-emoji="light_rail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f688.svg");
}

em[data-emoji=":steam_locomotive:"]::before,
em[data-emoji="steam_locomotive"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f682.svg");
}

em[data-emoji=":train2:"]::before,
em[data-emoji="train2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f686.svg");
}

em[data-emoji=":metro:"]::before,
em[data-emoji="metro"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f687.svg");
}

em[data-emoji=":tram:"]::before,
em[data-emoji="tram"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68a.svg");
}

em[data-emoji=":station:"]::before,
em[data-emoji="station"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f689.svg");
}

em[data-emoji=":airplane_departure:"]::before,
em[data-emoji="airplane_departure"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6eb.svg");
}

em[data-emoji=":airplane_arriving:"]::before,
em[data-emoji="airplane_arriving"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ec.svg");
}

em[data-emoji=":airplane_small:"]::before,
em[data-emoji="airplane_small"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e9.svg");
}

em[data-emoji=":seat:"]::before,
em[data-emoji="seat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ba.svg");
}

em[data-emoji=":satellite_orbital:"]::before,
em[data-emoji="satellite_orbital"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f0.svg");
}

em[data-emoji=":rocket:"]::before,
em[data-emoji="rocket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f680.svg");
}

em[data-emoji=":flying_saucer:"]::before,
em[data-emoji="flying_saucer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f8.svg");
}

em[data-emoji=":helicopter:"]::before,
em[data-emoji="helicopter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f681.svg");
}

em[data-emoji=":canoe:"]::before,
em[data-emoji="canoe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f6.svg");
}

em[data-emoji=":sailboat:"]::before,
em[data-emoji="sailboat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f5.svg");
}

em[data-emoji=":speedboat:"]::before,
em[data-emoji="speedboat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a4.svg");
}

em[data-emoji=":motorboat:"]::before,
em[data-emoji="motorboat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e5.svg");
}

em[data-emoji=":cruise_ship:"]::before,
em[data-emoji="cruise_ship"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6f3.svg");
}

em[data-emoji=":ferry:"]::before,
em[data-emoji="ferry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f4.svg");
}

em[data-emoji=":ship:"]::before,
em[data-emoji="ship"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a2.svg");
}

em[data-emoji=":ring_buoy:"]::before,
em[data-emoji="ring_buoy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6df.svg");
}

em[data-emoji=":hook:"]::before,
em[data-emoji="hook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9d.svg");
}

em[data-emoji=":fuelpump:"]::before,
em[data-emoji="fuelpump"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26fd.svg");
}

em[data-emoji=":construction:"]::before,
em[data-emoji="construction"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a7.svg");
}

em[data-emoji=":vertical_traffic_light:"]::before,
em[data-emoji="vertical_traffic_light"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a6.svg");
}

em[data-emoji=":traffic_light:"]::before,
em[data-emoji="traffic_light"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a5.svg");
}

em[data-emoji=":busstop:"]::before,
em[data-emoji="busstop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f68f.svg");
}

em[data-emoji=":map:"]::before,
em[data-emoji="map"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5fa.svg");
}

em[data-emoji=":moyai:"]::before,
em[data-emoji="moyai"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5ff.svg");
}

em[data-emoji=":statue_of_liberty:"]::before,
em[data-emoji="statue_of_liberty"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5fd.svg");
}

em[data-emoji=":tokyo_tower:"]::before,
em[data-emoji="tokyo_tower"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5fc.svg");
}

em[data-emoji=":european_castle:"]::before,
em[data-emoji="european_castle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f0.svg");
}

em[data-emoji=":japanese_castle:"]::before,
em[data-emoji="japanese_castle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ef.svg");
}

em[data-emoji=":stadium:"]::before,
em[data-emoji="stadium"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3df.svg");
}

em[data-emoji=":ferris_wheel:"]::before,
em[data-emoji="ferris_wheel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a1.svg");
}

em[data-emoji=":roller_coaster:"]::before,
em[data-emoji="roller_coaster"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a2.svg");
}

em[data-emoji=":carousel_horse:"]::before,
em[data-emoji="carousel_horse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a0.svg");
}

em[data-emoji=":fountain:"]::before,
em[data-emoji="fountain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f2.svg");
}

em[data-emoji=":beach_umbrella:"]::before,
em[data-emoji="beach_umbrella"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f1.svg");
}

em[data-emoji=":beach:"]::before,
em[data-emoji="beach"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d6.svg");
}

em[data-emoji=":island:"]::before,
em[data-emoji="island"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3dd.svg");
}

em[data-emoji=":desert:"]::before,
em[data-emoji="desert"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3dc.svg");
}

em[data-emoji=":volcano:"]::before,
em[data-emoji="volcano"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30b.svg");
}

em[data-emoji=":mountain:"]::before,
em[data-emoji="mountain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26f0.svg");
}

em[data-emoji=":mountain_snow:"]::before,
em[data-emoji="mountain_snow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d4.svg");
}

em[data-emoji=":mount_fuji:"]::before,
em[data-emoji="mount_fuji"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5fb.svg");
}

em[data-emoji=":camping:"]::before,
em[data-emoji="camping"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d5.svg");
}

em[data-emoji=":tent:"]::before,
em[data-emoji="tent"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26fa.svg");
}

em[data-emoji=":house:"]::before,
em[data-emoji="house"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e0.svg");
}

em[data-emoji=":house_with_garden:"]::before,
em[data-emoji="house_with_garden"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e1.svg");
}

em[data-emoji=":homes:"]::before,
em[data-emoji="homes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d8.svg");
}

em[data-emoji=":house_abandoned:"]::before,
em[data-emoji="house_abandoned"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3da.svg");
}

em[data-emoji=":hut:"]::before,
em[data-emoji="hut"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d6.svg");
}

em[data-emoji=":construction_site:"]::before,
em[data-emoji="construction_site"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d7.svg");
}

em[data-emoji=":factory:"]::before,
em[data-emoji="factory"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ed.svg");
}

em[data-emoji=":office:"]::before,
em[data-emoji="office"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e2.svg");
}

em[data-emoji=":department_store:"]::before,
em[data-emoji="department_store"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ec.svg");
}

em[data-emoji=":post_office:"]::before,
em[data-emoji="post_office"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e3.svg");
}

em[data-emoji=":european_post_office:"]::before,
em[data-emoji="european_post_office"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e4.svg");
}

em[data-emoji=":hospital:"]::before,
em[data-emoji="hospital"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e5.svg");
}

em[data-emoji=":bank:"]::before,
em[data-emoji="bank"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e6.svg");
}

em[data-emoji=":hotel:"]::before,
em[data-emoji="hotel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e8.svg");
}

em[data-emoji=":convenience_store:"]::before,
em[data-emoji="convenience_store"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ea.svg");
}

em[data-emoji=":school:"]::before,
em[data-emoji="school"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3eb.svg");
}

em[data-emoji=":love_hotel:"]::before,
em[data-emoji="love_hotel"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e9.svg");
}

em[data-emoji=":wedding:"]::before,
em[data-emoji="wedding"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f492.svg");
}

em[data-emoji=":classical_building:"]::before,
em[data-emoji="classical_building"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3db.svg");
}

em[data-emoji=":church:"]::before,
em[data-emoji="church"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26ea.svg");
}

em[data-emoji=":mosque:"]::before,
em[data-emoji="mosque"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f54c.svg");
}

em[data-emoji=":synagogue:"]::before,
em[data-emoji="synagogue"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f54d.svg");
}

em[data-emoji=":hindu_temple:"]::before,
em[data-emoji="hindu_temple"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d5.svg");
}

em[data-emoji=":kaaba:"]::before,
em[data-emoji="kaaba"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f54b.svg");
}

em[data-emoji=":shinto_shrine:"]::before,
em[data-emoji="shinto_shrine"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26e9.svg");
}

em[data-emoji=":railway_track:"]::before,
em[data-emoji="railway_track"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e4.svg");
}

em[data-emoji=":motorway:"]::before,
em[data-emoji="motorway"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e3.svg");
}

em[data-emoji=":japan:"]::before,
em[data-emoji="japan"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5fe.svg");
}

em[data-emoji=":rice_scene:"]::before,
em[data-emoji="rice_scene"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f391.svg");
}

em[data-emoji=":park:"]::before,
em[data-emoji="park"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3de.svg");
}

em[data-emoji=":sunrise:"]::before,
em[data-emoji="sunrise"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f305.svg");
}

em[data-emoji=":sunrise_over_mountains:"]::before,
em[data-emoji="sunrise_over_mountains"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f304.svg");
}

em[data-emoji=":stars:"]::before,
em[data-emoji="stars"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f320.svg");
}

em[data-emoji=":sparkler:"]::before,
em[data-emoji="sparkler"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f387.svg");
}

em[data-emoji=":fireworks:"]::before,
em[data-emoji="fireworks"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f386.svg");
}

em[data-emoji=":city_sunset:"]::before,
em[data-emoji="city_sunset"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f307.svg");
}

em[data-emoji=":city_dusk:"]::before,
em[data-emoji="city_dusk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f306.svg");
}

em[data-emoji=":cityscape:"]::before,
em[data-emoji="cityscape"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3d9.svg");
}

em[data-emoji=":night_with_stars:"]::before,
em[data-emoji="night_with_stars"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f303.svg");
}

em[data-emoji=":milky_way:"]::before,
em[data-emoji="milky_way"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f30c.svg");
}

em[data-emoji=":bridge_at_night:"]::before,
em[data-emoji="bridge_at_night"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f309.svg");
}

em[data-emoji=":foggy:"]::before,
em[data-emoji="foggy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f301.svg");
}

em[data-emoji=":watch:"]::before,
em[data-emoji="watch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/231a.svg");
}

em[data-emoji=":mobile_phone:"]::before,
em[data-emoji="mobile_phone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f1.svg");
}

em[data-emoji=":calling:"]::before,
em[data-emoji="calling"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f2.svg");
}

em[data-emoji=":computer:"]::before,
em[data-emoji="computer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4bb.svg");
}

em[data-emoji=":desktop:"]::before,
em[data-emoji="desktop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5a5.svg");
}

em[data-emoji=":printer:"]::before,
em[data-emoji="printer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5a8.svg");
}

em[data-emoji=":mouse_three_button:"]::before,
em[data-emoji="mouse_three_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5b1.svg");
}

em[data-emoji=":trackball:"]::before,
em[data-emoji="trackball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5b2.svg");
}

em[data-emoji=":joystick:"]::before,
em[data-emoji="joystick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f579.svg");
}

em[data-emoji=":compression:"]::before,
em[data-emoji="compression"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5dc.svg");
}

em[data-emoji=":minidisc:"]::before,
em[data-emoji="minidisc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4bd.svg");
}

em[data-emoji=":floppy_disk:"]::before,
em[data-emoji="floppy_disk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4be.svg");
}

em[data-emoji=":cd:"]::before,
em[data-emoji="cd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4bf.svg");
}

em[data-emoji=":dvd:"]::before,
em[data-emoji="dvd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c0.svg");
}

em[data-emoji=":vhs:"]::before,
em[data-emoji="vhs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4fc.svg");
}

em[data-emoji=":camera:"]::before,
em[data-emoji="camera"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f7.svg");
}

em[data-emoji=":camera_with_flash:"]::before,
em[data-emoji="camera_with_flash"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f8.svg");
}

em[data-emoji=":video_camera:"]::before,
em[data-emoji="video_camera"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f9.svg");
}

em[data-emoji=":movie_camera:"]::before,
em[data-emoji="movie_camera"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a5.svg");
}

em[data-emoji=":projector:"]::before,
em[data-emoji="projector"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4fd.svg");
}

em[data-emoji=":film_frames:"]::before,
em[data-emoji="film_frames"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f39e.svg");
}

em[data-emoji=":telephone_receiver:"]::before,
em[data-emoji="telephone_receiver"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4de.svg");
}

em[data-emoji=":telephone:"]::before,
em[data-emoji="telephone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/260e.svg");
}

em[data-emoji=":pager:"]::before,
em[data-emoji="pager"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4df.svg");
}

em[data-emoji=":fax:"]::before,
em[data-emoji="fax"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e0.svg");
}

em[data-emoji=":tv:"]::before,
em[data-emoji="tv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4fa.svg");
}

em[data-emoji=":radio:"]::before,
em[data-emoji="radio"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4fb.svg");
}

em[data-emoji=":microphone2:"]::before,
em[data-emoji="microphone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f399.svg");
}

em[data-emoji=":level_slider:"]::before,
em[data-emoji="level_slider"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f39a.svg");
}

em[data-emoji=":control_knobs:"]::before,
em[data-emoji="control_knobs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f39b.svg");
}

em[data-emoji=":compass:"]::before,
em[data-emoji="compass"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ed.svg");
}

em[data-emoji=":stopwatch:"]::before,
em[data-emoji="stopwatch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f1.svg");
}

em[data-emoji=":timer:"]::before,
em[data-emoji="timer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f2.svg");
}

em[data-emoji=":alarm_clock:"]::before,
em[data-emoji="alarm_clock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f0.svg");
}

em[data-emoji=":clock:"]::before,
em[data-emoji="clock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f570.svg");
}

em[data-emoji=":hourglass:"]::before,
em[data-emoji="hourglass"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/231b.svg");
}

em[data-emoji=":hourglass_flowing_sand:"]::before,
em[data-emoji="hourglass_flowing_sand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f3.svg");
}

em[data-emoji=":satellite:"]::before,
em[data-emoji="satellite"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e1.svg");
}

em[data-emoji=":battery:"]::before,
em[data-emoji="battery"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50b.svg");
}

em[data-emoji=":low_battery:"]::before,
em[data-emoji="low_battery"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faab.svg");
}

em[data-emoji=":electric_plug:"]::before,
em[data-emoji="electric_plug"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50c.svg");
}

em[data-emoji=":bulb:"]::before,
em[data-emoji="bulb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a1.svg");
}

em[data-emoji=":flashlight:"]::before,
em[data-emoji="flashlight"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f526.svg");
}

em[data-emoji=":candle:"]::before,
em[data-emoji="candle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f56f.svg");
}

em[data-emoji=":diya_lamp:"]::before,
em[data-emoji="diya_lamp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa94.svg");
}

em[data-emoji=":fire_extinguisher:"]::before,
em[data-emoji="fire_extinguisher"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ef.svg");
}

em[data-emoji=":oil:"]::before,
em[data-emoji="oil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e2.svg");
}

em[data-emoji=":money_with_wings:"]::before,
em[data-emoji="money_with_wings"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b8.svg");
}

em[data-emoji=":dollar:"]::before,
em[data-emoji="dollar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b5.svg");
}

em[data-emoji=":yen:"]::before,
em[data-emoji="yen"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b4.svg");
}

em[data-emoji=":euro:"]::before,
em[data-emoji="euro"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b6.svg");
}

em[data-emoji=":pound:"]::before,
em[data-emoji="pound"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b7.svg");
}

em[data-emoji=":coin:"]::before,
em[data-emoji="coin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa99.svg");
}

em[data-emoji=":moneybag:"]::before,
em[data-emoji="moneybag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b0.svg");
}

em[data-emoji=":credit_card:"]::before,
em[data-emoji="credit_card"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b3.svg");
}

em[data-emoji=":identification_card:"]::before,
em[data-emoji="identification_card"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faaa.svg");
}

em[data-emoji=":gem:"]::before,
em[data-emoji="gem"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48e.svg");
}

em[data-emoji=":ladder:"]::before,
em[data-emoji="ladder"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9c.svg");
}

em[data-emoji=":toolbox:"]::before,
em[data-emoji="toolbox"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f0.svg");
}

em[data-emoji=":screwdriver:"]::before,
em[data-emoji="screwdriver"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9b.svg");
}

em[data-emoji=":wrench:"]::before,
em[data-emoji="wrench"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f527.svg");
}

em[data-emoji=":hammer:"]::before,
em[data-emoji="hammer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f528.svg");
}

em[data-emoji=":tools:"]::before,
em[data-emoji="tools"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e0.svg");
}

em[data-emoji=":pick:"]::before,
em[data-emoji="pick"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26cf.svg");
}

em[data-emoji=":carpentry_saw:"]::before,
em[data-emoji="carpentry_saw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9a.svg");
}

em[data-emoji=":nut_and_bolt:"]::before,
em[data-emoji="nut_and_bolt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f529.svg");
}

em[data-emoji=":mouse_trap:"]::before,
em[data-emoji="mouse_trap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa4.svg");
}

em[data-emoji=":bricks:"]::before,
em[data-emoji="bricks"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f1.svg");
}

em[data-emoji=":chains:"]::before,
em[data-emoji="chains"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26d3.svg");
}

em[data-emoji=":magnet:"]::before,
em[data-emoji="magnet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f2.svg");
}

em[data-emoji=":gun:"]::before,
em[data-emoji="gun"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52b.svg");
}

em[data-emoji=":bomb:"]::before,
em[data-emoji="bomb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a3.svg");
}

em[data-emoji=":firecracker:"]::before,
em[data-emoji="firecracker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e8.svg");
}

em[data-emoji=":axe:"]::before,
em[data-emoji="axe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa93.svg");
}

em[data-emoji=":knife:"]::before,
em[data-emoji="knife"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52a.svg");
}

em[data-emoji=":dagger:"]::before,
em[data-emoji="dagger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5e1.svg");
}

em[data-emoji=":shield:"]::before,
em[data-emoji="shield"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6e1.svg");
}

em[data-emoji=":smoking:"]::before,
em[data-emoji="smoking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ac.svg");
}

em[data-emoji=":coffin:"]::before,
em[data-emoji="coffin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26b0.svg");
}

em[data-emoji=":headstone:"]::before,
em[data-emoji="headstone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa6.svg");
}

em[data-emoji=":urn:"]::before,
em[data-emoji="urn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26b1.svg");
}

em[data-emoji=":amphora:"]::before,
em[data-emoji="amphora"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3fa.svg");
}

em[data-emoji=":crystal_ball:"]::before,
em[data-emoji="crystal_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52e.svg");
}

em[data-emoji=":prayer_beads:"]::before,
em[data-emoji="prayer_beads"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ff.svg");
}

em[data-emoji=":nazar_amulet:"]::before,
em[data-emoji="nazar_amulet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ff.svg");
}

em[data-emoji=":hamsa:"]::before,
em[data-emoji="hamsa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faac.svg");
}

em[data-emoji=":barber:"]::before,
em[data-emoji="barber"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f488.svg");
}

em[data-emoji=":telescope:"]::before,
em[data-emoji="telescope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52d.svg");
}

em[data-emoji=":microscope:"]::before,
em[data-emoji="microscope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52c.svg");
}

em[data-emoji=":hole:"]::before,
em[data-emoji="hole"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f573.svg");
}

em[data-emoji=":x_ray:"]::before,
em[data-emoji="x_ray"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa7b.svg");
}

em[data-emoji=":adhesive_bandage:"]::before,
em[data-emoji="adhesive_bandage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa79.svg");
}

em[data-emoji=":stethoscope:"]::before,
em[data-emoji="stethoscope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa7a.svg");
}

em[data-emoji=":pill:"]::before,
em[data-emoji="pill"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48a.svg");
}

em[data-emoji=":syringe:"]::before,
em[data-emoji="syringe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f489.svg");
}

em[data-emoji=":drop_of_blood:"]::before,
em[data-emoji="drop_of_blood"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa78.svg");
}

em[data-emoji=":dna:"]::before,
em[data-emoji="dna"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ec.svg");
}

em[data-emoji=":microbe:"]::before,
em[data-emoji="microbe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9a0.svg");
}

em[data-emoji=":petri_dish:"]::before,
em[data-emoji="petri_dish"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9eb.svg");
}

em[data-emoji=":test_tube:"]::before,
em[data-emoji="test_tube"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ea.svg");
}

em[data-emoji=":thermometer:"]::before,
em[data-emoji="thermometer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f321.svg");
}

em[data-emoji=":broom:"]::before,
em[data-emoji="broom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f9.svg");
}

em[data-emoji=":plunger:"]::before,
em[data-emoji="plunger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa0.svg");
}

em[data-emoji=":basket:"]::before,
em[data-emoji="basket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9fa.svg");
}

em[data-emoji=":roll_of_paper:"]::before,
em[data-emoji="roll_of_paper"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9fb.svg");
}

em[data-emoji=":toilet:"]::before,
em[data-emoji="toilet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6bd.svg");
}

em[data-emoji=":potable_water:"]::before,
em[data-emoji="potable_water"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b0.svg");
}

em[data-emoji=":shower:"]::before,
em[data-emoji="shower"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6bf.svg");
}

em[data-emoji=":bathtub:"]::before,
em[data-emoji="bathtub"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c1.svg");
}

em[data-emoji=":bath:"]::before,
em[data-emoji="bath"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0.svg");
}

em[data-emoji=":bath_tone1:"]::before,
em[data-emoji="bath_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0-1f3fb.svg");
}

em[data-emoji=":bath_tone2:"]::before,
em[data-emoji="bath_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0-1f3fc.svg");
}

em[data-emoji=":bath_tone3:"]::before,
em[data-emoji="bath_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0-1f3fd.svg");
}

em[data-emoji=":bath_tone4:"]::before,
em[data-emoji="bath_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0-1f3fe.svg");
}

em[data-emoji=":bath_tone5:"]::before,
em[data-emoji="bath_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c0-1f3ff.svg");
}

em[data-emoji=":soap:"]::before,
em[data-emoji="soap"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9fc.svg");
}

em[data-emoji=":toothbrush:"]::before,
em[data-emoji="toothbrush"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa5.svg");
}

em[data-emoji=":razor:"]::before,
em[data-emoji="razor"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa92.svg");
}

em[data-emoji=":sponge:"]::before,
em[data-emoji="sponge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9fd.svg");
}

em[data-emoji=":bucket:"]::before,
em[data-emoji="bucket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa3.svg");
}

em[data-emoji=":squeeze_bottle:"]::before,
em[data-emoji="squeeze_bottle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f4.svg");
}

em[data-emoji=":bellhop:"]::before,
em[data-emoji="bellhop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ce.svg");
}

em[data-emoji=":key:"]::before,
em[data-emoji="key"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f511.svg");
}

em[data-emoji=":key2:"]::before,
em[data-emoji="key2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5dd.svg");
}

em[data-emoji=":door:"]::before,
em[data-emoji="door"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6aa.svg");
}

em[data-emoji=":chair:"]::before,
em[data-emoji="chair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa91.svg");
}

em[data-emoji=":couch:"]::before,
em[data-emoji="couch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cb.svg");
}

em[data-emoji=":bed:"]::before,
em[data-emoji="bed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cf.svg");
}

em[data-emoji=":sleeping_accommodation:"]::before,
em[data-emoji="sleeping_accommodation"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc.svg");
}

em[data-emoji=":person_in_bed_tone1:"]::before,
em[data-emoji="person_in_bed_tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc-1f3fb.svg");
}

em[data-emoji=":person_in_bed_tone2:"]::before,
em[data-emoji="person_in_bed_tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc-1f3fc.svg");
}

em[data-emoji=":person_in_bed_tone3:"]::before,
em[data-emoji="person_in_bed_tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc-1f3fd.svg");
}

em[data-emoji=":person_in_bed_tone4:"]::before,
em[data-emoji="person_in_bed_tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc-1f3fe.svg");
}

em[data-emoji=":person_in_bed_tone5:"]::before,
em[data-emoji="person_in_bed_tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cc-1f3ff.svg");
}

em[data-emoji=":teddy_bear:"]::before,
em[data-emoji="teddy_bear"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f8.svg");
}

em[data-emoji=":nesting_dolls:"]::before,
em[data-emoji="nesting_dolls"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa86.svg");
}

em[data-emoji=":frame_photo:"]::before,
em[data-emoji="frame_photo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5bc.svg");
}

em[data-emoji=":mirror:"]::before,
em[data-emoji="mirror"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9e.svg");
}

em[data-emoji=":window:"]::before,
em[data-emoji="window"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa9f.svg");
}

em[data-emoji=":shopping_bags:"]::before,
em[data-emoji="shopping_bags"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6cd.svg");
}

em[data-emoji=":shopping_cart:"]::before,
em[data-emoji="shopping_cart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d2.svg");
}

em[data-emoji=":gift:"]::before,
em[data-emoji="gift"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f381.svg");
}

em[data-emoji=":balloon:"]::before,
em[data-emoji="balloon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f388.svg");
}

em[data-emoji=":flags:"]::before,
em[data-emoji="flags"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38f.svg");
}

em[data-emoji=":ribbon:"]::before,
em[data-emoji="ribbon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f380.svg");
}

em[data-emoji=":magic_wand:"]::before,
em[data-emoji="magic_wand"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa84.svg");
}

em[data-emoji=":pinata:"]::before,
em[data-emoji="pinata"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1fa85.svg");
}

em[data-emoji=":confetti_ball:"]::before,
em[data-emoji="confetti_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38a.svg");
}

em[data-emoji=":tada:"]::before,
em[data-emoji="tada"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f389.svg");
}

em[data-emoji=":dolls:"]::before,
em[data-emoji="dolls"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38e.svg");
}

em[data-emoji=":izakaya_lantern:"]::before,
em[data-emoji="izakaya_lantern"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ee.svg");
}

em[data-emoji=":wind_chime:"]::before,
em[data-emoji="wind_chime"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f390.svg");
}

em[data-emoji=":mirror_ball:"]::before,
em[data-emoji="mirror_ball"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa9.svg");
}

em[data-emoji=":red_envelope:"]::before,
em[data-emoji="red_envelope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e7.svg");
}

em[data-emoji=":envelope_with_arrow:"]::before,
em[data-emoji="envelope_with_arrow"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e9.svg");
}

em[data-emoji=":incoming_envelope:"]::before,
em[data-emoji="incoming_envelope"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e8.svg");
}

em[data-emoji=":e-mail:"]::before,
em[data-emoji="e-mail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e7.svg");
}

em[data-emoji=":love_letter:"]::before,
em[data-emoji="love_letter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f48c.svg");
}

em[data-emoji=":inbox_tray:"]::before,
em[data-emoji="inbox_tray"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e5.svg");
}

em[data-emoji=":outbox_tray:"]::before,
em[data-emoji="outbox_tray"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e4.svg");
}

em[data-emoji=":package:"]::before,
em[data-emoji="package"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e6.svg");
}

em[data-emoji=":label:"]::before,
em[data-emoji="label"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f7.svg");
}

em[data-emoji=":placard:"]::before,
em[data-emoji="placard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1faa7.svg");
}

em[data-emoji=":mailbox_closed:"]::before,
em[data-emoji="mailbox_closed"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ea.svg");
}

em[data-emoji=":mailbox:"]::before,
em[data-emoji="mailbox"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4eb.svg");
}

em[data-emoji=":mailbox_with_mail:"]::before,
em[data-emoji="mailbox_with_mail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ec.svg");
}

em[data-emoji=":mailbox_with_no_mail:"]::before,
em[data-emoji="mailbox_with_no_mail"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ed.svg");
}

em[data-emoji=":postbox:"]::before,
em[data-emoji="postbox"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ee.svg");
}

em[data-emoji=":postal_horn:"]::before,
em[data-emoji="postal_horn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ef.svg");
}

em[data-emoji=":scroll:"]::before,
em[data-emoji="scroll"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4dc.svg");
}

em[data-emoji=":page_with_curl:"]::before,
em[data-emoji="page_with_curl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c3.svg");
}

em[data-emoji=":page_facing_up:"]::before,
em[data-emoji="page_facing_up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c4.svg");
}

em[data-emoji=":bookmark_tabs:"]::before,
em[data-emoji="bookmark_tabs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d1.svg");
}

em[data-emoji=":receipt:"]::before,
em[data-emoji="receipt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9fe.svg");
}

em[data-emoji=":bar_chart:"]::before,
em[data-emoji="bar_chart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ca.svg");
}

em[data-emoji=":chart_with_upwards_trend:"]::before,
em[data-emoji="chart_with_upwards_trend"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c8.svg");
}

em[data-emoji=":chart_with_downwards_trend:"]::before,
em[data-emoji="chart_with_downwards_trend"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c9.svg");
}

em[data-emoji=":notepad_spiral:"]::before,
em[data-emoji="notepad_spiral"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5d2.svg");
}

em[data-emoji=":calendar_spiral:"]::before,
em[data-emoji="calendar_spiral"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5d3.svg");
}

em[data-emoji=":calendar:"]::before,
em[data-emoji="calendar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c6.svg");
}

em[data-emoji=":date:"]::before,
em[data-emoji="date"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c5.svg");
}

em[data-emoji=":wastebasket:"]::before,
em[data-emoji="wastebasket"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5d1.svg");
}

em[data-emoji=":card_index:"]::before,
em[data-emoji="card_index"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c7.svg");
}

em[data-emoji=":card_box:"]::before,
em[data-emoji="card_box"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5c3.svg");
}

em[data-emoji=":ballot_box:"]::before,
em[data-emoji="ballot_box"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5f3.svg");
}

em[data-emoji=":file_cabinet:"]::before,
em[data-emoji="file_cabinet"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5c4.svg");
}

em[data-emoji=":clipboard:"]::before,
em[data-emoji="clipboard"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4cb.svg");
}

em[data-emoji=":file_folder:"]::before,
em[data-emoji="file_folder"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c1.svg");
}

em[data-emoji=":open_file_folder:"]::before,
em[data-emoji="open_file_folder"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4c2.svg");
}

em[data-emoji=":dividers:"]::before,
em[data-emoji="dividers"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5c2.svg");
}

em[data-emoji=":newspaper2:"]::before,
em[data-emoji="newspaper2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5de.svg");
}

em[data-emoji=":newspaper:"]::before,
em[data-emoji="newspaper"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f0.svg");
}

em[data-emoji=":notebook:"]::before,
em[data-emoji="notebook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d3.svg");
}

em[data-emoji=":notebook_with_decorative_cover:"]::before,
em[data-emoji="notebook_with_decorative_cover"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d4.svg");
}

em[data-emoji=":ledger:"]::before,
em[data-emoji="ledger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d2.svg");
}

em[data-emoji=":closed_book:"]::before,
em[data-emoji="closed_book"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d5.svg");
}

em[data-emoji=":green_book:"]::before,
em[data-emoji="green_book"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d7.svg");
}

em[data-emoji=":blue_book:"]::before,
em[data-emoji="blue_book"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d8.svg");
}

em[data-emoji=":orange_book:"]::before,
em[data-emoji="orange_book"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d9.svg");
}

em[data-emoji=":books:"]::before,
em[data-emoji="books"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4da.svg");
}

em[data-emoji=":book:"]::before,
em[data-emoji="book"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d6.svg");
}

em[data-emoji=":bookmark:"]::before,
em[data-emoji="bookmark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f516.svg");
}

em[data-emoji=":safety_pin:"]::before,
em[data-emoji="safety_pin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9f7.svg");
}

em[data-emoji=":link:"]::before,
em[data-emoji="link"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f517.svg");
}

em[data-emoji=":paperclip:"]::before,
em[data-emoji="paperclip"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ce.svg");
}

em[data-emoji=":paperclips:"]::before,
em[data-emoji="paperclips"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f587.svg");
}

em[data-emoji=":triangular_ruler:"]::before,
em[data-emoji="triangular_ruler"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4d0.svg");
}

em[data-emoji=":straight_ruler:"]::before,
em[data-emoji="straight_ruler"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4cf.svg");
}

em[data-emoji=":abacus:"]::before,
em[data-emoji="abacus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9ee.svg");
}

em[data-emoji=":pushpin:"]::before,
em[data-emoji="pushpin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4cc.svg");
}

em[data-emoji=":round_pushpin:"]::before,
em[data-emoji="round_pushpin"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4cd.svg");
}

em[data-emoji=":pen_ballpoint:"]::before,
em[data-emoji="pen_ballpoint"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f58a.svg");
}

em[data-emoji=":pen_fountain:"]::before,
em[data-emoji="pen_fountain"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f58b.svg");
}

em[data-emoji=":paintbrush:"]::before,
em[data-emoji="paintbrush"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f58c.svg");
}

em[data-emoji=":crayon:"]::before,
em[data-emoji="crayon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f58d.svg");
}

em[data-emoji=":pencil:"]::before,
em[data-emoji="pencil"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4dd.svg");
}

em[data-emoji=":pencil2:"]::before,
em[data-emoji="pencil2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/270f.svg");
}

em[data-emoji=":mag:"]::before,
em[data-emoji="mag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50d.svg");
}

em[data-emoji=":mag_right:"]::before,
em[data-emoji="mag_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50e.svg");
}

em[data-emoji=":lock_with_ink_pen:"]::before,
em[data-emoji="lock_with_ink_pen"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50f.svg");
}

em[data-emoji=":closed_lock_with_key:"]::before,
em[data-emoji="closed_lock_with_key"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f510.svg");
}

em[data-emoji=":lock:"]::before,
em[data-emoji="lock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f512.svg");
}

em[data-emoji=":unlock:"]::before,
em[data-emoji="unlock"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f513.svg");
}

em[data-emoji=":orange_heart:"]::before,
em[data-emoji="orange_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f9e1.svg");
}

em[data-emoji=":yellow_heart:"]::before,
em[data-emoji="yellow_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49b.svg");
}

em[data-emoji=":green_heart:"]::before,
em[data-emoji="green_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49a.svg");
}

em[data-emoji=":blue_heart:"]::before,
em[data-emoji="blue_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f499.svg");
}

em[data-emoji=":purple_heart:"]::before,
em[data-emoji="purple_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49c.svg");
}

em[data-emoji=":black_heart:"]::before,
em[data-emoji="black_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5a4.svg");
}

em[data-emoji=":brown_heart:"]::before,
em[data-emoji="brown_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90e.svg");
}

em[data-emoji=":white_heart:"]::before,
em[data-emoji="white_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f90d.svg");
}

em[data-emoji=":broken_heart:"]::before,
em[data-emoji="broken_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f494.svg");
}

em[data-emoji=":two_hearts:"]::before,
em[data-emoji="two_hearts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f495.svg");
}

em[data-emoji=":revolving_hearts:"]::before,
em[data-emoji="revolving_hearts"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49e.svg");
}

em[data-emoji=":heartbeat:"]::before,
em[data-emoji="heartbeat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f493.svg");
}

em[data-emoji=":heartpulse:"]::before,
em[data-emoji="heartpulse"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f497.svg");
}

em[data-emoji=":sparkling_heart:"]::before,
em[data-emoji="sparkling_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f496.svg");
}

em[data-emoji=":cupid:"]::before,
em[data-emoji="cupid"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f498.svg");
}

em[data-emoji=":gift_heart:"]::before,
em[data-emoji="gift_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49d.svg");
}

em[data-emoji=":mending_heart:"]::before,
em[data-emoji="mending_heart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2764-fe0f-200d-1fa79.svg");
}

em[data-emoji=":heart_on_fire:"]::before,
em[data-emoji="heart_on_fire"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2764-fe0f-200d-1f525.svg");
}

em[data-emoji=":heart_decoration:"]::before,
em[data-emoji="heart_decoration"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f49f.svg");
}

em[data-emoji=":peace:"]::before,
em[data-emoji="peace"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/262e.svg");
}

em[data-emoji=":cross:"]::before,
em[data-emoji="cross"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/271d.svg");
}

em[data-emoji=":star_and_crescent:"]::before,
em[data-emoji="star_and_crescent"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/262a.svg");
}

em[data-emoji=":om_symbol:"]::before,
em[data-emoji="om_symbol"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f549.svg");
}

em[data-emoji=":six_pointed_star:"]::before,
em[data-emoji="six_pointed_star"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f52f.svg");
}

em[data-emoji=":menorah:"]::before,
em[data-emoji="menorah"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f54e.svg");
}

em[data-emoji=":yin_yang:"]::before,
em[data-emoji="yin_yang"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/262f.svg");
}

em[data-emoji=":place_of_worship:"]::before,
em[data-emoji="place_of_worship"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d0.svg");
}

em[data-emoji=":ophiuchus:"]::before,
em[data-emoji="ophiuchus"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26ce.svg");
}

em[data-emoji=":gemini:"]::before,
em[data-emoji="gemini"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264a.svg");
}

em[data-emoji=":cancer:"]::before,
em[data-emoji="cancer"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264b.svg");
}

em[data-emoji=":leo:"]::before,
em[data-emoji="leo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264c.svg");
}

em[data-emoji=":virgo:"]::before,
em[data-emoji="virgo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264d.svg");
}

em[data-emoji=":libra:"]::before,
em[data-emoji="libra"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264e.svg");
}

em[data-emoji=":scorpius:"]::before,
em[data-emoji="scorpius"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/264f.svg");
}

em[data-emoji=":id:"]::before,
em[data-emoji="id"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f194.svg");
}

em[data-emoji=":atom:"]::before,
em[data-emoji="atom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/269b.svg");
}

em[data-emoji=":accept:"]::before,
em[data-emoji="accept"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f251.svg");
}

em[data-emoji=":mobile_phone_off:"]::before,
em[data-emoji="mobile_phone_off"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f4.svg");
}

em[data-emoji=":vibration_mode:"]::before,
em[data-emoji="vibration_mode"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f3.svg");
}

em[data-emoji=":u6709:"]::before,
em[data-emoji="u6709"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f236.svg");
}

em[data-emoji=":u7121:"]::before,
em[data-emoji="u7121"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f21a.svg");
}

em[data-emoji=":u7533:"]::before,
em[data-emoji="u7533"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f238.svg");
}

em[data-emoji=":u55b6:"]::before,
em[data-emoji="u55b6"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f23a.svg");
}

em[data-emoji=":u6708:"]::before,
em[data-emoji="u6708"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f237.svg");
}

em[data-emoji=":vs:"]::before,
em[data-emoji="vs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f19a.svg");
}

em[data-emoji=":white_flower:"]::before,
em[data-emoji="white_flower"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ae.svg");
}

em[data-emoji=":ideograph_advantage:"]::before,
em[data-emoji="ideograph_advantage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f250.svg");
}

em[data-emoji=":u5408:"]::before,
em[data-emoji="u5408"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f234.svg");
}

em[data-emoji=":u6e80:"]::before,
em[data-emoji="u6e80"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f235.svg");
}

em[data-emoji=":u5272:"]::before,
em[data-emoji="u5272"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f239.svg");
}

em[data-emoji=":u7981:"]::before,
em[data-emoji="u7981"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f232.svg");
}

em[data-emoji=":a:"]::before,
em[data-emoji="a"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f170.svg");
}

em[data-emoji=":b:"]::before,
em[data-emoji="b"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f171.svg");
}

em[data-emoji=":ab:"]::before,
em[data-emoji="ab"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f18e.svg");
}

em[data-emoji=":cl:"]::before,
em[data-emoji="cl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f191.svg");
}

em[data-emoji=":o2:"]::before,
em[data-emoji="o2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f17e.svg");
}

em[data-emoji=":sos:"]::before,
em[data-emoji="sos"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f198.svg");
}

em[data-emoji=":x:"]::before,
em[data-emoji="x"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/274c.svg");
}

em[data-emoji=":o:"]::before,
em[data-emoji="o"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b55.svg");
}

em[data-emoji=":octagonal_sign:"]::before,
em[data-emoji="octagonal_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d1.svg");
}

em[data-emoji=":no_entry:"]::before,
em[data-emoji="no_entry"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26d4.svg");
}

em[data-emoji=":name_badge:"]::before,
em[data-emoji="name_badge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4db.svg");
}

em[data-emoji=":no_entry_sign:"]::before,
em[data-emoji="no_entry_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ab.svg");
}

em[data-emoji=":100:"]::before,
em[data-emoji="100"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4af.svg");
}

em[data-emoji=":anger:"]::before,
em[data-emoji="anger"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a2.svg");
}

em[data-emoji=":no_pedestrians:"]::before,
em[data-emoji="no_pedestrians"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b7.svg");
}

em[data-emoji=":do_not_litter:"]::before,
em[data-emoji="do_not_litter"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6af.svg");
}

em[data-emoji=":no_bicycles:"]::before,
em[data-emoji="no_bicycles"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b3.svg");
}

em[data-emoji=":non-potable_water:"]::before,
em[data-emoji="non-potable_water"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b1.svg");
}

em[data-emoji=":underage:"]::before,
em[data-emoji="underage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51e.svg");
}

em[data-emoji=":no_mobile_phones:"]::before,
em[data-emoji="no_mobile_phones"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f5.svg");
}

em[data-emoji=":no_smoking:"]::before,
em[data-emoji="no_smoking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ad.svg");
}

em[data-emoji=":bangbang:"]::before,
em[data-emoji="bangbang"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/203c.svg");
}

em[data-emoji=":low_brightness:"]::before,
em[data-emoji="low_brightness"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f505.svg");
}

em[data-emoji=":high_brightness:"]::before,
em[data-emoji="high_brightness"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f506.svg");
}

em[data-emoji=":part_alternation_mark:"]::before,
em[data-emoji="part_alternation_mark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/303d.svg");
}

em[data-emoji=":warning:"]::before,
em[data-emoji="warning"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26a0.svg");
}

em[data-emoji=":children_crossing:"]::before,
em[data-emoji="children_crossing"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b8.svg");
}

em[data-emoji=":trident:"]::before,
em[data-emoji="trident"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f531.svg");
}

em[data-emoji=":fleur-de-lis:"]::before,
em[data-emoji="fleur-de-lis"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/269c.svg");
}

em[data-emoji=":beginner:"]::before,
em[data-emoji="beginner"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f530.svg");
}

em[data-emoji=":recycle:"]::before,
em[data-emoji="recycle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/267b.svg");
}

em[data-emoji=":u6307:"]::before,
em[data-emoji="u6307"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f22f.svg");
}

em[data-emoji=":chart:"]::before,
em[data-emoji="chart"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b9.svg");
}

em[data-emoji=":negative_squared_cross_mark:"]::before,
em[data-emoji="negative_squared_cross_mark"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/274e.svg");
}

em[data-emoji=":globe_with_meridians:"]::before,
em[data-emoji="globe_with_meridians"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f310.svg");
}

em[data-emoji=":diamond_shape_with_a_dot_inside:"]::before,
em[data-emoji="diamond_shape_with_a_dot_inside"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a0.svg");
}

em[data-emoji=":m:"]::before,
em[data-emoji="m"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/24c2.svg");
}

em[data-emoji=":cyclone:"]::before,
em[data-emoji="cyclone"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f300.svg");
}

em[data-emoji=":zzz:"]::before,
em[data-emoji="zzz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4a4.svg");
}

em[data-emoji=":atm:"]::before,
em[data-emoji="atm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3e7.svg");
}

em[data-emoji=":wc:"]::before,
em[data-emoji="wc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6be.svg");
}

em[data-emoji=":wheelchair:"]::before,
em[data-emoji="wheelchair"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/267f.svg");
}

em[data-emoji=":parking:"]::before,
em[data-emoji="parking"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f17f.svg");
}

em[data-emoji=":u7a7a:"]::before,
em[data-emoji="u7a7a"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f233.svg");
}

em[data-emoji=":sa:"]::before,
em[data-emoji="sa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f202.svg");
}

em[data-emoji=":passport_control:"]::before,
em[data-emoji="passport_control"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c2.svg");
}

em[data-emoji=":customs:"]::before,
em[data-emoji="customs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c3.svg");
}

em[data-emoji=":baggage_claim:"]::before,
em[data-emoji="baggage_claim"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c4.svg");
}

em[data-emoji=":left_luggage:"]::before,
em[data-emoji="left_luggage"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6c5.svg");
}

em[data-emoji=":elevator:"]::before,
em[data-emoji="elevator"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6d7.svg");
}

em[data-emoji=":mens:"]::before,
em[data-emoji="mens"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6b9.svg");
}

em[data-emoji=":womens:"]::before,
em[data-emoji="womens"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ba.svg");
}

em[data-emoji=":baby_symbol:"]::before,
em[data-emoji="baby_symbol"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6bc.svg");
}

em[data-emoji=":restroom:"]::before,
em[data-emoji="restroom"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6bb.svg");
}

em[data-emoji=":put_litter_in_its_place:"]::before,
em[data-emoji="put_litter_in_its_place"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6ae.svg");
}

em[data-emoji=":cinema:"]::before,
em[data-emoji="cinema"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3a6.svg");
}

em[data-emoji=":signal_strength:"]::before,
em[data-emoji="signal_strength"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4f6.svg");
}

em[data-emoji=":koko:"]::before,
em[data-emoji="koko"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f201.svg");
}

em[data-emoji=":symbols:"]::before,
em[data-emoji="symbols"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f523.svg");
}

em[data-emoji=":abc:"]::before,
em[data-emoji="abc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f524.svg");
}

em[data-emoji=":abcd:"]::before,
em[data-emoji="abcd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f521.svg");
}

em[data-emoji=":capital_abcd:"]::before,
em[data-emoji="capital_abcd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f520.svg");
}

em[data-emoji=":ng:"]::before,
em[data-emoji="ng"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f196.svg");
}

em[data-emoji=":ok:"]::before,
em[data-emoji="ok"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f197.svg");
}

em[data-emoji=":up:"]::before,
em[data-emoji="up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f199.svg");
}

em[data-emoji=":cool:"]::before,
em[data-emoji="cool"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f192.svg");
}

em[data-emoji=":new:"]::before,
em[data-emoji="new"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f195.svg");
}

em[data-emoji=":free:"]::before,
em[data-emoji="free"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f193.svg");
}

em[data-emoji=":zero:"]::before,
em[data-emoji="zero"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/30-20e3.svg");
}

em[data-emoji=":one:"]::before,
em[data-emoji="one"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/31-20e3.svg");
}

em[data-emoji=":two:"]::before,
em[data-emoji="two"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/32-20e3.svg");
}

em[data-emoji=":three:"]::before,
em[data-emoji="three"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/33-20e3.svg");
}

em[data-emoji=":four:"]::before,
em[data-emoji="four"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/34-20e3.svg");
}

em[data-emoji=":five:"]::before,
em[data-emoji="five"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/35-20e3.svg");
}

em[data-emoji=":six:"]::before,
em[data-emoji="six"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/36-20e3.svg");
}

em[data-emoji=":seven:"]::before,
em[data-emoji="seven"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/37-20e3.svg");
}

em[data-emoji=":eight:"]::before,
em[data-emoji="eight"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/38-20e3.svg");
}

em[data-emoji=":nine:"]::before,
em[data-emoji="nine"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/39-20e3.svg");
}

em[data-emoji=":keycap_ten:"]::before,
em[data-emoji="keycap_ten"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51f.svg");
}

em[data-emoji=":1234:"]::before,
em[data-emoji="1234"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f522.svg");
}

em[data-emoji=":hash:"]::before,
em[data-emoji="hash"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23-20e3.svg");
}

em[data-emoji=":asterisk:"]::before,
em[data-emoji="asterisk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2a-20e3.svg");
}

em[data-emoji=":eject:"]::before,
em[data-emoji="eject"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23cf.svg");
}

em[data-emoji=":arrow_forward:"]::before,
em[data-emoji="arrow_forward"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25b6.svg");
}

em[data-emoji=":pause_button:"]::before,
em[data-emoji="pause_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f8.svg");
}

em[data-emoji=":play_pause:"]::before,
em[data-emoji="play_pause"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23ef.svg");
}

em[data-emoji=":stop_button:"]::before,
em[data-emoji="stop_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23f9.svg");
}

em[data-emoji=":record_button:"]::before,
em[data-emoji="record_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23fa.svg");
}

em[data-emoji=":track_next:"]::before,
em[data-emoji="track_next"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23ed.svg");
}

em[data-emoji=":track_previous:"]::before,
em[data-emoji="track_previous"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23ee.svg");
}

em[data-emoji=":fast_forward:"]::before,
em[data-emoji="fast_forward"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23e9.svg");
}

em[data-emoji=":rewind:"]::before,
em[data-emoji="rewind"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23ea.svg");
}

em[data-emoji=":arrow_double_up:"]::before,
em[data-emoji="arrow_double_up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23eb.svg");
}

em[data-emoji=":arrow_double_down:"]::before,
em[data-emoji="arrow_double_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/23ec.svg");
}

em[data-emoji=":arrow_backward:"]::before,
em[data-emoji="arrow_backward"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25c0.svg");
}

em[data-emoji=":arrow_up_small:"]::before,
em[data-emoji="arrow_up_small"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f53c.svg");
}

em[data-emoji=":arrow_down_small:"]::before,
em[data-emoji="arrow_down_small"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f53d.svg");
}

em[data-emoji=":arrow_right:"]::before,
em[data-emoji="arrow_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/27a1.svg");
}

em[data-emoji=":arrow_left:"]::before,
em[data-emoji="arrow_left"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b05.svg");
}

em[data-emoji=":arrow_up:"]::before,
em[data-emoji="arrow_up"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b06.svg");
}

em[data-emoji=":arrow_down:"]::before,
em[data-emoji="arrow_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b07.svg");
}

em[data-emoji=":arrow_right_hook:"]::before,
em[data-emoji="arrow_right_hook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/21aa.svg");
}

em[data-emoji=":leftwards_arrow_with_hook:"]::before,
em[data-emoji="leftwards_arrow_with_hook"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/21a9.svg");
}

em[data-emoji=":twisted_rightwards_arrows:"]::before,
em[data-emoji="twisted_rightwards_arrows"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f500.svg");
}

em[data-emoji=":repeat:"]::before,
em[data-emoji="repeat"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f501.svg");
}

em[data-emoji=":repeat_one:"]::before,
em[data-emoji="repeat_one"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f502.svg");
}

em[data-emoji=":arrows_counterclockwise:"]::before,
em[data-emoji="arrows_counterclockwise"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f504.svg");
}

em[data-emoji=":arrows_clockwise:"]::before,
em[data-emoji="arrows_clockwise"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f503.svg");
}

em[data-emoji=":musical_note:"]::before,
em[data-emoji="musical_note"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b5.svg");
}

em[data-emoji=":notes:"]::before,
em[data-emoji="notes"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b6.svg");
}

em[data-emoji=":heavy_equals_sign:"]::before,
em[data-emoji="heavy_equals_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7f0.svg");
}

em[data-emoji=":infinity:"]::before,
em[data-emoji="infinity"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/267e.svg");
}

em[data-emoji=":heavy_dollar_sign:"]::before,
em[data-emoji="heavy_dollar_sign"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b2.svg");
}

em[data-emoji=":currency_exchange:"]::before,
em[data-emoji="currency_exchange"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4b1.svg");
}

em[data-emoji=":copyright:"]::before,
em[data-emoji="copyright"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/a9.svg");
}

em[data-emoji=":registered:"]::before,
em[data-emoji="registered"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/ae.svg");
}

em[data-emoji=":curly_loop:"]::before,
em[data-emoji="curly_loop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/27b0.svg");
}

em[data-emoji=":loop:"]::before,
em[data-emoji="loop"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/27bf.svg");
}

em[data-emoji=":end:"]::before,
em[data-emoji="end"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51a.svg");
}

em[data-emoji=":back:"]::before,
em[data-emoji="back"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f519.svg");
}

em[data-emoji=":on:"]::before,
em[data-emoji="on"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51b.svg");
}

em[data-emoji=":top:"]::before,
em[data-emoji="top"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51d.svg");
}

em[data-emoji=":soon:"]::before,
em[data-emoji="soon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f51c.svg");
}

em[data-emoji=":radio_button:"]::before,
em[data-emoji="radio_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f518.svg");
}

em[data-emoji=":white_circle:"]::before,
em[data-emoji="white_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26aa.svg");
}

em[data-emoji=":black_circle:"]::before,
em[data-emoji="black_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26ab.svg");
}

em[data-emoji=":red_circle:"]::before,
em[data-emoji="red_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f534.svg");
}

em[data-emoji=":blue_circle:"]::before,
em[data-emoji="blue_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f535.svg");
}

em[data-emoji=":brown_circle:"]::before,
em[data-emoji="brown_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e4.svg");
}

em[data-emoji=":purple_circle:"]::before,
em[data-emoji="purple_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e3.svg");
}

em[data-emoji=":green_circle:"]::before,
em[data-emoji="green_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e2.svg");
}

em[data-emoji=":yellow_circle:"]::before,
em[data-emoji="yellow_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e1.svg");
}

em[data-emoji=":orange_circle:"]::before,
em[data-emoji="orange_circle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e0.svg");
}

em[data-emoji=":small_red_triangle:"]::before,
em[data-emoji="small_red_triangle"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f53a.svg");
}

em[data-emoji=":small_red_triangle_down:"]::before,
em[data-emoji="small_red_triangle_down"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f53b.svg");
}

em[data-emoji=":small_orange_diamond:"]::before,
em[data-emoji="small_orange_diamond"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f538.svg");
}

em[data-emoji=":small_blue_diamond:"]::before,
em[data-emoji="small_blue_diamond"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f539.svg");
}

em[data-emoji=":large_orange_diamond:"]::before,
em[data-emoji="large_orange_diamond"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f536.svg");
}

em[data-emoji=":large_blue_diamond:"]::before,
em[data-emoji="large_blue_diamond"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f537.svg");
}

em[data-emoji=":white_square_button:"]::before,
em[data-emoji="white_square_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f533.svg");
}

em[data-emoji=":black_square_button:"]::before,
em[data-emoji="black_square_button"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f532.svg");
}

em[data-emoji=":black_small_square:"]::before,
em[data-emoji="black_small_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25aa.svg");
}

em[data-emoji=":white_small_square:"]::before,
em[data-emoji="white_small_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25ab.svg");
}

em[data-emoji=":black_medium_small_square:"]::before,
em[data-emoji="black_medium_small_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25fe.svg");
}

em[data-emoji=":white_medium_small_square:"]::before,
em[data-emoji="white_medium_small_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25fd.svg");
}

em[data-emoji=":black_medium_square:"]::before,
em[data-emoji="black_medium_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25fc.svg");
}

em[data-emoji=":white_medium_square:"]::before,
em[data-emoji="white_medium_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/25fb.svg");
}

em[data-emoji=":black_large_square:"]::before,
em[data-emoji="black_large_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b1b.svg");
}

em[data-emoji=":white_large_square:"]::before,
em[data-emoji="white_large_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/2b1c.svg");
}

em[data-emoji=":orange_square:"]::before,
em[data-emoji="orange_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e7.svg");
}

em[data-emoji=":blue_square:"]::before,
em[data-emoji="blue_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e6.svg");
}

em[data-emoji=":red_square:"]::before,
em[data-emoji="red_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e5.svg");
}

em[data-emoji=":brown_square:"]::before,
em[data-emoji="brown_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7eb.svg");
}

em[data-emoji=":purple_square:"]::before,
em[data-emoji="purple_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7ea.svg");
}

em[data-emoji=":green_square:"]::before,
em[data-emoji="green_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e9.svg");
}

em[data-emoji=":yellow_square:"]::before,
em[data-emoji="yellow_square"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f7e8.svg");
}

em[data-emoji=":speaker:"]::before,
em[data-emoji="speaker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f508.svg");
}

em[data-emoji=":mute:"]::before,
em[data-emoji="mute"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f507.svg");
}

em[data-emoji=":sound:"]::before,
em[data-emoji="sound"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f509.svg");
}

em[data-emoji=":loud_sound:"]::before,
em[data-emoji="loud_sound"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f50a.svg");
}

em[data-emoji=":bell:"]::before,
em[data-emoji="bell"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f514.svg");
}

em[data-emoji=":no_bell:"]::before,
em[data-emoji="no_bell"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f515.svg");
}

em[data-emoji=":mega:"]::before,
em[data-emoji="mega"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e3.svg");
}

em[data-emoji=":loudspeaker:"]::before,
em[data-emoji="loudspeaker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4e2.svg");
}

em[data-emoji=":speech_left:"]::before,
em[data-emoji="speech_left"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5e8.svg");
}

em[data-emoji=":eye_in_speech_bubble:"]::before,
em[data-emoji="eye_in_speech_bubble"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f441-200d-1f5e8.svg");
}

em[data-emoji=":speech_balloon:"]::before,
em[data-emoji="speech_balloon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ac.svg");
}

em[data-emoji=":thought_balloon:"]::before,
em[data-emoji="thought_balloon"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f4ad.svg");
}

em[data-emoji=":anger_right:"]::before,
em[data-emoji="anger_right"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f5ef.svg");
}

em[data-emoji=":black_joker:"]::before,
em[data-emoji="black_joker"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f0cf.svg");
}

em[data-emoji=":flower_playing_cards:"]::before,
em[data-emoji="flower_playing_cards"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3b4.svg");
}

em[data-emoji=":mahjong:"]::before,
em[data-emoji="mahjong"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f004.svg");
}

em[data-emoji=":clock1:"]::before,
em[data-emoji="clock1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f550.svg");
}

em[data-emoji=":clock2:"]::before,
em[data-emoji="clock2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f551.svg");
}

em[data-emoji=":clock3:"]::before,
em[data-emoji="clock3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f552.svg");
}

em[data-emoji=":clock4:"]::before,
em[data-emoji="clock4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f553.svg");
}

em[data-emoji=":clock5:"]::before,
em[data-emoji="clock5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f554.svg");
}

em[data-emoji=":clock6:"]::before,
em[data-emoji="clock6"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f555.svg");
}

em[data-emoji=":clock7:"]::before,
em[data-emoji="clock7"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f556.svg");
}

em[data-emoji=":clock8:"]::before,
em[data-emoji="clock8"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f557.svg");
}

em[data-emoji=":clock9:"]::before,
em[data-emoji="clock9"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f558.svg");
}

em[data-emoji=":clock10:"]::before,
em[data-emoji="clock10"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f559.svg");
}

em[data-emoji=":clock11:"]::before,
em[data-emoji="clock11"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55a.svg");
}

em[data-emoji=":clock12:"]::before,
em[data-emoji="clock12"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55b.svg");
}

em[data-emoji=":clock130:"]::before,
em[data-emoji="clock130"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55c.svg");
}

em[data-emoji=":clock230:"]::before,
em[data-emoji="clock230"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55d.svg");
}

em[data-emoji=":clock330:"]::before,
em[data-emoji="clock330"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55e.svg");
}

em[data-emoji=":clock430:"]::before,
em[data-emoji="clock430"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f55f.svg");
}

em[data-emoji=":clock530:"]::before,
em[data-emoji="clock530"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f560.svg");
}

em[data-emoji=":clock630:"]::before,
em[data-emoji="clock630"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f561.svg");
}

em[data-emoji=":clock730:"]::before,
em[data-emoji="clock730"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f562.svg");
}

em[data-emoji=":clock830:"]::before,
em[data-emoji="clock830"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f563.svg");
}

em[data-emoji=":clock930:"]::before,
em[data-emoji="clock930"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f564.svg");
}

em[data-emoji=":clock1030:"]::before,
em[data-emoji="clock1030"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f565.svg");
}

em[data-emoji=":clock1130:"]::before,
em[data-emoji="clock1130"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f566.svg");
}

em[data-emoji=":clock1230:"]::before,
em[data-emoji="clock1230"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f567.svg");
}

em[data-emoji=":transgender_symbol:"]::before,
em[data-emoji="transgender_symbol"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/26a7.svg");
}

em[data-emoji=":flag_white:"]::before,
em[data-emoji="flag_white"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f3.svg");
}

em[data-emoji=":flag_black:"]::before,
em[data-emoji="flag_black"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4.svg");
}

em[data-emoji=":checkered_flag:"]::before,
em[data-emoji="checkered_flag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3c1.svg");
}

em[data-emoji=":triangular_flag_on_post:"]::before,
em[data-emoji="triangular_flag_on_post"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f6a9.svg");
}

em[data-emoji=":rainbow_flag:"]::before,
em[data-emoji="rainbow_flag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f3-fe0f-200d-1f308.svg");
}

em[data-emoji=":transgender_flag:"]::before,
em[data-emoji="transgender_flag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f3-fe0f-200d-26a7-fe0f.svg");
}

em[data-emoji=":pirate_flag:"]::before,
em[data-emoji="pirate_flag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-200d-2620-fe0f.svg");
}

em[data-emoji=":flag_af:"]::before,
em[data-emoji="flag_af"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1eb.svg");
}

em[data-emoji=":flag_ax:"]::before,
em[data-emoji="flag_ax"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fd.svg");
}

em[data-emoji=":flag_al:"]::before,
em[data-emoji="flag_al"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f1.svg");
}

em[data-emoji=":flag_dz:"]::before,
em[data-emoji="flag_dz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ff.svg");
}

em[data-emoji=":flag_as:"]::before,
em[data-emoji="flag_as"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f8.svg");
}

em[data-emoji=":flag_ad:"]::before,
em[data-emoji="flag_ad"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1e9.svg");
}

em[data-emoji=":flag_ao:"]::before,
em[data-emoji="flag_ao"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f4.svg");
}

em[data-emoji=":flag_ai:"]::before,
em[data-emoji="flag_ai"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ee.svg");
}

em[data-emoji=":flag_aq:"]::before,
em[data-emoji="flag_aq"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f6.svg");
}

em[data-emoji=":flag_ag:"]::before,
em[data-emoji="flag_ag"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ec.svg");
}

em[data-emoji=":flag_ar:"]::before,
em[data-emoji="flag_ar"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f7.svg");
}

em[data-emoji=":flag_am:"]::before,
em[data-emoji="flag_am"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f2.svg");
}

em[data-emoji=":flag_aw:"]::before,
em[data-emoji="flag_aw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fc.svg");
}

em[data-emoji=":flag_au:"]::before,
em[data-emoji="flag_au"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fa.svg");
}

em[data-emoji=":flag_at:"]::before,
em[data-emoji="flag_at"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f9.svg");
}

em[data-emoji=":flag_az:"]::before,
em[data-emoji="flag_az"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ff.svg");
}

em[data-emoji=":flag_bs:"]::before,
em[data-emoji="flag_bs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f8.svg");
}

em[data-emoji=":flag_bh:"]::before,
em[data-emoji="flag_bh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ed.svg");
}

em[data-emoji=":flag_bd:"]::before,
em[data-emoji="flag_bd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e9.svg");
}

em[data-emoji=":flag_bb:"]::before,
em[data-emoji="flag_bb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e7.svg");
}

em[data-emoji=":flag_by:"]::before,
em[data-emoji="flag_by"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fe.svg");
}

em[data-emoji=":flag_be:"]::before,
em[data-emoji="flag_be"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ea.svg");
}

em[data-emoji=":flag_bz:"]::before,
em[data-emoji="flag_bz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ff.svg");
}

em[data-emoji=":flag_bj:"]::before,
em[data-emoji="flag_bj"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ef.svg");
}

em[data-emoji=":flag_bm:"]::before,
em[data-emoji="flag_bm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f2.svg");
}

em[data-emoji=":flag_bt:"]::before,
em[data-emoji="flag_bt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f9.svg");
}

em[data-emoji=":flag_bo:"]::before,
em[data-emoji="flag_bo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f4.svg");
}

em[data-emoji=":flag_ba:"]::before,
em[data-emoji="flag_ba"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e6.svg");
}

em[data-emoji=":flag_bw:"]::before,
em[data-emoji="flag_bw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fc.svg");
}

em[data-emoji=":flag_br:"]::before,
em[data-emoji="flag_br"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f7.svg");
}

em[data-emoji=":flag_io:"]::before,
em[data-emoji="flag_io"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f4.svg");
}

em[data-emoji=":flag_vg:"]::before,
em[data-emoji="flag_vg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ec.svg");
}

em[data-emoji=":flag_bn:"]::before,
em[data-emoji="flag_bn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f3.svg");
}

em[data-emoji=":flag_bg:"]::before,
em[data-emoji="flag_bg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ec.svg");
}

em[data-emoji=":flag_bf:"]::before,
em[data-emoji="flag_bf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1eb.svg");
}

em[data-emoji=":flag_bi:"]::before,
em[data-emoji="flag_bi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ee.svg");
}

em[data-emoji=":flag_kh:"]::before,
em[data-emoji="flag_kh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ed.svg");
}

em[data-emoji=":flag_cm:"]::before,
em[data-emoji="flag_cm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f2.svg");
}

em[data-emoji=":flag_ca:"]::before,
em[data-emoji="flag_ca"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e6.svg");
}

em[data-emoji=":flag_ic:"]::before,
em[data-emoji="flag_ic"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1e8.svg");
}

em[data-emoji=":flag_cv:"]::before,
em[data-emoji="flag_cv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fb.svg");
}

em[data-emoji=":flag_bq:"]::before,
em[data-emoji="flag_bq"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f6.svg");
}

em[data-emoji=":flag_ky:"]::before,
em[data-emoji="flag_ky"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1fe.svg");
}

em[data-emoji=":flag_cf:"]::before,
em[data-emoji="flag_cf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1eb.svg");
}

em[data-emoji=":flag_td:"]::before,
em[data-emoji="flag_td"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e9.svg");
}

em[data-emoji=":flag_cl:"]::before,
em[data-emoji="flag_cl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f1.svg");
}

em[data-emoji=":flag_cn:"]::before,
em[data-emoji="flag_cn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f3.svg");
}

em[data-emoji=":flag_cx:"]::before,
em[data-emoji="flag_cx"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fd.svg");
}

em[data-emoji=":flag_cc:"]::before,
em[data-emoji="flag_cc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e8.svg");
}

em[data-emoji=":flag_co:"]::before,
em[data-emoji="flag_co"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f4.svg");
}

em[data-emoji=":flag_km:"]::before,
em[data-emoji="flag_km"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f2.svg");
}

em[data-emoji=":flag_cg:"]::before,
em[data-emoji="flag_cg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ec.svg");
}

em[data-emoji=":flag_cd:"]::before,
em[data-emoji="flag_cd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e9.svg");
}

em[data-emoji=":flag_ck:"]::before,
em[data-emoji="flag_ck"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f0.svg");
}

em[data-emoji=":flag_cr:"]::before,
em[data-emoji="flag_cr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f7.svg");
}

em[data-emoji=":flag_ci:"]::before,
em[data-emoji="flag_ci"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ee.svg");
}

em[data-emoji=":flag_hr:"]::before,
em[data-emoji="flag_hr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f7.svg");
}

em[data-emoji=":flag_cu:"]::before,
em[data-emoji="flag_cu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fa.svg");
}

em[data-emoji=":flag_cw:"]::before,
em[data-emoji="flag_cw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fc.svg");
}

em[data-emoji=":flag_cy:"]::before,
em[data-emoji="flag_cy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fe.svg");
}

em[data-emoji=":flag_cz:"]::before,
em[data-emoji="flag_cz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ff.svg");
}

em[data-emoji=":flag_dk:"]::before,
em[data-emoji="flag_dk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f0.svg");
}

em[data-emoji=":flag_dj:"]::before,
em[data-emoji="flag_dj"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ef.svg");
}

em[data-emoji=":flag_dm:"]::before,
em[data-emoji="flag_dm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f2.svg");
}

em[data-emoji=":flag_do:"]::before,
em[data-emoji="flag_do"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f4.svg");
}

em[data-emoji=":flag_ec:"]::before,
em[data-emoji="flag_ec"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1e8.svg");
}

em[data-emoji=":flag_eg:"]::before,
em[data-emoji="flag_eg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ec.svg");
}

em[data-emoji=":flag_sv:"]::before,
em[data-emoji="flag_sv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fb.svg");
}

em[data-emoji=":flag_gq:"]::before,
em[data-emoji="flag_gq"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f6.svg");
}

em[data-emoji=":flag_er:"]::before,
em[data-emoji="flag_er"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f7.svg");
}

em[data-emoji=":flag_ee:"]::before,
em[data-emoji="flag_ee"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ea.svg");
}

em[data-emoji=":flag_et:"]::before,
em[data-emoji="flag_et"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f9.svg");
}

em[data-emoji=":flag_eu:"]::before,
em[data-emoji="flag_eu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1fa.svg");
}

em[data-emoji=":flag_fk:"]::before,
em[data-emoji="flag_fk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f0.svg");
}

em[data-emoji=":flag_fo:"]::before,
em[data-emoji="flag_fo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f4.svg");
}

em[data-emoji=":flag_fj:"]::before,
em[data-emoji="flag_fj"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1ef.svg");
}

em[data-emoji=":flag_fi:"]::before,
em[data-emoji="flag_fi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1ee.svg");
}

em[data-emoji=":flag_fr:"]::before,
em[data-emoji="flag_fr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f7.svg");
}

em[data-emoji=":flag_gf:"]::before,
em[data-emoji="flag_gf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1eb.svg");
}

em[data-emoji=":flag_pf:"]::before,
em[data-emoji="flag_pf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1eb.svg");
}

em[data-emoji=":flag_tf:"]::before,
em[data-emoji="flag_tf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1eb.svg");
}

em[data-emoji=":flag_ga:"]::before,
em[data-emoji="flag_ga"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e6.svg");
}

em[data-emoji=":flag_gm:"]::before,
em[data-emoji="flag_gm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f2.svg");
}

em[data-emoji=":flag_ge:"]::before,
em[data-emoji="flag_ge"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ea.svg");
}

em[data-emoji=":flag_de:"]::before,
em[data-emoji="flag_de"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ea.svg");
}

em[data-emoji=":flag_gh:"]::before,
em[data-emoji="flag_gh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ed.svg");
}

em[data-emoji=":flag_gi:"]::before,
em[data-emoji="flag_gi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ee.svg");
}

em[data-emoji=":flag_gr:"]::before,
em[data-emoji="flag_gr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f7.svg");
}

em[data-emoji=":flag_gl:"]::before,
em[data-emoji="flag_gl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f1.svg");
}

em[data-emoji=":flag_gd:"]::before,
em[data-emoji="flag_gd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e9.svg");
}

em[data-emoji=":flag_gp:"]::before,
em[data-emoji="flag_gp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f5.svg");
}

em[data-emoji=":flag_gu:"]::before,
em[data-emoji="flag_gu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fa.svg");
}

em[data-emoji=":flag_gt:"]::before,
em[data-emoji="flag_gt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f9.svg");
}

em[data-emoji=":flag_gg:"]::before,
em[data-emoji="flag_gg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ec.svg");
}

em[data-emoji=":flag_gn:"]::before,
em[data-emoji="flag_gn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f3.svg");
}

em[data-emoji=":flag_gw:"]::before,
em[data-emoji="flag_gw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fc.svg");
}

em[data-emoji=":flag_gy:"]::before,
em[data-emoji="flag_gy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fe.svg");
}

em[data-emoji=":flag_ht:"]::before,
em[data-emoji="flag_ht"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f9.svg");
}

em[data-emoji=":flag_hn:"]::before,
em[data-emoji="flag_hn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f3.svg");
}

em[data-emoji=":flag_hk:"]::before,
em[data-emoji="flag_hk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f0.svg");
}

em[data-emoji=":flag_hu:"]::before,
em[data-emoji="flag_hu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1fa.svg");
}

em[data-emoji=":flag_is:"]::before,
em[data-emoji="flag_is"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f8.svg");
}

em[data-emoji=":flag_in:"]::before,
em[data-emoji="flag_in"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f3.svg");
}

em[data-emoji=":flag_id:"]::before,
em[data-emoji="flag_id"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1e9.svg");
}

em[data-emoji=":flag_ir:"]::before,
em[data-emoji="flag_ir"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f7.svg");
}

em[data-emoji=":flag_iq:"]::before,
em[data-emoji="flag_iq"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f6.svg");
}

em[data-emoji=":flag_ie:"]::before,
em[data-emoji="flag_ie"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1ea.svg");
}

em[data-emoji=":flag_im:"]::before,
em[data-emoji="flag_im"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f2.svg");
}

em[data-emoji=":flag_il:"]::before,
em[data-emoji="flag_il"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f1.svg");
}

em[data-emoji=":flag_it:"]::before,
em[data-emoji="flag_it"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f9.svg");
}

em[data-emoji=":flag_jm:"]::before,
em[data-emoji="flag_jm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f2.svg");
}

em[data-emoji=":flag_jp:"]::before,
em[data-emoji="flag_jp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f5.svg");
}

em[data-emoji=":crossed_flags:"]::before,
em[data-emoji="crossed_flags"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f38c.svg");
}

em[data-emoji=":flag_je:"]::before,
em[data-emoji="flag_je"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1ea.svg");
}

em[data-emoji=":flag_jo:"]::before,
em[data-emoji="flag_jo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f4.svg");
}

em[data-emoji=":flag_kz:"]::before,
em[data-emoji="flag_kz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ff.svg");
}

em[data-emoji=":flag_ke:"]::before,
em[data-emoji="flag_ke"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ea.svg");
}

em[data-emoji=":flag_ki:"]::before,
em[data-emoji="flag_ki"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ee.svg");
}

em[data-emoji=":flag_xk:"]::before,
em[data-emoji="flag_xk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fd-1f1f0.svg");
}

em[data-emoji=":flag_kw:"]::before,
em[data-emoji="flag_kw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1fc.svg");
}

em[data-emoji=":flag_kg:"]::before,
em[data-emoji="flag_kg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ec.svg");
}

em[data-emoji=":flag_la:"]::before,
em[data-emoji="flag_la"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e6.svg");
}

em[data-emoji=":flag_lv:"]::before,
em[data-emoji="flag_lv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fb.svg");
}

em[data-emoji=":flag_lb:"]::before,
em[data-emoji="flag_lb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e7.svg");
}

em[data-emoji=":flag_ls:"]::before,
em[data-emoji="flag_ls"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f8.svg");
}

em[data-emoji=":flag_lr:"]::before,
em[data-emoji="flag_lr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f7.svg");
}

em[data-emoji=":flag_ly:"]::before,
em[data-emoji="flag_ly"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fe.svg");
}

em[data-emoji=":flag_li:"]::before,
em[data-emoji="flag_li"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1ee.svg");
}

em[data-emoji=":flag_lt:"]::before,
em[data-emoji="flag_lt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f9.svg");
}

em[data-emoji=":flag_lu:"]::before,
em[data-emoji="flag_lu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fa.svg");
}

em[data-emoji=":flag_mo:"]::before,
em[data-emoji="flag_mo"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f4.svg");
}

em[data-emoji=":flag_mk:"]::before,
em[data-emoji="flag_mk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f0.svg");
}

em[data-emoji=":flag_mg:"]::before,
em[data-emoji="flag_mg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ec.svg");
}

em[data-emoji=":flag_mw:"]::before,
em[data-emoji="flag_mw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fc.svg");
}

em[data-emoji=":flag_my:"]::before,
em[data-emoji="flag_my"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fe.svg");
}

em[data-emoji=":flag_mv:"]::before,
em[data-emoji="flag_mv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fb.svg");
}

em[data-emoji=":flag_ml:"]::before,
em[data-emoji="flag_ml"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f1.svg");
}

em[data-emoji=":flag_mt:"]::before,
em[data-emoji="flag_mt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f9.svg");
}

em[data-emoji=":flag_mh:"]::before,
em[data-emoji="flag_mh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ed.svg");
}

em[data-emoji=":flag_mq:"]::before,
em[data-emoji="flag_mq"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f6.svg");
}

em[data-emoji=":flag_mr:"]::before,
em[data-emoji="flag_mr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f7.svg");
}

em[data-emoji=":flag_mu:"]::before,
em[data-emoji="flag_mu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fa.svg");
}

em[data-emoji=":flag_yt:"]::before,
em[data-emoji="flag_yt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fe-1f1f9.svg");
}

em[data-emoji=":flag_mx:"]::before,
em[data-emoji="flag_mx"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fd.svg");
}

em[data-emoji=":flag_fm:"]::before,
em[data-emoji="flag_fm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f2.svg");
}

em[data-emoji=":flag_md:"]::before,
em[data-emoji="flag_md"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e9.svg");
}

em[data-emoji=":flag_mc:"]::before,
em[data-emoji="flag_mc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e8.svg");
}

em[data-emoji=":flag_mn:"]::before,
em[data-emoji="flag_mn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f3.svg");
}

em[data-emoji=":flag_me:"]::before,
em[data-emoji="flag_me"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ea.svg");
}

em[data-emoji=":flag_ms:"]::before,
em[data-emoji="flag_ms"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f8.svg");
}

em[data-emoji=":flag_ma:"]::before,
em[data-emoji="flag_ma"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e6.svg");
}

em[data-emoji=":flag_mz:"]::before,
em[data-emoji="flag_mz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ff.svg");
}

em[data-emoji=":flag_mm:"]::before,
em[data-emoji="flag_mm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f2.svg");
}

em[data-emoji=":flag_na:"]::before,
em[data-emoji="flag_na"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1e6.svg");
}

em[data-emoji=":flag_nr:"]::before,
em[data-emoji="flag_nr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f7.svg");
}

em[data-emoji=":flag_np:"]::before,
em[data-emoji="flag_np"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f5.svg");
}

em[data-emoji=":flag_nl:"]::before,
em[data-emoji="flag_nl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f1.svg");
}

em[data-emoji=":flag_nc:"]::before,
em[data-emoji="flag_nc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1e8.svg");
}

em[data-emoji=":flag_nz:"]::before,
em[data-emoji="flag_nz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ff.svg");
}

em[data-emoji=":flag_ni:"]::before,
em[data-emoji="flag_ni"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ee.svg");
}

em[data-emoji=":flag_ne:"]::before,
em[data-emoji="flag_ne"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ea.svg");
}

em[data-emoji=":flag_ng:"]::before,
em[data-emoji="flag_ng"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ec.svg");
}

em[data-emoji=":flag_nu:"]::before,
em[data-emoji="flag_nu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1fa.svg");
}

em[data-emoji=":flag_nf:"]::before,
em[data-emoji="flag_nf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1eb.svg");
}

em[data-emoji=":flag_kp:"]::before,
em[data-emoji="flag_kp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f5.svg");
}

em[data-emoji=":flag_mp:"]::before,
em[data-emoji="flag_mp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f5.svg");
}

em[data-emoji=":flag_no:"]::before,
em[data-emoji="flag_no"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f4.svg");
}

em[data-emoji=":flag_om:"]::before,
em[data-emoji="flag_om"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f4-1f1f2.svg");
}

em[data-emoji=":flag_pk:"]::before,
em[data-emoji="flag_pk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f0.svg");
}

em[data-emoji=":flag_pw:"]::before,
em[data-emoji="flag_pw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1fc.svg");
}

em[data-emoji=":flag_ps:"]::before,
em[data-emoji="flag_ps"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f8.svg");
}

em[data-emoji=":flag_pa:"]::before,
em[data-emoji="flag_pa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1e6.svg");
}

em[data-emoji=":flag_pg:"]::before,
em[data-emoji="flag_pg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ec.svg");
}

em[data-emoji=":flag_py:"]::before,
em[data-emoji="flag_py"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1fe.svg");
}

em[data-emoji=":flag_pe:"]::before,
em[data-emoji="flag_pe"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ea.svg");
}

em[data-emoji=":flag_ph:"]::before,
em[data-emoji="flag_ph"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ed.svg");
}

em[data-emoji=":flag_pn:"]::before,
em[data-emoji="flag_pn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f3.svg");
}

em[data-emoji=":flag_pl:"]::before,
em[data-emoji="flag_pl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f1.svg");
}

em[data-emoji=":flag_pt:"]::before,
em[data-emoji="flag_pt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f9.svg");
}

em[data-emoji=":flag_pr:"]::before,
em[data-emoji="flag_pr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f7.svg");
}

em[data-emoji=":flag_qa:"]::before,
em[data-emoji="flag_qa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f6-1f1e6.svg");
}

em[data-emoji=":flag_re:"]::before,
em[data-emoji="flag_re"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1ea.svg");
}

em[data-emoji=":flag_ro:"]::before,
em[data-emoji="flag_ro"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1f4.svg");
}

em[data-emoji=":flag_ru:"]::before,
em[data-emoji="flag_ru"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1fa.svg");
}

em[data-emoji=":flag_rw:"]::before,
em[data-emoji="flag_rw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1fc.svg");
}

em[data-emoji=":flag_ws:"]::before,
em[data-emoji="flag_ws"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fc-1f1f8.svg");
}

em[data-emoji=":flag_sm:"]::before,
em[data-emoji="flag_sm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f2.svg");
}

em[data-emoji=":flag_st:"]::before,
em[data-emoji="flag_st"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f9.svg");
}

em[data-emoji=":flag_sa:"]::before,
em[data-emoji="flag_sa"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e6.svg");
}

em[data-emoji=":flag_sn:"]::before,
em[data-emoji="flag_sn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f3.svg");
}

em[data-emoji=":flag_rs:"]::before,
em[data-emoji="flag_rs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1f8.svg");
}

em[data-emoji=":flag_sc:"]::before,
em[data-emoji="flag_sc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e8.svg");
}

em[data-emoji=":flag_sl:"]::before,
em[data-emoji="flag_sl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f1.svg");
}

em[data-emoji=":flag_sg:"]::before,
em[data-emoji="flag_sg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ec.svg");
}

em[data-emoji=":flag_sx:"]::before,
em[data-emoji="flag_sx"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fd.svg");
}

em[data-emoji=":flag_sk:"]::before,
em[data-emoji="flag_sk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f0.svg");
}

em[data-emoji=":flag_si:"]::before,
em[data-emoji="flag_si"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ee.svg");
}

em[data-emoji=":flag_gs:"]::before,
em[data-emoji="flag_gs"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f8.svg");
}

em[data-emoji=":flag_sb:"]::before,
em[data-emoji="flag_sb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e7.svg");
}

em[data-emoji=":flag_so:"]::before,
em[data-emoji="flag_so"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f4.svg");
}

em[data-emoji=":flag_za:"]::before,
em[data-emoji="flag_za"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1e6.svg");
}

em[data-emoji=":flag_kr:"]::before,
em[data-emoji="flag_kr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f7.svg");
}

em[data-emoji=":flag_ss:"]::before,
em[data-emoji="flag_ss"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f8.svg");
}

em[data-emoji=":flag_es:"]::before,
em[data-emoji="flag_es"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f8.svg");
}

em[data-emoji=":flag_lk:"]::before,
em[data-emoji="flag_lk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f0.svg");
}

em[data-emoji=":flag_bl:"]::before,
em[data-emoji="flag_bl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f1.svg");
}

em[data-emoji=":flag_sh:"]::before,
em[data-emoji="flag_sh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ed.svg");
}

em[data-emoji=":flag_kn:"]::before,
em[data-emoji="flag_kn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f3.svg");
}

em[data-emoji=":flag_lc:"]::before,
em[data-emoji="flag_lc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e8.svg");
}

em[data-emoji=":flag_pm:"]::before,
em[data-emoji="flag_pm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f2.svg");
}

em[data-emoji=":flag_vc:"]::before,
em[data-emoji="flag_vc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1e8.svg");
}

em[data-emoji=":flag_sd:"]::before,
em[data-emoji="flag_sd"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e9.svg");
}

em[data-emoji=":flag_sr:"]::before,
em[data-emoji="flag_sr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f7.svg");
}

em[data-emoji=":flag_sz:"]::before,
em[data-emoji="flag_sz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ff.svg");
}

em[data-emoji=":flag_se:"]::before,
em[data-emoji="flag_se"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ea.svg");
}

em[data-emoji=":flag_ch:"]::before,
em[data-emoji="flag_ch"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ed.svg");
}

em[data-emoji=":flag_sy:"]::before,
em[data-emoji="flag_sy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fe.svg");
}

em[data-emoji=":flag_tw:"]::before,
em[data-emoji="flag_tw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1fc.svg");
}

em[data-emoji=":flag_tj:"]::before,
em[data-emoji="flag_tj"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ef.svg");
}

em[data-emoji=":flag_tz:"]::before,
em[data-emoji="flag_tz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ff.svg");
}

em[data-emoji=":flag_th:"]::before,
em[data-emoji="flag_th"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ed.svg");
}

em[data-emoji=":flag_tl:"]::before,
em[data-emoji="flag_tl"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f1.svg");
}

em[data-emoji=":flag_tg:"]::before,
em[data-emoji="flag_tg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ec.svg");
}

em[data-emoji=":flag_tk:"]::before,
em[data-emoji="flag_tk"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f0.svg");
}

em[data-emoji=":flag_to:"]::before,
em[data-emoji="flag_to"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f4.svg");
}

em[data-emoji=":flag_tt:"]::before,
em[data-emoji="flag_tt"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f9.svg");
}

em[data-emoji=":flag_tn:"]::before,
em[data-emoji="flag_tn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f3.svg");
}

em[data-emoji=":flag_tr:"]::before,
em[data-emoji="flag_tr"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f7.svg");
}

em[data-emoji=":flag_tm:"]::before,
em[data-emoji="flag_tm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f2.svg");
}

em[data-emoji=":flag_tc:"]::before,
em[data-emoji="flag_tc"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e8.svg");
}

em[data-emoji=":flag_vi:"]::before,
em[data-emoji="flag_vi"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ee.svg");
}

em[data-emoji=":flag_tv:"]::before,
em[data-emoji="flag_tv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1fb.svg");
}

em[data-emoji=":flag_ug:"]::before,
em[data-emoji="flag_ug"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1ec.svg");
}

em[data-emoji=":flag_ua:"]::before,
em[data-emoji="flag_ua"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1e6.svg");
}

em[data-emoji=":flag_ae:"]::before,
em[data-emoji="flag_ae"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ea.svg");
}

em[data-emoji=":flag_gb:"]::before,
em[data-emoji="flag_gb"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e7.svg");
}

em[data-emoji=":england:"]::before,
em[data-emoji="england"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0065-e006e-e0067-e007f.svg");
}

em[data-emoji=":scotland:"]::before,
em[data-emoji="scotland"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0073-e0063-e0074-e007f.svg");
}

em[data-emoji=":wales:"]::before,
em[data-emoji="wales"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0077-e006c-e0073-e007f.svg");
}

em[data-emoji=":flag_us:"]::before,
em[data-emoji="flag_us"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f8.svg");
}

em[data-emoji=":flag_uy:"]::before,
em[data-emoji="flag_uy"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1fe.svg");
}

em[data-emoji=":flag_uz:"]::before,
em[data-emoji="flag_uz"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1ff.svg");
}

em[data-emoji=":flag_vu:"]::before,
em[data-emoji="flag_vu"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1fa.svg");
}

em[data-emoji=":flag_va:"]::before,
em[data-emoji="flag_va"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1e6.svg");
}

em[data-emoji=":flag_ve:"]::before,
em[data-emoji="flag_ve"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ea.svg");
}

em[data-emoji=":flag_vn:"]::before,
em[data-emoji="flag_vn"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1f3.svg");
}

em[data-emoji=":flag_wf:"]::before,
em[data-emoji="flag_wf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fc-1f1eb.svg");
}

em[data-emoji=":flag_eh:"]::before,
em[data-emoji="flag_eh"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ed.svg");
}

em[data-emoji=":flag_ye:"]::before,
em[data-emoji="flag_ye"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fe-1f1ea.svg");
}

em[data-emoji=":flag_zm:"]::before,
em[data-emoji="flag_zm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1f2.svg");
}

em[data-emoji=":flag_zw:"]::before,
em[data-emoji="flag_zw"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1fc.svg");
}

em[data-emoji=":flag_ac:"]::before,
em[data-emoji="flag_ac"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1e8.svg");
}

em[data-emoji=":flag_bv:"]::before,
em[data-emoji="flag_bv"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fb.svg");
}

em[data-emoji=":flag_cp:"]::before,
em[data-emoji="flag_cp"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f5.svg");
}

em[data-emoji=":flag_ea:"]::before,
em[data-emoji="flag_ea"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1e6.svg");
}

em[data-emoji=":flag_dg:"]::before,
em[data-emoji="flag_dg"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ec.svg");
}

em[data-emoji=":flag_hm:"]::before,
em[data-emoji="flag_hm"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f2.svg");
}

em[data-emoji=":flag_mf:"]::before,
em[data-emoji="flag_mf"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1eb.svg");
}

em[data-emoji=":flag_sj:"]::before,
em[data-emoji="flag_sj"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ef.svg");
}

em[data-emoji=":flag_ta:"]::before,
em[data-emoji="flag_ta"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e6.svg");
}

em[data-emoji=":flag_um:"]::before,
em[data-emoji="flag_um"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f2.svg");
}

em[data-emoji=":united_nations:"]::before,
em[data-emoji="united_nations"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f3.svg");
}

em[data-emoji=":tone1:"]::before,
em[data-emoji="tone1"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3fb.svg");
}

em[data-emoji=":tone2:"]::before,
em[data-emoji="tone2"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3fc.svg");
}

em[data-emoji=":tone3:"]::before,
em[data-emoji="tone3"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3fd.svg");
}

em[data-emoji=":tone4:"]::before,
em[data-emoji="tone4"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3fe.svg");
}

em[data-emoji=":tone5:"]::before,
em[data-emoji="tone5"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3ff.svg");
}

em[data-emoji=":regional_indicator_z:"]::before,
em[data-emoji="regional_indicator_z"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff.svg");
}

em[data-emoji=":regional_indicator_y:"]::before,
em[data-emoji="regional_indicator_y"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fe.svg");
}

em[data-emoji=":regional_indicator_x:"]::before,
em[data-emoji="regional_indicator_x"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fd.svg");
}

em[data-emoji=":regional_indicator_w:"]::before,
em[data-emoji="regional_indicator_w"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fc.svg");
}

em[data-emoji=":regional_indicator_v:"]::before,
em[data-emoji="regional_indicator_v"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb.svg");
}

em[data-emoji=":regional_indicator_u:"]::before,
em[data-emoji="regional_indicator_u"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa.svg");
}

em[data-emoji=":regional_indicator_t:"]::before,
em[data-emoji="regional_indicator_t"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9.svg");
}

em[data-emoji=":regional_indicator_s:"]::before,
em[data-emoji="regional_indicator_s"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8.svg");
}

em[data-emoji=":regional_indicator_r:"]::before,
em[data-emoji="regional_indicator_r"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7.svg");
}

em[data-emoji=":regional_indicator_q:"]::before,
em[data-emoji="regional_indicator_q"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f6.svg");
}

em[data-emoji=":regional_indicator_p:"]::before,
em[data-emoji="regional_indicator_p"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5.svg");
}

em[data-emoji=":regional_indicator_o:"]::before,
em[data-emoji="regional_indicator_o"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f4.svg");
}

em[data-emoji=":regional_indicator_n:"]::before,
em[data-emoji="regional_indicator_n"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3.svg");
}

em[data-emoji=":regional_indicator_m:"]::before,
em[data-emoji="regional_indicator_m"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2.svg");
}

em[data-emoji=":regional_indicator_l:"]::before,
em[data-emoji="regional_indicator_l"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1.svg");
}

em[data-emoji=":regional_indicator_k:"]::before,
em[data-emoji="regional_indicator_k"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0.svg");
}

em[data-emoji=":regional_indicator_j:"]::before,
em[data-emoji="regional_indicator_j"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef.svg");
}

em[data-emoji=":regional_indicator_i:"]::before,
em[data-emoji="regional_indicator_i"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee.svg");
}

em[data-emoji=":regional_indicator_h:"]::before,
em[data-emoji="regional_indicator_h"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed.svg");
}

em[data-emoji=":regional_indicator_g:"]::before,
em[data-emoji="regional_indicator_g"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec.svg");
}

em[data-emoji=":regional_indicator_f:"]::before,
em[data-emoji="regional_indicator_f"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb.svg");
}

em[data-emoji=":regional_indicator_e:"]::before,
em[data-emoji="regional_indicator_e"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea.svg");
}

em[data-emoji=":regional_indicator_d:"]::before,
em[data-emoji="regional_indicator_d"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9.svg");
}

em[data-emoji=":regional_indicator_c:"]::before,
em[data-emoji="regional_indicator_c"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8.svg");
}

em[data-emoji=":regional_indicator_b:"]::before,
em[data-emoji="regional_indicator_b"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7.svg");
}

em[data-emoji=":regional_indicator_a:"]::before,
em[data-emoji="regional_indicator_a"]::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6.svg");
}

/* rtl:end:ignore */

/*******************************
             Emojis
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Flag
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Flag
*******************************/

i.flag:not(.icon) {
  speak: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

i.flag:not(.icon)::before {
  content: "\00A0\00A0\00A0\00A0\00A0\00A0\00A0";
  display: inline-block;
  line-height: 1em;
  background-repeat: no-repeat;
  background-position: center center;
}

i.flag.small {
  font-size: 1.5em;
  vertical-align: middle;
}

i.flag.medium {
  font-size: 3em;
  vertical-align: middle;
}

i.flag.large {
  font-size: 6em;
  vertical-align: middle;
}

i.flag.big {
  font-size: 7.5em;
  vertical-align: middle;
}

i.flag.huge {
  font-size: 9em;
  vertical-align: middle;
}

i.flag.massive {
  font-size: 12em;
  vertical-align: middle;
}

/* rtl:begin:ignore */

i.flag.transgender::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f3-fe0f-200d-26a7-fe0f.svg");
}

i.flag.rainbow::before,
i.flag.pride::before,
i.flag.lgbt::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f3-fe0f-200d-1f308.svg");
}

i.flag.pirate::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-200d-2620-fe0f.svg");
}

i.flag.af::before,
i.flag.afghanistan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1eb.svg");
}

i.flag.ax::before,
i.flag.aland.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fd.svg");
}

i.flag.al::before,
i.flag.albania::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f1.svg");
}

i.flag.dz::before,
i.flag.algeria::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ff.svg");
}

i.flag.as::before,
i.flag.american.samoa::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f8.svg");
}

i.flag.ad::before,
i.flag.andorra::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1e9.svg");
}

i.flag.ao::before,
i.flag.angola::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f4.svg");
}

i.flag.ai::before,
i.flag.anguilla::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ee.svg");
}

i.flag.aq::before,
i.flag.antarctica::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f6.svg");
}

i.flag.ag::before,
i.flag.antigua.and.barbuda::before,
i.flag.antigua::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ec.svg");
}

i.flag.ar::before,
i.flag.argentina::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f7.svg");
}

i.flag.am::before,
i.flag.armenia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f2.svg");
}

i.flag.aw::before,
i.flag.aruba::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fc.svg");
}

i.flag.au::before,
i.flag.australia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1fa.svg");
}

i.flag.at::before,
i.flag.austria::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1f9.svg");
}

i.flag.az::before,
i.flag.azerbaijan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ff.svg");
}

i.flag.bs::before,
i.flag.bahamas::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f8.svg");
}

i.flag.bh::before,
i.flag.bahrain::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ed.svg");
}

i.flag.bd::before,
i.flag.bangladesh::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e9.svg");
}

i.flag.bb::before,
i.flag.barbados::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e7.svg");
}

i.flag.by::before,
i.flag.belarus::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fe.svg");
}

i.flag.be::before,
i.flag.belgium::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ea.svg");
}

i.flag.bz::before,
i.flag.belize::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ff.svg");
}

i.flag.bj::before,
i.flag.benin::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ef.svg");
}

i.flag.bm::before,
i.flag.bermuda::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f2.svg");
}

i.flag.bt::before,
i.flag.bhutan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f9.svg");
}

i.flag.bo::before,
i.flag.bolivia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f4.svg");
}

i.flag.ba::before,
i.flag.bosnia.and.herzegovina::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1e6.svg");
}

i.flag.bw::before,
i.flag.botswana::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fc.svg");
}

i.flag.br::before,
i.flag.brazil::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f7.svg");
}

i.flag.io::before,
i.flag.british.indian.ocean.territory::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f4.svg");
}

i.flag.vg::before,
i.flag.british.virgin.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ec.svg");
}

i.flag.bn::before,
i.flag.brunei::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f3.svg");
}

i.flag.bg::before,
i.flag.bulgaria::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ec.svg");
}

i.flag.bf::before,
i.flag.burkina.faso::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1eb.svg");
}

i.flag.bi::before,
i.flag.burundi::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1ee.svg");
}

i.flag.kh::before,
i.flag.cambodia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ed.svg");
}

i.flag.cm::before,
i.flag.cameroon::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f2.svg");
}

i.flag.ca::before,
i.flag.canada::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e6.svg");
}

i.flag.ic::before,
i.flag.canary.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1e8.svg");
}

i.flag.cv::before,
i.flag.cape.verde::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fb.svg");
}

i.flag.bq::before,
i.flag.caribbean.netherlands::before,
i.flag.an::before,
i.flag.netherlands.antilles::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f6.svg");
}

i.flag.ky::before,
i.flag.cayman.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1fe.svg");
}

i.flag.cf::before,
i.flag.central.african.republic::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1eb.svg");
}

i.flag.td::before,
i.flag.chad::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e9.svg");
}

i.flag.cl::before,
i.flag.chile::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f1.svg");
}

i.flag.cn::before,
i.flag.china::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f3.svg");
}

i.flag.cx::before,
i.flag.christmas.island::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fd.svg");
}

i.flag.cc::before,
i.flag.cocos.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e8.svg");
}

i.flag.co::before,
i.flag.colombia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f4.svg");
}

i.flag.km::before,
i.flag.comoros::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f2.svg");
}

i.flag.cg::before,
i.flag.congo.brazzaville::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ec.svg");
}

i.flag.cd::before,
i.flag.congo.kinshasa::before,
i.flag.congo::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1e9.svg");
}

i.flag.ck::before,
i.flag.cook.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f0.svg");
}

i.flag.cr::before,
i.flag.costa.rica::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f7.svg");
}

i.flag.ci::before,
i.flag.côte.d’ivoire::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ee.svg");
}

i.flag.hr::before,
i.flag.croatia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f7.svg");
}

i.flag.cu::before,
i.flag.cuba::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fa.svg");
}

i.flag.cw::before,
i.flag.curacao::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fc.svg");
}

i.flag.cy::before,
i.flag.cyprus::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1fe.svg");
}

i.flag.cz::before,
i.flag.czechia::before,
i.flag.czech.republic::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ff.svg");
}

i.flag.dk::before,
i.flag.denmark::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f0.svg");
}

i.flag.dj::before,
i.flag.djibouti::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ef.svg");
}

i.flag.dm::before,
i.flag.dominica::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f2.svg");
}

i.flag.do::before,
i.flag.dominican.republic::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1f4.svg");
}

i.flag.ec::before,
i.flag.ecuador::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1e8.svg");
}

i.flag.eg::before,
i.flag.egypt::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ec.svg");
}

i.flag.sv::before,
i.flag.el.salvador::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fb.svg");
}

i.flag.gq::before,
i.flag.equatorial.guinea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f6.svg");
}

i.flag.er::before,
i.flag.eritrea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f7.svg");
}

i.flag.ee::before,
i.flag.estonia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ea.svg");
}

i.flag.et::before,
i.flag.ethiopia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f9.svg");
}

i.flag.eu::before,
i.flag.european.union::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1fa.svg");
}

i.flag.fk::before,
i.flag.falkland.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f0.svg");
}

i.flag.fo::before,
i.flag.faroe.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f4.svg");
}

i.flag.fj::before,
i.flag.fiji::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1ef.svg");
}

i.flag.fi::before,
i.flag.finland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1ee.svg");
}

i.flag.fr::before,
i.flag.france::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f7.svg");
}

i.flag.gf::before,
i.flag.french.guiana::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1eb.svg");
}

i.flag.pf::before,
i.flag.french.polynesia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1eb.svg");
}

i.flag.tf::before,
i.flag.french.southern.territories::before,
i.flag.french.territories::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1eb.svg");
}

i.flag.ga::before,
i.flag.gabon::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e6.svg");
}

i.flag.gm::before,
i.flag.gambia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f2.svg");
}

i.flag.ge::before,
i.flag.georgia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ea.svg");
}

i.flag.de::before,
i.flag.germany::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ea.svg");
}

i.flag.gh::before,
i.flag.ghana::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ed.svg");
}

i.flag.gi::before,
i.flag.gibraltar::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ee.svg");
}

i.flag.gr::before,
i.flag.greece::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f7.svg");
}

i.flag.gl::before,
i.flag.greenland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f1.svg");
}

i.flag.gd::before,
i.flag.grenada::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e9.svg");
}

i.flag.gp::before,
i.flag.guadeloupe::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f5.svg");
}

i.flag.gu::before,
i.flag.guam::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fa.svg");
}

i.flag.gt::before,
i.flag.guatemala::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f9.svg");
}

i.flag.gg::before,
i.flag.guernsey::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1ec.svg");
}

i.flag.gn::before,
i.flag.guinea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f3.svg");
}

i.flag.gw::before,
i.flag.guinea-bissau::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fc.svg");
}

i.flag.gy::before,
i.flag.guyana::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1fe.svg");
}

i.flag.ht::before,
i.flag.haiti::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f9.svg");
}

i.flag.hn::before,
i.flag.honduras::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f3.svg");
}

i.flag.hk::before,
i.flag.hong.kong.sar.china::before,
i.flag.hong.kong::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f0.svg");
}

i.flag.hu::before,
i.flag.hungary::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1fa.svg");
}

i.flag.is::before,
i.flag.iceland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f8.svg");
}

i.flag.in::before,
i.flag.india::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f3.svg");
}

i.flag.id::before,
i.flag.indonesia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1e9.svg");
}

i.flag.ir::before,
i.flag.iran::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f7.svg");
}

i.flag.iq::before,
i.flag.iraq::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f6.svg");
}

i.flag.ie::before,
i.flag.ireland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1ea.svg");
}

i.flag.im::before,
i.flag.isle.of.man::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f2.svg");
}

i.flag.il::before,
i.flag.israel::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f1.svg");
}

i.flag.it::before,
i.flag.italy::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ee-1f1f9.svg");
}

i.flag.jm::before,
i.flag.jamaica::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f2.svg");
}

i.flag.jp::before,
i.flag.japan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f5.svg");
}

i.flag.je::before,
i.flag.jersey::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1ea.svg");
}

i.flag.jo::before,
i.flag.jordan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ef-1f1f4.svg");
}

i.flag.kz::before,
i.flag.kazakhstan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ff.svg");
}

i.flag.ke::before,
i.flag.kenya::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ea.svg");
}

i.flag.ki::before,
i.flag.kiribati::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ee.svg");
}

i.flag.xk::before,
i.flag.kosovo::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fd-1f1f0.svg");
}

i.flag.kw::before,
i.flag.kuwait::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1fc.svg");
}

i.flag.kg::before,
i.flag.kyrgyzstan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1ec.svg");
}

i.flag.la::before,
i.flag.laos::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e6.svg");
}

i.flag.lv::before,
i.flag.latvia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fb.svg");
}

i.flag.lb::before,
i.flag.lebanon::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e7.svg");
}

i.flag.ls::before,
i.flag.lesotho::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f8.svg");
}

i.flag.lr::before,
i.flag.liberia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f7.svg");
}

i.flag.ly::before,
i.flag.libya::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fe.svg");
}

i.flag.li::before,
i.flag.liechtenstein::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1ee.svg");
}

i.flag.lt::before,
i.flag.lithuania::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f9.svg");
}

i.flag.lu::before,
i.flag.luxembourg::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1fa.svg");
}

i.flag.mo::before,
i.flag.macao.sar.china::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f4.svg");
}

i.flag.mk::before,
i.flag.macedonia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f0.svg");
}

i.flag.mg::before,
i.flag.madagascar::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ec.svg");
}

i.flag.mw::before,
i.flag.malawi::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fc.svg");
}

i.flag.my::before,
i.flag.malaysia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fe.svg");
}

i.flag.mv::before,
i.flag.maldives::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fb.svg");
}

i.flag.ml::before,
i.flag.mali::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f1.svg");
}

i.flag.mt::before,
i.flag.malta::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f9.svg");
}

i.flag.mh::before,
i.flag.marshall.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ed.svg");
}

i.flag.mq::before,
i.flag.martinique::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f6.svg");
}

i.flag.mr::before,
i.flag.mauritania::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f7.svg");
}

i.flag.mu::before,
i.flag.mauritius::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fa.svg");
}

i.flag.yt::before,
i.flag.mayotte::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fe-1f1f9.svg");
}

i.flag.mx::before,
i.flag.mexico::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1fd.svg");
}

i.flag.fm::before,
i.flag.micronesia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1eb-1f1f2.svg");
}

i.flag.md::before,
i.flag.moldova::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e9.svg");
}

i.flag.mc::before,
i.flag.monaco::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e8.svg");
}

i.flag.mn::before,
i.flag.mongolia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f3.svg");
}

i.flag.me::before,
i.flag.montenegro::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ea.svg");
}

i.flag.ms::before,
i.flag.montserrat::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f8.svg");
}

i.flag.ma::before,
i.flag.morocco::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1e6.svg");
}

i.flag.mz::before,
i.flag.mozambique::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1ff.svg");
}

i.flag.mm::before,
i.flag.myanmar::before,
i.flag.burma::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f2.svg");
}

i.flag.na::before,
i.flag.namibia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1e6.svg");
}

i.flag.nr::before,
i.flag.nauru::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f7.svg");
}

i.flag.np::before,
i.flag.nepal::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f5.svg");
}

i.flag.nl::before,
i.flag.netherlands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f1.svg");
}

i.flag.nc::before,
i.flag.new.caledonia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1e8.svg");
}

i.flag.nz::before,
i.flag.new.zealand::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ff.svg");
}

i.flag.ni::before,
i.flag.nicaragua::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ee.svg");
}

i.flag.ne::before,
i.flag.niger::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ea.svg");
}

i.flag.ng::before,
i.flag.nigeria::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1ec.svg");
}

i.flag.nu::before,
i.flag.niue::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1fa.svg");
}

i.flag.nf::before,
i.flag.norfolk.island::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1eb.svg");
}

i.flag.kp::before,
i.flag.north.korea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f5.svg");
}

i.flag.mp::before,
i.flag.northern.mariana.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1f5.svg");
}

i.flag.no::before,
i.flag.norway::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f3-1f1f4.svg");
}

i.flag.om::before,
i.flag.oman::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f4-1f1f2.svg");
}

i.flag.pk::before,
i.flag.pakistan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f0.svg");
}

i.flag.pw::before,
i.flag.palau::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1fc.svg");
}

i.flag.ps::before,
i.flag.palestinian.territories::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f8.svg");
}

i.flag.pa::before,
i.flag.panama::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1e6.svg");
}

i.flag.pg::before,
i.flag.papua.new.guinea::before,
i.flag.new.guinea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ec.svg");
}

i.flag.py::before,
i.flag.paraguay::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1fe.svg");
}

i.flag.pe::before,
i.flag.peru::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ea.svg");
}

i.flag.ph::before,
i.flag.philippines::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1ed.svg");
}

i.flag.pn::before,
i.flag.pitcairn.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f3.svg");
}

i.flag.pl::before,
i.flag.poland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f1.svg");
}

i.flag.pt::before,
i.flag.portugal::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f9.svg");
}

i.flag.pr::before,
i.flag.puerto.rico::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f7.svg");
}

i.flag.qa::before,
i.flag.qatar::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f6-1f1e6.svg");
}

i.flag.re::before,
i.flag.réunion::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1ea.svg");
}

i.flag.ro::before,
i.flag.romania::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1f4.svg");
}

i.flag.ru::before,
i.flag.russia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1fa.svg");
}

i.flag.rw::before,
i.flag.rwanda::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1fc.svg");
}

i.flag.ws::before,
i.flag.samoa::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fc-1f1f8.svg");
}

i.flag.sm::before,
i.flag.san.marino::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f2.svg");
}

i.flag.st::before,
i.flag.sao.tome.and.principe::before,
i.flag.samo.tome::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f9.svg");
}

i.flag.sa::before,
i.flag.saudi.arabia::before,
i.flag.saudiarabia::before,
i.flag.saudi::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e6.svg");
}

i.flag.sn::before,
i.flag.senegal::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f3.svg");
}

i.flag.rs::before,
i.flag.serbia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f7-1f1f8.svg");
}

i.flag.sc::before,
i.flag.seychelles::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e8.svg");
}

i.flag.sl::before,
i.flag.sierra.leone::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f1.svg");
}

i.flag.sg::before,
i.flag.singapore::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ec.svg");
}

i.flag.sx::before,
i.flag.sint.maarten::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fd.svg");
}

i.flag.sk::before,
i.flag.slovakia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f0.svg");
}

i.flag.si::before,
i.flag.slovenia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ee.svg");
}

i.flag.gs::before,
i.flag.south.georgia.and.south.sandwich.islands::before,
i.flag.sandwich.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1f8.svg");
}

i.flag.sb::before,
i.flag.solomon.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e7.svg");
}

i.flag.so::before,
i.flag.somalia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f4.svg");
}

i.flag.za::before,
i.flag.south.africa::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1e6.svg");
}

i.flag.kr::before,
i.flag.south.korea::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f7.svg");
}

i.flag.ss::before,
i.flag.south.sudan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f8.svg");
}

i.flag.es::before,
i.flag.spain::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1f8.svg");
}

i.flag.lk::before,
i.flag.sri.lanka::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1f0.svg");
}

i.flag.bl::before,
i.flag.st.barthelemy::before,
i.flag.saint.barth::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1f1.svg");
}

i.flag.sh::before,
i.flag.st.helena::before,
i.flag.saint.helena::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ed.svg");
}

i.flag.kn::before,
i.flag.st.kitts.and.nevis::before,
i.flag.saint.kitts.and.navis::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f0-1f1f3.svg");
}

i.flag.lc::before,
i.flag.st.lucia::before,
i.flag.saint.lucia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f1-1f1e8.svg");
}

i.flag.pm::before,
i.flag.st.pierre.and.miquelon::before,
i.flag.saint.pierre::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f5-1f1f2.svg");
}

i.flag.vc::before,
i.flag.st.vincent.and.grenadines::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1e8.svg");
}

i.flag.sd::before,
i.flag.sudan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1e9.svg");
}

i.flag.sr::before,
i.flag.suriname::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1f7.svg");
}

i.flag.sz::before,
i.flag.eswatini::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ff.svg");
}

i.flag.se::before,
i.flag.sweden::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ea.svg");
}

i.flag.ch::before,
i.flag.switzerland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1ed.svg");
}

i.flag.sy::before,
i.flag.syria::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1fe.svg");
}

i.flag.tw::before,
i.flag.taiwan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1fc.svg");
}

i.flag.tj::before,
i.flag.tajikistan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ef.svg");
}

i.flag.tz::before,
i.flag.tanzania::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ff.svg");
}

i.flag.th::before,
i.flag.thailand::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ed.svg");
}

i.flag.tl::before,
i.flag.timor-leste::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f1.svg");
}

i.flag.tg::before,
i.flag.togo::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1ec.svg");
}

i.flag.tk::before,
i.flag.tokelau::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f0.svg");
}

i.flag.to::before,
i.flag.tonga::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f4.svg");
}

i.flag.tt::before,
i.flag.trinidad.and.tobago::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f9.svg");
}

i.flag.tn::before,
i.flag.tunisia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f3.svg");
}

i.flag.tr::before,
i.flag.turkey::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f7.svg");
}

i.flag.tm::before,
i.flag.turkmenistan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1f2.svg");
}

i.flag.tc::before,
i.flag.turks.and.caicos.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e8.svg");
}

i.flag.vi::before,
i.flag.us.virgin.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ee.svg");
}

i.flag.tv::before,
i.flag.tuvalu::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1fb.svg");
}

i.flag.ug::before,
i.flag.uganda::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1ec.svg");
}

i.flag.ua::before,
i.flag.ukraine::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1e6.svg");
}

i.flag.ae::before,
i.flag.united.arab.emirates::before,
i.flag.uae::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1ea.svg");
}

i.flag.gb::before,
i.flag.united.kingdom::before,
i.flag.uk::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ec-1f1e7.svg");
}

i.flag.gb.eng::before,
i.flag.england::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0065-e006e-e0067-e007f.svg");
}

i.flag.gb.sct::before,
i.flag.scotland::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0073-e0063-e0074-e007f.svg");
}

i.flag.gb.wls::before,
i.flag.wales::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f3f4-e0067-e0062-e0077-e006c-e0073-e007f.svg");
}

i.flag.us::before,
i.flag.united.states::before,
i.flag.america::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f8.svg");
}

i.flag.uy::before,
i.flag.uruguay::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1fe.svg");
}

i.flag.uz::before,
i.flag.uzbekistan::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1ff.svg");
}

i.flag.vu::before,
i.flag.vanuatu::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1fa.svg");
}

i.flag.va::before,
i.flag.vatican.city::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1e6.svg");
}

i.flag.ve::before,
i.flag.venezuela::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1ea.svg");
}

i.flag.vn::before,
i.flag.vietnam::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fb-1f1f3.svg");
}

i.flag.wf::before,
i.flag.wallis.and.futuna::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fc-1f1eb.svg");
}

i.flag.eh::before,
i.flag.western.sahara::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1ed.svg");
}

i.flag.ye::before,
i.flag.yemen::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fe-1f1ea.svg");
}

i.flag.zm::before,
i.flag.zambia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1f2.svg");
}

i.flag.zw::before,
i.flag.zimbabwe::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ff-1f1fc.svg");
}

i.flag.ac::before,
i.flag.ascension.island::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e6-1f1e8.svg");
}

i.flag.bv::before,
i.flag.bouvet.island::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e7-1f1fb.svg");
}

i.flag.cp::before,
i.flag.clipperton.island::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e8-1f1f5.svg");
}

i.flag.ea::before,
i.flag.ceuta.and.melilla::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ea-1f1e6.svg");
}

i.flag.dg::before,
i.flag.diego.garcia::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1e9-1f1ec.svg");
}

i.flag.hm::before,
i.flag.heard.and.mcdonald.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1ed-1f1f2.svg");
}

i.flag.mf::before,
i.flag.st.martin::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f2-1f1eb.svg");
}

i.flag.sj::before,
i.flag.svalbard.and.jan.mayen::before,
i.flag.svalbard::before,
i.flag.jan.mayen::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f8-1f1ef.svg");
}

i.flag.ta::before,
i.flag.tristan.da.cunha::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1f9-1f1e6.svg");
}

i.flag.um::before,
i.flag.us.outlying.islands::before,
i.flag.minor.islands::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f2.svg");
}

i.flag.un::before,
i.flag.united.nations::before,
i.flag.united.nations::before {
  background-image: url("https://cdn.jsdelivr.net/gh/twitter/twemoji@latest/assets/svg/1f1fa-1f1f3.svg");
}

/* rtl:end:ignore */

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Header
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Header
*******************************/

/* Standard */

.ui.header {
  border: none;
  margin: calc(2rem - 0.1428571428571429em) 0 1rem;
  padding: 0 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-weight: bold;
  line-height: 1.28571429em;
  text-transform: none;
  color: rgba(0, 0, 0, 0.87);
}

.ui.header:first-child {
  margin-top: -0.14285714em;
}

.ui.header:last-child {
  margin-bottom: 0;
}

/* --------------
       Sub Header
    --------------- */

.ui.header .sub.header {
  display: block;
  font-weight: normal;
  padding: 0;
  margin: 0;
  font-size: 1rem;
  line-height: 1.2em;
  color: rgba(0, 0, 0, 0.6);
}

/* --------------
      Icon
--------------- */

.ui.header > .icons,
.ui.header > i.icon {
  display: table-cell;
  opacity: 1;
  font-size: 1.5em;
  padding-top: 0;
  vertical-align: middle;
}

/* With Text Node */

.ui.header:not(.icon) > .icons:only-child,
.ui.header:not(.icon) > i.icon:only-child {
  display: inline-block;
  padding: 0;
  margin-right: 0.75rem;
}

/* -------------------
        Image
-------------------- */

.ui.header > .image:not(.icon),
.ui.header > img {
  display: inline-block;
  margin-top: 0.14285714em;
  width: 2.5em;
  height: auto;
  vertical-align: middle;
}

.ui.header > .image:not(.icon):only-child,
.ui.header > img:only-child {
  margin-right: 0.75rem;
}

/* --------------
     Content
--------------- */

.ui.header .content {
  display: inline-block;
  vertical-align: top;
}

/* After Image */

.ui.header > img + .content,
.ui.header > .image + .content {
  padding-left: 0.75rem;
  vertical-align: middle;
}

/* After Icon */

.ui.header:not(.icon):not(.centered):not(.aligned) > .icons + .content,
.ui.header:not(.icon):not(.centered):not(.aligned) > i.icon + .content {
  padding-left: 0.75rem;
  display: table-cell;
  vertical-align: middle;
}

/* --------------
 Loose Coupling
--------------- */

.ui.header .ui.label {
  font-size: "";
  margin-left: 0.5rem;
  vertical-align: middle;
}

/* Positioning */

.ui.header + p {
  margin-top: 0;
}

/*******************************
            Types
*******************************/

/* --------------
     Page
--------------- */

h1.ui.header {
  font-size: 2rem;
}

h1.ui.header .sub.header {
  font-size: 1.14285714rem;
}

h2.ui.header {
  font-size: 1.71428571rem;
}

h2.ui.header .sub.header {
  font-size: 1.14285714rem;
}

h3.ui.header {
  font-size: 1.28571429rem;
}

h3.ui.header .sub.header {
  font-size: 1rem;
}

h4.ui.header {
  font-size: 1.07142857rem;
}

h4.ui.header .sub.header {
  font-size: 1rem;
}

h5.ui.header {
  font-size: 1rem;
}

h5.ui.header .sub.header {
  font-size: 0.92857143rem;
}

h6.ui.header {
  font-size: 0.85714286rem;
}

h6.ui.header .sub.header {
  font-size: 0.92857143rem;
}

/* --------------
 Content Heading
--------------- */

.ui.mini.header {
  font-size: 0.85714286em;
}

.ui.mini.header .sub.header {
  font-size: 0.92857143rem;
}

.ui.mini.sub.header {
  font-size: 0.78571429em;
}

.ui.tiny.header {
  font-size: 1em;
}

.ui.tiny.header .sub.header {
  font-size: 0.92857143rem;
}

.ui.tiny.sub.header {
  font-size: 0.78571429em;
}

.ui.small.header {
  font-size: 1.07142857em;
}

.ui.small.header .sub.header {
  font-size: 1rem;
}

.ui.small.sub.header {
  font-size: 0.78571429em;
}

.ui.large.header {
  font-size: 1.71428571em;
}

.ui.large.header .sub.header {
  font-size: 1.14285714rem;
}

.ui.large.sub.header {
  font-size: 0.92857143em;
}

.ui.big.header {
  font-size: 1.85714286em;
}

.ui.big.header .sub.header {
  font-size: 1.14285714rem;
}

.ui.big.sub.header {
  font-size: 1em;
}

.ui.huge.header {
  font-size: 2em;
  min-height: 1em;
}

.ui.huge.header .sub.header {
  font-size: 1.14285714rem;
}

.ui.huge.sub.header {
  font-size: 1em;
}

.ui.massive.header {
  font-size: 2.28571429em;
  min-height: 1em;
}

.ui.massive.header .sub.header {
  font-size: 1.42857143rem;
}

.ui.massive.sub.header {
  font-size: 1.14285714em;
}

/* --------------
       Sub Heading
    --------------- */

.ui.sub.header {
  padding: 0;
  margin-bottom: 0.14285714rem;
  font-weight: bold;
  font-size: 0.85714286em;
  text-transform: uppercase;
  color: "";
}

/* -------------------
            Icon
    -------------------- */

.ui.icon.header {
  display: inline-block;
  text-align: center;
  margin: 2rem 0 1rem;
}

.ui.icon.header::after {
  content: "";
  display: block;
  height: 0;
  clear: both;
  visibility: hidden;
}

.ui.icon.header:first-child {
  margin-top: 0;
}

.ui.icon.header > .image.icon,
.ui.icon.header > .icons,
.ui.icon.header > i.icon {
  float: none;
  display: block;
  width: auto;
  height: auto;
  line-height: 1;
  padding: 0;
  font-size: 3em;
  margin: 0 auto 0.5rem;
  opacity: 1;
}

.ui.icon.header .corner.icon {
  font-size: calc(3em * 0.45);
}

.ui.icon.header .content {
  display: block;
  padding: 0;
}

.ui.icon.header > i.circular {
  font-size: 2em;
}

.ui.icon.header > i.square {
  font-size: 2em;
}

.ui.block.icon.header > .image.icon,
.ui.block.icon.header > .icons,
.ui.block.icon.header > i.icon {
  margin-bottom: 0;
}

.ui.icon.header.aligned {
  margin-left: auto;
  margin-right: auto;
  display: block;
}

/*******************************
            States
*******************************/

.ui.disabled.header {
  opacity: 0.45;
}

/*******************************
           Variations
*******************************/

/* -------------------
          Inverted
    -------------------- */

.ui.inverted.header {
  color: #fff;
}

.ui.inverted.header .sub.header {
  color: rgba(255, 255, 255, 0.8);
}

.ui.inverted.attached.header {
  background: #1b1c1d;
  box-shadow: none;
  border-color: transparent;
}

.ui.inverted.block.header {
  background: #545454 linear-gradient(transparent, rgba(0, 0, 0, 0.05));
  box-shadow: none;
  border-bottom: none;
}

/* -------------------
       Colors
-------------------- */

.ui.primary.header {
  color: #2185d0;
}

a.ui.primary.header:hover {
  color: #1678c2;
}

.ui.primary.dividing.header {
  border-bottom: 2px solid #2185d0;
}

.ui.inverted.primary.header.header.header {
  color: #54c8ff;
}

a.ui.inverted.primary.header.header.header:hover {
  color: #21b8ff;
}

.ui.inverted.primary.dividing.header {
  border-bottom: 2px solid #54c8ff;
}

.ui.secondary.header {
  color: #1b1c1d;
}

a.ui.secondary.header:hover {
  color: #27292a;
}

.ui.secondary.dividing.header {
  border-bottom: 2px solid #1b1c1d;
}

.ui.inverted.secondary.header.header.header {
  color: #545454;
}

a.ui.inverted.secondary.header.header.header:hover {
  color: #6e6e6e;
}

.ui.inverted.secondary.dividing.header {
  border-bottom: 2px solid #545454;
}

.ui.red.header {
  color: #db2828;
}

a.ui.red.header:hover {
  color: #d01919;
}

.ui.red.dividing.header {
  border-bottom: 2px solid #db2828;
}

.ui.inverted.red.header.header.header {
  color: #ff695e;
}

a.ui.inverted.red.header.header.header:hover {
  color: #ff392b;
}

.ui.inverted.red.dividing.header {
  border-bottom: 2px solid #ff695e;
}

.ui.orange.header {
  color: #f2711c;
}

a.ui.orange.header:hover {
  color: #f26202;
}

.ui.orange.dividing.header {
  border-bottom: 2px solid #f2711c;
}

.ui.inverted.orange.header.header.header {
  color: #ff851b;
}

a.ui.inverted.orange.header.header.header:hover {
  color: #e76b00;
}

.ui.inverted.orange.dividing.header {
  border-bottom: 2px solid #ff851b;
}

.ui.yellow.header {
  color: #fbbd08;
}

a.ui.yellow.header:hover {
  color: #eaae00;
}

.ui.yellow.dividing.header {
  border-bottom: 2px solid #fbbd08;
}

.ui.inverted.yellow.header.header.header {
  color: #ffe21f;
}

a.ui.inverted.yellow.header.header.header:hover {
  color: #ebcd00;
}

.ui.inverted.yellow.dividing.header {
  border-bottom: 2px solid #ffe21f;
}

.ui.olive.header {
  color: #b5cc18;
}

a.ui.olive.header:hover {
  color: #a7bd0d;
}

.ui.olive.dividing.header {
  border-bottom: 2px solid #b5cc18;
}

.ui.inverted.olive.header.header.header {
  color: #d9e778;
}

a.ui.inverted.olive.header.header.header:hover {
  color: #d2e745;
}

.ui.inverted.olive.dividing.header {
  border-bottom: 2px solid #d9e778;
}

.ui.green.header {
  color: #21ba45;
}

a.ui.green.header:hover {
  color: #16ab39;
}

.ui.green.dividing.header {
  border-bottom: 2px solid #21ba45;
}

.ui.inverted.green.header.header.header {
  color: #2ecc40;
}

a.ui.inverted.green.header.header.header:hover {
  color: #1ea92e;
}

.ui.inverted.green.dividing.header {
  border-bottom: 2px solid #2ecc40;
}

.ui.teal.header {
  color: #00b5ad;
}

a.ui.teal.header:hover {
  color: #009c95;
}

.ui.teal.dividing.header {
  border-bottom: 2px solid #00b5ad;
}

.ui.inverted.teal.header.header.header {
  color: #6dffff;
}

a.ui.inverted.teal.header.header.header:hover {
  color: #3affff;
}

.ui.inverted.teal.dividing.header {
  border-bottom: 2px solid #6dffff;
}

.ui.blue.header {
  color: #2185d0;
}

a.ui.blue.header:hover {
  color: #1678c2;
}

.ui.blue.dividing.header {
  border-bottom: 2px solid #2185d0;
}

.ui.inverted.blue.header.header.header {
  color: #54c8ff;
}

a.ui.inverted.blue.header.header.header:hover {
  color: #21b8ff;
}

.ui.inverted.blue.dividing.header {
  border-bottom: 2px solid #54c8ff;
}

.ui.violet.header {
  color: #6435c9;
}

a.ui.violet.header:hover {
  color: #5829bb;
}

.ui.violet.dividing.header {
  border-bottom: 2px solid #6435c9;
}

.ui.inverted.violet.header.header.header {
  color: #a291fb;
}

a.ui.inverted.violet.header.header.header:hover {
  color: #745aff;
}

.ui.inverted.violet.dividing.header {
  border-bottom: 2px solid #a291fb;
}

.ui.purple.header {
  color: #a333c8;
}

a.ui.purple.header:hover {
  color: #9627ba;
}

.ui.purple.dividing.header {
  border-bottom: 2px solid #a333c8;
}

.ui.inverted.purple.header.header.header {
  color: #dc73ff;
}

a.ui.inverted.purple.header.header.header:hover {
  color: #cf40ff;
}

.ui.inverted.purple.dividing.header {
  border-bottom: 2px solid #dc73ff;
}

.ui.pink.header {
  color: #e03997;
}

a.ui.pink.header:hover {
  color: #e61a8d;
}

.ui.pink.dividing.header {
  border-bottom: 2px solid #e03997;
}

.ui.inverted.pink.header.header.header {
  color: #ff8edf;
}

a.ui.inverted.pink.header.header.header:hover {
  color: #ff5bd1;
}

.ui.inverted.pink.dividing.header {
  border-bottom: 2px solid #ff8edf;
}

.ui.brown.header {
  color: #a5673f;
}

a.ui.brown.header:hover {
  color: #975b33;
}

.ui.brown.dividing.header {
  border-bottom: 2px solid #a5673f;
}

.ui.inverted.brown.header.header.header {
  color: #d67c1c;
}

a.ui.inverted.brown.header.header.header:hover {
  color: #b0620f;
}

.ui.inverted.brown.dividing.header {
  border-bottom: 2px solid #d67c1c;
}

.ui.grey.header {
  color: #767676;
}

a.ui.grey.header:hover {
  color: #838383;
}

.ui.grey.dividing.header {
  border-bottom: 2px solid #767676;
}

.ui.inverted.grey.header.header.header {
  color: #dcddde;
}

a.ui.inverted.grey.header.header.header:hover {
  color: #c2c4c5;
}

.ui.inverted.grey.dividing.header {
  border-bottom: 2px solid #dcddde;
}

.ui.black.header {
  color: #1b1c1d;
}

a.ui.black.header:hover {
  color: #27292a;
}

.ui.black.dividing.header {
  border-bottom: 2px solid #1b1c1d;
}

.ui.inverted.black.header.header.header {
  color: #545454;
}

a.ui.inverted.black.header.header.header:hover {
  color: #000000;
}

.ui.inverted.black.dividing.header {
  border-bottom: 2px solid #545454;
}

/* -------------------
           Aligned
    -------------------- */

.ui.left.aligned.header {
  text-align: left;
}

.ui.right.aligned.header {
  text-align: right;
}

.ui.centered.header,
.ui.center.aligned.header {
  text-align: center;
}

.ui.justified.header {
  text-align: justify;
}

.ui.justified.header::after {
  display: inline-block;
  content: "";
  width: 100%;
}

/* -------------------
           Floated
    -------------------- */

.ui.floated.header,
.ui[class*="left floated"].header {
  float: left;
  margin-top: 0;
  margin-right: 0.5em;
}

.ui[class*="right floated"].header {
  float: right;
  margin-top: 0;
  margin-left: 0.5em;
}

/* -------------------
           Fitted
    -------------------- */

.ui.fitted.header {
  padding: 0;
}

/* -------------------
          Dividing
    -------------------- */

.ui.dividing.header {
  padding-bottom: 0.21428571rem;
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.dividing.header .sub.header {
  padding-bottom: 0.21428571rem;
}

.ui.dividing.header i.icon {
  margin-bottom: 0;
}

.ui.inverted.dividing.header {
  border-bottom-color: rgba(255, 255, 255, 0.1);
}

/* -------------------
            Block
    -------------------- */

.ui.block.header {
  background: #f3f4f5;
  padding: 0.78571429rem 1rem;
  box-shadow: none;
  border: 1px solid #d4d4d5;
  border-radius: 0.28571429rem;
}

.ui.block.header:not(h1):not(h2):not(h3):not(h4):not(h5):not(h6) {
  font-size: 1rem;
}

.ui.mini.block.header {
  font-size: 0.78571429rem;
}

.ui.tiny.block.header {
  font-size: 0.85714286rem;
}

.ui.small.block.header {
  font-size: 0.92857143rem;
}

.ui.large.block.header {
  font-size: 1.14285714rem;
}

.ui.big.block.header {
  font-size: 1.28571429rem;
}

.ui.huge.block.header {
  font-size: 1.42857143rem;
}

.ui.massive.block.header {
  font-size: 1.71428571rem;
}

/* -------------------
           Attached
    -------------------- */

.ui.attached.header {
  background: #fff;
  padding: 0.78571429rem 1rem;
  margin: 0 -1px;
  box-shadow: none;
  border: 1px solid #d4d4d5;
  border-radius: 0;
}

.ui.attached.block.header {
  background: #f3f4f5;
}

.ui.attached:not(.top).header {
  border-top: none;
}

.ui.top.attached.header {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.bottom.attached.header {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* Attached Sizes */

.ui.attached.header:not(h1):not(h2):not(h3):not(h4):not(h5):not(h6) {
  font-size: 1em;
}

.ui.mini.attached.header {
  font-size: 0.78571429em;
}

.ui.tiny.attached.header {
  font-size: 0.85714286em;
}

.ui.small.attached.header {
  font-size: 0.92857143em;
}

.ui.large.attached.header {
  font-size: 1.14285714em;
}

.ui.big.attached.header {
  font-size: 1.28571429em;
}

.ui.huge.attached.header {
  font-size: 1.42857143em;
}

.ui.massive.attached.header {
  font-size: 1.71428571em;
}

/* -------------------
        Sizing
-------------------- */

.ui.header:not(h1):not(h2):not(h3):not(h4):not(h5):not(h6) {
  font-size: 1.28571429em;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Icon
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Icon
*******************************/

@font-face {
  font-family: "icons";
  src: url("./themes/default/assets/fonts/icons.woff2") format("woff2") , url("./themes/default/assets/fonts/icons.woff") format("woff");
  font-style: normal;
  font-weight: normal;
  font-display: block;
  font-variant: normal;
  text-decoration: inherit;
  text-transform: none;
}

@font-face {
  font-family: "outline-icons";
  src: url("./themes/default/assets/fonts/outline-icons.woff2") format("woff2") , url("./themes/default/assets/fonts/outline-icons.woff") format("woff");
  font-style: normal;
  font-weight: normal;
  font-display: block;
  font-variant: normal;
  text-decoration: inherit;
  text-transform: none;
}

@font-face {
  font-family: "brand-icons";
  src: url("./themes/default/assets/fonts/brand-icons.woff2") format("woff2") , url("./themes/default/assets/fonts/brand-icons.woff") format("woff");
  font-style: normal;
  font-weight: normal;
  font-display: block;
  font-variant: normal;
  text-decoration: inherit;
  text-transform: none;
}

i.icon {
  display: inline-block;
  opacity: 1;
  margin: 0 0.25rem 0 0;
  width: 1.18em;
  height: 1em;
  font-family: "icons";
  font-style: normal;
  font-weight: normal;
  text-decoration: inherit;
  text-align: center;
  speak: none;
  -moz-osx-font-smoothing: grayscale;
  -webkit-font-smoothing: antialiased;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

i.icon::before {
  background: none;
}

/*******************************
             Types
*******************************/

/* --------------
        Loading
    --------------- */

i.loading.icon {
  height: 1em;
  line-height: 1;
}

i.loading.icon,
i.loading.icons {
  animation: loader 2s linear infinite;
}

/*******************************
             States
*******************************/

i.icon:hover,
i.icons:hover,
i.icon:active,
i.icons:active,
i.emphasized.icon:not(.disabled),
i.emphasized.icons:not(.disabled) {
  opacity: 1;
}

i.disabled.icon,
i.disabled.icons {
  opacity: 0.45;
  cursor: default;
  pointer-events: none;
}

/*******************************
           Variations
*******************************/

/* -------------------
            Fitted
    -------------------- */

i.fitted.icons,
i.fitted.icon {
  width: auto;
  margin: 0 !important;
}

/* -------------------
             Link
    -------------------- */

i.link.icon:not(.disabled),
i.link.icons:not(.disabled) {
  cursor: pointer;
  opacity: 0.8;
  transition: opacity 0.1s ease;
}

i.link.icon:hover,
i.link.icons:hover {
  opacity: 1;
}

/* -------------------
          Circular
    -------------------- */

i.circular.icon {
  border-radius: 500em !important;
  line-height: 1 !important;
  padding: 0.5em 0 !important;
  box-shadow: 0 0 0 0.1em rgba(0, 0, 0, 0.1) inset;
  width: 2em !important;
  height: 2em !important;
}

i.circular.icon.colored {
  box-shadow: 0 0 0 0.1em currentColor inset;
}

i.circular.inverted.icon {
  border: none;
  box-shadow: none;
}

/* -------------------
          Flipped
    -------------------- */

i.flipped.icon,
i.horizontally.flipped.icon {
  transform: scale(-1, 1);
}

i.vertically.flipped.icon {
  transform: scale(1, -1);
}

.icons i.flipped.icon:not(.corner):not(:first-child),
.icons i.horizontally.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(-1, 1);
}

.icons i.vertically.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(1, -1);
}

/* -------------------
          Rotated
    -------------------- */

i.rotated.icon,
i.right.rotated.icon,
i.clockwise.rotated.icon {
  transform: rotate(90deg);
}

i.left.rotated.icon,
i.counterclockwise.rotated.icon {
  transform: rotate(-90deg);
}

i.halfway.rotated.icon {
  transform: rotate(180deg);
}

.icons i.rotated.rotated.icon:not(.corner):not(:first-child),
.icons i.right.rotated.icon:not(.corner):not(:first-child),
.icons i.clockwise.rotated.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) rotate(90deg);
}

.icons i.left.rotated.icon:not(.corner):not(:first-child),
.icons i.counterclockwise.rotated.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) rotate(-90deg);
}

.icons i.halfway.rotated.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) rotate(180deg);
}

/* --------------------------
          Flipped & Rotated
    --------------------------- */

i.rotated.flipped.icon,
i.right.rotated.flipped.icon,
i.clockwise.rotated.flipped.icon {
  transform: scale(-1, 1) rotate(90deg);
}

i.left.rotated.flipped.icon,
i.counterclockwise.rotated.flipped.icon {
  transform: scale(-1, 1) rotate(-90deg);
}

i.halfway.rotated.flipped.icon {
  transform: scale(-1, 1) rotate(180deg);
}

i.rotated.vertically.flipped.icon,
i.right.rotated.vertically.flipped.icon,
i.clockwise.rotated.vertically.flipped.icon {
  transform: scale(1, -1) rotate(90deg);
}

i.left.rotated.vertically.flipped.icon,
i.counterclockwise.rotated.vertically.flipped.icon {
  transform: scale(1, -1) rotate(-90deg);
}

i.halfway.rotated.vertically.flipped.icon {
  transform: scale(1, -1) rotate(180deg);
}

.icons i.rotated.flipped.icon:not(.corner):not(:first-child),
.icons i.right.rotated.flipped.icon:not(.corner):not(:first-child),
.icons i.clockwise.rotated.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(-1, 1) rotate(90deg);
}

.icons i.left.rotated.flipped.icon:not(.corner):not(:first-child),
.icons i.counterclockwise.rotated.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(-1, 1) rotate(-90deg);
}

.icons i.halfway.rotated.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(-1, 1) rotate(180deg);
}

.icons i.rotated.vertically.flipped.icon:not(.corner):not(:first-child),
.icons i.right.rotated.vertically.flipped.icon:not(.corner):not(:first-child),
.icons i.clockwise.rotated.vertically.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(1, -1) rotate(90deg);
}

.icons i.left.rotated.vertically.flipped.icon:not(.corner):not(:first-child),
.icons i.counterclockwise.rotated.vertically.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(1, -1) rotate(-90deg);
}

.icons i.halfway.rotated.vertically.flipped.icon:not(.corner):not(:first-child) {
  transform: translateX(-50%) translateY(-50%) scale(1, -1) rotate(180deg);
}

/* -------------------
          Bordered
    -------------------- */

i.bordered.icon {
  line-height: 1;
  vertical-align: baseline;
  width: 2em;
  height: 2em;
  padding: 0.5em 0 !important;
  box-shadow: 0 0 0 0.1em rgba(0, 0, 0, 0.1) inset;
}

i.bordered.icon.colored {
  box-shadow: 0 0 0 0.1em currentColor inset;
}

i.bordered.inverted.icon {
  border: none;
  box-shadow: none;
}

/* -------------------
          Inverted
    -------------------- */

/* Inverted Shapes */

i.inverted.bordered.icon,
i.inverted.circular.icon {
  background-color: #1b1c1d;
  color: #fff;
}

i.inverted.icon {
  color: #fff;
}

/* -------------------
       Colors
-------------------- */

i.primary.icon.icon.icon.icon.icon.icon {
  color: #2185d0;
}

i.inverted.primary.icon.icon.icon.icon.icon.icon {
  color: #54c8ff;
}

i.inverted.bordered.primary.icon.icon.icon.icon.icon.icon,
i.inverted.circular.primary.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.primary.icons,
i.inverted.circular.primary.icons {
  background-color: #2185d0;
  color: #fff;
}

i.secondary.icon.icon.icon.icon.icon.icon {
  color: #1b1c1d;
}

i.inverted.secondary.icon.icon.icon.icon.icon.icon {
  color: #545454;
}

i.inverted.bordered.secondary.icon.icon.icon.icon.icon.icon,
i.inverted.circular.secondary.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.secondary.icons,
i.inverted.circular.secondary.icons {
  background-color: #1b1c1d;
  color: #fff;
}

i.red.icon.icon.icon.icon.icon.icon {
  color: #db2828;
}

i.inverted.red.icon.icon.icon.icon.icon.icon {
  color: #ff695e;
}

i.inverted.bordered.red.icon.icon.icon.icon.icon.icon,
i.inverted.circular.red.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.red.icons,
i.inverted.circular.red.icons {
  background-color: #db2828;
  color: #fff;
}

i.orange.icon.icon.icon.icon.icon.icon {
  color: #f2711c;
}

i.inverted.orange.icon.icon.icon.icon.icon.icon {
  color: #ff851b;
}

i.inverted.bordered.orange.icon.icon.icon.icon.icon.icon,
i.inverted.circular.orange.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.orange.icons,
i.inverted.circular.orange.icons {
  background-color: #f2711c;
  color: #fff;
}

i.yellow.icon.icon.icon.icon.icon.icon {
  color: #fbbd08;
}

i.inverted.yellow.icon.icon.icon.icon.icon.icon {
  color: #ffe21f;
}

i.inverted.bordered.yellow.icon.icon.icon.icon.icon.icon,
i.inverted.circular.yellow.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.yellow.icons,
i.inverted.circular.yellow.icons {
  background-color: #fbbd08;
  color: #fff;
}

i.olive.icon.icon.icon.icon.icon.icon {
  color: #b5cc18;
}

i.inverted.olive.icon.icon.icon.icon.icon.icon {
  color: #d9e778;
}

i.inverted.bordered.olive.icon.icon.icon.icon.icon.icon,
i.inverted.circular.olive.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.olive.icons,
i.inverted.circular.olive.icons {
  background-color: #b5cc18;
  color: #fff;
}

i.green.icon.icon.icon.icon.icon.icon {
  color: #21ba45;
}

i.inverted.green.icon.icon.icon.icon.icon.icon {
  color: #2ecc40;
}

i.inverted.bordered.green.icon.icon.icon.icon.icon.icon,
i.inverted.circular.green.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.green.icons,
i.inverted.circular.green.icons {
  background-color: #21ba45;
  color: #fff;
}

i.teal.icon.icon.icon.icon.icon.icon {
  color: #00b5ad;
}

i.inverted.teal.icon.icon.icon.icon.icon.icon {
  color: #6dffff;
}

i.inverted.bordered.teal.icon.icon.icon.icon.icon.icon,
i.inverted.circular.teal.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.teal.icons,
i.inverted.circular.teal.icons {
  background-color: #00b5ad;
  color: #fff;
}

i.blue.icon.icon.icon.icon.icon.icon {
  color: #2185d0;
}

i.inverted.blue.icon.icon.icon.icon.icon.icon {
  color: #54c8ff;
}

i.inverted.bordered.blue.icon.icon.icon.icon.icon.icon,
i.inverted.circular.blue.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.blue.icons,
i.inverted.circular.blue.icons {
  background-color: #2185d0;
  color: #fff;
}

i.violet.icon.icon.icon.icon.icon.icon {
  color: #6435c9;
}

i.inverted.violet.icon.icon.icon.icon.icon.icon {
  color: #a291fb;
}

i.inverted.bordered.violet.icon.icon.icon.icon.icon.icon,
i.inverted.circular.violet.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.violet.icons,
i.inverted.circular.violet.icons {
  background-color: #6435c9;
  color: #fff;
}

i.purple.icon.icon.icon.icon.icon.icon {
  color: #a333c8;
}

i.inverted.purple.icon.icon.icon.icon.icon.icon {
  color: #dc73ff;
}

i.inverted.bordered.purple.icon.icon.icon.icon.icon.icon,
i.inverted.circular.purple.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.purple.icons,
i.inverted.circular.purple.icons {
  background-color: #a333c8;
  color: #fff;
}

i.pink.icon.icon.icon.icon.icon.icon {
  color: #e03997;
}

i.inverted.pink.icon.icon.icon.icon.icon.icon {
  color: #ff8edf;
}

i.inverted.bordered.pink.icon.icon.icon.icon.icon.icon,
i.inverted.circular.pink.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.pink.icons,
i.inverted.circular.pink.icons {
  background-color: #e03997;
  color: #fff;
}

i.brown.icon.icon.icon.icon.icon.icon {
  color: #a5673f;
}

i.inverted.brown.icon.icon.icon.icon.icon.icon {
  color: #d67c1c;
}

i.inverted.bordered.brown.icon.icon.icon.icon.icon.icon,
i.inverted.circular.brown.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.brown.icons,
i.inverted.circular.brown.icons {
  background-color: #a5673f;
  color: #fff;
}

i.grey.icon.icon.icon.icon.icon.icon {
  color: #767676;
}

i.inverted.grey.icon.icon.icon.icon.icon.icon {
  color: #dcddde;
}

i.inverted.bordered.grey.icon.icon.icon.icon.icon.icon,
i.inverted.circular.grey.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.grey.icons,
i.inverted.circular.grey.icons {
  background-color: #767676;
  color: #fff;
}

i.black.icon.icon.icon.icon.icon.icon {
  color: #1b1c1d;
}

i.inverted.black.icon.icon.icon.icon.icon.icon {
  color: #545454;
}

i.inverted.bordered.black.icon.icon.icon.icon.icon.icon,
i.inverted.circular.black.icon.icon.icon.icon.icon.icon,
i.inverted.bordered.black.icons,
i.inverted.circular.black.icons {
  background-color: #1b1c1d;
  color: #fff;
}

/* -------------------
        Sizes
-------------------- */

i.icon,
i.icons {
  font-size: 1em;
  line-height: 1;
  font-style: normal;
}

i.mini.mini.mini.icon,
i.mini.mini.mini.icons {
  font-size: 0.4em;
  vertical-align: middle;
}

i.tiny.tiny.tiny.icon,
i.tiny.tiny.tiny.icons {
  font-size: 0.5em;
  vertical-align: middle;
}

i.small.small.small.icon,
i.small.small.small.icons {
  font-size: 0.75em;
  vertical-align: middle;
}

i.large.large.large.icon,
i.large.large.large.icons {
  font-size: 1.5em;
  vertical-align: middle;
}

i.big.big.big.icon,
i.big.big.big.icons {
  font-size: 2em;
  vertical-align: middle;
}

i.huge.huge.huge.icon,
i.huge.huge.huge.icons {
  font-size: 4em;
  vertical-align: middle;
}

i.massive.massive.massive.icon,
i.massive.massive.massive.icons {
  font-size: 8em;
  vertical-align: middle;
}

/*******************************
                Groups
    *******************************/

i.icons {
  display: inline-block;
  position: relative;
  line-height: 1;
  min-width: 1.18em;
  min-height: 1em;
  margin: 0 0.25rem 0 0;
  text-align: center;
}

i.icons .icon {
  position: absolute;
  top: 50%;
  left: 50%;
  margin: 0;
}

i.icons .icon:not(.corner):not(.rotated):not(.flipped) {
  transform: translateX(-50%) translateY(-50%);
}

i.icons .icon:first-child {
  position: static;
  width: auto;
  height: auto;
  vertical-align: top;
}

i.icons:not(.bordered):not(.circular) .icon:first-child:not(.rotated):not(.flipped) {
  transform: none;
}

/* Corner Icon */

i.icons .corner.icon {
  top: auto;
  left: auto;
  right: 0;
  bottom: 0;
  font-size: 0.45em;
  text-shadow: -1px -1px 0 #fff, 1px -1px 0 #fff, -1px 1px 0 #fff, 1px 1px 0 #fff;
}

i.icons .corner.icon:not(.rotated):not(.flipped) {
  transform: none;
}

i.icons .icon.corner[class*="top right"] {
  top: 0;
  left: auto;
  right: 0;
  bottom: auto;
}

i.icons .icon.corner[class*="top left"] {
  top: 0;
  left: 0;
  right: auto;
  bottom: auto;
}

i.icons .icon.corner[class*="bottom left"] {
  top: auto;
  left: 0;
  right: auto;
  bottom: 0;
}

i.icons .icon.corner[class*="bottom right"] {
  top: auto;
  left: auto;
  right: 0;
  bottom: 0;
}

i.icons .inverted.corner.icon {
  text-shadow: -1px -1px 0 #1b1c1d, 1px -1px 0 #1b1c1d, -1px 1px 0 #1b1c1d, 1px 1px 0 #1b1c1d;
}

/*************************************************
       Bordered/circular with corner or group icons
    *************************************************/

i.bordered.icons,
i.circular.icons {
  width: 2em;
  height: 2em;
  box-shadow: 0 0 0 0.1em rgba(0, 0, 0, 0.1) inset;
  vertical-align: middle;
}

i.bordered.icons.colored,
i.circular.icons.colored {
  box-shadow: 0 0 0 0.1em currentColor inset;
}

i.circular.icons {
  border-radius: 500em;
}

i.bordered.icons i.icon:first-child,
i.circular.icons i.icon:first-child {
  position: absolute;
  transform: translateX(-50%) translateY(-50%);
}

/* Inverted Icon */

i.bordered.inverted.icons,
i.circular.inverted.icons {
  border: none;
  box-shadow: none;
  background-color: #1b1c1d;
  color: #fff;
}

/* Corner Icon */

i.bordered.icons .icon.corner,
i.circular.icons .icon.corner,
i.bordered.icons .icon.corner[class*="bottom right"],
i.circular.icons .icon.corner[class*="bottom right"] {
  top: auto;
  left: auto;
  right: 1.15em;
  bottom: 1.15em;
}

i.bordered.icons .icon.corner[class*="top right"],
i.circular.icons .icon.corner[class*="top right"] {
  top: 1.15em;
  left: auto;
  right: 1.15em;
  bottom: auto;
}

i.bordered.icons .icon.corner[class*="top left"],
i.circular.icons .icon.corner[class*="top left"] {
  top: 1.15em;
  left: 1.15em;
  right: auto;
  bottom: auto;
}

i.bordered.icons .icon.corner[class*="bottom left"],
i.circular.icons .icon.corner[class*="bottom left"] {
  top: auto;
  left: 1.15em;
  right: auto;
  bottom: 1.15em;
}

/* Deprecated *In/Out Naming Conflict) */

i.icon.linkedin.in::before {
  content: "\f0e1";
}

i.icon.zoom.in::before {
  content: "\f00e";
}

i.icon.zoom.out::before {
  content: "\f010";
}

i.icon.sign.in::before {
  content: "\f2f6";
}

i.icon.in.cart::before {
  content: "\f218";
}

i.icon.log.out::before {
  content: "\f2f5";
}

i.icon.sign.out::before {
  content: "\f2f5";
}

i.icon.sign.in.alternate::before {
  content: "\f2f6";
}

i.icon.sign.out.alternate::before {
  content: "\f2f5";
}

/*******************************
            Solid Icons
    *******************************/

/* Icons */

i.icon.ad::before {
  content: "\f641";
}

i.icon.address.book::before {
  content: "\f2b9";
}

i.icon.address.card::before {
  content: "\f2bb";
}

i.icon.adjust::before {
  content: "\f042";
}

i.icon.air.freshener::before {
  content: "\f5d0";
}

i.icon.align.center::before {
  content: "\f037";
}

i.icon.align.justify::before {
  content: "\f039";
}

i.icon.align.left::before {
  content: "\f036";
}

i.icon.align.right::before {
  content: "\f038";
}

i.icon.allergies::before {
  content: "\f461";
}

i.icon.ambulance::before {
  content: "\f0f9";
}

i.icon.american.sign.language.interpreting::before {
  content: "\f2a3";
}

i.icon.anchor::before {
  content: "\f13d";
}

i.icon.angle.double.down::before {
  content: "\f103";
}

i.icon.angle.double.left::before {
  content: "\f100";
}

i.icon.angle.double.right::before {
  content: "\f101";
}

i.icon.angle.double.up::before {
  content: "\f102";
}

i.icon.angle.left::before {
  content: "\f104";
}

i.icon.angle.right::before {
  content: "\f105";
}

i.icon.angle.up::before {
  content: "\f106";
}

i.icon.angle.down::before {
  content: "\f107";
}

i.icon.angry::before {
  content: "\f556";
}

i.icon.ankh::before {
  content: "\f644";
}

i.icon.archive::before {
  content: "\f187";
}

i.icon.archway::before {
  content: "\f557";
}

i.icon.arrow.alternate.circle.down::before {
  content: "\f358";
}

i.icon.arrow.alternate.circle.left::before {
  content: "\f359";
}

i.icon.arrow.alternate.circle.right::before {
  content: "\f35a";
}

i.icon.arrow.alternate.circle.up::before {
  content: "\f35b";
}

i.icon.arrow.circle.down::before {
  content: "\f0ab";
}

i.icon.arrow.circle.left::before {
  content: "\f0a8";
}

i.icon.arrow.circle.right::before {
  content: "\f0a9";
}

i.icon.arrow.circle.up::before {
  content: "\f0aa";
}

i.icon.arrow.down::before {
  content: "\f063";
}

i.icon.arrow.left::before {
  content: "\f060";
}

i.icon.arrow.right::before {
  content: "\f061";
}

i.icon.arrow.up::before {
  content: "\f062";
}

i.icon.arrows.alternate::before {
  content: "\f0b2";
}

i.icon.arrows.alternate.horizontal::before {
  content: "\f337";
}

i.icon.arrows.alternate.vertical::before {
  content: "\f338";
}

i.icon.assistive.listening.systems::before {
  content: "\f2a2";
}

i.icon.asterisk::before {
  content: "\f069";
}

i.icon.at::before {
  content: "\f1fa";
}

i.icon.atlas::before {
  content: "\f558";
}

i.icon.atom::before {
  content: "\f5d2";
}

i.icon.audio.description::before {
  content: "\f29e";
}

i.icon.award::before {
  content: "\f559";
}

i.icon.baby::before {
  content: "\f77c";
}

i.icon.baby.carriage::before {
  content: "\f77d";
}

i.icon.backspace::before {
  content: "\f55a";
}

i.icon.backward::before {
  content: "\f04a";
}

i.icon.bacon::before {
  content: "\f7e5";
}

i.icon.bacteria::before {
  content: "\e059";
}

i.icon.bacterium::before {
  content: "\e05a";
}

i.icon.bahai::before {
  content: "\f666";
}

i.icon.balance.scale::before {
  content: "\f24e";
}

i.icon.balance.scale.left::before {
  content: "\f515";
}

i.icon.balance.scale.right::before {
  content: "\f516";
}

i.icon.ban::before {
  content: "\f05e";
}

i.icon.band.aid::before {
  content: "\f462";
}

i.icon.barcode::before {
  content: "\f02a";
}

i.icon.bars::before {
  content: "\f0c9";
}

i.icon.baseball.ball::before {
  content: "\f433";
}

i.icon.basketball.ball::before {
  content: "\f434";
}

i.icon.bath::before {
  content: "\f2cd";
}

i.icon.battery.empty::before {
  content: "\f244";
}

i.icon.battery.full::before {
  content: "\f240";
}

i.icon.battery.half::before {
  content: "\f242";
}

i.icon.battery.quarter::before {
  content: "\f243";
}

i.icon.battery.three.quarters::before {
  content: "\f241";
}

i.icon.bed::before {
  content: "\f236";
}

i.icon.beer::before {
  content: "\f0fc";
}

i.icon.bell::before {
  content: "\f0f3";
}

i.icon.bell.slash::before {
  content: "\f1f6";
}

i.icon.bezier.curve::before {
  content: "\f55b";
}

i.icon.bible::before {
  content: "\f647";
}

i.icon.bicycle::before {
  content: "\f206";
}

i.icon.biking::before {
  content: "\f84a";
}

i.icon.binoculars::before {
  content: "\f1e5";
}

i.icon.biohazard::before {
  content: "\f780";
}

i.icon.birthday.cake::before {
  content: "\f1fd";
}

i.icon.blender::before {
  content: "\f517";
}

i.icon.blender.phone::before {
  content: "\f6b6";
}

i.icon.blind::before {
  content: "\f29d";
}

i.icon.blog::before {
  content: "\f781";
}

i.icon.bold::before {
  content: "\f032";
}

i.icon.bolt::before {
  content: "\f0e7";
}

i.icon.bomb::before {
  content: "\f1e2";
}

i.icon.bone::before {
  content: "\f5d7";
}

i.icon.bong::before {
  content: "\f55c";
}

i.icon.book::before {
  content: "\f02d";
}

i.icon.book.dead::before {
  content: "\f6b7";
}

i.icon.book.medical::before {
  content: "\f7e6";
}

i.icon.book.open::before {
  content: "\f518";
}

i.icon.book.reader::before {
  content: "\f5da";
}

i.icon.bookmark::before {
  content: "\f02e";
}

i.icon.border.all::before {
  content: "\f84c";
}

i.icon.border.none::before {
  content: "\f850";
}

i.icon.border.style::before {
  content: "\f853";
}

i.icon.bowling.ball::before {
  content: "\f436";
}

i.icon.box::before {
  content: "\f466";
}

i.icon.box.open::before {
  content: "\f49e";
}

i.icon.box.tissue::before {
  content: "\e05b";
}

i.icon.boxes::before {
  content: "\f468";
}

i.icon.braille::before {
  content: "\f2a1";
}

i.icon.brain::before {
  content: "\f5dc";
}

i.icon.bread.slice::before {
  content: "\f7ec";
}

i.icon.briefcase::before {
  content: "\f0b1";
}

i.icon.briefcase.medical::before {
  content: "\f469";
}

i.icon.broadcast.tower::before {
  content: "\f519";
}

i.icon.broom::before {
  content: "\f51a";
}

i.icon.brush::before {
  content: "\f55d";
}

i.icon.bug::before {
  content: "\f188";
}

i.icon.building::before {
  content: "\f1ad";
}

i.icon.bullhorn::before {
  content: "\f0a1";
}

i.icon.bullseye::before {
  content: "\f140";
}

i.icon.burn::before {
  content: "\f46a";
}

i.icon.bus::before {
  content: "\f207";
}

i.icon.bus.alternate::before {
  content: "\f55e";
}

i.icon.business.time::before {
  content: "\f64a";
}

i.icon.calculator::before {
  content: "\f1ec";
}

i.icon.calendar::before {
  content: "\f133";
}

i.icon.calendar.alternate::before {
  content: "\f073";
}

i.icon.calendar.check::before {
  content: "\f274";
}

i.icon.calendar.day::before {
  content: "\f783";
}

i.icon.calendar.minus::before {
  content: "\f272";
}

i.icon.calendar.plus::before {
  content: "\f271";
}

i.icon.calendar.times::before {
  content: "\f273";
}

i.icon.calendar.week::before {
  content: "\f784";
}

i.icon.camera::before {
  content: "\f030";
}

i.icon.camera.retro::before {
  content: "\f083";
}

i.icon.campground::before {
  content: "\f6bb";
}

i.icon.candy.cane::before {
  content: "\f786";
}

i.icon.cannabis::before {
  content: "\f55f";
}

i.icon.capsules::before {
  content: "\f46b";
}

i.icon.car::before {
  content: "\f1b9";
}

i.icon.car.alternate::before {
  content: "\f5de";
}

i.icon.car.battery::before {
  content: "\f5df";
}

i.icon.car.crash::before {
  content: "\f5e1";
}

i.icon.car.side::before {
  content: "\f5e4";
}

i.icon.caravan::before {
  content: "\f8ff";
}

i.icon.caret.down::before {
  content: "\f0d7";
}

i.icon.caret.left::before {
  content: "\f0d9";
}

i.icon.caret.right::before {
  content: "\f0da";
}

i.icon.caret.square.down::before {
  content: "\f150";
}

i.icon.caret.square.left::before {
  content: "\f191";
}

i.icon.caret.square.right::before {
  content: "\f152";
}

i.icon.caret.square.up::before {
  content: "\f151";
}

i.icon.caret.up::before {
  content: "\f0d8";
}

i.icon.carrot::before {
  content: "\f787";
}

i.icon.cart.arrow.down::before {
  content: "\f218";
}

i.icon.cart.plus::before {
  content: "\f217";
}

i.icon.cash.register::before {
  content: "\f788";
}

i.icon.cat::before {
  content: "\f6be";
}

i.icon.certificate::before {
  content: "\f0a3";
}

i.icon.chair::before {
  content: "\f6c0";
}

i.icon.chalkboard::before {
  content: "\f51b";
}

i.icon.chalkboard.teacher::before {
  content: "\f51c";
}

i.icon.charging.station::before {
  content: "\f5e7";
}

i.icon.chart.area::before {
  content: "\f1fe";
}

i.icon.chart.bar::before {
  content: "\f080";
}

i.icon.chart.pie::before {
  content: "\f200";
}

i.icon.chartline::before {
  content: "\f201";
}

i.icon.check::before {
  content: "\f00c";
}

i.icon.check.circle::before {
  content: "\f058";
}

i.icon.check.double::before {
  content: "\f560";
}

i.icon.check.square::before {
  content: "\f14a";
}

i.icon.cheese::before {
  content: "\f7ef";
}

i.icon.chess::before {
  content: "\f439";
}

i.icon.chess.bishop::before {
  content: "\f43a";
}

i.icon.chess.board::before {
  content: "\f43c";
}

i.icon.chess.king::before {
  content: "\f43f";
}

i.icon.chess.knight::before {
  content: "\f441";
}

i.icon.chess.pawn::before {
  content: "\f443";
}

i.icon.chess.queen::before {
  content: "\f445";
}

i.icon.chess.rook::before {
  content: "\f447";
}

i.icon.chevron.circle.down::before {
  content: "\f13a";
}

i.icon.chevron.circle.left::before {
  content: "\f137";
}

i.icon.chevron.circle.right::before {
  content: "\f138";
}

i.icon.chevron.circle.up::before {
  content: "\f139";
}

i.icon.chevron.down::before {
  content: "\f078";
}

i.icon.chevron.left::before {
  content: "\f053";
}

i.icon.chevron.right::before {
  content: "\f054";
}

i.icon.chevron.up::before {
  content: "\f077";
}

i.icon.child::before {
  content: "\f1ae";
}

i.icon.church::before {
  content: "\f51d";
}

i.icon.circle::before {
  content: "\f111";
}

i.icon.circle.notch::before {
  content: "\f1ce";
}

i.icon.city::before {
  content: "\f64f";
}

i.icon.clinic.medical::before {
  content: "\f7f2";
}

i.icon.clipboard::before {
  content: "\f328";
}

i.icon.clipboard.check::before {
  content: "\f46c";
}

i.icon.clipboard.list::before {
  content: "\f46d";
}

i.icon.clock::before {
  content: "\f017";
}

i.icon.clone::before {
  content: "\f24d";
}

i.icon.closed.captioning::before {
  content: "\f20a";
}

i.icon.cloud::before {
  content: "\f0c2";
}

i.icon.cloud.download.alternate::before {
  content: "\f381";
}

i.icon.cloud.meatball::before {
  content: "\f73b";
}

i.icon.cloud.moon::before {
  content: "\f6c3";
}

i.icon.cloud.moon.rain::before {
  content: "\f73c";
}

i.icon.cloud.rain::before {
  content: "\f73d";
}

i.icon.cloud.showers.heavy::before {
  content: "\f740";
}

i.icon.cloud.sun::before {
  content: "\f6c4";
}

i.icon.cloud.sun.rain::before {
  content: "\f743";
}

i.icon.cloud.upload.alternate::before {
  content: "\f382";
}

i.icon.cocktail::before {
  content: "\f561";
}

i.icon.code::before {
  content: "\f121";
}

i.icon.code.branch::before {
  content: "\f126";
}

i.icon.coffee::before {
  content: "\f0f4";
}

i.icon.cog::before {
  content: "\f013";
}

i.icon.cogs::before {
  content: "\f085";
}

i.icon.coins::before {
  content: "\f51e";
}

i.icon.columns::before {
  content: "\f0db";
}

i.icon.comment::before {
  content: "\f075";
}

i.icon.comment.alternate::before {
  content: "\f27a";
}

i.icon.comment.dollar::before {
  content: "\f651";
}

i.icon.comment.dots::before {
  content: "\f4ad";
}

i.icon.comment.medical::before {
  content: "\f7f5";
}

i.icon.comment.slash::before {
  content: "\f4b3";
}

i.icon.comments::before {
  content: "\f086";
}

i.icon.comments.dollar::before {
  content: "\f653";
}

i.icon.compact.disc::before {
  content: "\f51f";
}

i.icon.compass::before {
  content: "\f14e";
}

i.icon.compress::before {
  content: "\f066";
}

i.icon.compress.alternate::before {
  content: "\f422";
}

i.icon.compress.arrows.alternate::before {
  content: "\f78c";
}

i.icon.concierge.bell::before {
  content: "\f562";
}

i.icon.cookie::before {
  content: "\f563";
}

i.icon.cookie.bite::before {
  content: "\f564";
}

i.icon.copy::before {
  content: "\f0c5";
}

i.icon.copyright::before {
  content: "\f1f9";
}

i.icon.couch::before {
  content: "\f4b8";
}

i.icon.credit.card::before {
  content: "\f09d";
}

i.icon.crop::before {
  content: "\f125";
}

i.icon.crop.alternate::before {
  content: "\f565";
}

i.icon.cross::before {
  content: "\f654";
}

i.icon.crosshairs::before {
  content: "\f05b";
}

i.icon.crow::before {
  content: "\f520";
}

i.icon.crown::before {
  content: "\f521";
}

i.icon.crutch::before {
  content: "\f7f7";
}

i.icon.cube::before {
  content: "\f1b2";
}

i.icon.cubes::before {
  content: "\f1b3";
}

i.icon.cut::before {
  content: "\f0c4";
}

i.icon.database::before {
  content: "\f1c0";
}

i.icon.deaf::before {
  content: "\f2a4";
}

i.icon.democrat::before {
  content: "\f747";
}

i.icon.desktop::before {
  content: "\f108";
}

i.icon.dharmachakra::before {
  content: "\f655";
}

i.icon.diagnoses::before {
  content: "\f470";
}

i.icon.dice::before {
  content: "\f522";
}

i.icon.dice.d20::before {
  content: "\f6cf";
}

i.icon.dice.d6::before {
  content: "\f6d1";
}

i.icon.dice.five::before {
  content: "\f523";
}

i.icon.dice.four::before {
  content: "\f524";
}

i.icon.dice.one::before {
  content: "\f525";
}

i.icon.dice.six::before {
  content: "\f526";
}

i.icon.dice.three::before {
  content: "\f527";
}

i.icon.dice.two::before {
  content: "\f528";
}

i.icon.digital.tachograph::before {
  content: "\f566";
}

i.icon.directions::before {
  content: "\f5eb";
}

i.icon.disease::before {
  content: "\f7fa";
}

i.icon.divide::before {
  content: "\f529";
}

i.icon.dizzy::before {
  content: "\f567";
}

i.icon.dna::before {
  content: "\f471";
}

i.icon.dog::before {
  content: "\f6d3";
}

i.icon.dollar.sign::before {
  content: "\f155";
}

i.icon.dolly::before {
  content: "\f472";
}

i.icon.dolly.flatbed::before {
  content: "\f474";
}

i.icon.donate::before {
  content: "\f4b9";
}

i.icon.door.closed::before {
  content: "\f52a";
}

i.icon.door.open::before {
  content: "\f52b";
}

i.icon.dot.circle::before {
  content: "\f192";
}

i.icon.dove::before {
  content: "\f4ba";
}

i.icon.download::before {
  content: "\f019";
}

i.icon.drafting.compass::before {
  content: "\f568";
}

i.icon.dragon::before {
  content: "\f6d5";
}

i.icon.draw.polygon::before {
  content: "\f5ee";
}

i.icon.drum::before {
  content: "\f569";
}

i.icon.drum.steelpan::before {
  content: "\f56a";
}

i.icon.drumstick.bite::before {
  content: "\f6d7";
}

i.icon.dumbbell::before {
  content: "\f44b";
}

i.icon.dumpster::before {
  content: "\f793";
}

i.icon.dumpster.fire::before {
  content: "\f794";
}

i.icon.dungeon::before {
  content: "\f6d9";
}

i.icon.edit::before {
  content: "\f044";
}

i.icon.egg::before {
  content: "\f7fb";
}

i.icon.eject::before {
  content: "\f052";
}

i.icon.ellipsis.horizontal::before {
  content: "\f141";
}

i.icon.ellipsis.vertical::before {
  content: "\f142";
}

i.icon.envelope::before {
  content: "\f0e0";
}

i.icon.envelope.open::before {
  content: "\f2b6";
}

i.icon.envelope.open.text::before {
  content: "\f658";
}

i.icon.envelope.square::before {
  content: "\f199";
}

i.icon.equals::before {
  content: "\f52c";
}

i.icon.eraser::before {
  content: "\f12d";
}

i.icon.ethernet::before {
  content: "\f796";
}

i.icon.euro.sign::before {
  content: "\f153";
}

i.icon.exchange.alternate::before {
  content: "\f362";
}

i.icon.exclamation::before {
  content: "\f12a";
}

i.icon.exclamation.circle::before {
  content: "\f06a";
}

i.icon.exclamation.triangle::before {
  content: "\f071";
}

i.icon.expand::before {
  content: "\f065";
}

i.icon.expand.alternate::before {
  content: "\f424";
}

i.icon.expand.arrows.alternate::before {
  content: "\f31e";
}

i.icon.external.alternate::before {
  content: "\f35d";
}

i.icon.external.link.square.alternate::before {
  content: "\f360";
}

i.icon.eye::before {
  content: "\f06e";
}

i.icon.eye.dropper::before {
  content: "\f1fb";
}

i.icon.eye.slash::before {
  content: "\f070";
}

i.icon.fan::before {
  content: "\f863";
}

i.icon.fast.backward::before {
  content: "\f049";
}

i.icon.fast.forward::before {
  content: "\f050";
}

i.icon.faucet::before {
  content: "\e005";
}

i.icon.fax::before {
  content: "\f1ac";
}

i.icon.feather::before {
  content: "\f52d";
}

i.icon.feather.alternate::before {
  content: "\f56b";
}

i.icon.female::before {
  content: "\f182";
}

i.icon.fighter.jet::before {
  content: "\f0fb";
}

i.icon.file::before {
  content: "\f15b";
}

i.icon.file.alternate::before {
  content: "\f15c";
}

i.icon.file.archive::before {
  content: "\f1c6";
}

i.icon.file.audio::before {
  content: "\f1c7";
}

i.icon.file.code::before {
  content: "\f1c9";
}

i.icon.file.contract::before {
  content: "\f56c";
}

i.icon.file.csv::before {
  content: "\f6dd";
}

i.icon.file.download::before {
  content: "\f56d";
}

i.icon.file.excel::before {
  content: "\f1c3";
}

i.icon.file.export::before {
  content: "\f56e";
}

i.icon.file.image::before {
  content: "\f1c5";
}

i.icon.file.import::before {
  content: "\f56f";
}

i.icon.file.invoice::before {
  content: "\f570";
}

i.icon.file.invoice.dollar::before {
  content: "\f571";
}

i.icon.file.medical::before {
  content: "\f477";
}

i.icon.file.medical.alternate::before {
  content: "\f478";
}

i.icon.file.pdf::before {
  content: "\f1c1";
}

i.icon.file.powerpoint::before {
  content: "\f1c4";
}

i.icon.file.prescription::before {
  content: "\f572";
}

i.icon.file.signature::before {
  content: "\f573";
}

i.icon.file.upload::before {
  content: "\f574";
}

i.icon.file.video::before {
  content: "\f1c8";
}

i.icon.file.word::before {
  content: "\f1c2";
}

i.icon.fill::before {
  content: "\f575";
}

i.icon.fill.drip::before {
  content: "\f576";
}

i.icon.film::before {
  content: "\f008";
}

i.icon.filter::before {
  content: "\f0b0";
}

i.icon.fingerprint::before {
  content: "\f577";
}

i.icon.fire::before {
  content: "\f06d";
}

i.icon.fire.alternate::before {
  content: "\f7e4";
}

i.icon.fire.extinguisher::before {
  content: "\f134";
}

i.icon.first.aid::before {
  content: "\f479";
}

i.icon.fish::before {
  content: "\f578";
}

i.icon.fist.raised::before {
  content: "\f6de";
}

i.icon.flag::before {
  content: "\f024";
}

i.icon.flag.checkered::before {
  content: "\f11e";
}

i.icon.flag.usa::before {
  content: "\f74d";
}

i.icon.flask::before {
  content: "\f0c3";
}

i.icon.flushed::before {
  content: "\f579";
}

i.icon.folder::before {
  content: "\f07b";
}

i.icon.folder.minus::before {
  content: "\f65d";
}

i.icon.folder.open::before {
  content: "\f07c";
}

i.icon.folder.plus::before {
  content: "\f65e";
}

i.icon.font::before {
  content: "\f031";
}

i.icon.football.ball::before {
  content: "\f44e";
}

i.icon.forward::before {
  content: "\f04e";
}

i.icon.frog::before {
  content: "\f52e";
}

i.icon.frown::before {
  content: "\f119";
}

i.icon.frown.open::before {
  content: "\f57a";
}

i.icon.fruit-apple::before {
  content: "\f5d1";
}

i.icon.funnel.dollar::before {
  content: "\f662";
}

i.icon.futbol::before {
  content: "\f1e3";
}

i.icon.gamepad::before {
  content: "\f11b";
}

i.icon.gas.pump::before {
  content: "\f52f";
}

i.icon.gavel::before {
  content: "\f0e3";
}

i.icon.gem::before {
  content: "\f3a5";
}

i.icon.genderless::before {
  content: "\f22d";
}

i.icon.ghost::before {
  content: "\f6e2";
}

i.icon.gift::before {
  content: "\f06b";
}

i.icon.gifts::before {
  content: "\f79c";
}

i.icon.glass.cheers::before {
  content: "\f79f";
}

i.icon.glass.martini::before {
  content: "\f000";
}

i.icon.glass.martini.alternate::before {
  content: "\f57b";
}

i.icon.glass.whiskey::before {
  content: "\f7a0";
}

i.icon.glasses::before {
  content: "\f530";
}

i.icon.globe::before {
  content: "\f0ac";
}

i.icon.globe.africa::before {
  content: "\f57c";
}

i.icon.globe.americas::before {
  content: "\f57d";
}

i.icon.globe.asia::before {
  content: "\f57e";
}

i.icon.globe.europe::before {
  content: "\f7a2";
}

i.icon.golf.ball::before {
  content: "\f450";
}

i.icon.gopuram::before {
  content: "\f664";
}

i.icon.graduation.cap::before {
  content: "\f19d";
}

i.icon.greater.than::before {
  content: "\f531";
}

i.icon.greater.than.equal::before {
  content: "\f532";
}

i.icon.grimace::before {
  content: "\f57f";
}

i.icon.grin::before {
  content: "\f580";
}

i.icon.grin.alternate::before {
  content: "\f581";
}

i.icon.grin.beam::before {
  content: "\f582";
}

i.icon.grin.beam.sweat::before {
  content: "\f583";
}

i.icon.grin.hearts::before {
  content: "\f584";
}

i.icon.grin.squint::before {
  content: "\f585";
}

i.icon.grin.squint.tears::before {
  content: "\f586";
}

i.icon.grin.stars::before {
  content: "\f587";
}

i.icon.grin.tears::before {
  content: "\f588";
}

i.icon.grin.tongue::before {
  content: "\f589";
}

i.icon.grin.tongue.squint::before {
  content: "\f58a";
}

i.icon.grin.tongue.wink::before {
  content: "\f58b";
}

i.icon.grin.wink::before {
  content: "\f58c";
}

i.icon.grip.horizontal::before {
  content: "\f58d";
}

i.icon.grip.lines::before {
  content: "\f7a4";
}

i.icon.grip.lines.vertical::before {
  content: "\f7a5";
}

i.icon.grip.vertical::before {
  content: "\f58e";
}

i.icon.guitar::before {
  content: "\f7a6";
}

i.icon.h.square::before {
  content: "\f0fd";
}

i.icon.hamburger::before {
  content: "\f805";
}

i.icon.hammer::before {
  content: "\f6e3";
}

i.icon.hamsa::before {
  content: "\f665";
}

i.icon.hand.holding::before {
  content: "\f4bd";
}

i.icon.hand.holding.heart::before {
  content: "\f4be";
}

i.icon.hand.holding.medical::before {
  content: "\e05c";
}

i.icon.hand.holding.usd::before {
  content: "\f4c0";
}

i.icon.hand.holding.water::before {
  content: "\f4c1";
}

i.icon.hand.lizard::before {
  content: "\f258";
}

i.icon.hand.middle.finger::before {
  content: "\f806";
}

i.icon.hand.paper::before {
  content: "\f256";
}

i.icon.hand.peace::before {
  content: "\f25b";
}

i.icon.hand.point.down::before {
  content: "\f0a7";
}

i.icon.hand.point.left::before {
  content: "\f0a5";
}

i.icon.hand.point.right::before {
  content: "\f0a4";
}

i.icon.hand.point.up::before {
  content: "\f0a6";
}

i.icon.hand.pointer::before {
  content: "\f25a";
}

i.icon.hand.rock::before {
  content: "\f255";
}

i.icon.hand.scissors::before {
  content: "\f257";
}

i.icon.hand.sparkles::before {
  content: "\e05d";
}

i.icon.hand.spock::before {
  content: "\f259";
}

i.icon.hands::before {
  content: "\f4c2";
}

i.icon.hands.helping::before {
  content: "\f4c4";
}

i.icon.hands.wash::before {
  content: "\e05e";
}

i.icon.handshake::before {
  content: "\f2b5";
}

i.icon.handshake.alternate.slash::before {
  content: "\e05f";
}

i.icon.handshake.slash::before {
  content: "\e060";
}

i.icon.hanukiah::before {
  content: "\f6e6";
}

i.icon.hard.hat::before {
  content: "\f807";
}

i.icon.hashtag::before {
  content: "\f292";
}

i.icon.hat.cowboy::before {
  content: "\f8c0";
}

i.icon.hat.cowboy.side::before {
  content: "\f8c1";
}

i.icon.hat.wizard::before {
  content: "\f6e8";
}

i.icon.hdd::before {
  content: "\f0a0";
}

i.icon.head.side.cough::before {
  content: "\e061";
}

i.icon.head.side.cough.slash::before {
  content: "\e062";
}

i.icon.head.side.mask::before {
  content: "\e063";
}

i.icon.head.side.virus::before {
  content: "\e064";
}

i.icon.heading::before {
  content: "\f1dc";
}

i.icon.headphones::before {
  content: "\f025";
}

i.icon.headphones.alternate::before {
  content: "\f58f";
}

i.icon.headset::before {
  content: "\f590";
}

i.icon.heart::before {
  content: "\f004";
}

i.icon.heart.broken::before {
  content: "\f7a9";
}

i.icon.heartbeat::before {
  content: "\f21e";
}

i.icon.helicopter::before {
  content: "\f533";
}

i.icon.highlighter::before {
  content: "\f591";
}

i.icon.hiking::before {
  content: "\f6ec";
}

i.icon.hippo::before {
  content: "\f6ed";
}

i.icon.history::before {
  content: "\f1da";
}

i.icon.hockey.puck::before {
  content: "\f453";
}

i.icon.holly.berry::before {
  content: "\f7aa";
}

i.icon.home::before {
  content: "\f015";
}

i.icon.horse::before {
  content: "\f6f0";
}

i.icon.horse.head::before {
  content: "\f7ab";
}

i.icon.hospital::before {
  content: "\f0f8";
}

i.icon.hospital.alternate::before {
  content: "\f47d";
}

i.icon.hospital.symbol::before {
  content: "\f47e";
}

i.icon.hospital.user::before {
  content: "\f80d";
}

i.icon.hot.tub::before {
  content: "\f593";
}

i.icon.hotdog::before {
  content: "\f80f";
}

i.icon.hotel::before {
  content: "\f594";
}

i.icon.hourglass::before {
  content: "\f254";
}

i.icon.hourglass.end::before {
  content: "\f253";
}

i.icon.hourglass.half::before {
  content: "\f252";
}

i.icon.hourglass.start::before {
  content: "\f251";
}

i.icon.house.damage::before {
  content: "\f6f1";
}

i.icon.house.user::before {
  content: "\e065";
}

i.icon.hryvnia::before {
  content: "\f6f2";
}

i.icon.i.cursor::before {
  content: "\f246";
}

i.icon.ice.cream::before {
  content: "\f810";
}

i.icon.icicles::before {
  content: "\f7ad";
}

i.icon.icons::before {
  content: "\f86d";
}

i.icon.id.badge::before {
  content: "\f2c1";
}

i.icon.id.card::before {
  content: "\f2c2";
}

i.icon.id.card.alternate::before {
  content: "\f47f";
}

i.icon.igloo::before {
  content: "\f7ae";
}

i.icon.image::before {
  content: "\f03e";
}

i.icon.images::before {
  content: "\f302";
}

i.icon.inbox::before {
  content: "\f01c";
}

i.icon.indent::before {
  content: "\f03c";
}

i.icon.industry::before {
  content: "\f275";
}

i.icon.infinity::before {
  content: "\f534";
}

i.icon.info::before {
  content: "\f129";
}

i.icon.info.circle::before {
  content: "\f05a";
}

i.icon.italic::before {
  content: "\f033";
}

i.icon.jedi::before {
  content: "\f669";
}

i.icon.joint::before {
  content: "\f595";
}

i.icon.journal.whills::before {
  content: "\f66a";
}

i.icon.kaaba::before {
  content: "\f66b";
}

i.icon.key::before {
  content: "\f084";
}

i.icon.keyboard::before {
  content: "\f11c";
}

i.icon.khanda::before {
  content: "\f66d";
}

i.icon.kiss::before {
  content: "\f596";
}

i.icon.kiss.beam::before {
  content: "\f597";
}

i.icon.kiss.wink.heart::before {
  content: "\f598";
}

i.icon.kiwi.bird::before {
  content: "\f535";
}

i.icon.landmark::before {
  content: "\f66f";
}

i.icon.language::before {
  content: "\f1ab";
}

i.icon.laptop::before {
  content: "\f109";
}

i.icon.laptop.code::before {
  content: "\f5fc";
}

i.icon.laptop.house::before {
  content: "\e066";
}

i.icon.laptop.medical::before {
  content: "\f812";
}

i.icon.laugh::before {
  content: "\f599";
}

i.icon.laugh.beam::before {
  content: "\f59a";
}

i.icon.laugh.squint::before {
  content: "\f59b";
}

i.icon.laugh.wink::before {
  content: "\f59c";
}

i.icon.layer.group::before {
  content: "\f5fd";
}

i.icon.leaf::before {
  content: "\f06c";
}

i.icon.lemon::before {
  content: "\f094";
}

i.icon.less.than::before {
  content: "\f536";
}

i.icon.less.than.equal::before {
  content: "\f537";
}

i.icon.level.down.alternate::before {
  content: "\f3be";
}

i.icon.level.up.alternate::before {
  content: "\f3bf";
}

i.icon.life.ring::before {
  content: "\f1cd";
}

i.icon.lightbulb::before {
  content: "\f0eb";
}

i.icon.linkify::before {
  content: "\f0c1";
}

i.icon.lira.sign::before {
  content: "\f195";
}

i.icon.list::before {
  content: "\f03a";
}

i.icon.list.alternate::before {
  content: "\f022";
}

i.icon.list.ol::before {
  content: "\f0cb";
}

i.icon.list.ul::before {
  content: "\f0ca";
}

i.icon.location.arrow::before {
  content: "\f124";
}

i.icon.lock::before {
  content: "\f023";
}

i.icon.lock.open::before {
  content: "\f3c1";
}

i.icon.long.arrow.alternate.down::before {
  content: "\f309";
}

i.icon.long.arrow.alternate.left::before {
  content: "\f30a";
}

i.icon.long.arrow.alternate.right::before {
  content: "\f30b";
}

i.icon.long.arrow.alternate.up::before {
  content: "\f30c";
}

i.icon.low.vision::before {
  content: "\f2a8";
}

i.icon.luggage.cart::before {
  content: "\f59d";
}

i.icon.lungs::before {
  content: "\f604";
}

i.icon.lungs.virus::before {
  content: "\e067";
}

i.icon.magic::before {
  content: "\f0d0";
}

i.icon.magnet::before {
  content: "\f076";
}

i.icon.mail.bulk::before {
  content: "\f674";
}

i.icon.male::before {
  content: "\f183";
}

i.icon.map::before {
  content: "\f279";
}

i.icon.map.marked::before {
  content: "\f59f";
}

i.icon.map.marked.alternate::before {
  content: "\f5a0";
}

i.icon.map.marker::before {
  content: "\f041";
}

i.icon.map.marker.alternate::before {
  content: "\f3c5";
}

i.icon.map.pin::before {
  content: "\f276";
}

i.icon.map.signs::before {
  content: "\f277";
}

i.icon.marker::before {
  content: "\f5a1";
}

i.icon.mars::before {
  content: "\f222";
}

i.icon.mars.double::before {
  content: "\f227";
}

i.icon.mars.stroke::before {
  content: "\f229";
}

i.icon.mars.stroke.horizontal::before {
  content: "\f22b";
}

i.icon.mars.stroke.vertical::before {
  content: "\f22a";
}

i.icon.mask::before {
  content: "\f6fa";
}

i.icon.medal::before {
  content: "\f5a2";
}

i.icon.medkit::before {
  content: "\f0fa";
}

i.icon.meh::before {
  content: "\f11a";
}

i.icon.meh.blank::before {
  content: "\f5a4";
}

i.icon.meh.rolling.eyes::before {
  content: "\f5a5";
}

i.icon.memory::before {
  content: "\f538";
}

i.icon.menorah::before {
  content: "\f676";
}

i.icon.mercury::before {
  content: "\f223";
}

i.icon.meteor::before {
  content: "\f753";
}

i.icon.microchip::before {
  content: "\f2db";
}

i.icon.microphone::before {
  content: "\f130";
}

i.icon.microphone.alternate::before {
  content: "\f3c9";
}

i.icon.microphone.alternate.slash::before {
  content: "\f539";
}

i.icon.microphone.slash::before {
  content: "\f131";
}

i.icon.microscope::before {
  content: "\f610";
}

i.icon.minus::before {
  content: "\f068";
}

i.icon.minus.circle::before {
  content: "\f056";
}

i.icon.minus.square::before {
  content: "\f146";
}

i.icon.mitten::before {
  content: "\f7b5";
}

i.icon.mobile::before {
  content: "\f10b";
}

i.icon.mobile.alternate::before {
  content: "\f3cd";
}

i.icon.money.bill::before {
  content: "\f0d6";
}

i.icon.money.bill.alternate::before {
  content: "\f3d1";
}

i.icon.money.bill.wave::before {
  content: "\f53a";
}

i.icon.money.bill.wave.alternate::before {
  content: "\f53b";
}

i.icon.money.check::before {
  content: "\f53c";
}

i.icon.money.check.alternate::before {
  content: "\f53d";
}

i.icon.monument::before {
  content: "\f5a6";
}

i.icon.moon::before {
  content: "\f186";
}

i.icon.mortar.pestle::before {
  content: "\f5a7";
}

i.icon.mosque::before {
  content: "\f678";
}

i.icon.motorcycle::before {
  content: "\f21c";
}

i.icon.mountain::before {
  content: "\f6fc";
}

i.icon.mouse::before {
  content: "\f8cc";
}

i.icon.mouse.pointer::before {
  content: "\f245";
}

i.icon.mug.hot::before {
  content: "\f7b6";
}

i.icon.music::before {
  content: "\f001";
}

i.icon.network.wired::before {
  content: "\f6ff";
}

i.icon.neuter::before {
  content: "\f22c";
}

i.icon.newspaper::before {
  content: "\f1ea";
}

i.icon.not.equal::before {
  content: "\f53e";
}

i.icon.notes.medical::before {
  content: "\f481";
}

i.icon.object.group::before {
  content: "\f247";
}

i.icon.object.ungroup::before {
  content: "\f248";
}

i.icon.oil.can::before {
  content: "\f613";
}

i.icon.om::before {
  content: "\f679";
}

i.icon.otter::before {
  content: "\f700";
}

i.icon.outdent::before {
  content: "\f03b";
}

i.icon.pager::before {
  content: "\f815";
}

i.icon.paint.brush::before {
  content: "\f1fc";
}

i.icon.paint.roller::before {
  content: "\f5aa";
}

i.icon.palette::before {
  content: "\f53f";
}

i.icon.pallet::before {
  content: "\f482";
}

i.icon.paper.plane::before {
  content: "\f1d8";
}

i.icon.paperclip::before {
  content: "\f0c6";
}

i.icon.parachute.box::before {
  content: "\f4cd";
}

i.icon.paragraph::before {
  content: "\f1dd";
}

i.icon.parking::before {
  content: "\f540";
}

i.icon.passport::before {
  content: "\f5ab";
}

i.icon.pastafarianism::before {
  content: "\f67b";
}

i.icon.paste::before {
  content: "\f0ea";
}

i.icon.pause::before {
  content: "\f04c";
}

i.icon.pause.circle::before {
  content: "\f28b";
}

i.icon.paw::before {
  content: "\f1b0";
}

i.icon.peace::before {
  content: "\f67c";
}

i.icon.pen::before {
  content: "\f304";
}

i.icon.pen.alternate::before {
  content: "\f305";
}

i.icon.pen.fancy::before {
  content: "\f5ac";
}

i.icon.pen.nib::before {
  content: "\f5ad";
}

i.icon.pen.square::before {
  content: "\f14b";
}

i.icon.pencil.alternate::before {
  content: "\f303";
}

i.icon.pencil.ruler::before {
  content: "\f5ae";
}

i.icon.people.arrows::before {
  content: "\e068";
}

i.icon.people.carry::before {
  content: "\f4ce";
}

i.icon.pepper.hot::before {
  content: "\f816";
}

i.icon.percent::before {
  content: "\f295";
}

i.icon.percentage::before {
  content: "\f541";
}

i.icon.person.booth::before {
  content: "\f756";
}

i.icon.phone::before {
  content: "\f095";
}

i.icon.phone.alternate::before {
  content: "\f879";
}

i.icon.phone.slash::before {
  content: "\f3dd";
}

i.icon.phone.square::before {
  content: "\f098";
}

i.icon.phone.square.alternate::before {
  content: "\f87b";
}

i.icon.phone.volume::before {
  content: "\f2a0";
}

i.icon.photo.video::before {
  content: "\f87c";
}

i.icon.piggy.bank::before {
  content: "\f4d3";
}

i.icon.pills::before {
  content: "\f484";
}

i.icon.pizza.slice::before {
  content: "\f818";
}

i.icon.place.of.worship::before {
  content: "\f67f";
}

i.icon.plane::before {
  content: "\f072";
}

i.icon.plane.arrival::before {
  content: "\f5af";
}

i.icon.plane.departure::before {
  content: "\f5b0";
}

i.icon.plane.slash::before {
  content: "\e069";
}

i.icon.play::before {
  content: "\f04b";
}

i.icon.play.circle::before {
  content: "\f144";
}

i.icon.plug::before {
  content: "\f1e6";
}

i.icon.plus::before {
  content: "\f067";
}

i.icon.plus.circle::before {
  content: "\f055";
}

i.icon.plus.square::before {
  content: "\f0fe";
}

i.icon.podcast::before {
  content: "\f2ce";
}

i.icon.poll::before {
  content: "\f681";
}

i.icon.poll.horizontal::before {
  content: "\f682";
}

i.icon.poo::before {
  content: "\f2fe";
}

i.icon.poo.storm::before {
  content: "\f75a";
}

i.icon.poop::before {
  content: "\f619";
}

i.icon.portrait::before {
  content: "\f3e0";
}

i.icon.pound.sign::before {
  content: "\f154";
}

i.icon.power.off::before {
  content: "\f011";
}

i.icon.pray::before {
  content: "\f683";
}

i.icon.praying.hands::before {
  content: "\f684";
}

i.icon.prescription::before {
  content: "\f5b1";
}

i.icon.prescription.bottle::before {
  content: "\f485";
}

i.icon.prescription.bottle.alternate::before {
  content: "\f486";
}

i.icon.print::before {
  content: "\f02f";
}

i.icon.procedures::before {
  content: "\f487";
}

i.icon.project.diagram::before {
  content: "\f542";
}

i.icon.pump.medical::before {
  content: "\e06a";
}

i.icon.pump.soap::before {
  content: "\e06b";
}

i.icon.puzzle.piece::before {
  content: "\f12e";
}

i.icon.qrcode::before {
  content: "\f029";
}

i.icon.question::before {
  content: "\f128";
}

i.icon.question.circle::before {
  content: "\f059";
}

i.icon.quidditch::before {
  content: "\f458";
}

i.icon.quote.left::before {
  content: "\f10d";
}

i.icon.quote.right::before {
  content: "\f10e";
}

i.icon.quran::before {
  content: "\f687";
}

i.icon.radiation::before {
  content: "\f7b9";
}

i.icon.radiation.alternate::before {
  content: "\f7ba";
}

i.icon.rainbow::before {
  content: "\f75b";
}

i.icon.random::before {
  content: "\f074";
}

i.icon.receipt::before {
  content: "\f543";
}

i.icon.record.vinyl::before {
  content: "\f8d9";
}

i.icon.recycle::before {
  content: "\f1b8";
}

i.icon.redo::before {
  content: "\f01e";
}

i.icon.redo.alternate::before {
  content: "\f2f9";
}

i.icon.registered::before {
  content: "\f25d";
}

i.icon.remove.format::before {
  content: "\f87d";
}

i.icon.reply::before {
  content: "\f3e5";
}

i.icon.reply.all::before {
  content: "\f122";
}

i.icon.republican::before {
  content: "\f75e";
}

i.icon.restroom::before {
  content: "\f7bd";
}

i.icon.retweet::before {
  content: "\f079";
}

i.icon.ribbon::before {
  content: "\f4d6";
}

i.icon.ring::before {
  content: "\f70b";
}

i.icon.road::before {
  content: "\f018";
}

i.icon.robot::before {
  content: "\f544";
}

i.icon.rocket::before {
  content: "\f135";
}

i.icon.route::before {
  content: "\f4d7";
}

i.icon.rss::before {
  content: "\f09e";
}

i.icon.rss.square::before {
  content: "\f143";
}

i.icon.ruble.sign::before {
  content: "\f158";
}

i.icon.ruler::before {
  content: "\f545";
}

i.icon.ruler.combined::before {
  content: "\f546";
}

i.icon.ruler.horizontal::before {
  content: "\f547";
}

i.icon.ruler.vertical::before {
  content: "\f548";
}

i.icon.running::before {
  content: "\f70c";
}

i.icon.rupee.sign::before {
  content: "\f156";
}

i.icon.sad.cry::before {
  content: "\f5b3";
}

i.icon.sad.tear::before {
  content: "\f5b4";
}

i.icon.satellite::before {
  content: "\f7bf";
}

i.icon.satellite.dish::before {
  content: "\f7c0";
}

i.icon.save::before {
  content: "\f0c7";
}

i.icon.school::before {
  content: "\f549";
}

i.icon.screwdriver::before {
  content: "\f54a";
}

i.icon.scroll::before {
  content: "\f70e";
}

i.icon.sd.card::before {
  content: "\f7c2";
}

i.icon.search::before {
  content: "\f002";
}

i.icon.search.dollar::before {
  content: "\f688";
}

i.icon.search.location::before {
  content: "\f689";
}

i.icon.search.minus::before {
  content: "\f010";
}

i.icon.search.plus::before {
  content: "\f00e";
}

i.icon.seedling::before {
  content: "\f4d8";
}

i.icon.server::before {
  content: "\f233";
}

i.icon.shapes::before {
  content: "\f61f";
}

i.icon.share::before {
  content: "\f064";
}

i.icon.share.alternate::before {
  content: "\f1e0";
}

i.icon.share.alternate.square::before {
  content: "\f1e1";
}

i.icon.share.square::before {
  content: "\f14d";
}

i.icon.shekel.sign::before {
  content: "\f20b";
}

i.icon.shield.alternate::before {
  content: "\f3ed";
}

i.icon.shield.virus::before {
  content: "\e06c";
}

i.icon.ship::before {
  content: "\f21a";
}

i.icon.shipping.fast::before {
  content: "\f48b";
}

i.icon.shoe.prints::before {
  content: "\f54b";
}

i.icon.shopping.bag::before {
  content: "\f290";
}

i.icon.shopping.basket::before {
  content: "\f291";
}

i.icon.shopping.cart::before {
  content: "\f07a";
}

i.icon.shower::before {
  content: "\f2cc";
}

i.icon.shuttle.van::before {
  content: "\f5b6";
}

i.icon.sign::before {
  content: "\f4d9";
}

i.icon.sign.language::before {
  content: "\f2a7";
}

i.icon.signal::before {
  content: "\f012";
}

i.icon.signature::before {
  content: "\f5b7";
}

i.icon.sim.card::before {
  content: "\f7c4";
}

i.icon.sink::before {
  content: "\e06d";
}

i.icon.sitemap::before {
  content: "\f0e8";
}

i.icon.skating::before {
  content: "\f7c5";
}

i.icon.skiing::before {
  content: "\f7c9";
}

i.icon.skiing.nordic::before {
  content: "\f7ca";
}

i.icon.skull::before {
  content: "\f54c";
}

i.icon.skull.crossbones::before {
  content: "\f714";
}

i.icon.slash::before {
  content: "\f715";
}

i.icon.sleigh::before {
  content: "\f7cc";
}

i.icon.sliders.horizontal::before {
  content: "\f1de";
}

i.icon.smile::before {
  content: "\f118";
}

i.icon.smile.beam::before {
  content: "\f5b8";
}

i.icon.smile.wink::before {
  content: "\f4da";
}

i.icon.smog::before {
  content: "\f75f";
}

i.icon.smoking::before {
  content: "\f48d";
}

i.icon.smoking.ban::before {
  content: "\f54d";
}

i.icon.sms::before {
  content: "\f7cd";
}

i.icon.snowboarding::before {
  content: "\f7ce";
}

i.icon.snowflake::before {
  content: "\f2dc";
}

i.icon.snowman::before {
  content: "\f7d0";
}

i.icon.snowplow::before {
  content: "\f7d2";
}

i.icon.soap::before {
  content: "\e06e";
}

i.icon.socks::before {
  content: "\f696";
}

i.icon.solar.panel::before {
  content: "\f5ba";
}

i.icon.sort::before {
  content: "\f0dc";
}

i.icon.sort.alphabet.down::before {
  content: "\f15d";
}

i.icon.sort.alphabet.down.alternate::before {
  content: "\f881";
}

i.icon.sort.alphabet.up::before {
  content: "\f15e";
}

i.icon.sort.alphabet.up.alternate::before {
  content: "\f882";
}

i.icon.sort.amount.down::before {
  content: "\f160";
}

i.icon.sort.amount.down.alternate::before {
  content: "\f884";
}

i.icon.sort.amount.up::before {
  content: "\f161";
}

i.icon.sort.amount.up.alternate::before {
  content: "\f885";
}

i.icon.sort.down::before {
  content: "\f0dd";
}

i.icon.sort.numeric.down::before {
  content: "\f162";
}

i.icon.sort.numeric.down.alternate::before {
  content: "\f886";
}

i.icon.sort.numeric.up::before {
  content: "\f163";
}

i.icon.sort.numeric.up.alternate::before {
  content: "\f887";
}

i.icon.sort.up::before {
  content: "\f0de";
}

i.icon.spa::before {
  content: "\f5bb";
}

i.icon.space.shuttle::before {
  content: "\f197";
}

i.icon.spell.check::before {
  content: "\f891";
}

i.icon.spider::before {
  content: "\f717";
}

i.icon.spinner::before {
  content: "\f110";
}

i.icon.splotch::before {
  content: "\f5bc";
}

i.icon.spray.can::before {
  content: "\f5bd";
}

i.icon.square::before {
  content: "\f0c8";
}

i.icon.square.full::before {
  content: "\f45c";
}

i.icon.square.root.alternate::before {
  content: "\f698";
}

i.icon.stamp::before {
  content: "\f5bf";
}

i.icon.star::before {
  content: "\f005";
}

i.icon.star.and.crescent::before {
  content: "\f699";
}

i.icon.star.half::before {
  content: "\f089";
}

i.icon.star.half.alternate::before {
  content: "\f5c0";
}

i.icon.star.of.david::before {
  content: "\f69a";
}

i.icon.star.of.life::before {
  content: "\f621";
}

i.icon.step.backward::before {
  content: "\f048";
}

i.icon.step.forward::before {
  content: "\f051";
}

i.icon.stethoscope::before {
  content: "\f0f1";
}

i.icon.sticky.note::before {
  content: "\f249";
}

i.icon.stop::before {
  content: "\f04d";
}

i.icon.stop.circle::before {
  content: "\f28d";
}

i.icon.stopwatch::before {
  content: "\f2f2";
}

i.icon.stopwatch.twenty::before {
  content: "\e06f";
}

i.icon.store::before {
  content: "\f54e";
}

i.icon.store.alternate::before {
  content: "\f54f";
}

i.icon.store.alternate.slash::before {
  content: "\e070";
}

i.icon.store.slash::before {
  content: "\e071";
}

i.icon.stream::before {
  content: "\f550";
}

i.icon.street.view::before {
  content: "\f21d";
}

i.icon.strikethrough::before {
  content: "\f0cc";
}

i.icon.stroopwafel::before {
  content: "\f551";
}

i.icon.subscript::before {
  content: "\f12c";
}

i.icon.subway::before {
  content: "\f239";
}

i.icon.suitcase::before {
  content: "\f0f2";
}

i.icon.suitcase.rolling::before {
  content: "\f5c1";
}

i.icon.sun::before {
  content: "\f185";
}

i.icon.superscript::before {
  content: "\f12b";
}

i.icon.surprise::before {
  content: "\f5c2";
}

i.icon.swatchbook::before {
  content: "\f5c3";
}

i.icon.swimmer::before {
  content: "\f5c4";
}

i.icon.swimming.pool::before {
  content: "\f5c5";
}

i.icon.synagogue::before {
  content: "\f69b";
}

i.icon.sync::before {
  content: "\f021";
}

i.icon.sync.alternate::before {
  content: "\f2f1";
}

i.icon.syringe::before {
  content: "\f48e";
}

i.icon.table::before {
  content: "\f0ce";
}

i.icon.table.tennis::before {
  content: "\f45d";
}

i.icon.tablet::before {
  content: "\f10a";
}

i.icon.tablet.alternate::before {
  content: "\f3fa";
}

i.icon.tablets::before {
  content: "\f490";
}

i.icon.tachometer.alternate::before {
  content: "\f3fd";
}

i.icon.tag::before {
  content: "\f02b";
}

i.icon.tags::before {
  content: "\f02c";
}

i.icon.tape::before {
  content: "\f4db";
}

i.icon.tasks::before {
  content: "\f0ae";
}

i.icon.taxi::before {
  content: "\f1ba";
}

i.icon.teeth::before {
  content: "\f62e";
}

i.icon.teeth.open::before {
  content: "\f62f";
}

i.icon.temperature.high::before {
  content: "\f769";
}

i.icon.temperature.low::before {
  content: "\f76b";
}

i.icon.tenge::before {
  content: "\f7d7";
}

i.icon.terminal::before {
  content: "\f120";
}

i.icon.text.height::before {
  content: "\f034";
}

i.icon.text.width::before {
  content: "\f035";
}

i.icon.th::before {
  content: "\f00a";
}

i.icon.th.large::before {
  content: "\f009";
}

i.icon.th.list::before {
  content: "\f00b";
}

i.icon.theater.masks::before {
  content: "\f630";
}

i.icon.thermometer::before {
  content: "\f491";
}

i.icon.thermometer.empty::before {
  content: "\f2cb";
}

i.icon.thermometer.full::before {
  content: "\f2c7";
}

i.icon.thermometer.half::before {
  content: "\f2c9";
}

i.icon.thermometer.quarter::before {
  content: "\f2ca";
}

i.icon.thermometer.three.quarters::before {
  content: "\f2c8";
}

i.icon.thumbs.down::before {
  content: "\f165";
}

i.icon.thumbs.up::before {
  content: "\f164";
}

i.icon.thumbtack::before {
  content: "\f08d";
}

i.icon.ticket.alternate::before {
  content: "\f3ff";
}

i.icon.times::before {
  content: "\f00d";
}

i.icon.times.circle::before {
  content: "\f057";
}

i.icon.tint::before {
  content: "\f043";
}

i.icon.tint.slash::before {
  content: "\f5c7";
}

i.icon.tired::before {
  content: "\f5c8";
}

i.icon.toggle.off::before {
  content: "\f204";
}

i.icon.toggle.on::before {
  content: "\f205";
}

i.icon.toilet::before {
  content: "\f7d8";
}

i.icon.toilet.paper::before {
  content: "\f71e";
}

i.icon.toilet.paper.slash::before {
  content: "\e072";
}

i.icon.toolbox::before {
  content: "\f552";
}

i.icon.tools::before {
  content: "\f7d9";
}

i.icon.tooth::before {
  content: "\f5c9";
}

i.icon.torah::before {
  content: "\f6a0";
}

i.icon.torii.gate::before {
  content: "\f6a1";
}

i.icon.tractor::before {
  content: "\f722";
}

i.icon.trademark::before {
  content: "\f25c";
}

i.icon.traffic.light::before {
  content: "\f637";
}

i.icon.trailer::before {
  content: "\e041";
}

i.icon.train::before {
  content: "\f238";
}

i.icon.tram::before {
  content: "\f7da";
}

i.icon.transgender::before {
  content: "\f224";
}

i.icon.transgender.alternate::before {
  content: "\f225";
}

i.icon.trash::before {
  content: "\f1f8";
}

i.icon.trash.alternate::before {
  content: "\f2ed";
}

i.icon.trash.restore::before {
  content: "\f829";
}

i.icon.trash.restore.alternate::before {
  content: "\f82a";
}

i.icon.tree::before {
  content: "\f1bb";
}

i.icon.trophy::before {
  content: "\f091";
}

i.icon.truck::before {
  content: "\f0d1";
}

i.icon.truck.monster::before {
  content: "\f63b";
}

i.icon.truck.moving::before {
  content: "\f4df";
}

i.icon.truck.packing::before {
  content: "\f4de";
}

i.icon.truck.pickup::before {
  content: "\f63c";
}

i.icon.tshirt::before {
  content: "\f553";
}

i.icon.tty::before {
  content: "\f1e4";
}

i.icon.tv::before {
  content: "\f26c";
}

i.icon.umbrella::before {
  content: "\f0e9";
}

i.icon.umbrella.beach::before {
  content: "\f5ca";
}

i.icon.underline::before {
  content: "\f0cd";
}

i.icon.undo::before {
  content: "\f0e2";
}

i.icon.undo.alternate::before {
  content: "\f2ea";
}

i.icon.universal.access::before {
  content: "\f29a";
}

i.icon.university::before {
  content: "\f19c";
}

i.icon.unlink::before {
  content: "\f127";
}

i.icon.unlock::before {
  content: "\f09c";
}

i.icon.unlock.alternate::before {
  content: "\f13e";
}

i.icon.upload::before {
  content: "\f093";
}

i.icon.user::before {
  content: "\f007";
}

i.icon.user.alternate::before {
  content: "\f406";
}

i.icon.user.alternate.slash::before {
  content: "\f4fa";
}

i.icon.user.astronaut::before {
  content: "\f4fb";
}

i.icon.user.check::before {
  content: "\f4fc";
}

i.icon.user.circle::before {
  content: "\f2bd";
}

i.icon.user.clock::before {
  content: "\f4fd";
}

i.icon.user.cog::before {
  content: "\f4fe";
}

i.icon.user.edit::before {
  content: "\f4ff";
}

i.icon.user.friends::before {
  content: "\f500";
}

i.icon.user.graduate::before {
  content: "\f501";
}

i.icon.user.injured::before {
  content: "\f728";
}

i.icon.user.lock::before {
  content: "\f502";
}

i.icon.user.md::before {
  content: "\f0f0";
}

i.icon.user.minus::before {
  content: "\f503";
}

i.icon.user.ninja::before {
  content: "\f504";
}

i.icon.user.nurse::before {
  content: "\f82f";
}

i.icon.user.plus::before {
  content: "\f234";
}

i.icon.user.secret::before {
  content: "\f21b";
}

i.icon.user.shield::before {
  content: "\f505";
}

i.icon.user.slash::before {
  content: "\f506";
}

i.icon.user.tag::before {
  content: "\f507";
}

i.icon.user.tie::before {
  content: "\f508";
}

i.icon.user.times::before {
  content: "\f235";
}

i.icon.users::before {
  content: "\f0c0";
}

i.icon.users.cog::before {
  content: "\f509";
}

i.icon.users.slash::before {
  content: "\e073";
}

i.icon.utensil.spoon::before {
  content: "\f2e5";
}

i.icon.utensils::before {
  content: "\f2e7";
}

i.icon.vector.square::before {
  content: "\f5cb";
}

i.icon.venus::before {
  content: "\f221";
}

i.icon.venus.double::before {
  content: "\f226";
}

i.icon.venus.mars::before {
  content: "\f228";
}

i.icon.vest::before {
  content: "\e085";
}

i.icon.vest.patches::before {
  content: "\e086";
}

i.icon.vial::before {
  content: "\f492";
}

i.icon.vials::before {
  content: "\f493";
}

i.icon.video::before {
  content: "\f03d";
}

i.icon.video.slash::before {
  content: "\f4e2";
}

i.icon.vihara::before {
  content: "\f6a7";
}

i.icon.virus::before {
  content: "\e074";
}

i.icon.virus.slash::before {
  content: "\e075";
}

i.icon.viruses::before {
  content: "\e076";
}

i.icon.voicemail::before {
  content: "\f897";
}

i.icon.volleyball.ball::before {
  content: "\f45f";
}

i.icon.volume.down::before {
  content: "\f027";
}

i.icon.volume.mute::before {
  content: "\f6a9";
}

i.icon.volume.off::before {
  content: "\f026";
}

i.icon.volume.up::before {
  content: "\f028";
}

i.icon.vote.yea::before {
  content: "\f772";
}

i.icon.vr.cardboard::before {
  content: "\f729";
}

i.icon.walking::before {
  content: "\f554";
}

i.icon.wallet::before {
  content: "\f555";
}

i.icon.warehouse::before {
  content: "\f494";
}

i.icon.water::before {
  content: "\f773";
}

i.icon.wave.square::before {
  content: "\f83e";
}

i.icon.weight::before {
  content: "\f496";
}

i.icon.weight.hanging::before {
  content: "\f5cd";
}

i.icon.wheelchair::before {
  content: "\f193";
}

i.icon.wifi::before {
  content: "\f1eb";
}

i.icon.wind::before {
  content: "\f72e";
}

i.icon.window.close::before {
  content: "\f410";
}

i.icon.window.maximize::before {
  content: "\f2d0";
}

i.icon.window.minimize::before {
  content: "\f2d1";
}

i.icon.window.restore::before {
  content: "\f2d2";
}

i.icon.wine.bottle::before {
  content: "\f72f";
}

i.icon.wine.glass::before {
  content: "\f4e3";
}

i.icon.wine.glass.alternate::before {
  content: "\f5ce";
}

i.icon.won.sign::before {
  content: "\f159";
}

i.icon.wrench::before {
  content: "\f0ad";
}

i.icon.x.ray::before {
  content: "\f497";
}

i.icon.yen.sign::before {
  content: "\f157";
}

i.icon.yin.yang::before {
  content: "\f6ad";
}

/* Aliases */

i.icon.add::before {
  content: "\f067";
}

i.icon.add.circle::before {
  content: "\f055";
}

i.icon.add.square::before {
  content: "\f0fe";
}

i.icon.add.to.calendar::before {
  content: "\f271";
}

i.icon.add.to.cart::before {
  content: "\f217";
}

i.icon.add.user::before {
  content: "\f234";
}

i.icon.alarm::before {
  content: "\f0f3";
}

i.icon.alarm.mute::before {
  content: "\f1f6";
}

i.icon.ald::before {
  content: "\f2a2";
}

i.icon.als::before {
  content: "\f2a2";
}

i.icon.announcement::before {
  content: "\f0a1";
}

i.icon.area.chart::before {
  content: "\f1fe";
}

i.icon.area.graph::before {
  content: "\f1fe";
}

i.icon.arrow.down.cart::before {
  content: "\f218";
}

i.icon.asexual::before {
  content: "\f22d";
}

i.icon.asl::before {
  content: "\f2a3";
}

i.icon.asl.interpreting::before {
  content: "\f2a3";
}

i.icon.assistive.listening.devices::before {
  content: "\f2a2";
}

i.icon.attach::before {
  content: "\f0c6";
}

i.icon.attention::before {
  content: "\f06a";
}

i.icon.balance::before {
  content: "\f24e";
}

i.icon.bar::before {
  content: "\f0fc";
}

i.icon.bathtub::before {
  content: "\f2cd";
}

i.icon.battery.four::before {
  content: "\f240";
}

i.icon.battery.high::before {
  content: "\f241";
}

i.icon.battery.low::before {
  content: "\f243";
}

i.icon.battery.medium::before {
  content: "\f242";
}

i.icon.battery.one::before {
  content: "\f243";
}

i.icon.battery.three::before {
  content: "\f241";
}

i.icon.battery.two::before {
  content: "\f242";
}

i.icon.battery.zero::before {
  content: "\f244";
}

i.icon.birthday::before {
  content: "\f1fd";
}

i.icon.block.layout::before {
  content: "\f009";
}

i.icon.broken.chain::before {
  content: "\f127";
}

i.icon.browser::before {
  content: "\f022";
}

i.icon.call::before {
  content: "\f095";
}

i.icon.call.square::before {
  content: "\f098";
}

i.icon.cancel::before {
  content: "\f00d";
}

i.icon.cart::before {
  content: "\f07a";
}

i.icon.cc::before {
  content: "\f20a";
}

i.icon.chain::before {
  content: "\f0c1";
}

i.icon.chat::before {
  content: "\f075";
}

i.icon.checked.calendar::before {
  content: "\f274";
}

i.icon.checkmark::before {
  content: "\f00c";
}

i.icon.checkmark.box::before {
  content: "\f14a";
}

i.icon.chess.rock::before {
  content: "\f447";
}

i.icon.circle.notched::before {
  content: "\f1ce";
}

i.icon.circle.thin::before {
  content: "\f111";
}

i.icon.close::before {
  content: "\f00d";
}

i.icon.cloud.download::before {
  content: "\f381";
}

i.icon.cloud.upload::before {
  content: "\f382";
}

i.icon.cny::before {
  content: "\f157";
}

i.icon.cocktail::before {
  content: "\f000";
}

i.icon.commenting::before {
  content: "\f27a";
}

i.icon.compose::before {
  content: "\f303";
}

i.icon.computer::before {
  content: "\f108";
}

i.icon.configure::before {
  content: "\f0ad";
}

i.icon.content::before {
  content: "\f0c9";
}

i.icon.conversation::before {
  content: "\f086";
}

i.icon.credit.card.alternative::before {
  content: "\f09d";
}

i.icon.currency::before {
  content: "\f3d1";
}

i.icon.dashboard::before {
  content: "\f3fd";
}

i.icon.deafness::before {
  content: "\f2a4";
}

i.icon.delete::before {
  content: "\f00d";
}

i.icon.delete.calendar::before {
  content: "\f273";
}

i.icon.detective::before {
  content: "\f21b";
}

i.icon.diamond::before {
  content: "\f3a5";
}

i.icon.discussions::before {
  content: "\f086";
}

i.icon.disk::before {
  content: "\f0a0";
}

i.icon.doctor::before {
  content: "\f0f0";
}

i.icon.dollar::before {
  content: "\f155";
}

i.icon.dont::before {
  content: "\f05e";
}

i.icon.drivers.license::before {
  content: "\f2c2";
}

i.icon.dropdown::before {
  content: "\f0d7";
}

i.icon.emergency::before {
  content: "\f0f9";
}

i.icon.erase::before {
  content: "\f12d";
}

i.icon.eur::before {
  content: "\f153";
}

i.icon.euro::before {
  content: "\f153";
}

i.icon.exchange::before {
  content: "\f362";
}

i.icon.external::before {
  content: "\f35d";
}

i.icon.external.share::before {
  content: "\f14d";
}

i.icon.external.square::before {
  content: "\f360";
}

i.icon.eyedropper::before {
  content: "\f1fb";
}

i.icon.factory::before {
  content: "\f275";
}

i.icon.favorite::before {
  content: "\f005";
}

i.icon.feed::before {
  content: "\f09e";
}

i.icon.female.homosexual::before {
  content: "\f226";
}

i.icon.file.text::before {
  content: "\f15c";
}

i.icon.find::before {
  content: "\f1e5";
}

i.icon.first.aid::before {
  content: "\f0fa";
}

i.icon.food::before {
  content: "\f2e7";
}

i.icon.fork::before {
  content: "\f126";
}

i.icon.game::before {
  content: "\f11b";
}

i.icon.gay::before {
  content: "\f227";
}

i.icon.gbp::before {
  content: "\f154";
}

i.icon.grab::before {
  content: "\f255";
}

i.icon.graduation::before {
  content: "\f19d";
}

i.icon.grid.layout::before {
  content: "\f00a";
}

i.icon.group::before {
  content: "\f0c0";
}

i.icon.h::before {
  content: "\f0fd";
}

i.icon.hamburger::before {
  content: "\f0c9";
}

i.icon.hand.victory::before {
  content: "\f25b";
}

i.icon.handicap::before {
  content: "\f193";
}

i.icon.hard.of.hearing::before {
  content: "\f2a4";
}

i.icon.header::before {
  content: "\f1dc";
}

i.icon.heart.empty::before {
  content: "\f004";
}

i.icon.help::before {
  content: "\f128";
}

i.icon.help.circle::before {
  content: "\f059";
}

i.icon.heterosexual::before {
  content: "\f228";
}

i.icon.hide::before {
  content: "\f070";
}

i.icon.hotel::before {
  content: "\f236";
}

i.icon.hourglass.four::before {
  content: "\f254";
}

i.icon.hourglass.full::before {
  content: "\f254";
}

i.icon.hourglass.one::before {
  content: "\f251";
}

i.icon.hourglass.three::before {
  content: "\f253";
}

i.icon.hourglass.two::before {
  content: "\f252";
}

i.icon.hourglass.zero::before {
  content: "\f253";
}

i.icon.idea::before {
  content: "\f0eb";
}

i.icon.ils::before {
  content: "\f20b";
}

i.icon.inr::before {
  content: "\f156";
}

i.icon.intergender::before {
  content: "\f224";
}

i.icon.intersex::before {
  content: "\f224";
}

i.icon.jpy::before {
  content: "\f157";
}

i.icon.krw::before {
  content: "\f159";
}

i.icon.lab::before {
  content: "\f0c3";
}

i.icon.law::before {
  content: "\f24e";
}

i.icon.legal::before {
  content: "\f0e3";
}

i.icon.lesbian::before {
  content: "\f226";
}

i.icon.level.down::before {
  content: "\f3be";
}

i.icon.level.up::before {
  content: "\f3bf";
}

i.icon.lightning::before {
  content: "\f0e7";
}

i.icon.like::before {
  content: "\f004";
}

i.icon.linegraph::before {
  content: "\f201";
}

i.icon.linkify::before {
  content: "\f0c1";
}

i.icon.lira::before {
  content: "\f195";
}

i.icon.list.layout::before {
  content: "\f00b";
}

i.icon.magnify::before {
  content: "\f00e";
}

i.icon.mail::before {
  content: "\f0e0";
}

i.icon.mail.forward::before {
  content: "\f064";
}

i.icon.mail.square::before {
  content: "\f199";
}

i.icon.male.homosexual::before {
  content: "\f227";
}

i.icon.man::before {
  content: "\f222";
}

i.icon.marker::before {
  content: "\f041";
}

i.icon.mars.alternate::before {
  content: "\f229";
}

i.icon.mars.horizontal::before {
  content: "\f22b";
}

i.icon.mars.vertical::before {
  content: "\f22a";
}

i.icon.meanpath::before {
  content: "\f0c8";
}

i.icon.military::before {
  content: "\f0fb";
}

i.icon.money::before {
  content: "\f3d1";
}

i.icon.move::before {
  content: "\f0b2";
}

i.icon.mute::before {
  content: "\f131";
}

i.icon.non.binary.transgender::before {
  content: "\f223";
}

i.icon.numbered.list::before {
  content: "\f0cb";
}

i.icon.options::before {
  content: "\f1de";
}

i.icon.ordered.list::before {
  content: "\f0cb";
}

i.icon.other.gender::before {
  content: "\f229";
}

i.icon.other.gender.horizontal::before {
  content: "\f22b";
}

i.icon.other.gender.vertical::before {
  content: "\f22a";
}

i.icon.payment::before {
  content: "\f09d";
}

i.icon.pencil::before {
  content: "\f303";
}

i.icon.pencil.square::before {
  content: "\f14b";
}

i.icon.photo::before {
  content: "\f030";
}

i.icon.picture::before {
  content: "\f03e";
}

i.icon.pie.chart::before {
  content: "\f200";
}

i.icon.pie.graph::before {
  content: "\f200";
}

i.icon.pin::before {
  content: "\f08d";
}

i.icon.plus.cart::before {
  content: "\f217";
}

i.icon.point::before {
  content: "\f041";
}

i.icon.pointing.down::before {
  content: "\f0a7";
}

i.icon.pointing.left::before {
  content: "\f0a5";
}

i.icon.pointing.right::before {
  content: "\f0a4";
}

i.icon.pointing.up::before {
  content: "\f0a6";
}

i.icon.pound::before {
  content: "\f154";
}

i.icon.power::before {
  content: "\f011";
}

i.icon.power.cord::before {
  content: "\f1e6";
}

i.icon.privacy::before {
  content: "\f084";
}

i.icon.protect::before {
  content: "\f023";
}

i.icon.puzzle::before {
  content: "\f12e";
}

i.icon.r.circle::before {
  content: "\f25d";
}

i.icon.radio::before {
  content: "\f192";
}

i.icon.rain::before {
  content: "\f0e9";
}

i.icon.record::before {
  content: "\f03d";
}

i.icon.refresh::before {
  content: "\f021";
}

i.icon.remove::before {
  content: "\f00d";
}

i.icon.remove.bookmark::before {
  content: "\f02e";
}

i.icon.remove.circle::before {
  content: "\f057";
}

i.icon.remove.from.calendar::before {
  content: "\f272";
}

i.icon.remove.user::before {
  content: "\f235";
}

i.icon.repeat::before {
  content: "\f01e";
}

i.icon.resize.horizontal::before {
  content: "\f337";
}

i.icon.resize.vertical::before {
  content: "\f338";
}

i.icon.rmb::before {
  content: "\f157";
}

i.icon.rouble::before {
  content: "\f158";
}

i.icon.rub::before {
  content: "\f158";
}

i.icon.ruble::before {
  content: "\f158";
}

i.icon.rupee::before {
  content: "\f156";
}

i.icon.s15::before {
  content: "\f2cd";
}

i.icon.selected.radio::before {
  content: "\f192";
}

i.icon.send::before {
  content: "\f1d8";
}

i.icon.setting::before {
  content: "\f013";
}

i.icon.settings::before {
  content: "\f085";
}

i.icon.shekel::before {
  content: "\f20b";
}

i.icon.sheqel::before {
  content: "\f20b";
}

i.icon.shield::before {
  content: "\f3ed";
}

i.icon.shipping::before {
  content: "\f0d1";
}

i.icon.shop::before {
  content: "\f07a";
}

i.icon.shuffle::before {
  content: "\f074";
}

i.icon.shutdown::before {
  content: "\f011";
}

i.icon.sidebar::before {
  content: "\f0c9";
}

i.icon.signing::before {
  content: "\f2a7";
}

i.icon.signup::before {
  content: "\f044";
}

i.icon.sliders::before {
  content: "\f1de";
}

i.icon.soccer::before {
  content: "\f1e3";
}

i.icon.sort.alphabet.ascending::before {
  content: "\f15d";
}

i.icon.sort.alphabet.descending::before {
  content: "\f15e";
}

i.icon.sort.ascending::before {
  content: "\f0de";
}

i.icon.sort.content.ascending::before {
  content: "\f160";
}

i.icon.sort.content.descending::before {
  content: "\f161";
}

i.icon.sort.descending::before {
  content: "\f0dd";
}

i.icon.sort.numeric.ascending::before {
  content: "\f162";
}

i.icon.sort.numeric.descending::before {
  content: "\f163";
}

i.icon.sound::before {
  content: "\f025";
}

i.icon.spoon::before {
  content: "\f2e5";
}

i.icon.spy::before {
  content: "\f21b";
}

i.icon.star.empty::before {
  content: "\f005";
}

i.icon.star.half.empty::before {
  content: "\f089";
}

i.icon.star.half.full::before {
  content: "\f089";
}

i.icon.student::before {
  content: "\f19d";
}

i.icon.talk::before {
  content: "\f27a";
}

i.icon.target::before {
  content: "\f140";
}

i.icon.teletype::before {
  content: "\f1e4";
}

i.icon.television::before {
  content: "\f26c";
}

i.icon.text.cursor::before {
  content: "\f246";
}

i.icon.text.telephone::before {
  content: "\f1e4";
}

i.icon.theme::before {
  content: "\f043";
}

i.icon.thermometer::before {
  content: "\f2c7";
}

i.icon.thumb.tack::before {
  content: "\f08d";
}

i.icon.ticket::before {
  content: "\f3ff";
}

i.icon.time::before {
  content: "\f017";
}

i.icon.times.rectangle::before {
  content: "\f410";
}

i.icon.tm::before {
  content: "\f25c";
}

i.icon.toggle.down::before {
  content: "\f150";
}

i.icon.toggle.left::before {
  content: "\f191";
}

i.icon.toggle.right::before {
  content: "\f152";
}

i.icon.toggle.up::before {
  content: "\f151";
}

i.icon.translate::before {
  content: "\f1ab";
}

i.icon.travel::before {
  content: "\f0b1";
}

i.icon.treatment::before {
  content: "\f0f1";
}

i.icon.triangle.down::before {
  content: "\f0d7";
}

i.icon.triangle.left::before {
  content: "\f0d9";
}

i.icon.triangle.right::before {
  content: "\f0da";
}

i.icon.triangle.up::before {
  content: "\f0d8";
}

i.icon.try::before {
  content: "\f195";
}

i.icon.unhide::before {
  content: "\f06e";
}

i.icon.unlinkify::before {
  content: "\f127";
}

i.icon.unmute::before {
  content: "\f130";
}

i.icon.unordered.list::before {
  content: "\f0ca";
}

i.icon.usd::before {
  content: "\f155";
}

i.icon.user.cancel::before {
  content: "\f235";
}

i.icon.user.close::before {
  content: "\f235";
}

i.icon.user.delete::before {
  content: "\f235";
}

i.icon.user.doctor::before {
  content: "\f0f0";
}

i.icon.user.x::before {
  content: "\f235";
}

i.icon.vcard::before {
  content: "\f2bb";
}

i.icon.video.camera::before {
  content: "\f03d";
}

i.icon.video.play::before {
  content: "\f144";
}

i.icon.volume.control.phone::before {
  content: "\f2a0";
}

i.icon.wait::before {
  content: "\f017";
}

i.icon.warning::before {
  content: "\f12a";
}

i.icon.warning.circle::before {
  content: "\f06a";
}

i.icon.warning.sign::before {
  content: "\f071";
}

i.icon.wi.fi::before {
  content: "\f1eb";
}

i.icon.winner::before {
  content: "\f091";
}

i.icon.wizard::before {
  content: "\f0d0";
}

i.icon.woman::before {
  content: "\f221";
}

i.icon.won::before {
  content: "\f159";
}

i.icon.world::before {
  content: "\f0ac";
}

i.icon.write::before {
  content: "\f303";
}

i.icon.write.square::before {
  content: "\f14b";
}

i.icon.x::before {
  content: "\f00d";
}

i.icon.yen::before {
  content: "\f157";
}

i.icon.zip::before {
  content: "\f187";
}

i.icon.zoom::before {
  content: "\f00e";
}

/*******************************
             Outline Icons
    *******************************/

i.icon.outline {
  font-family: "outline-icons";
}

/* Icons */

i.icon.address.book.outline::before {
  content: "\f2b9";
}

i.icon.address.card.outline::before {
  content: "\f2bb";
}

i.icon.angry.outline::before {
  content: "\f556";
}

i.icon.arrow.alternate.circle.down.outline::before {
  content: "\f358";
}

i.icon.arrow.alternate.circle.left.outline::before {
  content: "\f359";
}

i.icon.arrow.alternate.circle.right.outline::before {
  content: "\f35a";
}

i.icon.arrow.alternate.circle.up.outline::before {
  content: "\f35b";
}

i.icon.bell.outline::before {
  content: "\f0f3";
}

i.icon.bell.slash.outline::before {
  content: "\f1f6";
}

i.icon.bookmark.outline::before {
  content: "\f02e";
}

i.icon.building.outline::before {
  content: "\f1ad";
}

i.icon.calendar.alternate.outline::before {
  content: "\f073";
}

i.icon.calendar.check.outline::before {
  content: "\f274";
}

i.icon.calendar.minus.outline::before {
  content: "\f272";
}

i.icon.calendar.outline::before {
  content: "\f133";
}

i.icon.calendar.plus.outline::before {
  content: "\f271";
}

i.icon.calendar.times.outline::before {
  content: "\f273";
}

i.icon.caret.square.down.outline::before {
  content: "\f150";
}

i.icon.caret.square.left.outline::before {
  content: "\f191";
}

i.icon.caret.square.right.outline::before {
  content: "\f152";
}

i.icon.caret.square.up.outline::before {
  content: "\f151";
}

i.icon.chart.bar.outline::before {
  content: "\f080";
}

i.icon.check.circle.outline::before {
  content: "\f058";
}

i.icon.check.square.outline::before {
  content: "\f14a";
}

i.icon.circle.outline::before {
  content: "\f111";
}

i.icon.clipboard.outline::before {
  content: "\f328";
}

i.icon.clock.outline::before {
  content: "\f017";
}

i.icon.clone.outline::before {
  content: "\f24d";
}

i.icon.closed.captioning.outline::before {
  content: "\f20a";
}

i.icon.comment.alternate.outline::before {
  content: "\f27a";
}

i.icon.comment.dots.outline::before {
  content: "\f4ad";
}

i.icon.comment.outline::before {
  content: "\f075";
}

i.icon.comments.outline::before {
  content: "\f086";
}

i.icon.compass.outline::before {
  content: "\f14e";
}

i.icon.copy.outline::before {
  content: "\f0c5";
}

i.icon.copyright.outline::before {
  content: "\f1f9";
}

i.icon.credit.card.outline::before {
  content: "\f09d";
}

i.icon.dizzy.outline::before {
  content: "\f567";
}

i.icon.dot.circle.outline::before {
  content: "\f192";
}

i.icon.edit.outline::before {
  content: "\f044";
}

i.icon.envelope.open.outline::before {
  content: "\f2b6";
}

i.icon.envelope.outline::before {
  content: "\f0e0";
}

i.icon.eye.outline::before {
  content: "\f06e";
}

i.icon.eye.slash.outline::before {
  content: "\f070";
}

i.icon.file.alternate.outline::before {
  content: "\f15c";
}

i.icon.file.archive.outline::before {
  content: "\f1c6";
}

i.icon.file.audio.outline::before {
  content: "\f1c7";
}

i.icon.file.code.outline::before {
  content: "\f1c9";
}

i.icon.file.excel.outline::before {
  content: "\f1c3";
}

i.icon.file.image.outline::before {
  content: "\f1c5";
}

i.icon.file.outline::before {
  content: "\f15b";
}

i.icon.file.pdf.outline::before {
  content: "\f1c1";
}

i.icon.file.powerpoint.outline::before {
  content: "\f1c4";
}

i.icon.file.video.outline::before {
  content: "\f1c8";
}

i.icon.file.word.outline::before {
  content: "\f1c2";
}

i.icon.flag.outline::before {
  content: "\f024";
}

i.icon.flushed.outline::before {
  content: "\f579";
}

i.icon.folder.open.outline::before {
  content: "\f07c";
}

i.icon.folder.outline::before {
  content: "\f07b";
}

i.icon.frown.open.outline::before {
  content: "\f57a";
}

i.icon.frown.outline::before {
  content: "\f119";
}

i.icon.futbol.outline::before {
  content: "\f1e3";
}

i.icon.gem.outline::before {
  content: "\f3a5";
}

i.icon.grimace.outline::before {
  content: "\f57f";
}

i.icon.grin.alternate.outline::before {
  content: "\f581";
}

i.icon.grin.beam.outline::before {
  content: "\f582";
}

i.icon.grin.beam.sweat.outline::before {
  content: "\f583";
}

i.icon.grin.hearts.outline::before {
  content: "\f584";
}

i.icon.grin.outline::before {
  content: "\f580";
}

i.icon.grin.squint.outline::before {
  content: "\f585";
}

i.icon.grin.squint.tears.outline::before {
  content: "\f586";
}

i.icon.grin.stars.outline::before {
  content: "\f587";
}

i.icon.grin.tears.outline::before {
  content: "\f588";
}

i.icon.grin.tongue.outline::before {
  content: "\f589";
}

i.icon.grin.tongue.squint.outline::before {
  content: "\f58a";
}

i.icon.grin.tongue.wink.outline::before {
  content: "\f58b";
}

i.icon.grin.wink.outline::before {
  content: "\f58c";
}

i.icon.hand.lizard.outline::before {
  content: "\f258";
}

i.icon.hand.paper.outline::before {
  content: "\f256";
}

i.icon.hand.peace.outline::before {
  content: "\f25b";
}

i.icon.hand.point.down.outline::before {
  content: "\f0a7";
}

i.icon.hand.point.left.outline::before {
  content: "\f0a5";
}

i.icon.hand.point.right.outline::before {
  content: "\f0a4";
}

i.icon.hand.point.up.outline::before {
  content: "\f0a6";
}

i.icon.hand.pointer.outline::before {
  content: "\f25a";
}

i.icon.hand.rock.outline::before {
  content: "\f255";
}

i.icon.hand.scissors.outline::before {
  content: "\f257";
}

i.icon.hand.spock.outline::before {
  content: "\f259";
}

i.icon.handshake.outline::before {
  content: "\f2b5";
}

i.icon.hdd.outline::before {
  content: "\f0a0";
}

i.icon.heart.outline::before {
  content: "\f004";
}

i.icon.hospital.outline::before {
  content: "\f0f8";
}

i.icon.hourglass.outline::before {
  content: "\f254";
}

i.icon.id.badge.outline::before {
  content: "\f2c1";
}

i.icon.id.card.outline::before {
  content: "\f2c2";
}

i.icon.image.outline::before {
  content: "\f03e";
}

i.icon.images.outline::before {
  content: "\f302";
}

i.icon.keyboard.outline::before {
  content: "\f11c";
}

i.icon.kiss.beam.outline::before {
  content: "\f597";
}

i.icon.kiss.outline::before {
  content: "\f596";
}

i.icon.kiss.wink.heart.outline::before {
  content: "\f598";
}

i.icon.laugh.beam.outline::before {
  content: "\f59a";
}

i.icon.laugh.outline::before {
  content: "\f599";
}

i.icon.laugh.squint.outline::before {
  content: "\f59b";
}

i.icon.laugh.wink.outline::before {
  content: "\f59c";
}

i.icon.lemon.outline::before {
  content: "\f094";
}

i.icon.life.ring.outline::before {
  content: "\f1cd";
}

i.icon.lightbulb.outline::before {
  content: "\f0eb";
}

i.icon.list.alternate.outline::before {
  content: "\f022";
}

i.icon.map.outline::before {
  content: "\f279";
}

i.icon.meh.blank.outline::before {
  content: "\f5a4";
}

i.icon.meh.outline::before {
  content: "\f11a";
}

i.icon.meh.rolling.eyes.outline::before {
  content: "\f5a5";
}

i.icon.minus.square.outline::before {
  content: "\f146";
}

i.icon.money.bill.alternate.outline::before {
  content: "\f3d1";
}

i.icon.moon.outline::before {
  content: "\f186";
}

i.icon.newspaper.outline::before {
  content: "\f1ea";
}

i.icon.object.group.outline::before {
  content: "\f247";
}

i.icon.object.ungroup.outline::before {
  content: "\f248";
}

i.icon.paper.plane.outline::before {
  content: "\f1d8";
}

i.icon.pause.circle.outline::before {
  content: "\f28b";
}

i.icon.play.circle.outline::before {
  content: "\f144";
}

i.icon.plus.square.outline::before {
  content: "\f0fe";
}

i.icon.question.circle.outline::before {
  content: "\f059";
}

i.icon.registered.outline::before {
  content: "\f25d";
}

i.icon.sad.cry.outline::before {
  content: "\f5b3";
}

i.icon.sad.tear.outline::before {
  content: "\f5b4";
}

i.icon.save.outline::before {
  content: "\f0c7";
}

i.icon.share.square.outline::before {
  content: "\f14d";
}

i.icon.smile.beam.outline::before {
  content: "\f5b8";
}

i.icon.smile.outline::before {
  content: "\f118";
}

i.icon.smile.wink.outline::before {
  content: "\f4da";
}

i.icon.snowflake.outline::before {
  content: "\f2dc";
}

i.icon.square.outline::before {
  content: "\f0c8";
}

i.icon.star.half.outline::before {
  content: "\f089";
}

i.icon.star.outline::before {
  content: "\f005";
}

i.icon.sticky.note.outline::before {
  content: "\f249";
}

i.icon.stop.circle.outline::before {
  content: "\f28d";
}

i.icon.sun.outline::before {
  content: "\f185";
}

i.icon.surprise.outline::before {
  content: "\f5c2";
}

i.icon.thumbs.down.outline::before {
  content: "\f165";
}

i.icon.thumbs.up.outline::before {
  content: "\f164";
}

i.icon.times.circle.outline::before {
  content: "\f057";
}

i.icon.tired.outline::before {
  content: "\f5c8";
}

i.icon.trash.alternate.outline::before {
  content: "\f2ed";
}

i.icon.user.circle.outline::before {
  content: "\f2bd";
}

i.icon.user.outline::before {
  content: "\f007";
}

i.icon.window.close.outline::before {
  content: "\f410";
}

i.icon.window.maximize.outline::before {
  content: "\f2d0";
}

i.icon.window.minimize.outline::before {
  content: "\f2d1";
}

i.icon.window.restore.outline::before {
  content: "\f2d2";
}

/* Aliases */

/*******************************
              Brand Icons
    *******************************/

/* Icons */

i.icon.\35 00px::before {
  content: "\f26e";
  font-family: "brand-icons";
}

i.icon.accessible::before {
  content: "\f368";
  font-family: "brand-icons";
}

i.icon.accusoft::before {
  content: "\f369";
  font-family: "brand-icons";
}

i.icon.acquisitions.incorporated::before {
  content: "\f6af";
  font-family: "brand-icons";
}

i.icon.adn::before {
  content: "\f170";
  font-family: "brand-icons";
}

i.icon.adversal::before {
  content: "\f36a";
  font-family: "brand-icons";
}

i.icon.affiliatetheme::before {
  content: "\f36b";
  font-family: "brand-icons";
}

i.icon.airbnb::before {
  content: "\f834";
  font-family: "brand-icons";
}

i.icon.algolia::before {
  content: "\f36c";
  font-family: "brand-icons";
}

i.icon.alipay::before {
  content: "\f642";
  font-family: "brand-icons";
}

i.icon.amazon::before {
  content: "\f270";
  font-family: "brand-icons";
}

i.icon.amazon.pay::before {
  content: "\f42c";
  font-family: "brand-icons";
}

i.icon.amilia::before {
  content: "\f36d";
  font-family: "brand-icons";
}

i.icon.android::before {
  content: "\f17b";
  font-family: "brand-icons";
}

i.icon.angellist::before {
  content: "\f209";
  font-family: "brand-icons";
}

i.icon.angrycreative::before {
  content: "\f36e";
  font-family: "brand-icons";
}

i.icon.angular::before {
  content: "\f420";
  font-family: "brand-icons";
}

i.icon.app.store::before {
  content: "\f36f";
  font-family: "brand-icons";
}

i.icon.app.store.ios::before {
  content: "\f370";
  font-family: "brand-icons";
}

i.icon.apper::before {
  content: "\f371";
  font-family: "brand-icons";
}

i.icon.apple::before {
  content: "\f179";
  font-family: "brand-icons";
}

i.icon.apple.pay::before {
  content: "\f415";
  font-family: "brand-icons";
}

i.icon.artstation::before {
  content: "\f77a";
  font-family: "brand-icons";
}

i.icon.asymmetrik::before {
  content: "\f372";
  font-family: "brand-icons";
}

i.icon.atlassian::before {
  content: "\f77b";
  font-family: "brand-icons";
}

i.icon.audible::before {
  content: "\f373";
  font-family: "brand-icons";
}

i.icon.autoprefixer::before {
  content: "\f41c";
  font-family: "brand-icons";
}

i.icon.avianex::before {
  content: "\f374";
  font-family: "brand-icons";
}

i.icon.aviato::before {
  content: "\f421";
  font-family: "brand-icons";
}

i.icon.aws::before {
  content: "\f375";
  font-family: "brand-icons";
}

i.icon.bandcamp::before {
  content: "\f2d5";
  font-family: "brand-icons";
}

i.icon.battle.net::before {
  content: "\f835";
  font-family: "brand-icons";
}

i.icon.behance::before {
  content: "\f1b4";
  font-family: "brand-icons";
}

i.icon.behance.square::before {
  content: "\f1b5";
  font-family: "brand-icons";
}

i.icon.bimobject::before {
  content: "\f378";
  font-family: "brand-icons";
}

i.icon.bitbucket::before {
  content: "\f171";
  font-family: "brand-icons";
}

i.icon.bitcoin::before {
  content: "\f379";
  font-family: "brand-icons";
}

i.icon.bity::before {
  content: "\f37a";
  font-family: "brand-icons";
}

i.icon.black.tie::before {
  content: "\f27e";
  font-family: "brand-icons";
}

i.icon.blackberry::before {
  content: "\f37b";
  font-family: "brand-icons";
}

i.icon.blogger::before {
  content: "\f37c";
  font-family: "brand-icons";
}

i.icon.blogger.b::before {
  content: "\f37d";
  font-family: "brand-icons";
}

i.icon.bluetooth::before {
  content: "\f293";
  font-family: "brand-icons";
}

i.icon.bluetooth.b::before {
  content: "\f294";
  font-family: "brand-icons";
}

i.icon.bootstrap::before {
  content: "\f836";
  font-family: "brand-icons";
}

i.icon.btc::before {
  content: "\f15a";
  font-family: "brand-icons";
}

i.icon.buffer::before {
  content: "\f837";
  font-family: "brand-icons";
}

i.icon.buromobelexperte::before {
  content: "\f37f";
  font-family: "brand-icons";
}

i.icon.buy.n.large::before {
  content: "\f8a6";
  font-family: "brand-icons";
}

i.icon.buysellads::before {
  content: "\f20d";
  font-family: "brand-icons";
}

i.icon.canadian.maple.leaf::before {
  content: "\f785";
  font-family: "brand-icons";
}

i.icon.cc.amazon.pay::before {
  content: "\f42d";
  font-family: "brand-icons";
}

i.icon.cc.amex::before {
  content: "\f1f3";
  font-family: "brand-icons";
}

i.icon.cc.apple.pay::before {
  content: "\f416";
  font-family: "brand-icons";
}

i.icon.cc.diners.club::before {
  content: "\f24c";
  font-family: "brand-icons";
}

i.icon.cc.discover::before {
  content: "\f1f2";
  font-family: "brand-icons";
}

i.icon.cc.jcb::before {
  content: "\f24b";
  font-family: "brand-icons";
}

i.icon.cc.mastercard::before {
  content: "\f1f1";
  font-family: "brand-icons";
}

i.icon.cc.paypal::before {
  content: "\f1f4";
  font-family: "brand-icons";
}

i.icon.cc.stripe::before {
  content: "\f1f5";
  font-family: "brand-icons";
}

i.icon.cc.visa::before {
  content: "\f1f0";
  font-family: "brand-icons";
}

i.icon.centercode::before {
  content: "\f380";
  font-family: "brand-icons";
}

i.icon.centos::before {
  content: "\f789";
  font-family: "brand-icons";
}

i.icon.chrome::before {
  content: "\f268";
  font-family: "brand-icons";
}

i.icon.chromecast::before {
  content: "\f838";
  font-family: "brand-icons";
}

i.icon.cloudflare::before {
  content: "\e07d";
  font-family: "brand-icons";
}

i.icon.cloudscale::before {
  content: "\f383";
  font-family: "brand-icons";
}

i.icon.cloudsmith::before {
  content: "\f384";
  font-family: "brand-icons";
}

i.icon.cloudversify::before {
  content: "\f385";
  font-family: "brand-icons";
}

i.icon.codepen::before {
  content: "\f1cb";
  font-family: "brand-icons";
}

i.icon.codiepie::before {
  content: "\f284";
  font-family: "brand-icons";
}

i.icon.confluence::before {
  content: "\f78d";
  font-family: "brand-icons";
}

i.icon.connectdevelop::before {
  content: "\f20e";
  font-family: "brand-icons";
}

i.icon.contao::before {
  content: "\f26d";
  font-family: "brand-icons";
}

i.icon.cotton.bureau::before {
  content: "\f89e";
  font-family: "brand-icons";
}

i.icon.cpanel::before {
  content: "\f388";
  font-family: "brand-icons";
}

i.icon.creative.commons::before {
  content: "\f25e";
  font-family: "brand-icons";
}

i.icon.creative.commons.by::before {
  content: "\f4e7";
  font-family: "brand-icons";
}

i.icon.creative.commons.nc::before {
  content: "\f4e8";
  font-family: "brand-icons";
}

i.icon.creative.commons.nc.eu::before {
  content: "\f4e9";
  font-family: "brand-icons";
}

i.icon.creative.commons.nc.jp::before {
  content: "\f4ea";
  font-family: "brand-icons";
}

i.icon.creative.commons.nd::before {
  content: "\f4eb";
  font-family: "brand-icons";
}

i.icon.creative.commons.pd::before {
  content: "\f4ec";
  font-family: "brand-icons";
}

i.icon.creative.commons.pd.alternate::before {
  content: "\f4ed";
  font-family: "brand-icons";
}

i.icon.creative.commons.remix::before {
  content: "\f4ee";
  font-family: "brand-icons";
}

i.icon.creative.commons.sa::before {
  content: "\f4ef";
  font-family: "brand-icons";
}

i.icon.creative.commons.sampling::before {
  content: "\f4f0";
  font-family: "brand-icons";
}

i.icon.creative.commons.sampling.plus::before {
  content: "\f4f1";
  font-family: "brand-icons";
}

i.icon.creative.commons.share::before {
  content: "\f4f2";
  font-family: "brand-icons";
}

i.icon.creative.commons.zero::before {
  content: "\f4f3";
  font-family: "brand-icons";
}

i.icon.critical.role::before {
  content: "\f6c9";
  font-family: "brand-icons";
}

i.icon.css3::before {
  content: "\f13c";
  font-family: "brand-icons";
}

i.icon.css3.alternate::before {
  content: "\f38b";
  font-family: "brand-icons";
}

i.icon.cuttlefish::before {
  content: "\f38c";
  font-family: "brand-icons";
}

i.icon.d.and.d::before {
  content: "\f38d";
  font-family: "brand-icons";
}

i.icon.d.and.d.beyond::before {
  content: "\f6ca";
  font-family: "brand-icons";
}

i.icon.dailymotion::before {
  content: "\e052";
  font-family: "brand-icons";
}

i.icon.dashcube::before {
  content: "\f210";
  font-family: "brand-icons";
}

i.icon.deezer::before {
  content: "\e077";
  font-family: "brand-icons";
}

i.icon.delicious::before {
  content: "\f1a5";
  font-family: "brand-icons";
}

i.icon.deploydog::before {
  content: "\f38e";
  font-family: "brand-icons";
}

i.icon.deskpro::before {
  content: "\f38f";
  font-family: "brand-icons";
}

i.icon.dev::before {
  content: "\f6cc";
  font-family: "brand-icons";
}

i.icon.deviantart::before {
  content: "\f1bd";
  font-family: "brand-icons";
}

i.icon.dhl::before {
  content: "\f790";
  font-family: "brand-icons";
}

i.icon.diaspora::before {
  content: "\f791";
  font-family: "brand-icons";
}

i.icon.digg::before {
  content: "\f1a6";
  font-family: "brand-icons";
}

i.icon.digital.ocean::before {
  content: "\f391";
  font-family: "brand-icons";
}

i.icon.discord::before {
  content: "\f392";
  font-family: "brand-icons";
}

i.icon.discourse::before {
  content: "\f393";
  font-family: "brand-icons";
}

i.icon.dochub::before {
  content: "\f394";
  font-family: "brand-icons";
}

i.icon.docker::before {
  content: "\f395";
  font-family: "brand-icons";
}

i.icon.draft2digital::before {
  content: "\f396";
  font-family: "brand-icons";
}

i.icon.dribbble::before {
  content: "\f17d";
  font-family: "brand-icons";
}

i.icon.dribbble.square::before {
  content: "\f397";
  font-family: "brand-icons";
}

i.icon.dropbox::before {
  content: "\f16b";
  font-family: "brand-icons";
}

i.icon.drupal::before {
  content: "\f1a9";
  font-family: "brand-icons";
}

i.icon.dyalog::before {
  content: "\f399";
  font-family: "brand-icons";
}

i.icon.earlybirds::before {
  content: "\f39a";
  font-family: "brand-icons";
}

i.icon.ebay::before {
  content: "\f4f4";
  font-family: "brand-icons";
}

i.icon.edge::before {
  content: "\f282";
  font-family: "brand-icons";
}

i.icon.edge.legacy::before {
  content: "\e078";
  font-family: "brand-icons";
}

i.icon.elementor::before {
  content: "\f430";
  font-family: "brand-icons";
}

i.icon.ello::before {
  content: "\f5f1";
  font-family: "brand-icons";
}

i.icon.ember::before {
  content: "\f423";
  font-family: "brand-icons";
}

i.icon.empire::before {
  content: "\f1d1";
  font-family: "brand-icons";
}

i.icon.envira::before {
  content: "\f299";
  font-family: "brand-icons";
}

i.icon.erlang::before {
  content: "\f39d";
  font-family: "brand-icons";
}

i.icon.ethereum::before {
  content: "\f42e";
  font-family: "brand-icons";
}

i.icon.etsy::before {
  content: "\f2d7";
  font-family: "brand-icons";
}

i.icon.evernote::before {
  content: "\f839";
  font-family: "brand-icons";
}

i.icon.expeditedssl::before {
  content: "\f23e";
  font-family: "brand-icons";
}

i.icon.facebook::before {
  content: "\f09a";
  font-family: "brand-icons";
}

i.icon.facebook.f::before {
  content: "\f39e";
  font-family: "brand-icons";
}

i.icon.facebook.messenger::before {
  content: "\f39f";
  font-family: "brand-icons";
}

i.icon.facebook.square::before {
  content: "\f082";
  font-family: "brand-icons";
}

i.icon.fantasy.flight.games::before {
  content: "\f6dc";
  font-family: "brand-icons";
}

i.icon.fedex::before {
  content: "\f797";
  font-family: "brand-icons";
}

i.icon.fedora::before {
  content: "\f798";
  font-family: "brand-icons";
}

i.icon.figma::before {
  content: "\f799";
  font-family: "brand-icons";
}

i.icon.firefox::before {
  content: "\f269";
  font-family: "brand-icons";
}

i.icon.firefox.browser::before {
  content: "\e007";
  font-family: "brand-icons";
}

i.icon.first.order::before {
  content: "\f2b0";
  font-family: "brand-icons";
}

i.icon.first.order.alternate::before {
  content: "\f50a";
  font-family: "brand-icons";
}

i.icon.firstdraft::before {
  content: "\f3a1";
  font-family: "brand-icons";
}

i.icon.flickr::before {
  content: "\f16e";
  font-family: "brand-icons";
}

i.icon.flipboard::before {
  content: "\f44d";
  font-family: "brand-icons";
}

i.icon.fly::before {
  content: "\f417";
  font-family: "brand-icons";
}

i.icon.font.awesome::before {
  content: "\f2b4";
  font-family: "brand-icons";
}

i.icon.font.awesome.alternate::before {
  content: "\f35c";
  font-family: "brand-icons";
}

i.icon.font.awesome.flag::before {
  content: "\f425";
  font-family: "brand-icons";
}

i.icon.fonticons::before {
  content: "\f280";
  font-family: "brand-icons";
}

i.icon.fonticons.fi::before {
  content: "\f3a2";
  font-family: "brand-icons";
}

i.icon.fort.awesome::before {
  content: "\f286";
  font-family: "brand-icons";
}

i.icon.fort.awesome.alternate::before {
  content: "\f3a3";
  font-family: "brand-icons";
}

i.icon.forumbee::before {
  content: "\f211";
  font-family: "brand-icons";
}

i.icon.foursquare::before {
  content: "\f180";
  font-family: "brand-icons";
}

i.icon.free.code.camp::before {
  content: "\f2c5";
  font-family: "brand-icons";
}

i.icon.freebsd::before {
  content: "\f3a4";
  font-family: "brand-icons";
}

i.icon.fulcrum::before {
  content: "\f50b";
  font-family: "brand-icons";
}

i.icon.galactic.republic::before {
  content: "\f50c";
  font-family: "brand-icons";
}

i.icon.galactic.senate::before {
  content: "\f50d";
  font-family: "brand-icons";
}

i.icon.get.pocket::before {
  content: "\f265";
  font-family: "brand-icons";
}

i.icon.gg::before {
  content: "\f260";
  font-family: "brand-icons";
}

i.icon.gg.circle::before {
  content: "\f261";
  font-family: "brand-icons";
}

i.icon.git::before {
  content: "\f1d3";
  font-family: "brand-icons";
}

i.icon.git.alternate::before {
  content: "\f841";
  font-family: "brand-icons";
}

i.icon.git.square::before {
  content: "\f1d2";
  font-family: "brand-icons";
}

i.icon.github::before {
  content: "\f09b";
  font-family: "brand-icons";
}

i.icon.github.alternate::before {
  content: "\f113";
  font-family: "brand-icons";
}

i.icon.github.square::before {
  content: "\f092";
  font-family: "brand-icons";
}

i.icon.gitkraken::before {
  content: "\f3a6";
  font-family: "brand-icons";
}

i.icon.gitlab::before {
  content: "\f296";
  font-family: "brand-icons";
}

i.icon.gitter::before {
  content: "\f426";
  font-family: "brand-icons";
}

i.icon.glide::before {
  content: "\f2a5";
  font-family: "brand-icons";
}

i.icon.glide.g::before {
  content: "\f2a6";
  font-family: "brand-icons";
}

i.icon.gofore::before {
  content: "\f3a7";
  font-family: "brand-icons";
}

i.icon.goodreads::before {
  content: "\f3a8";
  font-family: "brand-icons";
}

i.icon.goodreads.g::before {
  content: "\f3a9";
  font-family: "brand-icons";
}

i.icon.google::before {
  content: "\f1a0";
  font-family: "brand-icons";
}

i.icon.google.drive::before {
  content: "\f3aa";
  font-family: "brand-icons";
}

i.icon.google.pay::before {
  content: "\e079";
  font-family: "brand-icons";
}

i.icon.google.play::before {
  content: "\f3ab";
  font-family: "brand-icons";
}

i.icon.google.plus::before {
  content: "\f2b3";
  font-family: "brand-icons";
}

i.icon.google.plus.g::before {
  content: "\f0d5";
  font-family: "brand-icons";
}

i.icon.google.plus.square::before {
  content: "\f0d4";
  font-family: "brand-icons";
}

i.icon.google.wallet::before {
  content: "\f1ee";
  font-family: "brand-icons";
}

i.icon.gratipay::before {
  content: "\f184";
  font-family: "brand-icons";
}

i.icon.grav::before {
  content: "\f2d6";
  font-family: "brand-icons";
}

i.icon.gripfire::before {
  content: "\f3ac";
  font-family: "brand-icons";
}

i.icon.grunt::before {
  content: "\f3ad";
  font-family: "brand-icons";
}

i.icon.guilded::before {
  content: "\e07e";
  font-family: "brand-icons";
}

i.icon.gulp::before {
  content: "\f3ae";
  font-family: "brand-icons";
}

i.icon.hacker.news::before {
  content: "\f1d4";
  font-family: "brand-icons";
}

i.icon.hacker.news.square::before {
  content: "\f3af";
  font-family: "brand-icons";
}

i.icon.hackerrank::before {
  content: "\f5f7";
  font-family: "brand-icons";
}

i.icon.hips::before {
  content: "\f452";
  font-family: "brand-icons";
}

i.icon.hire.a.helper::before {
  content: "\f3b0";
  font-family: "brand-icons";
}

i.icon.hive::before {
  content: "\e07f";
  font-family: "brand-icons";
}

i.icon.hooli::before {
  content: "\f427";
  font-family: "brand-icons";
}

i.icon.hornbill::before {
  content: "\f592";
  font-family: "brand-icons";
}

i.icon.hotjar::before {
  content: "\f3b1";
  font-family: "brand-icons";
}

i.icon.houzz::before {
  content: "\f27c";
  font-family: "brand-icons";
}

i.icon.html5::before {
  content: "\f13b";
  font-family: "brand-icons";
}

i.icon.hubspot::before {
  content: "\f3b2";
  font-family: "brand-icons";
}

i.icon.ideal::before {
  content: "\e013";
  font-family: "brand-icons";
}

i.icon.imdb::before {
  content: "\f2d8";
  font-family: "brand-icons";
}

i.icon.innosoft::before {
  content: "\e080";
  font-family: "brand-icons";
}

i.icon.instagram::before {
  content: "\f16d";
  font-family: "brand-icons";
}

i.icon.instagram.square::before {
  content: "\e055";
  font-family: "brand-icons";
}

i.icon.instalod::before {
  content: "\e081";
  font-family: "brand-icons";
}

i.icon.intercom::before {
  content: "\f7af";
  font-family: "brand-icons";
}

i.icon.internet.explorer::before {
  content: "\f26b";
  font-family: "brand-icons";
}

i.icon.invision::before {
  content: "\f7b0";
  font-family: "brand-icons";
}

i.icon.ioxhost::before {
  content: "\f208";
  font-family: "brand-icons";
}

i.icon.itch.io::before {
  content: "\f83a";
  font-family: "brand-icons";
}

i.icon.itunes::before {
  content: "\f3b4";
  font-family: "brand-icons";
}

i.icon.itunes.note::before {
  content: "\f3b5";
  font-family: "brand-icons";
}

i.icon.java::before {
  content: "\f4e4";
  font-family: "brand-icons";
}

i.icon.jedi.order::before {
  content: "\f50e";
  font-family: "brand-icons";
}

i.icon.jenkins::before {
  content: "\f3b6";
  font-family: "brand-icons";
}

i.icon.jira::before {
  content: "\f7b1";
  font-family: "brand-icons";
}

i.icon.joget::before {
  content: "\f3b7";
  font-family: "brand-icons";
}

i.icon.joomla::before {
  content: "\f1aa";
  font-family: "brand-icons";
}

i.icon.js::before {
  content: "\f3b8";
  font-family: "brand-icons";
}

i.icon.js.square::before {
  content: "\f3b9";
  font-family: "brand-icons";
}

i.icon.jsfiddle::before {
  content: "\f1cc";
  font-family: "brand-icons";
}

i.icon.kaggle::before {
  content: "\f5fa";
  font-family: "brand-icons";
}

i.icon.keybase::before {
  content: "\f4f5";
  font-family: "brand-icons";
}

i.icon.keycdn::before {
  content: "\f3ba";
  font-family: "brand-icons";
}

i.icon.kickstarter::before {
  content: "\f3bb";
  font-family: "brand-icons";
}

i.icon.kickstarter.k::before {
  content: "\f3bc";
  font-family: "brand-icons";
}

i.icon.korvue::before {
  content: "\f42f";
  font-family: "brand-icons";
}

i.icon.laravel::before {
  content: "\f3bd";
  font-family: "brand-icons";
}

i.icon.lastfm::before {
  content: "\f202";
  font-family: "brand-icons";
}

i.icon.lastfm.square::before {
  content: "\f203";
  font-family: "brand-icons";
}

i.icon.leanpub::before {
  content: "\f212";
  font-family: "brand-icons";
}

i.icon.lesscss::before {
  content: "\f41d";
  font-family: "brand-icons";
}

i.icon.linechat::before {
  content: "\f3c0";
  font-family: "brand-icons";
}

i.icon.linkedin::before {
  content: "\f08c";
  font-family: "brand-icons";
}

i.icon.linode::before {
  content: "\f2b8";
  font-family: "brand-icons";
}

i.icon.linux::before {
  content: "\f17c";
  font-family: "brand-icons";
}

i.icon.lyft::before {
  content: "\f3c3";
  font-family: "brand-icons";
}

i.icon.magento::before {
  content: "\f3c4";
  font-family: "brand-icons";
}

i.icon.mailchimp::before {
  content: "\f59e";
  font-family: "brand-icons";
}

i.icon.mandalorian::before {
  content: "\f50f";
  font-family: "brand-icons";
}

i.icon.markdown::before {
  content: "\f60f";
  font-family: "brand-icons";
}

i.icon.mastodon::before {
  content: "\f4f6";
  font-family: "brand-icons";
}

i.icon.maxcdn::before {
  content: "\f136";
  font-family: "brand-icons";
}

i.icon.mdb::before {
  content: "\f8ca";
  font-family: "brand-icons";
}

i.icon.medapps::before {
  content: "\f3c6";
  font-family: "brand-icons";
}

i.icon.medium::before {
  content: "\f23a";
  font-family: "brand-icons";
}

i.icon.medium.m::before {
  content: "\f3c7";
  font-family: "brand-icons";
}

i.icon.medrt::before {
  content: "\f3c8";
  font-family: "brand-icons";
}

i.icon.meetup::before {
  content: "\f2e0";
  font-family: "brand-icons";
}

i.icon.megaport::before {
  content: "\f5a3";
  font-family: "brand-icons";
}

i.icon.mendeley::before {
  content: "\f7b3";
  font-family: "brand-icons";
}

i.icon.microblog::before {
  content: "\e01a";
  font-family: "brand-icons";
}

i.icon.microsoft::before {
  content: "\f3ca";
  font-family: "brand-icons";
}

i.icon.mix::before {
  content: "\f3cb";
  font-family: "brand-icons";
}

i.icon.mixcloud::before {
  content: "\f289";
  font-family: "brand-icons";
}

i.icon.mixer::before {
  content: "\e056";
  font-family: "brand-icons";
}

i.icon.mizuni::before {
  content: "\f3cc";
  font-family: "brand-icons";
}

i.icon.modx::before {
  content: "\f285";
  font-family: "brand-icons";
}

i.icon.monero::before {
  content: "\f3d0";
  font-family: "brand-icons";
}

i.icon.napster::before {
  content: "\f3d2";
  font-family: "brand-icons";
}

i.icon.neos::before {
  content: "\f612";
  font-family: "brand-icons";
}

i.icon.nimblr::before {
  content: "\f5a8";
  font-family: "brand-icons";
}

i.icon.node::before {
  content: "\f419";
  font-family: "brand-icons";
}

i.icon.node.js::before {
  content: "\f3d3";
  font-family: "brand-icons";
}

i.icon.npm::before {
  content: "\f3d4";
  font-family: "brand-icons";
}

i.icon.ns8::before {
  content: "\f3d5";
  font-family: "brand-icons";
}

i.icon.nutritionix::before {
  content: "\f3d6";
  font-family: "brand-icons";
}

i.icon.octopus.deploy::before {
  content: "\e082";
  font-family: "brand-icons";
}

i.icon.odnoklassniki::before {
  content: "\f263";
  font-family: "brand-icons";
}

i.icon.odnoklassniki.square::before {
  content: "\f264";
  font-family: "brand-icons";
}

i.icon.old.republic::before {
  content: "\f510";
  font-family: "brand-icons";
}

i.icon.opencart::before {
  content: "\f23d";
  font-family: "brand-icons";
}

i.icon.openid::before {
  content: "\f19b";
  font-family: "brand-icons";
}

i.icon.opera::before {
  content: "\f26a";
  font-family: "brand-icons";
}

i.icon.optin.monster::before {
  content: "\f23c";
  font-family: "brand-icons";
}

i.icon.orcid::before {
  content: "\f8d2";
  font-family: "brand-icons";
}

i.icon.osi::before {
  content: "\f41a";
  font-family: "brand-icons";
}

i.icon.page4::before {
  content: "\f3d7";
  font-family: "brand-icons";
}

i.icon.pagelines::before {
  content: "\f18c";
  font-family: "brand-icons";
}

i.icon.palfed::before {
  content: "\f3d8";
  font-family: "brand-icons";
}

i.icon.patreon::before {
  content: "\f3d9";
  font-family: "brand-icons";
}

i.icon.paypal::before {
  content: "\f1ed";
  font-family: "brand-icons";
}

i.icon.penny.arcade::before {
  content: "\f704";
  font-family: "brand-icons";
}

i.icon.perbyte::before {
  content: "\e083";
  font-family: "brand-icons";
}

i.icon.periscope::before {
  content: "\f3da";
  font-family: "brand-icons";
}

i.icon.phabricator::before {
  content: "\f3db";
  font-family: "brand-icons";
}

i.icon.phoenix.framework::before {
  content: "\f3dc";
  font-family: "brand-icons";
}

i.icon.phoenix.squadron::before {
  content: "\f511";
  font-family: "brand-icons";
}

i.icon.php::before {
  content: "\f457";
  font-family: "brand-icons";
}

i.icon.pied.piper::before {
  content: "\f2ae";
  font-family: "brand-icons";
}

i.icon.pied.piper.alternate::before {
  content: "\f1a8";
  font-family: "brand-icons";
}

i.icon.pied.piper.hat::before {
  content: "\f4e5";
  font-family: "brand-icons";
}

i.icon.pied.piper.pp::before {
  content: "\f1a7";
  font-family: "brand-icons";
}

i.icon.pied.piper.square::before {
  content: "\e01e";
  font-family: "brand-icons";
}

i.icon.pinterest::before {
  content: "\f0d2";
  font-family: "brand-icons";
}

i.icon.pinterest.p::before {
  content: "\f231";
  font-family: "brand-icons";
}

i.icon.pinterest.square::before {
  content: "\f0d3";
  font-family: "brand-icons";
}

i.icon.playstation::before {
  content: "\f3df";
  font-family: "brand-icons";
}

i.icon.product.hunt::before {
  content: "\f288";
  font-family: "brand-icons";
}

i.icon.pushed::before {
  content: "\f3e1";
  font-family: "brand-icons";
}

i.icon.python::before {
  content: "\f3e2";
  font-family: "brand-icons";
}

i.icon.qq::before {
  content: "\f1d6";
  font-family: "brand-icons";
}

i.icon.quinscape::before {
  content: "\f459";
  font-family: "brand-icons";
}

i.icon.quora::before {
  content: "\f2c4";
  font-family: "brand-icons";
}

i.icon.r.project::before {
  content: "\f4f7";
  font-family: "brand-icons";
}

i.icon.raspberry.pi::before {
  content: "\f7bb";
  font-family: "brand-icons";
}

i.icon.ravelry::before {
  content: "\f2d9";
  font-family: "brand-icons";
}

i.icon.react::before {
  content: "\f41b";
  font-family: "brand-icons";
}

i.icon.reacteurope::before {
  content: "\f75d";
  font-family: "brand-icons";
}

i.icon.readme::before {
  content: "\f4d5";
  font-family: "brand-icons";
}

i.icon.rebel::before {
  content: "\f1d0";
  font-family: "brand-icons";
}

i.icon.reddit::before {
  content: "\f1a1";
  font-family: "brand-icons";
}

i.icon.reddit.alien::before {
  content: "\f281";
  font-family: "brand-icons";
}

i.icon.reddit.square::before {
  content: "\f1a2";
  font-family: "brand-icons";
}

i.icon.redhat::before {
  content: "\f7bc";
  font-family: "brand-icons";
}

i.icon.redriver::before {
  content: "\f3e3";
  font-family: "brand-icons";
}

i.icon.redyeti::before {
  content: "\f69d";
  font-family: "brand-icons";
}

i.icon.renren::before {
  content: "\f18b";
  font-family: "brand-icons";
}

i.icon.replyd::before {
  content: "\f3e6";
  font-family: "brand-icons";
}

i.icon.researchgate::before {
  content: "\f4f8";
  font-family: "brand-icons";
}

i.icon.resolving::before {
  content: "\f3e7";
  font-family: "brand-icons";
}

i.icon.rev::before {
  content: "\f5b2";
  font-family: "brand-icons";
}

i.icon.rocketchat::before {
  content: "\f3e8";
  font-family: "brand-icons";
}

i.icon.rockrms::before {
  content: "\f3e9";
  font-family: "brand-icons";
}

i.icon.rust::before {
  content: "\e07a";
  font-family: "brand-icons";
}

i.icon.safari::before {
  content: "\f267";
  font-family: "brand-icons";
}

i.icon.salesforce::before {
  content: "\f83b";
  font-family: "brand-icons";
}

i.icon.sass::before {
  content: "\f41e";
  font-family: "brand-icons";
}

i.icon.schlix::before {
  content: "\f3ea";
  font-family: "brand-icons";
}

i.icon.scribd::before {
  content: "\f28a";
  font-family: "brand-icons";
}

i.icon.searchengin::before {
  content: "\f3eb";
  font-family: "brand-icons";
}

i.icon.sellcast::before {
  content: "\f2da";
  font-family: "brand-icons";
}

i.icon.sellsy::before {
  content: "\f213";
  font-family: "brand-icons";
}

i.icon.servicestack::before {
  content: "\f3ec";
  font-family: "brand-icons";
}

i.icon.shirtsinbulk::before {
  content: "\f214";
  font-family: "brand-icons";
}

i.icon.shopify::before {
  content: "\e057";
  font-family: "brand-icons";
}

i.icon.shopware::before {
  content: "\f5b5";
  font-family: "brand-icons";
}

i.icon.simplybuilt::before {
  content: "\f215";
  font-family: "brand-icons";
}

i.icon.sistrix::before {
  content: "\f3ee";
  font-family: "brand-icons";
}

i.icon.sith::before {
  content: "\f512";
  font-family: "brand-icons";
}

i.icon.sketch::before {
  content: "\f7c6";
  font-family: "brand-icons";
}

i.icon.skyatlas::before {
  content: "\f216";
  font-family: "brand-icons";
}

i.icon.skype::before {
  content: "\f17e";
  font-family: "brand-icons";
}

i.icon.slack::before {
  content: "\f198";
  font-family: "brand-icons";
}

i.icon.slack.hash::before {
  content: "\f3ef";
  font-family: "brand-icons";
}

i.icon.slideshare::before {
  content: "\f1e7";
  font-family: "brand-icons";
}

i.icon.snapchat::before {
  content: "\f2ab";
  font-family: "brand-icons";
}

i.icon.snapchat.ghost::before {
  content: "\f2ac";
  font-family: "brand-icons";
}

i.icon.snapchat.square::before {
  content: "\f2ad";
  font-family: "brand-icons";
}

i.icon.soundcloud::before {
  content: "\f1be";
  font-family: "brand-icons";
}

i.icon.sourcetree::before {
  content: "\f7d3";
  font-family: "brand-icons";
}

i.icon.speakap::before {
  content: "\f3f3";
  font-family: "brand-icons";
}

i.icon.speaker.deck::before {
  content: "\f83c";
  font-family: "brand-icons";
}

i.icon.spotify::before {
  content: "\f1bc";
  font-family: "brand-icons";
}

i.icon.squarespace::before {
  content: "\f5be";
  font-family: "brand-icons";
}

i.icon.stack.exchange::before {
  content: "\f18d";
  font-family: "brand-icons";
}

i.icon.stack.overflow::before {
  content: "\f16c";
  font-family: "brand-icons";
}

i.icon.stackpath::before {
  content: "\f842";
  font-family: "brand-icons";
}

i.icon.staylinked::before {
  content: "\f3f5";
  font-family: "brand-icons";
}

i.icon.steam::before {
  content: "\f1b6";
  font-family: "brand-icons";
}

i.icon.steam.square::before {
  content: "\f1b7";
  font-family: "brand-icons";
}

i.icon.steam.symbol::before {
  content: "\f3f6";
  font-family: "brand-icons";
}

i.icon.sticker.mule::before {
  content: "\f3f7";
  font-family: "brand-icons";
}

i.icon.strava::before {
  content: "\f428";
  font-family: "brand-icons";
}

i.icon.stripe::before {
  content: "\f429";
  font-family: "brand-icons";
}

i.icon.stripe.s::before {
  content: "\f42a";
  font-family: "brand-icons";
}

i.icon.studiovinari::before {
  content: "\f3f8";
  font-family: "brand-icons";
}

i.icon.stumbleupon::before {
  content: "\f1a4";
  font-family: "brand-icons";
}

i.icon.stumbleupon.circle::before {
  content: "\f1a3";
  font-family: "brand-icons";
}

i.icon.superpowers::before {
  content: "\f2dd";
  font-family: "brand-icons";
}

i.icon.supple::before {
  content: "\f3f9";
  font-family: "brand-icons";
}

i.icon.suse::before {
  content: "\f7d6";
  font-family: "brand-icons";
}

i.icon.swift::before {
  content: "\f8e1";
  font-family: "brand-icons";
}

i.icon.symfony::before {
  content: "\f83d";
  font-family: "brand-icons";
}

i.icon.teamspeak::before {
  content: "\f4f9";
  font-family: "brand-icons";
}

i.icon.telegram::before {
  content: "\f2c6";
  font-family: "brand-icons";
}

i.icon.telegram.plane::before {
  content: "\f3fe";
  font-family: "brand-icons";
}

i.icon.tencent.weibo::before {
  content: "\f1d5";
  font-family: "brand-icons";
}

i.icon.themeco::before {
  content: "\f5c6";
  font-family: "brand-icons";
}

i.icon.themeisle::before {
  content: "\f2b2";
  font-family: "brand-icons";
}

i.icon.think.peaks::before {
  content: "\f731";
  font-family: "brand-icons";
}

i.icon.tiktok::before {
  content: "\e07b";
  font-family: "brand-icons";
}

i.icon.trade.federation::before {
  content: "\f513";
  font-family: "brand-icons";
}

i.icon.trello::before {
  content: "\f181";
  font-family: "brand-icons";
}

i.icon.tumblr::before {
  content: "\f173";
  font-family: "brand-icons";
}

i.icon.tumblr.square::before {
  content: "\f174";
  font-family: "brand-icons";
}

i.icon.twitch::before {
  content: "\f1e8";
  font-family: "brand-icons";
}

i.icon.twitter::before {
  content: "\f099";
  font-family: "brand-icons";
}

i.icon.twitter.square::before {
  content: "\f081";
  font-family: "brand-icons";
}

i.icon.typo3::before {
  content: "\f42b";
  font-family: "brand-icons";
}

i.icon.uber::before {
  content: "\f402";
  font-family: "brand-icons";
}

i.icon.ubuntu::before {
  content: "\f7df";
  font-family: "brand-icons";
}

i.icon.uikit::before {
  content: "\f403";
  font-family: "brand-icons";
}

i.icon.umbraco::before {
  content: "\f8e8";
  font-family: "brand-icons";
}

i.icon.uncharted::before {
  content: "\e084";
  font-family: "brand-icons";
}

i.icon.uniregistry::before {
  content: "\f404";
  font-family: "brand-icons";
}

i.icon.unity::before {
  content: "\e049";
  font-family: "brand-icons";
}

i.icon.unsplash::before {
  content: "\e07c";
  font-family: "brand-icons";
}

i.icon.untappd::before {
  content: "\f405";
  font-family: "brand-icons";
}

i.icon.ups::before {
  content: "\f7e0";
  font-family: "brand-icons";
}

i.icon.usb::before {
  content: "\f287";
  font-family: "brand-icons";
}

i.icon.usps::before {
  content: "\f7e1";
  font-family: "brand-icons";
}

i.icon.ussunnah::before {
  content: "\f407";
  font-family: "brand-icons";
}

i.icon.vaadin::before {
  content: "\f408";
  font-family: "brand-icons";
}

i.icon.viacoin::before {
  content: "\f237";
  font-family: "brand-icons";
}

i.icon.viadeo::before {
  content: "\f2a9";
  font-family: "brand-icons";
}

i.icon.viadeo.square::before {
  content: "\f2aa";
  font-family: "brand-icons";
}

i.icon.viber::before {
  content: "\f409";
  font-family: "brand-icons";
}

i.icon.vimeo::before {
  content: "\f40a";
  font-family: "brand-icons";
}

i.icon.vimeo.square::before {
  content: "\f194";
  font-family: "brand-icons";
}

i.icon.vimeo.v::before {
  content: "\f27d";
  font-family: "brand-icons";
}

i.icon.vine::before {
  content: "\f1ca";
  font-family: "brand-icons";
}

i.icon.vk::before {
  content: "\f189";
  font-family: "brand-icons";
}

i.icon.vnv::before {
  content: "\f40b";
  font-family: "brand-icons";
}

i.icon.vuejs::before {
  content: "\f41f";
  font-family: "brand-icons";
}

i.icon.watchman.monitoring::before {
  content: "\e087";
  font-family: "brand-icons";
}

i.icon.waze::before {
  content: "\f83f";
  font-family: "brand-icons";
}

i.icon.weebly::before {
  content: "\f5cc";
  font-family: "brand-icons";
}

i.icon.weibo::before {
  content: "\f18a";
  font-family: "brand-icons";
}

i.icon.weixin::before {
  content: "\f1d7";
  font-family: "brand-icons";
}

i.icon.whatsapp::before {
  content: "\f232";
  font-family: "brand-icons";
}

i.icon.whatsapp.square::before {
  content: "\f40c";
  font-family: "brand-icons";
}

i.icon.whmcs::before {
  content: "\f40d";
  font-family: "brand-icons";
}

i.icon.wikipedia.w::before {
  content: "\f266";
  font-family: "brand-icons";
}

i.icon.windows::before {
  content: "\f17a";
  font-family: "brand-icons";
}

i.icon.wix::before {
  content: "\f5cf";
  font-family: "brand-icons";
}

i.icon.wizards.of.the.coast::before {
  content: "\f730";
  font-family: "brand-icons";
}

i.icon.wodu::before {
  content: "\e088";
  font-family: "brand-icons";
}

i.icon.wolf.pack.battalion::before {
  content: "\f514";
  font-family: "brand-icons";
}

i.icon.wordpress::before {
  content: "\f19a";
  font-family: "brand-icons";
}

i.icon.wordpress.simple::before {
  content: "\f411";
  font-family: "brand-icons";
}

i.icon.wpbeginner::before {
  content: "\f297";
  font-family: "brand-icons";
}

i.icon.wpexplorer::before {
  content: "\f2de";
  font-family: "brand-icons";
}

i.icon.wpforms::before {
  content: "\f298";
  font-family: "brand-icons";
}

i.icon.wpressr::before {
  content: "\f3e4";
  font-family: "brand-icons";
}

i.icon.xbox::before {
  content: "\f412";
  font-family: "brand-icons";
}

i.icon.xing::before {
  content: "\f168";
  font-family: "brand-icons";
}

i.icon.xing.square::before {
  content: "\f169";
  font-family: "brand-icons";
}

i.icon.y.combinator::before {
  content: "\f23b";
  font-family: "brand-icons";
}

i.icon.yahoo::before {
  content: "\f19e";
  font-family: "brand-icons";
}

i.icon.yammer::before {
  content: "\f840";
  font-family: "brand-icons";
}

i.icon.yandex::before {
  content: "\f413";
  font-family: "brand-icons";
}

i.icon.yandex.international::before {
  content: "\f414";
  font-family: "brand-icons";
}

i.icon.yarn::before {
  content: "\f7e3";
  font-family: "brand-icons";
}

i.icon.yelp::before {
  content: "\f1e9";
  font-family: "brand-icons";
}

i.icon.yoast::before {
  content: "\f2b1";
  font-family: "brand-icons";
}

i.icon.youtube::before {
  content: "\f167";
  font-family: "brand-icons";
}

i.icon.youtube.square::before {
  content: "\f431";
  font-family: "brand-icons";
}

i.icon.zhihu::before {
  content: "\f63f";
  font-family: "brand-icons";
}

/* Aliases */

i.icon.american.express::before {
  content: "\f1f3";
  font-family: "brand-icons";
}

i.icon.american.express.card::before {
  content: "\f1f3";
  font-family: "brand-icons";
}

i.icon.amex::before {
  content: "\f1f3";
  font-family: "brand-icons";
}

i.icon.bitbucket.square::before {
  content: "\f171";
  font-family: "brand-icons";
}

i.icon.bluetooth.alternative::before {
  content: "\f294";
  font-family: "brand-icons";
}

i.icon.credit.card.amazon.pay::before {
  content: "\f42d";
  font-family: "brand-icons";
}

i.icon.credit.card.american.express::before {
  content: "\f1f3";
  font-family: "brand-icons";
}

i.icon.credit.card.diners.club::before {
  content: "\f24c";
  font-family: "brand-icons";
}

i.icon.credit.card.discover::before {
  content: "\f1f2";
  font-family: "brand-icons";
}

i.icon.credit.card.jcb::before {
  content: "\f24b";
  font-family: "brand-icons";
}

i.icon.credit.card.mastercard::before {
  content: "\f1f1";
  font-family: "brand-icons";
}

i.icon.credit.card.paypal::before {
  content: "\f1f4";
  font-family: "brand-icons";
}

i.icon.credit.card.stripe::before {
  content: "\f1f5";
  font-family: "brand-icons";
}

i.icon.credit.card.visa::before {
  content: "\f1f0";
  font-family: "brand-icons";
}

i.icon.diners.club::before {
  content: "\f24c";
  font-family: "brand-icons";
}

i.icon.diners.club.card::before {
  content: "\f24c";
  font-family: "brand-icons";
}

i.icon.discover::before {
  content: "\f1f2";
  font-family: "brand-icons";
}

i.icon.discover.card::before {
  content: "\f1f2";
  font-family: "brand-icons";
}

i.icon.disk.outline::before {
  content: "\f369";
  font-family: "brand-icons";
}

i.icon.dribble::before {
  content: "\f17d";
  font-family: "brand-icons";
}

i.icon.eercast::before {
  content: "\f2da";
  font-family: "brand-icons";
}

i.icon.envira.gallery::before {
  content: "\f299";
  font-family: "brand-icons";
}

i.icon.fa::before {
  content: "\f2b4";
  font-family: "brand-icons";
}

i.icon.facebook.official::before {
  content: "\f082";
  font-family: "brand-icons";
}

i.icon.five.hundred.pixels::before {
  content: "\f26e";
  font-family: "brand-icons";
}

i.icon.gittip::before {
  content: "\f184";
  font-family: "brand-icons";
}

i.icon.google.plus.circle::before {
  content: "\f2b3";
  font-family: "brand-icons";
}

i.icon.google.plus.official::before {
  content: "\f2b3";
  font-family: "brand-icons";
}

i.icon.japan.credit.bureau::before {
  content: "\f24b";
  font-family: "brand-icons";
}

i.icon.japan.credit.bureau.card::before {
  content: "\f24b";
  font-family: "brand-icons";
}

i.icon.jcb::before {
  content: "\f24b";
  font-family: "brand-icons";
}

i.icon.linkedin.square::before {
  content: "\f08c";
  font-family: "brand-icons";
}

i.icon.mastercard::before {
  content: "\f1f1";
  font-family: "brand-icons";
}

i.icon.mastercard.card::before {
  content: "\f1f1";
  font-family: "brand-icons";
}

i.icon.microsoft.edge::before {
  content: "\f282";
  font-family: "brand-icons";
}

i.icon.ms.edge::before {
  content: "\f282";
  font-family: "brand-icons";
}

i.icon.new.pied.piper::before {
  content: "\f2ae";
  font-family: "brand-icons";
}

i.icon.optinmonster::before {
  content: "\f23c";
  font-family: "brand-icons";
}

i.icon.paypal.card::before {
  content: "\f1f4";
  font-family: "brand-icons";
}

i.icon.pied.piper.hat::before {
  content: "\f2ae";
  font-family: "brand-icons";
}

i.icon.pocket::before {
  content: "\f265";
  font-family: "brand-icons";
}

i.icon.stripe.card::before {
  content: "\f1f5";
  font-family: "brand-icons";
}

i.icon.theme.isle::before {
  content: "\f2b2";
  font-family: "brand-icons";
}

i.icon.visa::before {
  content: "\f1f0";
  font-family: "brand-icons";
}

i.icon.visa.card::before {
  content: "\f1f0";
  font-family: "brand-icons";
}

i.icon.wechat::before {
  content: "\f1d7";
  font-family: "brand-icons";
}

i.icon.wikipedia::before {
  content: "\f266";
  font-family: "brand-icons";
}

i.icon.wordpress.beginner::before {
  content: "\f297";
  font-family: "brand-icons";
}

i.icon.wordpress.forms::before {
  content: "\f298";
  font-family: "brand-icons";
}

i.icon.yc::before {
  content: "\f23b";
  font-family: "brand-icons";
}

i.icon.ycombinator::before {
  content: "\f23b";
  font-family: "brand-icons";
}

i.icon.youtube.play::before {
  content: "\f167";
  font-family: "brand-icons";
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Image
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Image
*******************************/

.ui.image {
  position: relative;
  display: inline-block;
  vertical-align: middle;
  max-width: 100%;
  background-color: transparent;
}

img.ui.image {
  display: block;
}

.ui.image svg,
.ui.image img {
  display: block;
  max-width: 100%;
  height: auto;
}

/*******************************
            States
*******************************/

.ui.hidden.images,
.ui.ui.hidden.image {
  display: none;
}

.ui.hidden.transition.images,
.ui.hidden.transition.image {
  display: block;
  visibility: hidden;
}

.ui.images > .hidden.transition {
  display: inline-block;
  visibility: hidden;
}

.ui.disabled.images,
.ui.disabled.image {
  cursor: default;
  opacity: 0.45;
}

/*******************************
          Variations
*******************************/

/* --------------
         Inline
    --------------- */

.ui.inline.image,
.ui.inline.image svg,
.ui.inline.image img {
  display: inline-block;
}

/* ------------------
      Vertical Aligned
    ------------------- */

.ui.top.aligned.image,
.ui.top.aligned.image svg,
.ui.top.aligned.image img {
  display: inline-block;
  vertical-align: top;
}

.ui.middle.aligned.image,
.ui.middle.aligned.image svg,
.ui.middle.aligned.image img {
  display: inline-block;
  vertical-align: middle;
}

.ui.bottom.aligned.image,
.ui.bottom.aligned.image svg,
.ui.bottom.aligned.image img {
  display: inline-block;
  vertical-align: bottom;
}

.ui.top.aligned.images .image,
.ui.images .ui.top.aligned.image {
  align-self: flex-start;
}

.ui.middle.aligned.images .image,
.ui.images .ui.middle.aligned.image {
  align-self: center;
}

.ui.bottom.aligned.images .image,
.ui.images .ui.bottom.aligned.image {
  align-self: flex-end;
}

/* --------------
         Rounded
    --------------- */

.ui.rounded.images .image,
.ui.rounded.image,
.ui.rounded.images .image > *,
.ui.rounded.image > * {
  border-radius: 0.3125em;
}

/* --------------
        Bordered
    --------------- */

.ui.bordered.images .image,
.ui.bordered.images img,
.ui.bordered.images svg,
.ui.bordered.image img,
.ui.bordered.image svg,
img.ui.bordered.image {
  border: 1px solid rgba(0, 0, 0, 0.1);
}

/* --------------
        Circular
    --------------- */

.ui.circular.images,
.ui.circular.image {
  overflow: hidden;
}

.ui.circular.images .image,
.ui.circular.image,
.ui.circular.images .image > *,
.ui.circular.image > * {
  border-radius: 500rem;
}

/* --------------
         Fluid
    --------------- */

.ui.fluid.images,
.ui.fluid.image,
.ui.fluid.images img,
.ui.fluid.images svg,
.ui.fluid.image svg,
.ui.fluid.image img {
  display: block;
  width: 100%;
  height: auto;
}

/* --------------
         Avatar
    --------------- */

.ui.avatar.images .image,
.ui.avatar.images img,
.ui.avatar.images svg,
.ui.avatar.image img,
.ui.avatar.image svg,
.ui.avatar.image {
  margin-right: 0.25em;
  display: inline-block;
  width: 2em;
  height: 2em;
  border-radius: 500rem;
}

/* -------------------
           Spaced
    -------------------- */

.ui.spaced.image {
  display: inline-block !important;
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.ui[class*="left spaced"].image {
  margin-left: 0.5em;
  margin-right: 0;
}

.ui[class*="right spaced"].image {
  margin-left: 0;
  margin-right: 0.5em;
}

/* -------------------
           Floated
    -------------------- */

.ui.floated.image,
.ui.floated.images {
  float: left;
  margin-right: 1em;
  margin-bottom: 1em;
}

.ui.right.floated.images,
.ui.right.floated.image {
  float: right;
  margin-right: 0;
  margin-bottom: 1em;
  margin-left: 1em;
}

.ui.floated.images:last-child,
.ui.floated.image:last-child {
  margin-bottom: 0;
}

.ui.centered.image {
  display: block;
  margin-left: auto;
  margin-right: auto;
}

.ui.centered.images {
  display: flex;
  flex-flow: row wrap;
  align-items: stretch;
  justify-content: center;
}

/* --------------
     Sizes
--------------- */

.ui.medium.images .image,
.ui.medium.images img,
.ui.medium.images svg,
.ui.medium.image {
  width: 300px;
  height: auto;
  font-size: 1rem;
}

.ui.mini.images .image,
.ui.mini.images img,
.ui.mini.images svg,
.ui.mini.image {
  width: 35px;
  height: auto;
  font-size: 0.78571429rem;
}

.ui.tiny.images .image,
.ui.tiny.images img,
.ui.tiny.images svg,
.ui.tiny.image {
  width: 80px;
  height: auto;
  font-size: 0.85714286rem;
}

.ui.small.images .image,
.ui.small.images img,
.ui.small.images svg,
.ui.small.image {
  width: 150px;
  height: auto;
  font-size: 0.92857143rem;
}

.ui.large.images .image,
.ui.large.images img,
.ui.large.images svg,
.ui.large.image {
  width: 450px;
  height: auto;
  font-size: 1.14285714rem;
}

.ui.big.images .image,
.ui.big.images img,
.ui.big.images svg,
.ui.big.image {
  width: 600px;
  height: auto;
  font-size: 1.28571429rem;
}

.ui.huge.images .image,
.ui.huge.images img,
.ui.huge.images svg,
.ui.huge.image {
  width: 800px;
  height: auto;
  font-size: 1.42857143rem;
}

.ui.massive.images .image,
.ui.massive.images img,
.ui.massive.images svg,
.ui.massive.image {
  width: 960px;
  height: auto;
  font-size: 1.71428571rem;
}

/*******************************
                  Groups
    *******************************/

.ui.images {
  font-size: 0;
  margin: 0 -0.25rem;
}

.ui.images .image,
.ui.images > img,
.ui.images > svg {
  display: inline-block;
  margin: 0 0.25rem 0.5rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Input
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           Standard
*******************************/

/* --------------------
        Inputs
--------------------- */

.ui.input {
  position: relative;
  font-weight: normal;
  font-style: normal;
  display: inline-flex;
  color: rgba(0, 0, 0, 0.87);
}

.ui.input > input {
  margin: 0;
  max-width: 100%;
  flex: 1 0 auto;
  outline: none;
  -webkit-tap-highlight-color: rgba(255, 255, 255, 0);
  text-align: left;
  line-height: 1.21428571em;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0.28571429rem;
  transition: box-shadow 0.1s ease, border-color 0.1s ease;
  box-shadow: none;
}

.ui.input > input:not([type="color"]) {
  padding: 0.67857143em 1em;
}

.ui.input > input::-webkit-calendar-picker-indicator {
  padding: 0;
  opacity: 0.5;
  -webkit-transition: opacity 0.3s ease;
  transition: opacity 0.3s ease;
  cursor: pointer;
}

/* --------------------
      Placeholder
--------------------- */

/* browsers require these rules separate */

.ui.input > input:-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.input > input::-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.input > input::placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.input > input::-moz-placeholder {
  opacity: 1;
}

.ui.input > input:-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87);
}

/*******************************
            States
*******************************/

/* --------------------
            Disabled
    --------------------- */

.ui.disabled.input,
.ui.input:not(.disabled) input[disabled] {
  opacity: 0.45;
}

.ui.disabled.input > input,
.ui.input:not(.disabled) input[disabled] {
  pointer-events: none;
}

/* --------------------
        Active
--------------------- */

.ui.input > input:active,
.ui.input.down input {
  border-color: rgba(0, 0, 0, 0.3);
  background: #fafafa;
  color: rgba(0, 0, 0, 0.87);
  box-shadow: none;
}

/* --------------------
           Loading
    --------------------- */

.ui.ui.loading.input > i.icon::before {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.1);
}

.ui.ui.loading.input > i.icon::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
}

/* --------------------
        Focus
--------------------- */

.ui.input.focus > input,
.ui.input > input:focus {
  border-color: #85b7d9;
  background: #fff;
  color: rgba(0, 0, 0, 0.8);
  box-shadow: none;
}

.ui.input.focus > input:-ms-input-placeholder,
.ui.input > input:focus:-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.input.focus > input::-ms-input-placeholder,
.ui.input > input:focus::-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.input.focus > input::placeholder,
.ui.input > input:focus::placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.input.focus > input:-ms-input-placeholder,
.ui.input > input:focus:-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87);
}

/* --------------------
            States
    --------------------- */

.ui.input.error > input {
  background-color: #fff6f6;
  border-color: #e0b4b4;
  color: #9f3a38;
  box-shadow: none;
}

/* Placeholder */

.ui.input.error > input:-ms-input-placeholder {
  color: #e7bdbc;
}

.ui.input.error > input::-ms-input-placeholder {
  color: #e7bdbc;
}

.ui.input.error > input::placeholder {
  color: #e7bdbc;
}

.ui.input.error > input:-ms-input-placeholder {
  color: #e7bdbc !important;
}

/* Focused Placeholder */

.ui.input.error > input:focus:-ms-input-placeholder {
  color: #da9796;
}

.ui.input.error > input:focus::-ms-input-placeholder {
  color: #da9796;
}

.ui.input.error > input:focus::placeholder {
  color: #da9796;
}

.ui.input.error > input:focus:-ms-input-placeholder {
  color: #da9796 !important;
}

.ui.input.info > input {
  background-color: #f8ffff;
  border-color: #a9d5de;
  color: #276f86;
  box-shadow: none;
}

/* Placeholder */

.ui.input.info > input:-ms-input-placeholder {
  color: #98cfe1;
}

.ui.input.info > input::-ms-input-placeholder {
  color: #98cfe1;
}

.ui.input.info > input::placeholder {
  color: #98cfe1;
}

.ui.input.info > input:-ms-input-placeholder {
  color: #98cfe1 !important;
}

/* Focused Placeholder */

.ui.input.info > input:focus:-ms-input-placeholder {
  color: #70bdd6;
}

.ui.input.info > input:focus::-ms-input-placeholder {
  color: #70bdd6;
}

.ui.input.info > input:focus::placeholder {
  color: #70bdd6;
}

.ui.input.info > input:focus:-ms-input-placeholder {
  color: #70bdd6 !important;
}

.ui.input.success > input {
  background-color: #fcfff5;
  border-color: #a3c293;
  color: #2c662d;
  box-shadow: none;
}

/* Placeholder */

.ui.input.success > input:-ms-input-placeholder {
  color: #8fcf90;
}

.ui.input.success > input::-ms-input-placeholder {
  color: #8fcf90;
}

.ui.input.success > input::placeholder {
  color: #8fcf90;
}

.ui.input.success > input:-ms-input-placeholder {
  color: #8fcf90 !important;
}

/* Focused Placeholder */

.ui.input.success > input:focus:-ms-input-placeholder {
  color: #6cbf6d;
}

.ui.input.success > input:focus::-ms-input-placeholder {
  color: #6cbf6d;
}

.ui.input.success > input:focus::placeholder {
  color: #6cbf6d;
}

.ui.input.success > input:focus:-ms-input-placeholder {
  color: #6cbf6d !important;
}

.ui.input.warning > input {
  background-color: #fffaf3;
  border-color: #c9ba9b;
  color: #573a08;
  box-shadow: none;
}

/* Placeholder */

.ui.input.warning > input:-ms-input-placeholder {
  color: #edad3e;
}

.ui.input.warning > input::-ms-input-placeholder {
  color: #edad3e;
}

.ui.input.warning > input::placeholder {
  color: #edad3e;
}

.ui.input.warning > input:-ms-input-placeholder {
  color: #edad3e !important;
}

/* Focused Placeholder */

.ui.input.warning > input:focus:-ms-input-placeholder {
  color: #e39715;
}

.ui.input.warning > input:focus::-ms-input-placeholder {
  color: #e39715;
}

.ui.input.warning > input:focus::placeholder {
  color: #e39715;
}

.ui.input.warning > input:focus:-ms-input-placeholder {
  color: #e39715 !important;
}

/*******************************
           Variations
*******************************/

/* --------------------
          Transparent
    --------------------- */

.ui.transparent.input > textarea,
.ui.transparent.input > input {
  border-color: transparent !important;
  background-color: transparent !important;
  padding: 0;
  box-shadow: none !important;
  border-radius: 0 !important;
}

.field .ui.transparent.input > textarea {
  padding: 0.67857143em 1em;
}

/* Transparent Icon */

:not(.field) > .ui.transparent.icon.input > i.icon {
  width: 1.1em;
}

:not(.field) > .ui.ui.ui.transparent.icon.input > input {
  padding-left: 0;
  padding-right: 2em;
}

:not(.field) > .ui.ui.ui.transparent[class*="left icon"].input > input {
  padding-left: 2em;
  padding-right: 0;
}

/* Transparent Inverted */

.ui.transparent.inverted.input {
  color: #fff;
}

.ui.ui.transparent.inverted.input > textarea,
.ui.ui.transparent.inverted.input > input {
  color: inherit;
}

.ui.transparent.inverted.input > input:-ms-input-placeholder {
  color: rgba(255, 255, 255, 0.5);
}

.ui.transparent.inverted.input > input::-ms-input-placeholder {
  color: rgba(255, 255, 255, 0.5);
}

.ui.transparent.inverted.input > input::placeholder {
  color: rgba(255, 255, 255, 0.5);
}

.ui.transparent.inverted.input > input:-ms-input-placeholder {
  color: rgba(255, 255, 255, 0.5);
}

/* --------------------
             Icon
    --------------------- */

.ui.icon.input > i.icon {
  cursor: default;
  position: absolute;
  line-height: 1;
  text-align: center;
  top: 0;
  right: 0;
  margin: 0;
  height: 100%;
  width: 2.67142857em;
  opacity: 0.5;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
  transition: opacity 0.3s ease;
}

.ui.icon.input > i.icon:not(.link) {
  pointer-events: none;
}

.ui.ui.ui.ui.icon.input:not(.corner) > textarea,
.ui.ui.ui.ui.icon.input:not(.corner) > input {
  padding-right: 2.67142857em;
}

.ui.icon.input > i.icon::before,
.ui.icon.input > i.icon::after {
  left: 0;
  position: absolute;
  text-align: center;
  top: 50%;
  width: 100%;
  margin-top: -0.5em;
}

.ui.icon.input > i.link.icon {
  cursor: pointer;
}

.ui.icon.input > i.circular.icon {
  top: 0.35em;
  right: 0.5em;
}

/* Left Icon Input */

.ui[class*="left icon"].input > i.icon {
  right: auto;
  left: 1px;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui[class*="left icon"].input > i.circular.icon {
  right: auto;
  left: 0.5em;
}

.ui.ui.ui.ui[class*="left icon"]:not([class*="left corner"]).input > textarea,
.ui.ui.ui.ui[class*="left icon"]:not([class*="left corner"]).input > input {
  padding-left: 2.67142857em;
}

.ui.ui.ui.ui[class*="left icon"]:not(.corner).input > textarea,
.ui.ui.ui.ui[class*="left icon"]:not(.corner).input > input {
  padding-right: 1em;
}

/* Focus */

.ui.input > input:focus::-webkit-calendar-picker-indicator,
.ui.icon.input > textarea:focus ~ i.icon,
.ui.icon.input > input:focus ~ i.icon {
  opacity: 1;
}

/* --------------------
            Labeled
    --------------------- */

/* Adjacent Label */

.ui.labeled.input > .label {
  flex: 0 0 auto;
  margin: 0;
  font-size: 1em;
}

.ui.labeled.input > .label:not(.corner) {
  padding-top: 0.78571429em;
  padding-bottom: 0.78571429em;
}

/* Regular Label on Left */

.ui.labeled.input:not([class*="corner labeled"]) .label:first-child {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.labeled.input:not([class*="corner labeled"]) .label:first-child + input {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-left-color: transparent;
}

.ui.labeled.input:not([class*="corner labeled"]) .label:first-child + input:focus {
  border-left-color: #85b7d9;
}

/* Regular Label on Right */

.ui[class*="right labeled"].input > input {
  border-top-right-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
  border-right-color: transparent !important;
}

.ui[class*="right labeled"].input > input + .label {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui[class*="right labeled"].input > input:focus {
  border-right-color: #85b7d9 !important;
}

/* Corner Label */

.ui.labeled.input .corner.label {
  top: 1px;
  right: 1px;
  font-size: 0.64285714em;
  border-radius: 0 0.28571429rem 0 0;
}

/* Spacing with corner label */

.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > .ui.dropdown,
.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > textarea,
.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > input {
  padding-right: 2.5em;
}

.ui[class*="corner labeled"].icon.input:not([class*="left corner labeled"]) > .ui.dropdown,
.ui[class*="corner labeled"].icon.input:not([class*="left corner labeled"]) > textarea,
.ui[class*="corner labeled"].icon.input:not([class*="left corner labeled"]) > input {
  padding-right: 3.25em;
}

.ui[class*="corner labeled"].icon.input:not([class*="left corner labeled"]) > i.icon {
  margin-right: 1.25em;
}

/* Left Labeled */

.ui[class*="left icon"].input > .ui.dropdown:first-child,
.ui[class*="left icon"].input > i.icon + .ui.dropdown,
.ui[class*="left corner labeled"].input > .ui.dropdown,
.ui[class*="left corner labeled"].input > textarea,
.ui[class*="left corner labeled"].input > input {
  padding-left: 2.5em;
}

.ui[class*="corner labeled"]:not([class*="left corner labeled"])[class*="left icon"].input > .ui.dropdown,
.ui[class*="corner labeled"]:not([class*="left corner labeled"])[class*="left icon"].input > textarea,
.ui[class*="corner labeled"]:not([class*="left corner labeled"])[class*="left icon"].input > input {
  padding-right: 3.25em;
}

.ui.ui[class*="left corner labeled"][class*="left icon"].input > .ui.dropdown,
.ui.ui[class*="left corner labeled"][class*="left icon"].input > textarea,
.ui.ui[class*="left corner labeled"][class*="left icon"].input > input {
  padding-left: 4em;
}

.ui[class*="left corner labeled"].icon.input > i.icon {
  margin-left: 1.25em;
}

.ui[class*="left corner labeled"].icon:not([class*="left icon"]).input > input {
  padding-right: 2.5em;
}

.ui[class*="corner labeled"]:not([class*="left corner labeled"]).icon:not([class*="left icon"]).input > input {
  padding-right: 5em;
}

.ui[class*="left icon"].input > .ui.dropdown > .search,
.ui[class*="left corner labeled"].input > .ui.dropdown > .search {
  padding-left: 2.5em;
}

.ui[class*="left icon"].input > .ui.dropdown > .menu,
.ui[class*="left corner labeled"].input > .ui.dropdown > .menu {
  padding-left: 1.25em;
}

.ui[class*="left icon"].input > .ui.dropdown > .menu > .item,
.ui[class*="left corner labeled"].input > .ui.dropdown > .menu > .item {
  padding-left: 2.5em;
  margin-left: -1.25em;
}

.ui[class*="left corner labeled"][class*="left icon"].input > .ui.dropdown > .search {
  padding-left: 4em;
}

.ui[class*="left corner labeled"][class*="left icon"].input > .ui.dropdown > .menu > .item {
  padding-left: 4em;
}

.ui.icon.input:not([class*="left icon"]) > .ui.dropdown > .search,
.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > .ui.dropdown > .search {
  padding-right: 5.75em;
}

.ui.icon.input:not([class*="left icon"]) > .ui.dropdown > .remove.icon,
.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > .ui.dropdown > .remove.icon,
.ui.icon.input:not([class*="left icon"]) > .ui.dropdown > .dropdown.icon,
.ui[class*="corner labeled"]:not([class*="left corner labeled"]).input > .ui.dropdown > .dropdown.icon {
  padding-right: 2.5em;
}

@supports selector(:has(.f)) {
  .ui.icon.input:not([class*="left icon"]) > .ui.dropdown > .dropdown.icon {
    padding-right: initial;
  }

  .ui.icon.input:not([class*="left icon"]):not(:has(.ui.dropdown ~ input)) > .ui.dropdown > .dropdown.icon {
    padding-right: 2.5em;
  }
}

.ui.ui[class*="corner labeled"]:not([class*="left corner labeled"]).icon:not([class*="left icon"]).input > .ui.dropdown > .search {
  padding-right: 7.25em;
}

.ui.ui[class*="corner labeled"]:not([class*="left corner labeled"]).icon:not([class*="left icon"]).input > .ui.dropdown > .remove.icon,
.ui.ui[class*="corner labeled"]:not([class*="left corner labeled"]).icon:not([class*="left icon"]).input > .ui.dropdown > .dropdown.icon {
  padding-right: 4em;
}

.ui.icon.input > .ui.visible.dropdown ~ i.icon,
.ui.icon.input > .ui.active.dropdown ~ i.icon,
.ui[class*="corner labeled"].input > .ui.visible.dropdown ~ .ui.corner.label,
.ui[class*="corner labeled"].input > .ui.active.dropdown ~ .ui.corner.label {
  z-index: 10;
}

.ui.icon.input > textarea ~ i.icon {
  height: 3em;
}

:not(.field) > .ui.transparent.icon.input > textarea ~ i.icon {
  height: 1.3em;
}

/* Corner Label Position */

.ui.input > .ui.corner.label {
  top: 1px;
  right: 1px;
}

.ui.input > .ui.left.corner.label {
  right: auto;
  left: 1px;
}

/* Labeled and action input states */

.ui.form .field.error > .ui.action.input > .ui.button,
.ui.form .field.error > .ui.labeled.input:not([class*="corner labeled"]) > .ui.label,
.ui.action.input.error > .ui.button,
.ui.labeled.input.error:not([class*="corner labeled"]) > .ui.label {
  border-top: 1px solid #e0b4b4;
  border-bottom: 1px solid #e0b4b4;
}

.ui.form .field.error > .ui[class*="left action"].input > .ui.button,
.ui.form .field.error > .ui.labeled.input:not(.right):not([class*="corner labeled"]) > .ui.label,
.ui[class*="left action"].input.error > .ui.button,
.ui.labeled.input.error:not(.right):not([class*="corner labeled"]) > .ui.label {
  border-left: 1px solid #e0b4b4;
}

.ui.form .field.error > .ui.action.input:not([class*="left action"]) > input + .ui.button,
.ui.form .field.error > .ui.right.labeled.input:not([class*="corner labeled"]) > input + .ui.label,
.ui.action.input.error:not([class*="left action"]) > input + .ui.button,
.ui.right.labeled.input.error:not([class*="corner labeled"]) > input + .ui.label {
  border-right: 1px solid #e0b4b4;
}

.ui.form .field.error > .ui.right.labeled.input:not([class*="corner labeled"]) > .ui.label:first-child,
.ui.right.labeled.input.error:not([class*="corner labeled"]) > .ui.label:first-child {
  border-left: 1px solid #e0b4b4;
}

.ui.form .field.info > .ui.action.input > .ui.button,
.ui.form .field.info > .ui.labeled.input:not([class*="corner labeled"]) > .ui.label,
.ui.action.input.info > .ui.button,
.ui.labeled.input.info:not([class*="corner labeled"]) > .ui.label {
  border-top: 1px solid #a9d5de;
  border-bottom: 1px solid #a9d5de;
}

.ui.form .field.info > .ui[class*="left action"].input > .ui.button,
.ui.form .field.info > .ui.labeled.input:not(.right):not([class*="corner labeled"]) > .ui.label,
.ui[class*="left action"].input.info > .ui.button,
.ui.labeled.input.info:not(.right):not([class*="corner labeled"]) > .ui.label {
  border-left: 1px solid #a9d5de;
}

.ui.form .field.info > .ui.action.input:not([class*="left action"]) > input + .ui.button,
.ui.form .field.info > .ui.right.labeled.input:not([class*="corner labeled"]) > input + .ui.label,
.ui.action.input.info:not([class*="left action"]) > input + .ui.button,
.ui.right.labeled.input.info:not([class*="corner labeled"]) > input + .ui.label {
  border-right: 1px solid #a9d5de;
}

.ui.form .field.info > .ui.right.labeled.input:not([class*="corner labeled"]) > .ui.label:first-child,
.ui.right.labeled.input.info:not([class*="corner labeled"]) > .ui.label:first-child {
  border-left: 1px solid #a9d5de;
}

.ui.form .field.success > .ui.action.input > .ui.button,
.ui.form .field.success > .ui.labeled.input:not([class*="corner labeled"]) > .ui.label,
.ui.action.input.success > .ui.button,
.ui.labeled.input.success:not([class*="corner labeled"]) > .ui.label {
  border-top: 1px solid #a3c293;
  border-bottom: 1px solid #a3c293;
}

.ui.form .field.success > .ui[class*="left action"].input > .ui.button,
.ui.form .field.success > .ui.labeled.input:not(.right):not([class*="corner labeled"]) > .ui.label,
.ui[class*="left action"].input.success > .ui.button,
.ui.labeled.input.success:not(.right):not([class*="corner labeled"]) > .ui.label {
  border-left: 1px solid #a3c293;
}

.ui.form .field.success > .ui.action.input:not([class*="left action"]) > input + .ui.button,
.ui.form .field.success > .ui.right.labeled.input:not([class*="corner labeled"]) > input + .ui.label,
.ui.action.input.success:not([class*="left action"]) > input + .ui.button,
.ui.right.labeled.input.success:not([class*="corner labeled"]) > input + .ui.label {
  border-right: 1px solid #a3c293;
}

.ui.form .field.success > .ui.right.labeled.input:not([class*="corner labeled"]) > .ui.label:first-child,
.ui.right.labeled.input.success:not([class*="corner labeled"]) > .ui.label:first-child {
  border-left: 1px solid #a3c293;
}

.ui.form .field.warning > .ui.action.input > .ui.button,
.ui.form .field.warning > .ui.labeled.input:not([class*="corner labeled"]) > .ui.label,
.ui.action.input.warning > .ui.button,
.ui.labeled.input.warning:not([class*="corner labeled"]) > .ui.label {
  border-top: 1px solid #c9ba9b;
  border-bottom: 1px solid #c9ba9b;
}

.ui.form .field.warning > .ui[class*="left action"].input > .ui.button,
.ui.form .field.warning > .ui.labeled.input:not(.right):not([class*="corner labeled"]) > .ui.label,
.ui[class*="left action"].input.warning > .ui.button,
.ui.labeled.input.warning:not(.right):not([class*="corner labeled"]) > .ui.label {
  border-left: 1px solid #c9ba9b;
}

.ui.form .field.warning > .ui.action.input:not([class*="left action"]) > input + .ui.button,
.ui.form .field.warning > .ui.right.labeled.input:not([class*="corner labeled"]) > input + .ui.label,
.ui.action.input.warning:not([class*="left action"]) > input + .ui.button,
.ui.right.labeled.input.warning:not([class*="corner labeled"]) > input + .ui.label {
  border-right: 1px solid #c9ba9b;
}

.ui.form .field.warning > .ui.right.labeled.input:not([class*="corner labeled"]) > .ui.label:first-child,
.ui.right.labeled.input.warning:not([class*="corner labeled"]) > .ui.label:first-child {
  border-left: 1px solid #c9ba9b;
}

/* --------------------
            Action
    --------------------- */

.ui.action.input > .button,
.ui.action.input > .buttons {
  display: flex;
  align-items: center;
  flex: 0 0 auto;
}

.ui.action.input > .button,
.ui.action.input > .buttons > .button {
  padding-top: 0.78571429em;
  padding-bottom: 0.78571429em;
  margin: 0;
}

/* Input when ui Left */

.ui[class*="left action"].input > input {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-left-color: transparent;
}

/* Input when ui Right */

.ui.action.input:not([class*="left action"]) > input {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
  border-right-color: transparent;
}

/* Button and Dropdown */

.ui.action.input > .dropdown:first-child,
.ui.action.input > .button:first-child,
.ui.action.input > .buttons:first-child > .button {
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui.action.input > .dropdown:not(:first-child),
.ui.action.input > .button:not(:first-child),
.ui.action.input > .buttons:not(:first-child) > .button {
  border-radius: 0;
}

.ui.action.input > .dropdown:last-child,
.ui.action.input > .button:last-child,
.ui.action.input > .buttons:last-child > .button {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

/* Input Focus */

.ui.action.input:not([class*="left action"]) > input:focus {
  border-right-color: #85b7d9;
}

.ui.ui[class*="left action"].input > input:focus {
  border-left-color: #85b7d9;
}

/* --------------------
           Inverted
    --------------------- */

/* Standard */

.ui.inverted.input > input {
  border: none;
}

/* --------------------
            Fluid
    --------------------- */

.ui.fluid.input {
  display: flex;
}

.ui.fluid.input > input {
  width: 0 !important;
}

/* --------------------
            File
    --------------------- */

/* width hack for chrome/edge */

.ui.file.input {
  width: 100%;
}

.ui.file.input input[type="file"] {
  width: 0;
}

.ui.form .field > input[type="file"],
.ui.file.input:not(.action) input[type="file"] {
  padding: 0;
}

.ui.action.file.input input[type="file"]::-webkit-file-upload-button {
  display: none;
}

.ui.form .field input[type="file"]::-webkit-file-upload-button,
.ui.file.input input[type="file"]::-webkit-file-upload-button {
  border: none;
  cursor: pointer;
  padding: 0.67857143em 1em;
  margin-right: 1em;
  background: #e0e1e2;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.6);
}

.ui.form .field input[type="file"]::-webkit-file-upload-button:hover,
.ui.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #cacbcd;
  color: rgba(0, 0, 0, 0.6);
}

.ui.action.file.input input[type="file"]::-ms-browse {
  display: none;
}

.ui.form .field input[type="file"]::-ms-browse,
.ui.file.input input[type="file"]::-ms-browse {
  border: none;
  cursor: pointer;
  padding: 0.67857143em 1em;
  margin: 0;
  background: #e0e1e2;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.6);
}

.ui.form .field input[type="file"]::-ms-browse:hover,
.ui.file.input input[type="file"]::-ms-browse:hover {
  background: #cacbcd;
  color: rgba(0, 0, 0, 0.6);
}

/* IE needs additional styling for input field :S */

@media all and (-ms-high-contrast: none) {
  .ui.file.input > input[type="file"],
  input[type="file"].ui.file.input {
    padding: 0 !important;
  }
}

.ui.action.file.input input[type="file"]::-webkit-file-upload-button {
  display: none;
}

.ui.action.file.input input[type="file"]::file-selector-button {
  display: none;
}

.ui.form .field input[type="file"]::-webkit-file-upload-button,
.ui.file.input input[type="file"]::-webkit-file-upload-button {
  border: none;
  cursor: pointer;
  padding: 0.67857143em 1em;
  margin-right: 1em;
  background: #e0e1e2;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.6);
}

.ui.form .field input[type="file"]::file-selector-button,
.ui.file.input input[type="file"]::file-selector-button {
  border: none;
  cursor: pointer;
  padding: 0.67857143em 1em;
  margin-right: 1em;
  background: #e0e1e2;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.6);
}

.ui.form .field input[type="file"]::-webkit-file-upload-button:hover,
.ui.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #cacbcd;
  color: rgba(0, 0, 0, 0.6);
}

.ui.form .field input[type="file"]::file-selector-button:hover,
.ui.file.input input[type="file"]::file-selector-button:hover {
  background: #cacbcd;
  color: rgba(0, 0, 0, 0.6);
}

input[type="file"].ui.invisible.file.input,
.ui.invisible.file.input input[type="file"] {
  left: -99999px;
  position: absolute;
}

input[type="file"].ui.file.input:focus + label.ui.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.button:not(.basic):not(.tertiary) {
  background: #cacbcd;
  color: rgba(0, 0, 0, 0.8);
}

input[type="file"].ui.file.input:focus + label.ui.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.button:not(.basic):not(.tertiary).inverted {
  background: #fff;
}

/* this is related to existing buttons, so the button color variable is used here! */

input[type="file"].ui.file.input:focus + label.ui.primary.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.primary.button:not(.basic):not(.tertiary) {
  background-color: #1678c2;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.primary.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.primary.button:not(.basic):not(.tertiary).inverted {
  background-color: #21b8ff;
}

input[type="file"].ui.file.input:focus + label.ui.secondary.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.secondary.button:not(.basic):not(.tertiary) {
  background-color: #27292a;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.secondary.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.secondary.button:not(.basic):not(.tertiary).inverted {
  background-color: #6e6e6e;
}

input[type="file"].ui.file.input:focus + label.ui.red.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.red.button:not(.basic):not(.tertiary) {
  background-color: #d01919;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.red.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.red.button:not(.basic):not(.tertiary).inverted {
  background-color: #ff392b;
}

input[type="file"].ui.file.input:focus + label.ui.orange.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.orange.button:not(.basic):not(.tertiary) {
  background-color: #f26202;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.orange.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.orange.button:not(.basic):not(.tertiary).inverted {
  background-color: #e76b00;
}

input[type="file"].ui.file.input:focus + label.ui.yellow.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.yellow.button:not(.basic):not(.tertiary) {
  background-color: #eaae00;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.yellow.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.yellow.button:not(.basic):not(.tertiary).inverted {
  background-color: #ebcd00;
}

input[type="file"].ui.file.input:focus + label.ui.olive.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.olive.button:not(.basic):not(.tertiary) {
  background-color: #a7bd0d;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.olive.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.olive.button:not(.basic):not(.tertiary).inverted {
  background-color: #d2e745;
}

input[type="file"].ui.file.input:focus + label.ui.green.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.green.button:not(.basic):not(.tertiary) {
  background-color: #16ab39;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.green.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.green.button:not(.basic):not(.tertiary).inverted {
  background-color: #1ea92e;
}

input[type="file"].ui.file.input:focus + label.ui.teal.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.teal.button:not(.basic):not(.tertiary) {
  background-color: #009c95;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.teal.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.teal.button:not(.basic):not(.tertiary).inverted {
  background-color: #3affff;
}

input[type="file"].ui.file.input:focus + label.ui.blue.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.blue.button:not(.basic):not(.tertiary) {
  background-color: #1678c2;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.blue.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.blue.button:not(.basic):not(.tertiary).inverted {
  background-color: #21b8ff;
}

input[type="file"].ui.file.input:focus + label.ui.violet.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.violet.button:not(.basic):not(.tertiary) {
  background-color: #5829bb;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.violet.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.violet.button:not(.basic):not(.tertiary).inverted {
  background-color: #745aff;
}

input[type="file"].ui.file.input:focus + label.ui.purple.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.purple.button:not(.basic):not(.tertiary) {
  background-color: #9627ba;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.purple.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.purple.button:not(.basic):not(.tertiary).inverted {
  background-color: #cf40ff;
}

input[type="file"].ui.file.input:focus + label.ui.pink.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.pink.button:not(.basic):not(.tertiary) {
  background-color: #e61a8d;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.pink.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.pink.button:not(.basic):not(.tertiary).inverted {
  background-color: #ff5bd1;
}

input[type="file"].ui.file.input:focus + label.ui.brown.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.brown.button:not(.basic):not(.tertiary) {
  background-color: #975b33;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.brown.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.brown.button:not(.basic):not(.tertiary).inverted {
  background-color: #b0620f;
}

input[type="file"].ui.file.input:focus + label.ui.grey.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.grey.button:not(.basic):not(.tertiary) {
  background-color: #838383;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.grey.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.grey.button:not(.basic):not(.tertiary).inverted {
  background-color: #c2c4c5;
}

input[type="file"].ui.file.input:focus + label.ui.black.button:not(.basic):not(.tertiary),
.ui.file.input input[type="file"]:focus + label.ui.black.button:not(.basic):not(.tertiary) {
  background-color: #27292a;
  color: #fff;
}

input[type="file"].ui.file.input:focus + label.ui.black.button:not(.basic):not(.tertiary).inverted,
.ui.file.input input[type="file"]:focus + label.ui.black.button:not(.basic):not(.tertiary).inverted {
  background-color: #000000;
}

input[type="file"].ui.primary.file.input::-webkit-file-upload-button,
.ui.primary.file.input input[type="file"]::-webkit-file-upload-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.primary.file.input::-webkit-file-upload-button:hover,
.ui.primary.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #1678c2;
}

input[type="file"].ui.primary.file.input::-ms-browse,
.ui.primary.file.input input[type="file"]::-ms-browse {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.primary.file.input::-ms-browse:hover,
.ui.primary.file.input input[type="file"]::-ms-browse:hover {
  background: #1678c2;
}

input[type="file"].ui.primary.file.input::-webkit-file-upload-button,
.ui.primary.file.input input[type="file"]::-webkit-file-upload-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.primary.file.input::file-selector-button,
.ui.primary.file.input input[type="file"]::file-selector-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.primary.file.input::-webkit-file-upload-button:hover,
.ui.primary.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #1678c2;
}

input[type="file"].ui.primary.file.input::file-selector-button:hover,
.ui.primary.file.input input[type="file"]::file-selector-button:hover {
  background: #1678c2;
}

input[type="file"].ui.secondary.file.input::-webkit-file-upload-button,
.ui.secondary.file.input input[type="file"]::-webkit-file-upload-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.secondary.file.input::-webkit-file-upload-button:hover,
.ui.secondary.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #27292a;
}

input[type="file"].ui.secondary.file.input::-ms-browse,
.ui.secondary.file.input input[type="file"]::-ms-browse {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.secondary.file.input::-ms-browse:hover,
.ui.secondary.file.input input[type="file"]::-ms-browse:hover {
  background: #27292a;
}

input[type="file"].ui.secondary.file.input::-webkit-file-upload-button,
.ui.secondary.file.input input[type="file"]::-webkit-file-upload-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.secondary.file.input::file-selector-button,
.ui.secondary.file.input input[type="file"]::file-selector-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.secondary.file.input::-webkit-file-upload-button:hover,
.ui.secondary.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #27292a;
}

input[type="file"].ui.secondary.file.input::file-selector-button:hover,
.ui.secondary.file.input input[type="file"]::file-selector-button:hover {
  background: #27292a;
}

input[type="file"].ui.red.file.input::-webkit-file-upload-button,
.ui.red.file.input input[type="file"]::-webkit-file-upload-button {
  background: #db2828;
  color: #fff;
}

input[type="file"].ui.red.file.input::-webkit-file-upload-button:hover,
.ui.red.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #d01919;
}

input[type="file"].ui.red.file.input::-ms-browse,
.ui.red.file.input input[type="file"]::-ms-browse {
  background: #db2828;
  color: #fff;
}

input[type="file"].ui.red.file.input::-ms-browse:hover,
.ui.red.file.input input[type="file"]::-ms-browse:hover {
  background: #d01919;
}

input[type="file"].ui.red.file.input::-webkit-file-upload-button,
.ui.red.file.input input[type="file"]::-webkit-file-upload-button {
  background: #db2828;
  color: #fff;
}

input[type="file"].ui.red.file.input::file-selector-button,
.ui.red.file.input input[type="file"]::file-selector-button {
  background: #db2828;
  color: #fff;
}

input[type="file"].ui.red.file.input::-webkit-file-upload-button:hover,
.ui.red.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #d01919;
}

input[type="file"].ui.red.file.input::file-selector-button:hover,
.ui.red.file.input input[type="file"]::file-selector-button:hover {
  background: #d01919;
}

input[type="file"].ui.orange.file.input::-webkit-file-upload-button,
.ui.orange.file.input input[type="file"]::-webkit-file-upload-button {
  background: #f2711c;
  color: #fff;
}

input[type="file"].ui.orange.file.input::-webkit-file-upload-button:hover,
.ui.orange.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #f26202;
}

input[type="file"].ui.orange.file.input::-ms-browse,
.ui.orange.file.input input[type="file"]::-ms-browse {
  background: #f2711c;
  color: #fff;
}

input[type="file"].ui.orange.file.input::-ms-browse:hover,
.ui.orange.file.input input[type="file"]::-ms-browse:hover {
  background: #f26202;
}

input[type="file"].ui.orange.file.input::-webkit-file-upload-button,
.ui.orange.file.input input[type="file"]::-webkit-file-upload-button {
  background: #f2711c;
  color: #fff;
}

input[type="file"].ui.orange.file.input::file-selector-button,
.ui.orange.file.input input[type="file"]::file-selector-button {
  background: #f2711c;
  color: #fff;
}

input[type="file"].ui.orange.file.input::-webkit-file-upload-button:hover,
.ui.orange.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #f26202;
}

input[type="file"].ui.orange.file.input::file-selector-button:hover,
.ui.orange.file.input input[type="file"]::file-selector-button:hover {
  background: #f26202;
}

input[type="file"].ui.yellow.file.input::-webkit-file-upload-button,
.ui.yellow.file.input input[type="file"]::-webkit-file-upload-button {
  background: #fbbd08;
  color: #fff;
}

input[type="file"].ui.yellow.file.input::-webkit-file-upload-button:hover,
.ui.yellow.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #eaae00;
}

input[type="file"].ui.yellow.file.input::-ms-browse,
.ui.yellow.file.input input[type="file"]::-ms-browse {
  background: #fbbd08;
  color: #fff;
}

input[type="file"].ui.yellow.file.input::-ms-browse:hover,
.ui.yellow.file.input input[type="file"]::-ms-browse:hover {
  background: #eaae00;
}

input[type="file"].ui.yellow.file.input::-webkit-file-upload-button,
.ui.yellow.file.input input[type="file"]::-webkit-file-upload-button {
  background: #fbbd08;
  color: #fff;
}

input[type="file"].ui.yellow.file.input::file-selector-button,
.ui.yellow.file.input input[type="file"]::file-selector-button {
  background: #fbbd08;
  color: #fff;
}

input[type="file"].ui.yellow.file.input::-webkit-file-upload-button:hover,
.ui.yellow.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #eaae00;
}

input[type="file"].ui.yellow.file.input::file-selector-button:hover,
.ui.yellow.file.input input[type="file"]::file-selector-button:hover {
  background: #eaae00;
}

input[type="file"].ui.olive.file.input::-webkit-file-upload-button,
.ui.olive.file.input input[type="file"]::-webkit-file-upload-button {
  background: #b5cc18;
  color: #fff;
}

input[type="file"].ui.olive.file.input::-webkit-file-upload-button:hover,
.ui.olive.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #a7bd0d;
}

input[type="file"].ui.olive.file.input::-ms-browse,
.ui.olive.file.input input[type="file"]::-ms-browse {
  background: #b5cc18;
  color: #fff;
}

input[type="file"].ui.olive.file.input::-ms-browse:hover,
.ui.olive.file.input input[type="file"]::-ms-browse:hover {
  background: #a7bd0d;
}

input[type="file"].ui.olive.file.input::-webkit-file-upload-button,
.ui.olive.file.input input[type="file"]::-webkit-file-upload-button {
  background: #b5cc18;
  color: #fff;
}

input[type="file"].ui.olive.file.input::file-selector-button,
.ui.olive.file.input input[type="file"]::file-selector-button {
  background: #b5cc18;
  color: #fff;
}

input[type="file"].ui.olive.file.input::-webkit-file-upload-button:hover,
.ui.olive.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #a7bd0d;
}

input[type="file"].ui.olive.file.input::file-selector-button:hover,
.ui.olive.file.input input[type="file"]::file-selector-button:hover {
  background: #a7bd0d;
}

input[type="file"].ui.green.file.input::-webkit-file-upload-button,
.ui.green.file.input input[type="file"]::-webkit-file-upload-button {
  background: #21ba45;
  color: #fff;
}

input[type="file"].ui.green.file.input::-webkit-file-upload-button:hover,
.ui.green.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #16ab39;
}

input[type="file"].ui.green.file.input::-ms-browse,
.ui.green.file.input input[type="file"]::-ms-browse {
  background: #21ba45;
  color: #fff;
}

input[type="file"].ui.green.file.input::-ms-browse:hover,
.ui.green.file.input input[type="file"]::-ms-browse:hover {
  background: #16ab39;
}

input[type="file"].ui.green.file.input::-webkit-file-upload-button,
.ui.green.file.input input[type="file"]::-webkit-file-upload-button {
  background: #21ba45;
  color: #fff;
}

input[type="file"].ui.green.file.input::file-selector-button,
.ui.green.file.input input[type="file"]::file-selector-button {
  background: #21ba45;
  color: #fff;
}

input[type="file"].ui.green.file.input::-webkit-file-upload-button:hover,
.ui.green.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #16ab39;
}

input[type="file"].ui.green.file.input::file-selector-button:hover,
.ui.green.file.input input[type="file"]::file-selector-button:hover {
  background: #16ab39;
}

input[type="file"].ui.teal.file.input::-webkit-file-upload-button,
.ui.teal.file.input input[type="file"]::-webkit-file-upload-button {
  background: #00b5ad;
  color: #fff;
}

input[type="file"].ui.teal.file.input::-webkit-file-upload-button:hover,
.ui.teal.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #009c95;
}

input[type="file"].ui.teal.file.input::-ms-browse,
.ui.teal.file.input input[type="file"]::-ms-browse {
  background: #00b5ad;
  color: #fff;
}

input[type="file"].ui.teal.file.input::-ms-browse:hover,
.ui.teal.file.input input[type="file"]::-ms-browse:hover {
  background: #009c95;
}

input[type="file"].ui.teal.file.input::-webkit-file-upload-button,
.ui.teal.file.input input[type="file"]::-webkit-file-upload-button {
  background: #00b5ad;
  color: #fff;
}

input[type="file"].ui.teal.file.input::file-selector-button,
.ui.teal.file.input input[type="file"]::file-selector-button {
  background: #00b5ad;
  color: #fff;
}

input[type="file"].ui.teal.file.input::-webkit-file-upload-button:hover,
.ui.teal.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #009c95;
}

input[type="file"].ui.teal.file.input::file-selector-button:hover,
.ui.teal.file.input input[type="file"]::file-selector-button:hover {
  background: #009c95;
}

input[type="file"].ui.blue.file.input::-webkit-file-upload-button,
.ui.blue.file.input input[type="file"]::-webkit-file-upload-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.blue.file.input::-webkit-file-upload-button:hover,
.ui.blue.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #1678c2;
}

input[type="file"].ui.blue.file.input::-ms-browse,
.ui.blue.file.input input[type="file"]::-ms-browse {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.blue.file.input::-ms-browse:hover,
.ui.blue.file.input input[type="file"]::-ms-browse:hover {
  background: #1678c2;
}

input[type="file"].ui.blue.file.input::-webkit-file-upload-button,
.ui.blue.file.input input[type="file"]::-webkit-file-upload-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.blue.file.input::file-selector-button,
.ui.blue.file.input input[type="file"]::file-selector-button {
  background: #2185d0;
  color: #fff;
}

input[type="file"].ui.blue.file.input::-webkit-file-upload-button:hover,
.ui.blue.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #1678c2;
}

input[type="file"].ui.blue.file.input::file-selector-button:hover,
.ui.blue.file.input input[type="file"]::file-selector-button:hover {
  background: #1678c2;
}

input[type="file"].ui.violet.file.input::-webkit-file-upload-button,
.ui.violet.file.input input[type="file"]::-webkit-file-upload-button {
  background: #6435c9;
  color: #fff;
}

input[type="file"].ui.violet.file.input::-webkit-file-upload-button:hover,
.ui.violet.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #5829bb;
}

input[type="file"].ui.violet.file.input::-ms-browse,
.ui.violet.file.input input[type="file"]::-ms-browse {
  background: #6435c9;
  color: #fff;
}

input[type="file"].ui.violet.file.input::-ms-browse:hover,
.ui.violet.file.input input[type="file"]::-ms-browse:hover {
  background: #5829bb;
}

input[type="file"].ui.violet.file.input::-webkit-file-upload-button,
.ui.violet.file.input input[type="file"]::-webkit-file-upload-button {
  background: #6435c9;
  color: #fff;
}

input[type="file"].ui.violet.file.input::file-selector-button,
.ui.violet.file.input input[type="file"]::file-selector-button {
  background: #6435c9;
  color: #fff;
}

input[type="file"].ui.violet.file.input::-webkit-file-upload-button:hover,
.ui.violet.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #5829bb;
}

input[type="file"].ui.violet.file.input::file-selector-button:hover,
.ui.violet.file.input input[type="file"]::file-selector-button:hover {
  background: #5829bb;
}

input[type="file"].ui.purple.file.input::-webkit-file-upload-button,
.ui.purple.file.input input[type="file"]::-webkit-file-upload-button {
  background: #a333c8;
  color: #fff;
}

input[type="file"].ui.purple.file.input::-webkit-file-upload-button:hover,
.ui.purple.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #9627ba;
}

input[type="file"].ui.purple.file.input::-ms-browse,
.ui.purple.file.input input[type="file"]::-ms-browse {
  background: #a333c8;
  color: #fff;
}

input[type="file"].ui.purple.file.input::-ms-browse:hover,
.ui.purple.file.input input[type="file"]::-ms-browse:hover {
  background: #9627ba;
}

input[type="file"].ui.purple.file.input::-webkit-file-upload-button,
.ui.purple.file.input input[type="file"]::-webkit-file-upload-button {
  background: #a333c8;
  color: #fff;
}

input[type="file"].ui.purple.file.input::file-selector-button,
.ui.purple.file.input input[type="file"]::file-selector-button {
  background: #a333c8;
  color: #fff;
}

input[type="file"].ui.purple.file.input::-webkit-file-upload-button:hover,
.ui.purple.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #9627ba;
}

input[type="file"].ui.purple.file.input::file-selector-button:hover,
.ui.purple.file.input input[type="file"]::file-selector-button:hover {
  background: #9627ba;
}

input[type="file"].ui.pink.file.input::-webkit-file-upload-button,
.ui.pink.file.input input[type="file"]::-webkit-file-upload-button {
  background: #e03997;
  color: #fff;
}

input[type="file"].ui.pink.file.input::-webkit-file-upload-button:hover,
.ui.pink.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #e61a8d;
}

input[type="file"].ui.pink.file.input::-ms-browse,
.ui.pink.file.input input[type="file"]::-ms-browse {
  background: #e03997;
  color: #fff;
}

input[type="file"].ui.pink.file.input::-ms-browse:hover,
.ui.pink.file.input input[type="file"]::-ms-browse:hover {
  background: #e61a8d;
}

input[type="file"].ui.pink.file.input::-webkit-file-upload-button,
.ui.pink.file.input input[type="file"]::-webkit-file-upload-button {
  background: #e03997;
  color: #fff;
}

input[type="file"].ui.pink.file.input::file-selector-button,
.ui.pink.file.input input[type="file"]::file-selector-button {
  background: #e03997;
  color: #fff;
}

input[type="file"].ui.pink.file.input::-webkit-file-upload-button:hover,
.ui.pink.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #e61a8d;
}

input[type="file"].ui.pink.file.input::file-selector-button:hover,
.ui.pink.file.input input[type="file"]::file-selector-button:hover {
  background: #e61a8d;
}

input[type="file"].ui.brown.file.input::-webkit-file-upload-button,
.ui.brown.file.input input[type="file"]::-webkit-file-upload-button {
  background: #a5673f;
  color: #fff;
}

input[type="file"].ui.brown.file.input::-webkit-file-upload-button:hover,
.ui.brown.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #975b33;
}

input[type="file"].ui.brown.file.input::-ms-browse,
.ui.brown.file.input input[type="file"]::-ms-browse {
  background: #a5673f;
  color: #fff;
}

input[type="file"].ui.brown.file.input::-ms-browse:hover,
.ui.brown.file.input input[type="file"]::-ms-browse:hover {
  background: #975b33;
}

input[type="file"].ui.brown.file.input::-webkit-file-upload-button,
.ui.brown.file.input input[type="file"]::-webkit-file-upload-button {
  background: #a5673f;
  color: #fff;
}

input[type="file"].ui.brown.file.input::file-selector-button,
.ui.brown.file.input input[type="file"]::file-selector-button {
  background: #a5673f;
  color: #fff;
}

input[type="file"].ui.brown.file.input::-webkit-file-upload-button:hover,
.ui.brown.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #975b33;
}

input[type="file"].ui.brown.file.input::file-selector-button:hover,
.ui.brown.file.input input[type="file"]::file-selector-button:hover {
  background: #975b33;
}

input[type="file"].ui.grey.file.input::-webkit-file-upload-button,
.ui.grey.file.input input[type="file"]::-webkit-file-upload-button {
  background: #767676;
  color: #fff;
}

input[type="file"].ui.grey.file.input::-webkit-file-upload-button:hover,
.ui.grey.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #838383;
}

input[type="file"].ui.grey.file.input::-ms-browse,
.ui.grey.file.input input[type="file"]::-ms-browse {
  background: #767676;
  color: #fff;
}

input[type="file"].ui.grey.file.input::-ms-browse:hover,
.ui.grey.file.input input[type="file"]::-ms-browse:hover {
  background: #838383;
}

input[type="file"].ui.grey.file.input::-webkit-file-upload-button,
.ui.grey.file.input input[type="file"]::-webkit-file-upload-button {
  background: #767676;
  color: #fff;
}

input[type="file"].ui.grey.file.input::file-selector-button,
.ui.grey.file.input input[type="file"]::file-selector-button {
  background: #767676;
  color: #fff;
}

input[type="file"].ui.grey.file.input::-webkit-file-upload-button:hover,
.ui.grey.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #838383;
}

input[type="file"].ui.grey.file.input::file-selector-button:hover,
.ui.grey.file.input input[type="file"]::file-selector-button:hover {
  background: #838383;
}

input[type="file"].ui.black.file.input::-webkit-file-upload-button,
.ui.black.file.input input[type="file"]::-webkit-file-upload-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.black.file.input::-webkit-file-upload-button:hover,
.ui.black.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #27292a;
}

input[type="file"].ui.black.file.input::-ms-browse,
.ui.black.file.input input[type="file"]::-ms-browse {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.black.file.input::-ms-browse:hover,
.ui.black.file.input input[type="file"]::-ms-browse:hover {
  background: #27292a;
}

input[type="file"].ui.black.file.input::-webkit-file-upload-button,
.ui.black.file.input input[type="file"]::-webkit-file-upload-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.black.file.input::file-selector-button,
.ui.black.file.input input[type="file"]::file-selector-button {
  background: #1b1c1d;
  color: #fff;
}

input[type="file"].ui.black.file.input::-webkit-file-upload-button:hover,
.ui.black.file.input input[type="file"]::-webkit-file-upload-button:hover {
  background: #27292a;
}

input[type="file"].ui.black.file.input::file-selector-button:hover,
.ui.black.file.input input[type="file"]::file-selector-button:hover {
  background: #27292a;
}

/* --------------------
        Size
--------------------- */

.ui.input {
  font-size: 1em;
}

.ui.mini.input {
  font-size: 0.78571429em;
}

.ui.tiny.input {
  font-size: 0.85714286em;
}

.ui.small.input {
  font-size: 0.92857143em;
}

.ui.large.input {
  font-size: 1.14285714em;
}

.ui.big.input {
  font-size: 1.28571429em;
}

.ui.huge.input {
  font-size: 1.42857143em;
}

.ui.massive.input {
  font-size: 1.71428571em;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Label
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Label
*******************************/

.ui.label {
  display: inline-block;
  line-height: 1;
  vertical-align: baseline;
  margin: 0 0.14285714em;
  background-color: #e8e8e8;
  background-image: none;
  padding: 0.5833em 0.833em;
  color: rgba(0, 0, 0, 0.6);
  text-transform: none;
  font-weight: bold;
  border: 0 solid transparent;
  border-radius: 0.28571429rem;
  transition: background 0.1s ease;
}

.ui.label:first-child {
  margin-left: 0;
}

.ui.label:last-child {
  margin-right: 0;
}

/* Link */

a.ui.label {
  cursor: pointer;
}

/* Inside Link */

.ui.label > a {
  cursor: pointer;
  color: inherit;
  opacity: 0.5;
  transition: 0.1s opacity ease;
}

.ui.label > a:hover {
  opacity: 1;
}

/* Image */

.ui.label > img {
  width: auto !important;
  vertical-align: middle;
  height: 2.1666em;
}

/* Icon */

.ui.ui[class*="left icon"].label > .icon,
.ui.label > .icon {
  width: auto;
  margin: 0 0.75em 0 0;
}

/* Detail */

.ui.label > .detail {
  display: inline-block;
  vertical-align: top;
  font-weight: bold;
  margin-left: 1em;
  opacity: 0.8;
}

.ui.label > .detail .icon {
  margin: 0 0.25em 0 0;
}

/* Removable label */

.ui.label > .close.icon,
.ui.label > .delete.icon {
  cursor: pointer;
  font-size: 0.92857143em;
  opacity: 0.5;
  transition: background 0.1s ease;
}

.ui.label > .close.icon:hover,
.ui.label > .delete.icon:hover {
  opacity: 1;
}

/* Backward compatible positioning */

.ui.label[class*="left icon"] > .close.icon,
.ui.label[class*="left icon"] > .delete.icon {
  margin: 0 0.5em 0 0;
}

.ui.label[class*="left icon"] > .close.icon.right,
.ui.label[class*="left icon"] > .delete.icon.right {
  margin: 0 0 0 0.5em;
}

.ui.label:not(.icon) > .close.icon,
.ui.label:not(.icon) > .delete.icon {
  margin: 0 0 0 0.5em;
}

/* Label for only an icon */

.ui.icon.label > .icon {
  margin: 0 auto;
}

/* Right Side Icon */

.ui[class*="right icon"].label > .icon {
  margin: 0 0 0 0.75em;
}

/* -------------------
       Group
-------------------- */

.ui.labels > .label {
  margin: 0 0.5em 0.5em 0;
}

/* -------------------
       Coupling
-------------------- */

.ui.header > .ui.label {
  margin-top: -0.29165em;
}

/* Remove border radius on attached segment */

.ui.attached.segment > .ui.top.left.attached.label,
.ui.bottom.attached.segment > .ui.top.left.attached.label {
  border-top-left-radius: 0;
}

.ui.attached.segment > .ui.top.right.attached.label,
.ui.bottom.attached.segment > .ui.top.right.attached.label {
  border-top-right-radius: 0;
}

.ui.top.attached.segment > .ui.bottom.left.attached.label {
  border-bottom-left-radius: 0;
}

.ui.top.attached.segment > .ui.bottom.right.attached.label {
  border-bottom-right-radius: 0;
}

/* Padding on next content after a label */

.ui.top.attached.label ~ .ui.bottom.attached.label + :not(.attached),
.ui.top.attached.label + :not(.attached) {
  margin-top: 2rem !important;
}

.ui.bottom.attached.label ~ :last-child:not(.attached) {
  margin-top: 0;
  margin-bottom: 2rem !important;
}

.ui.segment:not(.basic) > .ui.top.attached.label {
  margin-top: -1px;
}

.ui.segment:not(.basic) > .ui.bottom.attached.label {
  margin-bottom: -1px;
}

.ui.segment:not(.basic) > .ui.attached.label:not(.right) {
  margin-left: -1px;
}

.ui.segment:not(.basic) > .ui.right.attached.label {
  margin-right: -1px;
}

.ui.segment:not(.basic) > .ui.attached.label:not(.left):not(.right) {
  width: calc(100% + 2px);
}

/*******************************
             Types
*******************************/

.ui.image.label {
  width: auto;
  margin-top: 0;
  margin-bottom: 0;
  max-width: 9999px;
  vertical-align: baseline;
  text-transform: none;
  background: #e8e8e8;
  padding: 0.5833em 0.833em 0.5833em 0.5em;
  border-radius: 0.28571429rem;
  box-shadow: none;
}

.ui.image.label.attached:not(.basic) {
  padding: 0.5833em 0.833em 0.5833em 0.5em;
}

.ui.image.label img {
  display: inline-block;
  vertical-align: top;
  height: 2.1666em;
  margin: -0.5833em 0.5em -0.5833em -0.5em;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui.image.label .detail {
  background: rgba(0, 0, 0, 0.1);
  margin: -0.5833em -0.833em -0.5833em 0.5em;
  padding: 0.5833em 0.833em;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui.bottom.attached.image.label:not(.right) > img,
.ui.top.right.attached.image.label > img {
  border-top-left-radius: 0;
}

.ui.top.attached.image.label:not(.right) > img,
.ui.bottom.right.attached.image.label > img {
  border-bottom-left-radius: 0;
}

/* -------------------
             Tag
    -------------------- */

.ui.tag.labels .label,
.ui.tag.label {
  margin-left: 1em;
  position: relative;
  padding-left: 1.5em;
  padding-right: 1.5em;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
  transition: none;
}

.ui.tag.labels .label::before,
.ui.tag.label::before {
  position: absolute;
  transform: translateY(-50%) translateX(50%) rotate(-45deg);
  top: 50%;
  right: 100%;
  content: "";
  background-color: inherit;
  background-image: none;
  width: 1.56em;
  height: 1.56em;
  transition: none;
}

.ui.tag.labels .label::after,
.ui.tag.label::after {
  position: absolute;
  content: "";
  top: 50%;
  left: -0.25em;
  margin-top: -0.25em;
  background-color: #fff;
  width: 0.5em;
  height: 0.5em;
  box-shadow: 0 -1px 1px 0 rgba(0, 0, 0, 0.3);
  border-radius: 500rem;
}

.ui.basic.tag.labels .label::before,
.ui.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  right: calc(100% + 1px);
}

.ui.basic.tag.labels .label::after,
.ui.basic.tag.label::after {
  box-shadow: 0 -1px 3px 0 rgba(0, 0, 0, 0.8);
}

/* -------------------
        Corner Label
    -------------------- */

.ui.corner.label {
  position: absolute;
  top: 0;
  right: 0;
  margin: 0;
  padding: 0;
  text-align: center;
  border-color: #e8e8e8;
  width: 4em;
  height: 4em;
  z-index: 1;
  transition: border-color 0.1s ease;
}

/* Icon Label */

.ui.corner.label {
  background-color: transparent !important;
}

.ui.corner.label::after {
  position: absolute;
  content: "";
  right: 0;
  top: 0;
  z-index: -1;
  width: 0;
  height: 0;
  background-color: transparent;
  border-top: 0 solid transparent;
  border-right: 4em solid transparent;
  border-bottom: 4em solid transparent;
  border-left: 0 solid transparent;
  border-right-color: inherit;
  transition: border-color 0.1s ease;
}

.ui.corner.label .icon {
  cursor: inherit;
  position: absolute;
  top: 0.64285714em;
  left: auto;
  right: 0.57142857em;
  font-size: 1.14285714em;
  margin: 0;
}

/* Left Corner */

.ui.left.corner.label,
.ui.left.corner.label::after {
  right: auto;
  left: 0;
}

.ui.left.corner.label::after {
  border-top: 4em solid transparent;
  border-right: 4em solid transparent;
  border-bottom: 0 solid transparent;
  border-left: 0 solid transparent;
  border-top-color: inherit;
}

.ui.left.corner.label .icon {
  left: 0.57142857em;
  right: auto;
}

/* Segment */

.ui.segment > .ui.corner.label {
  top: -1px;
  right: -1px;
}

.ui.segment > .ui.left.corner.label {
  right: auto;
  left: -1px;
}

/* -------------------
           Ribbon
    -------------------- */

.ui.ribbon.label {
  position: relative;
  margin: 0;
  min-width: -webkit-max-content;
  min-width: max-content;
  border-radius: 0 0.28571429rem 0.28571429rem 0;
  border-color: rgba(0, 0, 0, 0.15);
}

.ui.ribbon.label::after {
  position: absolute;
  content: "";
  top: 100%;
  left: 0;
  background-color: transparent;
  border-style: solid;
  border-width: 0 1.2em 1.2em 0;
  border-color: transparent;
  border-right-color: inherit;
  width: 0;
  height: 0;
}

/* Positioning */

.ui.ribbon.label {
  left: calc(-1rem - 1.2em);
  margin-right: -1.2em;
  padding-left: calc(1rem + 1.2em);
  padding-right: 1.2em;
}

.ui[class*="right ribbon"].label {
  left: calc(100% + 1rem + 1.2em);
  padding-left: 1.2em;
  padding-right: calc(1rem + 1.2em);
}

.ui.basic.ribbon.label {
  padding-top: calc(0.5833em - 1px);
  padding-bottom: calc(0.5833em - 1px);
}

.ui.basic.ribbon.label:not([class*="right ribbon"]) {
  padding-left: calc(1rem + 1.2em - 1px);
  padding-right: calc(1.2em - 1px);
}

.ui.basic[class*="right ribbon"].label {
  padding-left: calc(1.2em - 1px);
  padding-right: calc(1rem + 1.2em - 1px);
}

.ui.basic.ribbon.label::after {
  top: calc(100% + 1px);
}

.ui.basic.ribbon.label:not([class*="right ribbon"])::after {
  left: -1px;
}

.ui.basic[class*="right ribbon"].label::after {
  right: -1px;
}

/* Right Ribbon */

.ui[class*="right ribbon"].label {
  text-align: left;
  transform: translateX(-100%);
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui[class*="right ribbon"].label::after {
  left: auto;
  right: 0;
  border-style: solid;
  border-width: 1.2em 1.2em 0 0;
  border-color: transparent;
  border-top-color: inherit;
}

.ui.ui.center.ribbon.label {
  top: calc(50% - 1em);
  z-index: 1;
}

.ui.bottom.ribbon.label::after {
  bottom: 100%;
  top: auto;
  border-width: 1.2em 1.2em 0 0;
}

.ui.bottom[class*="right ribbon"].label::after {
  border-width: 1.2em 0 0 1.2em;
  border-color: transparent;
  border-left-color: inherit;
}

/* Inside Image */

.ui.image > .ribbon.label,
.ui.card .image > .ribbon.label {
  position: absolute;
  top: 1rem;
}

.ui.card .image > .ui.ribbon.label,
.ui.image > .ui.ribbon.label {
  left: calc(0.05rem - 1.2em);
}

.ui.card .image > .ui[class*="right ribbon"].label,
.ui.image > .ui[class*="right ribbon"].label {
  left: calc(100% + -0.05rem + 1.2em);
  padding-left: 0.833em;
}

.ui.card .image > .ui.bottom.ribbon.label,
.ui.image > .ui.bottom.ribbon.label {
  bottom: 1rem;
  top: auto;
}

/* Inside Table */

.ui.table td > .ui.ribbon.label {
  left: calc(-1em - 1.2em);
}

.ui.table td > .ui[class*="right ribbon"].label {
  left: calc(100% + 1em + 1.2em);
  padding-left: 0.833em;
}

/* -------------------
          Attached
    -------------------- */

.ui[class*="top attached"].label,
.ui.attached.label {
  width: 100%;
  position: absolute;
  margin: 0;
  top: 0;
  left: 0;
  padding: 0.75em 1em;
  border-radius: 0.21428571rem 0.21428571rem 0 0;
}

.ui[class*="bottom attached"].label {
  top: auto;
  bottom: 0;
  border-radius: 0 0 0.21428571rem 0.21428571rem;
}

.ui[class*="top left attached"].label {
  width: auto;
  margin-top: 0;
  border-radius: 0.21428571rem 0 0.28571429rem 0;
}

.ui[class*="top right attached"].label {
  width: auto;
  left: auto;
  right: 0;
  border-radius: 0 0.21428571rem 0 0.28571429rem;
}

.ui[class*="bottom left attached"].label {
  width: auto;
  top: auto;
  bottom: 0;
  border-radius: 0 0.28571429rem 0 0.21428571rem;
}

.ui[class*="bottom right attached"].label {
  top: auto;
  bottom: 0;
  left: auto;
  right: 0;
  width: auto;
  border-radius: 0.28571429rem 0 0.21428571rem 0;
}

/*******************************
             States
*******************************/

/* -------------------
      Disabled
-------------------- */

.ui.disabled.labels .label,
.ui.label.disabled {
  opacity: 0.45;
  pointer-events: none;
}

/* -------------------
        Hover
-------------------- */

.ui.labels a.label:hover,
a.ui.label:hover {
  background-color: #e0e0e0;
  border-color: #e0e0e0;
  background-image: none;
  color: rgba(0, 0, 0, 0.8);
}

.ui.labels a.label:hover::before,
a.ui.label:hover::before {
  color: rgba(0, 0, 0, 0.8);
}

/* -------------------
        Active
-------------------- */

.ui.active.label {
  background-color: #d0d0d0;
  border-color: #d0d0d0;
  background-image: none;
  color: rgba(0, 0, 0, 0.95);
}

.ui.active.label::before {
  background-color: #d0d0d0;
  background-image: none;
  color: rgba(0, 0, 0, 0.95);
}

/* -------------------
     Active Hover
-------------------- */

.ui.labels a.active.label:hover,
a.ui.active.label:hover {
  background-color: #c8c8c8;
  border-color: #c8c8c8;
  background-image: none;
  color: rgba(0, 0, 0, 0.95);
}

.ui.labels a.active.label:hover::before,
a.ui.active.label:hover::before {
  background-color: #c8c8c8;
  background-image: none;
  color: rgba(0, 0, 0, 0.95);
}

/* -------------------
      Visible
-------------------- */

.ui.labels.visible .label,
.ui.label.visible:not(.dropdown) {
  display: inline-block !important;
}

/* -------------------
      Hidden
-------------------- */

.ui.labels.hidden .label,
.ui.label.hidden {
  display: none !important;
}

/*******************************
           Variations
*******************************/

/* -------------------
            Basic
    -------------------- */

.ui.basic.labels .label,
.ui.basic.label {
  background: none #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  color: rgba(0, 0, 0, 0.87);
  box-shadow: none;
  padding-top: calc(0.5833em - 1px);
  padding-bottom: calc(0.5833em - 1px);
  padding-right: calc(0.833em - 1px);
}

.ui.basic.labels:not(.tag):not(.image):not(.ribbon) .label,
.ui.basic.label:not(.tag):not(.image):not(.ribbon) {
  padding-left: calc(0.833em - 1px);
}

.ui.basic.image.label {
  padding-left: calc(0.5em - 1px);
}

/* Link */

.ui.basic.labels a.label:hover,
a.ui.basic.label:hover {
  text-decoration: none;
  background: none #fff;
  color: #1e70bf;
  box-shadow: none;
}

/* Pointing */

.ui.basic.pointing.label::before {
  border-color: inherit;
}

/* -------------------
           Fluid
    -------------------- */

.ui.label.fluid,
.ui.fluid.labels > .label {
  width: 100%;
  box-sizing: border-box;
}

.ui.centered.labels .label,
.ui.centered.label {
  text-align: center;
}

/* -------------------
           Inverted
    -------------------- */

.ui.inverted.labels .label,
.ui.inverted.label {
  color: rgba(255, 255, 255, 0.9);
  background-color: #b5b5b5;
}

.ui.inverted.corner.label {
  border-color: #b5b5b5;
}

.ui.inverted.corner.label:hover {
  border-color: #e8e8e8;
  transition: none;
}

.ui.inverted.basic.labels .label,
.ui.inverted.basic.label,
.ui.inverted.basic.label:hover {
  border-color: rgba(255, 255, 255, 0.5);
  background: #1b1c1d;
}

.ui.inverted.basic.label:hover {
  color: #4183c4;
}

/* -------------------
       Colors
-------------------- */

.ui.primary.labels .label,
.ui.ui.ui.primary.label {
  background-color: #2185d0;
  border-color: #2185d0;
  color: rgba(255, 255, 255, 0.9);
}

/* Link */

.ui.primary.labels a.label:hover,
a.ui.ui.ui.primary.label:hover {
  background-color: #1678c2;
  border-color: #1678c2;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.primary.ribbon.label {
  border-color: #1a69a4;
}

/* Basic */

.ui.basic.labels .primary.label,
.ui.ui.ui.basic.primary.label {
  background: none #fff;
  border-color: #2185d0;
  color: #2185d0;
}

.ui.basic.labels a.primary.label:hover,
a.ui.ui.ui.basic.primary.label:hover {
  background: none #fff;
  border-color: #1678c2;
  color: #1678c2;
}

/* Inverted */

.ui.inverted.labels .primary.label,
.ui.ui.ui.inverted.primary.label {
  background-color: #54c8ff;
  border-color: #54c8ff;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.primary.label:hover,
a.ui.ui.ui.inverted.primary.label:hover {
  background-color: #21b8ff;
  border-color: #21b8ff;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.primary.ribbon.label {
  border-color: #21b8ff;
}

/* Inverted Basic */

.ui.inverted.basic.labels .primary.label,
.ui.ui.ui.inverted.basic.primary.label {
  background-color: #1b1c1d;
  border-color: #54c8ff;
  color: #54c8ff;
}

.ui.inverted.basic.labels a.primary.label:hover,
a.ui.ui.ui.inverted.basic.primary.label:hover {
  border-color: #21b8ff;
  background-color: #1b1c1d;
  color: #21b8ff;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .primary.label,
.ui.ui.ui.inverted.primary.basic.tag.label {
  border: 1px solid #54c8ff;
}

.ui.inverted.basic.tag.labels .primary.label::before,
.ui.ui.ui.inverted.primary.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.secondary.labels .label,
.ui.ui.ui.secondary.label {
  background-color: #1b1c1d;
  border-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

/* Link */

.ui.secondary.labels a.label:hover,
a.ui.ui.ui.secondary.label:hover {
  background-color: #27292a;
  border-color: #27292a;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.secondary.ribbon.label {
  border-color: #020203;
}

/* Basic */

.ui.basic.labels .secondary.label,
.ui.ui.ui.basic.secondary.label {
  background: none #fff;
  border-color: #1b1c1d;
  color: #1b1c1d;
}

.ui.basic.labels a.secondary.label:hover,
a.ui.ui.ui.basic.secondary.label:hover {
  background: none #fff;
  border-color: #27292a;
  color: #27292a;
}

/* Inverted */

.ui.inverted.labels .secondary.label,
.ui.ui.ui.inverted.secondary.label {
  background-color: #545454;
  border-color: #545454;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.secondary.label:hover,
a.ui.ui.ui.inverted.secondary.label:hover {
  background-color: #6e6e6e;
  border-color: #6e6e6e;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.secondary.ribbon.label {
  border-color: #3b3b3b;
}

/* Inverted Basic */

.ui.inverted.basic.labels .secondary.label,
.ui.ui.ui.inverted.basic.secondary.label {
  background-color: #1b1c1d;
  border-color: #545454;
  color: #545454;
}

.ui.inverted.basic.labels a.secondary.label:hover,
a.ui.ui.ui.inverted.basic.secondary.label:hover {
  border-color: #6e6e6e;
  background-color: #1b1c1d;
  color: #6e6e6e;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .secondary.label,
.ui.ui.ui.inverted.secondary.basic.tag.label {
  border: 1px solid #545454;
}

.ui.inverted.basic.tag.labels .secondary.label::before,
.ui.ui.ui.inverted.secondary.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.red.labels .label,
.ui.ui.ui.red.label {
  background-color: #db2828;
  border-color: #db2828;
  color: #fff;
}

/* Link */

.ui.red.labels a.label:hover,
a.ui.ui.ui.red.label:hover {
  background-color: #d01919;
  border-color: #d01919;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.red.ribbon.label {
  border-color: #b21e1e;
}

/* Basic */

.ui.basic.labels .red.label,
.ui.ui.ui.basic.red.label {
  background: none #fff;
  border-color: #db2828;
  color: #db2828;
}

.ui.basic.labels a.red.label:hover,
a.ui.ui.ui.basic.red.label:hover {
  background: none #fff;
  border-color: #d01919;
  color: #d01919;
}

/* Inverted */

.ui.inverted.labels .red.label,
.ui.ui.ui.inverted.red.label {
  background-color: #ff695e;
  border-color: #ff695e;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.red.label:hover,
a.ui.ui.ui.inverted.red.label:hover {
  background-color: #ff392b;
  border-color: #ff392b;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.red.ribbon.label {
  border-color: #ff392b;
}

/* Inverted Basic */

.ui.inverted.basic.labels .red.label,
.ui.ui.ui.inverted.basic.red.label {
  background-color: #1b1c1d;
  border-color: #ff695e;
  color: #ff695e;
}

.ui.inverted.basic.labels a.red.label:hover,
a.ui.ui.ui.inverted.basic.red.label:hover {
  border-color: #ff392b;
  background-color: #1b1c1d;
  color: #ff392b;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .red.label,
.ui.ui.ui.inverted.red.basic.tag.label {
  border: 1px solid #ff695e;
}

.ui.inverted.basic.tag.labels .red.label::before,
.ui.ui.ui.inverted.red.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.orange.labels .label,
.ui.ui.ui.orange.label {
  background-color: #f2711c;
  border-color: #f2711c;
  color: #fff;
}

/* Link */

.ui.orange.labels a.label:hover,
a.ui.ui.ui.orange.label:hover {
  background-color: #f26202;
  border-color: #f26202;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.orange.ribbon.label {
  border-color: #cf590c;
}

/* Basic */

.ui.basic.labels .orange.label,
.ui.ui.ui.basic.orange.label {
  background: none #fff;
  border-color: #f2711c;
  color: #f2711c;
}

.ui.basic.labels a.orange.label:hover,
a.ui.ui.ui.basic.orange.label:hover {
  background: none #fff;
  border-color: #f26202;
  color: #f26202;
}

/* Inverted */

.ui.inverted.labels .orange.label,
.ui.ui.ui.inverted.orange.label {
  background-color: #ff851b;
  border-color: #ff851b;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.orange.label:hover,
a.ui.ui.ui.inverted.orange.label:hover {
  background-color: #e76b00;
  border-color: #e76b00;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.orange.ribbon.label {
  border-color: #e76b00;
}

/* Inverted Basic */

.ui.inverted.basic.labels .orange.label,
.ui.ui.ui.inverted.basic.orange.label {
  background-color: #1b1c1d;
  border-color: #ff851b;
  color: #ff851b;
}

.ui.inverted.basic.labels a.orange.label:hover,
a.ui.ui.ui.inverted.basic.orange.label:hover {
  border-color: #e76b00;
  background-color: #1b1c1d;
  color: #e76b00;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .orange.label,
.ui.ui.ui.inverted.orange.basic.tag.label {
  border: 1px solid #ff851b;
}

.ui.inverted.basic.tag.labels .orange.label::before,
.ui.ui.ui.inverted.orange.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.yellow.labels .label,
.ui.ui.ui.yellow.label {
  background-color: #fbbd08;
  border-color: #fbbd08;
  color: #fff;
}

/* Link */

.ui.yellow.labels a.label:hover,
a.ui.ui.ui.yellow.label:hover {
  background-color: #eaae00;
  border-color: #eaae00;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.yellow.ribbon.label {
  border-color: #cd9903;
}

/* Basic */

.ui.basic.labels .yellow.label,
.ui.ui.ui.basic.yellow.label {
  background: none #fff;
  border-color: #fbbd08;
  color: #fbbd08;
}

.ui.basic.labels a.yellow.label:hover,
a.ui.ui.ui.basic.yellow.label:hover {
  background: none #fff;
  border-color: #eaae00;
  color: #eaae00;
}

/* Inverted */

.ui.inverted.labels .yellow.label,
.ui.ui.ui.inverted.yellow.label {
  background-color: #ffe21f;
  border-color: #ffe21f;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.yellow.label:hover,
a.ui.ui.ui.inverted.yellow.label:hover {
  background-color: #ebcd00;
  border-color: #ebcd00;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.yellow.ribbon.label {
  border-color: #ebcd00;
}

/* Inverted Basic */

.ui.inverted.basic.labels .yellow.label,
.ui.ui.ui.inverted.basic.yellow.label {
  background-color: #1b1c1d;
  border-color: #ffe21f;
  color: #ffe21f;
}

.ui.inverted.basic.labels a.yellow.label:hover,
a.ui.ui.ui.inverted.basic.yellow.label:hover {
  border-color: #ebcd00;
  background-color: #1b1c1d;
  color: #ebcd00;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .yellow.label,
.ui.ui.ui.inverted.yellow.basic.tag.label {
  border: 1px solid #ffe21f;
}

.ui.inverted.basic.tag.labels .yellow.label::before,
.ui.ui.ui.inverted.yellow.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.olive.labels .label,
.ui.ui.ui.olive.label {
  background-color: #b5cc18;
  border-color: #b5cc18;
  color: #fff;
}

/* Link */

.ui.olive.labels a.label:hover,
a.ui.ui.ui.olive.label:hover {
  background-color: #a7bd0d;
  border-color: #a7bd0d;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.olive.ribbon.label {
  border-color: #8d9e13;
}

/* Basic */

.ui.basic.labels .olive.label,
.ui.ui.ui.basic.olive.label {
  background: none #fff;
  border-color: #b5cc18;
  color: #b5cc18;
}

.ui.basic.labels a.olive.label:hover,
a.ui.ui.ui.basic.olive.label:hover {
  background: none #fff;
  border-color: #a7bd0d;
  color: #a7bd0d;
}

/* Inverted */

.ui.inverted.labels .olive.label,
.ui.ui.ui.inverted.olive.label {
  background-color: #d9e778;
  border-color: #d9e778;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.olive.label:hover,
a.ui.ui.ui.inverted.olive.label:hover {
  background-color: #d2e745;
  border-color: #d2e745;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.olive.ribbon.label {
  border-color: #cddf4d;
}

/* Inverted Basic */

.ui.inverted.basic.labels .olive.label,
.ui.ui.ui.inverted.basic.olive.label {
  background-color: #1b1c1d;
  border-color: #d9e778;
  color: #d9e778;
}

.ui.inverted.basic.labels a.olive.label:hover,
a.ui.ui.ui.inverted.basic.olive.label:hover {
  border-color: #d2e745;
  background-color: #1b1c1d;
  color: #d2e745;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .olive.label,
.ui.ui.ui.inverted.olive.basic.tag.label {
  border: 1px solid #d9e778;
}

.ui.inverted.basic.tag.labels .olive.label::before,
.ui.ui.ui.inverted.olive.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.green.labels .label,
.ui.ui.ui.green.label {
  background-color: #21ba45;
  border-color: #21ba45;
  color: #fff;
}

/* Link */

.ui.green.labels a.label:hover,
a.ui.ui.ui.green.label:hover {
  background-color: #16ab39;
  border-color: #16ab39;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.green.ribbon.label {
  border-color: #198f35;
}

/* Basic */

.ui.basic.labels .green.label,
.ui.ui.ui.basic.green.label {
  background: none #fff;
  border-color: #21ba45;
  color: #21ba45;
}

.ui.basic.labels a.green.label:hover,
a.ui.ui.ui.basic.green.label:hover {
  background: none #fff;
  border-color: #16ab39;
  color: #16ab39;
}

/* Inverted */

.ui.inverted.labels .green.label,
.ui.ui.ui.inverted.green.label {
  background-color: #2ecc40;
  border-color: #2ecc40;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.green.label:hover,
a.ui.ui.ui.inverted.green.label:hover {
  background-color: #1ea92e;
  border-color: #1ea92e;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.green.ribbon.label {
  border-color: #25a233;
}

/* Inverted Basic */

.ui.inverted.basic.labels .green.label,
.ui.ui.ui.inverted.basic.green.label {
  background-color: #1b1c1d;
  border-color: #2ecc40;
  color: #2ecc40;
}

.ui.inverted.basic.labels a.green.label:hover,
a.ui.ui.ui.inverted.basic.green.label:hover {
  border-color: #1ea92e;
  background-color: #1b1c1d;
  color: #1ea92e;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .green.label,
.ui.ui.ui.inverted.green.basic.tag.label {
  border: 1px solid #2ecc40;
}

.ui.inverted.basic.tag.labels .green.label::before,
.ui.ui.ui.inverted.green.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.teal.labels .label,
.ui.ui.ui.teal.label {
  background-color: #00b5ad;
  border-color: #00b5ad;
  color: #fff;
}

/* Link */

.ui.teal.labels a.label:hover,
a.ui.ui.ui.teal.label:hover {
  background-color: #009c95;
  border-color: #009c95;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.teal.ribbon.label {
  border-color: #00827c;
}

/* Basic */

.ui.basic.labels .teal.label,
.ui.ui.ui.basic.teal.label {
  background: none #fff;
  border-color: #00b5ad;
  color: #00b5ad;
}

.ui.basic.labels a.teal.label:hover,
a.ui.ui.ui.basic.teal.label:hover {
  background: none #fff;
  border-color: #009c95;
  color: #009c95;
}

/* Inverted */

.ui.inverted.labels .teal.label,
.ui.ui.ui.inverted.teal.label {
  background-color: #6dffff;
  border-color: #6dffff;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.teal.label:hover,
a.ui.ui.ui.inverted.teal.label:hover {
  background-color: #3affff;
  border-color: #3affff;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.teal.ribbon.label {
  border-color: #3affff;
}

/* Inverted Basic */

.ui.inverted.basic.labels .teal.label,
.ui.ui.ui.inverted.basic.teal.label {
  background-color: #1b1c1d;
  border-color: #6dffff;
  color: #6dffff;
}

.ui.inverted.basic.labels a.teal.label:hover,
a.ui.ui.ui.inverted.basic.teal.label:hover {
  border-color: #3affff;
  background-color: #1b1c1d;
  color: #3affff;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .teal.label,
.ui.ui.ui.inverted.teal.basic.tag.label {
  border: 1px solid #6dffff;
}

.ui.inverted.basic.tag.labels .teal.label::before,
.ui.ui.ui.inverted.teal.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.blue.labels .label,
.ui.ui.ui.blue.label {
  background-color: #2185d0;
  border-color: #2185d0;
  color: #fff;
}

/* Link */

.ui.blue.labels a.label:hover,
a.ui.ui.ui.blue.label:hover {
  background-color: #1678c2;
  border-color: #1678c2;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.blue.ribbon.label {
  border-color: #1a69a4;
}

/* Basic */

.ui.basic.labels .blue.label,
.ui.ui.ui.basic.blue.label {
  background: none #fff;
  border-color: #2185d0;
  color: #2185d0;
}

.ui.basic.labels a.blue.label:hover,
a.ui.ui.ui.basic.blue.label:hover {
  background: none #fff;
  border-color: #1678c2;
  color: #1678c2;
}

/* Inverted */

.ui.inverted.labels .blue.label,
.ui.ui.ui.inverted.blue.label {
  background-color: #54c8ff;
  border-color: #54c8ff;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.blue.label:hover,
a.ui.ui.ui.inverted.blue.label:hover {
  background-color: #21b8ff;
  border-color: #21b8ff;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.blue.ribbon.label {
  border-color: #21b8ff;
}

/* Inverted Basic */

.ui.inverted.basic.labels .blue.label,
.ui.ui.ui.inverted.basic.blue.label {
  background-color: #1b1c1d;
  border-color: #54c8ff;
  color: #54c8ff;
}

.ui.inverted.basic.labels a.blue.label:hover,
a.ui.ui.ui.inverted.basic.blue.label:hover {
  border-color: #21b8ff;
  background-color: #1b1c1d;
  color: #21b8ff;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .blue.label,
.ui.ui.ui.inverted.blue.basic.tag.label {
  border: 1px solid #54c8ff;
}

.ui.inverted.basic.tag.labels .blue.label::before,
.ui.ui.ui.inverted.blue.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.violet.labels .label,
.ui.ui.ui.violet.label {
  background-color: #6435c9;
  border-color: #6435c9;
  color: #fff;
}

/* Link */

.ui.violet.labels a.label:hover,
a.ui.ui.ui.violet.label:hover {
  background-color: #5829bb;
  border-color: #5829bb;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.violet.ribbon.label {
  border-color: #502aa1;
}

/* Basic */

.ui.basic.labels .violet.label,
.ui.ui.ui.basic.violet.label {
  background: none #fff;
  border-color: #6435c9;
  color: #6435c9;
}

.ui.basic.labels a.violet.label:hover,
a.ui.ui.ui.basic.violet.label:hover {
  background: none #fff;
  border-color: #5829bb;
  color: #5829bb;
}

/* Inverted */

.ui.inverted.labels .violet.label,
.ui.ui.ui.inverted.violet.label {
  background-color: #a291fb;
  border-color: #a291fb;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.violet.label:hover,
a.ui.ui.ui.inverted.violet.label:hover {
  background-color: #745aff;
  border-color: #745aff;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.violet.ribbon.label {
  border-color: #7860f9;
}

/* Inverted Basic */

.ui.inverted.basic.labels .violet.label,
.ui.ui.ui.inverted.basic.violet.label {
  background-color: #1b1c1d;
  border-color: #a291fb;
  color: #a291fb;
}

.ui.inverted.basic.labels a.violet.label:hover,
a.ui.ui.ui.inverted.basic.violet.label:hover {
  border-color: #745aff;
  background-color: #1b1c1d;
  color: #745aff;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .violet.label,
.ui.ui.ui.inverted.violet.basic.tag.label {
  border: 1px solid #a291fb;
}

.ui.inverted.basic.tag.labels .violet.label::before,
.ui.ui.ui.inverted.violet.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.purple.labels .label,
.ui.ui.ui.purple.label {
  background-color: #a333c8;
  border-color: #a333c8;
  color: #fff;
}

/* Link */

.ui.purple.labels a.label:hover,
a.ui.ui.ui.purple.label:hover {
  background-color: #9627ba;
  border-color: #9627ba;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.purple.ribbon.label {
  border-color: #82299f;
}

/* Basic */

.ui.basic.labels .purple.label,
.ui.ui.ui.basic.purple.label {
  background: none #fff;
  border-color: #a333c8;
  color: #a333c8;
}

.ui.basic.labels a.purple.label:hover,
a.ui.ui.ui.basic.purple.label:hover {
  background: none #fff;
  border-color: #9627ba;
  color: #9627ba;
}

/* Inverted */

.ui.inverted.labels .purple.label,
.ui.ui.ui.inverted.purple.label {
  background-color: #dc73ff;
  border-color: #dc73ff;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.purple.label:hover,
a.ui.ui.ui.inverted.purple.label:hover {
  background-color: #cf40ff;
  border-color: #cf40ff;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.purple.ribbon.label {
  border-color: #cf40ff;
}

/* Inverted Basic */

.ui.inverted.basic.labels .purple.label,
.ui.ui.ui.inverted.basic.purple.label {
  background-color: #1b1c1d;
  border-color: #dc73ff;
  color: #dc73ff;
}

.ui.inverted.basic.labels a.purple.label:hover,
a.ui.ui.ui.inverted.basic.purple.label:hover {
  border-color: #cf40ff;
  background-color: #1b1c1d;
  color: #cf40ff;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .purple.label,
.ui.ui.ui.inverted.purple.basic.tag.label {
  border: 1px solid #dc73ff;
}

.ui.inverted.basic.tag.labels .purple.label::before,
.ui.ui.ui.inverted.purple.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.pink.labels .label,
.ui.ui.ui.pink.label {
  background-color: #e03997;
  border-color: #e03997;
  color: #fff;
}

/* Link */

.ui.pink.labels a.label:hover,
a.ui.ui.ui.pink.label:hover {
  background-color: #e61a8d;
  border-color: #e61a8d;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.pink.ribbon.label {
  border-color: #c71f7e;
}

/* Basic */

.ui.basic.labels .pink.label,
.ui.ui.ui.basic.pink.label {
  background: none #fff;
  border-color: #e03997;
  color: #e03997;
}

.ui.basic.labels a.pink.label:hover,
a.ui.ui.ui.basic.pink.label:hover {
  background: none #fff;
  border-color: #e61a8d;
  color: #e61a8d;
}

/* Inverted */

.ui.inverted.labels .pink.label,
.ui.ui.ui.inverted.pink.label {
  background-color: #ff8edf;
  border-color: #ff8edf;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.pink.label:hover,
a.ui.ui.ui.inverted.pink.label:hover {
  background-color: #ff5bd1;
  border-color: #ff5bd1;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.pink.ribbon.label {
  border-color: #ff5bd1;
}

/* Inverted Basic */

.ui.inverted.basic.labels .pink.label,
.ui.ui.ui.inverted.basic.pink.label {
  background-color: #1b1c1d;
  border-color: #ff8edf;
  color: #ff8edf;
}

.ui.inverted.basic.labels a.pink.label:hover,
a.ui.ui.ui.inverted.basic.pink.label:hover {
  border-color: #ff5bd1;
  background-color: #1b1c1d;
  color: #ff5bd1;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .pink.label,
.ui.ui.ui.inverted.pink.basic.tag.label {
  border: 1px solid #ff8edf;
}

.ui.inverted.basic.tag.labels .pink.label::before,
.ui.ui.ui.inverted.pink.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.brown.labels .label,
.ui.ui.ui.brown.label {
  background-color: #a5673f;
  border-color: #a5673f;
  color: #fff;
}

/* Link */

.ui.brown.labels a.label:hover,
a.ui.ui.ui.brown.label:hover {
  background-color: #975b33;
  border-color: #975b33;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.brown.ribbon.label {
  border-color: #805031;
}

/* Basic */

.ui.basic.labels .brown.label,
.ui.ui.ui.basic.brown.label {
  background: none #fff;
  border-color: #a5673f;
  color: #a5673f;
}

.ui.basic.labels a.brown.label:hover,
a.ui.ui.ui.basic.brown.label:hover {
  background: none #fff;
  border-color: #975b33;
  color: #975b33;
}

/* Inverted */

.ui.inverted.labels .brown.label,
.ui.ui.ui.inverted.brown.label {
  background-color: #d67c1c;
  border-color: #d67c1c;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.brown.label:hover,
a.ui.ui.ui.inverted.brown.label:hover {
  background-color: #b0620f;
  border-color: #b0620f;
  color: #1b1c1d;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.brown.ribbon.label {
  border-color: #a96216;
}

/* Inverted Basic */

.ui.inverted.basic.labels .brown.label,
.ui.ui.ui.inverted.basic.brown.label {
  background-color: #1b1c1d;
  border-color: #d67c1c;
  color: #d67c1c;
}

.ui.inverted.basic.labels a.brown.label:hover,
a.ui.ui.ui.inverted.basic.brown.label:hover {
  border-color: #b0620f;
  background-color: #1b1c1d;
  color: #b0620f;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .brown.label,
.ui.ui.ui.inverted.brown.basic.tag.label {
  border: 1px solid #d67c1c;
}

.ui.inverted.basic.tag.labels .brown.label::before,
.ui.ui.ui.inverted.brown.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.grey.labels .label,
.ui.ui.ui.grey.label {
  background-color: #767676;
  border-color: #767676;
  color: #fff;
}

/* Link */

.ui.grey.labels a.label:hover,
a.ui.ui.ui.grey.label:hover {
  background-color: #838383;
  border-color: #838383;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.grey.ribbon.label {
  border-color: #5d5d5d;
}

/* Basic */

.ui.basic.labels .grey.label,
.ui.ui.ui.basic.grey.label {
  background: none #fff;
  border-color: #767676;
  color: #767676;
}

.ui.basic.labels a.grey.label:hover,
a.ui.ui.ui.basic.grey.label:hover {
  background: none #fff;
  border-color: #838383;
  color: #838383;
}

/* Inverted */

.ui.inverted.labels .grey.label,
.ui.ui.ui.inverted.grey.label {
  background-color: #dcddde;
  border-color: #dcddde;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.grey.label:hover,
a.ui.ui.ui.inverted.grey.label:hover {
  background-color: #c2c4c5;
  border-color: #c2c4c5;
  color: #fff;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.grey.ribbon.label {
  border-color: #e9eaea;
}

/* Inverted Basic */

.ui.inverted.basic.labels .grey.label,
.ui.ui.ui.inverted.basic.grey.label {
  background-color: #1b1c1d;
  border-color: #dcddde;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.basic.labels a.grey.label:hover,
a.ui.ui.ui.inverted.basic.grey.label:hover {
  border-color: #c2c4c5;
  background-color: #1b1c1d;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .grey.label,
.ui.ui.ui.inverted.grey.basic.tag.label {
  border: 1px solid #dcddde;
}

.ui.inverted.basic.tag.labels .grey.label::before,
.ui.ui.ui.inverted.grey.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

.ui.black.labels .label,
.ui.ui.ui.black.label {
  background-color: #1b1c1d;
  border-color: #1b1c1d;
  color: #fff;
}

/* Link */

.ui.black.labels a.label:hover,
a.ui.ui.ui.black.label:hover {
  background-color: #27292a;
  border-color: #27292a;
  color: #fff;
}

/* Ribbon */

.ui.ui.ui.black.ribbon.label {
  border-color: #020203;
}

/* Basic */

.ui.basic.labels .black.label,
.ui.ui.ui.basic.black.label {
  background: none #fff;
  border-color: #1b1c1d;
  color: #1b1c1d;
}

.ui.basic.labels a.black.label:hover,
a.ui.ui.ui.basic.black.label:hover {
  background: none #fff;
  border-color: #27292a;
  color: #27292a;
}

/* Inverted */

.ui.inverted.labels .black.label,
.ui.ui.ui.inverted.black.label {
  background-color: #545454;
  border-color: #545454;
  color: #1b1c1d;
}

/* Inverted Link */

.ui.inverted.labels a.black.label:hover,
a.ui.ui.ui.inverted.black.label:hover {
  background-color: #000000;
  border-color: #000000;
  color: #fff;
}

/* Inverted Ribbon */

.ui.ui.ui.inverted.black.ribbon.label {
  border-color: #616161;
}

/* Inverted Basic */

.ui.inverted.basic.labels .black.label,
.ui.ui.ui.inverted.basic.black.label {
  background-color: #1b1c1d;
  border-color: #545454;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.basic.labels a.black.label:hover,
a.ui.ui.ui.inverted.basic.black.label:hover {
  border-color: #000000;
  background-color: #1b1c1d;
}

/* Inverted Basic Tags */

.ui.inverted.basic.tag.labels .black.label,
.ui.ui.ui.inverted.black.basic.tag.label {
  border: 1px solid #545454;
}

.ui.inverted.basic.tag.labels .black.label::before,
.ui.ui.ui.inverted.black.basic.tag.label::before {
  border-color: inherit;
  border-width: 1px 0 0 1px;
  border-style: inherit;
  background-color: #1b1c1d;
  right: calc(100% + 1px);
}

/* -------------------
     Horizontal
-------------------- */

.ui.horizontal.labels .label,
.ui.horizontal.label {
  margin: 0 0.5em 0 0;
  padding: 0.4em 0.833em;
  min-width: 3em;
  text-align: center;
}

/* -------------------
           Circular
    -------------------- */

.ui.circular.labels .label,
.ui.circular.label {
  min-width: 2em;
  min-height: 2em;
  padding: 0.5em !important;
  line-height: 1em;
  text-align: center;
  border-radius: 500rem;
}

.ui.empty.circular.labels .label,
.ui.empty.circular.label {
  min-width: 0;
  min-height: 0;
  overflow: hidden;
  width: 0.5em;
  height: 0.5em;
  vertical-align: baseline;
}

/* -------------------
           Pointing
    -------------------- */

.ui.pointing.label {
  position: relative;
}

.ui.attached.pointing.label {
  position: absolute;
}

.ui.pointing.label::before {
  background-color: inherit;
  background-image: inherit;
  border-width: 0;
  border-style: solid;
  border-color: inherit;
}

/* Arrow */

.ui.pointing.label::before {
  position: absolute;
  content: "";
  transform: rotate(45deg);
  background-image: none;
  z-index: 2;
  width: 0.6666em;
  height: 0.6666em;
  transition: none;
}

/* --- Above --- */

.ui.pointing.label,
.ui[class*="pointing above"].label {
  margin-top: 1em;
}

.ui.pointing.label::before,
.ui[class*="pointing above"].label::before {
  border-width: 1px 0 0 1px;
  transform: translateX(-50%) translateY(-50%) rotate(45deg);
  top: 0;
  left: 50%;
}

/* --- Below --- */

.ui[class*="bottom pointing"].label,
.ui[class*="pointing below"].label {
  margin-top: 0;
  margin-bottom: 1em;
}

.ui[class*="bottom pointing"].label::before,
.ui[class*="pointing below"].label::before {
  border-width: 0 1px 1px 0;
  right: auto;
  transform: translateX(-50%) translateY(-50%) rotate(45deg);
  top: 100%;
  left: 50%;
}

/* --- Left --- */

.ui[class*="left pointing"].label {
  margin-top: 0;
  margin-left: 0.6666em;
}

.ui[class*="left pointing"].label::before {
  border-width: 0 0 1px 1px;
  transform: translateX(-50%) translateY(-50%) rotate(45deg);
  bottom: auto;
  right: auto;
  top: 50%;
  left: 0;
}

/* --- Right --- */

.ui[class*="right pointing"].label {
  margin-top: 0;
  margin-right: 0.6666em;
}

.ui[class*="right pointing"].label::before {
  border-width: 1px 1px 0 0;
  transform: translateX(50%) translateY(-50%) rotate(45deg);
  top: 50%;
  right: 0;
  bottom: auto;
  left: auto;
}

/* Basic Pointing */

/* --- Above --- */

.ui.basic.pointing.label::before,
.ui.basic[class*="pointing above"].label::before {
  margin-top: -1px;
}

/* --- Below --- */

.ui.basic[class*="bottom pointing"].label::before,
.ui.basic[class*="pointing below"].label::before {
  bottom: auto;
  top: 100%;
  margin-top: 1px;
}

/* --- Left --- */

.ui.basic[class*="left pointing"].label::before {
  top: 50%;
  left: -1px;
}

/* --- Right --- */

.ui.basic[class*="right pointing"].label::before {
  top: 50%;
  right: -1px;
}

/* ------------------
       Floating Label
    ------------------- */

.ui.floating.label {
  position: absolute;
  z-index: 100;
  top: -1em;
  right: 0;
  white-space: nowrap;
  transform: translateX(50%);
}

.ui.right.aligned.floating.label {
  transform: translateX(1.2em);
}

.ui.left.floating.label {
  left: 0;
  right: auto;
  transform: translateX(-50%);
}

.ui.left.aligned.floating.label {
  transform: translateX(-1.2em);
}

.ui.bottom.floating.label {
  top: auto;
  bottom: -1em;
}

/* -------------------
        Sizes
-------------------- */

.ui.labels .label,
.ui.label {
  font-size: 0.85714286rem;
}

.ui.mini.labels .label,
.ui.mini.label {
  font-size: 0.64285714rem;
}

.ui.tiny.labels .label,
.ui.tiny.label {
  font-size: 0.71428571rem;
}

.ui.small.labels .label,
.ui.small.label {
  font-size: 0.78571429rem;
}

.ui.large.labels .label,
.ui.large.label {
  font-size: 1rem;
}

.ui.big.labels .label,
.ui.big.label {
  font-size: 1.28571429rem;
}

.ui.huge.labels .label,
.ui.huge.label {
  font-size: 1.42857143rem;
}

.ui.massive.labels .label,
.ui.massive.label {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - List
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            List
*******************************/

ul.ui.list,
ol.ui.list,
.ui.list {
  list-style-type: none;
  margin: 1em 0;
  padding: 0 0;
}

ul.ui.list:first-child,
ol.ui.list:first-child,
.ui.list:first-child {
  margin-top: 0;
  padding-top: 0;
}

ul.ui.list:last-child,
ol.ui.list:last-child,
.ui.list:last-child {
  margin-bottom: 0;
  padding-bottom: 0;
}

/*******************************
            Content
*******************************/

/* List Item */

ul.ui.list li,
ol.ui.list li,
.ui.list > .item,
.ui.list .list > .item {
  display: list-item;
  table-layout: fixed;
  list-style-type: none;
  list-style-position: outside;
  padding: 0.21428571em 0;
  line-height: 1.14285714em;
}

ul.ui.list > li:first-child::after,
ol.ui.list > li:first-child::after,
.ui.list > .list > .item::after,
.ui.list > .item::after {
  content: "";
  display: block;
  height: 0;
  clear: both;
  visibility: hidden;
}

ul.ui.list li:first-child,
ol.ui.list li:first-child,
.ui.list .list > .item:first-child,
.ui.list > .item:first-child {
  padding-top: 0;
}

ul.ui.list li:last-child,
ol.ui.list li:last-child,
.ui.list .list > .item:last-child,
.ui.list > .item:last-child {
  padding-bottom: 0;
}

/* Child List */

ul.ui.list ul,
ol.ui.list ol,
.ui.list .list:not(.icon) {
  clear: both;
  margin: 0;
  padding: 0.75em 0 0.25em 0.5em;
}

/* Child Item */

ul.ui.list ul li,
ol.ui.list ol li,
.ui.list .list > .item {
  padding: 0.14285714em 0;
  line-height: inherit;
}

/* Icon */

.ui.list .list > .item > i.icon,
.ui.list > .item > i.icon {
  display: table-cell;
  min-width: 1.55em;
  margin: 0;
  padding-top: 0;
  transition: color 0.1s ease;
}

.ui.list .list > .item > i.icon:not(.loading),
.ui.list > .item > i.icon:not(.loading) {
  padding-right: 0.28571429em;
  vertical-align: top;
}

.ui.list .list > .item > i.icon:only-child,
.ui.list > .item > i.icon:only-child {
  display: inline-block;
  min-width: auto;
  vertical-align: top;
}

/* Image */

.ui.list .list > .item > .image,
.ui.list > .item > .image {
  display: table-cell;
  background-color: transparent;
  margin: 0;
  vertical-align: top;
}

.ui.list .list > .item > .image:not(:only-child):not(img),
.ui.list > .item > .image:not(:only-child):not(img) {
  padding-right: 0.5em;
}

.ui.list .list > .item > .image img,
.ui.list > .item > .image img {
  vertical-align: top;
}

.ui.list .list > .item > img.image,
.ui.list .list > .item > .image:only-child,
.ui.list > .item > img.image,
.ui.list > .item > .image:only-child {
  display: inline-block;
}

/* Content */

.ui.list .list > .item > .content,
.ui.list > .item > .content {
  line-height: 1.14285714em;
  color: rgba(0, 0, 0, 0.87);
}

.ui.list .list > .item > .image + .content,
.ui.list .list > .item > i.icon + .content,
.ui.list > .item > .image + .content,
.ui.list > .item > i.icon + .content {
  display: table-cell;
  width: 100%;
  padding: 0 0 0 0.5em;
  vertical-align: top;
}

.ui.list .list > .item > i.loading.icon + .content,
.ui.list > .item > i.loading.icon + .content {
  padding-left: calc(0.2857142857142857em + 0.5em);
}

.ui.list .list > .item > img.image + .content,
.ui.list > .item > img.image + .content {
  display: inline-block;
  width: auto;
}

.ui.list .list > .item > .content > .list,
.ui.list > .item > .content > .list {
  margin-left: 0;
  padding-left: 0;
}

/* Header */

.ui.list .list > .item .header,
.ui.list > .item .header {
  display: block;
  margin: 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-weight: bold;
  color: rgba(0, 0, 0, 0.87);
}

/* Description */

.ui.list .list > .item .description,
.ui.list > .item .description {
  display: block;
  color: rgba(0, 0, 0, 0.7);
}

/* Child Link */

.ui.list > .item a,
.ui.list .list > .item a {
  cursor: pointer;
}

/* Linking Item */

.ui.list .list > a.item,
.ui.list > a.item {
  cursor: pointer;
  color: #4183c4;
}

.ui.list .list > a.item:hover,
.ui.list > a.item:hover {
  color: #1e70bf;
}

/* Linked Item Icons */

.ui.list .list > a.item > i.icons,
.ui.list > a.item > i.icons,
.ui.list .list > a.item > i.icon,
.ui.list > a.item > i.icon {
  color: rgba(0, 0, 0, 0.4);
}

/* Header Link */

.ui.list .list > .item a.header,
.ui.list > .item a.header {
  cursor: pointer;
  color: #4183c4 !important;
}

.ui.list .list > .item > a.header:hover,
.ui.list > .item > a.header:hover {
  color: #1e70bf !important;
}

/* Floated Content */

.ui[class*="left floated"].list {
  float: left;
}

.ui[class*="right floated"].list {
  float: right;
}

.ui.list .list > .item [class*="left floated"],
.ui.list > .item [class*="left floated"] {
  float: left;
  margin: 0 1em 0 0;
}

.ui.list .list > .item [class*="right floated"],
.ui.list > .item [class*="right floated"] {
  float: right;
  margin: 0 0 0 1em;
}

/*******************************
            Coupling
*******************************/

.ui.menu .ui.list > .item,
.ui.menu .ui.list .list > .item {
  display: list-item;
  table-layout: fixed;
  background-color: transparent;
  list-style-type: none;
  list-style-position: outside;
  padding: 0.21428571em 0;
  line-height: 1.14285714em;
}

.ui.menu .ui.list .list > .item::before,
.ui.menu .ui.list > .item::before {
  border: none;
  background: none;
}

.ui.menu .ui.list .list > .item:first-child,
.ui.menu .ui.list > .item:first-child {
  padding-top: 0;
}

.ui.menu .ui.list .list > .item:last-child,
.ui.menu .ui.list > .item:last-child {
  padding-bottom: 0;
}

/*******************************
            Types
*******************************/

/* -------------------
          Horizontal
    -------------------- */

.ui.horizontal.list {
  display: inline-block;
  font-size: 0;
}

.ui.horizontal.list > .item {
  display: inline-block;
  margin-right: 1em;
  font-size: 1rem;
}

.ui.horizontal.list:not(.celled) > .item:last-child {
  margin-right: 0;
  padding-right: 0;
}

.ui.horizontal.list .list:not(.icon) {
  padding-left: 0;
  padding-bottom: 0;
}

.ui.horizontal.list > .item > .image,
.ui.horizontal.list .list > .item > .image,
.ui.horizontal.list > .item > i.icon,
.ui.horizontal.list .list > .item > i.icon,
.ui.horizontal.list > .item > .content,
.ui.horizontal.list .list > .item > .content {
  vertical-align: middle;
}

/* Padding on all elements */

.ui.horizontal.list > .item:first-child,
.ui.horizontal.list > .item:last-child {
  padding-top: 0.21428571em;
  padding-bottom: 0.21428571em;
}

/* Horizontal List */

.ui.horizontal.list > .item > i.icon,
.ui.horizontal.list .item > i.icons > i.icon {
  margin: 0;
  padding: 0 0.25em 0 0;
}

.ui.horizontal.list > .item > .image + .content,
.ui.horizontal.list > .item > i.icon,
.ui.horizontal.list > .item > i.icon + .content {
  float: none;
  display: inline-block;
  width: auto;
}

.ui.horizontal.list > .item > .image {
  display: inline-block;
}

/*******************************
             States
*******************************/

/* -------------------
           Disabled
    -------------------- */

.ui.list .list > .disabled.item,
.ui.list > .disabled.item {
  pointer-events: none;
  color: rgba(40, 40, 40, 0.3) !important;
}

.ui.inverted.list .list > .disabled.item,
.ui.inverted.list > .disabled.item {
  color: rgba(225, 225, 225, 0.3) !important;
}

/* -------------------
        Hover
-------------------- */

.ui.list .list > a.item:hover > .icons,
.ui.list > a.item:hover > .icons,
.ui.list .list > a.item:hover > i.icon,
.ui.list > a.item:hover > i.icon {
  color: rgba(0, 0, 0, 0.87);
}

/*******************************
           Variations
*******************************/

/* -------------------
           Inverted
    -------------------- */

.ui.inverted.list .list > a.item > i.icon,
.ui.inverted.list > a.item > i.icon {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.list .list > .item .header,
.ui.inverted.list > .item .header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.list .list > .item .description,
.ui.inverted.list > .item .description {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.list .list > .item > .content,
.ui.inverted.list > .item > .content {
  color: rgba(255, 255, 255, 0.7);
}

/* Item Link */

.ui.inverted.list .list > a.item,
.ui.inverted.list > a.item {
  cursor: pointer;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.list .list > a.item:hover,
.ui.inverted.list > a.item:hover {
  color: #1e70bf;
}

/* Linking Content */

.ui.inverted.list .item a:not(.ui) {
  color: rgba(255, 255, 255, 0.9) !important;
}

.ui.inverted.list .item a:not(.ui):hover {
  color: #1e70bf !important;
}

/* -------------------
           Aligned
    -------------------- */

.ui.list[class*="top aligned"] .image,
.ui.list[class*="top aligned"] .content,
.ui.list [class*="top aligned"] {
  vertical-align: top !important;
}

.ui.list[class*="middle aligned"] .image,
.ui.list[class*="middle aligned"] .content,
.ui.list [class*="middle aligned"] {
  vertical-align: middle !important;
}

.ui.list[class*="bottom aligned"] .image,
.ui.list[class*="bottom aligned"] .content,
.ui.list [class*="bottom aligned"] {
  vertical-align: bottom !important;
}

/* -------------------
           Link
    -------------------- */

.ui.link.list .item,
.ui.link.list a.item,
.ui.link.list .item a:not(.ui) {
  color: rgba(0, 0, 0, 0.4);
  transition: 0.1s color ease;
}

.ui.ui.link.list a.item:hover,
.ui.ui.link.list .item a:not(.ui):hover {
  color: rgba(0, 0, 0, 0.8);
}

.ui.ui.link.list a.item:active,
.ui.ui.link.list .item a:not(.ui):active {
  color: rgba(0, 0, 0, 0.9);
}

.ui.ui.link.list .active.item,
.ui.ui.link.list .active.item a:not(.ui) {
  color: rgba(0, 0, 0, 0.95);
}

/* Inverted */

.ui.inverted.link.list .item,
.ui.inverted.link.list a.item,
.ui.inverted.link.list .item a:not(.ui) {
  color: rgba(255, 255, 255, 0.5);
}

.ui.ui.inverted.link.list a.item:hover,
.ui.ui.inverted.link.list .item a:not(.ui):hover {
  color: #ffffff;
}

.ui.ui.inverted.link.list a.item:active,
.ui.ui.inverted.link.list .item a:not(.ui):active {
  color: #ffffff;
}

.ui.ui.inverted.link.list a.active.item,
.ui.ui.inverted.link.list .active.item a:not(.ui) {
  color: #ffffff;
}

/* -------------------
          Selection
    -------------------- */

.ui.selection.list .list > .item,
.ui.selection.list > .item {
  cursor: pointer;
  background: transparent;
  padding: 0.5em 0.5em;
  margin: 0;
  color: rgba(0, 0, 0, 0.4);
  border-radius: 0.5em;
  transition: 0.1s color ease, 0.1s padding-left ease, 0.1s background-color ease;
}

.ui.selection.list .list > .item:last-child,
.ui.selection.list > .item:last-child {
  margin-bottom: 0;
}

.ui.selection.list .list > .item:hover,
.ui.selection.list > .item:hover {
  background: rgba(0, 0, 0, 0.03);
  color: rgba(0, 0, 0, 0.8);
}

.ui.selection.list .list > .item:active,
.ui.selection.list > .item:active {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.9);
}

.ui.selection.list .list > .item.active,
.ui.selection.list > .item.active {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
}

/* Inverted */

.ui.inverted.selection.list > .item {
  background: transparent;
  color: rgba(255, 255, 255, 0.5);
}

.ui.inverted.selection.list > .item:hover {
  background: rgba(255, 255, 255, 0.02);
  color: #ffffff;
}

.ui.inverted.selection.list > .item:active {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

.ui.inverted.selection.list > .item.active {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

/* Celled / Divided Selection List */

.ui.celled.selection.list .list > .item,
.ui.divided.selection.list .list > .item,
.ui.celled.selection.list > .item,
.ui.divided.selection.list > .item {
  border-radius: 0;
}

/* -------------------
           Animated
    -------------------- */

.ui.animated.list > .item {
  transition: 0.25s color ease 0.1s, 0.25s padding-left ease 0.1s, 0.25s background-color ease 0.1s;
}

.ui.animated.list:not(.horizontal) > .item:hover {
  padding-left: 1em;
}

/* -------------------
           Fitted
    -------------------- */

.ui.fitted.list:not(.selection) .list > .item,
.ui.fitted.list:not(.selection) > .item {
  padding-left: 0;
  padding-right: 0;
}

.ui.fitted.selection.list .list > .item,
.ui.fitted.selection.list > .item {
  margin-left: -0.5em;
  margin-right: -0.5em;
}

/* -------------------
          Bulleted
    -------------------- */

ul.ui.list,
.ui.bulleted.list {
  margin-left: 1.25rem;
}

ul.ui.list li,
.ui.bulleted.list .list > .item,
.ui.bulleted.list > .item {
  position: relative;
}

ul.ui.list li::before,
.ui.bulleted.list .list > .item::before,
.ui.bulleted.list > .item::before {
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  pointer-events: none;
  position: absolute;
  top: auto;
  left: auto;
  font-weight: normal;
  margin-left: -1.25rem;
  content: "\2022";
  opacity: 1;
  color: inherit;
  vertical-align: top;
}

ul.ui.list li::before,
.ui.bulleted.list .list > a.item::before,
.ui.bulleted.list > a.item::before {
  color: rgba(0, 0, 0, 0.87);
}

ul.ui.list ul,
.ui.bulleted.list .list:not(.icon) {
  padding-left: 1.25rem;
}

/* Horizontal Bulleted */

ul.ui.horizontal.bulleted.list,
.ui.horizontal.bulleted.list {
  margin-left: 0;
}

ul.ui.horizontal.bulleted.list li,
.ui.horizontal.bulleted.list > .item {
  margin-left: 1.75rem;
}

ul.ui.horizontal.bulleted.list li:first-child,
.ui.horizontal.bulleted.list > .item:first-child {
  margin-left: 0;
}

ul.ui.horizontal.bulleted.list li::before,
.ui.horizontal.bulleted.list > .item::before {
  color: rgba(0, 0, 0, 0.87);
}

ul.ui.horizontal.bulleted.list li:first-child::before,
.ui.horizontal.bulleted.list > .item:first-child::before {
  display: none;
}

/* -------------------
           Ordered
    -------------------- */

ol.ui.list,
.ui.ordered.list,
.ui.ordered.list .list:not(.icon),
ol.ui.list ol {
  counter-reset: ordered;
  margin-left: 1.25rem;
  list-style-type: none;
}

ol.ui.list li,
.ui.ordered.list .list > .item,
.ui.ordered.list > .item {
  list-style-type: none;
  position: relative;
}

ol.ui.list li::before,
.ui.ordered.list .list > .item::before,
.ui.ordered.list > .item::before {
  position: absolute;
  top: auto;
  left: auto;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  pointer-events: none;
  margin-left: -1.25rem;
  counter-increment: ordered;
  content: counters(ordered, ".") " ";
  text-align: right;
  color: rgba(0, 0, 0, 0.87);
  vertical-align: middle;
  opacity: 0.8;
}

ol.ui.inverted.list li::before,
.ui.ordered.inverted.list .list > .item::before,
.ui.ordered.inverted.list > .item::before {
  color: rgba(255, 255, 255, 0.7);
}

/* Value */

.ui.ordered.list .list > .item[data-value]::before,
.ui.ordered.list > .item[data-value]::before {
  content: attr(data-value);
}

ol.ui.list li[value]::before {
  content: attr(value);
}

/* Child Lists */

ol.ui.list ol,
.ui.ordered.list .list:not(.icon) {
  margin-left: 1em;
}

ol.ui.list ol li::before,
.ui.ordered.list .list > .item::before {
  margin-left: -2em;
}

/* Horizontal Ordered */

ol.ui.horizontal.list,
.ui.ordered.horizontal.list {
  margin-left: 0;
}

ol.ui.horizontal.list li::before,
.ui.ordered.horizontal.list .list > .item::before,
.ui.ordered.horizontal.list > .item::before {
  position: static;
  margin: 0 0.5em 0 0;
}

/* Suffixed Ordered */

ol.ui.suffixed.list li::before,
.ui.suffixed.ordered.list .list > .item::before,
.ui.suffixed.ordered.list > .item::before {
  content: counters(ordered, ".") ".";
}

/* -------------------
           Divided
    -------------------- */

.ui.divided.list > .item {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.divided.list .list > .item {
  border-top: none;
}

.ui.divided.list .item .list > .item {
  border-top: none;
}

.ui.divided.list .list > .item:first-child,
.ui.divided.list > .item:first-child {
  border-top: none;
}

/* Sub Menu */

.ui.divided.list:not(.horizontal) .list > .item:first-child {
  border-top-width: 1px;
}

/* Divided bulleted */

.ui.divided.bulleted.list:not(.horizontal),
.ui.divided.bulleted.list .list:not(.icon) {
  margin-left: 0;
  padding-left: 0;
}

.ui.divided.bulleted.list > .item:not(.horizontal) {
  padding-left: 1.25rem;
}

/* Divided Ordered */

.ui.divided.ordered.list {
  margin-left: 0;
}

.ui.divided.ordered.list .list > .item,
.ui.divided.ordered.list > .item {
  padding-left: 1.25rem;
}

.ui.divided.ordered.list .item .list:not(.icon) {
  margin-left: 0;
  margin-right: 0;
  padding-bottom: 0.21428571em;
}

.ui.divided.ordered.list .item .list > .item {
  padding-left: 1em;
}

/* Divided Selection */

.ui.divided.selection.list .list > .item,
.ui.divided.selection.list > .item {
  margin: 0;
  border-radius: 0;
}

/* Divided horizontal */

.ui.divided.horizontal.list {
  margin-left: 0;
}

.ui.divided.horizontal.list > .item {
  padding-left: 0.5em;
}

.ui.divided.horizontal.list > .item:not(:last-child) {
  padding-right: 0.5em;
}

.ui.divided.horizontal.list > .item {
  border-top: none;
  border-right: 1px solid rgba(34, 36, 38, 0.15);
  margin: 0;
  line-height: 0.6;
}

.ui.horizontal.divided.list > .item:last-child {
  border-right: none;
}

/* Inverted */

.ui.divided.inverted.list > .item,
.ui.divided.inverted.list > .list,
.ui.divided.inverted.horizontal.list > .item {
  border-color: rgba(255, 255, 255, 0.1);
}

/* -------------------
            Celled
    -------------------- */

.ui.celled.list > .item,
.ui.celled.list > .list {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  padding-left: 0.5em;
  padding-right: 0.5em;
}

.ui.celled.list > .item:last-child {
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

/* Padding on all elements */

.ui.celled.list > .item:first-child,
.ui.celled.list > .item:last-child {
  padding-top: 0.21428571em;
  padding-bottom: 0.21428571em;
}

/* Sub Menu */

.ui.celled.list .item .list > .item {
  border-width: 0;
}

.ui.celled.list .list > .item:first-child {
  border-top-width: 0;
}

/* Celled Bulleted */

.ui.celled.bulleted.list {
  margin-left: 0;
}

.ui.celled.bulleted.list .list > .item,
.ui.celled.bulleted.list > .item {
  padding-left: 1.25rem;
}

.ui.celled.bulleted.list .item .list:not(.icon) {
  margin-left: -1.25rem;
  margin-right: -1.25rem;
  padding-bottom: 0.21428571em;
}

/* Celled Ordered */

.ui.celled.ordered.list {
  margin-left: 0;
}

.ui.celled.ordered.list .list > .item,
.ui.celled.ordered.list > .item {
  padding-left: 1.25rem;
}

.ui.celled.ordered.list .item .list:not(.icon) {
  margin-left: 0;
  margin-right: 0;
  padding-bottom: 0.21428571em;
}

.ui.celled.ordered.list .list > .item {
  padding-left: 1em;
}

/* Celled Horizontal */

.ui.horizontal.celled.list {
  margin-left: 0;
}

.ui.horizontal.celled.list .list > .item,
.ui.horizontal.celled.list > .item {
  border-top: none;
  border-left: 1px solid rgba(34, 36, 38, 0.15);
  margin: 0;
  padding-left: 0.5em;
  padding-right: 0.5em;
  line-height: 0.6;
}

.ui.horizontal.celled.list .list > .item:last-child,
.ui.horizontal.celled.list > .item:last-child {
  border-bottom: none;
  border-right: 1px solid rgba(34, 36, 38, 0.15);
}

/* Inverted */

.ui.celled.inverted.list > .item,
.ui.celled.inverted.list > .list {
  border-color: rgba(255, 255, 255, 0.1);
}

.ui.celled.inverted.horizontal.list .list > .item,
.ui.celled.inverted.horizontal.list > .item {
  border-color: rgba(255, 255, 255, 0.1);
}

/* -------------------
           Relaxed
    -------------------- */

.ui.relaxed.list:not(.horizontal) > .item:not(:first-child) {
  padding-top: 0.42857143em;
}

.ui.relaxed.list:not(.horizontal) > .item:not(:last-child) {
  padding-bottom: 0.42857143em;
}

.ui.horizontal.relaxed.list .list > .item:not(:first-child),
.ui.horizontal.relaxed.list > .item:not(:first-child) {
  padding-left: 1rem;
}

.ui.horizontal.relaxed.list .list > .item:not(:last-child),
.ui.horizontal.relaxed.list > .item:not(:last-child) {
  padding-right: 1rem;
}

/* Very Relaxed */

.ui[class*="very relaxed"].list:not(.horizontal) > .item:not(:first-child) {
  padding-top: 0.85714286em;
}

.ui[class*="very relaxed"].list:not(.horizontal) > .item:not(:last-child) {
  padding-bottom: 0.85714286em;
}

.ui.horizontal[class*="very relaxed"].list .list > .item:not(:first-child),
.ui.horizontal[class*="very relaxed"].list > .item:not(:first-child) {
  padding-left: 1.5rem;
}

.ui.horizontal[class*="very relaxed"].list .list > .item:not(:last-child),
.ui.horizontal[class*="very relaxed"].list > .item:not(:last-child) {
  padding-right: 1.5rem;
}

/* -------------------
      Sizes
-------------------- */

.ui.list {
  font-size: 1em;
}

.ui.mini.list {
  font-size: 0.78571429em;
}

.ui.mini.horizontal.list .list > .item,
.ui.mini.horizontal.list > .item {
  font-size: 0.78571429rem;
}

.ui.tiny.list {
  font-size: 0.85714286em;
}

.ui.tiny.horizontal.list .list > .item,
.ui.tiny.horizontal.list > .item {
  font-size: 0.85714286rem;
}

.ui.small.list {
  font-size: 0.92857143em;
}

.ui.small.horizontal.list .list > .item,
.ui.small.horizontal.list > .item {
  font-size: 0.92857143rem;
}

.ui.large.list {
  font-size: 1.14285714em;
}

.ui.large.horizontal.list .list > .item,
.ui.large.horizontal.list > .item {
  font-size: 1.14285714rem;
}

.ui.big.list {
  font-size: 1.28571429em;
}

.ui.big.horizontal.list .list > .item,
.ui.big.horizontal.list > .item {
  font-size: 1.28571429rem;
}

.ui.huge.list {
  font-size: 1.42857143em;
}

.ui.huge.horizontal.list .list > .item,
.ui.huge.horizontal.list > .item {
  font-size: 1.42857143rem;
}

.ui.massive.list {
  font-size: 1.71428571em;
}

.ui.massive.horizontal.list .list > .item,
.ui.massive.horizontal.list > .item {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Loader
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Loader
*******************************/

/* Standard Size */

.ui.loader {
  display: none;
  position: absolute;
  top: 50%;
  left: 50%;
  margin: 0;
  text-align: center;
  z-index: 1000;
  transform: translateX(-50%) translateY(-50%);
}

/* Static Shape */

.ui.loader::before {
  position: absolute;
  content: "";
  top: 0;
  left: 50%;
  width: 100%;
  height: 100%;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.1);
}

/* Active Shape */

.ui.loader::after {
  position: absolute;
  content: "";
  top: 0;
  left: 50%;
  width: 100%;
  height: 100%;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
}

/* Speeds */

.ui.ui.fast.loading::after,
.ui.ui.fast.loading .input > i.icon::after,
.ui.ui.fast.loading > i.icon::after,
.ui.fast.loader::after {
  animation-duration: 0.3s;
}

.ui.ui.slow.loading::after,
.ui.ui.slow.loading .input > i.icon::after,
.ui.ui.slow.loading > i.icon::after,
.ui.slow.loader::after {
  animation-duration: 0.9s;
}

/* Active Animation */

@keyframes loader {
  100% {
    transform: rotate(360deg);
  }
}

/* -------------------
      Coupling
-------------------- */

/* Show inside active dimmer */

.ui.dimmer > .loader {
  display: block;
}

/* Black Dimmer */

.ui.dimmer > .ui.loader {
  color: rgba(255, 255, 255, 0.9);
}

.ui.dimmer > .ui.loader:not(.elastic)::before {
  border-color: rgba(255, 255, 255, 0.15);
}

/* White Dimmer (Inverted) */

.ui.inverted.dimmer > .ui.loader {
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.dimmer > .ui.loader:not(.elastic)::before {
  border-color: rgba(0, 0, 0, 0.1);
}

/*******************************
             Types
*******************************/

/* -------------------
            Text
    -------------------- */

.ui.ui.ui.ui.text.loader {
  width: auto;
  height: auto;
  text-align: center;
  font-style: normal;
}

/*******************************
            States
*******************************/

.ui.indeterminate.loader::after {
  animation-direction: reverse;
  animation-duration: 1.2s;
}

.ui.loader.active,
.ui.loader.visible {
  display: block;
}

.ui.loader.disabled,
.ui.loader.hidden {
  display: none;
}

/*******************************
            Variations
*******************************/

/* -------------------
        Sizes
-------------------- */

.ui.loader {
  width: 2.28571429rem;
  height: 2.28571429rem;
  font-size: 1em;
}

.ui.loader::before,
.ui.loader::after {
  width: 2.28571429rem;
  height: 2.28571429rem;
  margin: 0 0 0 -1.14285714rem;
}

.ui.text.loader {
  min-width: 2.28571429rem;
  padding-top: 3.07142857rem;
}

.ui.mini.loader {
  width: 1rem;
  height: 1rem;
  font-size: 0.78571429em;
}

.ui.mini.loader::before,
.ui.mini.loader::after {
  width: 1rem;
  height: 1rem;
  margin: 0 0 0 -0.5rem;
}

.ui.mini.text.loader {
  min-width: 1rem;
  padding-top: 1.78571429rem;
}

.ui.tiny.loader {
  width: 1.14285714rem;
  height: 1.14285714rem;
  font-size: 0.85714286em;
}

.ui.tiny.loader::before,
.ui.tiny.loader::after {
  width: 1.14285714rem;
  height: 1.14285714rem;
  margin: 0 0 0 -0.57142857rem;
}

.ui.tiny.text.loader {
  min-width: 1.14285714rem;
  padding-top: 1.92857143rem;
}

.ui.small.loader {
  width: 1.71428571rem;
  height: 1.71428571rem;
  font-size: 0.92857143em;
}

.ui.small.loader::before,
.ui.small.loader::after {
  width: 1.71428571rem;
  height: 1.71428571rem;
  margin: 0 0 0 -0.85714286rem;
}

.ui.small.text.loader {
  min-width: 1.71428571rem;
  padding-top: 2.5rem;
}

.ui.large.loader {
  width: 3.42857143rem;
  height: 3.42857143rem;
  font-size: 1.14285714em;
}

.ui.large.loader::before,
.ui.large.loader::after {
  width: 3.42857143rem;
  height: 3.42857143rem;
  margin: 0 0 0 -1.71428571rem;
}

.ui.large.text.loader {
  min-width: 3.42857143rem;
  padding-top: 4.21428571rem;
}

.ui.big.loader {
  width: 3.71428571rem;
  height: 3.71428571rem;
  font-size: 1.28571429em;
}

.ui.big.loader::before,
.ui.big.loader::after {
  width: 3.71428571rem;
  height: 3.71428571rem;
  margin: 0 0 0 -1.85714286rem;
}

.ui.big.text.loader {
  min-width: 3.71428571rem;
  padding-top: 4.5rem;
}

.ui.huge.loader {
  width: 4.14285714rem;
  height: 4.14285714rem;
  font-size: 1.42857143em;
}

.ui.huge.loader::before,
.ui.huge.loader::after {
  width: 4.14285714rem;
  height: 4.14285714rem;
  margin: 0 0 0 -2.07142857rem;
}

.ui.huge.text.loader {
  min-width: 4.14285714rem;
  padding-top: 4.92857143rem;
}

.ui.massive.loader {
  width: 4.57142857rem;
  height: 4.57142857rem;
  font-size: 1.71428571em;
}

.ui.massive.loader::before,
.ui.massive.loader::after {
  width: 4.57142857rem;
  height: 4.57142857rem;
  margin: 0 0 0 -2.28571429rem;
}

.ui.massive.text.loader {
  min-width: 4.57142857rem;
  padding-top: 5.35714286rem;
}

/* -------------------
       Colors
-------------------- */

.ui.ui.primary.elastic.loader::before,
.ui.primary.basic.elastic.loading.button::before,
.ui.primary.basic.elastic.loading.button::after,
.ui.ui.ui.primary.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.primary.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.primary.elastic.loading > i.icon::before,
.ui.ui.ui.ui.primary.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.primary.loading .input > i.icon::after,
.ui.ui.ui.ui.primary.loading > i.icon::after,
.ui.ui.ui.primary.loader::after {
  color: #2185d0;
}

.ui.inverted.primary.elastic.loader::before,
.ui.ui.ui.inverted.primary.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.primary.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.primary.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.primary.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.primary.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.primary.loading > i.icon::after,
.ui.ui.ui.inverted.primary.loader::after {
  color: #54c8ff;
}

.ui.ui.secondary.elastic.loader::before,
.ui.secondary.basic.elastic.loading.button::before,
.ui.secondary.basic.elastic.loading.button::after,
.ui.ui.ui.secondary.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.secondary.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.secondary.elastic.loading > i.icon::before,
.ui.ui.ui.ui.secondary.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.secondary.loading .input > i.icon::after,
.ui.ui.ui.ui.secondary.loading > i.icon::after,
.ui.ui.ui.secondary.loader::after {
  color: #1b1c1d;
}

.ui.inverted.secondary.elastic.loader::before,
.ui.ui.ui.inverted.secondary.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.secondary.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.secondary.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.secondary.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.secondary.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.secondary.loading > i.icon::after,
.ui.ui.ui.inverted.secondary.loader::after {
  color: #545454;
}

.ui.ui.red.elastic.loader::before,
.ui.red.basic.elastic.loading.button::before,
.ui.red.basic.elastic.loading.button::after,
.ui.ui.ui.red.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.red.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.red.elastic.loading > i.icon::before,
.ui.ui.ui.ui.red.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.red.loading .input > i.icon::after,
.ui.ui.ui.ui.red.loading > i.icon::after,
.ui.ui.ui.red.loader::after {
  color: #db2828;
}

.ui.inverted.red.elastic.loader::before,
.ui.ui.ui.inverted.red.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.red.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.red.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.red.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.red.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.red.loading > i.icon::after,
.ui.ui.ui.inverted.red.loader::after {
  color: #ff695e;
}

.ui.ui.orange.elastic.loader::before,
.ui.orange.basic.elastic.loading.button::before,
.ui.orange.basic.elastic.loading.button::after,
.ui.ui.ui.orange.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.orange.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.orange.elastic.loading > i.icon::before,
.ui.ui.ui.ui.orange.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.orange.loading .input > i.icon::after,
.ui.ui.ui.ui.orange.loading > i.icon::after,
.ui.ui.ui.orange.loader::after {
  color: #f2711c;
}

.ui.inverted.orange.elastic.loader::before,
.ui.ui.ui.inverted.orange.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.orange.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.orange.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.orange.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.orange.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.orange.loading > i.icon::after,
.ui.ui.ui.inverted.orange.loader::after {
  color: #ff851b;
}

.ui.ui.yellow.elastic.loader::before,
.ui.yellow.basic.elastic.loading.button::before,
.ui.yellow.basic.elastic.loading.button::after,
.ui.ui.ui.yellow.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.yellow.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.yellow.elastic.loading > i.icon::before,
.ui.ui.ui.ui.yellow.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.yellow.loading .input > i.icon::after,
.ui.ui.ui.ui.yellow.loading > i.icon::after,
.ui.ui.ui.yellow.loader::after {
  color: #fbbd08;
}

.ui.inverted.yellow.elastic.loader::before,
.ui.ui.ui.inverted.yellow.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.yellow.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.yellow.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.yellow.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.yellow.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.yellow.loading > i.icon::after,
.ui.ui.ui.inverted.yellow.loader::after {
  color: #ffe21f;
}

.ui.ui.olive.elastic.loader::before,
.ui.olive.basic.elastic.loading.button::before,
.ui.olive.basic.elastic.loading.button::after,
.ui.ui.ui.olive.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.olive.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.olive.elastic.loading > i.icon::before,
.ui.ui.ui.ui.olive.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.olive.loading .input > i.icon::after,
.ui.ui.ui.ui.olive.loading > i.icon::after,
.ui.ui.ui.olive.loader::after {
  color: #b5cc18;
}

.ui.inverted.olive.elastic.loader::before,
.ui.ui.ui.inverted.olive.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.olive.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.olive.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.olive.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.olive.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.olive.loading > i.icon::after,
.ui.ui.ui.inverted.olive.loader::after {
  color: #d9e778;
}

.ui.ui.green.elastic.loader::before,
.ui.green.basic.elastic.loading.button::before,
.ui.green.basic.elastic.loading.button::after,
.ui.ui.ui.green.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.green.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.green.elastic.loading > i.icon::before,
.ui.ui.ui.ui.green.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.green.loading .input > i.icon::after,
.ui.ui.ui.ui.green.loading > i.icon::after,
.ui.ui.ui.green.loader::after {
  color: #21ba45;
}

.ui.inverted.green.elastic.loader::before,
.ui.ui.ui.inverted.green.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.green.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.green.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.green.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.green.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.green.loading > i.icon::after,
.ui.ui.ui.inverted.green.loader::after {
  color: #2ecc40;
}

.ui.ui.teal.elastic.loader::before,
.ui.teal.basic.elastic.loading.button::before,
.ui.teal.basic.elastic.loading.button::after,
.ui.ui.ui.teal.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.teal.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.teal.elastic.loading > i.icon::before,
.ui.ui.ui.ui.teal.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.teal.loading .input > i.icon::after,
.ui.ui.ui.ui.teal.loading > i.icon::after,
.ui.ui.ui.teal.loader::after {
  color: #00b5ad;
}

.ui.inverted.teal.elastic.loader::before,
.ui.ui.ui.inverted.teal.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.teal.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.teal.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.teal.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.teal.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.teal.loading > i.icon::after,
.ui.ui.ui.inverted.teal.loader::after {
  color: #6dffff;
}

.ui.ui.blue.elastic.loader::before,
.ui.blue.basic.elastic.loading.button::before,
.ui.blue.basic.elastic.loading.button::after,
.ui.ui.ui.blue.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.blue.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.blue.elastic.loading > i.icon::before,
.ui.ui.ui.ui.blue.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.blue.loading .input > i.icon::after,
.ui.ui.ui.ui.blue.loading > i.icon::after,
.ui.ui.ui.blue.loader::after {
  color: #2185d0;
}

.ui.inverted.blue.elastic.loader::before,
.ui.ui.ui.inverted.blue.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.blue.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.blue.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.blue.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.blue.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.blue.loading > i.icon::after,
.ui.ui.ui.inverted.blue.loader::after {
  color: #54c8ff;
}

.ui.ui.violet.elastic.loader::before,
.ui.violet.basic.elastic.loading.button::before,
.ui.violet.basic.elastic.loading.button::after,
.ui.ui.ui.violet.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.violet.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.violet.elastic.loading > i.icon::before,
.ui.ui.ui.ui.violet.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.violet.loading .input > i.icon::after,
.ui.ui.ui.ui.violet.loading > i.icon::after,
.ui.ui.ui.violet.loader::after {
  color: #6435c9;
}

.ui.inverted.violet.elastic.loader::before,
.ui.ui.ui.inverted.violet.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.violet.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.violet.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.violet.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.violet.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.violet.loading > i.icon::after,
.ui.ui.ui.inverted.violet.loader::after {
  color: #a291fb;
}

.ui.ui.purple.elastic.loader::before,
.ui.purple.basic.elastic.loading.button::before,
.ui.purple.basic.elastic.loading.button::after,
.ui.ui.ui.purple.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.purple.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.purple.elastic.loading > i.icon::before,
.ui.ui.ui.ui.purple.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.purple.loading .input > i.icon::after,
.ui.ui.ui.ui.purple.loading > i.icon::after,
.ui.ui.ui.purple.loader::after {
  color: #a333c8;
}

.ui.inverted.purple.elastic.loader::before,
.ui.ui.ui.inverted.purple.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.purple.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.purple.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.purple.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.purple.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.purple.loading > i.icon::after,
.ui.ui.ui.inverted.purple.loader::after {
  color: #dc73ff;
}

.ui.ui.pink.elastic.loader::before,
.ui.pink.basic.elastic.loading.button::before,
.ui.pink.basic.elastic.loading.button::after,
.ui.ui.ui.pink.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.pink.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.pink.elastic.loading > i.icon::before,
.ui.ui.ui.ui.pink.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.pink.loading .input > i.icon::after,
.ui.ui.ui.ui.pink.loading > i.icon::after,
.ui.ui.ui.pink.loader::after {
  color: #e03997;
}

.ui.inverted.pink.elastic.loader::before,
.ui.ui.ui.inverted.pink.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.pink.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.pink.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.pink.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.pink.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.pink.loading > i.icon::after,
.ui.ui.ui.inverted.pink.loader::after {
  color: #ff8edf;
}

.ui.ui.brown.elastic.loader::before,
.ui.brown.basic.elastic.loading.button::before,
.ui.brown.basic.elastic.loading.button::after,
.ui.ui.ui.brown.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.brown.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.brown.elastic.loading > i.icon::before,
.ui.ui.ui.ui.brown.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.brown.loading .input > i.icon::after,
.ui.ui.ui.ui.brown.loading > i.icon::after,
.ui.ui.ui.brown.loader::after {
  color: #a5673f;
}

.ui.inverted.brown.elastic.loader::before,
.ui.ui.ui.inverted.brown.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.brown.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.brown.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.brown.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.brown.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.brown.loading > i.icon::after,
.ui.ui.ui.inverted.brown.loader::after {
  color: #d67c1c;
}

.ui.ui.grey.elastic.loader::before,
.ui.grey.basic.elastic.loading.button::before,
.ui.grey.basic.elastic.loading.button::after,
.ui.ui.ui.grey.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.grey.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.grey.elastic.loading > i.icon::before,
.ui.ui.ui.ui.grey.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.grey.loading .input > i.icon::after,
.ui.ui.ui.ui.grey.loading > i.icon::after,
.ui.ui.ui.grey.loader::after {
  color: #767676;
}

.ui.inverted.grey.elastic.loader::before,
.ui.ui.ui.inverted.grey.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.grey.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.grey.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.grey.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.grey.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.grey.loading > i.icon::after,
.ui.ui.ui.inverted.grey.loader::after {
  color: #dcddde;
}

.ui.ui.black.elastic.loader::before,
.ui.black.basic.elastic.loading.button::before,
.ui.black.basic.elastic.loading.button::after,
.ui.ui.ui.black.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.black.elastic.loading .input > i.icon::before,
.ui.ui.ui.ui.black.elastic.loading > i.icon::before,
.ui.ui.ui.ui.black.loading:not(.usual):not(.button)::after,
.ui.ui.ui.ui.black.loading .input > i.icon::after,
.ui.ui.ui.ui.black.loading > i.icon::after,
.ui.ui.ui.black.loader::after {
  color: #1b1c1d;
}

.ui.inverted.black.elastic.loader::before,
.ui.ui.ui.inverted.black.elastic.loading:not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.inverted.black.elastic.loading .input > i.icon::before,
.ui.ui.ui.inverted.black.elastic.loading > i.icon::before,
.ui.ui.ui.ui.inverted.black.loading:not(.usual)::after,
.ui.ui.ui.ui.inverted.black.loading .input > i.icon::after,
.ui.ui.ui.ui.inverted.black.loading > i.icon::after,
.ui.ui.ui.inverted.black.loader::after {
  color: #545454;
}

.ui.ui.elastic.loader::before,
.ui.ui.ui.elastic.loading::before,
.ui.ui.ui.elastic.loading .input > i.icon::before,
.ui.ui.ui.elastic.loading > i.icon::before,
.ui.ui.ui.ui.loading:not(.usual)::after,
.ui.ui.ui.ui.loading .input > i.icon::after,
.ui.ui.ui.ui.loading > i.icon::after,
.ui.ui.ui.loader::after {
  border-color: currentColor;
}

.ui.ui.ui.ui.elastic.loading.button:not(.inverted):not(.basic)::before {
  color: #fff;
}

.ui.elastic.basic.loading.button::before,
.ui.elastic.basic.loading.button::after {
  color: #767676;
}

.ui.ui.ui.ui.double.loading.button::after {
  border-bottom-color: currentColor;
}

/* -------------------
           Inline
    -------------------- */

.ui.inline.loader {
  position: relative;
  vertical-align: middle;
  margin: 0;
  left: 0;
  top: 0;
  transform: none;
}

.ui.inline.loader.active,
.ui.inline.loader.visible {
  display: inline-block;
}

/* Centered Inline */

.ui.centered.inline.loader.active,
.ui.centered.inline.loader.visible {
  display: block;
  margin-left: auto;
  margin-right: auto;
}

.ui.ui.ui.ui.ui.ui.loading::after,
.ui.ui.ui.ui.ui.ui.loading .input > i.icon::after,
.ui.ui.ui.ui.ui.ui.loading > i.icon::after,
.ui.ui.ui.ui.ui.loader::after {
  border-left-color: transparent;
  border-right-color: transparent;
}

.ui.ui.ui.ui.ui.ui.ui.loading:not(.double)::after,
.ui.ui.ui.ui.ui.ui.ui.loading:not(.double) .input > i.icon::after,
.ui.ui.ui.ui.ui.ui.ui.loading:not(.double) > i.icon::after,
.ui.ui.ui.ui.ui.ui.loader:not(.double)::after {
  border-bottom-color: transparent;
}

.ui.ui.ui.ui.ui.ui.loading.card::after,
.ui.ui.ui.ui.ui.ui.loading.segments::after,
.ui.ui.ui.ui.ui.ui.loading.segment::after,
.ui.ui.ui.ui.ui.ui.loading.form::after {
  border-left-color: rgba(0, 0, 0, 0.1);
  border-right-color: rgba(0, 0, 0, 0.1);
}

.ui.ui.ui.ui.ui.ui.loading.card:not(.double)::after,
.ui.ui.ui.ui.ui.ui.loading.segments:not(.double)::after,
.ui.ui.ui.ui.ui.ui.loading.segment:not(.double)::after,
.ui.ui.ui.ui.ui.ui.loading.form:not(.double)::after {
  border-bottom-color: rgba(0, 0, 0, 0.1);
}

/* -------------------
           Elastic
    -------------------- */

.ui.dimmer > .ui.elastic.loader {
  color: #fff;
}

.ui.inverted.dimmer > .ui.elastic.loader {
  color: #767676;
}

.ui.ui.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::after,
.ui.ui.elastic.loading .input > i.icon::after,
.ui.ui.elastic.loading > i.icon::after,
.ui.ui.elastic.loader::after {
  animation: loader 1s infinite cubic-bezier(0.27, 1.05, 0.92, 0.61);
  animation-delay: 0.3s;
}

.ui.ui.ui.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.elastic.loading .input > i.icon::before,
.ui.ui.ui.elastic.loading > i.icon::before,
.ui.ui.elastic.loader::before {
  animation: elastic-loader 1s infinite cubic-bezier(0.27, 1.05, 0.92, 0.61);
  -moz-animation: currentcolor-elastic-loader 1s infinite cubic-bezier(0.27, 1.05, 0.92, 0.61);
  border-right-color: transparent;
}

.ui.elastic.inline.loader:empty {
  animation: loader 8s infinite linear;
}

.ui.ui.slow.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::after,
.ui.ui.slow.elastic.loading .input > i.icon::after,
.ui.ui.slow.elastic.loading > i.icon::after,
.ui.ui.slow.elastic.loader::after {
  animation-duration: 1.5s;
  animation-delay: 0.45s;
}

.ui.ui.ui.slow.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.slow.elastic.loading .input > i.icon::before,
.ui.ui.ui.slow.elastic.loading > i.icon::before,
.ui.ui.slow.elastic.loader::before {
  animation-duration: 1.5s;
}

.ui.ui.fast.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::after,
.ui.ui.fast.elastic.loading .input > i.icon::after,
.ui.ui.fast.elastic.loading > i.icon::after,
.ui.ui.fast.elastic.loader::after {
  animation-duration: 0.66s;
  animation-delay: 0.2s;
}

.ui.ui.ui.fast.elastic.loading:not(.form):not(.segment):not(.segments):not(.card)::before,
.ui.ui.ui.fast.elastic.loading .input > i.icon::before,
.ui.ui.ui.fast.elastic.loading > i.icon::before,
.ui.ui.fast.elastic.loader::before {
  animation-duration: 0.66s;
}

@keyframes elastic-loader {
  0%, 1% {
    border-left-color: transparent;
    border-bottom-color: transparent;
  }

  1.1%, 50% {
    border-left-color: inherit;
  }

  10%, 35.1% {
    border-bottom-color: transparent;
  }

  10.1%, 35% {
    border-bottom-color: inherit;
  }

  50.1% {
    border-left-color: transparent;
  }

  100% {
    border-left-color: transparent;
    border-bottom-color: transparent;
    transform: rotate(360deg);
  }
}

@keyframes currentcolor-elastic-loader {
  0%, 1% {
    border-left-color: transparent;
    border-bottom-color: transparent;
  }

  1.1%, 50% {
    border-left-color: currentColor;
  }

  10%, 35.1% {
    border-bottom-color: transparent;
  }

  10.1%, 35% {
    border-bottom-color: currentColor;
  }

  50.1% {
    border-left-color: transparent;
  }

  100% {
    border-left-color: transparent;
    border-bottom-color: transparent;
    transform: rotate(360deg);
  }
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Placeholder
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/* -------------------
      Content
-------------------- */

.ui.placeholder {
  position: static;
  overflow: hidden;
  animation: placeholderShimmer 2s linear;
  animation-iteration-count: infinite;
  background-color: #fff;
  background-image: linear-gradient(to right, rgba(0, 0, 0, 0.08) 0, rgba(0, 0, 0, 0.15) 15%, rgba(0, 0, 0, 0.08) 30%);
  background-size: 1200px 100%;
  max-width: 30rem;
}

@keyframes placeholderShimmer {
  0% {
    background-position: -1200px 0;
  }

  100% {
    background-position: 1200px 0;
  }
}

.ui.placeholder + .ui.placeholder {
  margin-top: 2rem;
}

.ui.placeholder + .ui.placeholder {
  animation-delay: 0.15s;
}

.ui.placeholder + .ui.placeholder + .ui.placeholder {
  animation-delay: 0.3s;
}

.ui.placeholder + .ui.placeholder + .ui.placeholder + .ui.placeholder {
  animation-delay: 0.45s;
}

.ui.placeholder + .ui.placeholder + .ui.placeholder + .ui.placeholder + .ui.placeholder {
  animation-delay: 0.6s;
}

.ui.placeholder,
.ui.placeholder > ::before,
.ui.placeholder .image.header::after,
.ui.placeholder .line,
.ui.placeholder .line::after {
  background-color: #fff;
}

.ui.placeholder.hidden {
  display: none;
}

/* Image */

.ui.placeholder .image:not(.header):not(.ui):not(.icon) {
  height: 100px;
}

.ui.placeholder .square.image:not(.header) {
  height: 0;
  overflow: hidden;
  /* 1/1 aspect ratio */
  padding-top: 100%;
}

.ui.placeholder .rectangular.image:not(.header) {
  height: 0;
  overflow: hidden;
  /* 4/3 aspect ratio */
  padding-top: 75%;
}

/* Lines */

.ui.placeholder .line {
  position: relative;
  height: 0.85714286em;
}

.ui.placeholder .line::before,
.ui.placeholder .line::after {
  top: 100%;
  position: absolute;
  content: "";
  background-color: inherit;
}

.ui.placeholder .line::before {
  left: 0;
}

.ui.placeholder .line::after {
  right: 0;
}

/* Any Lines */

.ui.placeholder .line {
  margin-bottom: 0.5em;
}

.ui.placeholder .line::before,
.ui.placeholder .line::after {
  height: 0.5em;
}

.ui.placeholder .line:not(:first-child) {
  margin-top: 0.5em;
}

/* Line Outdent */

.ui.placeholder .line:nth-child(1)::after {
  width: 0;
}

.ui.placeholder .line:nth-child(2)::after {
  width: 50%;
}

.ui.placeholder .line:nth-child(3)::after {
  width: 10%;
}

.ui.placeholder .line:nth-child(4)::after {
  width: 35%;
}

.ui.placeholder .line:nth-child(5)::after {
  width: 65%;
}

/* Header Image + 2 Lines */

.ui.placeholder .header {
  position: relative;
  overflow: hidden;
}

/* Header Line 1 & 2 */

.ui.placeholder .header .line {
  margin-bottom: 0.64285714em;
}

.ui.placeholder .header .line::before,
.ui.placeholder .header .line::after {
  height: 0.64285714em;
}

.ui.placeholder .header .line:not(:first-child) {
  margin-top: 0.64285714em;
}

.ui.placeholder .header .line::after {
  width: 20%;
}

.ui.placeholder .header .line:nth-child(2)::after {
  width: 60%;
}

/* Image Header */

.ui.placeholder .image.header .line {
  margin-left: 3em;
}

.ui.placeholder .image.header .line::before {
  width: 0.71428571rem;
}

.ui.placeholder .image.header::after {
  display: block;
  height: 0.85714286em;
  content: "";
  margin-left: 3em;
}

/* Spacing */

.ui.placeholder .image .line:first-child,
.ui.placeholder .paragraph .line:first-child,
.ui.placeholder .header .line:first-child {
  height: 0.01px;
}

.ui.placeholder .image:not(:first-child)::before,
.ui.placeholder .paragraph:not(:first-child)::before,
.ui.placeholder .header:not(:first-child)::before {
  height: 1.42857143em;
  content: "";
  display: block;
}

/* Inverted Content Loader */

.ui.inverted.placeholder {
  background-image: linear-gradient(to right, rgba(255, 255, 255, 0.08) 0, rgba(255, 255, 255, 0.14) 15%, rgba(255, 255, 255, 0.08) 30%);
}

.ui.inverted.placeholder,
.ui.inverted.placeholder > ::before,
.ui.inverted.placeholder .image.header::after,
.ui.inverted.placeholder .line,
.ui.inverted.placeholder .line::after {
  background-color: #1b1c1d;
}

/*******************************
            Variations
*******************************/

/* -------------------
        Sizes
-------------------- */

.ui.placeholder .full.line.line.line::after {
  width: 0;
}

.ui.placeholder .very.long.line.line.line::after {
  width: 10%;
}

.ui.placeholder .long.line.line.line::after {
  width: 35%;
}

.ui.placeholder .medium.line.line.line::after {
  width: 50%;
}

.ui.placeholder .short.line.line.line::after {
  width: 65%;
}

.ui.placeholder .very.short.line.line.line::after {
  width: 80%;
}

/* -------------------
            Fluid
    -------------------- */

.ui.fluid.placeholder {
  max-width: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Rail
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Rails
*******************************/

.ui.rail {
  position: absolute;
  top: 0;
  width: 300px;
  height: 100%;
}

.ui.left.rail {
  left: auto;
  right: 100%;
  padding: 0 2rem 0 0;
  margin: 0 2rem 0 0;
}

.ui.right.rail {
  left: 100%;
  right: auto;
  padding: 0 0 0 2rem;
  margin: 0 0 0 2rem;
}

/*******************************
           Variations
*******************************/

/* --------------
         Internal
    --------------- */

.ui.left.internal.rail {
  left: 0;
  right: auto;
  padding: 0 0 0 2rem;
  margin: 0 0 0 2rem;
}

.ui.right.internal.rail {
  left: auto;
  right: 0;
  padding: 0 2rem 0 0;
  margin: 0 2rem 0 0;
}

/* --------------
        Dividing
    --------------- */

.ui.dividing.rail {
  width: 302.5px;
}

.ui.left.dividing.rail {
  padding: 0 2.5rem 0 0;
  margin: 0 2.5rem 0 0;
  border-right: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.right.dividing.rail {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
  padding: 0 0 0 2.5rem;
  margin: 0 0 0 2.5rem;
}

/* --------------
        Distance
    --------------- */

.ui.close.rail {
  width: calc(300px + 1em);
}

.ui.close.left.rail {
  padding: 0 1em 0 0;
  margin: 0 1em 0 0;
}

.ui.close.right.rail {
  padding: 0 0 0 1em;
  margin: 0 0 0 1em;
}

.ui.very.close.rail {
  width: calc(300px + 0.5em);
}

.ui.very.close.left.rail {
  padding: 0 0.5em 0 0;
  margin: 0 0.5em 0 0;
}

.ui.very.close.right.rail {
  padding: 0 0 0 0.5em;
  margin: 0 0 0 0.5em;
}

/* --------------
        Attached
    --------------- */

.ui.attached.left.rail,
.ui.attached.right.rail {
  padding: 0;
  margin: 0;
}

/* --------------
     Sizing
--------------- */

.ui.rail {
  font-size: 1rem;
}

.ui.mini.rail {
  font-size: 0.78571429rem;
}

.ui.tiny.rail {
  font-size: 0.85714286rem;
}

.ui.small.rail {
  font-size: 0.92857143rem;
}

.ui.large.rail {
  font-size: 1.14285714rem;
}

.ui.big.rail {
  font-size: 1.28571429rem;
}

.ui.huge.rail {
  font-size: 1.42857143rem;
}

.ui.massive.rail {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Reveal
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Reveal
*******************************/

.ui.reveal {
  display: inherit;
  position: relative !important;
  font-size: 0;
}

.ui.reveal > .visible.content {
  position: absolute !important;
  top: 0 !important;
  left: 0 !important;
  z-index: 3 !important;
  transition: all 0.5s ease 0.1s;
}

.ui.reveal > .hidden.content {
  position: relative !important;
  z-index: 2 !important;
}

/* Make sure hovered element is on top of other reveal */

.ui.active.reveal .visible.content,
.ui.reveal:hover .visible.content {
  z-index: 4 !important;
}

/*******************************
              Types
*******************************/

/* --------------
          Slide
    --------------- */

.ui.slide.reveal {
  position: relative !important;
  overflow: hidden !important;
  white-space: nowrap;
}

.ui.slide.reveal > .content {
  display: block;
  width: 100%;
  white-space: normal;
  float: left;
  margin: 0;
  transition: transform 0.5s ease 0.1s;
}

.ui.slide.reveal > .visible.content {
  position: relative !important;
}

.ui.slide.reveal > .hidden.content {
  position: absolute !important;
  left: 0 !important;
  width: 100% !important;
  transform: translateX(100%) !important;
}

.ui.slide.active.reveal > .visible.content,
.ui.slide.reveal:hover > .visible.content {
  transform: translateX(-100%) !important;
}

.ui.slide.active.reveal > .hidden.content,
.ui.slide.reveal:hover > .hidden.content {
  transform: translateX(0) !important;
}

.ui.slide.right.reveal > .visible.content {
  transform: translateX(0) !important;
}

.ui.slide.right.reveal > .hidden.content {
  transform: translateX(-100%) !important;
}

.ui.slide.right.active.reveal > .visible.content,
.ui.slide.right.reveal:hover > .visible.content {
  transform: translateX(100%) !important;
}

.ui.slide.right.active.reveal > .hidden.content,
.ui.slide.right.reveal:hover > .hidden.content {
  transform: translateX(0) !important;
}

.ui.slide.up.reveal > .hidden.content {
  transform: translateY(100%) !important;
}

.ui.slide.up.active.reveal > .visible.content,
.ui.slide.up.reveal:hover > .visible.content {
  transform: translateY(-100%) !important;
}

.ui.slide.up.active.reveal > .hidden.content,
.ui.slide.up.reveal:hover > .hidden.content {
  transform: translateY(0) !important;
}

.ui.slide.down.reveal > .hidden.content {
  transform: translateY(-100%) !important;
}

.ui.slide.down.active.reveal > .visible.content,
.ui.slide.down.reveal:hover > .visible.content {
  transform: translateY(100%) !important;
}

.ui.slide.down.active.reveal > .hidden.content,
.ui.slide.down.reveal:hover > .hidden.content {
  transform: translateY(0) !important;
}

/* --------------
          Fade
    --------------- */

.ui.fade.reveal > .visible.content {
  opacity: 1;
}

.ui.fade.active.reveal > .visible.content,
.ui.fade.reveal:hover > .visible.content {
  opacity: 0;
}

/* --------------
          Move
    --------------- */

.ui.move.reveal {
  position: relative !important;
  overflow: hidden !important;
  white-space: nowrap;
}

.ui.move.reveal > .content {
  display: block;
  float: left;
  white-space: normal;
  margin: 0;
  transition: transform 0.5s cubic-bezier(0.175, 0.885, 0.32, 1) 0.1s;
}

.ui.move.reveal > .visible.content {
  position: relative !important;
}

.ui.move.reveal > .hidden.content {
  position: absolute !important;
  left: 0 !important;
  width: 100% !important;
}

.ui.move.active.reveal > .visible.content,
.ui.move.reveal:hover > .visible.content {
  transform: translateX(-100%) !important;
}

.ui.move.right.active.reveal > .visible.content,
.ui.move.right.reveal:hover > .visible.content {
  transform: translateX(100%) !important;
}

.ui.move.up.active.reveal > .visible.content,
.ui.move.up.reveal:hover > .visible.content {
  transform: translateY(-100%) !important;
}

.ui.move.down.active.reveal > .visible.content,
.ui.move.down.reveal:hover > .visible.content {
  transform: translateY(100%) !important;
}

/* --------------
         Rotate
    --------------- */

.ui.rotate.reveal > .visible.content {
  transition-duration: 0.5s;
  transform: rotate(0);
}

.ui.rotate.reveal > .visible.content,
.ui.rotate.right.reveal > .visible.content {
  transform-origin: bottom right;
}

.ui.rotate.active.reveal > .visible.content,
.ui.rotate.reveal:hover > .visible.content,
.ui.rotate.right.active.reveal > .visible.content,
.ui.rotate.right.reveal:hover > .visible.content {
  transform: rotate(110deg);
}

.ui.rotate.left.reveal > .visible.content {
  transform-origin: bottom left;
}

.ui.rotate.left.active.reveal > .visible.content,
.ui.rotate.left.reveal:hover > .visible.content {
  transform: rotate(-110deg);
}

/*******************************
              States
*******************************/

.ui.disabled.reveal:hover > .visible.visible.content {
  position: static !important;
  display: block !important;
  opacity: 1 !important;
  top: 0 !important;
  left: 0 !important;
  right: auto !important;
  bottom: auto !important;
  transform: none !important;
}

.ui.disabled.reveal:hover > .hidden.hidden.content {
  display: none !important;
}

/*******************************
           Coupling
*******************************/

.ui.reveal > .ui.ribbon.label {
  z-index: 5;
}

/*******************************
           Variations
*******************************/

/* --------------
     Visible
--------------- */

.ui.visible.reveal {
  overflow: visible;
}

/* --------------
     Instant
--------------- */

.ui.instant.reveal > .content {
  transition-delay: 0s !important;
}

/* --------------
     Sizing
--------------- */

.ui.reveal > .content {
  font-size: 1rem;
}

.ui.mini.reveal > .content {
  font-size: 0.78571429rem;
}

.ui.tiny.reveal > .content {
  font-size: 0.85714286rem;
}

.ui.small.reveal > .content {
  font-size: 0.92857143rem;
}

.ui.large.reveal > .content {
  font-size: 1.14285714rem;
}

.ui.big.reveal > .content {
  font-size: 1.28571429rem;
}

.ui.huge.reveal > .content {
  font-size: 1.42857143rem;
}

.ui.massive.reveal > .content {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Segment
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Segment
*******************************/

.ui.segment {
  position: relative;
  background: #fff;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
  margin: 1rem 0;
  padding: 1em 1em;
  border-radius: 0.28571429rem;
  border: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.segment:first-child {
  margin-top: 0;
}

.ui.segment:last-child {
  margin-bottom: 0;
}

/* Vertical */

.ui.vertical.segment {
  margin: 0;
  padding-left: 0;
  padding-right: 0;
  background: none transparent;
  border-radius: 0;
  box-shadow: none;
  border: none;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.vertical.segment:first-child {
  border-top: none;
}

/* -------------------
    Loose Coupling
-------------------- */

/* Header */

.ui.inverted.segments .segment > .ui.header .sub.header,
.ui.inverted.segments .segment > .ui.header,
.ui.inverted.segment > .ui.header .sub.header,
.ui.inverted.segment > .ui.header {
  color: #fff;
}

/* Label */

.ui[class*="bottom attached"].segment > [class*="top attached"].label {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.ui[class*="top attached"].segment > [class*="bottom attached"].label {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.attached.segment:not(.top):not(.bottom) > [class*="top attached"].label {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.ui.attached.segment:not(.top):not(.bottom) > [class*="bottom attached"].label {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

/* Grid */

.ui.page.grid.segment,
.ui.grid > .row > .ui.segment.column,
.ui.grid > .ui.segment.column {
  padding-top: 2em;
  padding-bottom: 2em;
}

.ui.grid.segment {
  margin: 1rem 0;
  border-radius: 0.28571429rem;
}

/* Table */

.ui.basic.table.segment {
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
}

.ui[class*="very basic"].table.segment {
  padding: 1em 1em;
}

/* Tab */

.ui.segment.tab:last-child {
  margin-bottom: 1rem;
}

/*******************************
             Types
*******************************/

/* -------------------
         Placeholder
    -------------------- */

.ui.placeholder.segment {
  display: flex;
  flex-direction: column;
  justify-content: center;
  align-items: stretch;
  max-width: initial;
  animation: none;
  overflow: visible;
  padding: 1em 1em;
  min-height: 18rem;
  background: #f9fafb;
  border-color: rgba(34, 36, 38, 0.15);
  box-shadow: 0 2px 25px 0 rgba(34, 36, 38, 0.05) inset;
}

.ui.placeholder.segment.tab {
  display: none;
}

.ui.placeholder.segment.tab.active {
  display: flex;
}

.ui.placeholder.segment .button,
.ui.placeholder.segment textarea {
  display: block;
}

.ui.placeholder.segment .field,
.ui.placeholder.segment textarea,
.ui.placeholder.segment > .ui.input,
.ui.placeholder.segment .button {
  max-width: 15rem;
  margin-left: auto;
  margin-right: auto;
}

.ui.placeholder.segment .column .button,
.ui.placeholder.segment .column .field,
.ui.placeholder.segment .column textarea,
.ui.placeholder.segment .column > .ui.input {
  max-width: 15rem;
  margin-left: auto;
  margin-right: auto;
}

.ui.placeholder.segment > .inline {
  align-self: center;
}

.ui.placeholder.segment > .inline > .button {
  display: inline-block;
  width: auto;
  margin: 0 0.35714286rem 0 0;
}

.ui.placeholder.segment > .inline > .button:last-child {
  margin-right: 0;
}

/* -------------------
            Piled
    -------------------- */

.ui.piled.segments,
.ui.piled.segment {
  margin: 3em 0;
  box-shadow: "";
  z-index: auto;
  background-color: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  color: rgba(0, 0, 0, 0.87);
}

.ui.piled.segment:first-child {
  margin-top: 0;
}

.ui.piled.segment:last-child {
  margin-bottom: 0;
}

.ui.piled.segments::after,
.ui.piled.segments::before,
.ui.piled.segment::after,
.ui.piled.segment::before {
  background-color: inherit;
  visibility: visible;
  content: "";
  display: block;
  height: 100%;
  left: 0;
  position: absolute;
  width: 100%;
  border: inherit;
  box-shadow: "";
}

.ui.inverted.piled.segment {
  background-color: #1b1c1d;
  border: 1px solid #555;
  color: rgba(255, 255, 255, 0.9);
}

.ui.piled.segments::before,
.ui.piled.segment::before {
  transform: rotate(-1.2deg);
  top: 0;
  z-index: -2;
}

.ui.piled.segments::after,
.ui.piled.segment::after {
  transform: rotate(1.2deg);
  top: 0;
  z-index: -1;
}

/* Piled Attached */

.ui[class*="top attached"].piled.segment {
  margin-top: 3em;
  margin-bottom: 0;
}

.ui.piled.segment[class*="top attached"]:first-child {
  margin-top: 0;
}

.ui.piled.segment[class*="bottom attached"] {
  margin-top: 0;
  margin-bottom: 3em;
}

.ui.piled.segment[class*="bottom attached"]:last-child {
  margin-bottom: 0;
}

@supports selector(:has(.f)) {
  *:has(> .ui.piled.segment) {
    z-index: 0;
    position: relative;
  }
}

/* -------------------
           Stacked
    -------------------- */

.ui.stacked.segment {
  padding-bottom: 1.4em;
}

.ui.stacked.segments::before,
.ui.stacked.segments::after,
.ui.stacked.segment::before,
.ui.stacked.segment::after {
  content: "";
  position: absolute;
  bottom: -3px;
  left: 0;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  background: rgba(0, 0, 0, 0.03);
  width: 100%;
  height: 6px;
  visibility: visible;
}

.ui.stacked.segments::before,
.ui.stacked.segment::before {
  display: none;
}

/* Add additional page */

.ui.tall.stacked.segments::before,
.ui.tall.stacked.segment::before {
  display: block;
  bottom: 0;
}

/* Inverted */

.ui.stacked.inverted.segments,
.ui.stacked.inverted.segment {
  border: 1px solid rgba(225, 225, 225, 0.5);
}

.ui.stacked.inverted.segments::before,
.ui.stacked.inverted.segment::before,
.ui.stacked.inverted.segments::after,
.ui.stacked.inverted.segment::after {
  background-color: rgba(0, 0, 0, 0.03);
  border-top: 1px solid rgba(225, 225, 225, 0.5);
}

/* -------------------
           Padded
    -------------------- */

.ui.padded.segment {
  padding: 1.5em;
}

.ui[class*="very padded"].segment {
  padding: 3em;
}

/* Padded vertical */

.ui.padded.segment.vertical.segment,
.ui[class*="very padded"].vertical.segment {
  padding-left: 0;
  padding-right: 0;
}

/* -------------------
           Compact
    -------------------- */

.ui.compact.segment {
  display: table;
}

/* Compact Group */

.ui.compact.segments {
  display: inline-flex;
}

.ui.compact.segments .segment,
.ui.segments .compact.segment {
  display: block;
  flex: 0 1 auto;
}

/* -------------------
           Circular
    -------------------- */

.ui.circular.segment {
  display: table-cell;
  padding: 2em;
  text-align: center;
  vertical-align: middle;
  border-radius: 500em;
}

/* -------------------
           Raised
    -------------------- */

.ui.ui.raised.segments,
.ui.ui.raised.segment {
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.ui.inverted.raised.segments,
.ui.ui.inverted.raised.segment {
  box-shadow: 0 2px 4px 0 rgba(225, 225, 225, 0.1), 0 2px 10px 0 rgba(225, 225, 225, 0.5);
}

/*******************************
                Groups
    *******************************/

/* Group */

.ui.segments {
  flex-direction: column;
  position: relative;
  margin: 1rem 0;
  border: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
}

.ui.segments:first-child {
  margin-top: 0;
}

.ui.segments:last-child {
  margin-bottom: 0;
}

/* Nested Segment */

.ui.segments > .segment {
  top: 0;
  bottom: 0;
  border-radius: 0;
  margin: 0;
  width: auto;
  box-shadow: none;
  border: none;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.segments:not(.horizontal) > .segment:first-child {
  top: 0;
  bottom: 0;
  border-top: none;
  margin-top: 0;
  margin-bottom: 0;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

/* Bottom */

.ui.segments:not(.horizontal) > .segment:last-child {
  top: 0;
  bottom: 0;
  margin-top: 0;
  margin-bottom: 0;
  box-shadow: 0 2px 1px -1px rgba(34, 36, 38, 0.15);
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* Only */

.ui.segments:not(.horizontal) > .segment:only-child {
  border-radius: 0.28571429rem;
}

/* Nested Group */

.ui.segments > .ui.segments {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  margin: 1rem;
}

.ui.segments > .segments:first-child {
  border-top: none;
}

.ui.segments > .segment + .segments:not(.horizontal) {
  margin-top: 0;
}

/* Horizontal Group */

.ui.horizontal.segments {
  display: flex;
  flex-direction: row;
  background-color: transparent;
  padding: 0;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
  margin: 1rem 0;
  border-radius: 0.28571429rem;
  border: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.wrapping.horizontal.segments,
.ui.stackable.horizontal.segments {
  flex-wrap: wrap;
}

/* Nested Horizontal Group */

.ui.segments > .horizontal.segments {
  margin: 0;
  background-color: transparent;
  border-radius: 0;
  border: none;
  box-shadow: none;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

/* Horizontal Segment */

.ui.horizontal.segments:not(.compact) > .segment:not(.compact) {
  flex: 1 1 auto;
  -ms-flex: 1 1 0;
}

.ui.horizontal.segments > .segment {
  margin: 0;
  min-width: 0;
  border-radius: 0;
  border: none;
  box-shadow: none;
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

/* Border Fixes */

.ui.segments > .horizontal.segments:first-child {
  border-top: none;
}

.ui.horizontal.segments:not(.stackable):not(.wrapping) > .segment:first-child {
  border-left: none;
}

.ui.horizontal.segments > .segment:first-child {
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui.horizontal.segments > .segment:last-child {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

/* Equal Width */

.ui[class*="equal width"].horizontal.segments > .segment {
  width: 100%;
}

/*******************************
            States
*******************************/

/* --------------
        Disabled
    --------------- */

.ui.disabled.segments,
.ui.disabled.segment {
  opacity: 0.45;
  color: rgba(40, 40, 40, 0.3);
  pointer-events: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

/* --------------
        Loading
    --------------- */

.ui.loading.segments,
.ui.loading.segment {
  position: relative;
  cursor: default;
  pointer-events: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  transition: all 0s linear;
  min-height: 4.5em;
}

.ui.loading.segments::before,
.ui.loading.segment::before {
  position: absolute;
  content: "";
  top: 0;
  left: 0;
  background: rgba(255, 255, 255, 0.8);
  width: 100%;
  height: 100%;
  border-radius: 0.28571429rem;
  z-index: 100;
}

.ui.loading.segments::after,
.ui.loading.segment::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1.5em 0 0 -1.5em;
  width: 3em;
  height: 3em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
  visibility: visible;
  z-index: 101;
}

/*******************************
           Variations
*******************************/

/* -------------------
           Basic
    -------------------- */

.ui.basic.segment,
.ui.segments .ui.basic.segment,
.ui.basic.segments {
  background: none transparent;
  box-shadow: none;
  border: none;
  border-radius: 0;
}

/* -------------------
           Clearing
    -------------------- */

.ui.clearing.segment::after {
  content: "";
  display: block;
  clear: both;
}

/* -------------------
       Colors
-------------------- */

.ui.ui.ui.ui.ui.red.segment:not(.inverted) {
  border-top: 2px solid #db2828;
}

.ui.ui.ui.ui.ui.inverted.red.segment {
  background-color: #db2828;
  color: #fff;
}

.ui.ui.inverted.red.segment:not(.piled) {
  border-color: #db2828;
}

.ui.ui.ui.ui.ui.orange.segment:not(.inverted) {
  border-top: 2px solid #f2711c;
}

.ui.ui.ui.ui.ui.inverted.orange.segment {
  background-color: #f2711c;
  color: #fff;
}

.ui.ui.inverted.orange.segment:not(.piled) {
  border-color: #f2711c;
}

.ui.ui.ui.ui.ui.yellow.segment:not(.inverted) {
  border-top: 2px solid #fbbd08;
}

.ui.ui.ui.ui.ui.inverted.yellow.segment {
  background-color: #fbbd08;
  color: #fff;
}

.ui.ui.inverted.yellow.segment:not(.piled) {
  border-color: #fbbd08;
}

.ui.ui.ui.ui.ui.olive.segment:not(.inverted) {
  border-top: 2px solid #b5cc18;
}

.ui.ui.ui.ui.ui.inverted.olive.segment {
  background-color: #b5cc18;
  color: #fff;
}

.ui.ui.inverted.olive.segment:not(.piled) {
  border-color: #b5cc18;
}

.ui.ui.ui.ui.ui.green.segment:not(.inverted) {
  border-top: 2px solid #21ba45;
}

.ui.ui.ui.ui.ui.inverted.green.segment {
  background-color: #21ba45;
  color: #fff;
}

.ui.ui.inverted.green.segment:not(.piled) {
  border-color: #21ba45;
}

.ui.ui.ui.ui.ui.teal.segment:not(.inverted) {
  border-top: 2px solid #00b5ad;
}

.ui.ui.ui.ui.ui.inverted.teal.segment {
  background-color: #00b5ad;
  color: #fff;
}

.ui.ui.inverted.teal.segment:not(.piled) {
  border-color: #00b5ad;
}

.ui.ui.ui.ui.ui.blue.segment:not(.inverted) {
  border-top: 2px solid #2185d0;
}

.ui.ui.ui.ui.ui.inverted.blue.segment {
  background-color: #2185d0;
  color: #fff;
}

.ui.ui.inverted.blue.segment:not(.piled) {
  border-color: #2185d0;
}

.ui.ui.ui.ui.ui.violet.segment:not(.inverted) {
  border-top: 2px solid #6435c9;
}

.ui.ui.ui.ui.ui.inverted.violet.segment {
  background-color: #6435c9;
  color: #fff;
}

.ui.ui.inverted.violet.segment:not(.piled) {
  border-color: #6435c9;
}

.ui.ui.ui.ui.ui.purple.segment:not(.inverted) {
  border-top: 2px solid #a333c8;
}

.ui.ui.ui.ui.ui.inverted.purple.segment {
  background-color: #a333c8;
  color: #fff;
}

.ui.ui.inverted.purple.segment:not(.piled) {
  border-color: #a333c8;
}

.ui.ui.ui.ui.ui.pink.segment:not(.inverted) {
  border-top: 2px solid #e03997;
}

.ui.ui.ui.ui.ui.inverted.pink.segment {
  background-color: #e03997;
  color: #fff;
}

.ui.ui.inverted.pink.segment:not(.piled) {
  border-color: #e03997;
}

.ui.ui.ui.ui.ui.brown.segment:not(.inverted) {
  border-top: 2px solid #a5673f;
}

.ui.ui.ui.ui.ui.inverted.brown.segment {
  background-color: #a5673f;
  color: #fff;
}

.ui.ui.inverted.brown.segment:not(.piled) {
  border-color: #a5673f;
}

.ui.ui.ui.ui.ui.grey.segment:not(.inverted) {
  border-top: 2px solid #767676;
}

.ui.ui.ui.ui.ui.inverted.grey.segment {
  background-color: #767676;
  color: #fff;
}

.ui.ui.inverted.grey.segment:not(.piled) {
  border-color: #767676;
}

.ui.ui.ui.ui.ui.black.segment:not(.inverted) {
  border-top: 2px solid #1b1c1d;
}

.ui.ui.ui.ui.ui.inverted.black.segment {
  background-color: #1b1c1d;
  color: #fff;
}

.ui.ui.inverted.black.segment:not(.piled) {
  border-color: #1b1c1d;
}

/* -------------------
           Aligned
    -------------------- */

.ui[class*="left aligned"].segment {
  text-align: left;
}

.ui[class*="right aligned"].segment {
  text-align: right;
}

.ui[class*="center aligned"].segment {
  text-align: center;
}

/* -------------------
           Floated
    -------------------- */

.ui.floated.segment,
.ui[class*="left floated"].segment {
  float: left;
  margin-right: 1em;
}

.ui[class*="right floated"].segment {
  float: right;
  margin-left: 1em;
}

/* -------------------
          Inverted
    -------------------- */

.ui.inverted.segments,
.ui.inverted.segments .segment,
.ui.inverted.segment {
  border: none;
  box-shadow: none;
}

.ui.inverted.segments .segment,
.ui.inverted.segment,
.ui.primary.inverted.segment {
  background: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

/* Nested */

.ui.inverted.segment .segment {
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.segment .inverted.segment {
  color: rgba(255, 255, 255, 0.9);
}

/* Attached */

.ui.ui.inverted.attached.segment {
  border-color: #555;
}

/* Loading */

.ui.inverted.loading.segments,
.ui.inverted.loading.segment {
  color: #fff;
}

.ui.inverted.loading.segments::before,
.ui.inverted.loading.segment::before {
  background: rgba(0, 0, 0, 0.85);
}

/* -------------------
     Emphasis
-------------------- */

/* Secondary */

.ui.secondary.segment {
  background: #f3f4f5;
  color: rgba(0, 0, 0, 0.6);
}

.ui.secondary.inverted.segment {
  background: #4c4f52 linear-gradient(rgba(255, 255, 255, 0.2) 0, rgba(255, 255, 255, 0.2) 100%);
  color: rgba(255, 255, 255, 0.8);
}

/* Tertiary */

.ui.tertiary.segment {
  background: #dcddde;
  color: rgba(0, 0, 0, 0.6);
}

.ui.tertiary.inverted.segment {
  background: #717579 linear-gradient(rgba(255, 255, 255, 0.35) 0, rgba(255, 255, 255, 0.35) 100%);
  color: rgba(255, 255, 255, 0.8);
}

/* -------------------
          Attached
    -------------------- */

/* Middle */

.ui.attached.segment {
  top: 0;
  bottom: 0;
  border-radius: 0;
  margin: 0 -1px;
  width: calc(100% + 2px);
  max-width: calc(100% + 2px);
  box-shadow: none;
}

.ui.attached.segment:not(.basic) {
  border: 1px solid #d4d4d5;
}

.ui.attached:not(.message):not(.text) + .ui.attached.segment:not(.top):not([class*="left attached"]):not([class*="right attached"]) {
  border-top: none;
}

/* Top */

.ui.segment[class*="top attached"] {
  bottom: 0;
  margin-bottom: 0;
  top: 0;
  margin-top: 1rem;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.segment[class*="top attached"]:first-child {
  margin-top: 0;
}

.ui.tab.segment[class*="top attached"]:first-child {
  margin-top: 1rem;
}

/* Bottom */

.ui.segment[class*="bottom attached"] {
  bottom: 0;
  margin-top: 0;
  top: 0;
  margin-bottom: 1rem;
  box-shadow: 0 2px 1px -1px rgba(34, 36, 38, 0.15);
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.segment[class*="bottom attached"]:last-child {
  margin-bottom: 0;
}

.ui.tab.segment[class*="bottom attached"]:last-child {
  margin-bottom: 1rem;
}

.ui[class*="left attached"].segment {
  margin-right: 0;
}

.ui[class*="right attached"].segment {
  margin-left: 0;
}

.ui.seamless.attached.segment:not(.top):not(.bottom):not([class*="right attached"]):not([class*="left attached"]) {
  border-top: none;
  border-bottom: none;
}

.ui.seamless.attached.segment.top {
  border-bottom: none;
}

.ui.seamless.attached.segment.bottom {
  border-top: none;
}

.ui.seamless.attached.segment[class*="left attached"] {
  border-right: none;
}

.ui.seamless.attached.segment[class*="right attached"] {
  border-left: none;
}

/* --------------
         Fitted
    --------------- */

.ui.fitted.segment:not(.horizontally) {
  padding-top: 0;
  padding-bottom: 0;
}

.ui.fitted.segment:not(.vertically) {
  padding-left: 0;
  padding-right: 0;
}

/* --------------
      Scrolling
    --------------- */

.ui.scrolling.segment {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

@media only screen and (max-width: 767.98px) {
  .ui.scrolling.segment.short {
    max-height: 11.25em;
  }

  .ui.scrolling.segment.short.resizable {
    height: 11.25em;
  }

  .ui.scrolling.segment[class*="very short"] {
    max-height: 7.5em;
  }

  .ui.scrolling.segment[class*="very short"].resizable {
    height: 7.5em;
  }

  .ui.scrolling.segment {
    max-height: 15em;
  }

  .ui.scrolling.segment.resizable {
    height: 15em;
  }

  .ui.scrolling.segment.long {
    max-height: 30em;
  }

  .ui.scrolling.segment.long.resizable {
    height: 30em;
  }

  .ui.scrolling.segment[class*="very long"] {
    max-height: 45em;
  }

  .ui.scrolling.segment[class*="very long"].resizable {
    height: 45em;
  }
}

@media only screen and (min-width: 768px) {
  .ui.scrolling.segment.short {
    max-height: 13.5em;
  }

  .ui.scrolling.segment.short.resizable {
    height: 13.5em;
  }

  .ui.scrolling.segment[class*="very short"] {
    max-height: 9em;
  }

  .ui.scrolling.segment[class*="very short"].resizable {
    height: 9em;
  }

  .ui.scrolling.segment {
    max-height: 18em;
  }

  .ui.scrolling.segment.resizable {
    height: 18em;
  }

  .ui.scrolling.segment.long {
    max-height: 36em;
  }

  .ui.scrolling.segment.long.resizable {
    height: 36em;
  }

  .ui.scrolling.segment[class*="very long"] {
    max-height: 54em;
  }

  .ui.scrolling.segment[class*="very long"].resizable {
    height: 54em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.scrolling.segment.short {
    max-height: 18em;
  }

  .ui.scrolling.segment.short.resizable {
    height: 18em;
  }

  .ui.scrolling.segment[class*="very short"] {
    max-height: 12em;
  }

  .ui.scrolling.segment[class*="very short"].resizable {
    height: 12em;
  }

  .ui.scrolling.segment {
    max-height: 24em;
  }

  .ui.scrolling.segment.resizable {
    height: 24em;
  }

  .ui.scrolling.segment.long {
    max-height: 48em;
  }

  .ui.scrolling.segment.long.resizable {
    height: 48em;
  }

  .ui.scrolling.segment[class*="very long"] {
    max-height: 72em;
  }

  .ui.scrolling.segment[class*="very long"].resizable {
    height: 72em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.scrolling.segment.short {
    max-height: 22.5em;
  }

  .ui.scrolling.segment.short.resizable {
    height: 22.5em;
  }

  .ui.scrolling.segment[class*="very short"] {
    max-height: 15em;
  }

  .ui.scrolling.segment[class*="very short"].resizable {
    height: 15em;
  }

  .ui.scrolling.segment {
    max-height: 30em;
  }

  .ui.scrolling.segment.resizable {
    height: 30em;
  }

  .ui.scrolling.segment.long {
    max-height: 60em;
  }

  .ui.scrolling.segment.long.resizable {
    height: 60em;
  }

  .ui.scrolling.segment[class*="very long"] {
    max-height: 90em;
  }

  .ui.scrolling.segment[class*="very long"].resizable {
    height: 90em;
  }
}

.ui.resizable.scrolling.segment {
  resize: vertical;
  max-height: none;
}

/* -------------------
        Size
-------------------- */

.ui.segments .segment,
.ui.segment {
  font-size: 1rem;
}

.ui.mini.segments .segment,
.ui.mini.segment {
  font-size: 0.78571429rem;
}

.ui.tiny.segments .segment,
.ui.tiny.segment {
  font-size: 0.85714286rem;
}

.ui.small.segments .segment,
.ui.small.segment {
  font-size: 0.92857143rem;
}

.ui.large.segments .segment,
.ui.large.segment {
  font-size: 1.14285714rem;
}

.ui.big.segments .segment,
.ui.big.segment {
  font-size: 1.28571429rem;
}

.ui.huge.segments .segment,
.ui.huge.segment {
  font-size: 1.42857143rem;
}

.ui.massive.segments .segment,
.ui.massive.segment {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Step
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Plural
*******************************/

.ui.steps {
  display: inline-flex;
  flex-direction: row;
  align-items: stretch;
  margin: 1em 0;
  background: "";
  box-shadow: none;
  line-height: 1.14285714em;
  border-radius: 0.28571429rem;
  border: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.steps:not(.unstackable) {
  flex-wrap: wrap;
}

/* First Steps */

.ui.steps:first-child {
  margin-top: 0;
}

/* Last Steps */

.ui.steps:last-child {
  margin-bottom: 0;
}

/*******************************
           Singular
*******************************/

.ui.steps .step {
  position: relative;
  display: flex;
  flex: 1 0 auto;
  flex-flow: row wrap;
  vertical-align: middle;
  align-items: center;
  justify-content: center;
  margin: 0 0;
  padding: 1.14285714em 2em;
  background: #fff;
  color: rgba(0, 0, 0, 0.87);
  box-shadow: none;
  border-radius: 0;
  border: none;
  border-right: 1px solid rgba(34, 36, 38, 0.15);
  transition: background-color 0.1s ease, opacity 0.1s ease, color 0.1s ease, box-shadow 0.1s ease;
}

/* Arrow */

.ui.steps .step::after {
  display: none;
  position: absolute;
  z-index: 2;
  content: "";
  top: 50%;
  right: 0;
  background-color: #fff;
  width: 1.14285714em;
  height: 1.14285714em;
  border-style: solid;
  border-color: rgba(34, 36, 38, 0.15);
  border-width: 0 1px 1px 0;
  transition: background-color 0.1s ease, opacity 0.1s ease, color 0.1s ease, box-shadow 0.1s ease;
  transform: translateY(-50%) translateX(50%) rotate(-45deg);
}

/* First Step */

.ui.steps .step:first-child {
  padding-left: 2em;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

/* Last Step */

.ui.steps .step:last-child {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
  border-right: none;
  margin-right: 0;
}

/* Only Step */

.ui.steps .step:only-child {
  border-radius: 0.28571429rem;
}

/*******************************
            Content
*******************************/

/* Title */

.ui.steps .step .title {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1.14285714em;
  font-weight: bold;
}

.ui.steps .step > .title {
  width: 100%;
}

/* Description */

.ui.steps .step .description {
  font-weight: normal;
  font-size: 0.92857143em;
  color: rgba(0, 0, 0, 0.87);
}

.ui.steps .step > .description {
  width: 100%;
}

.ui.steps .step .title ~ .description {
  margin-top: 0.25em;
}

/* Icon */

.ui.steps .step > i.icon {
  line-height: 1;
  font-size: 2.5em;
  margin: 0 1rem 0 0;
}

.ui.steps .step > i.icon,
.ui.steps .step > i.icon ~ .content {
  display: block;
  flex: 0 1 auto;
  align-self: center;
}

/* Horizontal Icon */

.ui.steps:not(.vertical) .step > i.icon {
  width: auto;
}

/* Link */

.ui.steps .link.step,
.ui.steps a.step {
  cursor: pointer;
}

/*******************************
            Types
*******************************/

/* --------------
         Ordered
    --------------- */

.ui.ordered.steps {
  counter-reset: ordered;
}

.ui.ordered.steps .step::before {
  display: block;
  position: static;
  text-align: center;
  content: counter(ordered);
  align-self: center;
  margin-right: 1rem;
  font-size: 2.5em;
  counter-increment: ordered;
  font-family: inherit;
  font-weight: bold;
}

.ui.ordered.steps .step > * {
  display: block;
  align-self: center;
}

/* --------------
        Vertical
    --------------- */

.ui.vertical.steps {
  display: inline-flex;
  flex-direction: column;
  overflow: visible;
}

.ui.vertical.steps .step {
  justify-content: flex-start;
  border-radius: 0;
  padding: 1.14285714em 2em;
  border-right: none;
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.vertical.steps .step:first-child {
  padding: 1.14285714em 2em;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.vertical.steps .step:last-child {
  border-bottom: none;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.vertical.steps .step:only-child {
  border-radius: 0.28571429rem;
}

/* Arrow */

.ui.vertical.steps .step::after {
  top: 50%;
  right: 0;
  border-width: 0 1px 1px 0;
  display: none;
}

.ui.right.vertical.steps .step::after {
  border-width: 1px 0 0 1px;
  left: 0;
  right: 100%;
  transform: translateY(-50%) translateX(-50%) rotate(-45deg);
}

.ui.vertical.steps .active.step::after {
  display: block;
}

.ui.vertical.steps .step:last-child::after {
  display: none;
}

.ui.vertical.steps .active.step:last-child::after {
  display: block;
}

/* ---------------
    Responsive
---------------- */

/* Mobile (Default) */

@media only screen and (max-width: 767.98px) {
  .ui.steps:not(.unstackable) {
    display: inline-flex;
    overflow: visible;
    flex-direction: column;
  }

  .ui.steps:not(.unstackable) .step {
    width: 100% !important;
    flex-direction: column;
    border-radius: 0;
    padding: 1.14285714em 2em;
    border-right: none;
    border-bottom: 1px solid rgba(34, 36, 38, 0.15);
  }

  .ui.steps:not(.unstackable) .step:first-child {
    padding: 1.14285714em 2em;
    border-radius: 0.28571429rem 0.28571429rem 0 0;
  }

  .ui.steps:not(.unstackable) .step:last-child {
    border-radius: 0 0 0.28571429rem 0.28571429rem;
    border-bottom: none;
  }

  /* Arrow */

  .ui.steps:not(.unstackable) .step::after {
    top: unset;
    bottom: -1.14285714em;
    right: 50%;
    transform: translateY(-50%) translateX(50%) rotate(45deg);
  }

  .ui.vertical.steps .active.step:last-child::after {
    display: none;
  }

  /* Content */

  .ui.steps:not(.unstackable) .step .content {
    text-align: center;
  }

  /* Icon */

  .ui.steps:not(.unstackable) .step > i.icon,
  .ui.ordered.steps:not(.unstackable) .step::before {
    margin: 0 0 1rem 0;
  }
}

/*******************************
             States
*******************************/

/* Link Hover */

.ui.steps .link.step:hover::after,
.ui.steps .link.step:hover,
.ui.steps a.step:hover::after,
.ui.steps a.step:hover {
  background: #f9fafb;
  color: rgba(0, 0, 0, 0.8);
}

/* Link Down */

.ui.steps .link.step:active::after,
.ui.steps .link.step:active,
.ui.steps a.step:active::after,
.ui.steps a.step:active {
  background: #f3f4f5;
  color: rgba(0, 0, 0, 0.9);
}

/* Active */

.ui.steps .step.active {
  cursor: auto;
  background: #f3f4f5;
}

.ui.steps .step.active::after {
  background: #f3f4f5;
}

.ui.steps .step.active .title {
  color: #4183c4;
}

.ui.ordered.steps .step.active::before,
.ui.steps .active.step i.icon {
  color: rgba(0, 0, 0, 0.85);
}

/* Active Arrow */

.ui.steps .step::after {
  display: block;
}

.ui.steps .active.step::after {
  display: block;
}

.ui.steps .step:last-child::after {
  display: none;
}

.ui.steps .active.step:last-child::after {
  display: none;
}

/* Active Hover */

.ui.steps .link.active.step:hover::after,
.ui.steps .link.active.step:hover,
.ui.steps a.active.step:hover::after,
.ui.steps a.active.step:hover {
  cursor: pointer;
  background: #dcddde;
  color: rgba(0, 0, 0, 0.87);
}

/* Completed */

.ui.steps .step.completed > i.icon::before,
.ui.ordered.steps .step.completed::before {
  color: #21ba45;
}

/* Disabled */

.ui.steps .disabled.step {
  cursor: auto;
  background: #fff;
  pointer-events: none;
}

.ui.steps .disabled.step,
.ui.steps .disabled.step .title,
.ui.steps .disabled.step .description {
  color: rgba(40, 40, 40, 0.3);
}

.ui.steps .disabled.step::after {
  background: #fff;
}

/*******************************
           Variations
*******************************/

/* --------------
       Stackable
    --------------- */

/* Tablet Or Below */

@media only screen and (max-width: 991.98px) {
  .ui[class*="tablet stackable"].steps {
    display: inline-flex;
    overflow: visible;
    flex-direction: column;
  }

  /* Steps */

  .ui[class*="tablet stackable"].steps .step {
    flex-direction: column;
    border-radius: 0;
    padding: 1.14285714em 2em;
    border-right: none;
    border-bottom: 1px solid rgba(34, 36, 38, 0.15);
  }

  .ui[class*="tablet stackable"].steps .step:first-child {
    padding: 1.14285714em 2em;
    border-radius: 0.28571429rem 0.28571429rem 0 0;
  }

  .ui[class*="tablet stackable"].steps .step:last-child {
    border-radius: 0 0 0.28571429rem 0.28571429rem;
    border-bottom: none;
  }

  /* Arrow */

  .ui[class*="tablet stackable"].steps .step::after {
    top: unset;
    bottom: -1.14285714em;
    right: 50%;
    transform: translateY(-50%) translateX(50%) rotate(45deg);
  }

  /* Content */

  .ui[class*="tablet stackable"].steps .step .content {
    text-align: center;
  }

  /* Icon */

  .ui[class*="tablet stackable"].steps .step > i.icon,
  .ui[class*="tablet stackable"].ordered.steps .step::before {
    margin: 0 0 1rem 0;
  }
}

/* --------------
          Fluid
    --------------- */

/* Fluid */

.ui.fluid.steps {
  display: flex;
  width: 100%;
}

/* --------------
        Attached
    --------------- */

/* Top */

.ui.attached.steps {
  width: calc(100% + 2px) !important;
  margin: 0 -1px 0;
  max-width: calc(100% + 2px);
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.attached.steps .step:first-child {
  border-radius: 0.28571429rem 0 0 0;
}

.ui.attached.steps .step:last-child {
  border-radius: 0 0.28571429rem 0 0;
}

/* Bottom */

.ui.bottom.attached.steps {
  margin: 0 -1px 0;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.bottom.attached.steps .step:first-child {
  border-radius: 0 0 0 0.28571429rem;
}

.ui.bottom.attached.steps .step:last-child {
  border-radius: 0 0 0.28571429rem 0;
}

/* -------------------
        Evenly Divided
    -------------------- */

.ui.one.steps,
.ui.two.steps,
.ui.three.steps,
.ui.four.steps,
.ui.five.steps,
.ui.six.steps,
.ui.seven.steps,
.ui.eight.steps {
  width: 100%;
}

.ui.one.steps > .step,
.ui.two.steps > .step,
.ui.three.steps > .step,
.ui.four.steps > .step,
.ui.five.steps > .step,
.ui.six.steps > .step,
.ui.seven.steps > .step,
.ui.eight.steps > .step {
  flex-wrap: nowrap;
}

.ui.one.steps > .step {
  width: 100%;
}

.ui.two.steps > .step {
  width: 50%;
}

.ui.three.steps > .step {
  width: 33.333%;
}

.ui.four.steps > .step {
  width: 25%;
}

.ui.five.steps > .step {
  width: 20%;
}

.ui.six.steps > .step {
  width: 16.666%;
}

.ui.seven.steps > .step {
  width: 14.285%;
}

.ui.eight.steps > .step {
  width: 12.5%;
}

/* -------------------
       Sizes
-------------------- */

.ui.steps .step,
.ui.step {
  font-size: 1rem;
}

.ui.mini.steps .step,
.ui.mini.step {
  font-size: 0.78571429rem;
}

.ui.tiny.steps .step,
.ui.tiny.step {
  font-size: 0.85714286rem;
}

.ui.small.steps .step,
.ui.small.step {
  font-size: 0.92857143rem;
}

.ui.large.steps .step,
.ui.large.step {
  font-size: 1.14285714rem;
}

.ui.big.steps .step,
.ui.big.step {
  font-size: 1.28571429rem;
}

.ui.huge.steps .step,
.ui.huge.step {
  font-size: 1.42857143rem;
}

.ui.massive.steps .step,
.ui.massive.step {
  font-size: 1.71428571rem;
}

/* --------------
        Inverted
    --------------- */

.ui.inverted.steps {
  border: 1px solid #555;
}

.ui.inverted.steps .step {
  color: rgba(255, 255, 255, 0.9);
  background: #1b1c1d;
  border-color: #555;
}

.ui.inverted.steps .step::after {
  background-color: #1b1c1d;
  border-color: #555;
}

.ui.inverted.steps .step .description {
  color: rgba(255, 255, 255, 0.9);
}

/* Active */

.ui.inverted.steps .step.active,
.ui.inverted.steps .step.active::after {
  background: #333;
}

.ui.inverted.ordered.steps .step.active::before,
.ui.inverted.steps .active.step i.icon {
  color: #ffffff;
}

/* Disabled */

.ui.inverted.steps .disabled.step,
.ui.inverted.steps .disabled.step::after {
  background: #222;
}

.ui.inverted.steps .disabled.step,
.ui.inverted.steps .disabled.step .title,
.ui.inverted.steps .disabled.step .description {
  color: rgba(225, 225, 225, 0.3);
}

/* Link Hover */

.ui.inverted.steps .link.step:hover::after,
.ui.inverted.steps .link.step:hover,
.ui.inverted.steps a.step:hover::after,
.ui.inverted.steps a.step:hover {
  background: #3f3f3f;
  color: #ffffff;
}

/* Link Down */

.ui.inverted.steps .link.step:active::after,
.ui.inverted.steps .link.step:active,
.ui.inverted.steps a.step:active::after,
.ui.inverted.steps a.step:active {
  background: #444;
  color: #ffffff;
}

/*******************************
         Theme Overrides
*******************************/

@font-face {
  font-family: Step;
  src: url("data:application/font-woff;base64,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") format("woff");
}

.ui.steps .step.completed > .icon::before,
.ui.ordered.steps .step.completed::before {
  font-family: Step;
  content: "\e800";
  /* '' */
}

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Text
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Text
*******************************/

span.ui.text {
  line-height: 1;
}

span.ui.primary.text {
  color: #2185d0;
}

span.ui.inverted.primary.text {
  color: #54c8ff;
}

span.ui.secondary.text {
  color: #1b1c1d;
}

span.ui.inverted.secondary.text {
  color: #545454;
}

span.ui.red.text {
  color: #db2828;
}

span.ui.inverted.red.text {
  color: #ff695e;
}

span.ui.orange.text {
  color: #f2711c;
}

span.ui.inverted.orange.text {
  color: #ff851b;
}

span.ui.yellow.text {
  color: #fbbd08;
}

span.ui.inverted.yellow.text {
  color: #ffe21f;
}

span.ui.olive.text {
  color: #b5cc18;
}

span.ui.inverted.olive.text {
  color: #d9e778;
}

span.ui.green.text {
  color: #21ba45;
}

span.ui.inverted.green.text {
  color: #2ecc40;
}

span.ui.teal.text {
  color: #00b5ad;
}

span.ui.inverted.teal.text {
  color: #6dffff;
}

span.ui.blue.text {
  color: #2185d0;
}

span.ui.inverted.blue.text {
  color: #54c8ff;
}

span.ui.violet.text {
  color: #6435c9;
}

span.ui.inverted.violet.text {
  color: #a291fb;
}

span.ui.purple.text {
  color: #a333c8;
}

span.ui.inverted.purple.text {
  color: #dc73ff;
}

span.ui.pink.text {
  color: #e03997;
}

span.ui.inverted.pink.text {
  color: #ff8edf;
}

span.ui.brown.text {
  color: #a5673f;
}

span.ui.inverted.brown.text {
  color: #d67c1c;
}

span.ui.grey.text {
  color: #767676;
}

span.ui.inverted.grey.text {
  color: #dcddde;
}

span.ui.black.text {
  color: #1b1c1d;
}

span.ui.inverted.black.text {
  color: #545454;
}

span.ui.error.text {
  color: #db2828;
}

span.ui.info.text {
  color: #31ccec;
}

span.ui.success.text {
  color: #21ba45;
}

span.ui.warning.text {
  color: #f2c037;
}

span.ui.disabled.text {
  opacity: 0.45;
}

/* Sizes */

span.ui.medium.text {
  font-size: 1em;
}

span.ui.mini.text {
  font-size: 0.4em;
}

span.ui.tiny.text {
  font-size: 0.5em;
}

span.ui.small.text {
  font-size: 0.75em;
}

span.ui.large.text {
  font-size: 1.5em;
}

span.ui.big.text {
  font-size: 2em;
}

span.ui.huge.text {
  font-size: 4em;
}

span.ui.massive.text {
  font-size: 8em;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Breadcrumb
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           Breadcrumb
*******************************/

.ui.breadcrumb {
  line-height: 1.4285em;
  display: inline-block;
  margin: 0 0;
  vertical-align: middle;
}

.ui.breadcrumb:first-child {
  margin-top: 0;
}

.ui.breadcrumb:last-child {
  margin-bottom: 0;
}

/*******************************
          Content
*******************************/

/* Divider */

.ui.breadcrumb .divider {
  display: inline-block;
  opacity: 0.7;
  margin: 0 0.21428571rem;
  font-size: 0.92857143em;
  color: rgba(0, 0, 0, 0.4);
  vertical-align: baseline;
}

/* Link */

.ui.breadcrumb a {
  color: #4183c4;
}

.ui.breadcrumb a:hover {
  color: #1e70bf;
}

/* Icon Divider */

.ui.breadcrumb .icon.divider {
  font-size: 0.85714286em;
  vertical-align: baseline;
}

/* Section */

.ui.breadcrumb a.section {
  cursor: pointer;
}

.ui.breadcrumb .section {
  display: inline-block;
  margin: 0;
  padding: 0;
}

/* Loose Coupling */

.ui.breadcrumb.segment {
  display: inline-block;
  padding: 0.78571429em 1em;
}

/* Inverted */

.ui.inverted.breadcrumb {
  color: #dcddde;
}

.ui.inverted.breadcrumb > .active.section {
  color: #fff;
}

.ui.inverted.breadcrumb > .divider {
  color: rgba(255, 255, 255, 0.7);
}

/*******************************
            States
*******************************/

.ui.breadcrumb .active.section {
  font-weight: bold;
}

/*******************************
           Variations
*******************************/

.ui.breadcrumb {
  font-size: 1rem;
}

.ui.mini.breadcrumb {
  font-size: 0.78571429rem;
}

.ui.tiny.breadcrumb {
  font-size: 0.85714286rem;
}

.ui.small.breadcrumb {
  font-size: 0.92857143rem;
}

.ui.large.breadcrumb {
  font-size: 1.14285714rem;
}

.ui.big.breadcrumb {
  font-size: 1.28571429rem;
}

.ui.huge.breadcrumb {
  font-size: 1.42857143rem;
}

.ui.massive.breadcrumb {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Form
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Elements
*******************************/

/* --------------------
        Form
--------------------- */

.ui.form {
  position: relative;
  max-width: 100%;
}

/* --------------------
        Content
--------------------- */

.ui.form > p {
  margin: 1em 0;
}

/* --------------------
        Field
--------------------- */

.ui.form .field {
  clear: both;
  margin: 0 0 1em;
}

.ui.form .fields .fields,
.ui.form .field:last-child,
.ui.form .fields:last-child .field {
  margin-bottom: 0;
}

.ui.form .fields .field {
  clear: both;
  margin: 0;
}

/* --------------------
        Labels
--------------------- */

.ui.form .field > label {
  display: block;
  margin: 0 0 0.28571429rem 0;
  font-size: 0.92857143em;
  font-weight: bold;
  text-transform: none;
}

.ui.form:not(.inverted) .field > label:not(.button) {
  color: rgba(0, 0, 0, 0.87);
}

/* --------------------
    Standard Inputs
--------------------- */

.ui.form textarea,
.ui.form input:not([type]),
.ui.form input[type="color"],
.ui.form input[type="date"],
.ui.form input[type="datetime-local"],
.ui.form input[type="email"],
.ui.form input[type="month"],
.ui.form input[type="number"],
.ui.form input[type="password"],
.ui.form input[type="search"],
.ui.form input[type="tel"],
.ui.form input[type="time"],
.ui.form input[type="text"],
.ui.form input[type="file"],
.ui.form input[type="url"],
.ui.form input[type="week"] {
  width: 100%;
  vertical-align: top;
}

/* Set max height on unusual input */

.ui.form ::-webkit-datetime-edit,
.ui.form ::-webkit-inner-spin-button {
  height: 1.21428571em;
}

.ui.form input:not([type]),
.ui.form input[type="color"],
.ui.form input[type="date"],
.ui.form input[type="datetime-local"],
.ui.form input[type="email"],
.ui.form input[type="month"],
.ui.form input[type="number"],
.ui.form input[type="password"],
.ui.form input[type="search"],
.ui.form input[type="tel"],
.ui.form input[type="time"],
.ui.form input[type="text"],
.ui.form input[type="file"],
.ui.form input[type="url"],
.ui.form input[type="week"] {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  margin: 0;
  outline: none;
  -webkit-appearance: none;
  -webkit-tap-highlight-color: rgba(255, 255, 255, 0);
  line-height: 1.21428571em;
  padding: 0.67857143em 1em;
  font-size: 1em;
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0.28571429rem;
  box-shadow: 0 0 0 0 transparent inset;
  transition: color 0.1s ease, border-color 0.1s ease;
}

.ui.form input[type="color"] {
  padding: initial;
}

.ui.form input::-webkit-calendar-picker-indicator {
  padding: 0;
  opacity: 0.5;
  -webkit-transition: opacity 0.3s ease;
  transition: opacity 0.3s ease;
  cursor: pointer;
}

/* Text Area */

.ui.input textarea,
.ui.form textarea {
  margin: 0;
  -webkit-appearance: none;
  -webkit-tap-highlight-color: rgba(255, 255, 255, 0);
  padding: 0.78571429em 1em;
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  outline: none;
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0.28571429rem;
  box-shadow: 0 0 0 0 transparent inset;
  transition: color 0.1s ease, border-color 0.1s ease;
  font-size: 1em;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  line-height: 1.2857;
  resize: vertical;
}

.ui.form textarea:not([rows]) {
  height: 12em;
  min-height: 8em;
  max-height: 24em;
}

.ui.form textarea,
.ui.form input[type="checkbox"] {
  vertical-align: top;
}

/* --------------------
    Checkbox margin
--------------------- */

.ui.form .fields:not(.grouped):not(.inline) .field:not(:only-child) label + .ui.ui.checkbox {
  margin-top: 0.7em;
}

.ui.form .fields:not(.grouped):not(.inline) .field:not(:only-child) .ui.checkbox {
  margin-top: 2.41428571em;
}

.ui.form .fields:not(.grouped):not(.inline) .field:not(:only-child) .ui.toggle.checkbox {
  margin-top: 2.21428571em;
}

.ui.form .fields:not(.grouped):not(.inline) .field:not(:only-child) .ui.slider.checkbox {
  margin-top: 2.61428571em;
}

.ui.ui.form .field .fields .field:not(:only-child) .ui.checkbox {
  margin-top: 0.6em;
}

.ui.ui.form .field .fields .field:not(:only-child) .ui.toggle.checkbox {
  margin-top: 0.5em;
}

.ui.ui.form .field .fields .field:not(:only-child) .ui.slider.checkbox {
  margin-top: 0.7em;
}

/* --------------------
          Transparent
    --------------------- */

.ui.form .field .transparent.input:not(.icon) input,
.ui.form .field input.transparent,
.ui.form .field textarea.transparent {
  padding: 0.67857143em 1em;
}

.ui.form .field input.transparent,
.ui.form .field textarea.transparent {
  border-color: transparent !important;
  background-color: transparent !important;
  box-shadow: none !important;
}

/* --------------------------
  Input w/ attached Button
--------------------------- */

.ui.form input.attached {
  width: auto;
}

/* --------------------
     Basic Select
--------------------- */

.ui.form select {
  display: block;
  height: auto;
  width: 100%;
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
  box-shadow: 0 0 0 0 transparent inset;
  padding: 0.62em 1em;
  color: rgba(0, 0, 0, 0.87);
  transition: color 0.1s ease, border-color 0.1s ease;
}

/* --------------------
       Dropdown
--------------------- */

/* Block */

.ui.form .field > .selection.dropdown:not(.compact) {
  min-width: auto;
  width: 100%;
}

.ui.form .field > .selection.dropdown > .dropdown.icon {
  float: right;
}

/* Inline */

.ui.form .inline.fields .field > .selection.dropdown,
.ui.form .inline.field > .selection.dropdown {
  width: auto;
}

.ui.form .inline.fields .field > .selection.dropdown > .dropdown.icon,
.ui.form .inline.field > .selection.dropdown > .dropdown.icon {
  float: none;
}

/* --------------------
       UI Input
--------------------- */

/* Block */

.ui.form .field .ui.input,
.ui.form .fields .field .ui.input,
.ui.form .wide.field .ui.input {
  width: 100%;
}

/* Inline */

.ui.form .inline.fields .field:not(.wide) .ui.input,
.ui.form .inline.field:not(.wide) .ui.input {
  width: auto;
  vertical-align: middle;
}

/* Auto Input */

.ui.form .fields .field .ui.input input,
.ui.form .field .ui.input input {
  width: auto;
}

/* Full Width Input */

.ui.form .ten.fields .ui.input input,
.ui.form .nine.fields .ui.input input,
.ui.form .eight.fields .ui.input input,
.ui.form .seven.fields .ui.input input,
.ui.form .six.fields .ui.input input,
.ui.form .five.fields .ui.input input,
.ui.form .four.fields .ui.input input,
.ui.form .three.fields .ui.input input,
.ui.form .two.fields .ui.input input,
.ui.form .wide.field .ui.input input {
  flex: 1 0 auto;
  width: 0;
}

/* --------------------
   Types of Messages
--------------------- */

.ui.form .error.message,
.ui.form .error.message:empty {
  display: none;
}

.ui.form .info.message,
.ui.form .info.message:empty {
  display: none;
}

.ui.form .success.message,
.ui.form .success.message:empty {
  display: none;
}

.ui.form .warning.message,
.ui.form .warning.message:empty {
  display: none;
}

/* Assumptions */

.ui.form .message:first-child {
  margin-top: 0;
}

/* --------------------
   Validation Prompt
--------------------- */

.ui.form .field .prompt.label {
  white-space: normal;
  background: #fff !important;
  border: 1px solid #e0b4b4 !important;
  color: #9f3a38 !important;
}

.ui.form .field .prompt.label li::before {
  color: #9f3a38;
}

.ui.form .inline.fields .field .prompt,
.ui.form .inline.field .prompt {
  vertical-align: top;
  margin: -0.25em 0 -0.5em 0.5em;
}

.ui.form .inline.fields .field .prompt::before,
.ui.form .inline.field .prompt::before {
  border-width: 0 0 1px 1px;
  bottom: auto;
  right: auto;
  top: 50%;
  left: 0;
}

/*******************************
            States
*******************************/

/* --------------------
      Autofilled
--------------------- */

.ui.form .field.field input:-webkit-autofill {
  box-shadow: 0 0 0 100px #fffff0 inset !important;
  border-color: #e5dfa1 !important;
}

/* Focus */

.ui.form .field.field input:-webkit-autofill:focus {
  box-shadow: 0 0 0 100px #fffff0 inset !important;
  border-color: #d5c315 !important;
}

/* --------------------
      Placeholder
--------------------- */

/* browsers require these rules separate */

.ui.form :-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.form ::-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.form ::placeholder {
  color: rgba(191, 191, 191, 0.87);
}

.ui.form :-ms-input-placeholder {
  color: rgba(191, 191, 191, 0.87) !important;
}

.ui.form :focus:-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.form :focus::-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.form :focus::placeholder {
  color: rgba(115, 115, 115, 0.87);
}

.ui.form :focus:-ms-input-placeholder {
  color: rgba(115, 115, 115, 0.87) !important;
}

/* --------------------
        Focus
--------------------- */

.ui.form input:not([type]):focus,
.ui.form input[type="color"]:focus,
.ui.form input[type="date"]:focus,
.ui.form input[type="datetime-local"]:focus,
.ui.form input[type="email"]:focus,
.ui.form input[type="month"]:focus,
.ui.form input[type="number"]:focus,
.ui.form input[type="password"]:focus,
.ui.form input[type="search"]:focus,
.ui.form input[type="tel"]:focus,
.ui.form input[type="time"]:focus,
.ui.form input[type="text"]:focus,
.ui.form input[type="file"]:focus,
.ui.form input[type="url"]:focus,
.ui.form input[type="week"]:focus {
  color: rgba(0, 0, 0, 0.95);
  border-color: #85b7d9;
  border-radius: 0.28571429rem;
  background: #fff;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.35) inset;
}

.ui.form .ui.action.input:not([class*="left action"]) input:not([type]):focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="color"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="date"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="datetime-local"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="email"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="month"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="number"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="password"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="search"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="tel"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="time"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="text"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="file"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="url"]:focus,
.ui.form .ui.action.input:not([class*="left action"]) input[type="week"]:focus {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.form .ui[class*="left action"].input input:not([type]),
.ui.form .ui[class*="left action"].input input[type="color"],
.ui.form .ui[class*="left action"].input input[type="date"],
.ui.form .ui[class*="left action"].input input[type="datetime-local"],
.ui.form .ui[class*="left action"].input input[type="email"],
.ui.form .ui[class*="left action"].input input[type="month"],
.ui.form .ui[class*="left action"].input input[type="number"],
.ui.form .ui[class*="left action"].input input[type="password"],
.ui.form .ui[class*="left action"].input input[type="search"],
.ui.form .ui[class*="left action"].input input[type="tel"],
.ui.form .ui[class*="left action"].input input[type="time"],
.ui.form .ui[class*="left action"].input input[type="text"],
.ui.form .ui[class*="left action"].input input[type="file"],
.ui.form .ui[class*="left action"].input input[type="url"],
.ui.form .ui[class*="left action"].input input[type="week"] {
  border-bottom-left-radius: 0;
  border-top-left-radius: 0;
}

.ui.form textarea:focus {
  color: rgba(0, 0, 0, 0.95);
  border-color: #85b7d9;
  border-radius: 0.28571429rem;
  background: #fff;
  box-shadow: 0 0 0 0 rgba(34, 36, 38, 0.35) inset;
  -webkit-appearance: none;
}

/* Focus */

.ui.form input:focus::-webkit-calendar-picker-indicator {
  opacity: 1;
}

/* --------------------
            States
    --------------------- */

/* On Form */

.ui.form.error .error.message:not(:empty) {
  display: block;
}

.ui.form.error .compact.error.message:not(:empty) {
  display: inline-block;
}

.ui.form.error .icon.error.message:not(:empty) {
  display: flex;
}

/* On Field(s) */

.ui.form .fields.error .error.message:not(:empty),
.ui.form .field.error .error.message:not(:empty) {
  display: block;
}

.ui.form .fields.error .compact.error.message:not(:empty),
.ui.form .field.error .compact.error.message:not(:empty) {
  display: inline-block;
}

.ui.form .fields.error .icon.error.message:not(:empty),
.ui.form .field.error .icon.error.message:not(:empty) {
  display: flex;
}

.ui.ui.form .fields.error .field label,
.ui.ui.form .fields.error .field .ui.label:not(.corner),
.ui.ui.form .field.error label,
.ui.ui.form .field.error .ui.label:not(.corner),
.ui.ui.form .fields.error .field .input,
.ui.ui.form .field.error .input {
  color: #9f3a38;
}

@supports selector(:has(.f)) {
  .ui.form .fields:has(.error) > label {
    color: #9f3a38;
  }
}

.ui.form .fields.error .field .ui.label,
.ui.form .field.error .ui.label {
  background-color: #ecd1d1;
}

.ui.form .fields.error .field .corner.label,
.ui.form .field.error .corner.label {
  border-color: #9f3a38;
  color: #fff;
}

.ui.form .fields.error .field textarea,
.ui.form .fields.error .field select,
.ui.form .fields.error .field input:not([type]),
.ui.form .fields.error .field input[type="color"],
.ui.form .fields.error .field input[type="date"],
.ui.form .fields.error .field input[type="datetime-local"],
.ui.form .fields.error .field input[type="email"],
.ui.form .fields.error .field input[type="month"],
.ui.form .fields.error .field input[type="number"],
.ui.form .fields.error .field input[type="password"],
.ui.form .fields.error .field input[type="search"],
.ui.form .fields.error .field input[type="tel"],
.ui.form .fields.error .field input[type="time"],
.ui.form .fields.error .field input[type="text"],
.ui.form .fields.error .field input[type="file"],
.ui.form .fields.error .field input[type="url"],
.ui.form .fields.error .field input[type="week"],
.ui.form .field.error textarea,
.ui.form .field.error select,
.ui.form .field.error input:not([type]),
.ui.form .field.error input[type="color"],
.ui.form .field.error input[type="date"],
.ui.form .field.error input[type="datetime-local"],
.ui.form .field.error input[type="email"],
.ui.form .field.error input[type="month"],
.ui.form .field.error input[type="number"],
.ui.form .field.error input[type="password"],
.ui.form .field.error input[type="search"],
.ui.form .field.error input[type="tel"],
.ui.form .field.error input[type="time"],
.ui.form .field.error input[type="text"],
.ui.form .field.error input[type="file"],
.ui.form .field.error input[type="url"],
.ui.form .field.error input[type="week"] {
  color: #9f3a38;
  background: #fff6f6;
  border-color: #e0b4b4;
  border-radius: "";
  box-shadow: none;
}

.ui.form:not(.initial) .field input:invalid {
  color: #9f3a38;
  background: #fff6f6;
  border-color: #e0b4b4;
  border-radius: "";
  box-shadow: none;
}

.ui.form .field.error textarea:focus,
.ui.form .field.error select:focus,
.ui.form .field.error input:not([type]):focus,
.ui.form .field.error input[type="color"]:focus,
.ui.form .field.error input[type="date"]:focus,
.ui.form .field.error input[type="datetime-local"]:focus,
.ui.form .field.error input[type="email"]:focus,
.ui.form .field.error input[type="month"]:focus,
.ui.form .field.error input[type="number"]:focus,
.ui.form .field.error input[type="password"]:focus,
.ui.form .field.error input[type="search"]:focus,
.ui.form .field.error input[type="tel"]:focus,
.ui.form .field.error input[type="time"]:focus,
.ui.form .field.error input[type="text"]:focus,
.ui.form .field.error input[type="file"]:focus,
.ui.form .field.error input[type="url"]:focus,
.ui.form .field.error input[type="week"]:focus {
  background: #fff6f6;
  border-color: #e0b4b4;
  color: #9f3a38;
  box-shadow: none;
}

/* Preserve Native Select Stylings */

.ui.form .field.error select {
  -webkit-appearance: menulist-button;
}

/*------------------
            Input State
        --------------------*/

/* Transparent */

.ui.form .field.error .transparent.input input,
.ui.form .field.error .transparent.input textarea,
.ui.form .field.error input.transparent,
.ui.form .field.error textarea.transparent {
  background-color: #fff6f6 !important;
  color: #9f3a38 !important;
}

/* Autofilled */

.ui.form .error.error input:-webkit-autofill {
  box-shadow: 0 0 0 100px #fffaf0 inset !important;
  border-color: #e0b4b4 !important;
}

/* Placeholder */

.ui.form .error :-ms-input-placeholder {
  color: #e7bdbc;
}

.ui.form .error ::-ms-input-placeholder {
  color: #e7bdbc;
}

.ui.form .error ::placeholder {
  color: #e7bdbc;
}

.ui.form .error :-ms-input-placeholder {
  color: #e7bdbc !important;
}

.ui.form .error :focus:-ms-input-placeholder {
  color: #da9796;
}

.ui.form .error :focus::-ms-input-placeholder {
  color: #da9796;
}

.ui.form .error :focus::placeholder {
  color: #da9796;
}

.ui.form .error :focus:-ms-input-placeholder {
  color: #da9796 !important;
}

/*------------------
            Dropdown State
        --------------------*/

.ui.form .fields.error .field .ui.dropdown,
.ui.form .fields.error .field .ui.dropdown .item,
.ui.form .field.error .ui.dropdown,
.ui.form .field.error .ui.dropdown > .text,
.ui.form .field.error .ui.dropdown .item {
  background: #fff6f6;
  color: #9f3a38;
}

.ui.form .fields.error .field .ui.dropdown,
.ui.form .field.error .ui.dropdown {
  border-color: #e0b4b4 !important;
}

.ui.form .fields.error .field .ui.dropdown:hover,
.ui.form .field.error .ui.dropdown:hover {
  border-color: #e0b4b4 !important;
}

.ui.form .fields.error .field .ui.dropdown:hover .menu,
.ui.form .field.error .ui.dropdown:hover .menu {
  border-color: #e0b4b4;
}

.ui.form .fields.error .field .ui.multiple.selection.dropdown > .label,
.ui.form .field.error .ui.multiple.selection.dropdown > .label {
  background-color: #eacbcb;
  color: #9f3a38;
}

/* Hover */

.ui.form .fields.error .field .ui.dropdown .menu .item:hover,
.ui.form .field.error .ui.dropdown .menu .item:hover {
  background-color: #fbe7e7;
}

/* Selected */

.ui.form .fields.error .field .ui.dropdown .menu .selected.item,
.ui.form .field.error .ui.dropdown .menu .selected.item {
  background-color: #fbe7e7;
}

/* Active */

.ui.form .fields.error .field .ui.dropdown .menu .active.item,
.ui.form .field.error .ui.dropdown .menu .active.item {
  background-color: #fdcfcf !important;
}

/*--------------------
            Checkbox State
        ---------------------*/

.ui.form .fields.error .field .checkbox:not(.toggle):not(.slider) label,
.ui.form .field.error .checkbox:not(.toggle):not(.slider) label {
  color: #9f3a38;
}

.ui.form .fields.error .field .checkbox:not(.toggle):not(.slider) label::before,
.ui.form .field.error .checkbox:not(.toggle):not(.slider) label::before {
  background: #fff6f6;
  border-color: #e0b4b4;
}

.ui.form .fields.error .field .checkbox label::after,
.ui.form .field.error .checkbox label::after {
  color: #9f3a38;
}

.ui.inverted.form .fields.error .field label,
.ui.inverted.form .error.field label {
  color: #ecd1d1;
}

@supports selector(:has(.f)) {
  .ui.inverted.form .fields:has(.error) > label {
    color: #ecd1d1;
  }
}

/* On Form */

.ui.form.info .info.message:not(:empty) {
  display: block;
}

.ui.form.info .compact.info.message:not(:empty) {
  display: inline-block;
}

.ui.form.info .icon.info.message:not(:empty) {
  display: flex;
}

/* On Field(s) */

.ui.form .fields.info .info.message:not(:empty),
.ui.form .field.info .info.message:not(:empty) {
  display: block;
}

.ui.form .fields.info .compact.info.message:not(:empty),
.ui.form .field.info .compact.info.message:not(:empty) {
  display: inline-block;
}

.ui.form .fields.info .icon.info.message:not(:empty),
.ui.form .field.info .icon.info.message:not(:empty) {
  display: flex;
}

.ui.ui.form .fields.info .field label,
.ui.ui.form .fields.info .field .ui.label:not(.corner),
.ui.ui.form .field.info label,
.ui.ui.form .field.info .ui.label:not(.corner),
.ui.ui.form .fields.info .field .input,
.ui.ui.form .field.info .input {
  color: #276f86;
}

@supports selector(:has(.f)) {
  .ui.form .fields:has(.info) > label {
    color: #276f86;
  }
}

.ui.form .fields.info .field .ui.label,
.ui.form .field.info .ui.label {
  background-color: #c6e3e9;
}

.ui.form .fields.info .field .corner.label,
.ui.form .field.info .corner.label {
  border-color: #276f86;
  color: #fff;
}

.ui.form .fields.info .field textarea,
.ui.form .fields.info .field select,
.ui.form .fields.info .field input:not([type]),
.ui.form .fields.info .field input[type="color"],
.ui.form .fields.info .field input[type="date"],
.ui.form .fields.info .field input[type="datetime-local"],
.ui.form .fields.info .field input[type="email"],
.ui.form .fields.info .field input[type="month"],
.ui.form .fields.info .field input[type="number"],
.ui.form .fields.info .field input[type="password"],
.ui.form .fields.info .field input[type="search"],
.ui.form .fields.info .field input[type="tel"],
.ui.form .fields.info .field input[type="time"],
.ui.form .fields.info .field input[type="text"],
.ui.form .fields.info .field input[type="file"],
.ui.form .fields.info .field input[type="url"],
.ui.form .fields.info .field input[type="week"],
.ui.form .field.info textarea,
.ui.form .field.info select,
.ui.form .field.info input:not([type]),
.ui.form .field.info input[type="color"],
.ui.form .field.info input[type="date"],
.ui.form .field.info input[type="datetime-local"],
.ui.form .field.info input[type="email"],
.ui.form .field.info input[type="month"],
.ui.form .field.info input[type="number"],
.ui.form .field.info input[type="password"],
.ui.form .field.info input[type="search"],
.ui.form .field.info input[type="tel"],
.ui.form .field.info input[type="time"],
.ui.form .field.info input[type="text"],
.ui.form .field.info input[type="file"],
.ui.form .field.info input[type="url"],
.ui.form .field.info input[type="week"] {
  color: #276f86;
  background: #f8ffff;
  border-color: #a9d5de;
  border-radius: "";
  box-shadow: none;
}

.ui.form .field.info textarea:focus,
.ui.form .field.info select:focus,
.ui.form .field.info input:not([type]):focus,
.ui.form .field.info input[type="color"]:focus,
.ui.form .field.info input[type="date"]:focus,
.ui.form .field.info input[type="datetime-local"]:focus,
.ui.form .field.info input[type="email"]:focus,
.ui.form .field.info input[type="month"]:focus,
.ui.form .field.info input[type="number"]:focus,
.ui.form .field.info input[type="password"]:focus,
.ui.form .field.info input[type="search"]:focus,
.ui.form .field.info input[type="tel"]:focus,
.ui.form .field.info input[type="time"]:focus,
.ui.form .field.info input[type="text"]:focus,
.ui.form .field.info input[type="file"]:focus,
.ui.form .field.info input[type="url"]:focus,
.ui.form .field.info input[type="week"]:focus {
  background: #f8ffff;
  border-color: #a9d5de;
  color: #276f86;
  box-shadow: none;
}

/* Preserve Native Select Stylings */

.ui.form .field.info select {
  -webkit-appearance: menulist-button;
}

/*------------------
            Input State
        --------------------*/

/* Transparent */

.ui.form .field.info .transparent.input input,
.ui.form .field.info .transparent.input textarea,
.ui.form .field.info input.transparent,
.ui.form .field.info textarea.transparent {
  background-color: #f8ffff !important;
  color: #276f86 !important;
}

/* Autofilled */

.ui.form .info.info input:-webkit-autofill {
  box-shadow: 0 0 0 100px #f0faff inset !important;
  border-color: #b3e0e0 !important;
}

/* Placeholder */

.ui.form .info :-ms-input-placeholder {
  color: #98cfe1;
}

.ui.form .info ::-ms-input-placeholder {
  color: #98cfe1;
}

.ui.form .info ::placeholder {
  color: #98cfe1;
}

.ui.form .info :-ms-input-placeholder {
  color: #98cfe1 !important;
}

.ui.form .info :focus:-ms-input-placeholder {
  color: #70bdd6;
}

.ui.form .info :focus::-ms-input-placeholder {
  color: #70bdd6;
}

.ui.form .info :focus::placeholder {
  color: #70bdd6;
}

.ui.form .info :focus:-ms-input-placeholder {
  color: #70bdd6 !important;
}

/*------------------
            Dropdown State
        --------------------*/

.ui.form .fields.info .field .ui.dropdown,
.ui.form .fields.info .field .ui.dropdown .item,
.ui.form .field.info .ui.dropdown,
.ui.form .field.info .ui.dropdown > .text,
.ui.form .field.info .ui.dropdown .item {
  background: #f8ffff;
  color: #276f86;
}

.ui.form .fields.info .field .ui.dropdown,
.ui.form .field.info .ui.dropdown {
  border-color: #a9d5de !important;
}

.ui.form .fields.info .field .ui.dropdown:hover,
.ui.form .field.info .ui.dropdown:hover {
  border-color: #a9d5de !important;
}

.ui.form .fields.info .field .ui.dropdown:hover .menu,
.ui.form .field.info .ui.dropdown:hover .menu {
  border-color: #a9d5de;
}

.ui.form .fields.info .field .ui.multiple.selection.dropdown > .label,
.ui.form .field.info .ui.multiple.selection.dropdown > .label {
  background-color: #cce3ea;
  color: #276f86;
}

/* Hover */

.ui.form .fields.info .field .ui.dropdown .menu .item:hover,
.ui.form .field.info .ui.dropdown .menu .item:hover {
  background-color: #e9f2fb;
}

/* Selected */

.ui.form .fields.info .field .ui.dropdown .menu .selected.item,
.ui.form .field.info .ui.dropdown .menu .selected.item {
  background-color: #e9f2fb;
}

/* Active */

.ui.form .fields.info .field .ui.dropdown .menu .active.item,
.ui.form .field.info .ui.dropdown .menu .active.item {
  background-color: #cef1fd !important;
}

/*--------------------
            Checkbox State
        ---------------------*/

.ui.form .fields.info .field .checkbox:not(.toggle):not(.slider) label,
.ui.form .field.info .checkbox:not(.toggle):not(.slider) label {
  color: #276f86;
}

.ui.form .fields.info .field .checkbox:not(.toggle):not(.slider) label::before,
.ui.form .field.info .checkbox:not(.toggle):not(.slider) label::before {
  background: #f8ffff;
  border-color: #a9d5de;
}

.ui.form .fields.info .field .checkbox label::after,
.ui.form .field.info .checkbox label::after {
  color: #276f86;
}

.ui.inverted.form .fields.info .field label,
.ui.inverted.form .info.field label {
  color: #c6e3e9;
}

@supports selector(:has(.f)) {
  .ui.inverted.form .fields:has(.info) > label {
    color: #c6e3e9;
  }
}

/* On Form */

.ui.form.success .success.message:not(:empty) {
  display: block;
}

.ui.form.success .compact.success.message:not(:empty) {
  display: inline-block;
}

.ui.form.success .icon.success.message:not(:empty) {
  display: flex;
}

/* On Field(s) */

.ui.form .fields.success .success.message:not(:empty),
.ui.form .field.success .success.message:not(:empty) {
  display: block;
}

.ui.form .fields.success .compact.success.message:not(:empty),
.ui.form .field.success .compact.success.message:not(:empty) {
  display: inline-block;
}

.ui.form .fields.success .icon.success.message:not(:empty),
.ui.form .field.success .icon.success.message:not(:empty) {
  display: flex;
}

.ui.ui.form .fields.success .field label,
.ui.ui.form .fields.success .field .ui.label:not(.corner),
.ui.ui.form .field.success label,
.ui.ui.form .field.success .ui.label:not(.corner),
.ui.ui.form .fields.success .field .input,
.ui.ui.form .field.success .input {
  color: #2c662d;
}

@supports selector(:has(.f)) {
  .ui.form .fields:has(.success) > label {
    color: #2c662d;
  }
}

.ui.form .fields.success .field .ui.label,
.ui.form .field.success .ui.label {
  background-color: #b9d1ad;
}

.ui.form .fields.success .field .corner.label,
.ui.form .field.success .corner.label {
  border-color: #2c662d;
  color: #fff;
}

.ui.form .fields.success .field textarea,
.ui.form .fields.success .field select,
.ui.form .fields.success .field input:not([type]),
.ui.form .fields.success .field input[type="color"],
.ui.form .fields.success .field input[type="date"],
.ui.form .fields.success .field input[type="datetime-local"],
.ui.form .fields.success .field input[type="email"],
.ui.form .fields.success .field input[type="month"],
.ui.form .fields.success .field input[type="number"],
.ui.form .fields.success .field input[type="password"],
.ui.form .fields.success .field input[type="search"],
.ui.form .fields.success .field input[type="tel"],
.ui.form .fields.success .field input[type="time"],
.ui.form .fields.success .field input[type="text"],
.ui.form .fields.success .field input[type="file"],
.ui.form .fields.success .field input[type="url"],
.ui.form .fields.success .field input[type="week"],
.ui.form .field.success textarea,
.ui.form .field.success select,
.ui.form .field.success input:not([type]),
.ui.form .field.success input[type="color"],
.ui.form .field.success input[type="date"],
.ui.form .field.success input[type="datetime-local"],
.ui.form .field.success input[type="email"],
.ui.form .field.success input[type="month"],
.ui.form .field.success input[type="number"],
.ui.form .field.success input[type="password"],
.ui.form .field.success input[type="search"],
.ui.form .field.success input[type="tel"],
.ui.form .field.success input[type="time"],
.ui.form .field.success input[type="text"],
.ui.form .field.success input[type="file"],
.ui.form .field.success input[type="url"],
.ui.form .field.success input[type="week"] {
  color: #2c662d;
  background: #fcfff5;
  border-color: #a3c293;
  border-radius: "";
  box-shadow: none;
}

.ui.form .field.success textarea:focus,
.ui.form .field.success select:focus,
.ui.form .field.success input:not([type]):focus,
.ui.form .field.success input[type="color"]:focus,
.ui.form .field.success input[type="date"]:focus,
.ui.form .field.success input[type="datetime-local"]:focus,
.ui.form .field.success input[type="email"]:focus,
.ui.form .field.success input[type="month"]:focus,
.ui.form .field.success input[type="number"]:focus,
.ui.form .field.success input[type="password"]:focus,
.ui.form .field.success input[type="search"]:focus,
.ui.form .field.success input[type="tel"]:focus,
.ui.form .field.success input[type="time"]:focus,
.ui.form .field.success input[type="text"]:focus,
.ui.form .field.success input[type="file"]:focus,
.ui.form .field.success input[type="url"]:focus,
.ui.form .field.success input[type="week"]:focus {
  background: #fcfff5;
  border-color: #a3c293;
  color: #2c662d;
  box-shadow: none;
}

/* Preserve Native Select Stylings */

.ui.form .field.success select {
  -webkit-appearance: menulist-button;
}

/*------------------
            Input State
        --------------------*/

/* Transparent */

.ui.form .field.success .transparent.input input,
.ui.form .field.success .transparent.input textarea,
.ui.form .field.success input.transparent,
.ui.form .field.success textarea.transparent {
  background-color: #fcfff5 !important;
  color: #2c662d !important;
}

/* Autofilled */

.ui.form .success.success input:-webkit-autofill {
  box-shadow: 0 0 0 100px #f0fff0 inset !important;
  border-color: #bee0b3 !important;
}

/* Placeholder */

.ui.form .success :-ms-input-placeholder {
  color: #8fcf90;
}

.ui.form .success ::-ms-input-placeholder {
  color: #8fcf90;
}

.ui.form .success ::placeholder {
  color: #8fcf90;
}

.ui.form .success :-ms-input-placeholder {
  color: #8fcf90 !important;
}

.ui.form .success :focus:-ms-input-placeholder {
  color: #6cbf6d;
}

.ui.form .success :focus::-ms-input-placeholder {
  color: #6cbf6d;
}

.ui.form .success :focus::placeholder {
  color: #6cbf6d;
}

.ui.form .success :focus:-ms-input-placeholder {
  color: #6cbf6d !important;
}

/*------------------
            Dropdown State
        --------------------*/

.ui.form .fields.success .field .ui.dropdown,
.ui.form .fields.success .field .ui.dropdown .item,
.ui.form .field.success .ui.dropdown,
.ui.form .field.success .ui.dropdown > .text,
.ui.form .field.success .ui.dropdown .item {
  background: #fcfff5;
  color: #2c662d;
}

.ui.form .fields.success .field .ui.dropdown,
.ui.form .field.success .ui.dropdown {
  border-color: #a3c293 !important;
}

.ui.form .fields.success .field .ui.dropdown:hover,
.ui.form .field.success .ui.dropdown:hover {
  border-color: #a3c293 !important;
}

.ui.form .fields.success .field .ui.dropdown:hover .menu,
.ui.form .field.success .ui.dropdown:hover .menu {
  border-color: #a3c293;
}

.ui.form .fields.success .field .ui.multiple.selection.dropdown > .label,
.ui.form .field.success .ui.multiple.selection.dropdown > .label {
  background-color: #cceacc;
  color: #2c662d;
}

/* Hover */

.ui.form .fields.success .field .ui.dropdown .menu .item:hover,
.ui.form .field.success .ui.dropdown .menu .item:hover {
  background-color: #e9fbe9;
}

/* Selected */

.ui.form .fields.success .field .ui.dropdown .menu .selected.item,
.ui.form .field.success .ui.dropdown .menu .selected.item {
  background-color: #e9fbe9;
}

/* Active */

.ui.form .fields.success .field .ui.dropdown .menu .active.item,
.ui.form .field.success .ui.dropdown .menu .active.item {
  background-color: #dafdce !important;
}

/*--------------------
            Checkbox State
        ---------------------*/

.ui.form .fields.success .field .checkbox:not(.toggle):not(.slider) label,
.ui.form .field.success .checkbox:not(.toggle):not(.slider) label {
  color: #2c662d;
}

.ui.form .fields.success .field .checkbox:not(.toggle):not(.slider) label::before,
.ui.form .field.success .checkbox:not(.toggle):not(.slider) label::before {
  background: #fcfff5;
  border-color: #a3c293;
}

.ui.form .fields.success .field .checkbox label::after,
.ui.form .field.success .checkbox label::after {
  color: #2c662d;
}

.ui.inverted.form .fields.success .field label,
.ui.inverted.form .success.field label {
  color: #b9d1ad;
}

@supports selector(:has(.f)) {
  .ui.inverted.form .fields:has(.success) > label {
    color: #b9d1ad;
  }
}

/* On Form */

.ui.form.warning .warning.message:not(:empty) {
  display: block;
}

.ui.form.warning .compact.warning.message:not(:empty) {
  display: inline-block;
}

.ui.form.warning .icon.warning.message:not(:empty) {
  display: flex;
}

/* On Field(s) */

.ui.form .fields.warning .warning.message:not(:empty),
.ui.form .field.warning .warning.message:not(:empty) {
  display: block;
}

.ui.form .fields.warning .compact.warning.message:not(:empty),
.ui.form .field.warning .compact.warning.message:not(:empty) {
  display: inline-block;
}

.ui.form .fields.warning .icon.warning.message:not(:empty),
.ui.form .field.warning .icon.warning.message:not(:empty) {
  display: flex;
}

.ui.ui.form .fields.warning .field label,
.ui.ui.form .fields.warning .field .ui.label:not(.corner),
.ui.ui.form .field.warning label,
.ui.ui.form .field.warning .ui.label:not(.corner),
.ui.ui.form .fields.warning .field .input,
.ui.ui.form .field.warning .input {
  color: #573a08;
}

@supports selector(:has(.f)) {
  .ui.form .fields:has(.warning) > label {
    color: #573a08;
  }
}

.ui.form .fields.warning .field .ui.label,
.ui.form .field.warning .ui.label {
  background-color: #d7ccb5;
}

.ui.form .fields.warning .field .corner.label,
.ui.form .field.warning .corner.label {
  border-color: #573a08;
  color: #fff;
}

.ui.form .fields.warning .field textarea,
.ui.form .fields.warning .field select,
.ui.form .fields.warning .field input:not([type]),
.ui.form .fields.warning .field input[type="color"],
.ui.form .fields.warning .field input[type="date"],
.ui.form .fields.warning .field input[type="datetime-local"],
.ui.form .fields.warning .field input[type="email"],
.ui.form .fields.warning .field input[type="month"],
.ui.form .fields.warning .field input[type="number"],
.ui.form .fields.warning .field input[type="password"],
.ui.form .fields.warning .field input[type="search"],
.ui.form .fields.warning .field input[type="tel"],
.ui.form .fields.warning .field input[type="time"],
.ui.form .fields.warning .field input[type="text"],
.ui.form .fields.warning .field input[type="file"],
.ui.form .fields.warning .field input[type="url"],
.ui.form .fields.warning .field input[type="week"],
.ui.form .field.warning textarea,
.ui.form .field.warning select,
.ui.form .field.warning input:not([type]),
.ui.form .field.warning input[type="color"],
.ui.form .field.warning input[type="date"],
.ui.form .field.warning input[type="datetime-local"],
.ui.form .field.warning input[type="email"],
.ui.form .field.warning input[type="month"],
.ui.form .field.warning input[type="number"],
.ui.form .field.warning input[type="password"],
.ui.form .field.warning input[type="search"],
.ui.form .field.warning input[type="tel"],
.ui.form .field.warning input[type="time"],
.ui.form .field.warning input[type="text"],
.ui.form .field.warning input[type="file"],
.ui.form .field.warning input[type="url"],
.ui.form .field.warning input[type="week"] {
  color: #573a08;
  background: #fffaf3;
  border-color: #c9ba9b;
  border-radius: "";
  box-shadow: none;
}

.ui.form .field.warning textarea:focus,
.ui.form .field.warning select:focus,
.ui.form .field.warning input:not([type]):focus,
.ui.form .field.warning input[type="color"]:focus,
.ui.form .field.warning input[type="date"]:focus,
.ui.form .field.warning input[type="datetime-local"]:focus,
.ui.form .field.warning input[type="email"]:focus,
.ui.form .field.warning input[type="month"]:focus,
.ui.form .field.warning input[type="number"]:focus,
.ui.form .field.warning input[type="password"]:focus,
.ui.form .field.warning input[type="search"]:focus,
.ui.form .field.warning input[type="tel"]:focus,
.ui.form .field.warning input[type="time"]:focus,
.ui.form .field.warning input[type="text"]:focus,
.ui.form .field.warning input[type="file"]:focus,
.ui.form .field.warning input[type="url"]:focus,
.ui.form .field.warning input[type="week"]:focus {
  background: #fffaf3;
  border-color: #c9ba9b;
  color: #573a08;
  box-shadow: none;
}

/* Preserve Native Select Stylings */

.ui.form .field.warning select {
  -webkit-appearance: menulist-button;
}

/*------------------
            Input State
        --------------------*/

/* Transparent */

.ui.form .field.warning .transparent.input input,
.ui.form .field.warning .transparent.input textarea,
.ui.form .field.warning input.transparent,
.ui.form .field.warning textarea.transparent {
  background-color: #fffaf3 !important;
  color: #573a08 !important;
}

/* Autofilled */

.ui.form .warning.warning input:-webkit-autofill {
  box-shadow: 0 0 0 100px #ffffe0 inset !important;
  border-color: #e0e0b3 !important;
}

/* Placeholder */

.ui.form .warning :-ms-input-placeholder {
  color: #edad3e;
}

.ui.form .warning ::-ms-input-placeholder {
  color: #edad3e;
}

.ui.form .warning ::placeholder {
  color: #edad3e;
}

.ui.form .warning :-ms-input-placeholder {
  color: #edad3e !important;
}

.ui.form .warning :focus:-ms-input-placeholder {
  color: #e39715;
}

.ui.form .warning :focus::-ms-input-placeholder {
  color: #e39715;
}

.ui.form .warning :focus::placeholder {
  color: #e39715;
}

.ui.form .warning :focus:-ms-input-placeholder {
  color: #e39715 !important;
}

/*------------------
            Dropdown State
        --------------------*/

.ui.form .fields.warning .field .ui.dropdown,
.ui.form .fields.warning .field .ui.dropdown .item,
.ui.form .field.warning .ui.dropdown,
.ui.form .field.warning .ui.dropdown > .text,
.ui.form .field.warning .ui.dropdown .item {
  background: #fffaf3;
  color: #573a08;
}

.ui.form .fields.warning .field .ui.dropdown,
.ui.form .field.warning .ui.dropdown {
  border-color: #c9ba9b !important;
}

.ui.form .fields.warning .field .ui.dropdown:hover,
.ui.form .field.warning .ui.dropdown:hover {
  border-color: #c9ba9b !important;
}

.ui.form .fields.warning .field .ui.dropdown:hover .menu,
.ui.form .field.warning .ui.dropdown:hover .menu {
  border-color: #c9ba9b;
}

.ui.form .fields.warning .field .ui.multiple.selection.dropdown > .label,
.ui.form .field.warning .ui.multiple.selection.dropdown > .label {
  background-color: #eaeacc;
  color: #573a08;
}

/* Hover */

.ui.form .fields.warning .field .ui.dropdown .menu .item:hover,
.ui.form .field.warning .ui.dropdown .menu .item:hover {
  background-color: #fbfbe9;
}

/* Selected */

.ui.form .fields.warning .field .ui.dropdown .menu .selected.item,
.ui.form .field.warning .ui.dropdown .menu .selected.item {
  background-color: #fbfbe9;
}

/* Active */

.ui.form .fields.warning .field .ui.dropdown .menu .active.item,
.ui.form .field.warning .ui.dropdown .menu .active.item {
  background-color: #fdfdce !important;
}

/*--------------------
            Checkbox State
        ---------------------*/

.ui.form .fields.warning .field .checkbox:not(.toggle):not(.slider) label,
.ui.form .field.warning .checkbox:not(.toggle):not(.slider) label {
  color: #573a08;
}

.ui.form .fields.warning .field .checkbox:not(.toggle):not(.slider) label::before,
.ui.form .field.warning .checkbox:not(.toggle):not(.slider) label::before {
  background: #fffaf3;
  border-color: #c9ba9b;
}

.ui.form .fields.warning .field .checkbox label::after,
.ui.form .field.warning .checkbox label::after {
  color: #573a08;
}

.ui.inverted.form .fields.warning .field label,
.ui.inverted.form .warning.field label {
  color: #d7ccb5;
}

@supports selector(:has(.f)) {
  .ui.inverted.form .fields:has(.warning) > label {
    color: #d7ccb5;
  }
}

/* --------------------
           Disabled
    --------------------- */

.ui.form .disabled.fields .field,
.ui.form .disabled.field,
.ui.form .field :disabled {
  pointer-events: none;
  opacity: 0.45;
}

.ui.form .field.disabled > label,
.ui.form .fields.disabled > label {
  opacity: 0.45;
}

.ui.form .field.disabled :disabled {
  opacity: 1;
}

/* --------------
        Loading
    --------------- */

.ui.loading.form {
  position: relative;
  cursor: default;
  pointer-events: none;
}

.ui.loading.form::before {
  position: absolute;
  content: "";
  top: 0;
  left: 0;
  background: rgba(255, 255, 255, 0.8);
  width: 100%;
  height: 100%;
  z-index: 100;
}

.ui.loading.form.segments::before {
  border-radius: 0.28571429rem;
}

.ui.loading.form::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1.5em 0 0 -1.5em;
  width: 3em;
  height: 3em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
  visibility: visible;
  z-index: 101;
}

/*******************************
         Element Types
*******************************/

/* --------------------
         Required Field
    --------------------- */

.ui.form .required.fields:not(.grouped):not(.inline) > .field > label::after,
.ui.form .required.fields.inline > label::after,
.ui.form .required.fields.grouped > label::after,
.ui.form .required.field > label::after,
.ui.form .required.fields:not(.grouped):not(.inline) > .field > .checkbox::after,
.ui.form .required.field > .checkbox::after,
.ui.form label.required::after {
  margin: -0.2em 0 0 0.2em;
  content: "*";
  color: #db2828;
}

.ui.form .required.fields:not(.grouped):not(.inline) > .field > label::after,
.ui.form .required.fields.inline > label::after,
.ui.form .required.fields.grouped > label::after,
.ui.form .required.field > label::after,
.ui.form label.required::after {
  display: inline-block;
  vertical-align: top;
}

.ui.form .required.fields:not(.grouped):not(.inline) > .field > .checkbox::after,
.ui.form .required.field > .checkbox::after {
  position: absolute;
  top: 0;
  left: 100%;
}

.ui.ui.ui.ui.form .fields > label:empty::after,
.ui.ui.ui.ui.form .field > label:empty::after {
  content: " ";
  display: inline-block;
}

/*******************************
           Variations
*******************************/

/* --------------------
        Inverted Colors
    --------------------- */

.ui.inverted.form label,
.ui.form .inverted.segment label,
.ui.form .inverted.segment .ui.checkbox label,
.ui.inverted.form .ui.checkbox label,
.ui.inverted.form .inline.fields > label,
.ui.inverted.form .inline.fields .field > label,
.ui.inverted.form .inline.fields .field > p,
.ui.inverted.form .inline.field > label,
.ui.inverted.form .inline.field > p {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.loading.form {
  color: #fff;
}

.ui.inverted.loading.form::before {
  background: rgba(0, 0, 0, 0.85);
}

/* Inverted Field */

.ui.inverted.form input:not([type]),
.ui.inverted.form input[type="color"],
.ui.inverted.form input[type="date"],
.ui.inverted.form input[type="datetime-local"],
.ui.inverted.form input[type="email"],
.ui.inverted.form input[type="month"],
.ui.inverted.form input[type="number"],
.ui.inverted.form input[type="password"],
.ui.inverted.form input[type="search"],
.ui.inverted.form input[type="tel"],
.ui.inverted.form input[type="time"],
.ui.inverted.form input[type="text"],
.ui.inverted.form input[type="file"],
.ui.inverted.form input[type="url"],
.ui.inverted.form input[type="week"] {
  background: #fff;
  border-color: rgba(255, 255, 255, 0.1);
  color: rgba(0, 0, 0, 0.87);
  box-shadow: none;
}

/* --------------------
         Field Groups
    --------------------- */

/* Grouped Vertically */

.ui.form .grouped.fields {
  display: block;
  margin: 0 0 1em;
}

.ui.form .grouped.fields:last-child {
  margin-bottom: 0;
}

.ui.form .grouped.fields > label {
  margin: 0 0 0.28571429rem 0;
  color: rgba(0, 0, 0, 0.87);
  font-size: 0.92857143em;
  font-weight: bold;
  text-transform: none;
}

.ui.form .grouped.fields .field,
.ui.form .grouped.inline.fields .field {
  display: block;
  margin: 0.5em 0;
  padding: 0;
}

.ui.form .grouped.inline.fields .ui.checkbox {
  margin-bottom: 0.4em;
}

/* --------------------
        Fields
--------------------- */

/* Split fields */

.ui.form .fields {
  display: flex;
  flex-direction: row;
  margin: 0 -0.5em 1em;
}

.ui.form .fields > .field {
  flex: 0 1 auto;
  padding-left: 0.5em;
  padding-right: 0.5em;
}

.ui.form .fields > .field:first-child {
  border-left: none;
  box-shadow: none;
}

/* Other Combinations */

.ui.form .two.fields > .fields,
.ui.form .two.fields > .field {
  width: 50%;
}

.ui.form .three.fields > .fields,
.ui.form .three.fields > .field {
  width: 33.33333333%;
}

.ui.form .four.fields > .fields,
.ui.form .four.fields > .field {
  width: 25%;
}

.ui.form .five.fields > .fields,
.ui.form .five.fields > .field {
  width: 20%;
}

.ui.form .six.fields > .fields,
.ui.form .six.fields > .field {
  width: 16.66666667%;
}

.ui.form .seven.fields > .fields,
.ui.form .seven.fields > .field {
  width: 14.28571429%;
}

.ui.form .eight.fields > .fields,
.ui.form .eight.fields > .field {
  width: 12.5%;
}

.ui.form .nine.fields > .fields,
.ui.form .nine.fields > .field {
  width: 11.11111111%;
}

.ui.form .ten.fields > .fields,
.ui.form .ten.fields > .field {
  width: 10%;
}

/* Swap to full width on mobile */

@media only screen and (max-width: 767.98px) {
  .ui.form .fields {
    flex-wrap: wrap;
    margin-bottom: 0;
  }

  .ui.form:not(.unstackable) .fields:not(.unstackable) > .fields,
  .ui.form:not(.unstackable) .fields:not(.unstackable) > .field {
    width: 100%;
    margin: 0 0 1em;
  }
}

/* Sizing Combinations */

.ui.form .fields .wide.field {
  width: 6.25%;
  padding-left: 0.5em;
  padding-right: 0.5em;
}

.ui.form .one.wide.field {
  width: 6.25%;
}

.ui.form .two.wide.field {
  width: 12.5%;
}

.ui.form .three.wide.field {
  width: 18.75%;
}

.ui.form .four.wide.field {
  width: 25%;
}

.ui.form .five.wide.field {
  width: 31.25%;
}

.ui.form .six.wide.field {
  width: 37.5%;
}

.ui.form .seven.wide.field {
  width: 43.75%;
}

.ui.form .eight.wide.field {
  width: 50%;
}

.ui.form .nine.wide.field {
  width: 56.25%;
}

.ui.form .ten.wide.field {
  width: 62.5%;
}

.ui.form .eleven.wide.field {
  width: 68.75%;
}

.ui.form .twelve.wide.field {
  width: 75%;
}

.ui.form .thirteen.wide.field {
  width: 81.25%;
}

.ui.form .fourteen.wide.field {
  width: 87.5%;
}

.ui.form .fifteen.wide.field {
  width: 93.75%;
}

.ui.form .sixteen.wide.field {
  width: 100%;
}

/* --------------------
     Equal Width
--------------------- */

.ui[class*="equal width"].form .fields > .field,
.ui.form [class*="equal width"].fields > .field {
  width: 100%;
  flex: 1 1 auto;
}

/* --------------------
        Inline Fields
    --------------------- */

.ui.form .inline.fields {
  margin: 0 0 1em;
  align-items: center;
}

.ui.form .inline.fields .field {
  margin: 0;
  padding: 0 1em 0 0;
}

/* Inline Label */

.ui.form .inline.fields > label,
.ui.form .inline.fields .field > label,
.ui.form .inline.fields .field > p,
.ui.form .inline.field > label,
.ui.form .inline.field > p {
  display: inline-block;
  width: auto;
  margin-top: 0;
  margin-bottom: 0;
  vertical-align: baseline;
  font-size: 0.92857143em;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.87);
  text-transform: none;
}

/* Grouped Inline Label */

.ui.form .inline.fields > label {
  margin: 0.03571em 1em 0 0;
}

/* Inline Input */

.ui.form .inline.fields .field > input,
.ui.form .inline.fields .field > select,
.ui.form .inline.field > input,
.ui.form .inline.field > select {
  display: inline-block;
  width: auto;
  margin-top: 0;
  margin-bottom: 0;
  vertical-align: middle;
  font-size: 1em;
}

.ui.form .inline.fields .field .calendar:not(.popup),
.ui.form .inline.field .calendar:not(.popup) {
  display: inline-block;
}

.ui.form .inline.fields .field .calendar:not(.popup) > .input > input,
.ui.form .inline.field .calendar:not(.popup) > .input > input {
  width: 13.11em;
}

/* Label */

.ui.form .inline.fields .field > :first-child,
.ui.form .inline.field > :first-child {
  margin: 0 0.85714286em 0 0;
}

.ui.form .inline.fields .field > :only-child,
.ui.form .inline.field > :only-child {
  margin: 0;
}

/* Wide */

.ui.form .inline.fields .wide.field {
  display: flex;
  align-items: center;
}

.ui.form .inline.fields .wide.field > input,
.ui.form .inline.fields .wide.field > select {
  width: 100%;
}

/* --------------------
        Sizes
--------------------- */

.ui.form,
.ui.form .field .dropdown,
.ui.form .field .dropdown .menu > .item {
  font-size: 1rem;
}

.ui.mini.form,
.ui.mini.form .field .dropdown,
.ui.mini.form .field .dropdown .menu > .item {
  font-size: 0.78571429rem;
}

.ui.tiny.form,
.ui.tiny.form .field .dropdown,
.ui.tiny.form .field .dropdown .menu > .item {
  font-size: 0.85714286rem;
}

.ui.small.form,
.ui.small.form .field .dropdown,
.ui.small.form .field .dropdown .menu > .item {
  font-size: 0.92857143rem;
}

.ui.large.form,
.ui.large.form .field .dropdown,
.ui.large.form .field .dropdown .menu > .item {
  font-size: 1.14285714rem;
}

.ui.big.form,
.ui.big.form .field .dropdown,
.ui.big.form .field .dropdown .menu > .item {
  font-size: 1.28571429rem;
}

.ui.huge.form,
.ui.huge.form .field .dropdown,
.ui.huge.form .field .dropdown .menu > .item {
  font-size: 1.42857143rem;
}

.ui.massive.form,
.ui.massive.form .field .dropdown,
.ui.massive.form .field .dropdown .menu > .item {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Grid
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Standard
*******************************/

.ui.grid {
  display: flex;
  flex-flow: row wrap;
  align-items: stretch;
  padding: 0;
}

/* ----------------------
      Remove Gutters
----------------------- */

.ui.grid {
  margin: -1rem -1rem;
}

.ui.relaxed.grid {
  margin-left: -1.5rem;
  margin-right: -1.5rem;
}

.ui[class*="very relaxed"].grid {
  margin-left: -2.5rem;
  margin-right: -2.5rem;
}

/* Preserve Rows Spacing on Consecutive Grids */

.ui.grid + .grid {
  margin-top: 1rem;
}

/* -------------------
       Columns
-------------------- */

/* Standard 16 column */

.ui.grid > .column:not(.row),
.ui.grid > .row > .column {
  position: relative;
  display: inline-block;
  width: 6.25%;
  padding-left: 1rem;
  padding-right: 1rem;
  vertical-align: top;
}

.ui.grid > * {
  padding-left: 1rem;
  padding-right: 1rem;
}

/* -------------------
        Rows
-------------------- */

.ui.grid > .row {
  position: relative;
  display: flex;
  flex-flow: row wrap;
  justify-content: inherit;
  align-items: stretch;
  width: 100% !important;
  padding: 0;
  padding-top: 1rem;
  padding-bottom: 1rem;
}

/* -------------------
       Columns
-------------------- */

/* Vertical padding when no rows */

.ui.grid > .column:not(.row) {
  padding-top: 1rem;
  padding-bottom: 1rem;
}

.ui.grid > .row > .column {
  margin-top: 0;
  margin-bottom: 0;
}

/* -------------------
      Content
-------------------- */

.ui.grid > .row > img,
.ui.grid > .row > .column > img {
  max-width: 100%;
}

/* -------------------
    Loose Coupling
-------------------- */

/* Collapse Margin on Consecutive Grid */

.ui.grid > .ui.grid:first-child {
  margin-top: 0;
}

.ui.grid > .ui.grid:last-child {
  margin-bottom: 0;
}

/* Segment inside Aligned Grid */

.ui.grid .aligned.row > .column > .segment:not(.compact):not(.attached),
.ui.aligned.grid .column > .segment:not(.compact):not(.attached) {
  width: 100%;
}

/* Align Dividers with Gutter */

.ui.grid .row + .ui.divider {
  flex-grow: 1;
  margin: 1rem 1rem;
}

.ui.grid .column + .ui.vertical.divider {
  height: calc(50% - 1rem);
}

/* Remove Border on Last Horizontal Segment */

.ui.grid > .row > .column:last-child > .horizontal.segment,
.ui.grid > .column:last-child > .horizontal.segment {
  box-shadow: none;
}

/*******************************
           Variations
*******************************/

/* -----------------------
           Page Grid
    ------------------------- */

@media only screen and (max-width: 767.98px) {
  .ui.page.grid {
    width: auto;
    padding-left: 0;
    padding-right: 0;
    margin-left: 0;
    margin-right: 0;
  }
}

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.page.grid {
    width: auto;
    margin-left: 0;
    margin-right: 0;
    padding-left: 2em;
    padding-right: 2em;
  }
}

@media only screen and (min-width: 992px) and (max-width: 1199.98px) {
  .ui.page.grid {
    width: auto;
    margin-left: 0;
    margin-right: 0;
    padding-left: 3%;
    padding-right: 3%;
  }
}

@media only screen and (min-width: 1200px) and (max-width: 1919.98px) {
  .ui.page.grid {
    width: auto;
    margin-left: 0;
    margin-right: 0;
    padding-left: 15%;
    padding-right: 15%;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.page.grid {
    width: auto;
    margin-left: 0;
    margin-right: 0;
    padding-left: 23%;
    padding-right: 23%;
  }
}

/* -------------------
     Column Count
-------------------- */

/* Assume full width with one column */

.ui.grid > .column:only-child,
.ui.grid > .row > .column:only-child {
  width: 100%;
}

/* Grid Based */

.ui[class*="one column"].grid > .row > .column,
.ui[class*="one column"].grid > .column:not(.row) {
  width: 100%;
}

.ui[class*="two column"].grid > .row > .column,
.ui[class*="two column"].grid > .column:not(.row) {
  width: 50%;
}

.ui[class*="three column"].grid > .row > .column,
.ui[class*="three column"].grid > .column:not(.row) {
  width: 33.33333333%;
}

.ui[class*="four column"].grid > .row > .column,
.ui[class*="four column"].grid > .column:not(.row) {
  width: 25%;
}

.ui[class*="five column"].grid > .row > .column,
.ui[class*="five column"].grid > .column:not(.row) {
  width: 20%;
}

.ui[class*="six column"].grid > .row > .column,
.ui[class*="six column"].grid > .column:not(.row) {
  width: 16.66666667%;
}

.ui[class*="seven column"].grid > .row > .column,
.ui[class*="seven column"].grid > .column:not(.row) {
  width: 14.28571429%;
}

.ui[class*="eight column"].grid > .row > .column,
.ui[class*="eight column"].grid > .column:not(.row) {
  width: 12.5%;
}

.ui[class*="nine column"].grid > .row > .column,
.ui[class*="nine column"].grid > .column:not(.row) {
  width: 11.11111111%;
}

.ui[class*="ten column"].grid > .row > .column,
.ui[class*="ten column"].grid > .column:not(.row) {
  width: 10%;
}

.ui[class*="eleven column"].grid > .row > .column,
.ui[class*="eleven column"].grid > .column:not(.row) {
  width: 9.09090909%;
}

.ui[class*="twelve column"].grid > .row > .column,
.ui[class*="twelve column"].grid > .column:not(.row) {
  width: 8.33333333%;
}

.ui[class*="thirteen column"].grid > .row > .column,
.ui[class*="thirteen column"].grid > .column:not(.row) {
  width: 7.69230769%;
}

.ui[class*="fourteen column"].grid > .row > .column,
.ui[class*="fourteen column"].grid > .column:not(.row) {
  width: 7.14285714%;
}

.ui[class*="fifteen column"].grid > .row > .column,
.ui[class*="fifteen column"].grid > .column:not(.row) {
  width: 6.66666667%;
}

.ui[class*="sixteen column"].grid > .row > .column,
.ui[class*="sixteen column"].grid > .column:not(.row) {
  width: 6.25%;
}

/* Row Based Overrides */

.ui.grid > [class*="one column"].row > .column {
  width: 100% !important;
}

.ui.grid > [class*="two column"].row > .column {
  width: 50% !important;
}

.ui.grid > [class*="three column"].row > .column {
  width: 33.33333333% !important;
}

.ui.grid > [class*="four column"].row > .column {
  width: 25% !important;
}

.ui.grid > [class*="five column"].row > .column {
  width: 20% !important;
}

.ui.grid > [class*="six column"].row > .column {
  width: 16.66666667% !important;
}

.ui.grid > [class*="seven column"].row > .column {
  width: 14.28571429% !important;
}

.ui.grid > [class*="eight column"].row > .column {
  width: 12.5% !important;
}

.ui.grid > [class*="nine column"].row > .column {
  width: 11.11111111% !important;
}

.ui.grid > [class*="ten column"].row > .column {
  width: 10% !important;
}

.ui.grid > [class*="eleven column"].row > .column {
  width: 9.09090909% !important;
}

.ui.grid > [class*="twelve column"].row > .column {
  width: 8.33333333% !important;
}

.ui.grid > [class*="thirteen column"].row > .column {
  width: 7.69230769% !important;
}

.ui.grid > [class*="fourteen column"].row > .column {
  width: 7.14285714% !important;
}

.ui.grid > [class*="fifteen column"].row > .column {
  width: 6.66666667% !important;
}

.ui.grid > [class*="sixteen column"].row > .column {
  width: 6.25% !important;
}

/* Celled Page */

.ui.celled.page.grid {
  box-shadow: none;
}

/* -------------------
        Column Width
    -------------------- */

/* Sizing Combinations */

.ui.grid > .row > [class*="one wide"].column,
.ui.grid > .column.row > [class*="one wide"].column,
.ui.grid > [class*="one wide"].column,
.ui.column.grid > [class*="one wide"].column {
  width: 6.25% !important;
}

.ui.grid > .row > [class*="two wide"].column,
.ui.grid > .column.row > [class*="two wide"].column,
.ui.grid > [class*="two wide"].column,
.ui.column.grid > [class*="two wide"].column {
  width: 12.5% !important;
}

.ui.grid > .row > [class*="three wide"].column,
.ui.grid > .column.row > [class*="three wide"].column,
.ui.grid > [class*="three wide"].column,
.ui.column.grid > [class*="three wide"].column {
  width: 18.75% !important;
}

.ui.grid > .row > [class*="four wide"].column,
.ui.grid > .column.row > [class*="four wide"].column,
.ui.grid > [class*="four wide"].column,
.ui.column.grid > [class*="four wide"].column {
  width: 25% !important;
}

.ui.grid > .row > [class*="five wide"].column,
.ui.grid > .column.row > [class*="five wide"].column,
.ui.grid > [class*="five wide"].column,
.ui.column.grid > [class*="five wide"].column {
  width: 31.25% !important;
}

.ui.grid > .row > [class*="six wide"].column,
.ui.grid > .column.row > [class*="six wide"].column,
.ui.grid > [class*="six wide"].column,
.ui.column.grid > [class*="six wide"].column {
  width: 37.5% !important;
}

.ui.grid > .row > [class*="seven wide"].column,
.ui.grid > .column.row > [class*="seven wide"].column,
.ui.grid > [class*="seven wide"].column,
.ui.column.grid > [class*="seven wide"].column {
  width: 43.75% !important;
}

.ui.grid > .row > [class*="eight wide"].column,
.ui.grid > .column.row > [class*="eight wide"].column,
.ui.grid > [class*="eight wide"].column,
.ui.column.grid > [class*="eight wide"].column {
  width: 50% !important;
}

.ui.grid > .row > [class*="nine wide"].column,
.ui.grid > .column.row > [class*="nine wide"].column,
.ui.grid > [class*="nine wide"].column,
.ui.column.grid > [class*="nine wide"].column {
  width: 56.25% !important;
}

.ui.grid > .row > [class*="ten wide"].column,
.ui.grid > .column.row > [class*="ten wide"].column,
.ui.grid > [class*="ten wide"].column,
.ui.column.grid > [class*="ten wide"].column {
  width: 62.5% !important;
}

.ui.grid > .row > [class*="eleven wide"].column,
.ui.grid > .column.row > [class*="eleven wide"].column,
.ui.grid > [class*="eleven wide"].column,
.ui.column.grid > [class*="eleven wide"].column {
  width: 68.75% !important;
}

.ui.grid > .row > [class*="twelve wide"].column,
.ui.grid > .column.row > [class*="twelve wide"].column,
.ui.grid > [class*="twelve wide"].column,
.ui.column.grid > [class*="twelve wide"].column {
  width: 75% !important;
}

.ui.grid > .row > [class*="thirteen wide"].column,
.ui.grid > .column.row > [class*="thirteen wide"].column,
.ui.grid > [class*="thirteen wide"].column,
.ui.column.grid > [class*="thirteen wide"].column {
  width: 81.25% !important;
}

.ui.grid > .row > [class*="fourteen wide"].column,
.ui.grid > .column.row > [class*="fourteen wide"].column,
.ui.grid > [class*="fourteen wide"].column,
.ui.column.grid > [class*="fourteen wide"].column {
  width: 87.5% !important;
}

.ui.grid > .row > [class*="fifteen wide"].column,
.ui.grid > .column.row > [class*="fifteen wide"].column,
.ui.grid > [class*="fifteen wide"].column,
.ui.column.grid > [class*="fifteen wide"].column {
  width: 93.75% !important;
}

.ui.grid > .row > [class*="sixteen wide"].column,
.ui.grid > .column.row > [class*="sixteen wide"].column,
.ui.grid > [class*="sixteen wide"].column,
.ui.column.grid > [class*="sixteen wide"].column {
  width: 100% !important;
}

/* ----------------------
        Width per Device
    ----------------------- */

/* Mobile Sizing Combinations */

@media only screen and (min-width: 320px) and (max-width: 767.98px) {
  .ui.grid > .row > [class*="one wide mobile"].column,
  .ui.grid > .column.row > [class*="one wide mobile"].column,
  .ui.grid > [class*="one wide mobile"].column,
  .ui.column.grid > [class*="one wide mobile"].column {
    width: 6.25% !important;
  }

  .ui.grid > .row > [class*="two wide mobile"].column,
  .ui.grid > .column.row > [class*="two wide mobile"].column,
  .ui.grid > [class*="two wide mobile"].column,
  .ui.column.grid > [class*="two wide mobile"].column {
    width: 12.5% !important;
  }

  .ui.grid > .row > [class*="three wide mobile"].column,
  .ui.grid > .column.row > [class*="three wide mobile"].column,
  .ui.grid > [class*="three wide mobile"].column,
  .ui.column.grid > [class*="three wide mobile"].column {
    width: 18.75% !important;
  }

  .ui.grid > .row > [class*="four wide mobile"].column,
  .ui.grid > .column.row > [class*="four wide mobile"].column,
  .ui.grid > [class*="four wide mobile"].column,
  .ui.column.grid > [class*="four wide mobile"].column {
    width: 25% !important;
  }

  .ui.grid > .row > [class*="five wide mobile"].column,
  .ui.grid > .column.row > [class*="five wide mobile"].column,
  .ui.grid > [class*="five wide mobile"].column,
  .ui.column.grid > [class*="five wide mobile"].column {
    width: 31.25% !important;
  }

  .ui.grid > .row > [class*="six wide mobile"].column,
  .ui.grid > .column.row > [class*="six wide mobile"].column,
  .ui.grid > [class*="six wide mobile"].column,
  .ui.column.grid > [class*="six wide mobile"].column {
    width: 37.5% !important;
  }

  .ui.grid > .row > [class*="seven wide mobile"].column,
  .ui.grid > .column.row > [class*="seven wide mobile"].column,
  .ui.grid > [class*="seven wide mobile"].column,
  .ui.column.grid > [class*="seven wide mobile"].column {
    width: 43.75% !important;
  }

  .ui.grid > .row > [class*="eight wide mobile"].column,
  .ui.grid > .column.row > [class*="eight wide mobile"].column,
  .ui.grid > [class*="eight wide mobile"].column,
  .ui.column.grid > [class*="eight wide mobile"].column {
    width: 50% !important;
  }

  .ui.grid > .row > [class*="nine wide mobile"].column,
  .ui.grid > .column.row > [class*="nine wide mobile"].column,
  .ui.grid > [class*="nine wide mobile"].column,
  .ui.column.grid > [class*="nine wide mobile"].column {
    width: 56.25% !important;
  }

  .ui.grid > .row > [class*="ten wide mobile"].column,
  .ui.grid > .column.row > [class*="ten wide mobile"].column,
  .ui.grid > [class*="ten wide mobile"].column,
  .ui.column.grid > [class*="ten wide mobile"].column {
    width: 62.5% !important;
  }

  .ui.grid > .row > [class*="eleven wide mobile"].column,
  .ui.grid > .column.row > [class*="eleven wide mobile"].column,
  .ui.grid > [class*="eleven wide mobile"].column,
  .ui.column.grid > [class*="eleven wide mobile"].column {
    width: 68.75% !important;
  }

  .ui.grid > .row > [class*="twelve wide mobile"].column,
  .ui.grid > .column.row > [class*="twelve wide mobile"].column,
  .ui.grid > [class*="twelve wide mobile"].column,
  .ui.column.grid > [class*="twelve wide mobile"].column {
    width: 75% !important;
  }

  .ui.grid > .row > [class*="thirteen wide mobile"].column,
  .ui.grid > .column.row > [class*="thirteen wide mobile"].column,
  .ui.grid > [class*="thirteen wide mobile"].column,
  .ui.column.grid > [class*="thirteen wide mobile"].column {
    width: 81.25% !important;
  }

  .ui.grid > .row > [class*="fourteen wide mobile"].column,
  .ui.grid > .column.row > [class*="fourteen wide mobile"].column,
  .ui.grid > [class*="fourteen wide mobile"].column,
  .ui.column.grid > [class*="fourteen wide mobile"].column {
    width: 87.5% !important;
  }

  .ui.grid > .row > [class*="fifteen wide mobile"].column,
  .ui.grid > .column.row > [class*="fifteen wide mobile"].column,
  .ui.grid > [class*="fifteen wide mobile"].column,
  .ui.column.grid > [class*="fifteen wide mobile"].column {
    width: 93.75% !important;
  }

  .ui.grid > .row > [class*="sixteen wide mobile"].column,
  .ui.grid > .column.row > [class*="sixteen wide mobile"].column,
  .ui.grid > [class*="sixteen wide mobile"].column,
  .ui.column.grid > [class*="sixteen wide mobile"].column {
    width: 100% !important;
  }
}

/* Tablet Sizing Combinations */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.grid > .row > [class*="one wide tablet"].column,
  .ui.grid > .column.row > [class*="one wide tablet"].column,
  .ui.grid > [class*="one wide tablet"].column,
  .ui.column.grid > [class*="one wide tablet"].column {
    width: 6.25% !important;
  }

  .ui.grid > .row > [class*="two wide tablet"].column,
  .ui.grid > .column.row > [class*="two wide tablet"].column,
  .ui.grid > [class*="two wide tablet"].column,
  .ui.column.grid > [class*="two wide tablet"].column {
    width: 12.5% !important;
  }

  .ui.grid > .row > [class*="three wide tablet"].column,
  .ui.grid > .column.row > [class*="three wide tablet"].column,
  .ui.grid > [class*="three wide tablet"].column,
  .ui.column.grid > [class*="three wide tablet"].column {
    width: 18.75% !important;
  }

  .ui.grid > .row > [class*="four wide tablet"].column,
  .ui.grid > .column.row > [class*="four wide tablet"].column,
  .ui.grid > [class*="four wide tablet"].column,
  .ui.column.grid > [class*="four wide tablet"].column {
    width: 25% !important;
  }

  .ui.grid > .row > [class*="five wide tablet"].column,
  .ui.grid > .column.row > [class*="five wide tablet"].column,
  .ui.grid > [class*="five wide tablet"].column,
  .ui.column.grid > [class*="five wide tablet"].column {
    width: 31.25% !important;
  }

  .ui.grid > .row > [class*="six wide tablet"].column,
  .ui.grid > .column.row > [class*="six wide tablet"].column,
  .ui.grid > [class*="six wide tablet"].column,
  .ui.column.grid > [class*="six wide tablet"].column {
    width: 37.5% !important;
  }

  .ui.grid > .row > [class*="seven wide tablet"].column,
  .ui.grid > .column.row > [class*="seven wide tablet"].column,
  .ui.grid > [class*="seven wide tablet"].column,
  .ui.column.grid > [class*="seven wide tablet"].column {
    width: 43.75% !important;
  }

  .ui.grid > .row > [class*="eight wide tablet"].column,
  .ui.grid > .column.row > [class*="eight wide tablet"].column,
  .ui.grid > [class*="eight wide tablet"].column,
  .ui.column.grid > [class*="eight wide tablet"].column {
    width: 50% !important;
  }

  .ui.grid > .row > [class*="nine wide tablet"].column,
  .ui.grid > .column.row > [class*="nine wide tablet"].column,
  .ui.grid > [class*="nine wide tablet"].column,
  .ui.column.grid > [class*="nine wide tablet"].column {
    width: 56.25% !important;
  }

  .ui.grid > .row > [class*="ten wide tablet"].column,
  .ui.grid > .column.row > [class*="ten wide tablet"].column,
  .ui.grid > [class*="ten wide tablet"].column,
  .ui.column.grid > [class*="ten wide tablet"].column {
    width: 62.5% !important;
  }

  .ui.grid > .row > [class*="eleven wide tablet"].column,
  .ui.grid > .column.row > [class*="eleven wide tablet"].column,
  .ui.grid > [class*="eleven wide tablet"].column,
  .ui.column.grid > [class*="eleven wide tablet"].column {
    width: 68.75% !important;
  }

  .ui.grid > .row > [class*="twelve wide tablet"].column,
  .ui.grid > .column.row > [class*="twelve wide tablet"].column,
  .ui.grid > [class*="twelve wide tablet"].column,
  .ui.column.grid > [class*="twelve wide tablet"].column {
    width: 75% !important;
  }

  .ui.grid > .row > [class*="thirteen wide tablet"].column,
  .ui.grid > .column.row > [class*="thirteen wide tablet"].column,
  .ui.grid > [class*="thirteen wide tablet"].column,
  .ui.column.grid > [class*="thirteen wide tablet"].column {
    width: 81.25% !important;
  }

  .ui.grid > .row > [class*="fourteen wide tablet"].column,
  .ui.grid > .column.row > [class*="fourteen wide tablet"].column,
  .ui.grid > [class*="fourteen wide tablet"].column,
  .ui.column.grid > [class*="fourteen wide tablet"].column {
    width: 87.5% !important;
  }

  .ui.grid > .row > [class*="fifteen wide tablet"].column,
  .ui.grid > .column.row > [class*="fifteen wide tablet"].column,
  .ui.grid > [class*="fifteen wide tablet"].column,
  .ui.column.grid > [class*="fifteen wide tablet"].column {
    width: 93.75% !important;
  }

  .ui.grid > .row > [class*="sixteen wide tablet"].column,
  .ui.grid > .column.row > [class*="sixteen wide tablet"].column,
  .ui.grid > [class*="sixteen wide tablet"].column,
  .ui.column.grid > [class*="sixteen wide tablet"].column {
    width: 100% !important;
  }
}

/* Computer/Desktop Sizing Combinations */

@media only screen and (min-width: 992px) {
  .ui.grid > .row > [class*="one wide computer"].column,
  .ui.grid > .column.row > [class*="one wide computer"].column,
  .ui.grid > [class*="one wide computer"].column,
  .ui.column.grid > [class*="one wide computer"].column {
    width: 6.25% !important;
  }

  .ui.grid > .row > [class*="two wide computer"].column,
  .ui.grid > .column.row > [class*="two wide computer"].column,
  .ui.grid > [class*="two wide computer"].column,
  .ui.column.grid > [class*="two wide computer"].column {
    width: 12.5% !important;
  }

  .ui.grid > .row > [class*="three wide computer"].column,
  .ui.grid > .column.row > [class*="three wide computer"].column,
  .ui.grid > [class*="three wide computer"].column,
  .ui.column.grid > [class*="three wide computer"].column {
    width: 18.75% !important;
  }

  .ui.grid > .row > [class*="four wide computer"].column,
  .ui.grid > .column.row > [class*="four wide computer"].column,
  .ui.grid > [class*="four wide computer"].column,
  .ui.column.grid > [class*="four wide computer"].column {
    width: 25% !important;
  }

  .ui.grid > .row > [class*="five wide computer"].column,
  .ui.grid > .column.row > [class*="five wide computer"].column,
  .ui.grid > [class*="five wide computer"].column,
  .ui.column.grid > [class*="five wide computer"].column {
    width: 31.25% !important;
  }

  .ui.grid > .row > [class*="six wide computer"].column,
  .ui.grid > .column.row > [class*="six wide computer"].column,
  .ui.grid > [class*="six wide computer"].column,
  .ui.column.grid > [class*="six wide computer"].column {
    width: 37.5% !important;
  }

  .ui.grid > .row > [class*="seven wide computer"].column,
  .ui.grid > .column.row > [class*="seven wide computer"].column,
  .ui.grid > [class*="seven wide computer"].column,
  .ui.column.grid > [class*="seven wide computer"].column {
    width: 43.75% !important;
  }

  .ui.grid > .row > [class*="eight wide computer"].column,
  .ui.grid > .column.row > [class*="eight wide computer"].column,
  .ui.grid > [class*="eight wide computer"].column,
  .ui.column.grid > [class*="eight wide computer"].column {
    width: 50% !important;
  }

  .ui.grid > .row > [class*="nine wide computer"].column,
  .ui.grid > .column.row > [class*="nine wide computer"].column,
  .ui.grid > [class*="nine wide computer"].column,
  .ui.column.grid > [class*="nine wide computer"].column {
    width: 56.25% !important;
  }

  .ui.grid > .row > [class*="ten wide computer"].column,
  .ui.grid > .column.row > [class*="ten wide computer"].column,
  .ui.grid > [class*="ten wide computer"].column,
  .ui.column.grid > [class*="ten wide computer"].column {
    width: 62.5% !important;
  }

  .ui.grid > .row > [class*="eleven wide computer"].column,
  .ui.grid > .column.row > [class*="eleven wide computer"].column,
  .ui.grid > [class*="eleven wide computer"].column,
  .ui.column.grid > [class*="eleven wide computer"].column {
    width: 68.75% !important;
  }

  .ui.grid > .row > [class*="twelve wide computer"].column,
  .ui.grid > .column.row > [class*="twelve wide computer"].column,
  .ui.grid > [class*="twelve wide computer"].column,
  .ui.column.grid > [class*="twelve wide computer"].column {
    width: 75% !important;
  }

  .ui.grid > .row > [class*="thirteen wide computer"].column,
  .ui.grid > .column.row > [class*="thirteen wide computer"].column,
  .ui.grid > [class*="thirteen wide computer"].column,
  .ui.column.grid > [class*="thirteen wide computer"].column {
    width: 81.25% !important;
  }

  .ui.grid > .row > [class*="fourteen wide computer"].column,
  .ui.grid > .column.row > [class*="fourteen wide computer"].column,
  .ui.grid > [class*="fourteen wide computer"].column,
  .ui.column.grid > [class*="fourteen wide computer"].column {
    width: 87.5% !important;
  }

  .ui.grid > .row > [class*="fifteen wide computer"].column,
  .ui.grid > .column.row > [class*="fifteen wide computer"].column,
  .ui.grid > [class*="fifteen wide computer"].column,
  .ui.column.grid > [class*="fifteen wide computer"].column {
    width: 93.75% !important;
  }

  .ui.grid > .row > [class*="sixteen wide computer"].column,
  .ui.grid > .column.row > [class*="sixteen wide computer"].column,
  .ui.grid > [class*="sixteen wide computer"].column,
  .ui.column.grid > [class*="sixteen wide computer"].column {
    width: 100% !important;
  }
}

/* Large Monitor Sizing Combinations */

@media only screen and (min-width: 1200px) and (max-width: 1919.98px) {
  .ui.grid > .row > [class*="one wide large screen"].column,
  .ui.grid > .column.row > [class*="one wide large screen"].column,
  .ui.grid > [class*="one wide large screen"].column,
  .ui.column.grid > [class*="one wide large screen"].column {
    width: 6.25% !important;
  }

  .ui.grid > .row > [class*="two wide large screen"].column,
  .ui.grid > .column.row > [class*="two wide large screen"].column,
  .ui.grid > [class*="two wide large screen"].column,
  .ui.column.grid > [class*="two wide large screen"].column {
    width: 12.5% !important;
  }

  .ui.grid > .row > [class*="three wide large screen"].column,
  .ui.grid > .column.row > [class*="three wide large screen"].column,
  .ui.grid > [class*="three wide large screen"].column,
  .ui.column.grid > [class*="three wide large screen"].column {
    width: 18.75% !important;
  }

  .ui.grid > .row > [class*="four wide large screen"].column,
  .ui.grid > .column.row > [class*="four wide large screen"].column,
  .ui.grid > [class*="four wide large screen"].column,
  .ui.column.grid > [class*="four wide large screen"].column {
    width: 25% !important;
  }

  .ui.grid > .row > [class*="five wide large screen"].column,
  .ui.grid > .column.row > [class*="five wide large screen"].column,
  .ui.grid > [class*="five wide large screen"].column,
  .ui.column.grid > [class*="five wide large screen"].column {
    width: 31.25% !important;
  }

  .ui.grid > .row > [class*="six wide large screen"].column,
  .ui.grid > .column.row > [class*="six wide large screen"].column,
  .ui.grid > [class*="six wide large screen"].column,
  .ui.column.grid > [class*="six wide large screen"].column {
    width: 37.5% !important;
  }

  .ui.grid > .row > [class*="seven wide large screen"].column,
  .ui.grid > .column.row > [class*="seven wide large screen"].column,
  .ui.grid > [class*="seven wide large screen"].column,
  .ui.column.grid > [class*="seven wide large screen"].column {
    width: 43.75% !important;
  }

  .ui.grid > .row > [class*="eight wide large screen"].column,
  .ui.grid > .column.row > [class*="eight wide large screen"].column,
  .ui.grid > [class*="eight wide large screen"].column,
  .ui.column.grid > [class*="eight wide large screen"].column {
    width: 50% !important;
  }

  .ui.grid > .row > [class*="nine wide large screen"].column,
  .ui.grid > .column.row > [class*="nine wide large screen"].column,
  .ui.grid > [class*="nine wide large screen"].column,
  .ui.column.grid > [class*="nine wide large screen"].column {
    width: 56.25% !important;
  }

  .ui.grid > .row > [class*="ten wide large screen"].column,
  .ui.grid > .column.row > [class*="ten wide large screen"].column,
  .ui.grid > [class*="ten wide large screen"].column,
  .ui.column.grid > [class*="ten wide large screen"].column {
    width: 62.5% !important;
  }

  .ui.grid > .row > [class*="eleven wide large screen"].column,
  .ui.grid > .column.row > [class*="eleven wide large screen"].column,
  .ui.grid > [class*="eleven wide large screen"].column,
  .ui.column.grid > [class*="eleven wide large screen"].column {
    width: 68.75% !important;
  }

  .ui.grid > .row > [class*="twelve wide large screen"].column,
  .ui.grid > .column.row > [class*="twelve wide large screen"].column,
  .ui.grid > [class*="twelve wide large screen"].column,
  .ui.column.grid > [class*="twelve wide large screen"].column {
    width: 75% !important;
  }

  .ui.grid > .row > [class*="thirteen wide large screen"].column,
  .ui.grid > .column.row > [class*="thirteen wide large screen"].column,
  .ui.grid > [class*="thirteen wide large screen"].column,
  .ui.column.grid > [class*="thirteen wide large screen"].column {
    width: 81.25% !important;
  }

  .ui.grid > .row > [class*="fourteen wide large screen"].column,
  .ui.grid > .column.row > [class*="fourteen wide large screen"].column,
  .ui.grid > [class*="fourteen wide large screen"].column,
  .ui.column.grid > [class*="fourteen wide large screen"].column {
    width: 87.5% !important;
  }

  .ui.grid > .row > [class*="fifteen wide large screen"].column,
  .ui.grid > .column.row > [class*="fifteen wide large screen"].column,
  .ui.grid > [class*="fifteen wide large screen"].column,
  .ui.column.grid > [class*="fifteen wide large screen"].column {
    width: 93.75% !important;
  }

  .ui.grid > .row > [class*="sixteen wide large screen"].column,
  .ui.grid > .column.row > [class*="sixteen wide large screen"].column,
  .ui.grid > [class*="sixteen wide large screen"].column,
  .ui.column.grid > [class*="sixteen wide large screen"].column {
    width: 100% !important;
  }
}

/* Widescreen Sizing Combinations */

@media only screen and (min-width: 1920px) {
  .ui.grid > .row > [class*="one wide widescreen"].column,
  .ui.grid > .column.row > [class*="one wide widescreen"].column,
  .ui.grid > [class*="one wide widescreen"].column,
  .ui.column.grid > [class*="one wide widescreen"].column {
    width: 6.25% !important;
  }

  .ui.grid > .row > [class*="two wide widescreen"].column,
  .ui.grid > .column.row > [class*="two wide widescreen"].column,
  .ui.grid > [class*="two wide widescreen"].column,
  .ui.column.grid > [class*="two wide widescreen"].column {
    width: 12.5% !important;
  }

  .ui.grid > .row > [class*="three wide widescreen"].column,
  .ui.grid > .column.row > [class*="three wide widescreen"].column,
  .ui.grid > [class*="three wide widescreen"].column,
  .ui.column.grid > [class*="three wide widescreen"].column {
    width: 18.75% !important;
  }

  .ui.grid > .row > [class*="four wide widescreen"].column,
  .ui.grid > .column.row > [class*="four wide widescreen"].column,
  .ui.grid > [class*="four wide widescreen"].column,
  .ui.column.grid > [class*="four wide widescreen"].column {
    width: 25% !important;
  }

  .ui.grid > .row > [class*="five wide widescreen"].column,
  .ui.grid > .column.row > [class*="five wide widescreen"].column,
  .ui.grid > [class*="five wide widescreen"].column,
  .ui.column.grid > [class*="five wide widescreen"].column {
    width: 31.25% !important;
  }

  .ui.grid > .row > [class*="six wide widescreen"].column,
  .ui.grid > .column.row > [class*="six wide widescreen"].column,
  .ui.grid > [class*="six wide widescreen"].column,
  .ui.column.grid > [class*="six wide widescreen"].column {
    width: 37.5% !important;
  }

  .ui.grid > .row > [class*="seven wide widescreen"].column,
  .ui.grid > .column.row > [class*="seven wide widescreen"].column,
  .ui.grid > [class*="seven wide widescreen"].column,
  .ui.column.grid > [class*="seven wide widescreen"].column {
    width: 43.75% !important;
  }

  .ui.grid > .row > [class*="eight wide widescreen"].column,
  .ui.grid > .column.row > [class*="eight wide widescreen"].column,
  .ui.grid > [class*="eight wide widescreen"].column,
  .ui.column.grid > [class*="eight wide widescreen"].column {
    width: 50% !important;
  }

  .ui.grid > .row > [class*="nine wide widescreen"].column,
  .ui.grid > .column.row > [class*="nine wide widescreen"].column,
  .ui.grid > [class*="nine wide widescreen"].column,
  .ui.column.grid > [class*="nine wide widescreen"].column {
    width: 56.25% !important;
  }

  .ui.grid > .row > [class*="ten wide widescreen"].column,
  .ui.grid > .column.row > [class*="ten wide widescreen"].column,
  .ui.grid > [class*="ten wide widescreen"].column,
  .ui.column.grid > [class*="ten wide widescreen"].column {
    width: 62.5% !important;
  }

  .ui.grid > .row > [class*="eleven wide widescreen"].column,
  .ui.grid > .column.row > [class*="eleven wide widescreen"].column,
  .ui.grid > [class*="eleven wide widescreen"].column,
  .ui.column.grid > [class*="eleven wide widescreen"].column {
    width: 68.75% !important;
  }

  .ui.grid > .row > [class*="twelve wide widescreen"].column,
  .ui.grid > .column.row > [class*="twelve wide widescreen"].column,
  .ui.grid > [class*="twelve wide widescreen"].column,
  .ui.column.grid > [class*="twelve wide widescreen"].column {
    width: 75% !important;
  }

  .ui.grid > .row > [class*="thirteen wide widescreen"].column,
  .ui.grid > .column.row > [class*="thirteen wide widescreen"].column,
  .ui.grid > [class*="thirteen wide widescreen"].column,
  .ui.column.grid > [class*="thirteen wide widescreen"].column {
    width: 81.25% !important;
  }

  .ui.grid > .row > [class*="fourteen wide widescreen"].column,
  .ui.grid > .column.row > [class*="fourteen wide widescreen"].column,
  .ui.grid > [class*="fourteen wide widescreen"].column,
  .ui.column.grid > [class*="fourteen wide widescreen"].column {
    width: 87.5% !important;
  }

  .ui.grid > .row > [class*="fifteen wide widescreen"].column,
  .ui.grid > .column.row > [class*="fifteen wide widescreen"].column,
  .ui.grid > [class*="fifteen wide widescreen"].column,
  .ui.column.grid > [class*="fifteen wide widescreen"].column {
    width: 93.75% !important;
  }

  .ui.grid > .row > [class*="sixteen wide widescreen"].column,
  .ui.grid > .column.row > [class*="sixteen wide widescreen"].column,
  .ui.grid > [class*="sixteen wide widescreen"].column,
  .ui.column.grid > [class*="sixteen wide widescreen"].column {
    width: 100% !important;
  }
}

/* ----------------------
            Centered
    ----------------------- */

.ui.centered.grid,
.ui.centered.grid > .row,
.ui.grid > .centered.row {
  text-align: center;
  justify-content: center;
}

.ui.centered.grid > .column:not(.aligned):not(.justified):not(.row),
.ui.centered.grid > .row > .column:not(.aligned):not(.justified),
.ui.grid .centered.row > .column:not(.aligned):not(.justified) {
  text-align: left;
}

.ui.grid > .centered.column,
.ui.grid > .row > .centered.column {
  display: block;
  margin-left: auto;
  margin-right: auto;
}

/* ----------------------
            Relaxed
    ----------------------- */

.ui.relaxed.grid > .column:not(.row),
.ui.relaxed.grid > .row > .column,
.ui.grid > .relaxed.row > .column {
  padding-left: 1.5rem;
  padding-right: 1.5rem;
}

.ui[class*="very relaxed"].grid > .column:not(.row),
.ui[class*="very relaxed"].grid > .row > .column,
.ui.grid > [class*="very relaxed"].row > .column {
  padding-left: 2.5rem;
  padding-right: 2.5rem;
}

/* Coupling with UI Divider */

.ui.relaxed.grid .row + .ui.divider,
.ui.grid .relaxed.row + .ui.divider {
  margin-left: 1.5rem;
  margin-right: 1.5rem;
}

.ui[class*="very relaxed"].grid .row + .ui.divider,
.ui.grid [class*="very relaxed"].row + .ui.divider {
  margin-left: 2.5rem;
  margin-right: 2.5rem;
}

/* ----------------------
            Padded
    ----------------------- */

.ui.padded.grid:not(.vertically):not(.horizontally) {
  margin: 0;
}

[class*="horizontally padded"].ui.grid {
  margin-left: 0;
  margin-right: 0;
}

[class*="vertically padded"].ui.grid {
  margin-top: 0;
  margin-bottom: 0;
}

/* ----------------------
           "Floated"
    ----------------------- */

.ui.grid [class*="left floated"].column {
  margin-right: auto;
}

.ui.grid [class*="right floated"].column {
  margin-left: auto;
}

/* ----------------------
            Divided
    ----------------------- */

.ui.divided.grid:not([class*="vertically divided"]) > .column:not(.row),
.ui.divided.grid:not([class*="vertically divided"]) > .row > .column {
  box-shadow: -1px 0 0 0 rgba(34, 36, 38, 0.15);
}

/* Swap from padding to margin on columns to have dividers align */

.ui[class*="vertically divided"].grid > .column:not(.row),
.ui[class*="vertically divided"].grid > .row > .column {
  margin-top: 1rem;
  margin-bottom: 1rem;
  padding-top: 0;
  padding-bottom: 0;
}

.ui[class*="vertically divided"].grid > .row {
  margin-top: 0;
  margin-bottom: 0;
}

/* No divider on first column on row */

.ui.divided.grid:not([class*="vertically divided"]) > .column:first-child,
.ui.divided.grid:not([class*="vertically divided"]) > .row > .column:first-child {
  box-shadow: none;
}

/* No space on top of first row */

.ui[class*="vertically divided"].grid > .row:first-child > .column {
  margin-top: 0;
}

/* Divided Row */

.ui.grid > .divided.row > .column {
  box-shadow: -1px 0 0 0 rgba(34, 36, 38, 0.15);
}

.ui.grid > .divided.row > .column:first-child {
  box-shadow: none;
}

/* Vertically Divided */

.ui[class*="vertically divided"].grid > .row {
  position: relative;
}

.ui[class*="vertically divided"].grid > .row::before {
  position: absolute;
  content: "";
  top: 0;
  left: 0;
  width: calc(100% - 2rem);
  height: 1px;
  margin: 0 1rem;
  box-shadow: 0 -1px 0 0 rgba(34, 36, 38, 0.15);
}

/* Padded Horizontally Divided */

[class*="horizontally padded"].ui.divided.grid,
.ui.padded.divided.grid:not(.vertically):not(.horizontally) {
  width: 100%;
}

/* First Row Vertically Divided */

.ui[class*="vertically divided"].grid > .row:first-child::before {
  box-shadow: none;
}

/* Inverted Divided */

.ui.inverted.divided.grid:not([class*="vertically divided"]) > .column:not(.row),
.ui.inverted.divided.grid:not([class*="vertically divided"]) > .row > .column {
  box-shadow: -1px 0 0 0 rgba(255, 255, 255, 0.1);
}

.ui.inverted.divided.grid:not([class*="vertically divided"]) > .column:not(.row):first-child,
.ui.inverted.divided.grid:not([class*="vertically divided"]) > .row > .column:first-child {
  box-shadow: none;
}

.ui.inverted[class*="vertically divided"].grid > .row::before {
  box-shadow: 0 -1px 0 0 rgba(255, 255, 255, 0.1);
}

/* Relaxed */

.ui.relaxed[class*="vertically divided"].grid > .row::before {
  margin-left: 1.5rem;
  margin-right: 1.5rem;
  width: calc(100% - 3rem);
}

.ui[class*="very relaxed"][class*="vertically divided"].grid > .row::before {
  margin-left: 2.5rem;
  margin-right: 2.5rem;
  width: calc(100% - 5rem);
}

/* ----------------------
             Celled
    ----------------------- */

.ui.celled.grid {
  width: 100%;
  margin: 1em 0;
  box-shadow: 0 0 0 1px #d4d4d5;
}

.ui.celled.grid > .row {
  width: 100% !important;
  margin: 0;
  padding: 0;
  box-shadow: 0 -1px 0 0 #d4d4d5;
}

.ui.celled.grid > .column:not(.row),
.ui.celled.grid > .row > .column {
  box-shadow: -1px 0 0 0 #d4d4d5;
}

.ui.celled.grid > .column:first-child,
.ui.celled.grid > .row > .column:first-child {
  box-shadow: none;
}

.ui.celled.grid > .column:not(.row),
.ui.celled.grid > .row > .column {
  padding: 1em;
}

.ui.relaxed.celled.grid > .column:not(.row),
.ui.relaxed.celled.grid > .row > .column {
  padding: 1.5em;
}

.ui[class*="very relaxed"].celled.grid > .column:not(.row),
.ui[class*="very relaxed"].celled.grid > .row > .column {
  padding: 2em;
}

/* Internally Celled */

.ui[class*="internally celled"].grid {
  box-shadow: none;
  margin: 0;
}

.ui[class*="internally celled"].grid > .row:first-child {
  box-shadow: none;
}

.ui[class*="internally celled"].grid > .row > .column:first-child {
  box-shadow: none;
}

/* ----------------------
       Vertically Aligned
    ----------------------- */

/* Top Aligned */

.ui[class*="top aligned"].grid > .column:not(.row),
.ui[class*="top aligned"].grid > .row > .column,
.ui.grid > [class*="top aligned"].row > .column,
.ui.grid > [class*="top aligned"].column:not(.row),
.ui.grid > .row > [class*="top aligned"].column {
  flex-direction: column;
  vertical-align: top;
  align-self: flex-start !important;
}

/* Middle Aligned */

.ui[class*="middle aligned"].grid > .column:not(.row),
.ui[class*="middle aligned"].grid > .row > .column,
.ui.grid > [class*="middle aligned"].row > .column,
.ui.grid > [class*="middle aligned"].column:not(.row),
.ui.grid > .row > [class*="middle aligned"].column {
  flex-direction: column;
  vertical-align: middle;
  align-self: center !important;
}

/* Bottom Aligned */

.ui[class*="bottom aligned"].grid > .column:not(.row),
.ui[class*="bottom aligned"].grid > .row > .column,
.ui.grid > [class*="bottom aligned"].row > .column,
.ui.grid > [class*="bottom aligned"].column:not(.row),
.ui.grid > .row > [class*="bottom aligned"].column {
  flex-direction: column;
  vertical-align: bottom;
  align-self: flex-end !important;
}

/* Stretched */

.ui.stretched.grid > .row > .column,
.ui.stretched.grid > .column,
.ui.grid > .stretched.row > .column,
.ui.grid > .stretched.column:not(.row),
.ui.grid > .row > .stretched.column {
  display: inline-flex !important;
  align-self: stretch;
  flex-direction: column;
}

.ui.stretched.grid > .row > .column > *,
.ui.stretched.grid > .column > *,
.ui.grid > .stretched.row > .column > *,
.ui.grid > .stretched.column:not(.row) > *,
.ui.grid > .row > .stretched.column > * {
  flex-grow: 1;
}

/* ----------------------
      Horizontally Centered
    ----------------------- */

/* Left Aligned */

.ui[class*="left aligned"].grid > .column,
.ui[class*="left aligned"].grid > .row > .column,
.ui.grid > [class*="left aligned"].row > .column,
.ui.ui.grid > [class*="left aligned"].column,
.ui.ui.grid > .row > [class*="left aligned"].column {
  text-align: left;
  align-self: inherit;
}

/* Center Aligned */

.ui[class*="center aligned"].grid > .column,
.ui[class*="center aligned"].grid > .row > .column,
.ui.grid > [class*="center aligned"].row > .column,
.ui.ui.grid > [class*="center aligned"].column,
.ui.ui.grid > .row > [class*="center aligned"].column {
  text-align: center;
  align-self: inherit;
}

.ui[class*="center aligned"].grid {
  justify-content: center;
}

/* Right Aligned */

.ui[class*="right aligned"].grid > .column,
.ui[class*="right aligned"].grid > .row > .column,
.ui.grid > [class*="right aligned"].row > .column,
.ui.ui.grid > [class*="right aligned"].column,
.ui.ui.grid > .row > [class*="right aligned"].column {
  text-align: right;
  align-self: inherit;
}

/* Justified */

.ui.justified.grid > .column,
.ui.justified.grid > .row > .column,
.ui.grid > .justified.row > .column,
.ui.ui.grid > .justified.column,
.ui.ui.grid > .row > .justified.column {
  text-align: justify;
  -webkit-hyphens: auto;
  -ms-hyphens: auto;
  hyphens: auto;
}

/* ----------------------
         Colored
----------------------- */

.ui.grid > .primary.row,
.ui.grid > .primary.column,
.ui.grid > .row > .primary.column {
  background-color: #2185d0;
  color: #fff;
}

.ui.grid > .secondary.row,
.ui.grid > .secondary.column,
.ui.grid > .row > .secondary.column {
  background-color: #1b1c1d;
  color: #fff;
}

.ui.grid > .red.row,
.ui.grid > .red.column,
.ui.grid > .row > .red.column {
  background-color: #db2828;
  color: #fff;
}

.ui.grid > .orange.row,
.ui.grid > .orange.column,
.ui.grid > .row > .orange.column {
  background-color: #f2711c;
  color: #fff;
}

.ui.grid > .yellow.row,
.ui.grid > .yellow.column,
.ui.grid > .row > .yellow.column {
  background-color: #fbbd08;
  color: #fff;
}

.ui.grid > .olive.row,
.ui.grid > .olive.column,
.ui.grid > .row > .olive.column {
  background-color: #b5cc18;
  color: #fff;
}

.ui.grid > .green.row,
.ui.grid > .green.column,
.ui.grid > .row > .green.column {
  background-color: #21ba45;
  color: #fff;
}

.ui.grid > .teal.row,
.ui.grid > .teal.column,
.ui.grid > .row > .teal.column {
  background-color: #00b5ad;
  color: #fff;
}

.ui.grid > .blue.row,
.ui.grid > .blue.column,
.ui.grid > .row > .blue.column {
  background-color: #2185d0;
  color: #fff;
}

.ui.grid > .violet.row,
.ui.grid > .violet.column,
.ui.grid > .row > .violet.column {
  background-color: #6435c9;
  color: #fff;
}

.ui.grid > .purple.row,
.ui.grid > .purple.column,
.ui.grid > .row > .purple.column {
  background-color: #a333c8;
  color: #fff;
}

.ui.grid > .pink.row,
.ui.grid > .pink.column,
.ui.grid > .row > .pink.column {
  background-color: #e03997;
  color: #fff;
}

.ui.grid > .brown.row,
.ui.grid > .brown.column,
.ui.grid > .row > .brown.column {
  background-color: #a5673f;
  color: #fff;
}

.ui.grid > .grey.row,
.ui.grid > .grey.column,
.ui.grid > .row > .grey.column {
  background-color: #767676;
  color: #fff;
}

.ui.grid > .black.row,
.ui.grid > .black.column,
.ui.grid > .row > .black.column {
  background-color: #1b1c1d;
  color: #fff;
}

/* ----------------------
          Equal Width
    ----------------------- */

.ui[class*="equal width"].grid > .column:not(.row),
.ui[class*="equal width"].grid > .row > .column,
.ui.grid > [class*="equal width"].row > .column {
  display: inline-block;
  flex-grow: 1;
}

.ui[class*="equal width"].grid > .wide.column,
.ui[class*="equal width"].grid > .row > .wide.column,
.ui.grid > [class*="equal width"].row > .wide.column {
  flex-grow: 0;
}

/* ----------------------
            Reverse
    ----------------------- */

/* Mobile */

@media only screen and (max-width: 767.98px) {
  .ui[class*="mobile reversed"].grid,
  .ui[class*="mobile reversed"].grid > .row,
  .ui.grid > [class*="mobile reversed"].row {
    flex-direction: row-reverse;
  }

  .ui[class*="mobile vertically reversed"].grid,
  .ui.stackable[class*="mobile reversed"] {
    flex-direction: column-reverse;
  }

  /* Divided Reversed */

  .ui[class*="mobile reversed"].divided.grid:not([class*="vertically divided"]) > .column:first-child,
  .ui[class*="mobile reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:first-child {
    box-shadow: -1px 0 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui[class*="mobile reversed"].divided.grid:not([class*="vertically divided"]) > .column:last-child,
  .ui[class*="mobile reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:last-child {
    box-shadow: none;
  }

  /* Vertically Divided Reversed */

  .ui.grid[class*="vertically divided"][class*="mobile vertically reversed"] > .row:first-child::before {
    box-shadow: 0 -1px 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui.grid[class*="vertically divided"][class*="mobile vertically reversed"] > .row:last-child::before {
    box-shadow: none;
  }

  /* Celled Reversed */

  .ui[class*="mobile reversed"].celled.grid > .row > .column:first-child {
    box-shadow: -1px 0 0 0 #d4d4d5;
  }

  .ui[class*="mobile reversed"].celled.grid > .row > .column:last-child {
    box-shadow: none;
  }
}

/* Tablet */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui[class*="tablet reversed"].grid,
  .ui[class*="tablet reversed"].grid > .row,
  .ui.grid > [class*="tablet reversed"].row {
    flex-direction: row-reverse;
  }

  .ui[class*="tablet vertically reversed"].grid {
    flex-direction: column-reverse;
  }

  /* Divided Reversed */

  .ui[class*="tablet reversed"].divided.grid:not([class*="vertically divided"]) > .column:first-child,
  .ui[class*="tablet reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:first-child {
    box-shadow: -1px 0 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui[class*="tablet reversed"].divided.grid:not([class*="vertically divided"]) > .column:last-child,
  .ui[class*="tablet reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:last-child {
    box-shadow: none;
  }

  /* Vertically Divided Reversed */

  .ui.grid[class*="vertically divided"][class*="tablet vertically reversed"] > .row:first-child::before {
    box-shadow: 0 -1px 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui.grid[class*="vertically divided"][class*="tablet vertically reversed"] > .row:last-child::before {
    box-shadow: none;
  }

  /* Celled Reversed */

  .ui[class*="tablet reversed"].celled.grid > .row > .column:first-child {
    box-shadow: -1px 0 0 0 #d4d4d5;
  }

  .ui[class*="tablet reversed"].celled.grid > .row > .column:last-child {
    box-shadow: none;
  }
}

/* Computer */

@media only screen and (min-width: 992px) {
  .ui[class*="computer reversed"].grid,
  .ui[class*="computer reversed"].grid > .row,
  .ui.grid > [class*="computer reversed"].row {
    flex-direction: row-reverse;
  }

  .ui[class*="computer vertically reversed"].grid {
    flex-direction: column-reverse;
  }

  /* Divided Reversed */

  .ui[class*="computer reversed"].divided.grid:not([class*="vertically divided"]) > .column:first-child,
  .ui[class*="computer reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:first-child {
    box-shadow: -1px 0 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui[class*="computer reversed"].divided.grid:not([class*="vertically divided"]) > .column:last-child,
  .ui[class*="computer reversed"].divided.grid:not([class*="vertically divided"]) > .row > .column:last-child {
    box-shadow: none;
  }

  /* Vertically Divided Reversed */

  .ui.grid[class*="vertically divided"][class*="computer vertically reversed"] > .row:first-child::before {
    box-shadow: 0 -1px 0 0 rgba(34, 36, 38, 0.15);
  }

  .ui.grid[class*="vertically divided"][class*="computer vertically reversed"] > .row:last-child::before {
    box-shadow: none;
  }

  /* Celled Reversed */

  .ui[class*="computer reversed"].celled.grid > .row > .column:first-child {
    box-shadow: -1px 0 0 0 #d4d4d5;
  }

  .ui[class*="computer reversed"].celled.grid > .row > .column:last-child {
    box-shadow: none;
  }
}

/* -------------------
          Doubling
    -------------------- */

/* Tablet Only */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.doubling.grid {
    width: auto;
  }

  .ui.grid > .doubling.row,
  .ui.doubling.grid > .row {
    margin: 0 !important;
    padding: 0 !important;
  }

  .ui.grid > .doubling.row > .column,
  .ui.doubling.grid > .row > .column {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
    box-shadow: none !important;
    margin: 0;
  }

  .ui.grid:not(.stretched) > .doubling.row:not(.stretched) > .column:not(.stretched),
  .ui.doubling.grid:not(.stretched) > .row:not(.stretched) > .column:not(.stretched) {
    display: inline-block !important;
  }

  .ui[class*="two column"].doubling.grid > .row > .column,
  .ui[class*="two column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="two column"].doubling.row > .column {
    width: 100% !important;
  }

  .ui[class*="three column"].doubling.grid > .row > .column,
  .ui[class*="three column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="three column"].doubling.row > .column {
    width: 50% !important;
  }

  .ui[class*="four column"].doubling.grid > .row > .column,
  .ui[class*="four column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="four column"].doubling.row > .column {
    width: 50% !important;
  }

  .ui[class*="five column"].doubling.grid > .row > .column,
  .ui[class*="five column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="five column"].doubling.row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="six column"].doubling.grid > .row > .column,
  .ui[class*="six column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="six column"].doubling.row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="seven column"].doubling.grid > .row > .column,
  .ui[class*="seven column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="seven column"].doubling.row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="eight column"].doubling.grid > .row > .column,
  .ui[class*="eight column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="eight column"].doubling.row > .column {
    width: 25% !important;
  }

  .ui[class*="nine column"].doubling.grid > .row > .column,
  .ui[class*="nine column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="nine column"].doubling.row > .column {
    width: 25% !important;
  }

  .ui[class*="ten column"].doubling.grid > .row > .column,
  .ui[class*="ten column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="ten column"].doubling.row > .column {
    width: 20% !important;
  }

  .ui[class*="eleven column"].doubling.grid > .row > .column,
  .ui[class*="eleven column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="eleven column"].doubling.row > .column {
    width: 20% !important;
  }

  .ui[class*="twelve column"].doubling.grid > .row > .column,
  .ui[class*="twelve column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="twelve column"].doubling.row > .column {
    width: 16.66666667% !important;
  }

  .ui[class*="thirteen column"].doubling.grid > .row > .column,
  .ui[class*="thirteen column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="thirteen column"].doubling.row > .column {
    width: 16.66666667% !important;
  }

  .ui[class*="fourteen column"].doubling.grid > .row > .column,
  .ui[class*="fourteen column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="fourteen column"].doubling.row > .column {
    width: 14.28571429% !important;
  }

  .ui[class*="fifteen column"].doubling.grid > .row > .column,
  .ui[class*="fifteen column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="fifteen column"].doubling.row > .column {
    width: 14.28571429% !important;
  }

  .ui[class*="sixteen column"].doubling.grid > .row > .column,
  .ui[class*="sixteen column"].doubling.grid > .column:not(.row),
  .ui.ui.grid > [class*="sixteen column"].doubling.row > .column {
    width: 12.5% !important;
  }
}

/* Mobile Only */

@media only screen and (max-width: 767.98px) {
  .ui.grid > .doubling.row,
  .ui.doubling.grid > .row {
    margin: 0 !important;
    padding: 0 !important;
  }

  .ui.grid > .doubling.row > .column,
  .ui.doubling.grid > .row > .column {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
    margin: 0 !important;
    box-shadow: none !important;
  }

  .ui[class*="two column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="two column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="two column"].doubling:not(.stackable).row > .column {
    width: 100% !important;
  }

  .ui[class*="three column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="three column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="three column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="four column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="four column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="four column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="five column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="five column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="five column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="six column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="six column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="six column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="seven column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="seven column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="seven column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="eight column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="eight column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="eight column"].doubling:not(.stackable).row > .column {
    width: 50% !important;
  }

  .ui[class*="nine column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="nine column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="nine column"].doubling:not(.stackable).row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="ten column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="ten column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="ten column"].doubling:not(.stackable).row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="eleven column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="eleven column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="eleven column"].doubling:not(.stackable).row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="twelve column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="twelve column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="twelve column"].doubling:not(.stackable).row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="thirteen column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="thirteen column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="thirteen column"].doubling:not(.stackable).row > .column {
    width: 33.33333333% !important;
  }

  .ui[class*="fourteen column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="fourteen column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="fourteen column"].doubling:not(.stackable).row > .column {
    width: 25% !important;
  }

  .ui[class*="fifteen column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="fifteen column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="fifteen column"].doubling:not(.stackable).row > .column {
    width: 25% !important;
  }

  .ui[class*="sixteen column"].doubling:not(.stackable).grid > .row > .column,
  .ui[class*="sixteen column"].doubling:not(.stackable).grid > .column:not(.row),
  .ui.ui.grid > [class*="sixteen column"].doubling:not(.stackable).row > .column {
    width: 25% !important;
  }
}

/* -------------------
          Stackable
    -------------------- */

@media only screen and (max-width: 767.98px) {
  body > .ui.stackable.grid,
  .ui:not(.segment):not(.grid) .ui.stackable.grid {
    width: auto;
    margin-left: 0;
    margin-right: 0;
  }

  .ui.stackable.grid > .row > .wide.column,
  .ui.stackable.grid > .wide.column,
  .ui.stackable.grid > .column.grid > .column,
  .ui.stackable.grid > .column.row > .column,
  .ui.stackable.grid > .row > .column,
  .ui.stackable.grid > .column:not(.row),
  .ui.grid > .stackable.stackable.stackable.row > .column {
    width: 100% !important;
    margin: 0 !important;
    box-shadow: none !important;
    padding: 1rem 1rem;
  }

  .ui.stackable.grid:not(.vertically) > .row {
    margin: 0;
    padding: 0;
  }

  /* Coupling */

  .ui.container > .ui.stackable.grid > .column,
  .ui.container > .ui.stackable.grid > .row > .column {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }

  /* Don't pad inside segment or nested grid */

  .ui.grid .ui.stackable.grid,
  .ui.segment:not(.vertical) .ui.stackable.page.grid {
    margin-left: -1rem;
    margin-right: -1rem;
  }

  /* Divided Stackable */

  .ui.stackable.divided.grid > .row:first-child > .column:first-child,
  .ui.stackable.celled.grid > .row:first-child > .column:first-child,
  .ui.stackable.divided.grid > .column:not(.row):first-child,
  .ui.stackable.celled.grid > .column:not(.row):first-child {
    border-top: none !important;
  }

  .ui.inverted.stackable.celled.grid > .column:not(.row),
  .ui.inverted.stackable.divided.grid > .column:not(.row),
  .ui.inverted.stackable.celled.grid > .row > .column,
  .ui.inverted.stackable.divided.grid > .row > .column {
    border-top: 1px solid rgba(255, 255, 255, 0.1);
  }

  .ui.stackable.celled.grid > .column:not(.row),
  .ui.stackable.divided:not(.vertically).grid > .column:not(.row),
  .ui.stackable.celled.grid > .row > .column,
  .ui.stackable.divided:not(.vertically).grid > .row > .column {
    border-top: 1px solid rgba(34, 36, 38, 0.15);
    box-shadow: none !important;
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }

  .ui.stackable.celled.grid > .row {
    box-shadow: none !important;
  }

  .ui.stackable.divided:not(.vertically).grid > .column:not(.row),
  .ui.stackable.divided:not(.vertically).grid > .row > .column {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
}

/* ----------------------
     Only (Device)
----------------------- */

/* These include arbitrary class repetitions for forced specificity */

/* Mobile Only Hide */

@media only screen and (max-width: 767.98px) {
  .ui.ui.ui[class*="tablet only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="tablet only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="tablet only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="tablet only"].column:not(.mobile) {
    display: none !important;
  }

  .ui.ui.ui[class*="computer only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="computer only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="computer only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="computer only"].column:not(.mobile) {
    display: none !important;
  }

  .ui.ui.ui[class*="large screen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="large screen only"].column:not(.mobile) {
    display: none !important;
  }

  .ui.ui.ui[class*="widescreen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="widescreen only"].column:not(.mobile) {
    display: none !important;
  }
}

/* Tablet Only Hide */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.ui.ui[class*="mobile only"].grid:not(.tablet),
  .ui.ui.ui.grid > [class*="mobile only"].row:not(.tablet),
  .ui.ui.ui.grid > [class*="mobile only"].column:not(.tablet),
  .ui.ui.ui.grid > .row > [class*="mobile only"].column:not(.tablet) {
    display: none !important;
  }

  .ui.ui.ui[class*="computer only"].grid:not(.tablet),
  .ui.ui.ui.grid > [class*="computer only"].row:not(.tablet),
  .ui.ui.ui.grid > [class*="computer only"].column:not(.tablet),
  .ui.ui.ui.grid > .row > [class*="computer only"].column:not(.tablet) {
    display: none !important;
  }

  .ui.ui.ui[class*="large screen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="large screen only"].column:not(.mobile) {
    display: none !important;
  }

  .ui.ui.ui[class*="widescreen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="widescreen only"].column:not(.mobile) {
    display: none !important;
  }
}

/* Computer Only Hide */

@media only screen and (min-width: 992px) and (max-width: 1199.98px) {
  .ui.ui.ui[class*="mobile only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="mobile only"].column:not(.computer) {
    display: none !important;
  }

  .ui.ui.ui[class*="tablet only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="tablet only"].column:not(.computer) {
    display: none !important;
  }

  .ui.ui.ui[class*="large screen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="large screen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="large screen only"].column:not(.mobile) {
    display: none !important;
  }

  .ui.ui.ui[class*="widescreen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="widescreen only"].column:not(.mobile) {
    display: none !important;
  }
}

/* Large Screen Only Hide */

@media only screen and (min-width: 1200px) and (max-width: 1919.98px) {
  .ui.ui.ui[class*="mobile only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="mobile only"].column:not(.computer) {
    display: none !important;
  }

  .ui.ui.ui[class*="tablet only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="tablet only"].column:not(.computer) {
    display: none !important;
  }

  .ui.ui.ui[class*="widescreen only"].grid:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].row:not(.mobile),
  .ui.ui.ui.grid > [class*="widescreen only"].column:not(.mobile),
  .ui.ui.ui.grid > .row > [class*="widescreen only"].column:not(.mobile) {
    display: none !important;
  }
}

/* Widescreen Only Hide */

@media only screen and (min-width: 1920px) {
  .ui.ui.ui[class*="mobile only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="mobile only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="mobile only"].column:not(.computer) {
    display: none !important;
  }

  .ui.ui.ui[class*="tablet only"].grid:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].row:not(.computer),
  .ui.ui.ui.grid > [class*="tablet only"].column:not(.computer),
  .ui.ui.ui.grid > .row > [class*="tablet only"].column:not(.computer) {
    display: none !important;
  }
}

/* -----------------
          Compact
    ----------------- */

.ui.ui.ui.compact.grid {
  margin: -0.5rem;
}

.ui.ui.ui.compact.grid > .column:not(.row),
.ui.ui.ui.compact.grid > .row > .column {
  padding-left: 0.5rem;
  padding-right: 0.5rem;
}

.ui.ui.ui.compact.grid > * {
  padding-left: 0.5rem;
  padding-right: 0.5rem;
}

/* Row */

.ui.ui.ui.compact.grid > .row {
  padding: 0.5rem 0;
}

/* Columns */

.ui.ui.ui.compact.grid > .column:not(.row) {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
}

/* Relaxed + Celled */

.ui.compact.relaxed.celled.grid > .column:not(.row),
.ui.compact.relaxed.celled.grid > .row > .column {
  padding: 0.75em;
}

.ui.compact[class*="very relaxed"].celled.grid > .column:not(.row),
.ui.compact[class*="very relaxed"].celled.grid > .row > .column {
  padding: 1em;
}

/* -----------------
        Very compact
    ----------------- */

.ui.ui.ui[class*="very compact"].grid {
  margin: -0.25rem;
}

.ui.ui.ui[class*="very compact"].grid > .column:not(.row),
.ui.ui.ui[class*="very compact"].grid > .row > .column {
  padding-left: 0.25rem;
  padding-right: 0.25rem;
}

.ui.ui.ui[class*="very compact"].grid > * {
  padding-left: 0.25rem;
  padding-right: 0.25rem;
}

/* Row */

.ui.ui.ui[class*="very compact"].grid > .row {
  padding: 0.25rem 0;
}

/* Columns */

.ui.ui.ui[class*="very compact"].grid > .column:not(.row) {
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
}

/* Relaxed + Celled */

.ui[class*="very compact"].relaxed.celled.grid > .column:not(.row),
.ui[class*="very compact"].relaxed.celled.grid > .row > .column {
  padding: 0.375em;
}

.ui[class*="very compact"][class*="very relaxed"].celled.grid > .column:not(.row),
.ui[class*="very compact"][class*="very relaxed"].celled.grid > .row > .column {
  padding: 0.5em;
}

.ui.grid .left.attached.column {
  padding-right: 0;
}

.ui.grid .right.attached.column {
  padding-left: 0;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Menu
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Standard
*******************************/

/* --------------
      Menu
--------------- */

.ui.menu {
  display: flex;
  margin: 1rem 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  background: #fff;
  font-weight: normal;
  border: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
  min-height: 2.85714286em;
}

.ui.menu::after {
  content: "";
  display: block;
  height: 0;
  clear: both;
  visibility: hidden;
}

.ui.menu:first-child {
  margin-top: 0;
}

.ui.menu:last-child {
  margin-bottom: 0;
}

/* --------------
    Sub-Menu
--------------- */

.ui.menu .menu {
  margin: 0;
}

.ui.menu:not(.vertical) > .menu {
  display: flex;
}

/* --------------
      Item
--------------- */

.ui.menu:not(.vertical) .item {
  display: flex;
  align-items: center;
}

.ui.menu .item {
  position: relative;
  vertical-align: middle;
  line-height: 1;
  text-decoration: none;
  -webkit-tap-highlight-color: transparent;
  flex: 0 0 auto;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  background: none;
  padding: 0.92857143em 1.14285714em;
  text-transform: none;
  color: rgba(0, 0, 0, 0.87);
  font-weight: normal;
  transition: background 0.1s ease, box-shadow 0.1s ease, color 0.1s ease;
}

.ui.menu > .item:first-child {
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

/* Border */

.ui.menu .item::before {
  position: absolute;
  content: "";
  top: 0;
  right: 0;
  height: 100%;
  width: 1px;
  background: rgba(34, 36, 38, 0.1);
}

/* --------------
  Text Content
--------------- */

.ui.menu .text.item > *,
.ui.menu .item > a:not(.ui),
.ui.menu .item > p:only-child {
  -webkit-user-select: text;
  -ms-user-select: text;
  user-select: text;
  line-height: 1.3;
}

.ui.menu .item > p:first-child {
  margin-top: 0;
}

.ui.menu .item > p:last-child {
  margin-bottom: 0;
}

/* --------------
      Icons
--------------- */

.ui.menu .item > i.icon {
  opacity: 0.9;
  float: none;
  margin: 0 0.35714286em 0 0;
}

/* --------------
     Button
--------------- */

.ui.menu:not(.vertical) .item > .button {
  position: relative;
  top: 0;
  margin: -0.5em 0;
  padding-bottom: 0.78571429em;
  padding-top: 0.78571429em;
  font-size: 1em;
}

/* ----------------
 Grid / Container
----------------- */

.ui.menu > .grid,
.ui.menu > .container {
  display: flex;
  align-items: inherit;
  flex-direction: inherit;
}

/* --------------
     Inputs
--------------- */

.ui.menu .item > .input {
  width: 100%;
}

.ui.menu:not(.vertical) .item > .input {
  position: relative;
  top: 0;
  margin: -0.5em 0;
}

.ui.menu .item > .input input {
  font-size: 1em;
  padding-top: 0.57142857em;
  padding-bottom: 0.57142857em;
}

/* --------------
     Header
--------------- */

.ui.menu .header.item,
.ui.vertical.menu .header.item {
  margin: 0;
  background: "";
  text-transform: normal;
  font-weight: bold;
}

.ui.vertical.menu .item > .header:not(.ui) {
  margin: 0 0 0.5em;
  font-size: 1em;
  font-weight: bold;
}

/* --------------
    Dropdowns
--------------- */

/* Dropdown Icon */

.ui.menu .item > i.dropdown.icon {
  padding: 0;
  float: right;
  margin: 0 0 0 1em;
}

/* Menu */

.ui.menu .dropdown.item .menu {
  min-width: calc(100% - 1px);
  border-radius: 0 0 0.28571429rem 0.28571429rem;
  background: #fff;
  margin: 0 0 0;
  box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.08);
}

.ui.menu .dropdown.item:not(.column) .menu {
  flex-direction: column;
}

/* Menu Items */

.ui.menu .ui.dropdown .menu > .item {
  margin: 0;
  text-align: left;
  font-size: 1em !important;
  padding: 0.78571429em 1.14285714em !important;
  background: transparent !important;
  color: rgba(0, 0, 0, 0.87) !important;
  text-transform: none !important;
  font-weight: normal !important;
  box-shadow: none !important;
  transition: none !important;
}

.ui.menu .ui.dropdown .menu > .item:hover {
  background: rgba(0, 0, 0, 0.05) !important;
  color: rgba(0, 0, 0, 0.95) !important;
}

.ui.menu .ui.dropdown .menu > .selected.item {
  background: rgba(0, 0, 0, 0.05) !important;
  color: rgba(0, 0, 0, 0.95) !important;
}

.ui.menu .ui.dropdown .menu > .active.item {
  background: rgba(0, 0, 0, 0.03) !important;
  font-weight: bold !important;
  color: rgba(0, 0, 0, 0.95) !important;
}

.ui.menu .ui.dropdown.item .menu .item:not(.filtered) {
  display: block;
}

.ui.menu .ui.dropdown .menu > .item > .icons,
.ui.menu .ui.dropdown .menu > .item > i.icon:not(.dropdown) {
  display: inline-block;
  font-size: 1em !important;
  float: none;
  margin: 0 0.75em 0 0 !important;
}

/* Secondary */

.ui.secondary.menu .dropdown.item > .menu,
.ui.text.menu .dropdown.item > .menu {
  border-radius: 0.28571429rem;
  margin-top: 0.35714286em;
}

/* Pointing */

.ui.menu .pointing.dropdown.item .menu {
  margin-top: 0.75em;
}

/* Inverted */

.ui.inverted.menu .search.dropdown.item > .search,
.ui.inverted.menu .search.dropdown.item > .text {
  color: rgba(255, 255, 255, 0.9);
}

/* Vertical */

.ui.vertical.menu .dropdown.item > i.icon {
  float: right;
  content: "\f0da";
  margin-left: 1em;
}

.ui.vertical.menu .dropdown.item .menu {
  left: 100%;
  /* IE needs 0, all others support max-content to show dropdown icon inline, so keep both settings! */
  min-width: 0;
  min-width: -webkit-max-content;
  min-width: max-content;
  margin: 0 0 0 0;
  box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.08);
  border-radius: 0 0.28571429rem 0.28571429rem 0.28571429rem;
}

.ui.vertical.menu .dropdown.item.upward .menu {
  bottom: 0;
}

.ui.vertical.menu .dropdown.item:not(.upward) .menu {
  top: 0;
}

.ui.vertical.menu .active.dropdown.item {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.vertical.menu .dropdown.active.item {
  box-shadow: none;
}

/* Evenly Divided */

.ui.item.menu .dropdown .menu .item {
  width: 100%;
}

/* --------------
     Labels
--------------- */

.ui.menu .item > .label:not(.floating) {
  margin-left: 1em;
  padding: 0.3em 0.78571429em;
}

.ui.vertical.menu .item > .label {
  margin-top: -0.15em;
  margin-bottom: -0.15em;
  padding: 0.3em 0.78571429em;
}

.ui.menu .item > .floating.label {
  padding: 0.3em 0.78571429em;
}

.ui.menu .item > .label {
  background: #999;
  color: #fff;
}

.ui.menu .item > .image.label img {
  margin: -0.2833em 0.8em -0.2833em -0.8em;
  height: 1.5666em;
}

/* --------------
     Images
--------------- */

.ui.menu .item > img:not(.ui) {
  display: inline-block;
  vertical-align: middle;
  margin: -0.3em 0;
  width: 2.5em;
}

.ui.vertical.menu .item > img:not(.ui):only-child {
  display: block;
  max-width: 100%;
  width: auto;
}

/*******************************
          Coupling
*******************************/

/* --------------
     List
--------------- */

/* Menu divider shouldn't apply */

.ui.menu .list .item::before {
  background: none !important;
}

/* --------------
         Sidebar
    --------------- */

/* Show vertical dividers below last */

.ui.vertical.sidebar.menu > .item:first-child::before {
  display: block !important;
}

.ui.vertical.sidebar.menu > .item::before {
  top: auto;
  bottom: 0;
}

/* --------------
    Container
--------------- */

@media only screen and (max-width: 767.98px) {
  .ui.menu > .ui.container {
    width: 100% !important;
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
}

@media only screen and (min-width: 768px) {
  .ui.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) > .container > .item:not(.right):not(.borderless):first-child {
    border-left: 1px solid rgba(34, 36, 38, 0.1);
  }

  .ui.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) > .container > .right.item:not(.borderless):last-child,
  .ui.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) > .container > .right.menu > .item:not(.borderless):last-child {
    border-right: 1px solid rgba(34, 36, 38, 0.1);
  }
}

/*******************************
             States
*******************************/

/* --------------
      Hover
--------------- */

.ui.link.menu .item:hover,
.ui.menu .dropdown.item:hover,
.ui.menu .link.item:hover,
.ui.menu a.item:hover {
  cursor: pointer;
  background: rgba(0, 0, 0, 0.03);
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
     Pressed
--------------- */

.ui.link.menu .item:active,
.ui.menu .link.item:active,
.ui.menu a.item:active {
  background: rgba(0, 0, 0, 0.03);
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
     Active
--------------- */

.ui.menu .active.item {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
  font-weight: normal;
  box-shadow: none;
}

.ui.menu .active.item > i.icon {
  opacity: 1;
}

/* --------------
  Active Hover
--------------- */

.ui.menu .active.item:hover,
.ui.vertical.menu .active.item:hover {
  background-color: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
     Disabled
--------------- */

.ui.ui.menu .item.disabled {
  cursor: default;
  background-color: transparent;
  color: rgba(40, 40, 40, 0.3);
  pointer-events: none;
}

/*******************************
             Types
*******************************/

/* ------------------
Floated Menu / Item
------------------- */

/* Left Floated */

.ui.menu:not(.vertical) .left.item,
.ui.menu:not(.vertical) .left.menu {
  display: flex;
  margin-right: auto !important;
}

/* Right Floated */

.ui.menu:not(.vertical) .right.item,
.ui.menu:not(.vertical) .right.menu {
  display: flex;
  margin-left: auto !important;
}

.ui.menu:not(.vertical) :not(.dropdown) > .left.menu,
.ui.menu:not(.vertical) :not(.dropdown) > .right.menu {
  display: inherit;
}

/* Center */

.ui.menu:not(.vertical) .center.item,
.ui.menu:not(.vertical) .center.menu {
  display: flex;
  margin-left: auto !important;
  margin-right: auto !important;
}

/* Swapped Borders */

.ui.menu .right.item::before,
.ui.menu .right.menu > .item::before {
  right: auto;
  left: 0;
}

/* Remove Outer Borders */

.ui.menu .center.item:last-child::before,
.ui.menu .center.menu > .item:last-child::before {
  display: none;
}

/* --------------
        Vertical
    --------------- */

.ui.vertical.menu {
  display: block;
  flex-direction: column;
  background: #fff;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15);
}

/* --- Item --- */

.ui.vertical.menu .item {
  display: block;
  background: none;
  border-top: none;
  border-right: none;
}

.ui.vertical.menu > .item:first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.vertical.menu > .item:last-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* --- Label --- */

.ui.vertical.menu .item > .label {
  float: right;
  text-align: center;
}

/* --- Icon --- */

.ui.vertical.menu .item > i.icon,
.ui.vertical.menu .item > i.icons {
  width: 1.18em;
  float: right;
  margin: 0 0 0 0.5em;
}

.ui.vertical.menu .item > .label + i.icon {
  float: none;
  margin: 0 0.5em 0 0;
}

/* --- Border --- */

.ui.vertical.menu .item::before {
  position: absolute;
  content: "";
  top: 0;
  left: 0;
  width: 100%;
  height: 1px;
  background: rgba(34, 36, 38, 0.1);
}

.ui.vertical.menu .item:first-child::before {
  display: none !important;
}

/* --- Sub Menu --- */

.ui.vertical.menu .item > .menu {
  margin: 0.5em -1.14285714em 0;
}

.ui.vertical.menu .menu .item {
  background: none;
  padding: 0.5em 1.33333333em;
  font-size: 0.85714286em;
  color: rgba(0, 0, 0, 0.5);
}

.ui.vertical.menu .item .menu a.item:hover,
.ui.vertical.menu .item .menu .link.item:hover {
  color: rgba(0, 0, 0, 0.85);
}

.ui.vertical.menu .menu .item::before {
  display: none;
}

/* Vertical Active */

.ui.vertical.menu .active.item {
  background: rgba(0, 0, 0, 0.05);
  border-radius: 0;
  box-shadow: none;
}

.ui.vertical.menu > .active.item:first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.vertical.menu > .active.item:last-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.vertical.menu > .active.item:only-child {
  border-radius: 0.28571429rem;
}

.ui.vertical.menu .active.item .menu .active.item {
  border-left: none;
}

.ui.vertical.menu .item .menu .active.item {
  background-color: transparent;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
         Tabular
    --------------- */

.ui.tabular.menu {
  border-radius: 0;
  box-shadow: none !important;
  border: none;
  background: none transparent;
  border-bottom: 1px solid #d4d4d5;
}

.ui.tabular.fluid.menu {
  width: calc(100% + 2px) !important;
}

.ui.tabular.menu .item {
  background: transparent;
  border-bottom: none;
  border-left: 1px solid transparent;
  border-right: 1px solid transparent;
  border-top: 2px solid transparent;
  padding: 0.92857143em 1.42857143em;
  color: rgba(0, 0, 0, 0.87);
}

.ui.tabular.menu .item::before {
  display: none;
}

/* Hover */

.ui.tabular.menu .item:hover {
  background-color: transparent;
  color: rgba(0, 0, 0, 0.8);
}

/* Active */

.ui.tabular.menu .active.item {
  background: none #fff;
  color: rgba(0, 0, 0, 0.95);
  border-top-width: 1px;
  border-color: #d4d4d5;
  font-weight: bold;
  margin-bottom: -1px;
  box-shadow: none;
  border-radius: 0.28571429rem 0.28571429rem 0 0 !important;
}

.ui.tabular.menu .active.item:hover {
  cursor: default;
}

/* Coupling with segment for attachment */

.ui.tabular.menu ~ .attached:not(.top).segment {
  border-top: none;
  margin-left: 0;
  margin-top: 0;
  margin-right: 0;
  width: 100%;
}

.top.attached.segment + .ui.bottom.tabular.menu {
  position: relative;
  width: calc(100% + 2px);
  left: -1px;
}

/* Bottom Vertical Tabular */

.ui.bottom.tabular.menu {
  background: none transparent;
  border-radius: 0;
  box-shadow: none !important;
  border-bottom: none;
  border-top: 1px solid #d4d4d5;
}

.ui.bottom.tabular.menu .item {
  background: none;
  border-left: 1px solid transparent;
  border-right: 1px solid transparent;
  border-bottom: 1px solid transparent;
  border-top: none;
}

.ui.bottom.tabular.menu .active.item {
  background: none #fff;
  color: rgba(0, 0, 0, 0.95);
  border-color: #d4d4d5;
  margin: -1px 0 0 0;
  border-radius: 0 0 0.28571429rem 0.28571429rem !important;
}

/* Vertical Tabular (Left) */

.ui.vertical.tabular.menu {
  background: none transparent;
  border-radius: 0;
  box-shadow: none !important;
  border-bottom: none;
  border-right: 1px solid #d4d4d5;
}

.ui.vertical.tabular.menu .item {
  background: none;
  border-left: 1px solid transparent;
  border-bottom: 1px solid transparent;
  border-top: 1px solid transparent;
  border-right: none;
}

.ui.vertical.tabular.menu .active.item {
  background: none #fff;
  color: rgba(0, 0, 0, 0.95);
  border-color: #d4d4d5;
  margin: 0 -1px 0 0;
  border-radius: 0.28571429rem 0 0 0.28571429rem !important;
}

/* Vertical Right Tabular */

.ui.vertical.right.tabular.menu {
  background: none transparent;
  border-radius: 0;
  box-shadow: none !important;
  border-bottom: none;
  border-right: none;
  border-left: 1px solid #d4d4d5;
}

.ui.vertical.right.tabular.menu .item {
  background: none;
  border-right: 1px solid transparent;
  border-bottom: 1px solid transparent;
  border-top: 1px solid transparent;
  border-left: none;
}

.ui.vertical.right.tabular.menu .active.item {
  background: none #fff;
  color: rgba(0, 0, 0, 0.95);
  border-color: #d4d4d5;
  margin: 0 0 0 -1px;
  border-radius: 0 0.28571429rem 0.28571429rem 0 !important;
}

/* Dropdown */

.ui.tabular.menu .active.dropdown.item {
  margin-bottom: 0;
  border-left: 1px solid transparent;
  border-right: 1px solid transparent;
  border-top: 2px solid transparent;
  border-bottom: none;
}

.ui.inverted.tabular.menu .active.item,
.ui.inverted.tabular.menu .active.item:hover {
  background: none #1b1c1d;
  border-color: #555;
}

.ui.inverted.tabular.menu .item:not(.active):hover {
  color: #ffffff;
  background: transparent;
  cursor: pointer;
}

/* --------------
       Pagination
    --------------- */

.ui.pagination.menu {
  margin: 0;
  display: inline-flex;
  vertical-align: middle;
}

.ui.pagination.menu .item:last-child {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui.compact.menu .item:last-child {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui.pagination.menu:not(.wrapping) .item:last-child::before {
  display: none;
}

.ui.pagination.menu .item {
  min-width: 3em;
  text-align: center;
}

.ui.pagination.menu .icon.item i.icon {
  vertical-align: top;
}

/* Active */

.ui.pagination.menu .active.item {
  border-top: none;
  padding-top: 0.92857143em;
  background-color: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
  box-shadow: none;
}

/* --------------
       Secondary
    --------------- */

.ui.secondary.menu {
  background: none;
  margin-left: -0.35714286em;
  margin-right: -0.35714286em;
  border-radius: 0;
  border: none;
  box-shadow: none;
}

/* Item */

.ui.secondary.menu .item {
  align-self: center;
  box-shadow: none;
  border: none;
  padding: 0.78571429em 0.92857143em;
  margin: 0 0.35714286em;
  background: none;
  transition: color 0.1s ease;
  border-radius: 0.28571429rem;
}

/* No Divider */

.ui.secondary.menu .item::before {
  display: none !important;
}

/* Header */

.ui.secondary.menu .header.item {
  border-radius: 0;
  border-right: none;
  background: none transparent;
}

/* Image */

.ui.secondary.menu .item > img:not(.ui) {
  margin: 0;
}

/* Hover */

.ui.secondary.menu .dropdown.item:hover,
.ui.secondary.menu .link.item:hover,
.ui.secondary.menu a.item:hover {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
}

/* Active */

.ui.secondary.menu .active.item {
  box-shadow: none;
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
  border-radius: 0.28571429rem;
}

/* Active Hover */

.ui.secondary.menu .active.item:hover {
  box-shadow: none;
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
}

/* Inverted */

.ui.secondary.inverted.menu .link.item:not(.disabled),
.ui.secondary.inverted.menu a.item:not(.disabled) {
  color: rgba(255, 255, 255, 0.7);
}

.ui.secondary.inverted.menu .dropdown.item:hover,
.ui.secondary.inverted.menu .link.item:hover,
.ui.secondary.inverted.menu a.item:hover {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

.ui.secondary.inverted.menu .active.item {
  background: rgba(255, 255, 255, 0.15);
  color: #ffffff;
}

/* Fix item margins */

.ui.secondary.item.menu {
  margin-left: 0;
  margin-right: 0;
}

.ui.secondary.item.menu .item:last-child {
  margin-right: 0;
}

.ui.secondary.attached.menu {
  box-shadow: none;
}

/* ---------------------
           Secondary Vertical
        ----------------------- */

/* Sub Menu */

.ui.vertical.secondary.menu .item:not(.dropdown) > .menu {
  margin: 0 -0.92857143em;
}

.ui.vertical.secondary.menu .item:not(.dropdown) > .menu > .item {
  margin: 0;
  padding: 0.5em 1.33333333em;
}

.ui.secondary.vertical.menu > .item {
  border: none;
  margin: 0 0 0.35714286em;
  border-radius: 0.28571429rem !important;
}

.ui.secondary.vertical.menu > .header.item {
  border-radius: 0;
}

/* Sub Menu */

.ui.vertical.secondary.menu .item > .menu .item {
  background-color: transparent;
}

/* Inverted */

.ui.secondary.inverted.menu {
  background-color: transparent;
}

/* ---------------------
           Secondary Pointing
        ----------------------- */

.ui.secondary.pointing.menu {
  margin-left: 0;
  margin-right: 0;
  border-bottom: 2px solid rgba(34, 36, 38, 0.15);
}

.ui.secondary.pointing.menu .item {
  border-bottom: 2px solid transparent;
  border-radius: 0;
  align-self: flex-end;
  margin: 0 0 -2px;
  padding: 0.85714286em 1.14285714em;
  transition: color 0.1s ease;
}

.ui.secondary.pointing.menu .ui.dropdown .menu .item {
  border-bottom-width: 0;
}

.ui.secondary.pointing.menu .item > .label:not(.floating) {
  margin-top: -0.3em;
  margin-bottom: -0.3em;
}

.ui.secondary.pointing.menu .item > .circular.label {
  margin-top: -0.5em;
  margin-bottom: -0.5em;
}

/* Item Types */

.ui.secondary.pointing.menu .header.item {
  color: rgba(0, 0, 0, 0.85) !important;
}

.ui.secondary.pointing.menu .text.item {
  box-shadow: none !important;
}

.ui.secondary.pointing.menu .item::after {
  display: none;
}

/* Hover */

.ui.secondary.pointing.menu .dropdown.item:hover,
.ui.secondary.pointing.menu .link.item:hover,
.ui.secondary.pointing.menu a.item:hover {
  background-color: transparent;
  color: rgba(0, 0, 0, 0.87);
}

/* Pressed */

.ui.secondary.pointing.menu .dropdown.item:active,
.ui.secondary.pointing.menu .link.item:active,
.ui.secondary.pointing.menu a.item:active {
  background-color: transparent;
  border-color: rgba(34, 36, 38, 0.15);
}

/* Active */

.ui.secondary.pointing.menu .active.item {
  background-color: transparent;
  box-shadow: none;
  border-color: currentColor;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.95);
}

/* Active Hover */

.ui.secondary.pointing.menu .active.item:hover {
  border-color: currentColor;
  color: rgba(0, 0, 0, 0.95);
}

/* Active Dropdown */

.ui.secondary.pointing.menu .active.dropdown.item {
  border-color: transparent;
}

/* Vertical Pointing */

.ui.secondary.vertical.pointing.menu {
  border-bottom-width: 0;
  border-right: 2px solid rgba(34, 36, 38, 0.15);
}

.ui.secondary.vertical.pointing.menu .item {
  border-bottom: none;
  border-right: 2px solid transparent;
  border-radius: 0 !important;
  margin: 0 -2px 0 0;
}

/* Vertical Active */

.ui.secondary.vertical.pointing.menu .active.item {
  border-color: currentColor;
}

/* Inverted */

.ui.secondary.inverted.pointing.menu {
  border-color: rgba(255, 255, 255, 0.1);
}

.ui.secondary.inverted.pointing.menu .item:not(.disabled) {
  color: rgba(255, 255, 255, 0.9);
}

.ui.secondary.inverted.pointing.menu .header.item {
  color: #fff !important;
}

/* Hover */

.ui.secondary.inverted.pointing.menu .link.item:hover,
.ui.secondary.inverted.pointing.menu a.item:hover {
  color: #ffffff;
}

/* Active */

.ui.ui.secondary.inverted.pointing.menu .active.item {
  border-color: #fff;
  color: #ffffff;
  background-color: transparent;
}

/* --------------
        Text Menu
    --------------- */

.ui.text.menu {
  background: none transparent;
  border-radius: 0;
  box-shadow: none;
  border: none;
  margin: 1em -0.5em;
}

.ui.text.menu .item {
  border-radius: 0;
  box-shadow: none;
  align-self: center;
  margin: 0;
  padding: 0.35714286em 0.5em;
  font-weight: normal;
  color: rgba(0, 0, 0, 0.6);
  transition: opacity 0.1s ease;
}

/* Border */

.ui.text.menu .item::before,
.ui.text.menu .menu .item::before {
  display: none !important;
}

/* Header */

.ui.text.menu .header.item {
  background-color: transparent;
  opacity: 1;
  color: rgba(0, 0, 0, 0.85);
  font-size: 0.92857143em;
  text-transform: uppercase;
  font-weight: bold;
}

/* Image */

.ui.text.menu .item > img:not(.ui) {
  margin: 0;
}

/* --- fluid text --- */

.ui.text.item.menu .item {
  margin: 0;
}

/* --- vertical text --- */

.ui.vertical.text.menu {
  margin: 1em 0;
}

.ui.vertical.text.menu:first-child {
  margin-top: 0;
}

.ui.vertical.text.menu:last-child {
  margin-bottom: 0;
}

.ui.vertical.text.menu .item {
  margin: 0.57142857em 0;
  padding-left: 0;
  padding-right: 0;
}

.ui.vertical.text.menu .item > i.icon {
  float: none;
  margin: 0 0.35714286em 0 0;
}

.ui.vertical.text.menu .header.item {
  margin: 0.57142857em 0 0.71428571em;
}

/* Vertical Sub Menu */

.ui.vertical.text.menu .item:not(.dropdown) > .menu {
  margin: 0;
}

.ui.vertical.text.menu .item:not(.dropdown) > .menu > .item {
  margin: 0;
  padding: 0.5em 0;
}

/* --- hover --- */

.ui.text.menu .item:hover {
  opacity: 1;
  background-color: transparent;
}

/* --- active --- */

.ui.text.menu .active.item {
  background-color: transparent;
  border: none;
  box-shadow: none;
  font-weight: normal;
  color: rgba(0, 0, 0, 0.95);
}

/* --- active hover --- */

.ui.text.menu .active.item:hover {
  background-color: transparent;
}

/* Disable Bariations */

.ui.text.pointing.menu .active.item::after {
  box-shadow: none;
}

.ui.text.attached.menu {
  box-shadow: none;
}

/* Inverted */

.ui.inverted.text.menu,
.ui.inverted.text.menu .item,
.ui.inverted.text.menu .item:hover,
.ui.inverted.text.menu .active.item {
  background-color: transparent;
}

/* Fluid */

.ui.fluid.text.menu {
  margin-left: 0;
  margin-right: 0;
}

/* --------------
        Icon Only
    --------------- */

/* Vertical Menu */

.ui.vertical.icon.menu {
  display: inline-block;
  width: auto;
}

/* Item */

.ui.icon.menu .item {
  height: auto;
  text-align: center;
  color: #1b1c1d;
}

/* Icon */

.ui.icon.menu .item > i.icon:not(.dropdown) {
  margin: 0;
  opacity: 1;
}

/* Icon Glyph */

.ui.icon.menu i.icon::before {
  opacity: 1;
}

/* (x) Item Icon */

.ui.menu .icon.item > i.icon {
  width: auto;
  margin: 0 auto;
}

/* Vertical Icon */

.ui.vertical.icon.menu .item > i.icon:not(.dropdown) {
  display: block;
  opacity: 1;
  margin: 0 auto;
  float: none;
}

/* Inverted */

.ui.inverted.icon.menu .item {
  color: #fff;
}

/* --------------
       Labeled Icon
    --------------- */

/* Menu */

.ui.labeled.icon.menu {
  text-align: center;
}

/* Item */

.ui.labeled.icon.menu .item {
  min-width: 6em;
  flex-direction: column;
}

/* Icon */

.ui.labeled.icon.menu > .right.menu > .item > i.icon:not(.dropdown),
.ui.labeled.icon.menu > .item > i.icon:not(.dropdown) {
  height: 1em;
  display: block;
  font-size: 1.71428571em !important;
  margin: 0 auto 0.5rem !important;
}

/* Fluid */

.ui.fluid.labeled.icon.menu > .item {
  min-width: 0;
}

/*******************************
           Variations
*******************************/

/* --------------
        Stackable
    --------------- */

@media only screen and (max-width: 767.98px) {
  .ui.stackable.menu {
    flex-direction: column;
  }

  .ui.stackable.menu.pointing .active.item::after {
    display: none;
  }

  .ui.stackable.menu .item {
    width: 100% !important;
  }

  .ui.stackable.menu .item::before {
    position: absolute;
    content: "";
    top: auto;
    bottom: 0;
    left: 0;
    width: 100%;
    height: 1px;
    background: rgba(34, 36, 38, 0.1);
  }

  .ui.stackable.menu .left.menu,
  .ui.stackable.menu .left.item {
    margin-right: 0 !important;
  }

  .ui.stackable.menu .right.menu,
  .ui.stackable.menu .right.item {
    margin-left: 0 !important;
  }

  .ui.stackable.menu .center.menu,
  .ui.stackable.menu .center.item {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .ui.stackable.menu .right.menu,
  .ui.stackable.menu .center.menu,
  .ui.stackable.menu .left.menu {
    flex-direction: column;
  }
}

/* --------------
     Colors
--------------- */

.ui.ui.ui.menu .primary.active.item,
.ui.ui.primary.menu .active.item:hover,
.ui.ui.primary.menu .active.item {
  color: #2185d0;
}

.ui.ui.ui.menu .red.active.item,
.ui.ui.red.menu .active.item:hover,
.ui.ui.red.menu .active.item {
  color: #db2828;
}

.ui.ui.ui.menu .orange.active.item,
.ui.ui.orange.menu .active.item:hover,
.ui.ui.orange.menu .active.item {
  color: #f2711c;
}

.ui.ui.ui.menu .yellow.active.item,
.ui.ui.yellow.menu .active.item:hover,
.ui.ui.yellow.menu .active.item {
  color: #fbbd08;
}

.ui.ui.ui.menu .olive.active.item,
.ui.ui.olive.menu .active.item:hover,
.ui.ui.olive.menu .active.item {
  color: #b5cc18;
}

.ui.ui.ui.menu .green.active.item,
.ui.ui.green.menu .active.item:hover,
.ui.ui.green.menu .active.item {
  color: #21ba45;
}

.ui.ui.ui.menu .teal.active.item,
.ui.ui.teal.menu .active.item:hover,
.ui.ui.teal.menu .active.item {
  color: #00b5ad;
}

.ui.ui.ui.menu .blue.active.item,
.ui.ui.blue.menu .active.item:hover,
.ui.ui.blue.menu .active.item {
  color: #2185d0;
}

.ui.ui.ui.menu .violet.active.item,
.ui.ui.violet.menu .active.item:hover,
.ui.ui.violet.menu .active.item {
  color: #6435c9;
}

.ui.ui.ui.menu .purple.active.item,
.ui.ui.purple.menu .active.item:hover,
.ui.ui.purple.menu .active.item {
  color: #a333c8;
}

.ui.ui.ui.menu .pink.active.item,
.ui.ui.pink.menu .active.item:hover,
.ui.ui.pink.menu .active.item {
  color: #e03997;
}

.ui.ui.ui.menu .brown.active.item,
.ui.ui.brown.menu .active.item:hover,
.ui.ui.brown.menu .active.item {
  color: #a5673f;
}

.ui.ui.ui.menu .grey.active.item,
.ui.ui.grey.menu .active.item:hover,
.ui.ui.grey.menu .active.item {
  color: #767676;
}

.ui.ui.ui.menu .black.active.item,
.ui.ui.black.menu .active.item:hover,
.ui.ui.black.menu .active.item {
  color: #1b1c1d;
}

/* --------------
        Inverted
    --------------- */

.ui.inverted.menu {
  border: 0 solid transparent;
  background: #1b1c1d;
  box-shadow: none;
}

/* Menu Item */

.ui.inverted.menu .item,
.ui.inverted.menu .item > a:not(.ui) {
  background: transparent;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.menu .item.menu {
  background: transparent;
}

/* --- Border --- */

.ui.inverted.menu .item::before {
  background: rgba(255, 255, 255, 0.08);
}

.ui.vertical.inverted.menu .item::before {
  background: rgba(255, 255, 255, 0.08);
}

/* Sub Menu */

.ui.vertical.inverted.menu .menu .item,
.ui.vertical.inverted.menu .menu .item a:not(.ui) {
  color: rgba(255, 255, 255, 0.5);
}

/* Header */

.ui.inverted.menu .header.item {
  margin: 0;
  background: transparent;
  box-shadow: none;
}

/* Disabled */

.ui.ui.inverted.menu .item.disabled {
  color: rgba(225, 225, 225, 0.3);
}

/* --- Hover --- */

.ui.link.inverted.menu .item:hover,
.ui.inverted.menu .dropdown.item:hover,
.ui.inverted.menu .link.item:hover,
.ui.inverted.menu a.item:hover {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

.ui.vertical.inverted.menu .item .menu a.item:hover,
.ui.vertical.inverted.menu .item .menu .link.item:hover {
  background: transparent;
  color: #ffffff;
}

/* --- Pressed --- */

.ui.inverted.menu a.item:active,
.ui.inverted.menu .link.item:active {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

/* --- Active --- */

.ui.inverted.menu .active.item {
  background: #3d3e3f;
  color: #ffffff !important;
}

.ui.inverted.vertical.menu .item .menu .active.item {
  background: transparent;
  color: #fff;
}

.ui.inverted.pointing.menu .active.item::after {
  background: #3d3e3f;
  margin: 0 !important;
  box-shadow: none !important;
  border: none !important;
}

/* --- Active Hover --- */

.ui.inverted.menu .active.item:hover {
  background: #3d3e3f;
  color: #fff !important;
}

.ui.inverted.pointing.menu .active.item:hover::after {
  background: #3d3e3f;
}

/* --------------
         Floated
    --------------- */

.ui.floated.menu {
  float: left;
  margin: 0 0.5rem 0 0;
}

.ui.floated.menu .item:last-child::before {
  display: none;
}

.ui.right.floated.menu {
  float: right;
  margin: 0 0 0 0.5rem;
}

.ui.center.aligned.menu:not(.fluid),
.ui.centered.menu:not(.fluid) {
  display: inline-flex;
  transform: translateX(-50%);
  margin-left: 50%;
}

.ui.center.aligned.menu .item,
.ui.centered.menu .item {
  flex: 1 0 auto;
  justify-content: center;
}

/* --------------
        Inverted
    --------------- */

.ui.ui.ui.inverted.menu .primary.active.item,
.ui.ui.inverted.primary.menu {
  background-color: #2185d0;
}

.ui.inverted.primary.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.primary.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.primary.menu .active.item {
  background-color: #1678c2;
}

.ui.ui.ui.inverted.menu .red.active.item,
.ui.ui.inverted.red.menu {
  background-color: #db2828;
}

.ui.inverted.red.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.red.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.red.menu .active.item {
  background-color: #d01919;
}

.ui.ui.ui.inverted.menu .orange.active.item,
.ui.ui.inverted.orange.menu {
  background-color: #f2711c;
}

.ui.inverted.orange.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.orange.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.orange.menu .active.item {
  background-color: #f26202;
}

.ui.ui.ui.inverted.menu .yellow.active.item,
.ui.ui.inverted.yellow.menu {
  background-color: #fbbd08;
}

.ui.inverted.yellow.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.yellow.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.yellow.menu .active.item {
  background-color: #eaae00;
}

.ui.ui.ui.inverted.menu .olive.active.item,
.ui.ui.inverted.olive.menu {
  background-color: #b5cc18;
}

.ui.inverted.olive.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.olive.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.olive.menu .active.item {
  background-color: #a7bd0d;
}

.ui.ui.ui.inverted.menu .green.active.item,
.ui.ui.inverted.green.menu {
  background-color: #21ba45;
}

.ui.inverted.green.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.green.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.green.menu .active.item {
  background-color: #16ab39;
}

.ui.ui.ui.inverted.menu .teal.active.item,
.ui.ui.inverted.teal.menu {
  background-color: #00b5ad;
}

.ui.inverted.teal.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.teal.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.teal.menu .active.item {
  background-color: #009c95;
}

.ui.ui.ui.inverted.menu .blue.active.item,
.ui.ui.inverted.blue.menu {
  background-color: #2185d0;
}

.ui.inverted.blue.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.blue.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.blue.menu .active.item {
  background-color: #1678c2;
}

.ui.ui.ui.inverted.menu .violet.active.item,
.ui.ui.inverted.violet.menu {
  background-color: #6435c9;
}

.ui.inverted.violet.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.violet.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.violet.menu .active.item {
  background-color: #5829bb;
}

.ui.ui.ui.inverted.menu .purple.active.item,
.ui.ui.inverted.purple.menu {
  background-color: #a333c8;
}

.ui.inverted.purple.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.purple.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.purple.menu .active.item {
  background-color: #9627ba;
}

.ui.ui.ui.inverted.menu .pink.active.item,
.ui.ui.inverted.pink.menu {
  background-color: #e03997;
}

.ui.inverted.pink.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.pink.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.pink.menu .active.item {
  background-color: #e61a8d;
}

.ui.ui.ui.inverted.menu .brown.active.item,
.ui.ui.inverted.brown.menu {
  background-color: #a5673f;
}

.ui.inverted.brown.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.brown.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.brown.menu .active.item {
  background-color: #975b33;
}

.ui.ui.ui.inverted.menu .grey.active.item,
.ui.ui.inverted.grey.menu {
  background-color: #767676;
}

.ui.inverted.grey.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.grey.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.grey.menu .active.item {
  background-color: #838383;
}

.ui.ui.ui.inverted.menu .black.active.item,
.ui.ui.inverted.black.menu {
  background-color: #1b1c1d;
}

.ui.inverted.black.menu .item::before {
  background-color: rgba(34, 36, 38, 0.1);
}

.ui.ui.inverted.black.menu .active.item {
  background-color: rgba(0, 0, 0, 0.1);
}

.ui.inverted.pointing.black.menu .active.item {
  background-color: #27292a;
}

.ui.ui.ui.inverted.pointing.menu .active.item::after {
  background-color: inherit;
}

/* --------------
         Fitted
    --------------- */

.ui.fitted.menu .item,
.ui.fitted.menu .item .menu .item,
.ui.menu .fitted.item {
  padding: 0;
}

.ui.horizontally.fitted.menu .item,
.ui.horizontally.fitted.menu .item .menu .item,
.ui.menu .horizontally.fitted.item {
  padding-top: 0.92857143em;
  padding-bottom: 0.92857143em;
}

.ui.vertically.fitted.menu .item,
.ui.vertically.fitted.menu .item .menu .item,
.ui.menu .vertically.fitted.item {
  padding-left: 1.14285714em;
  padding-right: 1.14285714em;
}

/* --------------
       Borderless
    --------------- */

.ui.borderless.menu .item::before,
.ui.borderless.menu .item .menu .item::before,
.ui.menu .borderless.item::before {
  background: none !important;
}

/* -------------------
           Compact
    -------------------- */

.ui.compact.menu {
  display: inline-flex;
  margin: 0;
  vertical-align: middle;
}

.ui.compact.vertical.menu {
  /* IE hack to make dropdown icons appear inline */
  display: -ms-inline-flexbox !important;
  display: inline-block;
}

.ui.compact.menu:not(.secondary) .item:last-child {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui.compact.menu .item:last-child::before {
  display: none;
}

.ui.compact.vertical.menu {
  width: auto !important;
}

.ui.compact.vertical.menu .item:last-child::before {
  display: block;
}

/* -------------------
            Fluid
    -------------------- */

.ui.menu.fluid,
.ui.vertical.menu.fluid {
  width: 100% !important;
}

/* -------------------
          Evenly Sized
    -------------------- */

.ui[class*="equal width"].menu > .item {
  flex: 1;
}

.ui[class*="equal width"].menu > .item,
.ui.item.menu,
.ui.item.menu .item {
  width: 100%;
  padding-left: 0 !important;
  padding-right: 0 !important;
  margin-left: 0 !important;
  margin-right: 0 !important;
  text-align: center;
  justify-content: center;
}

.ui.attached[class*="equal width"].menu:not(.tabular),
.ui.attached.item.menu:not(.tabular) {
  margin: 0 -1px !important;
}

.ui[class*="equal width"].menu > .item:last-child::before,
.ui.item.menu .item:last-child::before {
  display: none;
}

.ui.menu.two.item .item {
  width: 50%;
}

.ui.menu.three.item .item {
  width: 33.333%;
}

.ui.menu.four.item .item {
  width: 25%;
}

.ui.menu.five.item .item {
  width: 20%;
}

.ui.menu.six.item .item {
  width: 16.666%;
}

.ui.menu.seven.item .item {
  width: 14.285%;
}

.ui.menu.eight.item .item {
  width: 12.5%;
}

.ui.menu.nine.item .item {
  width: 11.11%;
}

.ui.menu.ten.item .item {
  width: 10%;
}

.ui.menu.eleven.item .item {
  width: 9.09%;
}

.ui.menu.twelve.item .item {
  width: 8.333%;
}

/* --------------
         Fixed
    --------------- */

.ui.menu.fixed {
  position: fixed;
  z-index: 101;
  margin: 0;
  width: 100%;
}

.ui.menu.fixed,
.ui.menu.fixed .item:first-child,
.ui.menu.fixed .item:last-child {
  border-radius: 0 !important;
}

.ui.fixed.menu,
.ui[class*="top fixed"].menu {
  top: 0;
  left: 0;
  right: auto;
  bottom: auto;
}

.ui[class*="top fixed"].menu {
  border-top: none;
  border-left: none;
  border-right: none;
}

.ui[class*="right fixed"].menu {
  border-top: none;
  border-bottom: none;
  border-right: none;
  top: 0;
  right: 0;
  left: auto;
  bottom: auto;
  width: auto;
  height: 100%;
}

.ui[class*="bottom fixed"].menu {
  border-bottom: none;
  border-left: none;
  border-right: none;
  bottom: 0;
  left: 0;
  top: auto;
  right: auto;
}

.ui[class*="left fixed"].menu {
  border-top: none;
  border-bottom: none;
  border-left: none;
  top: 0;
  left: 0;
  right: auto;
  bottom: auto;
  width: auto;
  height: 100%;
}

/* Coupling with Grid */

.ui.fixed.menu + .ui.grid {
  padding-top: 2.75rem;
}

/* -------------------
           Pointing
    -------------------- */

.ui.pointing.menu .item::after {
  visibility: hidden;
  position: absolute;
  content: "";
  top: 100%;
  left: 50%;
  transform: translateX(-50%) translateY(-50%) rotate(45deg);
  background: none;
  margin: 0.5px 0 0;
  width: 0.57142857em;
  height: 0.57142857em;
  border: none;
  border-bottom: 1px solid #d4d4d5;
  border-right: 1px solid #d4d4d5;
  z-index: 2;
  transition: background 0.1s ease;
}

.ui.vertical.pointing.menu .item::after {
  position: absolute;
  top: 50%;
  right: 0;
  bottom: auto;
  left: auto;
  transform: translateX(50%) translateY(-50%) rotate(45deg);
  margin: 0 -0.5px 0 0;
  border: none;
  border-top: 1px solid #d4d4d5;
  border-right: 1px solid #d4d4d5;
}

.ui.pointing.menu .ui.dropdown .menu .item::after,
.ui.vertical.pointing.menu .ui.dropdown .menu .item::after {
  display: none;
}

/* Active */

.ui.pointing.menu .active.item::after {
  visibility: visible;
}

.ui.pointing.menu .active.dropdown.item::after {
  visibility: hidden;
}

/* Don't double up pointers */

.ui.pointing.menu .dropdown.active.item::after,
.ui.pointing.menu .active.item .menu .active.item::after {
  display: none;
}

/* Colors */

.ui.pointing.menu .active.item:hover::after {
  background-color: #f2f2f2;
}

.ui.pointing.menu .active.item::after {
  background-color: #f2f2f2;
}

.ui.pointing.menu .active.item:hover::after {
  background-color: #f2f2f2;
}

.ui.vertical.pointing.menu .active.item:hover::after {
  background-color: #f2f2f2;
}

.ui.vertical.pointing.menu .active.item::after {
  background-color: #f2f2f2;
}

.ui.vertical.pointing.menu .menu .active.item::after {
  background-color: #fff;
}

@media only screen and (max-width: 767.98px) {
  .ui.stackable.grid .ui.fluid.vertical.pointing.menu .active.item::after {
    display: none;
  }
}

.ui.inverted.pointing.menu .primary.active.item::after {
  background-color: #2185d0;
}

.ui.inverted.pointing.menu .secondary.active.item::after {
  background-color: #1b1c1d;
}

.ui.inverted.pointing.menu .red.active.item::after {
  background-color: #db2828;
}

.ui.inverted.pointing.menu .orange.active.item::after {
  background-color: #f2711c;
}

.ui.inverted.pointing.menu .yellow.active.item::after {
  background-color: #fbbd08;
}

.ui.inverted.pointing.menu .olive.active.item::after {
  background-color: #b5cc18;
}

.ui.inverted.pointing.menu .green.active.item::after {
  background-color: #21ba45;
}

.ui.inverted.pointing.menu .teal.active.item::after {
  background-color: #00b5ad;
}

.ui.inverted.pointing.menu .blue.active.item::after {
  background-color: #2185d0;
}

.ui.inverted.pointing.menu .violet.active.item::after {
  background-color: #6435c9;
}

.ui.inverted.pointing.menu .purple.active.item::after {
  background-color: #a333c8;
}

.ui.inverted.pointing.menu .pink.active.item::after {
  background-color: #e03997;
}

.ui.inverted.pointing.menu .brown.active.item::after {
  background-color: #a5673f;
}

.ui.inverted.pointing.menu .grey.active.item::after {
  background-color: #767676;
}

.ui.inverted.pointing.menu .black.active.item::after {
  background-color: #1b1c1d;
}

/* --------------
        Attached
    --------------- */

/* Middle */

.ui.attached.menu {
  top: 0;
  bottom: 0;
  border-radius: 0;
  margin: 0 -1px;
  width: calc(100% + 2px);
  max-width: calc(100% + 2px);
  box-shadow: none;
}

.ui.attached + .ui.attached.menu:not(.top) {
  border-top: none;
}

/* Top */

.ui[class*="top attached"].menu {
  bottom: 0;
  margin-bottom: 0;
  top: 0;
  margin-top: 1rem;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.menu[class*="top attached"]:first-child {
  margin-top: 0;
}

/* Bottom */

.ui[class*="bottom attached"].menu {
  bottom: 0;
  margin-top: 0;
  top: 0;
  margin-bottom: 1rem;
  box-shadow: none;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui[class*="bottom attached"].menu:last-child {
  margin-bottom: 0;
}

/* Attached Menu Item */

.ui.top.attached.menu > .item:first-child {
  border-radius: 0.28571429rem 0 0 0;
}

.ui.bottom.attached.menu > .item:first-child {
  border-radius: 0 0 0 0.28571429rem;
}

/* Tabular Attached */

.ui.attached.menu:not(.tabular):not(.text) {
  border: 1px solid #d4d4d5;
}

.ui.attached.inverted.menu {
  border: none;
}

.ui[class*="top attached"].inverted.tabular.menu {
  border-bottom: 1px solid #555;
}

.ui.attached.tabular.menu {
  margin-left: 0;
  margin-right: 0;
  width: 100%;
}

.ui.wrapping.menu {
  flex-wrap: wrap;
}

.ui.wrapping.menu .item::before {
  right: auto;
  left: 0;
}

.ui.wrapping.menu .item:first-child::before {
  display: none;
}

.ui.wrapping.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) .item:last-of-type,
.ui.wrapping.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) .item:last-child {
  border-right: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.wrapped.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) .item:first-child {
  border-bottom-left-radius: 0;
}

.ui.wrapped.menu:not(.secondary):not(.text):not(.tabular):not(.borderless) .item:last-child {
  border-top-right-radius: 0;
}

/* --------------
     Sizes
--------------- */

.ui.menu {
  font-size: 1rem;
}

.ui.vertical.menu {
  width: 15rem;
}

.ui.mini.menu,
.ui.mini.menu .dropdown,
.ui.mini.menu .dropdown .menu > .item {
  font-size: 0.78571429rem;
}

.ui.mini.vertical.menu:not(.icon) {
  width: 9rem;
}

.ui.tiny.menu,
.ui.tiny.menu .dropdown,
.ui.tiny.menu .dropdown .menu > .item {
  font-size: 0.85714286rem;
}

.ui.tiny.vertical.menu:not(.icon) {
  width: 11rem;
}

.ui.small.menu,
.ui.small.menu .dropdown,
.ui.small.menu .dropdown .menu > .item {
  font-size: 0.92857143rem;
}

.ui.small.vertical.menu:not(.icon) {
  width: 13rem;
}

.ui.large.menu,
.ui.large.menu .dropdown,
.ui.large.menu .dropdown .menu > .item {
  font-size: 1.07142857rem;
}

.ui.large.vertical.menu:not(.icon) {
  width: 18rem;
}

.ui.big.menu,
.ui.big.menu .dropdown,
.ui.big.menu .dropdown .menu > .item {
  font-size: 1.14285714rem;
}

.ui.big.vertical.menu:not(.icon) {
  width: 20rem;
}

.ui.huge.menu,
.ui.huge.menu .dropdown,
.ui.huge.menu .dropdown .menu > .item {
  font-size: 1.21428571rem;
}

.ui.huge.vertical.menu:not(.icon) {
  width: 22rem;
}

.ui.massive.menu,
.ui.massive.menu .dropdown,
.ui.massive.menu .dropdown .menu > .item {
  font-size: 1.28571429rem;
}

.ui.massive.vertical.menu:not(.icon) {
  width: 25rem;
}

/* -------------------
  Inverted dropdowns
-------------------- */

.ui.menu .ui.inverted.inverted.dropdown.item .menu {
  background: #1b1c1d;
  box-shadow: none;
}

.ui.menu .ui.inverted.dropdown .menu > .item {
  color: rgba(255, 255, 255, 0.8) !important;
}

.ui.menu .ui.inverted.dropdown .menu > .active.item {
  background: transparent !important;
  color: rgba(255, 255, 255, 0.8) !important;
}

.ui.menu .ui.inverted.dropdown .menu > .item:hover {
  background: rgba(255, 255, 255, 0.08) !important;
  color: rgba(255, 255, 255, 0.8) !important;
}

.ui.menu .ui.inverted.dropdown .menu > .selected.item {
  background: rgba(255, 255, 255, 0.15) !important;
  color: rgba(255, 255, 255, 0.8) !important;
}

/* Vertical */

.ui.vertical.menu .inverted.dropdown.item .menu {
  box-shadow: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Message
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Message
*******************************/

.ui.message {
  position: relative;
  min-height: 1em;
  margin: 1em 0;
  background: #f8f8f9;
  padding: 1em 1.5em;
  line-height: 1.4285em;
  color: rgba(0, 0, 0, 0.87);
  transition: opacity 0.1s ease, color 0.1s ease, background 0.1s ease, box-shadow 0.1s ease;
  border-radius: 0.28571429rem;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.22) inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.message:first-child {
  margin-top: 0;
}

.ui.message:last-child {
  margin-bottom: 0;
}

/* --------------
     Content
--------------- */

/* Header */

.ui.message .header {
  display: block;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-weight: bold;
  margin: -0.14285714em 0 0 0;
}

/* Default font size */

.ui.message .header:not(.ui) {
  font-size: 1.14285714em;
}

/* Paragraph */

.ui.message p {
  opacity: 0.85;
  margin: 0.75em 0;
}

.ui.message p:first-child {
  margin-top: 0;
}

.ui.message p:last-child {
  margin-bottom: 0;
}

.ui.message .header + p {
  margin-top: 0.25em;
}

/* List */

.ui.message .list:not(.ui) {
  text-align: left;
  padding: 0;
  opacity: 0.85;
  list-style-position: inside;
  margin: 0.5em 0 0;
}

.ui.message .list:not(.ui):first-child {
  margin-top: 0;
}

.ui.message .list:not(.ui):last-child {
  margin-bottom: 0;
}

.ui.message .list:not(.ui) li {
  position: relative;
  list-style-type: none;
  margin: 0 0 0.3em 1em;
  padding: 0;
}

.ui.message .list:not(.ui) li::before {
  position: absolute;
  content: "•";
  left: -1em;
  height: 100%;
  vertical-align: baseline;
}

.ui.message .list:not(.ui) li:last-child {
  margin-bottom: 0;
}

/* Icon */

.ui.icon.message > .icons,
.ui.icon.message > i.icon {
  margin-right: 0.6em;
}

.ui.icon.message > .icons:last-child,
.ui.icon.message > i.icon:last-child {
  margin: 0 0 0 0.6em;
}

/* Close Icon */

.ui.message > .close.icon {
  cursor: pointer;
  position: absolute;
  margin: 0;
  top: 0.78575em;
  right: 0.5em;
  opacity: 0.7;
  transition: opacity 0.1s ease;
}

.ui.message > .close.icon:hover {
  opacity: 1;
}

/* First / Last Element */

.ui.message > :first-child {
  margin-top: 0;
}

.ui.message > :last-child {
  margin-bottom: 0;
}

/*******************************
            Coupling
*******************************/

.ui.dropdown .menu > .message {
  margin: 0 -1px;
}

/*******************************
            States
*******************************/

/* --------------
    Visible
--------------- */

.ui.visible.visible.visible.visible.message {
  display: block;
}

.ui.icon.visible.visible.visible.visible.message {
  display: flex;
}

/* --------------
     Hidden
--------------- */

.ui.hidden.hidden.hidden.hidden.message {
  display: none;
}

/*******************************
            Variations
*******************************/

.ui.centered.message,
.ui.center.aligned.message {
  text-align: center;
  justify-content: center;
}

.ui.centered.message > .content,
.ui.center.aligned.message > .content {
  flex: 0 0 auto;
}

.ui.right.aligned.message {
  text-align: right;
}

/* --------------
        Compact
    --------------- */

.ui.compact.message {
  display: inline-block;
}

.ui.compact.icon.message {
  display: inline-flex;
  width: auto;
}

/* --------------
        Attached
    --------------- */

.ui.attached.message {
  margin-bottom: -1px;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
  margin-left: -1px;
  margin-right: -1px;
}

.ui.attached + .ui.attached.message:not(.top):not(.bottom) {
  margin-top: -1px;
  border-radius: 0;
}

.ui.bottom.attached.message {
  margin-top: -1px;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15) inset, 0 1px 2px 0 rgba(34, 36, 38, 0.15);
}

.ui.bottom.attached.message:not(:last-child) {
  margin-bottom: 1em;
}

.ui.attached.icon.message {
  width: auto;
}

/* --------------
          Icon
    --------------- */

.ui.icon.message {
  display: flex;
  width: 100%;
  align-items: center;
}

.ui.icon.message > .icons,
.ui.icon.message > i.icon:not(.close) {
  display: block;
  flex: 0 0 auto;
  width: auto;
  line-height: 1;
  vertical-align: middle;
  font-size: 3em;
  opacity: 0.8;
}

.ui.icon.message > .content {
  display: block;
  flex: 1 1 auto;
  vertical-align: middle;
}

.ui.icon.message > .icons + .content,
.ui.icon.message > i.icon:not(.close) + .content {
  padding-left: 0;
}

.ui.icon.message > i.circular {
  width: 1em;
}

/* --------------
        Floating
    --------------- */

.ui.floating.message {
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.22) inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

/* --------------
     Colors
--------------- */

/* --------------
     Types
--------------- */

/* Colors */

.ui.positive.message {
  background-color: #fcfff5;
  color: #2c662d;
}

.ui.positive.message,
.ui.attached.positive.message {
  box-shadow: 0 0 0 1px #a3c293 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.positive.message {
  box-shadow: 0 0 0 1px #a3c293 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.positive.message .header {
  color: #1a531b;
}

.ui.inverted.positive.message,
.ui.inverted.positive.message .header {
  color: #a3c293;
}

.ui.negative.message {
  background-color: #fff6f6;
  color: #9f3a38;
}

.ui.negative.message,
.ui.attached.negative.message {
  box-shadow: 0 0 0 1px #e0b4b4 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.negative.message {
  box-shadow: 0 0 0 1px #e0b4b4 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.negative.message .header {
  color: #912d2b;
}

.ui.inverted.negative.message,
.ui.inverted.negative.message .header {
  color: #e0b4b4;
}

.ui.error.message {
  background-color: #fff6f6;
  color: #9f3a38;
}

.ui.error.message,
.ui.attached.error.message {
  box-shadow: 0 0 0 1px #e0b4b4 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.error.message {
  box-shadow: 0 0 0 1px #e0b4b4 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.error.message .header {
  color: #912d2b;
}

.ui.inverted.error.message,
.ui.inverted.error.message .header {
  color: #ecd1d1;
}

.ui.info.message {
  background-color: #f8ffff;
  color: #276f86;
}

.ui.info.message,
.ui.attached.info.message {
  box-shadow: 0 0 0 1px #a9d5de inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.info.message {
  box-shadow: 0 0 0 1px #a9d5de inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.info.message .header {
  color: #0e566c;
}

.ui.inverted.info.message,
.ui.inverted.info.message .header {
  color: #c6e3e9;
}

.ui.success.message {
  background-color: #fcfff5;
  color: #2c662d;
}

.ui.success.message,
.ui.attached.success.message {
  box-shadow: 0 0 0 1px #a3c293 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.success.message {
  box-shadow: 0 0 0 1px #a3c293 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.success.message .header {
  color: #1a531b;
}

.ui.inverted.success.message,
.ui.inverted.success.message .header {
  color: #b9d1ad;
}

.ui.warning.message {
  background-color: #fffaf3;
  color: #573a08;
}

.ui.warning.message,
.ui.attached.warning.message {
  box-shadow: 0 0 0 1px #c9ba9b inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.warning.message {
  box-shadow: 0 0 0 1px #c9ba9b inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.warning.message .header {
  color: #794b02;
}

.ui.inverted.warning.message,
.ui.inverted.warning.message .header {
  color: #d7ccb5;
}

.ui.primary.message {
  background-color: #dff0ff;
  color: rgba(255, 255, 255, 0.9);
}

.ui.primary.message,
.ui.attached.primary.message {
  box-shadow: 0 0 0 1px #2185d0 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.primary.message {
  box-shadow: 0 0 0 1px #2185d0 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.primary.message .header {
  color: rgba(242, 242, 242, 0.9);
}

.ui.secondary.message {
  background-color: #f4f4f4;
  color: rgba(255, 255, 255, 0.9);
}

.ui.secondary.message,
.ui.attached.secondary.message {
  box-shadow: 0 0 0 1px #1b1c1d inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.secondary.message {
  box-shadow: 0 0 0 1px #1b1c1d inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.secondary.message .header {
  color: rgba(242, 242, 242, 0.9);
}

.ui.red.message {
  background-color: #ffe8e6;
  color: #db2828;
}

.ui.red.message,
.ui.attached.red.message {
  box-shadow: 0 0 0 1px #db2828 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.red.message {
  box-shadow: 0 0 0 1px #db2828 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.red.message .header {
  color: #c82121;
}

.ui.orange.message {
  background-color: #ffedde;
  color: #f2711c;
}

.ui.orange.message,
.ui.attached.orange.message {
  box-shadow: 0 0 0 1px #f2711c inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.orange.message {
  box-shadow: 0 0 0 1px #f2711c inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.orange.message .header {
  color: #e7640d;
}

.ui.yellow.message {
  background-color: #fff8db;
  color: #b58105;
}

.ui.yellow.message,
.ui.attached.yellow.message {
  box-shadow: 0 0 0 1px #b58105 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.yellow.message {
  box-shadow: 0 0 0 1px #b58105 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.yellow.message .header {
  color: #9c6f04;
}

.ui.olive.message {
  background-color: #fbfdef;
  color: #8abc1e;
}

.ui.olive.message,
.ui.attached.olive.message {
  box-shadow: 0 0 0 1px #8abc1e inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.olive.message {
  box-shadow: 0 0 0 1px #8abc1e inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.olive.message .header {
  color: #7aa61a;
}

.ui.green.message {
  background-color: #e5f9e7;
  color: #1ebc30;
}

.ui.green.message,
.ui.attached.green.message {
  box-shadow: 0 0 0 1px #1ebc30 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.green.message {
  box-shadow: 0 0 0 1px #1ebc30 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.green.message .header {
  color: #1aa62a;
}

.ui.teal.message {
  background-color: #e1f7f7;
  color: #10a3a3;
}

.ui.teal.message,
.ui.attached.teal.message {
  box-shadow: 0 0 0 1px #10a3a3 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.teal.message {
  box-shadow: 0 0 0 1px #10a3a3 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.teal.message .header {
  color: #0e8c8c;
}

.ui.blue.message {
  background-color: #dff0ff;
  color: #2185d0;
}

.ui.blue.message,
.ui.attached.blue.message {
  box-shadow: 0 0 0 1px #2185d0 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.blue.message {
  box-shadow: 0 0 0 1px #2185d0 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.blue.message .header {
  color: #1e77ba;
}

.ui.violet.message {
  background-color: #eae7ff;
  color: #6435c9;
}

.ui.violet.message,
.ui.attached.violet.message {
  box-shadow: 0 0 0 1px #6435c9 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.violet.message {
  box-shadow: 0 0 0 1px #6435c9 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.violet.message .header {
  color: #5a30b5;
}

.ui.purple.message {
  background-color: #f6e7ff;
  color: #a333c8;
}

.ui.purple.message,
.ui.attached.purple.message {
  box-shadow: 0 0 0 1px #a333c8 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.purple.message {
  box-shadow: 0 0 0 1px #a333c8 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.purple.message .header {
  color: #922eb4;
}

.ui.pink.message {
  background-color: #ffe3fb;
  color: #e03997;
}

.ui.pink.message,
.ui.attached.pink.message {
  box-shadow: 0 0 0 1px #e03997 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.pink.message {
  box-shadow: 0 0 0 1px #e03997 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.pink.message .header {
  color: #dd238b;
}

.ui.brown.message {
  background-color: #f1e2d3;
  color: #a5673f;
}

.ui.brown.message,
.ui.attached.brown.message {
  box-shadow: 0 0 0 1px #a5673f inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.brown.message {
  box-shadow: 0 0 0 1px #a5673f inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.brown.message .header {
  color: #935b38;
}

.ui.grey.message {
  background-color: #f4f4f4;
  color: #767676;
}

.ui.grey.message,
.ui.attached.grey.message {
  box-shadow: 0 0 0 1px #767676 inset, 0 0 0 0 rgba(0, 0, 0, 0);
}

.ui.floating.grey.message {
  box-shadow: 0 0 0 1px #767676 inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.grey.message .header {
  color: #696969;
}

.ui.black.message {
  background-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

.ui.black.message .header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.message {
  background-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

/* --------------
     Sizes
--------------- */

.ui.message {
  font-size: 1em;
}

.ui.mini.message {
  font-size: 0.78571429em;
}

.ui.tiny.message {
  font-size: 0.85714286em;
}

.ui.small.message {
  font-size: 0.92857143em;
}

.ui.large.message {
  font-size: 1.14285714em;
}

.ui.big.message {
  font-size: 1.28571429em;
}

.ui.huge.message {
  font-size: 1.42857143em;
}

.ui.massive.message {
  font-size: 1.71428571em;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Table
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Table
*******************************/

/* Prototype */

.ui.table {
  width: 100%;
  background: #fff;
  margin: 1em 0;
  border: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: none;
  border-radius: 0.28571429rem;
  text-align: left;
  vertical-align: middle;
  color: rgba(0, 0, 0, 0.87);
  border-collapse: separate;
  border-spacing: 0;
}

.ui.table:first-child {
  margin-top: 0;
}

.ui.table:last-child {
  margin-bottom: 0;
}

.ui.table > thead,
.ui.table > tbody {
  text-align: inherit;
  vertical-align: inherit;
}

/*******************************
             Parts
*******************************/

/* Table Content */

.ui.table th,
.ui.table td {
  transition: background 0.1s ease, color 0.1s ease;
}

/* Rowspan helper class */

.ui.table th.rowspanned,
.ui.table td.rowspanned {
  display: none;
}

/* Headers */

.ui.table > thead {
  box-shadow: none;
}

.ui.table > thead > tr > th {
  cursor: auto;
  background: #f9fafb;
  text-align: inherit;
  color: rgba(0, 0, 0, 0.87);
  padding: 0.92857143em 0.78571429em;
  vertical-align: inherit;
  font-style: none;
  font-weight: bold;
  text-transform: none;
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
  border-left: none;
}

.ui.table > thead > tr > th:first-child {
  border-left: none;
}

.ui.table > thead > tr:first-child > th:first-child {
  border-radius: 0.28571429rem 0 0 0;
}

.ui.table > thead > tr:first-child > th:last-child {
  border-radius: 0 0.28571429rem 0 0;
}

.ui.table > thead > tr:first-child > th:only-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

/* Footer */

.ui.table > tfoot {
  box-shadow: none;
}

.ui.table > tfoot > tr > th,
.ui.table > tfoot > tr > td {
  cursor: auto;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  background: #f9fafb;
  text-align: inherit;
  color: rgba(0, 0, 0, 0.87);
  padding: 0.78571429em 0.78571429em;
  vertical-align: inherit;
  font-style: normal;
  font-weight: normal;
  text-transform: none;
}

.ui.table > tfoot > tr > th:first-child,
.ui.table > tfoot > tr > td:first-child {
  border-left: none;
}

.ui.table > tfoot > tr:first-child > th:first-child,
.ui.table > tfoot > tr:first-child > td:first-child {
  border-radius: 0 0 0 0.28571429rem;
}

.ui.table > tfoot > tr:first-child > th:last-child,
.ui.table > tfoot > tr:first-child > td:last-child {
  border-radius: 0 0 0.28571429rem 0;
}

.ui.table > tfoot > tr:first-child > th:only-child,
.ui.table > tfoot > tr:first-child > td:only-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* Table Row */

.ui.table > tr > td,
.ui.table > tbody > tr > td {
  border-top: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.table > tr:first-child > td,
.ui.table > tbody > tr:first-child > td {
  border-top: none;
}

/* Repeated tbody */

.ui.table > tbody + tbody tr:first-child > td {
  border-top: 1px solid rgba(34, 36, 38, 0.1);
}

/* Table Cells */

.ui.table > tbody > tr > td,
.ui.table > tr > td {
  padding: 0.78571429em 0.78571429em;
  text-align: inherit;
}

/* Icons */

.ui.table > i.icon {
  vertical-align: baseline;
}

.ui.table > i.icon:only-child {
  margin: 0;
}

/* Table Segment */

.ui.table.segment {
  padding: 0;
}

.ui.table.segment::after {
  display: none;
}

.ui.table.segment.stacked::after {
  display: block;
}

/* Responsive */

@media only screen and (max-width: 767.98px) {
  .ui.table:not(.unstackable) {
    width: 100%;
    padding: 0;
  }

  .ui.table:not(.unstackable) > thead,
  .ui.table:not(.unstackable) > thead > tr,
  .ui.table:not(.unstackable) > tfoot,
  .ui.table:not(.unstackable) > tfoot > tr,
  .ui.table:not(.unstackable) > tbody,
  .ui.table:not(.unstackable) > tr,
  .ui.table:not(.unstackable) > tbody > tr,
  .ui.table:not(.unstackable) > tr > th:not(.rowspanned),
  .ui.table:not(.unstackable) > thead > tr > th:not(.rowspanned),
  .ui.table:not(.unstackable) > tbody > tr > th:not(.rowspanned),
  .ui.table:not(.unstackable) > tfoot > tr > th:not(.rowspanned),
  .ui.table:not(.unstackable) > tr > td:not(.rowspanned),
  .ui.table:not(.unstackable) > tbody > tr > td:not(.rowspanned),
  .ui.table:not(.unstackable) > tfoot > tr > td:not(.rowspanned) {
    display: block !important;
    width: auto !important;
  }

  .ui.table:not(.unstackable) > thead {
    display: block;
  }

  .ui.table:not(.unstackable) > tfoot {
    display: block;
  }

  .ui.ui.ui.ui.table:not(.unstackable) > tr,
  .ui.ui.ui.ui.table:not(.unstackable) > thead > tr,
  .ui.ui.ui.ui.table:not(.unstackable) > tbody > tr,
  .ui.ui.ui.ui.table:not(.unstackable) > tfoot > tr {
    padding-top: 1em;
    padding-bottom: 1em;
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) > tr > th,
  .ui.ui.ui.ui.table:not(.unstackable) > thead > tr > th,
  .ui.ui.ui.ui.table:not(.unstackable) > tbody > tr > th,
  .ui.ui.ui.ui.table:not(.unstackable) > tfoot > tr > th,
  .ui.ui.ui.ui.table:not(.unstackable) > tr > td,
  .ui.ui.ui.ui.table:not(.unstackable) > tbody > tr > td,
  .ui.ui.ui.ui.table:not(.unstackable) > tfoot > tr > td {
    background: none;
    border: none;
    padding: 0.25em 0.75em;
    box-shadow: none;
  }

  .ui.table:not(.unstackable) > tr > th:first-child,
  .ui.table:not(.unstackable) > thead > tr > th:first-child,
  .ui.table:not(.unstackable) > tbody > tr > th:first-child,
  .ui.table:not(.unstackable) > tfoot > tr > th:first-child,
  .ui.table:not(.unstackable) > tr > td:first-child,
  .ui.table:not(.unstackable) > tbody > tr > td:first-child,
  .ui.table:not(.unstackable) > tfoot > tr > td:first-child {
    font-weight: bold;
  }

  /* Definition Table */

  .ui.definition.table:not(.unstackable) > thead > tr > th:first-child {
    box-shadow: none !important;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="primary marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2185d0 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="primary marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2185d0 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="primary marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #54c8ff inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="primary marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #54c8ff inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="secondary marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #1b1c1d inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="secondary marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #1b1c1d inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="secondary marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #545454 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="secondary marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #545454 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="red marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #db2828 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="red marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #db2828 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="red marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff695e inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="red marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff695e inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="orange marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #f2711c inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="orange marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #f2711c inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="orange marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff851b inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="orange marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff851b inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="yellow marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #fbbd08 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="yellow marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #fbbd08 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="yellow marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ffe21f inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="yellow marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ffe21f inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="olive marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #b5cc18 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="olive marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #b5cc18 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="olive marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #d9e778 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="olive marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #d9e778 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="green marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #21ba45 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="green marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #21ba45 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="green marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2ecc40 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="green marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2ecc40 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="teal marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #00b5ad inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="teal marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #00b5ad inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="teal marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #6dffff inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="teal marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #6dffff inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="blue marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2185d0 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="blue marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2185d0 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="blue marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #54c8ff inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="blue marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #54c8ff inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="violet marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #6435c9 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="violet marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #6435c9 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="violet marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a291fb inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="violet marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a291fb inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="purple marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a333c8 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="purple marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a333c8 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="purple marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #dc73ff inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="purple marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #dc73ff inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="pink marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #e03997 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="pink marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #e03997 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="pink marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff8edf inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="pink marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff8edf inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="brown marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a5673f inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="brown marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a5673f inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="brown marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #d67c1c inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="brown marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #d67c1c inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="grey marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #767676 inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="grey marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #767676 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="grey marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #dcddde inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="grey marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #dcddde inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="black marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #1b1c1d inset;
  }

  .ui.ui.ui.ui.table:not(.unstackable) tr[class*="black marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #1b1c1d inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="black marked"].left {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #545454 inset;
  }

  .ui.ui.ui.ui.inverted.table:not(.unstackable) tr[class*="black marked"].right {
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #545454 inset;
  }

  .ui.scrolling.table.short > tbody {
    max-height: 6.74983929em;
  }

  .resizable.ui.scrolling.table.short > tbody {
    height: 6.74983929em;
  }

  .ui.scrolling.table[class*="very short"] > tbody {
    max-height: 4.49989286em;
  }

  .resizable.ui.scrolling.table[class*="very short"] > tbody {
    height: 4.49989286em;
  }

  .ui.scrolling.table > tbody {
    max-height: 8.99978571em;
  }

  .resizable.ui.scrolling.table > tbody {
    height: 8.99978571em;
  }

  .ui.scrolling.table.long > tbody {
    max-height: 17.99957143em;
  }

  .resizable.ui.scrolling.table.long > tbody {
    height: 17.99957143em;
  }

  .ui.scrolling.table[class*="very long"] > tbody {
    max-height: 26.99935714em;
  }

  .resizable.ui.scrolling.table[class*="very long"] > tbody {
    height: 26.99935714em;
  }

  .ui.overflowing.table.short {
    max-height: 11.24973214em;
  }

  .ui.overflowing.table[class*="very short"] {
    max-height: 7.49982143em;
  }

  .ui.overflowing.table {
    max-height: 14.99964286em;
  }

  .ui.overflowing.table.long {
    max-height: 29.99928571em;
  }

  .ui.overflowing.table[class*="very long"] {
    max-height: 44.99892857em;
  }
}

/* --------------
      Scrolling
    --------------- */

@media only screen and (min-width: 768px) {
  .ui.scrolling.table.short > tbody {
    max-height: 8.99978571em;
  }

  .resizable.ui.scrolling.table.short > tbody {
    height: 8.99978571em;
  }

  .ui.scrolling.table[class*="very short"] > tbody {
    max-height: 5.99985714em;
  }

  .resizable.ui.scrolling.table[class*="very short"] > tbody {
    height: 5.99985714em;
  }

  .ui.scrolling.table > tbody {
    max-height: 11.99971429em;
  }

  .resizable.ui.scrolling.table > tbody {
    height: 11.99971429em;
  }

  .ui.scrolling.table.long > tbody {
    max-height: 23.99942857em;
  }

  .resizable.ui.scrolling.table.long > tbody {
    height: 23.99942857em;
  }

  .ui.scrolling.table[class*="very long"] > tbody {
    max-height: 35.99914286em;
  }

  .resizable.ui.scrolling.table[class*="very long"] > tbody {
    height: 35.99914286em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.scrolling.table.short > tbody {
    max-height: 13.49967857em;
  }

  .resizable.ui.scrolling.table.short > tbody {
    height: 13.49967857em;
  }

  .ui.scrolling.table[class*="very short"] > tbody {
    max-height: 8.99978571em;
  }

  .resizable.ui.scrolling.table[class*="very short"] > tbody {
    height: 8.99978571em;
  }

  .ui.scrolling.table > tbody {
    max-height: 17.99957143em;
  }

  .resizable.ui.scrolling.table > tbody {
    height: 17.99957143em;
  }

  .ui.scrolling.table.long > tbody {
    max-height: 35.99914286em;
  }

  .resizable.ui.scrolling.table.long > tbody {
    height: 35.99914286em;
  }

  .ui.scrolling.table[class*="very long"] > tbody {
    max-height: 53.99871429em;
  }

  .resizable.ui.scrolling.table[class*="very long"] > tbody {
    height: 53.99871429em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.scrolling.table.short > tbody {
    max-height: 17.99957143em;
  }

  .resizable.ui.scrolling.table.short > tbody {
    height: 17.99957143em;
  }

  .ui.scrolling.table[class*="very short"] > tbody {
    max-height: 11.99971429em;
  }

  .resizable.ui.scrolling.table[class*="very short"] > tbody {
    height: 11.99971429em;
  }

  .ui.scrolling.table > tbody {
    max-height: 23.99942857em;
  }

  .resizable.ui.scrolling.table > tbody {
    height: 23.99942857em;
  }

  .ui.scrolling.table.long > tbody {
    max-height: 47.99885714em;
  }

  .resizable.ui.scrolling.table.long > tbody {
    height: 47.99885714em;
  }

  .ui.scrolling.table[class*="very long"] > tbody {
    max-height: 71.99828571em;
  }

  .resizable.ui.scrolling.table[class*="very long"] > tbody {
    height: 71.99828571em;
  }
}

.ui.scrolling.table > thead,
.ui.scrolling.table > tfoot,
.ui.scrolling.table > tbody {
  display: block;
  overflow-y: scroll;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  scrollbar-width: thin;
  /* Firefox */
}

.ui.scrolling.table > thead > tr,
.ui.scrolling.table > tfoot > tr,
.ui.scrolling.table > tbody > tr {
  display: table;
  table-layout: fixed;
  width: 100%;
}

/* Camouflage scrollbars, we need them only to gain the same width as tbody */

.ui.scrolling.table > thead {
  background: #f9fafb;
  color: #f9fafb;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.scrolling.table > tfoot {
  background: #f9fafb;
  color: #f9fafb;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.inverted.scrolling.table > thead {
  background: rgba(0, 0, 0, 0.15);
  color: rgba(0, 0, 0, 0.15);
}

.ui.inverted.scrolling.table > tfoot {
  background: rgba(0, 0, 0, 0.15);
  color: rgba(0, 0, 0, 0.15);
}

.ui.scrolling.table > thead::-webkit-scrollbar-track,
.ui.scrolling.table > tfoot::-webkit-scrollbar-track {
  background: inherit;
  border-radius: 0.28571429rem;
}

/* Firefox & IE */

.ui.scrolling.table > thead,
.ui.scrolling.table > tfoot {
  scrollbar-color: currentColor currentColor;
  scrollbar-face-color: currentColor;
  scrollbar-shadow-color: currentColor;
  scrollbar-track-color: currentColor;
  scrollbar-arrow-color: currentColor;
}

/* IE scrollbar color needs hex values */

@media all and (-ms-high-contrast: none) {
  .ui.scrolling.table > thead {
    color: #f9fafb;
  }

  .ui.scrolling.table > tfoot {
    color: #f9fafb;
  }

  .ui.inverted.scrolling.table > thead {
    color: #252525;
  }

  .ui.inverted.scrolling.table > tfoot {
    color: #252525;
  }
}

.ui.inverted.scrolling.table > tbody::-webkit-scrollbar-track {
  background: rgba(255, 255, 255, 0.1);
}

.ui.inverted.scrolling.table > tbody::-webkit-scrollbar-thumb {
  background: rgba(255, 255, 255, 0.25);
}

.ui.inverted.scrolling.table > tbody::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(255, 255, 255, 0.15);
}

.ui.inverted.scrolling.table > tbody::-webkit-scrollbar-thumb:hover {
  background: rgba(255, 255, 255, 0.35);
}

.ui.inverted.scrolling.table > tbody {
  /* IE11 */
  scrollbar-face-color: #656565;
  scrollbar-shadow-color: #656565;
  scrollbar-track-color: #323232;
  scrollbar-arrow-color: #323232;
  /* firefox: first color thumb, second track */
  scrollbar-color: rgba(255, 255, 255, 0.25) rgba(255, 255, 255, 0.1);
}

.ui.resizable.scrolling.table > tbody {
  resize: vertical;
  max-height: none;
}

/* --------------
      Overflowing
    --------------- */

.ui.overflowing.table {
  display: block;
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

@media only screen and (min-width: 768px) {
  .ui.overflowing.table.short {
    max-height: 13.49967857em;
  }

  .ui.overflowing.table[class*="very short"] {
    max-height: 8.99978571em;
  }

  .ui.overflowing.table {
    max-height: 17.99957143em;
  }

  .ui.overflowing.table.long {
    max-height: 35.99914286em;
  }

  .ui.overflowing.table[class*="very long"] {
    max-height: 53.99871429em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.overflowing.table.short {
    max-height: 17.99957143em;
  }

  .ui.overflowing.table[class*="very short"] {
    max-height: 11.99971429em;
  }

  .ui.overflowing.table {
    max-height: 23.99942857em;
  }

  .ui.overflowing.table.long {
    max-height: 47.99885714em;
  }

  .ui.overflowing.table[class*="very long"] {
    max-height: 71.99828571em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.overflowing.table.short {
    max-height: 22.49946429em;
  }

  .ui.overflowing.table[class*="very short"] {
    max-height: 14.99964286em;
  }

  .ui.overflowing.table {
    max-height: 29.99928571em;
  }

  .ui.overflowing.table.long {
    max-height: 59.99857143em;
  }

  .ui.overflowing.table[class*="very long"] {
    max-height: 89.99785714em;
  }
}

/*******************************
            Coupling
*******************************/

/* UI Image */

.ui.table .collapsing .image,
.ui.table .collapsing .image img {
  max-width: none;
}

/*******************************
             Types
*******************************/

/* --------------
    Complex
--------------- */

.ui.structured.table {
  border-collapse: collapse;
}

.ui.structured.table > thead > tr > th {
  border-left: none;
  border-right: none;
}

.ui.structured.sortable.table > thead > tr > th {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
  border-right: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.structured.basic.table > tr > th,
.ui.structured.basic.table > thead > tr > th,
.ui.structured.basic.table > tbody > tr > th,
.ui.structured.basic.table > tfoot > tr > th {
  border-left: none;
  border-right: none;
}

.ui.structured.celled.table > tr > th,
.ui.structured.celled.table > thead > tr > th,
.ui.structured.celled.table > tbody > tr > th,
.ui.structured.celled.table > tfoot > tr > th,
.ui.structured.celled.table > tr > td,
.ui.structured.celled.table > tbody > tr > td,
.ui.structured.celled.table > tfoot > tr > td {
  border-left: 1px solid rgba(34, 36, 38, 0.1);
  border-right: 1px solid rgba(34, 36, 38, 0.1);
}

/* --------------
       Definition
    --------------- */

.ui.definition.table > thead:not(.full-width) > tr > th:first-child {
  pointer-events: none;
  background: #fff;
  font-weight: normal;
  color: rgba(0, 0, 0, 0.4);
  box-shadow: -0.1em -0.2em 0 0.1em #fff;
  -moz-transform: scale(1);
}

.ui.definition.table > thead:not(.full-width) > tr > th:first-child:not(:empty) {
  pointer-events: auto;
}

.ui.definition.table > tfoot:not(.full-width) > tr > th:first-child {
  pointer-events: none;
  background: #fff;
  font-weight: normal;
  color: rgba(0, 0, 0, 0.4);
  box-shadow: -0.1em 0.2em 0 0.1em #fff;
  -moz-transform: scale(1);
}

/* Highlight Defining Column */

.ui.definition.table > tr > td:first-child:not(.ignored),
.ui.definition.table > tbody > tr > td:first-child:not(.ignored),
.ui.definition.table > tfoot > tr > td:first-child:not(.ignored),
.ui.definition.table tr td.definition {
  background: rgba(0, 0, 0, 0.03);
  font-weight: bold;
  color: rgba(0, 0, 0, 0.95);
  text-transform: "";
  box-shadow: "";
  text-align: "";
  font-size: 1em;
  padding-left: "";
  padding-right: "";
}

/* Fix 2nd Column */

.ui.definition.table > thead:not(.full-width) > tr > th:nth-child(2) {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.definition.table > tfoot:not(.full-width) > tr > th:nth-child(2),
.ui.definition.table > tfoot:not(.full-width) > tr > td:nth-child(2) {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.definition.table > tr > td:nth-child(2),
.ui.definition.table > tbody > tr > td:nth-child(2) {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

/*******************************
             States
*******************************/

/* --------------
        Positive
    --------------- */

.ui.ui.ui.ui.table tr.positive,
.ui.ui.table td.positive {
  box-shadow: 0 0 0 #a3c293 inset;
  background: #fcfff5;
  color: #2c662d;
}

.ui.ui.ui.ui.inverted.table tr.positive,
.ui.ui.inverted.table td.positive {
  background: #a3c293;
  color: #2c662d;
}

/* --------------
         Negative
    --------------- */

.ui.ui.ui.ui.table tr.negative,
.ui.ui.table td.negative {
  box-shadow: 0 0 0 #e0b4b4 inset;
  background: #fff6f6;
  color: #9f3a38;
}

.ui.ui.ui.ui.inverted.table tr.negative,
.ui.ui.inverted.table td.negative {
  background: #e0b4b4;
  color: #9f3a38;
}

/* --------------
          Error
    --------------- */

.ui.ui.ui.ui.table tr.error,
.ui.ui.table td.error {
  box-shadow: 0 0 0 #e0b4b4 inset;
  background: #fff6f6;
  color: #9f3a38;
}

.ui.ui.ui.ui.inverted.table tr.error,
.ui.ui.inverted.table td.error {
  background: #e0b4b4;
  color: #9f3a38;
}

/* --------------
         Warning
    --------------- */

.ui.ui.ui.ui.table tr.warning,
.ui.ui.table td.warning {
  box-shadow: 0 0 0 #c9ba9b inset;
  background: #fffaf3;
  color: #573a08;
}

.ui.ui.ui.ui.inverted.table tr.warning,
.ui.ui.inverted.table td.warning {
  background: #eec97e;
  color: #573a08;
}

/* --------------
         Active
    --------------- */

.ui.ui.ui.ui.table tr.active,
.ui.ui.table td.active {
  box-shadow: 0 0 0 rgba(0, 0, 0, 0.87) inset;
  background: #e0e0e0;
  color: rgba(0, 0, 0, 0.87);
}

.ui.ui.ui.ui.inverted.table tr.active,
.ui.ui.inverted.table td.active {
  background: #a0a0a0;
  color: rgba(0, 0, 0, 0.87);
}

/* --------------
         Disabled
    --------------- */

.ui.ui.ui.table tr.disabled td,
.ui.ui.ui.table tr td.disabled,
.ui.table tr.disabled:hover,
.ui.table tr:hover td.disabled {
  pointer-events: none;
  color: rgba(40, 40, 40, 0.3);
}

/*******************************
          Variations
*******************************/

/* --------------
        Stackable
    --------------- */

@media only screen and (max-width: 991.98px) {
  .ui[class*="tablet stackable"].table,
  .ui[class*="tablet stackable"].table > thead,
  .ui[class*="tablet stackable"].table > thead > tr,
  .ui[class*="tablet stackable"].table > tfoot,
  .ui[class*="tablet stackable"].table > tfoot > tr,
  .ui[class*="tablet stackable"].table > tbody,
  .ui[class*="tablet stackable"].table > tbody > tr,
  .ui[class*="tablet stackable"].table > tr,
  .ui[class*="tablet stackable"].table > thead > tr > th:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tbody > tr > th:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tfoot > tr > th:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tr > th:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tbody > tr > td:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tfoot > tr > td:not(.rowspanned),
  .ui[class*="tablet stackable"].table > tr > td:not(.rowspanned) {
    display: block !important;
    width: 100% !important;
  }

  .ui[class*="tablet stackable"].table {
    padding: 0;
  }

  .ui[class*="tablet stackable"].table > thead {
    display: block;
  }

  .ui[class*="tablet stackable"].table > tfoot {
    display: block;
  }

  .ui.ui.ui.ui[class*="tablet stackable"].table > thead > tr,
  .ui.ui.ui.ui[class*="tablet stackable"].table > tbody > tr,
  .ui.ui.ui.ui[class*="tablet stackable"].table > tfoot > tr,
  .ui.ui.ui.ui[class*="tablet stackable"].table > tr {
    padding-top: 1em;
    padding-bottom: 1em;
    box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset;
  }

  .ui[class*="tablet stackable"].table > thead > tr > th,
  .ui[class*="tablet stackable"].table > tbody > tr > th,
  .ui[class*="tablet stackable"].table > tfoot > tr > th,
  .ui[class*="tablet stackable"].table > tr > th,
  .ui[class*="tablet stackable"].table > tbody > tr > td,
  .ui[class*="tablet stackable"].table > tfoot > tr > td,
  .ui[class*="tablet stackable"].table > tr > td {
    background: none;
    border: none !important;
    padding: 0.25em 0.75em;
    box-shadow: none;
  }

  /* Definition Table */

  .ui.definition[class*="tablet stackable"].table > thead > tr > th:first-child {
    box-shadow: none !important;
  }
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="primary marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2185d0 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="primary marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2185d0 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="primary marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #54c8ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="primary marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #54c8ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="secondary marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #1b1c1d inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="secondary marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #1b1c1d inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="secondary marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #545454 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="secondary marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #545454 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="red marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #db2828 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="red marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #db2828 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="red marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff695e inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="red marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff695e inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="orange marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #f2711c inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="orange marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #f2711c inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="orange marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff851b inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="orange marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff851b inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="yellow marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #fbbd08 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="yellow marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #fbbd08 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="yellow marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ffe21f inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="yellow marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ffe21f inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="olive marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #b5cc18 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="olive marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #b5cc18 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="olive marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #d9e778 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="olive marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #d9e778 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="green marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #21ba45 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="green marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #21ba45 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="green marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2ecc40 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="green marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2ecc40 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="teal marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #00b5ad inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="teal marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #00b5ad inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="teal marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #6dffff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="teal marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #6dffff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="blue marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #2185d0 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="blue marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #2185d0 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="blue marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #54c8ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="blue marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #54c8ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="violet marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #6435c9 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="violet marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #6435c9 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="violet marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a291fb inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="violet marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a291fb inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="purple marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a333c8 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="purple marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a333c8 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="purple marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #dc73ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="purple marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #dc73ff inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="pink marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #e03997 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="pink marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #e03997 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="pink marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #ff8edf inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="pink marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #ff8edf inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="brown marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #a5673f inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="brown marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #a5673f inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="brown marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #d67c1c inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="brown marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #d67c1c inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="grey marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #767676 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="grey marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #767676 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="grey marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #dcddde inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="grey marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #dcddde inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="black marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #1b1c1d inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].table tr[class*="black marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #1b1c1d inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="black marked"].left {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, 0.2em 0 0 0 #545454 inset;
}

.ui.ui.ui.ui[class*="tablet stackable"].inverted.table tr[class*="black marked"].right {
  box-shadow: 0 -1px 0 0 rgba(0, 0, 0, 0.1) inset, -0.2em 0 0 0 #545454 inset;
}

/* --------------
     Text Alignment
    --------------- */

.ui.table[class*="left aligned"],
.ui.table [class*="left aligned"] {
  text-align: left;
}

.ui.table[class*="center aligned"],
.ui.table [class*="center aligned"] {
  text-align: center;
}

.ui.table[class*="right aligned"],
.ui.table [class*="right aligned"] {
  text-align: right;
}

/* ------------------
     Vertical Alignment
    ------------------ */

.ui.table[class*="top aligned"],
.ui.table [class*="top aligned"] {
  vertical-align: top;
}

.ui.table[class*="middle aligned"],
.ui.table [class*="middle aligned"] {
  vertical-align: middle;
}

.ui.table[class*="bottom aligned"],
.ui.table [class*="bottom aligned"] {
  vertical-align: bottom;
}

/* --------------
        Collapsing
    --------------- */

.ui.table th.collapsing,
.ui.table td.collapsing {
  width: 1px;
  white-space: nowrap;
}

/* --------------
         Fixed
    --------------- */

.ui.fixed.table {
  table-layout: fixed;
}

.ui.fixed.table th,
.ui.fixed.table td {
  overflow: hidden;
  text-overflow: ellipsis;
}

/* --------------
       Selectable
    --------------- */

.ui.ui.selectable.table > tbody > tr:hover,
.ui.table tbody tr td.selectable:hover {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
}

.ui.ui.selectable.inverted.table > tbody > tr:hover,
.ui.inverted.table tbody tr td.selectable:hover {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
}

/* Selectable Cell Link */

.ui.table tbody tr td.selectable {
  padding: 0;
}

.ui.table tbody tr td.selectable > a:not(.ui) {
  display: block;
  color: inherit;
}

.ui.table:not(.compact) tbody tr td.selectable > a:not(.ui) {
  padding: 0.78571429em 0.78571429em;
}

.ui.table > tr > td.selectable,
.ui.table > tbody > tr > td.selectable,
.ui.selectable.table > tbody > tr,
.ui.selectable.table > tr {
  cursor: pointer;
}

/* Other States */

.ui.ui.selectable.table tr.error:hover,
.ui.table tr td.selectable.error:hover,
.ui.selectable.table tr:hover td.error {
  background: #ffe7e7;
  color: #943634;
}

.ui.ui.selectable.table tr.warning:hover,
.ui.table tr td.selectable.warning:hover,
.ui.selectable.table tr:hover td.warning {
  background: #fff4e4;
  color: #493107;
}

.ui.ui.selectable.table tr.active:hover,
.ui.table tr td.selectable.active:hover,
.ui.selectable.table tr:hover td.active {
  background: #e0e0e0;
  color: rgba(0, 0, 0, 0.87);
}

.ui.ui.selectable.table tr.positive:hover,
.ui.table tr td.selectable.positive:hover,
.ui.selectable.table tr:hover td.positive {
  background: #f7ffe6;
  color: #275b28;
}

.ui.ui.selectable.table tr.negative:hover,
.ui.table tr td.selectable.negative:hover,
.ui.selectable.table tr:hover td.negative {
  background: #ffe7e7;
  color: #943634;
}

/* -------------------
          Attached
    -------------------- */

/* Middle */

.ui.attached.table {
  top: 0;
  bottom: 0;
  border-radius: 0;
  margin: 0 -1px;
  width: calc(100% + 2px);
  max-width: calc(100% + 2px);
  box-shadow: none;
  border: 1px solid #d4d4d5;
}

.ui.attached + .ui.attached.table:not(.top) {
  border-top: none;
}

/* Top */

.ui[class*="top attached"].table {
  bottom: 0;
  margin-bottom: 0;
  top: 0;
  margin-top: 1em;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.table[class*="top attached"]:first-child {
  margin-top: 0;
}

/* Bottom */

.ui[class*="bottom attached"].table {
  bottom: 0;
  margin-top: 0;
  top: 0;
  margin-bottom: 1em;
  box-shadow: none;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui[class*="bottom attached"].table:last-child {
  margin-bottom: 0;
}

/* --------------
         Striped
    --------------- */

/* Table Striping */

.ui.striped.table > tr:nth-child(2n),
.ui.striped.table > tbody > tr:nth-child(2n) {
  background-color: rgba(0, 0, 50, 0.02);
}

/* Stripes */

.ui.inverted.striped.table > tr:nth-child(2n),
.ui.inverted.striped.table > tbody > tr:nth-child(2n) {
  background-color: rgba(255, 255, 255, 0.05);
}

/* Allow striped active hover */

.ui.striped.selectable.selectable.selectable.table tbody tr.active:hover {
  background: #efefef;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
   Single Line
--------------- */

.ui.table[class*="single line"],
.ui.table [class*="single line"] {
  white-space: nowrap;
}

/* -------------------
       Colors
-------------------- */

.ui.primary.table {
  border-top: 0.2em solid #2185d0;
}

.ui.inverted.primary.table {
  background: #2185d0;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.primary,
.ui.scrolling.table > tfoot.primary {
  background: #ddf4ff;
  color: #ddf4ff;
}

.ui.scrolling.table > thead.primary > tr > th,
.ui.scrolling.table > tfoot.primary > tr > th,
.ui.scrolling.table > thead.primary > tr > td,
.ui.scrolling.table > tfoot.primary > tr > td {
  background: inherit;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.scrolling.table > thead.primary,
.ui.inverted.scrolling.table > tfoot.primary {
  background: #2185d0;
  color: #2185d0;
}

.ui.inverted.scrolling.table > thead.primary > tr > th,
.ui.inverted.scrolling.table > tfoot.primary > tr > th,
.ui.inverted.scrolling.table > thead.primary > tr > td,
.ui.inverted.scrolling.table > tfoot.primary > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="primary colored"],
.ui.ui.table th[class*="primary colored"],
.ui.ui.table td[class*="primary colored"],
.ui.ui.ui.ui.table tr.primary:not(.marked),
.ui.ui.table th.primary:not(.marked),
.ui.ui.table td.primary:not(.marked) {
  background: #ddf4ff;
  color: rgba(255, 255, 255, 0.9);
}

.ui.table > thead > tr[class*="primary colored"] > th,
.ui.table > tfoot > tr[class*="primary colored"] > th,
.ui.table > tfoot > tr[class*="primary colored"] > td,
.ui.table > thead > tr.primary:not(.marked) > th,
.ui.table > tfoot > tr.primary:not(.marked) > th,
.ui.table > tfoot > tr.primary:not(.marked) > td {
  background: inherit;
  color: rgba(255, 255, 255, 0.9);
}

.ui.ui.ui.ui.inverted.table tr[class*="primary colored"],
.ui.ui.inverted.table th[class*="primary colored"],
.ui.ui.inverted.table td[class*="primary colored"],
.ui.ui.ui.ui.inverted.table tr.primary:not(.marked),
.ui.ui.inverted.table th.primary:not(.marked),
.ui.ui.inverted.table td.primary:not(.marked) {
  background: #2185d0;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="primary colored"] > th,
.ui.inverted.table > tfoot > tr[class*="primary colored"] > th,
.ui.inverted.table > tfoot > tr[class*="primary colored"] > td,
.ui.inverted.table > thead > tr.primary:not(.marked) > th,
.ui.inverted.table > tfoot > tr.primary:not(.marked) > th,
.ui.inverted.table > tfoot > tr.primary:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="primary colored"]:hover,
.ui.table tr td.selectable[class*="primary colored"]:hover,
.ui.selectable.table tr:hover td[class*="primary colored"],
.ui.ui.selectable.table tr.primary:not(.marked):hover,
.ui.table tr td.selectable.primary:not(.marked):hover,
.ui.selectable.table tr:hover td.primary:not(.marked) {
  background: #d3f1ff;
  color: rgba(255, 255, 255, 0.9);
}

.ui.ui.inverted.selectable.table tr[class*="primary colored"]:hover,
.ui.inverted.table tr td.selectable[class*="primary colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="primary colored"],
.ui.ui.inverted.selectable.table tr.primary:not(.marked):hover,
.ui.inverted.table tr td.selectable.primary:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.primary:not(.marked) {
  background: #21b8ff;
  color: #fff;
}

.ui.table td[class*="primary marked"].left,
.ui.table tr[class*="primary marked"].left {
  box-shadow: 0.2em 0 0 0 #2185d0 inset;
}

.ui.table td[class*="primary marked"].right,
.ui.table tr[class*="primary marked"].right {
  box-shadow: -0.2em 0 0 0 #2185d0 inset;
}

.ui.inverted.table td[class*="primary marked"].left,
.ui.inverted.table tr[class*="primary marked"].left {
  box-shadow: 0.2em 0 0 0 #54c8ff inset;
}

.ui.inverted.table td[class*="primary marked"].right,
.ui.inverted.table tr[class*="primary marked"].right {
  box-shadow: -0.2em 0 0 0 #54c8ff inset;
}

.ui.secondary.table {
  border-top: 0.2em solid #1b1c1d;
}

.ui.inverted.secondary.table {
  background: #1b1c1d;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.secondary,
.ui.scrolling.table > tfoot.secondary {
  background: #dddddd;
  color: #dddddd;
}

.ui.scrolling.table > thead.secondary > tr > th,
.ui.scrolling.table > tfoot.secondary > tr > th,
.ui.scrolling.table > thead.secondary > tr > td,
.ui.scrolling.table > tfoot.secondary > tr > td {
  background: inherit;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.scrolling.table > thead.secondary,
.ui.inverted.scrolling.table > tfoot.secondary {
  background: #1b1c1d;
  color: #1b1c1d;
}

.ui.inverted.scrolling.table > thead.secondary > tr > th,
.ui.inverted.scrolling.table > tfoot.secondary > tr > th,
.ui.inverted.scrolling.table > thead.secondary > tr > td,
.ui.inverted.scrolling.table > tfoot.secondary > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="secondary colored"],
.ui.ui.table th[class*="secondary colored"],
.ui.ui.table td[class*="secondary colored"],
.ui.ui.ui.ui.table tr.secondary:not(.marked),
.ui.ui.table th.secondary:not(.marked),
.ui.ui.table td.secondary:not(.marked) {
  background: #dddddd;
  color: rgba(255, 255, 255, 0.9);
}

.ui.table > thead > tr[class*="secondary colored"] > th,
.ui.table > tfoot > tr[class*="secondary colored"] > th,
.ui.table > tfoot > tr[class*="secondary colored"] > td,
.ui.table > thead > tr.secondary:not(.marked) > th,
.ui.table > tfoot > tr.secondary:not(.marked) > th,
.ui.table > tfoot > tr.secondary:not(.marked) > td {
  background: inherit;
  color: rgba(255, 255, 255, 0.9);
}

.ui.ui.ui.ui.inverted.table tr[class*="secondary colored"],
.ui.ui.inverted.table th[class*="secondary colored"],
.ui.ui.inverted.table td[class*="secondary colored"],
.ui.ui.ui.ui.inverted.table tr.secondary:not(.marked),
.ui.ui.inverted.table th.secondary:not(.marked),
.ui.ui.inverted.table td.secondary:not(.marked) {
  background: #1b1c1d;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="secondary colored"] > th,
.ui.inverted.table > tfoot > tr[class*="secondary colored"] > th,
.ui.inverted.table > tfoot > tr[class*="secondary colored"] > td,
.ui.inverted.table > thead > tr.secondary:not(.marked) > th,
.ui.inverted.table > tfoot > tr.secondary:not(.marked) > th,
.ui.inverted.table > tfoot > tr.secondary:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="secondary colored"]:hover,
.ui.table tr td.selectable[class*="secondary colored"]:hover,
.ui.selectable.table tr:hover td[class*="secondary colored"],
.ui.ui.selectable.table tr.secondary:not(.marked):hover,
.ui.table tr td.selectable.secondary:not(.marked):hover,
.ui.selectable.table tr:hover td.secondary:not(.marked) {
  background: #e2e2e2;
  color: rgba(255, 255, 255, 0.9);
}

.ui.ui.inverted.selectable.table tr[class*="secondary colored"]:hover,
.ui.inverted.table tr td.selectable[class*="secondary colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="secondary colored"],
.ui.ui.inverted.selectable.table tr.secondary:not(.marked):hover,
.ui.inverted.table tr td.selectable.secondary:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.secondary:not(.marked) {
  background: #6e6e6e;
  color: #fff;
}

.ui.table td[class*="secondary marked"].left,
.ui.table tr[class*="secondary marked"].left {
  box-shadow: 0.2em 0 0 0 #1b1c1d inset;
}

.ui.table td[class*="secondary marked"].right,
.ui.table tr[class*="secondary marked"].right {
  box-shadow: -0.2em 0 0 0 #1b1c1d inset;
}

.ui.inverted.table td[class*="secondary marked"].left,
.ui.inverted.table tr[class*="secondary marked"].left {
  box-shadow: 0.2em 0 0 0 #545454 inset;
}

.ui.inverted.table td[class*="secondary marked"].right,
.ui.inverted.table tr[class*="secondary marked"].right {
  box-shadow: -0.2em 0 0 0 #545454 inset;
}

.ui.red.table {
  border-top: 0.2em solid #db2828;
}

.ui.inverted.red.table {
  background: #db2828;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.red,
.ui.scrolling.table > tfoot.red {
  background: #ffe1df;
  color: #ffe1df;
}

.ui.scrolling.table > thead.red > tr > th,
.ui.scrolling.table > tfoot.red > tr > th,
.ui.scrolling.table > thead.red > tr > td,
.ui.scrolling.table > tfoot.red > tr > td {
  background: inherit;
  color: #db2828;
}

.ui.inverted.scrolling.table > thead.red,
.ui.inverted.scrolling.table > tfoot.red {
  background: #db2828;
  color: #db2828;
}

.ui.inverted.scrolling.table > thead.red > tr > th,
.ui.inverted.scrolling.table > tfoot.red > tr > th,
.ui.inverted.scrolling.table > thead.red > tr > td,
.ui.inverted.scrolling.table > tfoot.red > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="red colored"],
.ui.ui.table th[class*="red colored"],
.ui.ui.table td[class*="red colored"],
.ui.ui.ui.ui.table tr.red:not(.marked),
.ui.ui.table th.red:not(.marked),
.ui.ui.table td.red:not(.marked) {
  background: #ffe1df;
  color: #db2828;
}

.ui.table > thead > tr[class*="red colored"] > th,
.ui.table > tfoot > tr[class*="red colored"] > th,
.ui.table > tfoot > tr[class*="red colored"] > td,
.ui.table > thead > tr.red:not(.marked) > th,
.ui.table > tfoot > tr.red:not(.marked) > th,
.ui.table > tfoot > tr.red:not(.marked) > td {
  background: inherit;
  color: #db2828;
}

.ui.ui.ui.ui.inverted.table tr[class*="red colored"],
.ui.ui.inverted.table th[class*="red colored"],
.ui.ui.inverted.table td[class*="red colored"],
.ui.ui.ui.ui.inverted.table tr.red:not(.marked),
.ui.ui.inverted.table th.red:not(.marked),
.ui.ui.inverted.table td.red:not(.marked) {
  background: #db2828;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="red colored"] > th,
.ui.inverted.table > tfoot > tr[class*="red colored"] > th,
.ui.inverted.table > tfoot > tr[class*="red colored"] > td,
.ui.inverted.table > thead > tr.red:not(.marked) > th,
.ui.inverted.table > tfoot > tr.red:not(.marked) > th,
.ui.inverted.table > tfoot > tr.red:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="red colored"]:hover,
.ui.table tr td.selectable[class*="red colored"]:hover,
.ui.selectable.table tr:hover td[class*="red colored"],
.ui.ui.selectable.table tr.red:not(.marked):hover,
.ui.table tr td.selectable.red:not(.marked):hover,
.ui.selectable.table tr:hover td.red:not(.marked) {
  background: #ffd7d5;
  color: #db2828;
}

.ui.ui.inverted.selectable.table tr[class*="red colored"]:hover,
.ui.inverted.table tr td.selectable[class*="red colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="red colored"],
.ui.ui.inverted.selectable.table tr.red:not(.marked):hover,
.ui.inverted.table tr td.selectable.red:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.red:not(.marked) {
  background: #ff392b;
  color: #fff;
}

.ui.table td[class*="red marked"].left,
.ui.table tr[class*="red marked"].left {
  box-shadow: 0.2em 0 0 0 #db2828 inset;
}

.ui.table td[class*="red marked"].right,
.ui.table tr[class*="red marked"].right {
  box-shadow: -0.2em 0 0 0 #db2828 inset;
}

.ui.inverted.table td[class*="red marked"].left,
.ui.inverted.table tr[class*="red marked"].left {
  box-shadow: 0.2em 0 0 0 #ff695e inset;
}

.ui.inverted.table td[class*="red marked"].right,
.ui.inverted.table tr[class*="red marked"].right {
  box-shadow: -0.2em 0 0 0 #ff695e inset;
}

.ui.orange.table {
  border-top: 0.2em solid #f2711c;
}

.ui.inverted.orange.table {
  background: #f2711c;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.orange,
.ui.scrolling.table > tfoot.orange {
  background: #ffe7d1;
  color: #ffe7d1;
}

.ui.scrolling.table > thead.orange > tr > th,
.ui.scrolling.table > tfoot.orange > tr > th,
.ui.scrolling.table > thead.orange > tr > td,
.ui.scrolling.table > tfoot.orange > tr > td {
  background: inherit;
  color: #f2711c;
}

.ui.inverted.scrolling.table > thead.orange,
.ui.inverted.scrolling.table > tfoot.orange {
  background: #f2711c;
  color: #f2711c;
}

.ui.inverted.scrolling.table > thead.orange > tr > th,
.ui.inverted.scrolling.table > tfoot.orange > tr > th,
.ui.inverted.scrolling.table > thead.orange > tr > td,
.ui.inverted.scrolling.table > tfoot.orange > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="orange colored"],
.ui.ui.table th[class*="orange colored"],
.ui.ui.table td[class*="orange colored"],
.ui.ui.ui.ui.table tr.orange:not(.marked),
.ui.ui.table th.orange:not(.marked),
.ui.ui.table td.orange:not(.marked) {
  background: #ffe7d1;
  color: #f2711c;
}

.ui.table > thead > tr[class*="orange colored"] > th,
.ui.table > tfoot > tr[class*="orange colored"] > th,
.ui.table > tfoot > tr[class*="orange colored"] > td,
.ui.table > thead > tr.orange:not(.marked) > th,
.ui.table > tfoot > tr.orange:not(.marked) > th,
.ui.table > tfoot > tr.orange:not(.marked) > td {
  background: inherit;
  color: #f2711c;
}

.ui.ui.ui.ui.inverted.table tr[class*="orange colored"],
.ui.ui.inverted.table th[class*="orange colored"],
.ui.ui.inverted.table td[class*="orange colored"],
.ui.ui.ui.ui.inverted.table tr.orange:not(.marked),
.ui.ui.inverted.table th.orange:not(.marked),
.ui.ui.inverted.table td.orange:not(.marked) {
  background: #f2711c;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="orange colored"] > th,
.ui.inverted.table > tfoot > tr[class*="orange colored"] > th,
.ui.inverted.table > tfoot > tr[class*="orange colored"] > td,
.ui.inverted.table > thead > tr.orange:not(.marked) > th,
.ui.inverted.table > tfoot > tr.orange:not(.marked) > th,
.ui.inverted.table > tfoot > tr.orange:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="orange colored"]:hover,
.ui.table tr td.selectable[class*="orange colored"]:hover,
.ui.selectable.table tr:hover td[class*="orange colored"],
.ui.ui.selectable.table tr.orange:not(.marked):hover,
.ui.table tr td.selectable.orange:not(.marked):hover,
.ui.selectable.table tr:hover td.orange:not(.marked) {
  background: #fae1cc;
  color: #f2711c;
}

.ui.ui.inverted.selectable.table tr[class*="orange colored"]:hover,
.ui.inverted.table tr td.selectable[class*="orange colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="orange colored"],
.ui.ui.inverted.selectable.table tr.orange:not(.marked):hover,
.ui.inverted.table tr td.selectable.orange:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.orange:not(.marked) {
  background: #e76b00;
  color: #fff;
}

.ui.table td[class*="orange marked"].left,
.ui.table tr[class*="orange marked"].left {
  box-shadow: 0.2em 0 0 0 #f2711c inset;
}

.ui.table td[class*="orange marked"].right,
.ui.table tr[class*="orange marked"].right {
  box-shadow: -0.2em 0 0 0 #f2711c inset;
}

.ui.inverted.table td[class*="orange marked"].left,
.ui.inverted.table tr[class*="orange marked"].left {
  box-shadow: 0.2em 0 0 0 #ff851b inset;
}

.ui.inverted.table td[class*="orange marked"].right,
.ui.inverted.table tr[class*="orange marked"].right {
  box-shadow: -0.2em 0 0 0 #ff851b inset;
}

.ui.yellow.table {
  border-top: 0.2em solid #fbbd08;
}

.ui.inverted.yellow.table {
  background: #fbbd08;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.yellow,
.ui.scrolling.table > tfoot.yellow {
  background: #fff9d2;
  color: #fff9d2;
}

.ui.scrolling.table > thead.yellow > tr > th,
.ui.scrolling.table > tfoot.yellow > tr > th,
.ui.scrolling.table > thead.yellow > tr > td,
.ui.scrolling.table > tfoot.yellow > tr > td {
  background: inherit;
  color: #b58105;
}

.ui.inverted.scrolling.table > thead.yellow,
.ui.inverted.scrolling.table > tfoot.yellow {
  background: #fbbd08;
  color: #fbbd08;
}

.ui.inverted.scrolling.table > thead.yellow > tr > th,
.ui.inverted.scrolling.table > tfoot.yellow > tr > th,
.ui.inverted.scrolling.table > thead.yellow > tr > td,
.ui.inverted.scrolling.table > tfoot.yellow > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="yellow colored"],
.ui.ui.table th[class*="yellow colored"],
.ui.ui.table td[class*="yellow colored"],
.ui.ui.ui.ui.table tr.yellow:not(.marked),
.ui.ui.table th.yellow:not(.marked),
.ui.ui.table td.yellow:not(.marked) {
  background: #fff9d2;
  color: #b58105;
}

.ui.table > thead > tr[class*="yellow colored"] > th,
.ui.table > tfoot > tr[class*="yellow colored"] > th,
.ui.table > tfoot > tr[class*="yellow colored"] > td,
.ui.table > thead > tr.yellow:not(.marked) > th,
.ui.table > tfoot > tr.yellow:not(.marked) > th,
.ui.table > tfoot > tr.yellow:not(.marked) > td {
  background: inherit;
  color: #b58105;
}

.ui.ui.ui.ui.inverted.table tr[class*="yellow colored"],
.ui.ui.inverted.table th[class*="yellow colored"],
.ui.ui.inverted.table td[class*="yellow colored"],
.ui.ui.ui.ui.inverted.table tr.yellow:not(.marked),
.ui.ui.inverted.table th.yellow:not(.marked),
.ui.ui.inverted.table td.yellow:not(.marked) {
  background: #fbbd08;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="yellow colored"] > th,
.ui.inverted.table > tfoot > tr[class*="yellow colored"] > th,
.ui.inverted.table > tfoot > tr[class*="yellow colored"] > td,
.ui.inverted.table > thead > tr.yellow:not(.marked) > th,
.ui.inverted.table > tfoot > tr.yellow:not(.marked) > th,
.ui.inverted.table > tfoot > tr.yellow:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="yellow colored"]:hover,
.ui.table tr td.selectable[class*="yellow colored"]:hover,
.ui.selectable.table tr:hover td[class*="yellow colored"],
.ui.ui.selectable.table tr.yellow:not(.marked):hover,
.ui.table tr td.selectable.yellow:not(.marked):hover,
.ui.selectable.table tr:hover td.yellow:not(.marked) {
  background: #fbf5cc;
  color: #b58105;
}

.ui.ui.inverted.selectable.table tr[class*="yellow colored"]:hover,
.ui.inverted.table tr td.selectable[class*="yellow colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="yellow colored"],
.ui.ui.inverted.selectable.table tr.yellow:not(.marked):hover,
.ui.inverted.table tr td.selectable.yellow:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.yellow:not(.marked) {
  background: #ebcd00;
  color: #fff;
}

.ui.table td[class*="yellow marked"].left,
.ui.table tr[class*="yellow marked"].left {
  box-shadow: 0.2em 0 0 0 #fbbd08 inset;
}

.ui.table td[class*="yellow marked"].right,
.ui.table tr[class*="yellow marked"].right {
  box-shadow: -0.2em 0 0 0 #fbbd08 inset;
}

.ui.inverted.table td[class*="yellow marked"].left,
.ui.inverted.table tr[class*="yellow marked"].left {
  box-shadow: 0.2em 0 0 0 #ffe21f inset;
}

.ui.inverted.table td[class*="yellow marked"].right,
.ui.inverted.table tr[class*="yellow marked"].right {
  box-shadow: -0.2em 0 0 0 #ffe21f inset;
}

.ui.olive.table {
  border-top: 0.2em solid #b5cc18;
}

.ui.inverted.olive.table {
  background: #b5cc18;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.olive,
.ui.scrolling.table > tfoot.olive {
  background: #f7fae4;
  color: #f7fae4;
}

.ui.scrolling.table > thead.olive > tr > th,
.ui.scrolling.table > tfoot.olive > tr > th,
.ui.scrolling.table > thead.olive > tr > td,
.ui.scrolling.table > tfoot.olive > tr > td {
  background: inherit;
  color: #8abc1e;
}

.ui.inverted.scrolling.table > thead.olive,
.ui.inverted.scrolling.table > tfoot.olive {
  background: #b5cc18;
  color: #b5cc18;
}

.ui.inverted.scrolling.table > thead.olive > tr > th,
.ui.inverted.scrolling.table > tfoot.olive > tr > th,
.ui.inverted.scrolling.table > thead.olive > tr > td,
.ui.inverted.scrolling.table > tfoot.olive > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="olive colored"],
.ui.ui.table th[class*="olive colored"],
.ui.ui.table td[class*="olive colored"],
.ui.ui.ui.ui.table tr.olive:not(.marked),
.ui.ui.table th.olive:not(.marked),
.ui.ui.table td.olive:not(.marked) {
  background: #f7fae4;
  color: #8abc1e;
}

.ui.table > thead > tr[class*="olive colored"] > th,
.ui.table > tfoot > tr[class*="olive colored"] > th,
.ui.table > tfoot > tr[class*="olive colored"] > td,
.ui.table > thead > tr.olive:not(.marked) > th,
.ui.table > tfoot > tr.olive:not(.marked) > th,
.ui.table > tfoot > tr.olive:not(.marked) > td {
  background: inherit;
  color: #8abc1e;
}

.ui.ui.ui.ui.inverted.table tr[class*="olive colored"],
.ui.ui.inverted.table th[class*="olive colored"],
.ui.ui.inverted.table td[class*="olive colored"],
.ui.ui.ui.ui.inverted.table tr.olive:not(.marked),
.ui.ui.inverted.table th.olive:not(.marked),
.ui.ui.inverted.table td.olive:not(.marked) {
  background: #b5cc18;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="olive colored"] > th,
.ui.inverted.table > tfoot > tr[class*="olive colored"] > th,
.ui.inverted.table > tfoot > tr[class*="olive colored"] > td,
.ui.inverted.table > thead > tr.olive:not(.marked) > th,
.ui.inverted.table > tfoot > tr.olive:not(.marked) > th,
.ui.inverted.table > tfoot > tr.olive:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="olive colored"]:hover,
.ui.table tr td.selectable[class*="olive colored"]:hover,
.ui.selectable.table tr:hover td[class*="olive colored"],
.ui.ui.selectable.table tr.olive:not(.marked):hover,
.ui.table tr td.selectable.olive:not(.marked):hover,
.ui.selectable.table tr:hover td.olive:not(.marked) {
  background: #f6fada;
  color: #8abc1e;
}

.ui.ui.inverted.selectable.table tr[class*="olive colored"]:hover,
.ui.inverted.table tr td.selectable[class*="olive colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="olive colored"],
.ui.ui.inverted.selectable.table tr.olive:not(.marked):hover,
.ui.inverted.table tr td.selectable.olive:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.olive:not(.marked) {
  background: #d2e745;
  color: #fff;
}

.ui.table td[class*="olive marked"].left,
.ui.table tr[class*="olive marked"].left {
  box-shadow: 0.2em 0 0 0 #b5cc18 inset;
}

.ui.table td[class*="olive marked"].right,
.ui.table tr[class*="olive marked"].right {
  box-shadow: -0.2em 0 0 0 #b5cc18 inset;
}

.ui.inverted.table td[class*="olive marked"].left,
.ui.inverted.table tr[class*="olive marked"].left {
  box-shadow: 0.2em 0 0 0 #d9e778 inset;
}

.ui.inverted.table td[class*="olive marked"].right,
.ui.inverted.table tr[class*="olive marked"].right {
  box-shadow: -0.2em 0 0 0 #d9e778 inset;
}

.ui.green.table {
  border-top: 0.2em solid #21ba45;
}

.ui.inverted.green.table {
  background: #21ba45;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.green,
.ui.scrolling.table > tfoot.green {
  background: #d5f5d9;
  color: #d5f5d9;
}

.ui.scrolling.table > thead.green > tr > th,
.ui.scrolling.table > tfoot.green > tr > th,
.ui.scrolling.table > thead.green > tr > td,
.ui.scrolling.table > tfoot.green > tr > td {
  background: inherit;
  color: #1ebc30;
}

.ui.inverted.scrolling.table > thead.green,
.ui.inverted.scrolling.table > tfoot.green {
  background: #21ba45;
  color: #21ba45;
}

.ui.inverted.scrolling.table > thead.green > tr > th,
.ui.inverted.scrolling.table > tfoot.green > tr > th,
.ui.inverted.scrolling.table > thead.green > tr > td,
.ui.inverted.scrolling.table > tfoot.green > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="green colored"],
.ui.ui.table th[class*="green colored"],
.ui.ui.table td[class*="green colored"],
.ui.ui.ui.ui.table tr.green:not(.marked),
.ui.ui.table th.green:not(.marked),
.ui.ui.table td.green:not(.marked) {
  background: #d5f5d9;
  color: #1ebc30;
}

.ui.table > thead > tr[class*="green colored"] > th,
.ui.table > tfoot > tr[class*="green colored"] > th,
.ui.table > tfoot > tr[class*="green colored"] > td,
.ui.table > thead > tr.green:not(.marked) > th,
.ui.table > tfoot > tr.green:not(.marked) > th,
.ui.table > tfoot > tr.green:not(.marked) > td {
  background: inherit;
  color: #1ebc30;
}

.ui.ui.ui.ui.inverted.table tr[class*="green colored"],
.ui.ui.inverted.table th[class*="green colored"],
.ui.ui.inverted.table td[class*="green colored"],
.ui.ui.ui.ui.inverted.table tr.green:not(.marked),
.ui.ui.inverted.table th.green:not(.marked),
.ui.ui.inverted.table td.green:not(.marked) {
  background: #21ba45;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="green colored"] > th,
.ui.inverted.table > tfoot > tr[class*="green colored"] > th,
.ui.inverted.table > tfoot > tr[class*="green colored"] > td,
.ui.inverted.table > thead > tr.green:not(.marked) > th,
.ui.inverted.table > tfoot > tr.green:not(.marked) > th,
.ui.inverted.table > tfoot > tr.green:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="green colored"]:hover,
.ui.table tr td.selectable[class*="green colored"]:hover,
.ui.selectable.table tr:hover td[class*="green colored"],
.ui.ui.selectable.table tr.green:not(.marked):hover,
.ui.table tr td.selectable.green:not(.marked):hover,
.ui.selectable.table tr:hover td.green:not(.marked) {
  background: #d2eed5;
  color: #1ebc30;
}

.ui.ui.inverted.selectable.table tr[class*="green colored"]:hover,
.ui.inverted.table tr td.selectable[class*="green colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="green colored"],
.ui.ui.inverted.selectable.table tr.green:not(.marked):hover,
.ui.inverted.table tr td.selectable.green:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.green:not(.marked) {
  background: #1ea92e;
  color: #fff;
}

.ui.table td[class*="green marked"].left,
.ui.table tr[class*="green marked"].left {
  box-shadow: 0.2em 0 0 0 #21ba45 inset;
}

.ui.table td[class*="green marked"].right,
.ui.table tr[class*="green marked"].right {
  box-shadow: -0.2em 0 0 0 #21ba45 inset;
}

.ui.inverted.table td[class*="green marked"].left,
.ui.inverted.table tr[class*="green marked"].left {
  box-shadow: 0.2em 0 0 0 #2ecc40 inset;
}

.ui.inverted.table td[class*="green marked"].right,
.ui.inverted.table tr[class*="green marked"].right {
  box-shadow: -0.2em 0 0 0 #2ecc40 inset;
}

.ui.teal.table {
  border-top: 0.2em solid #00b5ad;
}

.ui.inverted.teal.table {
  background: #00b5ad;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.teal,
.ui.scrolling.table > tfoot.teal {
  background: #e2ffff;
  color: #e2ffff;
}

.ui.scrolling.table > thead.teal > tr > th,
.ui.scrolling.table > tfoot.teal > tr > th,
.ui.scrolling.table > thead.teal > tr > td,
.ui.scrolling.table > tfoot.teal > tr > td {
  background: inherit;
  color: #10a3a3;
}

.ui.inverted.scrolling.table > thead.teal,
.ui.inverted.scrolling.table > tfoot.teal {
  background: #00b5ad;
  color: #00b5ad;
}

.ui.inverted.scrolling.table > thead.teal > tr > th,
.ui.inverted.scrolling.table > tfoot.teal > tr > th,
.ui.inverted.scrolling.table > thead.teal > tr > td,
.ui.inverted.scrolling.table > tfoot.teal > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="teal colored"],
.ui.ui.table th[class*="teal colored"],
.ui.ui.table td[class*="teal colored"],
.ui.ui.ui.ui.table tr.teal:not(.marked),
.ui.ui.table th.teal:not(.marked),
.ui.ui.table td.teal:not(.marked) {
  background: #e2ffff;
  color: #10a3a3;
}

.ui.table > thead > tr[class*="teal colored"] > th,
.ui.table > tfoot > tr[class*="teal colored"] > th,
.ui.table > tfoot > tr[class*="teal colored"] > td,
.ui.table > thead > tr.teal:not(.marked) > th,
.ui.table > tfoot > tr.teal:not(.marked) > th,
.ui.table > tfoot > tr.teal:not(.marked) > td {
  background: inherit;
  color: #10a3a3;
}

.ui.ui.ui.ui.inverted.table tr[class*="teal colored"],
.ui.ui.inverted.table th[class*="teal colored"],
.ui.ui.inverted.table td[class*="teal colored"],
.ui.ui.ui.ui.inverted.table tr.teal:not(.marked),
.ui.ui.inverted.table th.teal:not(.marked),
.ui.ui.inverted.table td.teal:not(.marked) {
  background: #00b5ad;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="teal colored"] > th,
.ui.inverted.table > tfoot > tr[class*="teal colored"] > th,
.ui.inverted.table > tfoot > tr[class*="teal colored"] > td,
.ui.inverted.table > thead > tr.teal:not(.marked) > th,
.ui.inverted.table > tfoot > tr.teal:not(.marked) > th,
.ui.inverted.table > tfoot > tr.teal:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="teal colored"]:hover,
.ui.table tr td.selectable[class*="teal colored"]:hover,
.ui.selectable.table tr:hover td[class*="teal colored"],
.ui.ui.selectable.table tr.teal:not(.marked):hover,
.ui.table tr td.selectable.teal:not(.marked):hover,
.ui.selectable.table tr:hover td.teal:not(.marked) {
  background: #d8ffff;
  color: #10a3a3;
}

.ui.ui.inverted.selectable.table tr[class*="teal colored"]:hover,
.ui.inverted.table tr td.selectable[class*="teal colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="teal colored"],
.ui.ui.inverted.selectable.table tr.teal:not(.marked):hover,
.ui.inverted.table tr td.selectable.teal:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.teal:not(.marked) {
  background: #3affff;
  color: #fff;
}

.ui.table td[class*="teal marked"].left,
.ui.table tr[class*="teal marked"].left {
  box-shadow: 0.2em 0 0 0 #00b5ad inset;
}

.ui.table td[class*="teal marked"].right,
.ui.table tr[class*="teal marked"].right {
  box-shadow: -0.2em 0 0 0 #00b5ad inset;
}

.ui.inverted.table td[class*="teal marked"].left,
.ui.inverted.table tr[class*="teal marked"].left {
  box-shadow: 0.2em 0 0 0 #6dffff inset;
}

.ui.inverted.table td[class*="teal marked"].right,
.ui.inverted.table tr[class*="teal marked"].right {
  box-shadow: -0.2em 0 0 0 #6dffff inset;
}

.ui.blue.table {
  border-top: 0.2em solid #2185d0;
}

.ui.inverted.blue.table {
  background: #2185d0;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.blue,
.ui.scrolling.table > tfoot.blue {
  background: #ddf4ff;
  color: #ddf4ff;
}

.ui.scrolling.table > thead.blue > tr > th,
.ui.scrolling.table > tfoot.blue > tr > th,
.ui.scrolling.table > thead.blue > tr > td,
.ui.scrolling.table > tfoot.blue > tr > td {
  background: inherit;
  color: #2185d0;
}

.ui.inverted.scrolling.table > thead.blue,
.ui.inverted.scrolling.table > tfoot.blue {
  background: #2185d0;
  color: #2185d0;
}

.ui.inverted.scrolling.table > thead.blue > tr > th,
.ui.inverted.scrolling.table > tfoot.blue > tr > th,
.ui.inverted.scrolling.table > thead.blue > tr > td,
.ui.inverted.scrolling.table > tfoot.blue > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="blue colored"],
.ui.ui.table th[class*="blue colored"],
.ui.ui.table td[class*="blue colored"],
.ui.ui.ui.ui.table tr.blue:not(.marked),
.ui.ui.table th.blue:not(.marked),
.ui.ui.table td.blue:not(.marked) {
  background: #ddf4ff;
  color: #2185d0;
}

.ui.table > thead > tr[class*="blue colored"] > th,
.ui.table > tfoot > tr[class*="blue colored"] > th,
.ui.table > tfoot > tr[class*="blue colored"] > td,
.ui.table > thead > tr.blue:not(.marked) > th,
.ui.table > tfoot > tr.blue:not(.marked) > th,
.ui.table > tfoot > tr.blue:not(.marked) > td {
  background: inherit;
  color: #2185d0;
}

.ui.ui.ui.ui.inverted.table tr[class*="blue colored"],
.ui.ui.inverted.table th[class*="blue colored"],
.ui.ui.inverted.table td[class*="blue colored"],
.ui.ui.ui.ui.inverted.table tr.blue:not(.marked),
.ui.ui.inverted.table th.blue:not(.marked),
.ui.ui.inverted.table td.blue:not(.marked) {
  background: #2185d0;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="blue colored"] > th,
.ui.inverted.table > tfoot > tr[class*="blue colored"] > th,
.ui.inverted.table > tfoot > tr[class*="blue colored"] > td,
.ui.inverted.table > thead > tr.blue:not(.marked) > th,
.ui.inverted.table > tfoot > tr.blue:not(.marked) > th,
.ui.inverted.table > tfoot > tr.blue:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="blue colored"]:hover,
.ui.table tr td.selectable[class*="blue colored"]:hover,
.ui.selectable.table tr:hover td[class*="blue colored"],
.ui.ui.selectable.table tr.blue:not(.marked):hover,
.ui.table tr td.selectable.blue:not(.marked):hover,
.ui.selectable.table tr:hover td.blue:not(.marked) {
  background: #d3f1ff;
  color: #2185d0;
}

.ui.ui.inverted.selectable.table tr[class*="blue colored"]:hover,
.ui.inverted.table tr td.selectable[class*="blue colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="blue colored"],
.ui.ui.inverted.selectable.table tr.blue:not(.marked):hover,
.ui.inverted.table tr td.selectable.blue:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.blue:not(.marked) {
  background: #21b8ff;
  color: #fff;
}

.ui.table td[class*="blue marked"].left,
.ui.table tr[class*="blue marked"].left {
  box-shadow: 0.2em 0 0 0 #2185d0 inset;
}

.ui.table td[class*="blue marked"].right,
.ui.table tr[class*="blue marked"].right {
  box-shadow: -0.2em 0 0 0 #2185d0 inset;
}

.ui.inverted.table td[class*="blue marked"].left,
.ui.inverted.table tr[class*="blue marked"].left {
  box-shadow: 0.2em 0 0 0 #54c8ff inset;
}

.ui.inverted.table td[class*="blue marked"].right,
.ui.inverted.table tr[class*="blue marked"].right {
  box-shadow: -0.2em 0 0 0 #54c8ff inset;
}

.ui.violet.table {
  border-top: 0.2em solid #6435c9;
}

.ui.inverted.violet.table {
  background: #6435c9;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.violet,
.ui.scrolling.table > tfoot.violet {
  background: #ece9fe;
  color: #ece9fe;
}

.ui.scrolling.table > thead.violet > tr > th,
.ui.scrolling.table > tfoot.violet > tr > th,
.ui.scrolling.table > thead.violet > tr > td,
.ui.scrolling.table > tfoot.violet > tr > td {
  background: inherit;
  color: #6435c9;
}

.ui.inverted.scrolling.table > thead.violet,
.ui.inverted.scrolling.table > tfoot.violet {
  background: #6435c9;
  color: #6435c9;
}

.ui.inverted.scrolling.table > thead.violet > tr > th,
.ui.inverted.scrolling.table > tfoot.violet > tr > th,
.ui.inverted.scrolling.table > thead.violet > tr > td,
.ui.inverted.scrolling.table > tfoot.violet > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="violet colored"],
.ui.ui.table th[class*="violet colored"],
.ui.ui.table td[class*="violet colored"],
.ui.ui.ui.ui.table tr.violet:not(.marked),
.ui.ui.table th.violet:not(.marked),
.ui.ui.table td.violet:not(.marked) {
  background: #ece9fe;
  color: #6435c9;
}

.ui.table > thead > tr[class*="violet colored"] > th,
.ui.table > tfoot > tr[class*="violet colored"] > th,
.ui.table > tfoot > tr[class*="violet colored"] > td,
.ui.table > thead > tr.violet:not(.marked) > th,
.ui.table > tfoot > tr.violet:not(.marked) > th,
.ui.table > tfoot > tr.violet:not(.marked) > td {
  background: inherit;
  color: #6435c9;
}

.ui.ui.ui.ui.inverted.table tr[class*="violet colored"],
.ui.ui.inverted.table th[class*="violet colored"],
.ui.ui.inverted.table td[class*="violet colored"],
.ui.ui.ui.ui.inverted.table tr.violet:not(.marked),
.ui.ui.inverted.table th.violet:not(.marked),
.ui.ui.inverted.table td.violet:not(.marked) {
  background: #6435c9;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="violet colored"] > th,
.ui.inverted.table > tfoot > tr[class*="violet colored"] > th,
.ui.inverted.table > tfoot > tr[class*="violet colored"] > td,
.ui.inverted.table > thead > tr.violet:not(.marked) > th,
.ui.inverted.table > tfoot > tr.violet:not(.marked) > th,
.ui.inverted.table > tfoot > tr.violet:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="violet colored"]:hover,
.ui.table tr td.selectable[class*="violet colored"]:hover,
.ui.selectable.table tr:hover td[class*="violet colored"],
.ui.ui.selectable.table tr.violet:not(.marked):hover,
.ui.table tr td.selectable.violet:not(.marked):hover,
.ui.selectable.table tr:hover td.violet:not(.marked) {
  background: #e3deff;
  color: #6435c9;
}

.ui.ui.inverted.selectable.table tr[class*="violet colored"]:hover,
.ui.inverted.table tr td.selectable[class*="violet colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="violet colored"],
.ui.ui.inverted.selectable.table tr.violet:not(.marked):hover,
.ui.inverted.table tr td.selectable.violet:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.violet:not(.marked) {
  background: #745aff;
  color: #fff;
}

.ui.table td[class*="violet marked"].left,
.ui.table tr[class*="violet marked"].left {
  box-shadow: 0.2em 0 0 0 #6435c9 inset;
}

.ui.table td[class*="violet marked"].right,
.ui.table tr[class*="violet marked"].right {
  box-shadow: -0.2em 0 0 0 #6435c9 inset;
}

.ui.inverted.table td[class*="violet marked"].left,
.ui.inverted.table tr[class*="violet marked"].left {
  box-shadow: 0.2em 0 0 0 #a291fb inset;
}

.ui.inverted.table td[class*="violet marked"].right,
.ui.inverted.table tr[class*="violet marked"].right {
  box-shadow: -0.2em 0 0 0 #a291fb inset;
}

.ui.purple.table {
  border-top: 0.2em solid #a333c8;
}

.ui.inverted.purple.table {
  background: #a333c8;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.purple,
.ui.scrolling.table > tfoot.purple {
  background: #f8e3ff;
  color: #f8e3ff;
}

.ui.scrolling.table > thead.purple > tr > th,
.ui.scrolling.table > tfoot.purple > tr > th,
.ui.scrolling.table > thead.purple > tr > td,
.ui.scrolling.table > tfoot.purple > tr > td {
  background: inherit;
  color: #a333c8;
}

.ui.inverted.scrolling.table > thead.purple,
.ui.inverted.scrolling.table > tfoot.purple {
  background: #a333c8;
  color: #a333c8;
}

.ui.inverted.scrolling.table > thead.purple > tr > th,
.ui.inverted.scrolling.table > tfoot.purple > tr > th,
.ui.inverted.scrolling.table > thead.purple > tr > td,
.ui.inverted.scrolling.table > tfoot.purple > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="purple colored"],
.ui.ui.table th[class*="purple colored"],
.ui.ui.table td[class*="purple colored"],
.ui.ui.ui.ui.table tr.purple:not(.marked),
.ui.ui.table th.purple:not(.marked),
.ui.ui.table td.purple:not(.marked) {
  background: #f8e3ff;
  color: #a333c8;
}

.ui.table > thead > tr[class*="purple colored"] > th,
.ui.table > tfoot > tr[class*="purple colored"] > th,
.ui.table > tfoot > tr[class*="purple colored"] > td,
.ui.table > thead > tr.purple:not(.marked) > th,
.ui.table > tfoot > tr.purple:not(.marked) > th,
.ui.table > tfoot > tr.purple:not(.marked) > td {
  background: inherit;
  color: #a333c8;
}

.ui.ui.ui.ui.inverted.table tr[class*="purple colored"],
.ui.ui.inverted.table th[class*="purple colored"],
.ui.ui.inverted.table td[class*="purple colored"],
.ui.ui.ui.ui.inverted.table tr.purple:not(.marked),
.ui.ui.inverted.table th.purple:not(.marked),
.ui.ui.inverted.table td.purple:not(.marked) {
  background: #a333c8;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="purple colored"] > th,
.ui.inverted.table > tfoot > tr[class*="purple colored"] > th,
.ui.inverted.table > tfoot > tr[class*="purple colored"] > td,
.ui.inverted.table > thead > tr.purple:not(.marked) > th,
.ui.inverted.table > tfoot > tr.purple:not(.marked) > th,
.ui.inverted.table > tfoot > tr.purple:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="purple colored"]:hover,
.ui.table tr td.selectable[class*="purple colored"]:hover,
.ui.selectable.table tr:hover td[class*="purple colored"],
.ui.ui.selectable.table tr.purple:not(.marked):hover,
.ui.table tr td.selectable.purple:not(.marked):hover,
.ui.selectable.table tr:hover td.purple:not(.marked) {
  background: #f5d9ff;
  color: #a333c8;
}

.ui.ui.inverted.selectable.table tr[class*="purple colored"]:hover,
.ui.inverted.table tr td.selectable[class*="purple colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="purple colored"],
.ui.ui.inverted.selectable.table tr.purple:not(.marked):hover,
.ui.inverted.table tr td.selectable.purple:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.purple:not(.marked) {
  background: #cf40ff;
  color: #fff;
}

.ui.table td[class*="purple marked"].left,
.ui.table tr[class*="purple marked"].left {
  box-shadow: 0.2em 0 0 0 #a333c8 inset;
}

.ui.table td[class*="purple marked"].right,
.ui.table tr[class*="purple marked"].right {
  box-shadow: -0.2em 0 0 0 #a333c8 inset;
}

.ui.inverted.table td[class*="purple marked"].left,
.ui.inverted.table tr[class*="purple marked"].left {
  box-shadow: 0.2em 0 0 0 #dc73ff inset;
}

.ui.inverted.table td[class*="purple marked"].right,
.ui.inverted.table tr[class*="purple marked"].right {
  box-shadow: -0.2em 0 0 0 #dc73ff inset;
}

.ui.pink.table {
  border-top: 0.2em solid #e03997;
}

.ui.inverted.pink.table {
  background: #e03997;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.pink,
.ui.scrolling.table > tfoot.pink {
  background: #ffe8f9;
  color: #ffe8f9;
}

.ui.scrolling.table > thead.pink > tr > th,
.ui.scrolling.table > tfoot.pink > tr > th,
.ui.scrolling.table > thead.pink > tr > td,
.ui.scrolling.table > tfoot.pink > tr > td {
  background: inherit;
  color: #e03997;
}

.ui.inverted.scrolling.table > thead.pink,
.ui.inverted.scrolling.table > tfoot.pink {
  background: #e03997;
  color: #e03997;
}

.ui.inverted.scrolling.table > thead.pink > tr > th,
.ui.inverted.scrolling.table > tfoot.pink > tr > th,
.ui.inverted.scrolling.table > thead.pink > tr > td,
.ui.inverted.scrolling.table > tfoot.pink > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="pink colored"],
.ui.ui.table th[class*="pink colored"],
.ui.ui.table td[class*="pink colored"],
.ui.ui.ui.ui.table tr.pink:not(.marked),
.ui.ui.table th.pink:not(.marked),
.ui.ui.table td.pink:not(.marked) {
  background: #ffe8f9;
  color: #e03997;
}

.ui.table > thead > tr[class*="pink colored"] > th,
.ui.table > tfoot > tr[class*="pink colored"] > th,
.ui.table > tfoot > tr[class*="pink colored"] > td,
.ui.table > thead > tr.pink:not(.marked) > th,
.ui.table > tfoot > tr.pink:not(.marked) > th,
.ui.table > tfoot > tr.pink:not(.marked) > td {
  background: inherit;
  color: #e03997;
}

.ui.ui.ui.ui.inverted.table tr[class*="pink colored"],
.ui.ui.inverted.table th[class*="pink colored"],
.ui.ui.inverted.table td[class*="pink colored"],
.ui.ui.ui.ui.inverted.table tr.pink:not(.marked),
.ui.ui.inverted.table th.pink:not(.marked),
.ui.ui.inverted.table td.pink:not(.marked) {
  background: #e03997;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="pink colored"] > th,
.ui.inverted.table > tfoot > tr[class*="pink colored"] > th,
.ui.inverted.table > tfoot > tr[class*="pink colored"] > td,
.ui.inverted.table > thead > tr.pink:not(.marked) > th,
.ui.inverted.table > tfoot > tr.pink:not(.marked) > th,
.ui.inverted.table > tfoot > tr.pink:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="pink colored"]:hover,
.ui.table tr td.selectable[class*="pink colored"]:hover,
.ui.selectable.table tr:hover td[class*="pink colored"],
.ui.ui.selectable.table tr.pink:not(.marked):hover,
.ui.table tr td.selectable.pink:not(.marked):hover,
.ui.selectable.table tr:hover td.pink:not(.marked) {
  background: #ffdef6;
  color: #e03997;
}

.ui.ui.inverted.selectable.table tr[class*="pink colored"]:hover,
.ui.inverted.table tr td.selectable[class*="pink colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="pink colored"],
.ui.ui.inverted.selectable.table tr.pink:not(.marked):hover,
.ui.inverted.table tr td.selectable.pink:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.pink:not(.marked) {
  background: #ff5bd1;
  color: #fff;
}

.ui.table td[class*="pink marked"].left,
.ui.table tr[class*="pink marked"].left {
  box-shadow: 0.2em 0 0 0 #e03997 inset;
}

.ui.table td[class*="pink marked"].right,
.ui.table tr[class*="pink marked"].right {
  box-shadow: -0.2em 0 0 0 #e03997 inset;
}

.ui.inverted.table td[class*="pink marked"].left,
.ui.inverted.table tr[class*="pink marked"].left {
  box-shadow: 0.2em 0 0 0 #ff8edf inset;
}

.ui.inverted.table td[class*="pink marked"].right,
.ui.inverted.table tr[class*="pink marked"].right {
  box-shadow: -0.2em 0 0 0 #ff8edf inset;
}

.ui.brown.table {
  border-top: 0.2em solid #a5673f;
}

.ui.inverted.brown.table {
  background: #a5673f;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.brown,
.ui.scrolling.table > tfoot.brown {
  background: #f7e5d2;
  color: #f7e5d2;
}

.ui.scrolling.table > thead.brown > tr > th,
.ui.scrolling.table > tfoot.brown > tr > th,
.ui.scrolling.table > thead.brown > tr > td,
.ui.scrolling.table > tfoot.brown > tr > td {
  background: inherit;
  color: #a5673f;
}

.ui.inverted.scrolling.table > thead.brown,
.ui.inverted.scrolling.table > tfoot.brown {
  background: #a5673f;
  color: #a5673f;
}

.ui.inverted.scrolling.table > thead.brown > tr > th,
.ui.inverted.scrolling.table > tfoot.brown > tr > th,
.ui.inverted.scrolling.table > thead.brown > tr > td,
.ui.inverted.scrolling.table > tfoot.brown > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="brown colored"],
.ui.ui.table th[class*="brown colored"],
.ui.ui.table td[class*="brown colored"],
.ui.ui.ui.ui.table tr.brown:not(.marked),
.ui.ui.table th.brown:not(.marked),
.ui.ui.table td.brown:not(.marked) {
  background: #f7e5d2;
  color: #a5673f;
}

.ui.table > thead > tr[class*="brown colored"] > th,
.ui.table > tfoot > tr[class*="brown colored"] > th,
.ui.table > tfoot > tr[class*="brown colored"] > td,
.ui.table > thead > tr.brown:not(.marked) > th,
.ui.table > tfoot > tr.brown:not(.marked) > th,
.ui.table > tfoot > tr.brown:not(.marked) > td {
  background: inherit;
  color: #a5673f;
}

.ui.ui.ui.ui.inverted.table tr[class*="brown colored"],
.ui.ui.inverted.table th[class*="brown colored"],
.ui.ui.inverted.table td[class*="brown colored"],
.ui.ui.ui.ui.inverted.table tr.brown:not(.marked),
.ui.ui.inverted.table th.brown:not(.marked),
.ui.ui.inverted.table td.brown:not(.marked) {
  background: #a5673f;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="brown colored"] > th,
.ui.inverted.table > tfoot > tr[class*="brown colored"] > th,
.ui.inverted.table > tfoot > tr[class*="brown colored"] > td,
.ui.inverted.table > thead > tr.brown:not(.marked) > th,
.ui.inverted.table > tfoot > tr.brown:not(.marked) > th,
.ui.inverted.table > tfoot > tr.brown:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="brown colored"]:hover,
.ui.table tr td.selectable[class*="brown colored"]:hover,
.ui.selectable.table tr:hover td[class*="brown colored"],
.ui.ui.selectable.table tr.brown:not(.marked):hover,
.ui.table tr td.selectable.brown:not(.marked):hover,
.ui.selectable.table tr:hover td.brown:not(.marked) {
  background: #efe0cf;
  color: #a5673f;
}

.ui.ui.inverted.selectable.table tr[class*="brown colored"]:hover,
.ui.inverted.table tr td.selectable[class*="brown colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="brown colored"],
.ui.ui.inverted.selectable.table tr.brown:not(.marked):hover,
.ui.inverted.table tr td.selectable.brown:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.brown:not(.marked) {
  background: #b0620f;
  color: #fff;
}

.ui.table td[class*="brown marked"].left,
.ui.table tr[class*="brown marked"].left {
  box-shadow: 0.2em 0 0 0 #a5673f inset;
}

.ui.table td[class*="brown marked"].right,
.ui.table tr[class*="brown marked"].right {
  box-shadow: -0.2em 0 0 0 #a5673f inset;
}

.ui.inverted.table td[class*="brown marked"].left,
.ui.inverted.table tr[class*="brown marked"].left {
  box-shadow: 0.2em 0 0 0 #d67c1c inset;
}

.ui.inverted.table td[class*="brown marked"].right,
.ui.inverted.table tr[class*="brown marked"].right {
  box-shadow: -0.2em 0 0 0 #d67c1c inset;
}

.ui.grey.table {
  border-top: 0.2em solid #767676;
}

.ui.inverted.grey.table {
  background: #767676;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.grey,
.ui.scrolling.table > tfoot.grey {
  background: #dcddde;
  color: #dcddde;
}

.ui.scrolling.table > thead.grey > tr > th,
.ui.scrolling.table > tfoot.grey > tr > th,
.ui.scrolling.table > thead.grey > tr > td,
.ui.scrolling.table > tfoot.grey > tr > td {
  background: inherit;
  color: #767676;
}

.ui.inverted.scrolling.table > thead.grey,
.ui.inverted.scrolling.table > tfoot.grey {
  background: #767676;
  color: #767676;
}

.ui.inverted.scrolling.table > thead.grey > tr > th,
.ui.inverted.scrolling.table > tfoot.grey > tr > th,
.ui.inverted.scrolling.table > thead.grey > tr > td,
.ui.inverted.scrolling.table > tfoot.grey > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="grey colored"],
.ui.ui.table th[class*="grey colored"],
.ui.ui.table td[class*="grey colored"],
.ui.ui.ui.ui.table tr.grey:not(.marked),
.ui.ui.table th.grey:not(.marked),
.ui.ui.table td.grey:not(.marked) {
  background: #dcddde;
  color: #767676;
}

.ui.table > thead > tr[class*="grey colored"] > th,
.ui.table > tfoot > tr[class*="grey colored"] > th,
.ui.table > tfoot > tr[class*="grey colored"] > td,
.ui.table > thead > tr.grey:not(.marked) > th,
.ui.table > tfoot > tr.grey:not(.marked) > th,
.ui.table > tfoot > tr.grey:not(.marked) > td {
  background: inherit;
  color: #767676;
}

.ui.ui.ui.ui.inverted.table tr[class*="grey colored"],
.ui.ui.inverted.table th[class*="grey colored"],
.ui.ui.inverted.table td[class*="grey colored"],
.ui.ui.ui.ui.inverted.table tr.grey:not(.marked),
.ui.ui.inverted.table th.grey:not(.marked),
.ui.ui.inverted.table td.grey:not(.marked) {
  background: #767676;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="grey colored"] > th,
.ui.inverted.table > tfoot > tr[class*="grey colored"] > th,
.ui.inverted.table > tfoot > tr[class*="grey colored"] > td,
.ui.inverted.table > thead > tr.grey:not(.marked) > th,
.ui.inverted.table > tfoot > tr.grey:not(.marked) > th,
.ui.inverted.table > tfoot > tr.grey:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="grey colored"]:hover,
.ui.table tr td.selectable[class*="grey colored"]:hover,
.ui.selectable.table tr:hover td[class*="grey colored"],
.ui.ui.selectable.table tr.grey:not(.marked):hover,
.ui.table tr td.selectable.grey:not(.marked):hover,
.ui.selectable.table tr:hover td.grey:not(.marked) {
  background: #c2c4c5;
  color: #767676;
}

.ui.ui.inverted.selectable.table tr[class*="grey colored"]:hover,
.ui.inverted.table tr td.selectable[class*="grey colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="grey colored"],
.ui.ui.inverted.selectable.table tr.grey:not(.marked):hover,
.ui.inverted.table tr td.selectable.grey:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.grey:not(.marked) {
  background: #c2c4c5;
  color: #fff;
}

.ui.table td[class*="grey marked"].left,
.ui.table tr[class*="grey marked"].left {
  box-shadow: 0.2em 0 0 0 #767676 inset;
}

.ui.table td[class*="grey marked"].right,
.ui.table tr[class*="grey marked"].right {
  box-shadow: -0.2em 0 0 0 #767676 inset;
}

.ui.inverted.table td[class*="grey marked"].left,
.ui.inverted.table tr[class*="grey marked"].left {
  box-shadow: 0.2em 0 0 0 #dcddde inset;
}

.ui.inverted.table td[class*="grey marked"].right,
.ui.inverted.table tr[class*="grey marked"].right {
  box-shadow: -0.2em 0 0 0 #dcddde inset;
}

.ui.black.table {
  border-top: 0.2em solid #1b1c1d;
}

.ui.inverted.black.table {
  background: #1b1c1d;
  color: #fff;
}

/* Same color for background and color to camouflage the scrollbar */

.ui.scrolling.table > thead.black,
.ui.scrolling.table > tfoot.black {
  background: #545454;
  color: #545454;
}

.ui.scrolling.table > thead.black > tr > th,
.ui.scrolling.table > tfoot.black > tr > th,
.ui.scrolling.table > thead.black > tr > td,
.ui.scrolling.table > tfoot.black > tr > td {
  background: inherit;
  color: #fff;
}

.ui.inverted.scrolling.table > thead.black,
.ui.inverted.scrolling.table > tfoot.black {
  background: #1b1c1d;
  color: #1b1c1d;
}

.ui.inverted.scrolling.table > thead.black > tr > th,
.ui.inverted.scrolling.table > tfoot.black > tr > th,
.ui.inverted.scrolling.table > thead.black > tr > td,
.ui.inverted.scrolling.table > tfoot.black > tr > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.table tr[class*="black colored"],
.ui.ui.table th[class*="black colored"],
.ui.ui.table td[class*="black colored"],
.ui.ui.ui.ui.table tr.black:not(.marked),
.ui.ui.table th.black:not(.marked),
.ui.ui.table td.black:not(.marked) {
  background: #545454;
  color: #fff;
}

.ui.table > thead > tr[class*="black colored"] > th,
.ui.table > tfoot > tr[class*="black colored"] > th,
.ui.table > tfoot > tr[class*="black colored"] > td,
.ui.table > thead > tr.black:not(.marked) > th,
.ui.table > tfoot > tr.black:not(.marked) > th,
.ui.table > tfoot > tr.black:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.ui.ui.inverted.table tr[class*="black colored"],
.ui.ui.inverted.table th[class*="black colored"],
.ui.ui.inverted.table td[class*="black colored"],
.ui.ui.ui.ui.inverted.table tr.black:not(.marked),
.ui.ui.inverted.table th.black:not(.marked),
.ui.ui.inverted.table td.black:not(.marked) {
  background: #1b1c1d;
  color: #fff;
}

.ui.inverted.table > thead > tr[class*="black colored"] > th,
.ui.inverted.table > tfoot > tr[class*="black colored"] > th,
.ui.inverted.table > tfoot > tr[class*="black colored"] > td,
.ui.inverted.table > thead > tr.black:not(.marked) > th,
.ui.inverted.table > tfoot > tr.black:not(.marked) > th,
.ui.inverted.table > tfoot > tr.black:not(.marked) > td {
  background: inherit;
  color: #fff;
}

.ui.ui.selectable.table tr[class*="black colored"]:hover,
.ui.table tr td.selectable[class*="black colored"]:hover,
.ui.selectable.table tr:hover td[class*="black colored"],
.ui.ui.selectable.table tr.black:not(.marked):hover,
.ui.table tr td.selectable.black:not(.marked):hover,
.ui.selectable.table tr:hover td.black:not(.marked) {
  background: #000000;
  color: #fff;
}

.ui.ui.inverted.selectable.table tr[class*="black colored"]:hover,
.ui.inverted.table tr td.selectable[class*="black colored"]:hover,
.ui.inverted.selectable.table tr:hover td[class*="black colored"],
.ui.ui.inverted.selectable.table tr.black:not(.marked):hover,
.ui.inverted.table tr td.selectable.black:not(.marked):hover,
.ui.inverted.selectable.table tr:hover td.black:not(.marked) {
  background: #000000;
  color: #1b1c1d;
}

.ui.table td[class*="black marked"].left,
.ui.table tr[class*="black marked"].left {
  box-shadow: 0.2em 0 0 0 #1b1c1d inset;
}

.ui.table td[class*="black marked"].right,
.ui.table tr[class*="black marked"].right {
  box-shadow: -0.2em 0 0 0 #1b1c1d inset;
}

.ui.inverted.table td[class*="black marked"].left,
.ui.inverted.table tr[class*="black marked"].left {
  box-shadow: 0.2em 0 0 0 #545454 inset;
}

.ui.inverted.table td[class*="black marked"].right,
.ui.inverted.table tr[class*="black marked"].right {
  box-shadow: -0.2em 0 0 0 #545454 inset;
}

/* --------------
      Column Count
    --------------- */

/* Grid Based */

.ui.one.column.table td {
  width: 100%;
}

.ui.two.column.table td {
  width: 50%;
}

.ui.three.column.table td {
  width: 33.33333333%;
}

.ui.four.column.table td {
  width: 25%;
}

.ui.five.column.table td {
  width: 20%;
}

.ui.six.column.table td {
  width: 16.66666667%;
}

.ui.seven.column.table td {
  width: 14.28571429%;
}

.ui.eight.column.table td {
  width: 12.5%;
}

.ui.nine.column.table td {
  width: 11.11111111%;
}

.ui.ten.column.table td {
  width: 10%;
}

.ui.eleven.column.table td {
  width: 9.09090909%;
}

.ui.twelve.column.table td {
  width: 8.33333333%;
}

.ui.thirteen.column.table td {
  width: 7.69230769%;
}

.ui.fourteen.column.table td {
  width: 7.14285714%;
}

.ui.fifteen.column.table td {
  width: 6.66666667%;
}

.ui.sixteen.column.table td {
  width: 6.25%;
}

/* Column Width */

.ui.table th.one.wide,
.ui.table td.one.wide {
  width: 6.25%;
}

.ui.table th.two.wide,
.ui.table td.two.wide {
  width: 12.5%;
}

.ui.table th.three.wide,
.ui.table td.three.wide {
  width: 18.75%;
}

.ui.table th.four.wide,
.ui.table td.four.wide {
  width: 25%;
}

.ui.table th.five.wide,
.ui.table td.five.wide {
  width: 31.25%;
}

.ui.table th.six.wide,
.ui.table td.six.wide {
  width: 37.5%;
}

.ui.table th.seven.wide,
.ui.table td.seven.wide {
  width: 43.75%;
}

.ui.table th.eight.wide,
.ui.table td.eight.wide {
  width: 50%;
}

.ui.table th.nine.wide,
.ui.table td.nine.wide {
  width: 56.25%;
}

.ui.table th.ten.wide,
.ui.table td.ten.wide {
  width: 62.5%;
}

.ui.table th.eleven.wide,
.ui.table td.eleven.wide {
  width: 68.75%;
}

.ui.table th.twelve.wide,
.ui.table td.twelve.wide {
  width: 75%;
}

.ui.table th.thirteen.wide,
.ui.table td.thirteen.wide {
  width: 81.25%;
}

.ui.table th.fourteen.wide,
.ui.table td.fourteen.wide {
  width: 87.5%;
}

.ui.table th.fifteen.wide,
.ui.table td.fifteen.wide {
  width: 93.75%;
}

.ui.table th.sixteen.wide,
.ui.table td.sixteen.wide {
  width: 100%;
}

/* --------------
        Sortable
    --------------- */

.ui.sortable.table > thead > tr > th {
  cursor: pointer;
  white-space: nowrap;
  color: rgba(0, 0, 0, 0.87);
}

.ui.celled.sortable.table > thead > tr > th:not(:first-child) {
  border-left: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.sortable.table thead th.sorted,
.ui.sortable.table thead th.sorted:hover {
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.ui.sortable.table > thead > tr > th::after {
  display: none;
  font-style: normal;
  font-weight: normal;
  text-decoration: inherit;
  content: "";
  height: 1em;
  width: auto;
  opacity: 0.8;
  margin: 0 0 0 0.5em;
  font-family: "Icons";
}

.ui.sortable.table thead th.ascending::after {
  content: "\f0d8";
}

.ui.sortable.table thead th.descending::after {
  content: "\f0d7";
}

/* Hover */

.ui.sortable.table th.disabled:hover {
  cursor: auto;
  color: rgba(40, 40, 40, 0.3);
}

.ui.sortable.table > thead > tr > th:hover {
  color: rgba(0, 0, 0, 0.8);
}

.ui.sortable.table:not(.basic) > thead > tr > th:hover {
  background: rgba(0, 0, 0, 0.05);
}

/* Sorted */

.ui.sortable.table thead th.sorted {
  color: rgba(0, 0, 0, 0.95);
}

.ui.sortable.table:not(.basic) thead th.sorted {
  background: rgba(0, 0, 0, 0.05);
}

.ui.sortable.table thead th.sorted::after {
  display: inline-block;
}

/* Sorted Hover */

.ui.sortable.table thead th.sorted:hover {
  color: rgba(0, 0, 0, 0.95);
}

.ui.sortable.table:not(.basic) thead th.sorted:hover {
  background: rgba(0, 0, 0, 0.05);
}

/* Inverted */

.ui.inverted.sortable.table thead th.sorted {
  color: #ffffff;
}

.ui.inverted.sortable.table:not(.basic) thead th.sorted {
  background: rgba(255, 255, 255, 0.15) linear-gradient(transparent, rgba(0, 0, 0, 0.05));
}

.ui.inverted.sortable.table > thead > tr > th:hover {
  color: #ffffff;
}

.ui.inverted.sortable.table:not(.basic) > thead > tr > th:hover {
  background: rgba(255, 255, 255, 0.08) linear-gradient(transparent, rgba(0, 0, 0, 0.05));
}

.ui.inverted.sortable.table:not(.basic) > thead > tr > th {
  border-left-color: transparent;
  border-right-color: transparent;
}

/* --------------
        Inverted
    --------------- */

/* Text Color */

.ui.inverted.table {
  background: #333;
  color: rgba(255, 255, 255, 0.9);
  border: none;
}

.ui.ui.inverted.table > thead > tr > th,
.ui.ui.inverted.table > tbody > tr > th,
.ui.ui.inverted.table > tr > th {
  background-color: rgba(0, 0, 0, 0.15);
  border-color: rgba(255, 255, 255, 0.1);
  color: rgba(255, 255, 255, 0.9);
}

.ui.ui.inverted.table > tfoot > tr > th,
.ui.ui.inverted.table > tfoot > tr > td {
  background-color: rgba(0, 0, 0, 0.15);
  border-color: rgba(255, 255, 255, 0.1);
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.table > tbody > tr > td,
.ui.inverted.table > tfoot > tr > td,
.ui.inverted.table > tr > td {
  border-color: rgba(255, 255, 255, 0.1);
}

.ui.ui.ui.inverted.table tr.disabled td,
.ui.ui.ui.inverted.table tr td.disabled,
.ui.inverted.table tr.disabled:hover td,
.ui.inverted.table tr:hover td.disabled {
  pointer-events: none;
  color: rgba(225, 225, 225, 0.3);
}

.ui.ui.ui.inverted.table tr td.disabled:not([class="disabled"]),
.ui.inverted.table tr.disabled:not([class="disabled"]) td,
.ui.inverted.table tr.disabled td[class]:not(.disabled),
.ui.inverted.table tr:hover td.disabled:not([class="disabled"]) {
  color: rgba(40, 40, 40, 0.3);
}

/* Definition */

.ui.inverted.definition.table > tfoot:not(.full-width) > tr > th:first-child,
.ui.inverted.definition.table > thead:not(.full-width) > tr > th:first-child {
  background: #fff;
}

.ui.inverted.definition.table > tbody > tr > td:first-child,
.ui.inverted.definition.table > tfoot > tr > td:first-child,
.ui.inverted.definition.table > tr > td:first-child {
  background: rgba(255, 255, 255, 0.02);
  color: #ffffff;
}

/* --------------
       Collapsing
    --------------- */

.ui.collapsing.table {
  width: auto;
}

/* --------------
          Basic
    --------------- */

.ui.basic.table {
  background: transparent;
  border: 1px solid rgba(34, 36, 38, 0.15);
  box-shadow: none;
}

.ui.basic.table > thead,
.ui.basic.table > tfoot {
  box-shadow: none;
}

.ui.basic.table > thead > tr > th,
.ui.basic.table > tbody > tr > th,
.ui.basic.table > tfoot > tr > th,
.ui.basic.table > tr > th {
  background: transparent;
  border-left: none;
}

.ui.basic.table > tbody > tr {
  border-bottom: 1px solid rgba(0, 0, 0, 0.1);
}

.ui.basic.table > tbody > tr > td,
.ui.basic.table > tfoot > tr > td,
.ui.basic.table > tr > td {
  background: transparent;
}

.ui.basic.striped.table > tbody > tr:nth-child(2n) {
  background-color: rgba(0, 0, 0, 0.05);
}

/* Very Basic */

.ui[class*="very basic"].table {
  border: none;
}

.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > th,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > thead > tr > th,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > th,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tfoot > tr > th,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > td,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > td {
  padding: "";
}

.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > th:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > thead > tr > th:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > th:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tfoot > tr > th:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > td:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > td:first-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tfoot > tr > td:first-child {
  padding-left: 0;
}

.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > th:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > thead > tr > th:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > th:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tfoot > tr > th:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tr > td:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tbody > tr > td:last-child,
.ui[class*="very basic"].table:not(.sortable):not(.striped) > tfoot > tr > td:last-child {
  padding-right: 0;
}

.ui[class*="very basic"].table:not(.sortable):not(.striped) > thead > tr:first-child > th {
  padding-top: 0;
}

/* --------------
         Celled
    --------------- */

.ui.celled.table > tr > th,
.ui.celled.table > thead > tr > th,
.ui.celled.table > tbody > tr > th,
.ui.celled.table > tfoot > tr > th,
.ui.celled.table > tr > td,
.ui.celled.table > tbody > tr > td,
.ui.celled.table > tfoot > tr > td {
  border-left: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.inverted.celled.table > tbody > tr > td,
.ui.inverted.celled.table > tr > td {
  border-left: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.celled.table > tr > th:first-child,
.ui.celled.table > thead > tr > th:first-child,
.ui.celled.table > tbody > tr > th:first-child,
.ui.celled.table > tfoot > tr > th:first-child,
.ui.celled.table > tr > td:first-child,
.ui.celled.table > tbody > tr > td:first-child,
.ui.celled.table > tfoot > tr > td:first-child {
  border-left: none;
}

/* --------------
         Padded
    --------------- */

.ui.padded.table > tr > th,
.ui.padded.table > thead > tr > th,
.ui.padded.table > tbody > tr > th,
.ui.padded.table > tfoot > tr > th {
  padding-left: 1em;
  padding-right: 1em;
}

.ui.padded.table > tr > th,
.ui.padded.table > thead > tr > th,
.ui.padded.table > tbody > tr > th,
.ui.padded.table > tfoot > tr > th,
.ui.padded.table > tr > td,
.ui.padded.table > tbody > tr > td,
.ui.padded.table > tfoot > tr > td {
  padding: 1em 1em;
}

/* Very */

.ui[class*="very padded"].table > tr > th,
.ui[class*="very padded"].table > thead > tr > th,
.ui[class*="very padded"].table > tbody > tr > th,
.ui[class*="very padded"].table > tfoot > tr > th {
  padding-left: 1.5em;
  padding-right: 1.5em;
}

.ui[class*="very padded"].table > tr > td,
.ui[class*="very padded"].table > tbody > tr > td,
.ui[class*="very padded"].table > tfoot > tr > td {
  padding: 1.5em 1.5em;
}

/* --------------
         Compact
    --------------- */

.ui.compact.table > tr > th,
.ui.compact.table > thead > tr > th,
.ui.compact.table > tbody > tr > th,
.ui.compact.table > tfoot > tr > th {
  padding-left: 0.7em;
  padding-right: 0.7em;
}

.ui.compact.table > tr > td,
.ui.compact.table > tbody > tr > td,
.ui.compact.table > tfoot > tr > td {
  padding: 0.5em 0.7em;
}

/* Very */

.ui[class*="very compact"].table > tr > th,
.ui[class*="very compact"].table > thead > tr > th,
.ui[class*="very compact"].table > tbody > tr > th,
.ui[class*="very compact"].table > tfoot > tr > th {
  padding-left: 0.6em;
  padding-right: 0.6em;
}

.ui[class*="very compact"].table > tr > td,
.ui[class*="very compact"].table > tbody > tr > td,
.ui[class*="very compact"].table > tfoot > tr > td {
  padding: 0.4em 0.6em;
}

/* --------------
         Stuck
    --------------- */

.ui.stuck.table:not(.inverted) {
  background: #fff;
}

.ui.stuck.table:not(.inverted) > thead > tr {
  background: #f9fafb;
}

.ui.stuck.table:not(.inverted) > tbody > tr {
  background: #fff;
}

.ui.stuck.table:not(.inverted) > tfoot > tr {
  background: #f9fafb;
}

.ui.stuck.table:not(.inverted).basic > thead > tr,
.ui.stuck.table:not(.inverted).basic > tfoot > tr {
  background: #fff;
}

.ui.inverted.stuck.table > thead > tr {
  background: #252525;
}

.ui.inverted.stuck.table > tbody > tr {
  background: #333;
}

.ui.inverted.stuck.table > tfoot > tr {
  background: #252525;
}

.ui.inverted.stuck.table.basic > thead > tr,
.ui.inverted.stuck.table.basic > tfoot > tr {
  background: #333;
}

.ui.head.stuck.table > thead,
.ui.foot.stuck.table > tfoot {
  position: sticky;
  z-index: 800;
}

.ui.head.stuck.table {
  border-top: 0;
}

.ui.head.stuck.table > thead {
  top: 0;
  bottom: auto;
}

.ui.head.stuck.table > thead > tr:first-child > th {
  border-top: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.head.stuck.table.inverted > thead > tr:first-child > th {
  border-top: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.foot.stuck.table {
  border-bottom: 0;
}

.ui.foot.stuck.table > tfoot {
  top: auto;
  bottom: 0;
}

.ui.foot.stuck.table > tfoot > tr:last-child > td,
.ui.foot.stuck.table > tfoot > tr:last-child > th {
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.foot.stuck.table.inverted > tfoot > tr:first-child > td,
.ui.foot.stuck.table.inverted > tfoot > tr:first-child > th {
  border-top: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.first.stuck.table {
  border-left: 0;
}

.ui.first.stuck.table th:first-child,
.ui.first.stuck.table td:first-child {
  position: sticky;
  left: 0;
  border-left: 1px solid rgba(34, 36, 38, 0.1);
  background: inherit;
}

.ui.first.stuck.table.inverted th:first-child,
.ui.first.stuck.table.inverted td:first-child {
  border-left: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.last.stuck.table {
  border-right: 0;
}

.ui.last.stuck.table th:last-child,
.ui.last.stuck.table td:last-child {
  position: sticky;
  right: 0;
  border-right: 1px solid rgba(34, 36, 38, 0.1);
  background: inherit;
}

.ui.last.stuck.table.inverted th:last-child,
.ui.last.stuck.table.inverted td:last-child {
  border-right: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.celled.first.stuck.table th:first-child,
.ui.celled.first.stuck.table td:first-child {
  border-right: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.celled.last.stuck.table th:last-child,
.ui.celled.last.stuck.table td:last-child {
  border-left: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.inverted.celled.first.stuck.table th:first-child,
.ui.inverted.celled.first.stuck.table td:first-child {
  border-right: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.inverted.celled.last.stuck.table th:last-child,
.ui.inverted.celled.last.stuck.table td:last-child {
  border-left: 1px solid rgba(255, 255, 255, 0.1);
}

/* --------------
      Sizes
--------------- */

/* Standard */

.ui.table {
  font-size: 1em;
}

.ui.mini.table {
  font-size: 0.78571429rem;
}

.ui.tiny.table {
  font-size: 0.85714286rem;
}

.ui.small.table {
  font-size: 0.9em;
}

.ui.large.table {
  font-size: 1.1em;
}

.ui.big.table {
  font-size: 1.28571429rem;
}

.ui.huge.table {
  font-size: 1.42857143rem;
}

.ui.massive.table {
  font-size: 1.71428571rem;
}

/*******************************
           Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Ad
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Copyright 2013 Contributors
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
         Advertisement
*******************************/

.ui.ad {
  display: block;
  overflow: hidden;
  margin: 1em 0;
}

.ui.ad:first-child {
  margin: 0;
}

.ui.ad:last-child {
  margin: 0;
}

.ui.ad iframe {
  margin: 0;
  padding: 0;
  border: none;
  overflow: hidden;
}

/* --------------
     Common
--------------- */

/* Leaderboard */

.ui.leaderboard.ad {
  width: 728px;
  height: 90px;
}

/* Medium Rectangle */

.ui[class*="medium rectangle"].ad {
  width: 300px;
  height: 250px;
}

/* Large Rectangle */

.ui[class*="large rectangle"].ad {
  width: 336px;
  height: 280px;
}

/* Half Page */

.ui[class*="half page"].ad {
  width: 300px;
  height: 600px;
}

/* --------------
         Square
    --------------- */

/* Square */

.ui.square.ad {
  width: 250px;
  height: 250px;
}

/* Small Square */

.ui[class*="small square"].ad {
  width: 200px;
  height: 200px;
}

/* --------------
        Rectangle
    --------------- */

/* Small Rectangle */

.ui[class*="small rectangle"].ad {
  width: 180px;
  height: 150px;
}

/* Vertical Rectangle */

.ui[class*="vertical rectangle"].ad {
  width: 240px;
  height: 400px;
}

/* --------------
         Button
    --------------- */

.ui.button.ad {
  width: 120px;
  height: 90px;
}

.ui[class*="square button"].ad {
  width: 125px;
  height: 125px;
}

.ui[class*="small button"].ad {
  width: 120px;
  height: 60px;
}

/* --------------
       Skyscrapers
    --------------- */

/* Skyscraper */

.ui.skyscraper.ad {
  width: 120px;
  height: 600px;
}

/* Wide Skyscraper */

.ui[class*="wide skyscraper"].ad {
  width: 160px;
}

/* --------------
         Banners
    --------------- */

/* Banner */

.ui.banner.ad {
  width: 468px;
  height: 60px;
}

/* Vertical Banner */

.ui[class*="vertical banner"].ad {
  width: 120px;
  height: 240px;
}

/* Top Banner */

.ui[class*="top banner"].ad {
  width: 930px;
  height: 180px;
}

/* Half Banner */

.ui[class*="half banner"].ad {
  width: 234px;
  height: 60px;
}

/* --------------
    Boards
--------------- */

/* Leaderboard */

.ui[class*="large leaderboard"].ad {
  width: 970px;
  height: 90px;
}

/* Billboard */

.ui.billboard.ad {
  width: 970px;
  height: 250px;
}

/* --------------
        Panorama
    --------------- */

/* Panorama */

.ui.panorama.ad {
  width: 980px;
  height: 120px;
}

/* --------------
         Netboard
    --------------- */

/* Netboard */

.ui.netboard.ad {
  width: 580px;
  height: 400px;
}

/* --------------
         Mobile
    --------------- */

/* Large Mobile Banner */

.ui[class*="large mobile banner"].ad {
  width: 320px;
  height: 100px;
}

/* Mobile Leaderboard */

.ui[class*="mobile leaderboard"].ad {
  width: 320px;
  height: 50px;
}

/*******************************
                 Types
    *******************************/

/* Mobile Sizes */

.ui.mobile.ad {
  display: none;
}

@media only screen and (max-width: 767.98px) {
  .ui.mobile.ad {
    display: block;
  }
}

/*******************************
           Variations
*******************************/

.ui.centered.ad {
  margin-left: auto;
  margin-right: auto;
}

.ui.test.ad {
  position: relative;
  background: #545454;
}

.ui.test.ad::after {
  position: absolute;
  top: 50%;
  left: 50%;
  width: 100%;
  text-align: center;
  transform: translateX(-50%) translateY(-50%);
  content: "Ad";
  color: #fff;
  font-size: 1em;
  font-weight: bold;
}

.ui.mobile.test.ad::after {
  font-size: 0.85714286em;
}

.ui.test.ad[data-text]::after {
  content: attr(data-text);
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Card
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Standard
*******************************/

/* --------------
      Card
--------------- */

.ui.cards > .card,
.ui.card {
  max-width: 100%;
  position: relative;
  display: flex;
  flex-direction: column;
  width: 290px;
  min-height: 0;
  background: #fff;
  padding: 0;
  border: none;
  border-radius: 0.28571429rem;
  box-shadow: 0 1px 3px 0 #d4d4d5, 0 0 0 1px #d4d4d5;
  transition: box-shadow 0.1s ease, transform 0.1s ease;
  z-index: "";
  word-wrap: break-word;
}

.ui.card {
  margin: 1em 0;
}

.ui.cards > .card a,
.ui.card a {
  cursor: pointer;
}

.ui.card:first-child {
  margin-top: 0;
}

.ui.card:last-child {
  margin-bottom: 0;
}

/* --------------
      Cards
--------------- */

.ui.cards {
  display: flex;
  margin: -0.875em -0.5em;
  flex-wrap: wrap;
}

.ui.cards > .card {
  display: flex;
  margin: 0.875em 0.5em;
  float: none;
}

/* Clearing */

.ui.cards::after,
.ui.card::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

/* Consecutive Card Groups Preserve Row Spacing */

.ui.cards ~ .ui.cards {
  margin-top: 0.875em;
}

/* --------------
  Rounded Edges
--------------- */

.ui.cards > .card > :first-child,
.ui.card > :first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0 !important;
  border-top: none !important;
}

.ui.cards > .card > :last-child,
.ui.card > :last-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem !important;
}

.ui.cards > .card > :only-child,
.ui.card > :only-child {
  border-radius: 0.28571429rem !important;
}

/* --------------
         Images
    --------------- */

.ui.cards > .card > .image,
.ui.card > .image {
  position: relative;
  display: block;
  flex: 0 0 auto;
  padding: 0;
  background: rgba(0, 0, 0, 0.05);
}

.ui.cards > .card > .image > img,
.ui.card > .image > img {
  display: block;
  width: 100%;
  height: auto;
  border-radius: inherit;
}

.ui.cards > .card > .image:not(.ui) > img,
.ui.card > .image:not(.ui) > img {
  border: none;
}

/* --------------
     Content
--------------- */

.ui.cards > .card > .content,
.ui.card > .content {
  flex-grow: 1;
  border: none;
  border-top: 1px solid rgba(34, 36, 38, 0.1);
  background: none;
  margin: 0;
  padding: 1em;
  box-shadow: none;
  font-size: 1em;
  border-radius: 0;
}

.ui.cards > .card > .content::after,
.ui.card > .content::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

.ui.cards > .card > .content > .header,
.ui.card > .content > .header {
  display: block;
  margin: "";
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  color: rgba(0, 0, 0, 0.85);
}

/* Default Header Size */

.ui.cards > .card > .content > .header:not(.ui),
.ui.card > .content > .header:not(.ui) {
  font-weight: bold;
  font-size: 1.28571429em;
  margin-top: -0.21425em;
  line-height: 1.28571429em;
}

.ui.cards > .card > .content > .meta + .description,
.ui.cards > .card > .content > .header + .description,
.ui.card > .content > .meta + .description,
.ui.card > .content > .header + .description {
  margin-top: 0.5em;
}

/* ----------------
     Floated Content
    ----------------- */

.ui.cards > .card [class*="left floated"],
.ui.card [class*="left floated"] {
  float: left;
}

.ui.cards > .card [class*="right floated"],
.ui.card [class*="right floated"] {
  float: right;
}

/* --------------
         Aligned
    --------------- */

.ui.cards > .card [class*="left aligned"],
.ui.card [class*="left aligned"] {
  text-align: left;
}

.ui.cards > .card [class*="center aligned"],
.ui.card [class*="center aligned"] {
  text-align: center;
}

.ui.cards > .card [class*="right aligned"],
.ui.card [class*="right aligned"] {
  text-align: right;
}

/* --------------
      Content Image
    --------------- */

.ui.cards > .card .content img,
.ui.card .content img {
  display: inline-block;
  vertical-align: middle;
  width: "";
}

.ui.cards > .card img.avatar,
.ui.cards > .card .avatar img,
.ui.card img.avatar,
.ui.card .avatar img {
  width: 2em;
  height: 2em;
  border-radius: 500rem;
}

/* --------------
       Description
    --------------- */

.ui.cards > .card > .content > .description,
.ui.card > .content > .description {
  clear: both;
  color: rgba(0, 0, 0, 0.68);
}

/* --------------
    Paragraph
--------------- */

.ui.cards > .card > .content p,
.ui.card > .content p {
  margin: 0 0 0.5em;
}

.ui.cards > .card > .content p:last-child,
.ui.card > .content p:last-child {
  margin-bottom: 0;
}

/* --------------
          Meta
    --------------- */

.ui.cards > .card .meta,
.ui.card .meta {
  font-size: 1em;
  color: rgba(0, 0, 0, 0.4);
}

.ui.cards > .card .meta *,
.ui.card .meta * {
  margin-right: 0.3em;
}

.ui.cards > .card .meta :last-child,
.ui.card .meta :last-child {
  margin-right: 0;
}

.ui.cards > .card .meta [class*="right floated"],
.ui.card .meta [class*="right floated"] {
  margin-right: 0;
  margin-left: 0.3em;
}

/* --------------
      Links
--------------- */

/* Generic */

.ui.cards > .card > .content a:not(.ui),
.ui.card > .content a:not(.ui) {
  color: "";
  transition: color 0.1s ease;
}

.ui.cards > .card > .content a:not(.ui):hover,
.ui.card > .content a:not(.ui):hover {
  color: "";
}

/* Header */

.ui.cards > .card > .content > a.header,
.ui.card > .content > a.header {
  color: rgba(0, 0, 0, 0.85);
}

.ui.cards > .card > .content > a.header:hover,
.ui.card > .content > a.header:hover {
  color: #1e70bf;
}

/* Meta */

.ui.cards > .card .meta > a:not(.ui),
.ui.card .meta > a:not(.ui) {
  color: rgba(0, 0, 0, 0.4);
}

.ui.cards > .card .meta > a:not(.ui):hover,
.ui.card .meta > a:not(.ui):hover {
  color: rgba(0, 0, 0, 0.87);
}

/* --------------
         Buttons
    --------------- */

.ui.cards > .card > .buttons,
.ui.card > .buttons,
.ui.cards > .card > .button,
.ui.card > .button {
  margin: 0 -1px;
  width: calc(100% + 2px);
}

.ui.cards > .card > .buttons:last-child,
.ui.card > .buttons:last-child,
.ui.cards > .card > .button:last-child,
.ui.card > .button:last-child {
  margin-bottom: -1px;
}

/* --------------
       Basic
    --------------- */

.ui.cards:not(.raised) > .basic.card:not(.raised),
.ui.ui.ui.basic.cards:not(.raised) > .card:not(.raised),
.ui.ui.ui.basic.card:not(.raised) {
  box-shadow: none;
}

.ui.cards:not(.raised) > .basic.card:not(.raised) > .buttons,
.ui.ui.ui.basic.cards:not(.raised) > .card:not(.raised) > .buttons,
.ui.ui.ui.basic.card:not(.raised) > .buttons,
.ui.cards:not(.raised) > .basic.card:not(.raised) > .button,
.ui.ui.ui.basic.cards:not(.raised) > .card:not(.raised) > .button,
.ui.ui.ui.basic.card:not(.raised) > .button {
  margin: 0;
  width: 100%;
}

.ui.ui.ui.ui.ui.basic.card:not(.raised):hover,
.ui.ui.ui.ui.ui.basic.cards:not(.raised) .card:not(.raised):hover {
  box-shadow: none;
}

/* --------------
      Dimmer
--------------- */

.ui.cards > .card .dimmer,
.ui.card .dimmer {
  background: "";
  z-index: 10;
}

/* --------------
     Labels
--------------- */

/* -----Star----- */

/* Icon */

.ui.cards > .card > .content .star.icon,
.ui.card > .content .star.icon {
  cursor: pointer;
  opacity: 0.75;
  transition: color 0.1s ease;
}

.ui.cards > .card > .content .star.icon:hover,
.ui.card > .content .star.icon:hover {
  opacity: 1;
  color: #ffb70a;
}

.ui.cards > .card > .content .active.star.icon,
.ui.card > .content .active.star.icon {
  color: #ffe623;
}

/* -----Like----- */

/* Icon */

.ui.cards > .card > .content .like.icon,
.ui.card > .content .like.icon {
  cursor: pointer;
  opacity: 0.75;
  transition: color 0.1s ease;
}

.ui.cards > .card > .content .like.icon:hover,
.ui.card > .content .like.icon:hover {
  opacity: 1;
  color: #ff2733;
}

.ui.cards > .card > .content .active.like.icon,
.ui.card > .content .active.like.icon {
  color: #ff2733;
}

/* ----------------
      Extra Content
    ----------------- */

.ui.cards > .card > .extra,
.ui.card > .extra {
  max-width: 100%;
  min-height: 0 !important;
  flex-grow: 0;
  border-top: 1px solid rgba(0, 0, 0, 0.05) !important;
  position: static;
  background: none;
  width: auto;
  margin: 0;
  padding: 0.75em 1em;
  top: 0;
  left: 0;
  color: rgba(0, 0, 0, 0.4);
  box-shadow: none;
  transition: color 0.1s ease;
}

.ui.cards > .card > .extra a:not(.ui),
.ui.card > .extra a:not(.ui) {
  color: rgba(0, 0, 0, 0.4);
}

.ui.cards > .card > .extra a:not(.ui):hover,
.ui.card > .extra a:not(.ui):hover {
  color: #1e70bf;
}

/*******************************
            States
*******************************/

/* --------------
        Disabled
    --------------- */

.ui.disabled.card {
  opacity: 0.45;
  color: rgba(40, 40, 40, 0.3);
  pointer-events: none;
}

/* --------------
        Loading
    --------------- */

.ui.loading.card {
  position: relative;
  cursor: default;
  pointer-events: none;
  transition: all 0s linear;
}

.ui.loading.card::before {
  position: absolute;
  content: "";
  top: 0;
  left: 0;
  background: rgba(255, 255, 255, 0.8);
  width: 100%;
  height: 100%;
  border-radius: 0.28571429rem;
  z-index: 100;
}

.ui.loading.card::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1.5em 0 0 -1.5em;
  width: 3em;
  height: 3em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
  visibility: visible;
  z-index: 101;
}

/*******************************
           Variations
*******************************/

/* -------------------
          Horizontal
    -------------------- */

.ui.horizontal.cards > .card,
.ui.card.horizontal {
  flex-flow: row wrap;
  min-width: 270px;
  width: 400px;
  max-width: 100%;
}

.ui.horizontal.cards > .card > .image,
.ui.card.horizontal > .image {
  border-radius: 0.28571429rem 0 0 0.28571429rem;
  width: 150px;
}

.ui.horizontal.cards > .card > .image > img,
.ui.card.horizontal > .image > img {
  background-size: cover;
  background-repeat: no-repeat;
  background-position: center;
  justify-content: center;
  align-items: center;
  display: flex;
  width: 100%;
  border-radius: 0.28571429rem 0 0 0.28571429rem;
}

.ui.horizontal.cards > .card > .image:last-child > img,
.ui.card.horizontal > .image:last-child > img {
  border-radius: 0 0.28571429rem 0.28571429rem 0;
}

.ui.horizontal.cards > .card > .content,
.ui.horizontal.card > .content {
  border-top: none;
  flex-basis: 1px;
}

.ui.horizontal.cards > .card > .extra,
.ui.horizontal.card > .extra {
  flex-basis: 100%;
}

/* -------------------
           Raised
    -------------------- */

.ui.cards > .raised.card,
.ui.raised.cards > .card,
.ui.raised.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.raised.cards a.card:hover,
.ui.link.cards .raised.card:hover,
a.ui.raised.card:hover,
.ui.link.raised.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 4px 0 rgba(34, 36, 38, 0.15), 0 2px 10px 0 rgba(34, 36, 38, 0.25);
}

.ui.basic.cards > .raised.card,
.ui.basic.raised.cards > .card,
.ui.raised.cards > .basic.card,
.ui.basic.raised.card {
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.basic.raised.cards a.card:hover,
.ui.link.cards .basic.raised.card:hover,
.ui.link.basic.cards .raised.card:hover,
.ui.link.basic.raised.cards .card:hover,
a.ui.basic.raised.card:hover,
.ui.link.basic.raised.card:hover {
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.15), 0 2px 10px 0 rgba(34, 36, 38, 0.25);
}

/* -------------------
           Centered
    -------------------- */

.ui.centered.cards {
  justify-content: center;
}

.ui.centered.card {
  margin-left: auto;
  margin-right: auto;
}

/* -------------------
            Fluid
    -------------------- */

.ui.fluid.card {
  width: 100%;
  max-width: 9999px;
}

/* -------------------
            Link
    -------------------- */

.ui.cards a.card,
.ui.link.cards .card,
a.ui.card,
.ui.link.card {
  transform: none;
}

.ui.cards a.card:hover,
.ui.link.cards .card:not(.icon):hover,
a.ui.card:hover,
.ui.link.card:hover {
  cursor: pointer;
  z-index: 5;
  background: #fff;
  border: none;
  box-shadow: 0 1px 3px 0 #bcbdbd, 0 0 0 1px #d4d4d5;
  transform: translateY(-3px);
}

/* -------------------
       Colors
-------------------- */

.ui.primary.cards > .card,
.ui.cards > .primary.card,
.ui.primary.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #2185d0, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.primary.card:hover,
.ui.primary.cards a.card:hover,
.ui.link.primary.cards .card:not(.icon):hover,
.ui.link.cards .primary.card:not(.icon):hover,
a.ui.primary.card:hover,
.ui.link.primary.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #1678c2, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.primary.card,
.ui.basic.primary.cards > .card,
.ui.basic.cards > .primary.card,
.ui.basic.primary.card {
  background: #54c8ff;
}

.ui.basic.cards a.primary.card:hover,
.ui.cards a.basic.primary.card:hover,
.ui.basic.primary.cards a.card:hover,
.ui.link.cards .basic.primary.card:not(.icon):hover,
.ui.link.basic.primary.cards .card:not(.icon):hover,
.ui.link.basic.cards .primary.card:not(.icon):hover,
a.ui.basic.primary.card:hover,
.ui.link.basic.primary.card:hover {
  background: #21b8ff;
}

.ui.inverted.primary.cards > .card,
.ui.inverted.cards > .primary.card,
.ui.inverted.primary.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #54c8ff, 0 0 0 1px #555;
}

.ui.inverted.cards a.primary.card:hover,
.ui.inverted.primary.cards a.card:hover,
.ui.link.inverted.primary.cards .card:not(.icon):hover,
.ui.link.inverted.cards .primary.card:not(.icon):hover,
a.ui.inverted.primary.card:hover,
.ui.link.inverted.primary.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #21b8ff, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.primary.card,
.ui.basic.inverted.primary.cards > .card,
.ui.basic.inverted.cards > .primary.card,
.ui.basic.inverted.primary.card {
  background: #2185d0;
}

.ui.inverted.cards a.basic.primary.card:hover,
.ui.basic.inverted.cards a.primary.card:hover,
.ui.basic.inverted.primary.cards a.card:hover,
.ui.link.inverted.cards .basic.primary.card:not(.icon):hover,
.ui.link.basic.inverted.primary.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .primary.card:not(.icon):hover,
a.ui.basic.inverted.primary.card:hover,
.ui.link.basic.inverted.primary.card:hover {
  background: #1678c2;
}

.ui.secondary.cards > .card,
.ui.cards > .secondary.card,
.ui.secondary.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #1b1c1d, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.secondary.card:hover,
.ui.secondary.cards a.card:hover,
.ui.link.secondary.cards .card:not(.icon):hover,
.ui.link.cards .secondary.card:not(.icon):hover,
a.ui.secondary.card:hover,
.ui.link.secondary.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #27292a, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.secondary.card,
.ui.basic.secondary.cards > .card,
.ui.basic.cards > .secondary.card,
.ui.basic.secondary.card {
  background: #545454;
}

.ui.basic.cards a.secondary.card:hover,
.ui.cards a.basic.secondary.card:hover,
.ui.basic.secondary.cards a.card:hover,
.ui.link.cards .basic.secondary.card:not(.icon):hover,
.ui.link.basic.secondary.cards .card:not(.icon):hover,
.ui.link.basic.cards .secondary.card:not(.icon):hover,
a.ui.basic.secondary.card:hover,
.ui.link.basic.secondary.card:hover {
  background: #6e6e6e;
}

.ui.inverted.secondary.cards > .card,
.ui.inverted.cards > .secondary.card,
.ui.inverted.secondary.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #545454, 0 0 0 1px #555;
}

.ui.inverted.cards a.secondary.card:hover,
.ui.inverted.secondary.cards a.card:hover,
.ui.link.inverted.secondary.cards .card:not(.icon):hover,
.ui.link.inverted.cards .secondary.card:not(.icon):hover,
a.ui.inverted.secondary.card:hover,
.ui.link.inverted.secondary.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #6e6e6e, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.secondary.card,
.ui.basic.inverted.secondary.cards > .card,
.ui.basic.inverted.cards > .secondary.card,
.ui.basic.inverted.secondary.card {
  background: #1b1c1d;
}

.ui.inverted.cards a.basic.secondary.card:hover,
.ui.basic.inverted.cards a.secondary.card:hover,
.ui.basic.inverted.secondary.cards a.card:hover,
.ui.link.inverted.cards .basic.secondary.card:not(.icon):hover,
.ui.link.basic.inverted.secondary.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .secondary.card:not(.icon):hover,
a.ui.basic.inverted.secondary.card:hover,
.ui.link.basic.inverted.secondary.card:hover {
  background: #27292a;
}

.ui.red.cards > .card,
.ui.cards > .red.card,
.ui.red.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #db2828, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.red.card:hover,
.ui.red.cards a.card:hover,
.ui.link.red.cards .card:not(.icon):hover,
.ui.link.cards .red.card:not(.icon):hover,
a.ui.red.card:hover,
.ui.link.red.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #d01919, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.red.card,
.ui.basic.red.cards > .card,
.ui.basic.cards > .red.card,
.ui.basic.red.card {
  background: #ff695e;
}

.ui.basic.cards a.red.card:hover,
.ui.cards a.basic.red.card:hover,
.ui.basic.red.cards a.card:hover,
.ui.link.cards .basic.red.card:not(.icon):hover,
.ui.link.basic.red.cards .card:not(.icon):hover,
.ui.link.basic.cards .red.card:not(.icon):hover,
a.ui.basic.red.card:hover,
.ui.link.basic.red.card:hover {
  background: #ff392b;
}

.ui.inverted.red.cards > .card,
.ui.inverted.cards > .red.card,
.ui.inverted.red.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ff695e, 0 0 0 1px #555;
}

.ui.inverted.cards a.red.card:hover,
.ui.inverted.red.cards a.card:hover,
.ui.link.inverted.red.cards .card:not(.icon):hover,
.ui.link.inverted.cards .red.card:not(.icon):hover,
a.ui.inverted.red.card:hover,
.ui.link.inverted.red.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ff392b, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.red.card,
.ui.basic.inverted.red.cards > .card,
.ui.basic.inverted.cards > .red.card,
.ui.basic.inverted.red.card {
  background: #db2828;
}

.ui.inverted.cards a.basic.red.card:hover,
.ui.basic.inverted.cards a.red.card:hover,
.ui.basic.inverted.red.cards a.card:hover,
.ui.link.inverted.cards .basic.red.card:not(.icon):hover,
.ui.link.basic.inverted.red.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .red.card:not(.icon):hover,
a.ui.basic.inverted.red.card:hover,
.ui.link.basic.inverted.red.card:hover {
  background: #d01919;
}

.ui.orange.cards > .card,
.ui.cards > .orange.card,
.ui.orange.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #f2711c, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.orange.card:hover,
.ui.orange.cards a.card:hover,
.ui.link.orange.cards .card:not(.icon):hover,
.ui.link.cards .orange.card:not(.icon):hover,
a.ui.orange.card:hover,
.ui.link.orange.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #f26202, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.orange.card,
.ui.basic.orange.cards > .card,
.ui.basic.cards > .orange.card,
.ui.basic.orange.card {
  background: #ff851b;
}

.ui.basic.cards a.orange.card:hover,
.ui.cards a.basic.orange.card:hover,
.ui.basic.orange.cards a.card:hover,
.ui.link.cards .basic.orange.card:not(.icon):hover,
.ui.link.basic.orange.cards .card:not(.icon):hover,
.ui.link.basic.cards .orange.card:not(.icon):hover,
a.ui.basic.orange.card:hover,
.ui.link.basic.orange.card:hover {
  background: #e76b00;
}

.ui.inverted.orange.cards > .card,
.ui.inverted.cards > .orange.card,
.ui.inverted.orange.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ff851b, 0 0 0 1px #555;
}

.ui.inverted.cards a.orange.card:hover,
.ui.inverted.orange.cards a.card:hover,
.ui.link.inverted.orange.cards .card:not(.icon):hover,
.ui.link.inverted.cards .orange.card:not(.icon):hover,
a.ui.inverted.orange.card:hover,
.ui.link.inverted.orange.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #e76b00, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.orange.card,
.ui.basic.inverted.orange.cards > .card,
.ui.basic.inverted.cards > .orange.card,
.ui.basic.inverted.orange.card {
  background: #f2711c;
}

.ui.inverted.cards a.basic.orange.card:hover,
.ui.basic.inverted.cards a.orange.card:hover,
.ui.basic.inverted.orange.cards a.card:hover,
.ui.link.inverted.cards .basic.orange.card:not(.icon):hover,
.ui.link.basic.inverted.orange.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .orange.card:not(.icon):hover,
a.ui.basic.inverted.orange.card:hover,
.ui.link.basic.inverted.orange.card:hover {
  background: #f26202;
}

.ui.yellow.cards > .card,
.ui.cards > .yellow.card,
.ui.yellow.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #fbbd08, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.yellow.card:hover,
.ui.yellow.cards a.card:hover,
.ui.link.yellow.cards .card:not(.icon):hover,
.ui.link.cards .yellow.card:not(.icon):hover,
a.ui.yellow.card:hover,
.ui.link.yellow.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #eaae00, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.yellow.card,
.ui.basic.yellow.cards > .card,
.ui.basic.cards > .yellow.card,
.ui.basic.yellow.card {
  background: #ffe21f;
}

.ui.basic.cards a.yellow.card:hover,
.ui.cards a.basic.yellow.card:hover,
.ui.basic.yellow.cards a.card:hover,
.ui.link.cards .basic.yellow.card:not(.icon):hover,
.ui.link.basic.yellow.cards .card:not(.icon):hover,
.ui.link.basic.cards .yellow.card:not(.icon):hover,
a.ui.basic.yellow.card:hover,
.ui.link.basic.yellow.card:hover {
  background: #ebcd00;
}

.ui.inverted.yellow.cards > .card,
.ui.inverted.cards > .yellow.card,
.ui.inverted.yellow.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ffe21f, 0 0 0 1px #555;
}

.ui.inverted.cards a.yellow.card:hover,
.ui.inverted.yellow.cards a.card:hover,
.ui.link.inverted.yellow.cards .card:not(.icon):hover,
.ui.link.inverted.cards .yellow.card:not(.icon):hover,
a.ui.inverted.yellow.card:hover,
.ui.link.inverted.yellow.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ebcd00, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.yellow.card,
.ui.basic.inverted.yellow.cards > .card,
.ui.basic.inverted.cards > .yellow.card,
.ui.basic.inverted.yellow.card {
  background: #fbbd08;
}

.ui.inverted.cards a.basic.yellow.card:hover,
.ui.basic.inverted.cards a.yellow.card:hover,
.ui.basic.inverted.yellow.cards a.card:hover,
.ui.link.inverted.cards .basic.yellow.card:not(.icon):hover,
.ui.link.basic.inverted.yellow.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .yellow.card:not(.icon):hover,
a.ui.basic.inverted.yellow.card:hover,
.ui.link.basic.inverted.yellow.card:hover {
  background: #eaae00;
}

.ui.olive.cards > .card,
.ui.cards > .olive.card,
.ui.olive.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #b5cc18, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.olive.card:hover,
.ui.olive.cards a.card:hover,
.ui.link.olive.cards .card:not(.icon):hover,
.ui.link.cards .olive.card:not(.icon):hover,
a.ui.olive.card:hover,
.ui.link.olive.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #a7bd0d, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.olive.card,
.ui.basic.olive.cards > .card,
.ui.basic.cards > .olive.card,
.ui.basic.olive.card {
  background: #d9e778;
}

.ui.basic.cards a.olive.card:hover,
.ui.cards a.basic.olive.card:hover,
.ui.basic.olive.cards a.card:hover,
.ui.link.cards .basic.olive.card:not(.icon):hover,
.ui.link.basic.olive.cards .card:not(.icon):hover,
.ui.link.basic.cards .olive.card:not(.icon):hover,
a.ui.basic.olive.card:hover,
.ui.link.basic.olive.card:hover {
  background: #d2e745;
}

.ui.inverted.olive.cards > .card,
.ui.inverted.cards > .olive.card,
.ui.inverted.olive.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #d9e778, 0 0 0 1px #555;
}

.ui.inverted.cards a.olive.card:hover,
.ui.inverted.olive.cards a.card:hover,
.ui.link.inverted.olive.cards .card:not(.icon):hover,
.ui.link.inverted.cards .olive.card:not(.icon):hover,
a.ui.inverted.olive.card:hover,
.ui.link.inverted.olive.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #d2e745, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.olive.card,
.ui.basic.inverted.olive.cards > .card,
.ui.basic.inverted.cards > .olive.card,
.ui.basic.inverted.olive.card {
  background: #b5cc18;
}

.ui.inverted.cards a.basic.olive.card:hover,
.ui.basic.inverted.cards a.olive.card:hover,
.ui.basic.inverted.olive.cards a.card:hover,
.ui.link.inverted.cards .basic.olive.card:not(.icon):hover,
.ui.link.basic.inverted.olive.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .olive.card:not(.icon):hover,
a.ui.basic.inverted.olive.card:hover,
.ui.link.basic.inverted.olive.card:hover {
  background: #a7bd0d;
}

.ui.green.cards > .card,
.ui.cards > .green.card,
.ui.green.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #21ba45, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.green.card:hover,
.ui.green.cards a.card:hover,
.ui.link.green.cards .card:not(.icon):hover,
.ui.link.cards .green.card:not(.icon):hover,
a.ui.green.card:hover,
.ui.link.green.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #16ab39, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.green.card,
.ui.basic.green.cards > .card,
.ui.basic.cards > .green.card,
.ui.basic.green.card {
  background: #2ecc40;
}

.ui.basic.cards a.green.card:hover,
.ui.cards a.basic.green.card:hover,
.ui.basic.green.cards a.card:hover,
.ui.link.cards .basic.green.card:not(.icon):hover,
.ui.link.basic.green.cards .card:not(.icon):hover,
.ui.link.basic.cards .green.card:not(.icon):hover,
a.ui.basic.green.card:hover,
.ui.link.basic.green.card:hover {
  background: #1ea92e;
}

.ui.inverted.green.cards > .card,
.ui.inverted.cards > .green.card,
.ui.inverted.green.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #2ecc40, 0 0 0 1px #555;
}

.ui.inverted.cards a.green.card:hover,
.ui.inverted.green.cards a.card:hover,
.ui.link.inverted.green.cards .card:not(.icon):hover,
.ui.link.inverted.cards .green.card:not(.icon):hover,
a.ui.inverted.green.card:hover,
.ui.link.inverted.green.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #1ea92e, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.green.card,
.ui.basic.inverted.green.cards > .card,
.ui.basic.inverted.cards > .green.card,
.ui.basic.inverted.green.card {
  background: #21ba45;
}

.ui.inverted.cards a.basic.green.card:hover,
.ui.basic.inverted.cards a.green.card:hover,
.ui.basic.inverted.green.cards a.card:hover,
.ui.link.inverted.cards .basic.green.card:not(.icon):hover,
.ui.link.basic.inverted.green.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .green.card:not(.icon):hover,
a.ui.basic.inverted.green.card:hover,
.ui.link.basic.inverted.green.card:hover {
  background: #16ab39;
}

.ui.teal.cards > .card,
.ui.cards > .teal.card,
.ui.teal.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #00b5ad, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.teal.card:hover,
.ui.teal.cards a.card:hover,
.ui.link.teal.cards .card:not(.icon):hover,
.ui.link.cards .teal.card:not(.icon):hover,
a.ui.teal.card:hover,
.ui.link.teal.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #009c95, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.teal.card,
.ui.basic.teal.cards > .card,
.ui.basic.cards > .teal.card,
.ui.basic.teal.card {
  background: #6dffff;
}

.ui.basic.cards a.teal.card:hover,
.ui.cards a.basic.teal.card:hover,
.ui.basic.teal.cards a.card:hover,
.ui.link.cards .basic.teal.card:not(.icon):hover,
.ui.link.basic.teal.cards .card:not(.icon):hover,
.ui.link.basic.cards .teal.card:not(.icon):hover,
a.ui.basic.teal.card:hover,
.ui.link.basic.teal.card:hover {
  background: #3affff;
}

.ui.inverted.teal.cards > .card,
.ui.inverted.cards > .teal.card,
.ui.inverted.teal.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #6dffff, 0 0 0 1px #555;
}

.ui.inverted.cards a.teal.card:hover,
.ui.inverted.teal.cards a.card:hover,
.ui.link.inverted.teal.cards .card:not(.icon):hover,
.ui.link.inverted.cards .teal.card:not(.icon):hover,
a.ui.inverted.teal.card:hover,
.ui.link.inverted.teal.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #3affff, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.teal.card,
.ui.basic.inverted.teal.cards > .card,
.ui.basic.inverted.cards > .teal.card,
.ui.basic.inverted.teal.card {
  background: #00b5ad;
}

.ui.inverted.cards a.basic.teal.card:hover,
.ui.basic.inverted.cards a.teal.card:hover,
.ui.basic.inverted.teal.cards a.card:hover,
.ui.link.inverted.cards .basic.teal.card:not(.icon):hover,
.ui.link.basic.inverted.teal.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .teal.card:not(.icon):hover,
a.ui.basic.inverted.teal.card:hover,
.ui.link.basic.inverted.teal.card:hover {
  background: #009c95;
}

.ui.blue.cards > .card,
.ui.cards > .blue.card,
.ui.blue.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #2185d0, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.blue.card:hover,
.ui.blue.cards a.card:hover,
.ui.link.blue.cards .card:not(.icon):hover,
.ui.link.cards .blue.card:not(.icon):hover,
a.ui.blue.card:hover,
.ui.link.blue.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #1678c2, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.blue.card,
.ui.basic.blue.cards > .card,
.ui.basic.cards > .blue.card,
.ui.basic.blue.card {
  background: #54c8ff;
}

.ui.basic.cards a.blue.card:hover,
.ui.cards a.basic.blue.card:hover,
.ui.basic.blue.cards a.card:hover,
.ui.link.cards .basic.blue.card:not(.icon):hover,
.ui.link.basic.blue.cards .card:not(.icon):hover,
.ui.link.basic.cards .blue.card:not(.icon):hover,
a.ui.basic.blue.card:hover,
.ui.link.basic.blue.card:hover {
  background: #21b8ff;
}

.ui.inverted.blue.cards > .card,
.ui.inverted.cards > .blue.card,
.ui.inverted.blue.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #54c8ff, 0 0 0 1px #555;
}

.ui.inverted.cards a.blue.card:hover,
.ui.inverted.blue.cards a.card:hover,
.ui.link.inverted.blue.cards .card:not(.icon):hover,
.ui.link.inverted.cards .blue.card:not(.icon):hover,
a.ui.inverted.blue.card:hover,
.ui.link.inverted.blue.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #21b8ff, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.blue.card,
.ui.basic.inverted.blue.cards > .card,
.ui.basic.inverted.cards > .blue.card,
.ui.basic.inverted.blue.card {
  background: #2185d0;
}

.ui.inverted.cards a.basic.blue.card:hover,
.ui.basic.inverted.cards a.blue.card:hover,
.ui.basic.inverted.blue.cards a.card:hover,
.ui.link.inverted.cards .basic.blue.card:not(.icon):hover,
.ui.link.basic.inverted.blue.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .blue.card:not(.icon):hover,
a.ui.basic.inverted.blue.card:hover,
.ui.link.basic.inverted.blue.card:hover {
  background: #1678c2;
}

.ui.violet.cards > .card,
.ui.cards > .violet.card,
.ui.violet.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #6435c9, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.violet.card:hover,
.ui.violet.cards a.card:hover,
.ui.link.violet.cards .card:not(.icon):hover,
.ui.link.cards .violet.card:not(.icon):hover,
a.ui.violet.card:hover,
.ui.link.violet.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #5829bb, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.violet.card,
.ui.basic.violet.cards > .card,
.ui.basic.cards > .violet.card,
.ui.basic.violet.card {
  background: #a291fb;
}

.ui.basic.cards a.violet.card:hover,
.ui.cards a.basic.violet.card:hover,
.ui.basic.violet.cards a.card:hover,
.ui.link.cards .basic.violet.card:not(.icon):hover,
.ui.link.basic.violet.cards .card:not(.icon):hover,
.ui.link.basic.cards .violet.card:not(.icon):hover,
a.ui.basic.violet.card:hover,
.ui.link.basic.violet.card:hover {
  background: #745aff;
}

.ui.inverted.violet.cards > .card,
.ui.inverted.cards > .violet.card,
.ui.inverted.violet.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #a291fb, 0 0 0 1px #555;
}

.ui.inverted.cards a.violet.card:hover,
.ui.inverted.violet.cards a.card:hover,
.ui.link.inverted.violet.cards .card:not(.icon):hover,
.ui.link.inverted.cards .violet.card:not(.icon):hover,
a.ui.inverted.violet.card:hover,
.ui.link.inverted.violet.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #745aff, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.violet.card,
.ui.basic.inverted.violet.cards > .card,
.ui.basic.inverted.cards > .violet.card,
.ui.basic.inverted.violet.card {
  background: #6435c9;
}

.ui.inverted.cards a.basic.violet.card:hover,
.ui.basic.inverted.cards a.violet.card:hover,
.ui.basic.inverted.violet.cards a.card:hover,
.ui.link.inverted.cards .basic.violet.card:not(.icon):hover,
.ui.link.basic.inverted.violet.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .violet.card:not(.icon):hover,
a.ui.basic.inverted.violet.card:hover,
.ui.link.basic.inverted.violet.card:hover {
  background: #5829bb;
}

.ui.purple.cards > .card,
.ui.cards > .purple.card,
.ui.purple.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #a333c8, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.purple.card:hover,
.ui.purple.cards a.card:hover,
.ui.link.purple.cards .card:not(.icon):hover,
.ui.link.cards .purple.card:not(.icon):hover,
a.ui.purple.card:hover,
.ui.link.purple.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #9627ba, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.purple.card,
.ui.basic.purple.cards > .card,
.ui.basic.cards > .purple.card,
.ui.basic.purple.card {
  background: #dc73ff;
}

.ui.basic.cards a.purple.card:hover,
.ui.cards a.basic.purple.card:hover,
.ui.basic.purple.cards a.card:hover,
.ui.link.cards .basic.purple.card:not(.icon):hover,
.ui.link.basic.purple.cards .card:not(.icon):hover,
.ui.link.basic.cards .purple.card:not(.icon):hover,
a.ui.basic.purple.card:hover,
.ui.link.basic.purple.card:hover {
  background: #cf40ff;
}

.ui.inverted.purple.cards > .card,
.ui.inverted.cards > .purple.card,
.ui.inverted.purple.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #dc73ff, 0 0 0 1px #555;
}

.ui.inverted.cards a.purple.card:hover,
.ui.inverted.purple.cards a.card:hover,
.ui.link.inverted.purple.cards .card:not(.icon):hover,
.ui.link.inverted.cards .purple.card:not(.icon):hover,
a.ui.inverted.purple.card:hover,
.ui.link.inverted.purple.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #cf40ff, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.purple.card,
.ui.basic.inverted.purple.cards > .card,
.ui.basic.inverted.cards > .purple.card,
.ui.basic.inverted.purple.card {
  background: #a333c8;
}

.ui.inverted.cards a.basic.purple.card:hover,
.ui.basic.inverted.cards a.purple.card:hover,
.ui.basic.inverted.purple.cards a.card:hover,
.ui.link.inverted.cards .basic.purple.card:not(.icon):hover,
.ui.link.basic.inverted.purple.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .purple.card:not(.icon):hover,
a.ui.basic.inverted.purple.card:hover,
.ui.link.basic.inverted.purple.card:hover {
  background: #9627ba;
}

.ui.pink.cards > .card,
.ui.cards > .pink.card,
.ui.pink.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #e03997, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.pink.card:hover,
.ui.pink.cards a.card:hover,
.ui.link.pink.cards .card:not(.icon):hover,
.ui.link.cards .pink.card:not(.icon):hover,
a.ui.pink.card:hover,
.ui.link.pink.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #e61a8d, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.pink.card,
.ui.basic.pink.cards > .card,
.ui.basic.cards > .pink.card,
.ui.basic.pink.card {
  background: #ff8edf;
}

.ui.basic.cards a.pink.card:hover,
.ui.cards a.basic.pink.card:hover,
.ui.basic.pink.cards a.card:hover,
.ui.link.cards .basic.pink.card:not(.icon):hover,
.ui.link.basic.pink.cards .card:not(.icon):hover,
.ui.link.basic.cards .pink.card:not(.icon):hover,
a.ui.basic.pink.card:hover,
.ui.link.basic.pink.card:hover {
  background: #ff5bd1;
}

.ui.inverted.pink.cards > .card,
.ui.inverted.cards > .pink.card,
.ui.inverted.pink.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ff8edf, 0 0 0 1px #555;
}

.ui.inverted.cards a.pink.card:hover,
.ui.inverted.pink.cards a.card:hover,
.ui.link.inverted.pink.cards .card:not(.icon):hover,
.ui.link.inverted.cards .pink.card:not(.icon):hover,
a.ui.inverted.pink.card:hover,
.ui.link.inverted.pink.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #ff5bd1, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.pink.card,
.ui.basic.inverted.pink.cards > .card,
.ui.basic.inverted.cards > .pink.card,
.ui.basic.inverted.pink.card {
  background: #e03997;
}

.ui.inverted.cards a.basic.pink.card:hover,
.ui.basic.inverted.cards a.pink.card:hover,
.ui.basic.inverted.pink.cards a.card:hover,
.ui.link.inverted.cards .basic.pink.card:not(.icon):hover,
.ui.link.basic.inverted.pink.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .pink.card:not(.icon):hover,
a.ui.basic.inverted.pink.card:hover,
.ui.link.basic.inverted.pink.card:hover {
  background: #e61a8d;
}

.ui.brown.cards > .card,
.ui.cards > .brown.card,
.ui.brown.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #a5673f, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.brown.card:hover,
.ui.brown.cards a.card:hover,
.ui.link.brown.cards .card:not(.icon):hover,
.ui.link.cards .brown.card:not(.icon):hover,
a.ui.brown.card:hover,
.ui.link.brown.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #975b33, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.brown.card,
.ui.basic.brown.cards > .card,
.ui.basic.cards > .brown.card,
.ui.basic.brown.card {
  background: #d67c1c;
}

.ui.basic.cards a.brown.card:hover,
.ui.cards a.basic.brown.card:hover,
.ui.basic.brown.cards a.card:hover,
.ui.link.cards .basic.brown.card:not(.icon):hover,
.ui.link.basic.brown.cards .card:not(.icon):hover,
.ui.link.basic.cards .brown.card:not(.icon):hover,
a.ui.basic.brown.card:hover,
.ui.link.basic.brown.card:hover {
  background: #b0620f;
}

.ui.inverted.brown.cards > .card,
.ui.inverted.cards > .brown.card,
.ui.inverted.brown.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #d67c1c, 0 0 0 1px #555;
}

.ui.inverted.cards a.brown.card:hover,
.ui.inverted.brown.cards a.card:hover,
.ui.link.inverted.brown.cards .card:not(.icon):hover,
.ui.link.inverted.cards .brown.card:not(.icon):hover,
a.ui.inverted.brown.card:hover,
.ui.link.inverted.brown.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #b0620f, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.brown.card,
.ui.basic.inverted.brown.cards > .card,
.ui.basic.inverted.cards > .brown.card,
.ui.basic.inverted.brown.card {
  background: #a5673f;
}

.ui.inverted.cards a.basic.brown.card:hover,
.ui.basic.inverted.cards a.brown.card:hover,
.ui.basic.inverted.brown.cards a.card:hover,
.ui.link.inverted.cards .basic.brown.card:not(.icon):hover,
.ui.link.basic.inverted.brown.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .brown.card:not(.icon):hover,
a.ui.basic.inverted.brown.card:hover,
.ui.link.basic.inverted.brown.card:hover {
  background: #975b33;
}

.ui.grey.cards > .card,
.ui.cards > .grey.card,
.ui.grey.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #767676, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.grey.card:hover,
.ui.grey.cards a.card:hover,
.ui.link.grey.cards .card:not(.icon):hover,
.ui.link.cards .grey.card:not(.icon):hover,
a.ui.grey.card:hover,
.ui.link.grey.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #838383, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.grey.card,
.ui.basic.grey.cards > .card,
.ui.basic.cards > .grey.card,
.ui.basic.grey.card {
  background: #dcddde;
}

.ui.basic.cards a.grey.card:hover,
.ui.cards a.basic.grey.card:hover,
.ui.basic.grey.cards a.card:hover,
.ui.link.cards .basic.grey.card:not(.icon):hover,
.ui.link.basic.grey.cards .card:not(.icon):hover,
.ui.link.basic.cards .grey.card:not(.icon):hover,
a.ui.basic.grey.card:hover,
.ui.link.basic.grey.card:hover {
  background: #c2c4c5;
}

.ui.inverted.grey.cards > .card,
.ui.inverted.cards > .grey.card,
.ui.inverted.grey.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #dcddde, 0 0 0 1px #555;
}

.ui.inverted.cards a.grey.card:hover,
.ui.inverted.grey.cards a.card:hover,
.ui.link.inverted.grey.cards .card:not(.icon):hover,
.ui.link.inverted.cards .grey.card:not(.icon):hover,
a.ui.inverted.grey.card:hover,
.ui.link.inverted.grey.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #c2c4c5, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.grey.card,
.ui.basic.inverted.grey.cards > .card,
.ui.basic.inverted.cards > .grey.card,
.ui.basic.inverted.grey.card {
  background: #767676;
}

.ui.inverted.cards a.basic.grey.card:hover,
.ui.basic.inverted.cards a.grey.card:hover,
.ui.basic.inverted.grey.cards a.card:hover,
.ui.link.inverted.cards .basic.grey.card:not(.icon):hover,
.ui.link.basic.inverted.grey.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .grey.card:not(.icon):hover,
a.ui.basic.inverted.grey.card:hover,
.ui.link.basic.inverted.grey.card:hover {
  background: #838383;
}

.ui.black.cards > .card,
.ui.cards > .black.card,
.ui.black.card {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #1b1c1d, 0 1px 3px 0 #d4d4d5;
}

.ui.cards a.black.card:hover,
.ui.black.cards a.card:hover,
.ui.link.black.cards .card:not(.icon):hover,
.ui.link.cards .black.card:not(.icon):hover,
a.ui.black.card:hover,
.ui.link.black.card:hover {
  box-shadow: 0 0 0 1px #d4d4d5, 0 2px 0 0 #27292a, 0 1px 3px 0 #bcbdbd;
}

.ui.cards > .basic.black.card,
.ui.basic.black.cards > .card,
.ui.basic.cards > .black.card,
.ui.basic.black.card {
  background: #545454;
}

.ui.cards > .basic.black.card .header,
.ui.basic.black.cards > .card .header,
.ui.basic.cards > .black.card .header,
.ui.basic.black.card .header,
.ui.cards > .basic.black.card .content,
.ui.basic.black.cards > .card .content,
.ui.basic.cards > .black.card .content,
.ui.basic.black.card .content,
.ui.cards > .basic.black.card .meta,
.ui.basic.black.cards > .card .meta,
.ui.basic.cards > .black.card .meta,
.ui.basic.black.card .meta,
.ui.cards > .basic.black.card .description,
.ui.basic.black.cards > .card .description,
.ui.basic.cards > .black.card .description,
.ui.basic.black.card .description {
  color: #fff;
}

.ui.basic.cards a.black.card:hover,
.ui.cards a.basic.black.card:hover,
.ui.basic.black.cards a.card:hover,
.ui.link.cards .basic.black.card:not(.icon):hover,
.ui.link.basic.black.cards .card:not(.icon):hover,
.ui.link.basic.cards .black.card:not(.icon):hover,
a.ui.basic.black.card:hover,
.ui.link.basic.black.card:hover {
  background: #000000;
}

.ui.inverted.black.cards > .card,
.ui.inverted.cards > .black.card,
.ui.inverted.black.card {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #545454, 0 0 0 1px #555;
}

.ui.inverted.cards a.black.card:hover,
.ui.inverted.black.cards a.card:hover,
.ui.link.inverted.black.cards .card:not(.icon):hover,
.ui.link.inverted.cards .black.card:not(.icon):hover,
a.ui.inverted.black.card:hover,
.ui.link.inverted.black.card:hover {
  box-shadow: 0 1px 3px 0 #555, 0 2px 0 0 #000000, 0 0 0 1px #555;
}

.ui.inverted.cards > .basic.black.card,
.ui.basic.inverted.black.cards > .card,
.ui.basic.inverted.cards > .black.card,
.ui.basic.inverted.black.card {
  background: #1b1c1d;
}

.ui.inverted.cards a.basic.black.card:hover,
.ui.basic.inverted.cards a.black.card:hover,
.ui.basic.inverted.black.cards a.card:hover,
.ui.link.inverted.cards .basic.black.card:not(.icon):hover,
.ui.link.basic.inverted.black.cards .card:not(.icon):hover,
.ui.link.basic.inverted.cards .black.card:not(.icon):hover,
a.ui.basic.inverted.black.card:hover,
.ui.link.basic.inverted.black.card:hover {
  background: #27292a;
}

/* --------------
       Card Count
    --------------- */

.ui.one.cards {
  margin-left: 0;
  margin-right: 0;
}

.ui.one.cards > .card {
  width: 100%;
}

.ui.two.cards {
  margin-left: -1em;
  margin-right: -1em;
}

.ui.two.cards > .card {
  width: calc(50% - 2em);
  margin-left: 1em;
  margin-right: 1em;
}

.ui.three.cards {
  margin-left: -1em;
  margin-right: -1em;
}

.ui.three.cards > .card {
  width: calc(33.33333333333333% - 2em);
  margin-left: 1em;
  margin-right: 1em;
}

.ui.four.cards {
  margin-left: -0.75em;
  margin-right: -0.75em;
}

.ui.four.cards > .card {
  width: calc(25% - 1.5em);
  margin-left: 0.75em;
  margin-right: 0.75em;
}

.ui.five.cards {
  margin-left: -0.75em;
  margin-right: -0.75em;
}

.ui.five.cards > .card {
  width: calc(20% - 1.5em);
  margin-left: 0.75em;
  margin-right: 0.75em;
}

.ui.six.cards {
  margin-left: -0.75em;
  margin-right: -0.75em;
}

.ui.six.cards > .card {
  width: calc(16.666666666666664% - 1.5em);
  margin-left: 0.75em;
  margin-right: 0.75em;
}

.ui.seven.cards {
  margin-left: -0.5em;
  margin-right: -0.5em;
}

.ui.seven.cards > .card {
  width: calc(14.285714285714285% - 1em);
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.ui.eight.cards {
  margin-left: -0.5em;
  margin-right: -0.5em;
}

.ui.eight.cards > .card {
  width: calc(12.5% - 1em);
  margin-left: 0.5em;
  margin-right: 0.5em;
  font-size: 11px;
}

.ui.nine.cards {
  margin-left: -0.5em;
  margin-right: -0.5em;
}

.ui.nine.cards > .card {
  width: calc(11.11111111111111% - 1em);
  margin-left: 0.5em;
  margin-right: 0.5em;
  font-size: 10px;
}

.ui.ten.cards {
  margin-left: -0.5em;
  margin-right: -0.5em;
}

.ui.ten.cards > .card {
  width: calc(10% - 1em);
  margin-left: 0.5em;
  margin-right: 0.5em;
}

/* -------------------
          Doubling
    -------------------- */

/* Mobile Only */

@media only screen and (max-width: 767.98px) {
  .ui.two.doubling.cards {
    margin-left: 0;
    margin-right: 0;
  }

  .ui.two.doubling.cards > .card {
    width: 100%;
    margin-left: 0;
    margin-right: 0;
  }

  .ui.three.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.three.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.four.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.four.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.five.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.five.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.six.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.six.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.seven.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.seven.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.eight.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.eight.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.nine.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.nine.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.ten.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.ten.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }
}

/* Tablet Only */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.two.doubling.cards {
    margin-left: 0;
    margin-right: 0;
  }

  .ui.two.doubling.cards > .card {
    width: 100%;
    margin-left: 0;
    margin-right: 0;
  }

  .ui.three.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.three.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.four.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.four.doubling.cards > .card {
    width: calc(50% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.five.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.five.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.six.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.six.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.eight.doubling.cards {
    margin-left: -1em;
    margin-right: -1em;
  }

  .ui.eight.doubling.cards > .card {
    width: calc(33.33333333333333% - 2em);
    margin-left: 1em;
    margin-right: 1em;
  }

  .ui.eight.doubling.cards {
    margin-left: -0.75em;
    margin-right: -0.75em;
  }

  .ui.eight.doubling.cards > .card {
    width: calc(25% - 1.5em);
    margin-left: 0.75em;
    margin-right: 0.75em;
  }

  .ui.nine.doubling.cards {
    margin-left: -0.75em;
    margin-right: -0.75em;
  }

  .ui.nine.doubling.cards > .card {
    width: calc(25% - 1.5em);
    margin-left: 0.75em;
    margin-right: 0.75em;
  }

  .ui.ten.doubling.cards {
    margin-left: -0.75em;
    margin-right: -0.75em;
  }

  .ui.ten.doubling.cards > .card {
    width: calc(20% - 1.5em);
    margin-left: 0.75em;
    margin-right: 0.75em;
  }
}

/* -------------------
          Stackable
    -------------------- */

@media only screen and (max-width: 767.98px) {
  .ui.stackable.cards {
    display: block !important;
  }

  .ui.stackable.cards .card:first-child {
    margin-top: 0 !important;
  }

  .ui.stackable.cards > .card {
    display: block !important;
    height: auto !important;
    margin: 1em 1em;
    padding: 0 !important;
    width: calc(100% - 2em) !important;
  }
}

/* --------------
      Size
--------------- */

.ui.cards > .card {
  font-size: 1em;
}

.ui.mini.card,
.ui.mini.cards .card {
  font-size: 0.78571429rem;
}

.ui.tiny.card,
.ui.tiny.cards .card {
  font-size: 0.85714286rem;
}

.ui.small.card,
.ui.small.cards .card {
  font-size: 0.92857143rem;
}

.ui.large.card,
.ui.large.cards .card {
  font-size: 1.14285714rem;
}

.ui.big.card,
.ui.big.cards .card {
  font-size: 1.28571429rem;
}

.ui.huge.card,
.ui.huge.cards .card {
  font-size: 1.42857143rem;
}

.ui.massive.card,
.ui.massive.cards .card {
  font-size: 1.71428571rem;
}

/* -----------------
          Inverted
    ------------------ */

.ui.inverted.cards > .card,
.ui.inverted.card {
  background: #1b1c1d;
  box-shadow: 0 1px 3px 0 #555, 0 0 0 1px #555;
}

/* Content */

.ui.inverted.cards > .card > .content,
.ui.inverted.card > .content {
  border-top: 1px solid rgba(255, 255, 255, 0.15);
}

/* Header */

.ui.inverted.cards > .card > .content > .header,
.ui.inverted.card > .content > .header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.cards > .card > .content > a.header,
.ui.inverted.card > .content > a.header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.cards > .card > .content > a.header:hover,
.ui.inverted.card > .content > a.header:hover {
  color: #1e70bf;
}

/* Description */

.ui.inverted.cards > .card > .content > .description,
.ui.inverted.card > .content > .description {
  color: rgba(255, 255, 255, 0.8);
}

/* Meta */

.ui.inverted.cards > .card .meta,
.ui.inverted.card .meta {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.cards > .card .meta > a:not(.ui),
.ui.inverted.card .meta > a:not(.ui) {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.cards > .card .meta > a:not(.ui):hover,
.ui.inverted.card .meta > a:not(.ui):hover {
  color: #ffffff;
}

/* Extra */

.ui.inverted.cards > .card > .extra,
.ui.inverted.card > .extra {
  border-top: 1px solid rgba(255, 255, 255, 0.15) !important;
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.cards > .card > .extra a:not(.ui),
.ui.inverted.card > .extra a:not(.ui) {
  color: rgba(255, 255, 255, 0.5);
}

.ui.inverted.cards > .card > .extra a:not(.ui):hover,
.ui.inverted.card > .extra a:not(.ui):hover {
  color: #1e70bf;
}

/* Link card(s) */

.ui.inverted.cards a.card:hover,
.ui.inverted.link.cards .card:not(.icon):hover,
a.inverted.ui.card:hover,
.ui.inverted.link.card:hover {
  background: #1b1c1d;
}

/* Loading */

.ui.inverted.loading.card {
  color: #fff;
}

.ui.inverted.loading.card::before {
  background: rgba(0, 0, 0, 0.85);
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Comment
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Standard
*******************************/

/* --------------
    Comments
--------------- */

.ui.comments {
  margin: 1.5em 0;
  max-width: 650px;
}

.ui.comments:first-child {
  margin-top: 0;
}

.ui.comments:last-child {
  margin-bottom: 0;
}

/* --------------
     Comment
--------------- */

.ui.comments .comment {
  position: relative;
  background: none;
  margin: 0.5em 0 0;
  padding: 0.5em 0 0;
  border: none;
  border-top: none;
  line-height: 1.2;
}

.ui.comments .comment:first-child {
  margin-top: 0;
  padding-top: 0;
}

/* --------------------
    Nested Comments
--------------------- */

.ui.comments .comment > .comments {
  margin: 0 0 0.5em 0.5em;
  padding: 1em 0 1em 1em;
}

.ui.comments .comment > .comments::before {
  position: absolute;
  top: 0;
  left: 0;
}

.ui.comments .comment > .comments .comment {
  border: none;
  border-top: none;
  background: none;
}

/* --------------
         Avatar
    --------------- */

.ui.comments .comment .avatar {
  display: block;
  width: 2.5em;
  height: auto;
  float: left;
  margin: 0.2em 0 0;
}

.ui.comments .comment img.avatar,
.ui.comments .comment .avatar img {
  display: block;
  margin: 0 auto;
  width: 100%;
  height: 100%;
  border-radius: 0.25rem;
}

/* --------------
     Content
--------------- */

.ui.comments .comment > .content {
  display: block;
}

/* If there is an avatar move content over */

.ui.comments .comment > .avatar ~ .content {
  margin-left: 3.5em;
}

/* --------------
         Author
    --------------- */

.ui.comments .comment .author {
  font-size: 1em;
  color: rgba(0, 0, 0, 0.87);
  font-weight: bold;
}

.ui.comments .comment a.author {
  cursor: pointer;
}

.ui.comments .comment a.author:hover {
  color: #1e70bf;
}

/* --------------
         Metadata
    --------------- */

.ui.comments .comment .metadata {
  display: inline-block;
  margin-left: 0.5em;
  color: rgba(0, 0, 0, 0.4);
  font-size: 0.875em;
}

.ui.comments .comment .metadata > * {
  display: inline-block;
  margin: 0 0.5em 0 0;
}

.ui.comments .comment .metadata > :last-child {
  margin-right: 0;
}

/* --------------------
     Comment Text
--------------------- */

.ui.comments .comment .text {
  margin: 0.25em 0 0.5em;
  font-size: 1em;
  word-wrap: break-word;
  color: rgba(0, 0, 0, 0.87);
  line-height: 1.3;
}

/* --------------------
         User Actions
    --------------------- */

.ui.comments .comment .actions {
  font-size: 0.875em;
}

.ui.comments .comment .actions a {
  cursor: pointer;
  display: inline-block;
  margin: 0 0.75em 0 0;
  color: rgba(0, 0, 0, 0.4);
}

.ui.comments .comment .actions a:last-child {
  margin-right: 0;
}

.ui.comments .comment .actions a.active,
.ui.comments .comment .actions a:hover {
  color: rgba(0, 0, 0, 0.8);
}

/* --------------------
          Reply Form
    --------------------- */

.ui.comments > .reply.form {
  margin-top: 1em;
}

.ui.comments .comment .reply.form {
  width: 100%;
  margin-top: 1em;
}

.ui.comments .reply.form textarea {
  font-size: 1em;
  height: 12em;
}

/*******************************
            State
*******************************/

.ui.collapsed.comments,
.ui.comments .collapsed.comments,
.ui.comments .collapsed.comment {
  display: none;
}

/*******************************
           Variations
*******************************/

/* --------------------
            Threaded
    --------------------- */

.ui.threaded.comments .comment > .comments {
  margin: -1.5em 0 -1em 1.25em;
  padding: 3em 0 2em 2.25em;
  box-shadow: -1px 0 0 rgba(34, 36, 38, 0.15);
}

/* --------------------
            Minimal
    --------------------- */

.ui.minimal.comments .comment .actions {
  opacity: 0;
  position: absolute;
  top: 0;
  right: 0;
  left: auto;
  transition: opacity 0.2s ease;
  transition-delay: 0.1s;
}

.ui.minimal.comments .comment > .content:hover > .actions {
  opacity: 1;
}

/* -------------------
        Sizes
-------------------- */

.ui.comments {
  font-size: 1rem;
}

.ui.mini.comments {
  font-size: 0.78571429rem;
}

.ui.tiny.comments {
  font-size: 0.85714286rem;
}

.ui.small.comments {
  font-size: 0.92857143rem;
}

.ui.large.comments {
  font-size: 1.14285714rem;
}

.ui.big.comments {
  font-size: 1.28571429rem;
}

.ui.huge.comments {
  font-size: 1.42857143rem;
}

.ui.massive.comments {
  font-size: 1.71428571rem;
}

/* -------------------
            Inverted
    -------------------- */

.ui.inverted.comments .comment {
  background-color: #1b1c1d;
}

.ui.inverted.comments .comment .author,
.ui.inverted.comments .comment .text {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.comments .comment .metadata,
.ui.inverted.comments .comment .actions a {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.comments .comment a.author:hover,
.ui.inverted.comments .comment .actions a.active,
.ui.inverted.comments .comment .actions a:hover {
  color: #ffffff;
}

.ui.inverted.threaded.comments .comment > .comments {
  box-shadow: -1px 0 0 #555;
}

.ui.disabled.comments,
.ui.comments .disabled.comment {
  opacity: 0.45;
  pointer-events: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Feed
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
         Activity Feed
*******************************/

.ui.feed {
  margin: 1em 0;
}

.ui.feed:first-child {
  margin-top: 0;
}

.ui.feed:last-child {
  margin-bottom: 0;
}

/*******************************
            Content
*******************************/

/* Event */

.ui.feed > .event {
  display: flex;
  flex-direction: row;
  width: 100%;
  padding: 0.21428571rem 0;
  margin: 0;
  background: none;
}

.ui.feed > .event:not(:first-child) {
  border-top: none;
}

.ui.feed:not(.connected) > .event:first-child {
  padding-top: 0;
}

.ui.feed > .event:last-child {
  padding-bottom: 0;
}

/* Event Label */

.ui.feed > .event > .label {
  display: block;
  flex: 0 0 auto;
  width: 2.5em;
  height: auto;
  align-self: stretch;
  text-align: left;
}

.ui.feed > .event > .label .icon {
  opacity: 1;
  font-size: 1.5em;
  width: 100%;
  padding: 0.25em;
  background: none;
  border: none;
  border-radius: 0;
  color: rgba(0, 0, 0, 0.6);
}

.ui.feed > .event > .label img {
  width: 100%;
  height: auto;
  border-radius: 500rem;
}

.ui.feed > .event > .label > .text {
  display: block;
  text-align: center;
  margin-top: 0.5em;
}

.ui.feed > .event > .label > .text.ui {
  margin-top: 0.65em;
}

.ui.feed > .event > .label > .text.multiline {
  margin-top: 0;
}

.ui.feed > .event > .label > .text.multiline.ui {
  margin-top: 0.3em;
}

.ui.feed > .event > .label > .ui.label {
  margin-top: 0.4em;
  position: relative;
  z-index: 1;
}

.ui.feed > .event > .label + .content {
  margin: 0.5em 0 0.35714286em 1.14285714em;
}

/* --------------
     Content
--------------- */

/* Content */

.ui.feed > .event > .content {
  display: block;
  flex: 1 1 auto;
  align-self: stretch;
  text-align: left;
  word-wrap: break-word;
}

.ui.feed > .event:last-child > .content {
  padding-bottom: 0;
}

/* Link */

.ui.feed > .event > .content a {
  cursor: pointer;
}

/* --------------
          Date
    --------------- */

.ui.feed > .event > .content .date {
  margin: -0.5rem 0 0;
  padding: 0;
  color: rgba(0, 0, 0, 0.4);
  font-weight: normal;
  font-size: 1em;
  font-style: normal;
}

/* --------------
         Summary
    --------------- */

.ui.feed > .event > .content .summary {
  margin: 0;
  font-size: 1em;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.87);
}

/* Summary Image */

.ui.feed > .event > .content .summary img {
  display: inline-block;
  width: auto;
  height: 10em;
  margin: -0.25em 0.25em 0 0;
  border-radius: 0.25em;
  vertical-align: middle;
}

/* --------------
           Inline Date
        --------------- */

/* Date inside Summary */

.ui.feed > .event > .content .summary > .date {
  display: inline-block;
  font-weight: normal;
  font-size: 0.85714286em;
  font-style: normal;
  margin: 0 0 0 0.5em;
  padding: 0;
  color: rgba(0, 0, 0, 0.4);
}

.ui.feed > .event > .content .summary > .date:not(.floated) {
  float: none;
}

/* --------------
          User
    --------------- */

.ui.feed > .event > .content .user {
  display: inline-block;
  font-weight: bold;
  margin-right: 0;
  vertical-align: baseline;
}

.ui.feed > .event > .content .user img {
  margin: -0.25em 0.25em 0 0;
  width: auto;
  height: 10em;
  vertical-align: middle;
}

/* --------------
      Extra Summary
    --------------- */

.ui.feed > .event > .content .extra {
  margin: 0.5em 0 0;
  background: none;
  padding: 0;
  color: rgba(0, 0, 0, 0.87);
}

/* Images */

.ui.feed > .event > .content .extra.images img {
  display: inline-block;
  margin: 0 0.25em 0 0;
  width: 6em;
}

/* Text */

.ui.feed > .event > .content .extra.text {
  padding: 0;
  border-left: none;
  font-size: 1em;
  max-width: 500px;
  line-height: 1.4285em;
}

/* --------------
          Meta
    --------------- */

.ui.feed > .event > .content .meta {
  display: inline-block;
  font-size: 0.85714286em;
  margin: 0.5em 0 0;
  background: none;
  border: none;
  border-radius: 0;
  box-shadow: none;
  padding: 0;
  color: rgba(0, 0, 0, 0.6);
}

.ui.feed > .event > .content .meta > * {
  position: relative;
  margin-left: 0.75em;
}

.ui.feed > .event > .content .meta > *::after {
  content: "";
  color: rgba(0, 0, 0, 0.2);
  top: 0;
  left: -1em;
  opacity: 1;
  position: absolute;
  vertical-align: top;
}

.ui.feed > .event > .content .meta .like {
  color: "";
  transition: 0.2s color ease;
}

.ui.feed > .event > .content .meta .like:hover i.icon {
  color: #ff2733;
}

.ui.feed > .event > .content .meta .active.like i.icon {
  color: #ef404a;
}

/* First element */

.ui.feed > .event > .content .meta > :first-child {
  margin-left: 0;
}

.ui.feed > .event > .content .meta > :first-child::after {
  display: none;
}

/* Action */

.ui.feed > .event > .content .meta a,
.ui.feed > .event > .content .meta > i.icon {
  cursor: pointer;
  opacity: 1;
  color: rgba(0, 0, 0, 0.5);
  transition: color 0.1s ease;
}

.ui.feed > .event > .content .meta a:hover,
.ui.feed > .event > .content .meta a:hover i.icon,
.ui.feed > .event > .content .meta > i.icon:hover {
  color: rgba(0, 0, 0, 0.95);
}

/*******************************
            Variations
*******************************/

/* rtl:rename */

.ui.feed .right.floated {
  float: right;
}

.ui.connected.feed > .event {
  position: relative;
}

.ui.connected.feed > .event:not(:last-child)::before {
  border-left: 2px solid rgba(34, 36, 38, 0.15);
  content: "";
  left: 1.2em;
  position: absolute;
  top: 2.1em;
  height: calc(100% - 1.5em);
}

.ui.connected.feed > .event > .label {
  position: relative;
}

.ui.inverted.connected.feed > .event::before {
  border-left-color: rgba(255, 255, 255, 0.1);
}

.ui.divided.feed > .event:not(:first-child) {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.inverted.divided.feed > .event {
  border-top-color: rgba(255, 255, 255, 0.1);
}

.ui.ui.feed > .event > .label[data-text]::before {
  content: attr(data-text);
}

.ui.feed > .event > .label[data-text]::before,
.ui.ordered.feed > .event > .label::before {
  text-align: center;
  line-height: 1;
  height: 2.5em;
  background: #bbbbbb;
  color: #fff;
  border-radius: 50%;
  padding-top: 0.6em;
  position: relative;
  display: block;
  margin-bottom: 0.3em;
  border: 2px solid transparent;
}

.ui.ordered.feed {
  counter-reset: ordered;
}

.ui.ordered.feed > .event > .label::before {
  counter-increment: ordered;
  content: counter(ordered);
}

.ui.ordered.basic.feed > .event > .label::before,
.ui.ordered.feed > .event > .basic.label::before {
  border-color: #bbbbbb;
  color: inherit;
  background: transparent;
}

.ui.ordered.connected.feed > .event::before {
  top: 2.7em;
  height: calc(100% - 2.5em);
}

/* workaround until all browsers support :has() */

.ui.connected.labeled.feed > .event::before,
.ui.connected.feed > .labeled.event::before {
  top: 4.85em;
  height: calc(100% - 4.6499999999999995em);
}

@supports selector(:has(.f)) {
  .ui.connected.feed > .event:has(> .label[data-text] > *)::before,
  .ui.ordered.connected.feed > .event:has(> .label > *)::before {
    top: 4.85em;
    height: calc(100% - 4.6499999999999995em);
  }
}

.ui.feed {
  font-size: 1rem;
}

.ui.mini.feed {
  font-size: 0.78571429rem;
}

.ui.tiny.feed {
  font-size: 0.85714286rem;
}

.ui.small.feed {
  font-size: 0.92857143rem;
}

.ui.large.feed {
  font-size: 1.14285714rem;
}

.ui.big.feed {
  font-size: 1.28571429rem;
}

.ui.huge.feed {
  font-size: 1.42857143rem;
}

.ui.massive.feed {
  font-size: 1.71428571rem;
}

/* ------------------
          Inverted
    ------------------- */

.ui.inverted.feed > .event {
  background: #1b1c1d;
}

.ui.inverted.feed > .event > .label i.icon {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.feed > .event > .label .text:not(.ui) {
  color: #fff;
}

.ui.inverted.feed > .event > .content .date,
.ui.inverted.feed > .event > .content .meta .like {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.feed > .event > .content .summary,
.ui.inverted.feed > .event > .content .extra.text {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.feed > .event > .content .meta .like:hover {
  color: #ffffff;
}

.ui.inverted.feed > .event > .content .meta .active.like i.icon {
  color: rgba(255, 255, 255, 0.7);
}

/* --------------
     Colors
--------------- */

.ui.ui.feed > .event > .primary.label::before,
.ui.primary.feed > .event > .label::before {
  background: #2185d0;
}

.ui.primary.connected.feed > .event::before,
.ui.connected.feed > .primary.event::before {
  border-color: #2185d0;
}

.ui.inverted.primary.connected.feed > .event::before,
.ui.connected.feed > .inverted.primary.event::before {
  border-color: #54c8ff;
}

.ui.ui.feed > .event > .primary.basic.label::before,
.ui.primary.basic.feed > .event > .label::before {
  color: #2185d0;
}

.ui.feed > .event > .inverted.primary.label::before,
.ui.inverted.primary.feed > .event > .label::before {
  background: #54c8ff;
}

.ui.feed > .event > .inverted.primary.basic.label::before,
.ui.inverted.primary.basic.feed > .event > .label::before {
  color: #54c8ff;
}

.ui.ui.feed > .event > .secondary.label::before,
.ui.secondary.feed > .event > .label::before {
  background: #1b1c1d;
}

.ui.secondary.connected.feed > .event::before,
.ui.connected.feed > .secondary.event::before {
  border-color: #1b1c1d;
}

.ui.inverted.secondary.connected.feed > .event::before,
.ui.connected.feed > .inverted.secondary.event::before {
  border-color: #545454;
}

.ui.ui.feed > .event > .secondary.basic.label::before,
.ui.secondary.basic.feed > .event > .label::before {
  color: #1b1c1d;
}

.ui.feed > .event > .inverted.secondary.label::before,
.ui.inverted.secondary.feed > .event > .label::before {
  background: #545454;
}

.ui.feed > .event > .inverted.secondary.basic.label::before,
.ui.inverted.secondary.basic.feed > .event > .label::before {
  color: #545454;
}

.ui.ui.feed > .event > .red.label::before,
.ui.red.feed > .event > .label::before {
  background: #db2828;
}

.ui.red.connected.feed > .event::before,
.ui.connected.feed > .red.event::before {
  border-color: #db2828;
}

.ui.inverted.red.connected.feed > .event::before,
.ui.connected.feed > .inverted.red.event::before {
  border-color: #ff695e;
}

.ui.ui.feed > .event > .red.basic.label::before,
.ui.red.basic.feed > .event > .label::before {
  color: #db2828;
}

.ui.feed > .event > .inverted.red.label::before,
.ui.inverted.red.feed > .event > .label::before {
  background: #ff695e;
}

.ui.feed > .event > .inverted.red.basic.label::before,
.ui.inverted.red.basic.feed > .event > .label::before {
  color: #ff695e;
}

.ui.ui.feed > .event > .orange.label::before,
.ui.orange.feed > .event > .label::before {
  background: #f2711c;
}

.ui.orange.connected.feed > .event::before,
.ui.connected.feed > .orange.event::before {
  border-color: #f2711c;
}

.ui.inverted.orange.connected.feed > .event::before,
.ui.connected.feed > .inverted.orange.event::before {
  border-color: #ff851b;
}

.ui.ui.feed > .event > .orange.basic.label::before,
.ui.orange.basic.feed > .event > .label::before {
  color: #f2711c;
}

.ui.feed > .event > .inverted.orange.label::before,
.ui.inverted.orange.feed > .event > .label::before {
  background: #ff851b;
}

.ui.feed > .event > .inverted.orange.basic.label::before,
.ui.inverted.orange.basic.feed > .event > .label::before {
  color: #ff851b;
}

.ui.ui.feed > .event > .yellow.label::before,
.ui.yellow.feed > .event > .label::before {
  background: #fbbd08;
}

.ui.yellow.connected.feed > .event::before,
.ui.connected.feed > .yellow.event::before {
  border-color: #fbbd08;
}

.ui.inverted.yellow.connected.feed > .event::before,
.ui.connected.feed > .inverted.yellow.event::before {
  border-color: #ffe21f;
}

.ui.ui.feed > .event > .yellow.basic.label::before,
.ui.yellow.basic.feed > .event > .label::before {
  color: #fbbd08;
}

.ui.feed > .event > .inverted.yellow.label::before,
.ui.inverted.yellow.feed > .event > .label::before {
  background: #ffe21f;
}

.ui.feed > .event > .inverted.yellow.basic.label::before,
.ui.inverted.yellow.basic.feed > .event > .label::before {
  color: #ffe21f;
}

.ui.ui.feed > .event > .olive.label::before,
.ui.olive.feed > .event > .label::before {
  background: #b5cc18;
}

.ui.olive.connected.feed > .event::before,
.ui.connected.feed > .olive.event::before {
  border-color: #b5cc18;
}

.ui.inverted.olive.connected.feed > .event::before,
.ui.connected.feed > .inverted.olive.event::before {
  border-color: #d9e778;
}

.ui.ui.feed > .event > .olive.basic.label::before,
.ui.olive.basic.feed > .event > .label::before {
  color: #b5cc18;
}

.ui.feed > .event > .inverted.olive.label::before,
.ui.inverted.olive.feed > .event > .label::before {
  background: #d9e778;
}

.ui.feed > .event > .inverted.olive.basic.label::before,
.ui.inverted.olive.basic.feed > .event > .label::before {
  color: #d9e778;
}

.ui.ui.feed > .event > .green.label::before,
.ui.green.feed > .event > .label::before {
  background: #21ba45;
}

.ui.green.connected.feed > .event::before,
.ui.connected.feed > .green.event::before {
  border-color: #21ba45;
}

.ui.inverted.green.connected.feed > .event::before,
.ui.connected.feed > .inverted.green.event::before {
  border-color: #2ecc40;
}

.ui.ui.feed > .event > .green.basic.label::before,
.ui.green.basic.feed > .event > .label::before {
  color: #21ba45;
}

.ui.feed > .event > .inverted.green.label::before,
.ui.inverted.green.feed > .event > .label::before {
  background: #2ecc40;
}

.ui.feed > .event > .inverted.green.basic.label::before,
.ui.inverted.green.basic.feed > .event > .label::before {
  color: #2ecc40;
}

.ui.ui.feed > .event > .teal.label::before,
.ui.teal.feed > .event > .label::before {
  background: #00b5ad;
}

.ui.teal.connected.feed > .event::before,
.ui.connected.feed > .teal.event::before {
  border-color: #00b5ad;
}

.ui.inverted.teal.connected.feed > .event::before,
.ui.connected.feed > .inverted.teal.event::before {
  border-color: #6dffff;
}

.ui.ui.feed > .event > .teal.basic.label::before,
.ui.teal.basic.feed > .event > .label::before {
  color: #00b5ad;
}

.ui.feed > .event > .inverted.teal.label::before,
.ui.inverted.teal.feed > .event > .label::before {
  background: #6dffff;
}

.ui.feed > .event > .inverted.teal.basic.label::before,
.ui.inverted.teal.basic.feed > .event > .label::before {
  color: #6dffff;
}

.ui.ui.feed > .event > .blue.label::before,
.ui.blue.feed > .event > .label::before {
  background: #2185d0;
}

.ui.blue.connected.feed > .event::before,
.ui.connected.feed > .blue.event::before {
  border-color: #2185d0;
}

.ui.inverted.blue.connected.feed > .event::before,
.ui.connected.feed > .inverted.blue.event::before {
  border-color: #54c8ff;
}

.ui.ui.feed > .event > .blue.basic.label::before,
.ui.blue.basic.feed > .event > .label::before {
  color: #2185d0;
}

.ui.feed > .event > .inverted.blue.label::before,
.ui.inverted.blue.feed > .event > .label::before {
  background: #54c8ff;
}

.ui.feed > .event > .inverted.blue.basic.label::before,
.ui.inverted.blue.basic.feed > .event > .label::before {
  color: #54c8ff;
}

.ui.ui.feed > .event > .violet.label::before,
.ui.violet.feed > .event > .label::before {
  background: #6435c9;
}

.ui.violet.connected.feed > .event::before,
.ui.connected.feed > .violet.event::before {
  border-color: #6435c9;
}

.ui.inverted.violet.connected.feed > .event::before,
.ui.connected.feed > .inverted.violet.event::before {
  border-color: #a291fb;
}

.ui.ui.feed > .event > .violet.basic.label::before,
.ui.violet.basic.feed > .event > .label::before {
  color: #6435c9;
}

.ui.feed > .event > .inverted.violet.label::before,
.ui.inverted.violet.feed > .event > .label::before {
  background: #a291fb;
}

.ui.feed > .event > .inverted.violet.basic.label::before,
.ui.inverted.violet.basic.feed > .event > .label::before {
  color: #a291fb;
}

.ui.ui.feed > .event > .purple.label::before,
.ui.purple.feed > .event > .label::before {
  background: #a333c8;
}

.ui.purple.connected.feed > .event::before,
.ui.connected.feed > .purple.event::before {
  border-color: #a333c8;
}

.ui.inverted.purple.connected.feed > .event::before,
.ui.connected.feed > .inverted.purple.event::before {
  border-color: #dc73ff;
}

.ui.ui.feed > .event > .purple.basic.label::before,
.ui.purple.basic.feed > .event > .label::before {
  color: #a333c8;
}

.ui.feed > .event > .inverted.purple.label::before,
.ui.inverted.purple.feed > .event > .label::before {
  background: #dc73ff;
}

.ui.feed > .event > .inverted.purple.basic.label::before,
.ui.inverted.purple.basic.feed > .event > .label::before {
  color: #dc73ff;
}

.ui.ui.feed > .event > .pink.label::before,
.ui.pink.feed > .event > .label::before {
  background: #e03997;
}

.ui.pink.connected.feed > .event::before,
.ui.connected.feed > .pink.event::before {
  border-color: #e03997;
}

.ui.inverted.pink.connected.feed > .event::before,
.ui.connected.feed > .inverted.pink.event::before {
  border-color: #ff8edf;
}

.ui.ui.feed > .event > .pink.basic.label::before,
.ui.pink.basic.feed > .event > .label::before {
  color: #e03997;
}

.ui.feed > .event > .inverted.pink.label::before,
.ui.inverted.pink.feed > .event > .label::before {
  background: #ff8edf;
}

.ui.feed > .event > .inverted.pink.basic.label::before,
.ui.inverted.pink.basic.feed > .event > .label::before {
  color: #ff8edf;
}

.ui.ui.feed > .event > .brown.label::before,
.ui.brown.feed > .event > .label::before {
  background: #a5673f;
}

.ui.brown.connected.feed > .event::before,
.ui.connected.feed > .brown.event::before {
  border-color: #a5673f;
}

.ui.inverted.brown.connected.feed > .event::before,
.ui.connected.feed > .inverted.brown.event::before {
  border-color: #d67c1c;
}

.ui.ui.feed > .event > .brown.basic.label::before,
.ui.brown.basic.feed > .event > .label::before {
  color: #a5673f;
}

.ui.feed > .event > .inverted.brown.label::before,
.ui.inverted.brown.feed > .event > .label::before {
  background: #d67c1c;
}

.ui.feed > .event > .inverted.brown.basic.label::before,
.ui.inverted.brown.basic.feed > .event > .label::before {
  color: #d67c1c;
}

.ui.ui.feed > .event > .grey.label::before,
.ui.grey.feed > .event > .label::before {
  background: #767676;
}

.ui.grey.connected.feed > .event::before,
.ui.connected.feed > .grey.event::before {
  border-color: #767676;
}

.ui.inverted.grey.connected.feed > .event::before,
.ui.connected.feed > .inverted.grey.event::before {
  border-color: #dcddde;
}

.ui.ui.feed > .event > .grey.basic.label::before,
.ui.grey.basic.feed > .event > .label::before {
  color: #767676;
}

.ui.feed > .event > .inverted.grey.label::before,
.ui.inverted.grey.feed > .event > .label::before {
  background: #dcddde;
}

.ui.feed > .event > .inverted.grey.basic.label::before,
.ui.inverted.grey.basic.feed > .event > .label::before {
  color: #dcddde;
}

.ui.ui.feed > .event > .black.label::before,
.ui.black.feed > .event > .label::before {
  background: #1b1c1d;
}

.ui.black.connected.feed > .event::before,
.ui.connected.feed > .black.event::before {
  border-color: #1b1c1d;
}

.ui.inverted.black.connected.feed > .event::before,
.ui.connected.feed > .inverted.black.event::before {
  border-color: #545454;
}

.ui.ui.feed > .event > .black.basic.label::before,
.ui.black.basic.feed > .event > .label::before {
  color: #1b1c1d;
}

.ui.feed > .event > .inverted.black.label::before,
.ui.inverted.black.feed > .event > .label::before {
  background: #545454;
}

.ui.feed > .event > .inverted.black.basic.label::before,
.ui.inverted.black.basic.feed > .event > .label::before {
  color: #545454;
}

.ui.ui.feed > .event > .basic.label::before,
.ui.ui.ui.basic.feed > .event > .label::before {
  background: transparent;
  border-color: currentColor;
}

.ui.disabled.feed,
.ui.feed .disabled.event {
  opacity: 0.45;
  pointer-events: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Item
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Standard
*******************************/

/* --------------
      Item
--------------- */

.ui.items > .item {
  display: flex;
  margin: 1em 0;
  width: 100%;
  min-height: 0;
  background: transparent;
  padding: 0;
  border: none;
  border-radius: 0;
  box-shadow: none;
  transition: box-shadow 0.1s ease;
  z-index: "";
}

.ui.items > .item a {
  cursor: pointer;
}

/* --------------
      Items
--------------- */

.ui.items {
  margin: 1.5em 0;
}

.ui.items:first-child {
  margin-top: 0 !important;
}

.ui.items:last-child {
  margin-bottom: 0 !important;
}

/* --------------
      Item
--------------- */

.ui.items > .item::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

.ui.items > .item:first-child {
  margin-top: 0;
}

.ui.items > .item:last-child {
  margin-bottom: 0;
}

/* --------------
         Images
    --------------- */

.ui.items > .item > .image {
  position: relative;
  flex: 0 0 auto;
  display: block;
  float: none;
  margin: 0;
  padding: 0;
  max-height: "";
  align-self: start;
}

.ui.items > .item > .image > img {
  display: block;
  width: 100%;
  height: auto;
  border-radius: 0.125rem;
  border: none;
}

.ui.items > .item > .image:only-child > img {
  border-radius: 0;
}

/* --------------
     Content
--------------- */

.ui.items > .item > .content {
  display: block;
  flex: 1 1 auto;
  background: none;
  color: rgba(0, 0, 0, 0.87);
  margin: 0;
  padding: 0;
  box-shadow: none;
  font-size: 1em;
  border: none;
  border-radius: 0;
}

.ui.items > .item > .content::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

.ui.items > .item > .image + .content {
  min-width: 0;
  width: auto;
  display: block;
  margin-left: 0;
  align-self: start;
  padding-left: 1.5em;
}

.ui.items > .item > .content > .header {
  display: inline-block;
  margin: -0.21425em 0 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-weight: bold;
  color: rgba(0, 0, 0, 0.85);
}

/* Default Header Size */

.ui.items > .item > .content > .header:not(.ui) {
  font-size: 1.28571429em;
}

/* --------------
         Floated
    --------------- */

.ui.items > .item [class*="left floated"] {
  float: left;
}

.ui.items > .item [class*="right floated"] {
  float: right;
}

/* --------------
      Content Image
    --------------- */

.ui.items > .item .content img {
  align-self: center;
  width: "";
}

.ui.items > .item img.avatar,
.ui.items > .item .avatar img {
  width: "";
  height: "";
  border-radius: 500rem;
}

/* --------------
       Description
    --------------- */

.ui.items > .item > .content > .description {
  margin-top: 0.6em;
  max-width: none;
  font-size: 1em;
  line-height: 1.4285em;
  color: rgba(0, 0, 0, 0.87);
}

/* --------------
    Paragraph
--------------- */

.ui.items > .item > .content p {
  margin: 0 0 0.5em;
}

.ui.items > .item > .content p:last-child {
  margin-bottom: 0;
}

/* --------------
          Meta
    --------------- */

.ui.items > .item .meta {
  margin: 0.5em 0 0.5em;
  font-size: 1em;
  line-height: 1em;
  color: rgba(0, 0, 0, 0.6);
}

.ui.items > .item .meta * {
  margin-right: 0.3em;
}

.ui.items > .item .meta :last-child {
  margin-right: 0;
}

.ui.items > .item .meta [class*="right floated"] {
  margin-right: 0;
  margin-left: 0.3em;
}

/* --------------
      Links
--------------- */

/* Generic */

.ui.items > .item > .content a:not(.ui) {
  color: "";
  transition: color 0.1s ease;
}

.ui.items > .item > .content a:not(.ui):hover {
  color: "";
}

/* Header */

.ui.items > .item > .content > a.header {
  color: rgba(0, 0, 0, 0.85);
}

.ui.items > .item > .content > a.header:hover {
  color: #1e70bf;
}

/* Meta */

.ui.items > .item .meta > a:not(.ui) {
  color: rgba(0, 0, 0, 0.4);
}

.ui.items > .item .meta > a:not(.ui):hover {
  color: rgba(0, 0, 0, 0.87);
}

/* --------------
     Labels
--------------- */

/* -----Star----- */

/* Icon */

.ui.items > .item > .content .favorite.icon {
  cursor: pointer;
  opacity: 0.75;
  transition: color 0.1s ease;
}

.ui.items > .item > .content .favorite.icon:hover {
  opacity: 1;
  color: #ffb70a;
}

.ui.items > .item > .content .active.favorite.icon {
  color: #ffe623;
}

/* -----Like----- */

/* Icon */

.ui.items > .item > .content .like.icon {
  cursor: pointer;
  opacity: 0.75;
  transition: color 0.1s ease;
}

.ui.items > .item > .content .like.icon:hover {
  opacity: 1;
  color: #ff2733;
}

.ui.items > .item > .content .active.like.icon {
  color: #ff2733;
}

/* ----------------
      Extra Content
    ----------------- */

.ui.items > .item .extra {
  display: block;
  position: relative;
  background: none;
  margin: 0.5rem 0 0;
  width: 100%;
  padding: 0;
  top: 0;
  left: 0;
  color: rgba(0, 0, 0, 0.4);
  box-shadow: none;
  transition: color 0.1s ease;
  border-top: none;
}

.ui.items > .item .extra > * {
  margin: 0.25rem 0.5rem 0.25rem 0;
}

.ui.items > .item .extra > [class*="right floated"] {
  margin: 0.25rem 0 0.25rem 0.5rem;
}

.ui.items > .item .extra::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

/*******************************
          Responsive
*******************************/

/* Default Image Width */

.ui.items > .item > .image:not(.ui) {
  width: 175px;
}

/* Tablet Only */

@media only screen and (min-width: 768px) and (max-width: 991.98px) {
  .ui.items > .item {
    margin: 1em 0;
  }

  .ui.items > .item > .image:not(.ui) {
    width: 150px;
  }

  .ui.items > .item > .image + .content {
    display: block;
    padding: 0 0 0 1em;
  }
}

/* Mobile Only */

@media only screen and (max-width: 767.98px) {
  .ui.items:not(.unstackable) > .item {
    flex-direction: column;
    margin: 2em 0;
  }

  .ui.items:not(.unstackable) > .item > .image {
    display: block;
    margin-left: auto;
    margin-right: auto;
  }

  .ui.items:not(.unstackable) > .item > .image,
  .ui.items:not(.unstackable) > .item > .image > img {
    max-width: 100% !important;
    width: auto !important;
    max-height: 250px !important;
  }

  .ui.items:not(.unstackable) > .item > .image + .content {
    display: block;
    padding: 1.5em 0 0;
  }
}

/*******************************
           Variations
*******************************/

/* -------------------
           Aligned
    -------------------- */

.ui.items > .item > .image + [class*="top aligned"].content {
  align-self: flex-start;
}

.ui.items > .item > .image + [class*="middle aligned"].content {
  align-self: center;
}

.ui.items > .item > .image + [class*="bottom aligned"].content {
  align-self: flex-end;
}

/* --------------
         Relaxed
    --------------- */

.ui.relaxed.items > .item {
  margin: 1.5em 0;
}

.ui[class*="very relaxed"].items > .item {
  margin: 2em 0;
}

/* -------------------
          Divided
    -------------------- */

.ui.divided.items > .item {
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  margin: 0;
  padding: 1em 0;
}

.ui.divided.items > .item:first-child {
  border-top: none;
  margin-top: 0 !important;
  padding-top: 0 !important;
}

.ui.divided.items > .item:last-child {
  margin-bottom: 0 !important;
  padding-bottom: 0 !important;
}

/* Relaxed Divided */

.ui.relaxed.divided.items > .item {
  margin: 0;
  padding: 1.5em 0;
}

.ui[class*="very relaxed"].divided.items > .item {
  margin: 0;
  padding: 2em 0;
}

/* -------------------
            Link
    -------------------- */

.ui.items a.item:hover,
.ui.link.items > .item:hover {
  cursor: pointer;
}

.ui.items a.item:hover .content .header,
.ui.link.items > .item:hover .content .header {
  color: #1e70bf;
}

/* --------------
      Size
--------------- */

.ui.items > .item {
  font-size: 1em;
}

.ui.mini.items > .item {
  font-size: 0.78571429em;
}

.ui.tiny.items > .item {
  font-size: 0.85714286em;
}

.ui.small.items > .item {
  font-size: 0.92857143em;
}

.ui.large.items > .item {
  font-size: 1.14285714em;
}

.ui.big.items > .item {
  font-size: 1.28571429em;
}

.ui.huge.items > .item {
  font-size: 1.42857143em;
}

.ui.massive.items > .item {
  font-size: 1.71428571em;
}

/* ---------------
       Unstackable
    ---------------- */

@media only screen and (max-width: 767.98px) {
  .ui.unstackable.items > .item > .image,
  .ui.unstackable.items > .item > .image > img {
    width: 125px !important;
  }
}

/* --------------
         Inverted
    --------------- */

.ui.inverted.items > .item {
  background: transparent;
}

.ui.inverted.items > .item > .content {
  background: none;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.items > .item .extra {
  background: none;
}

.ui.inverted.items > .item > .content > .header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.items > .item > .content > .description {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.items > .item .meta {
  color: rgba(255, 255, 255, 0.8);
}

.ui.inverted.items > .item > .content a:not(.ui) {
  color: #57a4ef;
}

.ui.inverted.items > .item > .content a:not(.ui):hover {
  color: #4183c4;
}

.ui.inverted.items > .item > .content > a.header {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.items > .item > .content > a.header:hover {
  color: #ffffff;
}

.ui.inverted.items > .item .meta > a:not(.ui) {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.items > .item .meta > a:not(.ui):hover {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.items > .item > .content .favorite.icon:hover {
  color: #ffc63d;
}

.ui.inverted.items > .item > .content .active.favorite.icon {
  color: #ffec56;
}

.ui.inverted.items > .item > .content .like.icon:hover {
  color: #ff5a63;
}

.ui.inverted.items > .item > .content .active.like.icon {
  color: #ff5a63;
}

.ui.inverted.items > .item .extra {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.items a.item:hover .content .header,
.ui.inverted.link.items > .item:hover .content .header {
  color: #ffffff;
}

.ui.inverted.divided.items > .item {
  border-top: 1px solid rgba(255, 255, 255, 0.1);
}

.ui.inverted.divided.items > .item:first-child {
  border-top: none;
}

.ui.disabled.items,
.ui.items > .disabled.item {
  opacity: 0.45;
  pointer-events: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Statistic
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           Statistic
*******************************/

/* Standalone */

.ui.statistic {
  display: inline-flex;
  flex-direction: column;
  margin: 1em 0;
  max-width: none;
}

.ui.statistic + .ui.statistic {
  margin: 0 0 0 1.5em;
}

.ui.statistic:first-child {
  margin-top: 0;
}

.ui.statistic:last-child {
  margin-bottom: 0;
}

/*******************************
            Group
*******************************/

/* Grouped */

.ui.statistics {
  display: flex;
  align-items: flex-start;
  flex-wrap: wrap;
}

.ui.statistics > .statistic {
  display: inline-flex;
  flex: 0 1 auto;
  flex-direction: column;
  margin: 0 1.5em 1em;
  max-width: none;
}

.ui.statistics {
  display: flex;
  margin: 1em -1.5em -1em;
}

/* Clearing */

.ui.statistics::after {
  display: block;
  content: " ";
  height: 0;
  clear: both;
  overflow: hidden;
  visibility: hidden;
}

.ui.statistics:first-child {
  margin-top: 0;
}

/*******************************
            Content
*******************************/

/* --------------
      Value
--------------- */

.ui.statistics .statistic > .value,
.ui.statistic > .value {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 4rem;
  font-weight: normal;
  line-height: 1em;
  color: #1b1c1d;
  text-transform: uppercase;
  text-align: center;
}

/* --------------
     Label
--------------- */

.ui.statistics .statistic > .label,
.ui.statistic > .label {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1em;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.87);
  text-transform: uppercase;
  text-align: center;
}

/* Top Label */

.ui.statistics .statistic > .label ~ .value,
.ui.statistic > .label ~ .value {
  margin-top: 0;
}

/* Bottom Label */

.ui.statistics .statistic > .value ~ .label,
.ui.statistic > .value ~ .label {
  margin-top: 0;
}

/*******************************
             Types
*******************************/

/* --------------
   Icon Value
--------------- */

.ui.statistics .statistic > .value > i.icon,
.ui.statistic > .value > i.icon {
  opacity: 1;
  width: auto;
  margin: 0;
}

/* --------------
   Text Value
--------------- */

.ui.statistics .statistic > .text.value,
.ui.statistic > .text.value {
  line-height: 1em;
  min-height: 2em;
  font-weight: bold;
  text-align: center;
}

.ui.statistics .statistic > .text.value + .label,
.ui.statistic > .text.value + .label {
  text-align: center;
}

/* --------------
   Image Value
--------------- */

.ui.statistics .statistic > .value img,
.ui.statistic > .value img {
  max-height: 3rem;
  vertical-align: baseline;
}

/*******************************
            Variations
*******************************/

/* --------------
      Count
--------------- */

.ui.ten.statistics {
  margin: 0 0 -1em;
}

.ui.ten.statistics .statistic {
  min-width: 10%;
  margin: 0 0 1em;
}

.ui.nine.statistics {
  margin: 0 0 -1em;
}

.ui.nine.statistics .statistic {
  min-width: 11.11111111%;
  margin: 0 0 1em;
}

.ui.eight.statistics {
  margin: 0 0 -1em;
}

.ui.eight.statistics .statistic {
  min-width: 12.5%;
  margin: 0 0 1em;
}

.ui.seven.statistics {
  margin: 0 0 -1em;
}

.ui.seven.statistics .statistic {
  min-width: 14.28571429%;
  margin: 0 0 1em;
}

.ui.six.statistics {
  margin: 0 0 -1em;
}

.ui.six.statistics .statistic {
  min-width: 16.66666667%;
  margin: 0 0 1em;
}

.ui.five.statistics {
  margin: 0 0 -1em;
}

.ui.five.statistics .statistic {
  min-width: 20%;
  margin: 0 0 1em;
}

.ui.four.statistics {
  margin: 0 0 -1em;
}

.ui.four.statistics .statistic {
  min-width: 25%;
  margin: 0 0 1em;
}

.ui.three.statistics {
  margin: 0 0 -1em;
}

.ui.three.statistics .statistic {
  min-width: 33.33333333%;
  margin: 0 0 1em;
}

.ui.two.statistics {
  margin: 0 0 -1em;
}

.ui.two.statistics .statistic {
  min-width: 50%;
  margin: 0 0 1em;
}

.ui.one.statistics {
  margin: 0 0 -1em;
}

.ui.one.statistics .statistic {
  min-width: 100%;
  margin: 0 0 1em;
}

.ui.fluid.statistic {
  width: 100%;
}

/* --------------
       Horizontal
    --------------- */

.ui.horizontal.statistic {
  flex-direction: row;
  align-items: center;
}

.ui.horizontal.statistics {
  flex-direction: column;
  margin: 0;
  max-width: none;
}

.ui.horizontal.statistics .statistic {
  flex-direction: row;
  align-items: center;
  max-width: none;
  margin: 1em 0;
}

.ui.horizontal.statistic > .text.value,
.ui.horizontal.statistics > .statistic > .text.value {
  min-height: 0 !important;
}

.ui.horizontal.statistics .statistic > .value > i.icon,
.ui.horizontal.statistic > .value > i.icon {
  width: 1.18em;
}

.ui.horizontal.statistics .statistic > .value,
.ui.horizontal.statistic > .value {
  display: inline-block;
  vertical-align: middle;
}

.ui.horizontal.statistics .statistic > .label,
.ui.horizontal.statistic > .label {
  display: inline-block;
  vertical-align: middle;
  margin: 0 0 0 0.75em;
}

/* --------------
        Inverted
    --------------- */

.ui.inverted.statistics .statistic > .value,
.ui.inverted.statistic .value {
  color: #fff;
}

.ui.inverted.statistics .statistic > .label,
.ui.inverted.statistic .label {
  color: rgba(255, 255, 255, 0.9);
}

/* --------------
     Colors
--------------- */

.ui.primary.statistics .statistic > .value,
.ui.statistics .primary.statistic > .value,
.ui.primary.statistic > .value {
  color: #2185d0;
}

.ui.inverted.primary.statistics .statistic > .value,
.ui.statistics .inverted.primary.statistic > .value,
.ui.inverted.primary.statistic > .value {
  color: #54c8ff;
}

.ui.secondary.statistics .statistic > .value,
.ui.statistics .secondary.statistic > .value,
.ui.secondary.statistic > .value {
  color: #1b1c1d;
}

.ui.inverted.secondary.statistics .statistic > .value,
.ui.statistics .inverted.secondary.statistic > .value,
.ui.inverted.secondary.statistic > .value {
  color: #545454;
}

.ui.red.statistics .statistic > .value,
.ui.statistics .red.statistic > .value,
.ui.red.statistic > .value {
  color: #db2828;
}

.ui.inverted.red.statistics .statistic > .value,
.ui.statistics .inverted.red.statistic > .value,
.ui.inverted.red.statistic > .value {
  color: #ff695e;
}

.ui.orange.statistics .statistic > .value,
.ui.statistics .orange.statistic > .value,
.ui.orange.statistic > .value {
  color: #f2711c;
}

.ui.inverted.orange.statistics .statistic > .value,
.ui.statistics .inverted.orange.statistic > .value,
.ui.inverted.orange.statistic > .value {
  color: #ff851b;
}

.ui.yellow.statistics .statistic > .value,
.ui.statistics .yellow.statistic > .value,
.ui.yellow.statistic > .value {
  color: #fbbd08;
}

.ui.inverted.yellow.statistics .statistic > .value,
.ui.statistics .inverted.yellow.statistic > .value,
.ui.inverted.yellow.statistic > .value {
  color: #ffe21f;
}

.ui.olive.statistics .statistic > .value,
.ui.statistics .olive.statistic > .value,
.ui.olive.statistic > .value {
  color: #b5cc18;
}

.ui.inverted.olive.statistics .statistic > .value,
.ui.statistics .inverted.olive.statistic > .value,
.ui.inverted.olive.statistic > .value {
  color: #d9e778;
}

.ui.green.statistics .statistic > .value,
.ui.statistics .green.statistic > .value,
.ui.green.statistic > .value {
  color: #21ba45;
}

.ui.inverted.green.statistics .statistic > .value,
.ui.statistics .inverted.green.statistic > .value,
.ui.inverted.green.statistic > .value {
  color: #2ecc40;
}

.ui.teal.statistics .statistic > .value,
.ui.statistics .teal.statistic > .value,
.ui.teal.statistic > .value {
  color: #00b5ad;
}

.ui.inverted.teal.statistics .statistic > .value,
.ui.statistics .inverted.teal.statistic > .value,
.ui.inverted.teal.statistic > .value {
  color: #6dffff;
}

.ui.blue.statistics .statistic > .value,
.ui.statistics .blue.statistic > .value,
.ui.blue.statistic > .value {
  color: #2185d0;
}

.ui.inverted.blue.statistics .statistic > .value,
.ui.statistics .inverted.blue.statistic > .value,
.ui.inverted.blue.statistic > .value {
  color: #54c8ff;
}

.ui.violet.statistics .statistic > .value,
.ui.statistics .violet.statistic > .value,
.ui.violet.statistic > .value {
  color: #6435c9;
}

.ui.inverted.violet.statistics .statistic > .value,
.ui.statistics .inverted.violet.statistic > .value,
.ui.inverted.violet.statistic > .value {
  color: #a291fb;
}

.ui.purple.statistics .statistic > .value,
.ui.statistics .purple.statistic > .value,
.ui.purple.statistic > .value {
  color: #a333c8;
}

.ui.inverted.purple.statistics .statistic > .value,
.ui.statistics .inverted.purple.statistic > .value,
.ui.inverted.purple.statistic > .value {
  color: #dc73ff;
}

.ui.pink.statistics .statistic > .value,
.ui.statistics .pink.statistic > .value,
.ui.pink.statistic > .value {
  color: #e03997;
}

.ui.inverted.pink.statistics .statistic > .value,
.ui.statistics .inverted.pink.statistic > .value,
.ui.inverted.pink.statistic > .value {
  color: #ff8edf;
}

.ui.brown.statistics .statistic > .value,
.ui.statistics .brown.statistic > .value,
.ui.brown.statistic > .value {
  color: #a5673f;
}

.ui.inverted.brown.statistics .statistic > .value,
.ui.statistics .inverted.brown.statistic > .value,
.ui.inverted.brown.statistic > .value {
  color: #d67c1c;
}

.ui.grey.statistics .statistic > .value,
.ui.statistics .grey.statistic > .value,
.ui.grey.statistic > .value {
  color: #767676;
}

.ui.inverted.grey.statistics .statistic > .value,
.ui.statistics .inverted.grey.statistic > .value,
.ui.inverted.grey.statistic > .value {
  color: #dcddde;
}

.ui.black.statistics .statistic > .value,
.ui.statistics .black.statistic > .value,
.ui.black.statistic > .value {
  color: #1b1c1d;
}

.ui.inverted.black.statistics .statistic > .value,
.ui.statistics .inverted.black.statistic > .value,
.ui.inverted.black.statistic > .value {
  color: #545454;
}

/* --------------
        Floated
    --------------- */

.ui[class*="left floated"].statistic {
  float: left;
  margin: 0 2em 1em 0;
}

.ui[class*="right floated"].statistic {
  float: right;
  margin: 0 0 1em 2em;
}

.ui.floated.statistic:last-child {
  margin-bottom: 0;
}

/* --------------
        Stackable
    --------------- */

@media only screen and (max-width: 767.98px) {
  .ui.stackable.statistics {
    width: auto;
    margin-left: 0 !important;
    margin-right: 0 !important;
  }

  .ui.stackable.statistics > .statistic {
    width: 100% !important;
    margin: 0 !important;
    padding: 1rem 1rem !important;
  }
}

/* --------------
     Sizes
--------------- */

/* Medium */

.ui.statistics .statistic > .value,
.ui.statistic > .value {
  font-size: 4rem;
}

.ui.horizontal.statistics .statistic > .value,
.ui.horizontal.statistic > .value {
  font-size: 3rem;
}

.ui.statistics .statistic > .text.value,
.ui.statistic > .text.value {
  font-size: 2rem;
}

.ui.mini.statistics .statistic > .value,
.ui.mini.statistic > .value {
  font-size: 1.5rem;
}

.ui.mini.horizontal.statistics .statistic > .value,
.ui.mini.horizontal.statistic > .value {
  font-size: 1.5rem;
}

.ui.mini.statistics .statistic > .text.value,
.ui.mini.statistic > .text.value {
  font-size: 1rem;
}

.ui.tiny.statistics .statistic > .value,
.ui.tiny.statistic > .value {
  font-size: 2rem;
}

.ui.tiny.horizontal.statistics .statistic > .value,
.ui.tiny.horizontal.statistic > .value {
  font-size: 2rem;
}

.ui.tiny.statistics .statistic > .text.value,
.ui.tiny.statistic > .text.value {
  font-size: 1rem;
}

.ui.small.statistics .statistic > .value,
.ui.small.statistic > .value {
  font-size: 3rem;
}

.ui.small.horizontal.statistics .statistic > .value,
.ui.small.horizontal.statistic > .value {
  font-size: 2rem;
}

.ui.small.statistics .statistic > .text.value,
.ui.small.statistic > .text.value {
  font-size: 1rem;
}

.ui.large.statistics .statistic > .value,
.ui.large.statistic > .value {
  font-size: 5rem;
}

.ui.large.horizontal.statistics .statistic > .value,
.ui.large.horizontal.statistic > .value {
  font-size: 4rem;
}

.ui.large.statistics .statistic > .text.value,
.ui.large.statistic > .text.value {
  font-size: 2.5rem;
}

.ui.big.statistics .statistic > .value,
.ui.big.statistic > .value {
  font-size: 5.5rem;
}

.ui.big.horizontal.statistics .statistic > .value,
.ui.big.horizontal.statistic > .value {
  font-size: 4.5rem;
}

.ui.big.statistics .statistic > .text.value,
.ui.big.statistic > .text.value {
  font-size: 2.5rem;
}

.ui.huge.statistics .statistic > .value,
.ui.huge.statistic > .value {
  font-size: 6rem;
}

.ui.huge.horizontal.statistics .statistic > .value,
.ui.huge.horizontal.statistic > .value {
  font-size: 5rem;
}

.ui.huge.statistics .statistic > .text.value,
.ui.huge.statistic > .text.value {
  font-size: 2.5rem;
}

.ui.massive.statistics .statistic > .value,
.ui.massive.statistic > .value {
  font-size: 7rem;
}

.ui.massive.horizontal.statistics .statistic > .value,
.ui.massive.horizontal.statistic > .value {
  font-size: 6rem;
}

.ui.massive.statistics .statistic > .text.value,
.ui.massive.statistic > .text.value {
  font-size: 3rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Accordion
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Accordion
*******************************/

.ui.accordion,
.ui.accordion .accordion {
  max-width: 100%;
}

.ui.accordion .accordion {
  margin: 1em 0 0;
  padding: 0;
}

/* Title */

.ui.accordion.menu .item > .title,
.ui.accordion > .title,
.ui.accordion .accordion > .title {
  cursor: pointer;
  padding: 0.5em 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1em;
  color: rgba(0, 0, 0, 0.87);
  list-style: none;
  line-height: 1;
}

/* Default Styling */

.ui.accordion:not(.styled) .title ~ .content,
.ui.accordion:not(.styled) .accordion .title ~ .content {
  margin: "";
  padding: 0.5em 0 1em;
}

.ui.accordion:not(.styled) .title ~ .content:last-child {
  padding-bottom: 0;
}

/* Arrow */

.ui.accordion .ui.header > .dropdown.icon,
.ui.accordion .title > .dropdown.icon,
.ui.accordion .accordion .title > .dropdown.icon {
  display: inline-block;
  float: none;
  opacity: 1;
  width: 1.25em;
  height: 1em;
  margin: 0 0.25rem 0 0;
  padding: 0;
  font-size: 1em;
  transition: transform 0.1s ease, opacity 0.1s ease;
  vertical-align: baseline;
  transform: none;
}

.ui.accordion .ui.header > .dropdown.icon.right,
.ui.accordion .title > .dropdown.icon.right,
.ui.accordion .accordion .title > .dropdown.icon.right {
  float: right;
  transform: rotate(180deg);
}

/* --------------
    Coupling
--------------- */

/* Menu */

.ui.accordion.menu .item .title {
  display: block;
  padding: 0;
}

.ui.accordion.menu .item .title > .dropdown.icon {
  float: right;
  margin: 0 0.25rem 0 0;
  transform: rotate(180deg);
}

/* Header */

.ui.accordion .ui.header > .dropdown.icon {
  font-size: 1em;
  margin: 0 0.25rem 0 0;
}

/*******************************
            States
*******************************/

.ui.accordion[open] > .title > .dropdown.icon,
.ui.accordion .accordion[open] > .title > .dropdown.icon,
.ui.accordion .active.title > .dropdown.icon,
.ui.accordion .accordion .active.title > .dropdown.icon {
  transform: rotate(90deg);
}

.ui.accordion.menu .item .active.title > .dropdown.icon {
  transform: rotate(90deg);
}

/*******************************
            Types
*******************************/

/* --------------
         Styled
    --------------- */

.ui.styled.accordion {
  width: 600px;
}

.ui.styled.accordion,
.ui.styled.accordion .accordion {
  border-radius: 0.28571429rem;
  background: #fff;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15);
}

.ui.styled.accordion > .title,
.ui.styled.accordion .accordion > .title {
  margin: 0;
  padding: 0.75em 1em;
  color: rgba(0, 0, 0, 0.4);
  font-weight: bold;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  transition: background 0.1s ease, color 0.1s ease;
}

.ui.styled.accordion > .title:first-child,
.ui.styled.accordion .accordion > .title:first-child {
  border-top: none;
}

/* Content */

.ui.styled.accordion > .content {
  margin: 0;
  padding: 0.5em 1em 1.5em;
}

.ui.styled.accordion .accordion > .content {
  margin: 0;
  padding: 0.5em 1em 1.5em;
}

/* Hover */

.ui.styled.accordion > .title:hover {
  background: transparent;
  color: rgba(0, 0, 0, 0.87);
}

.ui.styled.accordion .accordion > .title:hover {
  background: transparent;
  color: rgba(0, 0, 0, 0.87);
}

/* Active */

.ui.styled.accordion[open] > .title,
.ui.styled.accordion .active.title {
  background: transparent;
  color: rgba(0, 0, 0, 0.95);
}

.ui.styled.accordion .accordion[open] > .title,
.ui.styled.accordion .accordion .active.title {
  background: transparent;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
         Compact
    --------------- */

/* Default Styling */

.ui.compact.accordion:not(.styled) > .title,
.ui.compact.accordion:not(.styled) .accordion > .title {
  padding: 0.25em 0;
}

.ui.compact.accordion:not(.styled) .title ~ .content,
.ui.compact.accordion:not(.styled) .accordion .title ~ .content {
  padding: 0.25em 0 0.5em;
}

/* Styled */

.ui.compact.styled.accordion > .title,
.ui.compact.styled.accordion .accordion > .title {
  padding: 0.375em 0.5em;
}

.ui.compact.styled.accordion .title ~ .content,
.ui.compact.styled.accordion .accordion .title ~ .content {
  padding: 0.25em 0.5em 0.75em;
}

/* --------------
    Very Compact
  --------------- */

.ui[class*="very compact"].accordion:not(.styled) > .title,
.ui[class*="very compact"].accordion:not(.styled) .accordion > .title {
  padding: 0.125em 0;
}

.ui[class*="very compact"].accordion:not(.styled) .title ~ .content,
.ui[class*="very compact"].accordion:not(.styled) .accordion .title ~ .content {
  padding: 0.125em 0 0.25em;
}

.ui[class*="very compact"].styled.accordion > .title,
.ui[class*="very compact"].styled.accordion .accordion > .title {
  padding: 0.1875em 0.25em;
}

.ui[class*="very compact"].styled.accordion .title ~ .content,
.ui[class*="very compact"].styled.accordion .accordion .title ~ .content {
  padding: 0.125em 0.25em 0.375em;
}

/*******************************
            States
*******************************/

/* --------------
   Not Active
--------------- */

.ui.accordion:not(details) .title ~ .content:not(.active),
.ui.accordion .accordion:not(details) .title ~ .content:not(.active) {
  display: none;
}

/*******************************
           Variations
*******************************/

/* --------------
         Fluid
    --------------- */

.ui.fluid.accordion,
.ui.fluid.accordion .accordion {
  width: 100%;
}

/* --------------
         Inverted
    --------------- */

.ui.inverted.accordion.menu .item > .title,
.ui.inverted.accordion > .title,
.ui.inverted.accordion .accordion > .title {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.styled.accordion,
.ui.inverted.styled.accordion .accordion {
  background: #1b1c1d;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(255, 255, 255, 0.1);
}

.ui.inverted.styled.accordion > .title,
.ui.inverted.styled.accordion .accordion > .title {
  color: rgba(255, 255, 255, 0.5);
  border-top: 1px solid rgba(255, 255, 255, 0.1);
}

/* Hover */

.ui.inverted.styled.accordion > .title:hover {
  background: transparent;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.styled.accordion .accordion > .title:hover {
  background: transparent;
  color: rgba(255, 255, 255, 0.9);
}

/* Active */

.ui.inverted.styled.accordion[open] > .title,
.ui.inverted.styled.accordion .active.title {
  background: transparent;
  color: #ffffff;
}

.ui.inverted.styled.accordion .accordion[open] > .title,
.ui.inverted.styled.accordion .accordion .active.title {
  background: transparent;
  color: #ffffff;
}

.ui.basic.styled.accordion,
.ui.basic.styled.accordion .accordion {
  background: transparent;
  box-shadow: none;
}

.ui.basic.styled.accordion > .title,
.ui.basic.styled.accordion .accordion > .title {
  border: none;
  color: rgba(0, 0, 0, 0.6);
}

.ui.basic.styled.accordion > .title:hover,
.ui.basic.styled.accordion .accordion > .title:hover {
  background: transparent;
  color: rgba(0, 0, 0, 0.87);
}

.ui.basic.styled.accordion[open] > .title,
.ui.basic.styled.accordion .active.title,
.ui.basic.styled.accordion .accordion[open] > .title,
.ui.basic.styled.accordion .accordion .active.title {
  background: transparent;
  color: rgba(0, 0, 0, 0.95);
}

.ui.inverted.basic.styled.accordion > .title,
.ui.inverted.basic.styled.accordion .accordion > .title {
  background: transparent;
  color: rgba(255, 255, 255, 0.8);
}

.ui.inverted.basic.styled.accordion > .title:hover,
.ui.inverted.basic.styled.accordion .accordion > .title:hover {
  background: transparent;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.basic.styled.accordion[open] > .title,
.ui.inverted.basic.styled.accordion .active.title,
.ui.inverted.basic.styled.accordion .accordion[open] > .title,
.ui.inverted.basic.styled.accordion .accordion .active.title {
  background: transparent;
  color: #ffffff;
}

.ui.tree.accordion:not(.styled) .title ~ .content,
.ui.tree.accordion:not(.styled) .accordion .title ~ .content {
  padding: 0;
}

.ui.tree.accordion > .content,
.ui.tree.accordion .accordion > .content {
  margin-left: 1.7em;
}

.ui.tree.accordion .accordion {
  margin-top: 0;
}

/*******************************
         Theme Overrides
*******************************/

@font-face {
  font-family: Accordion;
  src: url("data:application/font-woff;base64,d09GRk9UVE8AAAREAAsAAAAABkAAAQAAAAAAAAAAAAAAAAAAAAAAAAAAAABDRkYgAAAC7AAAAQoAAAE/j/fxwUZGVE0AAAQUAAAAHAAAAByNqvxQR0RFRgAAA/gAAAAcAAAAIAAnABhPUy8yAAABYAAAAEIAAABgTadW5mNtYXAAAAKMAAAAUQAAAWAFZuKDaGVhZAAAAQgAAAAyAAAANg8zT8toaGVhAAABPAAAABoAAAAkApkA5mhtdHgAAAQwAAAAEwAAABYBtwASbWF4cAAAAVgAAAAGAAAABgAGUABuYW1lAAABpAAAAOYAAAGq/HAhWXBvc3QAAALgAAAADAAAACAAAwAAeJxjYGRgYADi8KWRsvH8Nl8ZuJkYQOD838XvQPT9hGxNIOXJsJ0xD0hzMIClARzsCcoAAHicY2BkYGB88P8BkGQAg+0wBhywAgBgjgOAAAAAAFAAAAYAAHicY2BmOM84gYGVgYHRhzGNgYHBHUp/ZZBkaGFABYwCSJyANNcUBgcGhg+3GB/8fwCUfMCgAFIDV6AAhIwARVcMJwAAeJx1jrFOwzAURU/SNKgFIVgQ6uSRKWolZjaQOnQBqQNb2lpRpMqW3HTowhew8U9I/BU37WOjtmyfd/3utYEbvsn4G7lxRsmtcc4FD8YD6U/GhfjNeMglybjU/DQeM+FHrqwY2QsnzrgSnTjnmnvjgfSpcSF+Nh5yx7txKf3DeMwjX31soqajJdCoqLs26HzFq9yz1WVS6Zv9tk5nmv8Vl0pI7CRHXThmVPofS592bQxuVk3PGF+Ohk6WRhHhGNO3eTbSVhy0z1mra6HV91YyxdC5xgevFL9xq4Obr+MixlDxC/HTOrsAAHicY2BgYGaAYBkGRgYQiALyGMF8FgY7IM3FwMHABISMDAofbv3/+/8/WJXCh5tgNuP/x2A+RC8bUC0DUB8rWIgNiJkY8AJm/NJDHgAALqoOxgAAAHicY2BmwAsAAH0ABHicPY7BSsNAEIZnkk2zlhg1JKAYxOrJi6RepMdC8SKefAJP4kVBzCtkNw7ZjeK1j9C38Rn6CO5CDrppigwMM8P/f/8gMAaIGL4+vD09PwJ6gLCwp2AnaM88e+7bQ0aRTxE7GcPRffJL9D9EIX2bW+sFOXzu5QD7OSwPcuA5Xiew05N8CIDDLiSQuTqGSVmU0/KqKG+Kxaxv8+llMURvHwBAgRJrfEeCEaLns9HdRxzHP/FqaV4yaptWK25SkwZKu4Va0rUSWjYVVbz7CqmqhZBSiNodqGqk5l2vFqrW1PJ4tWbGqZxVKa31htA6gmN2aTBwBobzuYsU1YYkGheyZVBqLjKKxn8cSmvuAAB4nGNgZGBg4AFiAQYJBiYgzQLEIJoRggEFbQBAAAAAAQAAAADeTO04AAAAAM/9o+4AAAAA32BrKXicY2BAAEYQsZ1BiIEBAAPpAMsA") format("woff");
  font-weight: normal;
  font-style: normal;
}

/* Dropdown Icon */

.ui.accordion .ui.header > .dropdown.icon,
.ui.accordion .title > .dropdown.icon,
.ui.accordion .accordion .title > .dropdown.icon {
  font-family: Accordion;
  line-height: 1;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  font-weight: normal;
  font-style: normal;
  text-align: center;
}

.ui.accordion .ui.header > .dropdown.icon::before,
.ui.accordion .title > .dropdown.icon::before,
.ui.accordion .accordion .title > .dropdown.icon::before {
  content: "\f0da"
;
}

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Calendar
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Popup
*******************************/

.ui.calendar.popup {
  max-width: none;
  padding: 0;
  border: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.ui.calendar.popup.inverted::before {
  background: #2b2b2b;
}

/*******************************
            Calendar
*******************************/

.ui.calendar .calendar:focus,
.ui.calendar.popup:focus {
  outline: 0;
}

/*******************************
            Grid
*******************************/

.ui.calendar.popup .ui.grid {
  display: block;
  white-space: nowrap;
}

.ui.calendar.popup .ui.grid > .column {
  width: auto;
}

/*******************************
            Table
*******************************/

.ui.calendar .ui.table.year,
.ui.calendar .ui.table.month,
.ui.calendar .ui.table.minute {
  min-width: 15em;
}

.ui.calendar .ui.table.day {
  min-width: 18em;
}

.ui.calendar .ui.table.day.andweek {
  min-width: 22em;
}

.ui.calendar .ui.table.hour {
  min-width: 20em;
}

.ui.calendar .ui.table tr th,
.ui.calendar .ui.table tr td {
  padding: 0.5em;
  white-space: nowrap;
}

.ui.calendar .ui.table tr th {
  border-left: none;
}

.ui.calendar .ui.table tr th i.icon {
  margin: 0;
}

.ui.calendar .ui.table tr:first-child th {
  position: relative;
  padding-left: 0;
  padding-right: 0;
}

.ui.calendar .ui.table.day tr:first-child th {
  border: none;
}

.ui.calendar .ui.table.day tr:nth-child(2) th {
  padding-top: 0.2em;
  padding-bottom: 0.3em;
}

.ui.calendar .ui.table tr td {
  padding-left: 0.1em;
  padding-right: 0.1em;
}

.ui.calendar .ui.table tr .link {
  cursor: pointer;
}

.ui.calendar .ui.table tr .prev.link {
  width: 14.28571429%;
  position: absolute;
  left: 0;
}

.ui.calendar .ui.table tr .next.link {
  width: 14.28571429%;
  position: absolute;
  right: 0;
}

.ui.ui.calendar .ui.table tr .disabled {
  pointer-events: auto;
  cursor: default;
  color: rgba(40, 40, 40, 0.3);
}

.ui.calendar .ui.table tr .adjacent:not(.disabled):not(.active) {
  color: rgba(0, 0, 0, 0.6);
  background: rgba(0, 0, 0, 0.03);
}

/* --------------
     States
--------------- */

.ui.calendar .ui.table tr td.today {
  font-weight: bold;
}

.ui.calendar .ui.table tr td.range {
  background: rgba(0, 0, 0, 0.05);
  color: rgba(0, 0, 0, 0.95);
  box-shadow: none;
}

.ui.calendar:not(.disabled):focus .ui.table tbody tr td.focus,
.ui.calendar:not(.disabled).popup.active .ui.table tbody tr td.focus {
  box-shadow: inset 0 0 0 1px #85b7d9;
}

.ui.inverted.calendar .ui.table.inverted tr td.range {
  background: rgba(255, 255, 255, 0.08);
  color: #ffffff;
  box-shadow: none;
}

.ui.inverted.calendar:not(.disabled) .calendar:focus .ui.table.inverted tbody tr td.focus,
.ui.inverted.calendar:not(.disabled).popup.active .ui.table.inverted tbody tr td.focus {
  box-shadow: inset 0 0 0 1px #85b7d9;
}

.ui.inverted.calendar .ui.ui.ui.inverted.table tr .disabled {
  color: rgba(225, 225, 225, 0.3);
}

.ui.inverted.calendar .ui.inverted.table tr .adjacent:not(.disabled):not(.active) {
  color: rgba(255, 255, 255, 0.8);
  background: rgba(255, 255, 255, 0.02);
}

.ui.ui.ui.calendar.popup > .ui.ui.grid {
  margin: -1rem;
}

.ui.ui.ui.calendar.popup > .ui.ui.grid > .column:not(:first-child) {
  padding-left: 0;
}

.ui.ui.ui.calendar.popup > .ui.ui.grid > .column:not(:first-child) > .ui.table {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui.ui.ui.calendar.popup > .ui.ui.grid > .column:not(:last-child) {
  padding-right: 0;
}

.ui.ui.ui.calendar.popup > .ui.ui.grid > .column:not(:last-child) > .ui.table {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

/* --------------------
        Sizes
--------------------- */

.ui.mini.calendar,
.ui.mini.calendar .ui.table tr th,
.ui.mini.calendar .ui.table tr td {
  font-size: 0.78571429rem;
}

.ui.mini.calendar .ui.table.year,
.ui.mini.calendar .ui.table.month,
.ui.mini.calendar .ui.table.minute {
  min-width: 11.78571429em;
}

.ui.mini.calendar .ui.table.day {
  min-width: 14.14285714em;
}

.ui.mini.calendar .ui.table.day.andweek {
  min-width: 17.28571429em;
}

.ui.mini.calendar .ui.table.hour {
  min-width: 15.71428571em;
}

.ui.tiny.calendar,
.ui.tiny.calendar .ui.table tr th,
.ui.tiny.calendar .ui.table tr td {
  font-size: 0.85714286rem;
}

.ui.tiny.calendar .ui.table.year,
.ui.tiny.calendar .ui.table.month,
.ui.tiny.calendar .ui.table.minute {
  min-width: 12.85714286em;
}

.ui.tiny.calendar .ui.table.day {
  min-width: 15.42857143em;
}

.ui.tiny.calendar .ui.table.day.andweek {
  min-width: 18.85714286em;
}

.ui.tiny.calendar .ui.table.hour {
  min-width: 17.14285714em;
}

.ui.small.calendar,
.ui.small.calendar .ui.table tr th,
.ui.small.calendar .ui.table tr td {
  font-size: 0.92857143rem;
}

.ui.small.calendar .ui.table.year,
.ui.small.calendar .ui.table.month,
.ui.small.calendar .ui.table.minute {
  min-width: 13.92857143em;
}

.ui.small.calendar .ui.table.day {
  min-width: 16.71428571em;
}

.ui.small.calendar .ui.table.day.andweek {
  min-width: 20.42857143em;
}

.ui.small.calendar .ui.table.hour {
  min-width: 18.57142857em;
}

.ui.large.calendar,
.ui.large.calendar .ui.table tr th,
.ui.large.calendar .ui.table tr td {
  font-size: 1.14285714rem;
}

.ui.large.calendar .ui.table.year,
.ui.large.calendar .ui.table.month,
.ui.large.calendar .ui.table.minute {
  min-width: 17.14285714em;
}

.ui.large.calendar .ui.table.day {
  min-width: 20.57142857em;
}

.ui.large.calendar .ui.table.day.andweek {
  min-width: 25.14285714em;
}

.ui.large.calendar .ui.table.hour {
  min-width: 22.85714286em;
}

.ui.big.calendar,
.ui.big.calendar .ui.table tr th,
.ui.big.calendar .ui.table tr td {
  font-size: 1.28571429rem;
}

.ui.big.calendar .ui.table.year,
.ui.big.calendar .ui.table.month,
.ui.big.calendar .ui.table.minute {
  min-width: 19.28571429em;
}

.ui.big.calendar .ui.table.day {
  min-width: 23.14285714em;
}

.ui.big.calendar .ui.table.day.andweek {
  min-width: 28.28571429em;
}

.ui.big.calendar .ui.table.hour {
  min-width: 25.71428571em;
}

.ui.huge.calendar,
.ui.huge.calendar .ui.table tr th,
.ui.huge.calendar .ui.table tr td {
  font-size: 1.42857143rem;
}

.ui.huge.calendar .ui.table.year,
.ui.huge.calendar .ui.table.month,
.ui.huge.calendar .ui.table.minute {
  min-width: 21.42857143em;
}

.ui.huge.calendar .ui.table.day {
  min-width: 25.71428571em;
}

.ui.huge.calendar .ui.table.day.andweek {
  min-width: 31.42857143em;
}

.ui.huge.calendar .ui.table.hour {
  min-width: 28.57142857em;
}

.ui.massive.calendar,
.ui.massive.calendar .ui.table tr th,
.ui.massive.calendar .ui.table tr td {
  font-size: 1.71428571rem;
}

.ui.massive.calendar .ui.table.year,
.ui.massive.calendar .ui.table.month,
.ui.massive.calendar .ui.table.minute {
  min-width: 25.71428571em;
}

.ui.massive.calendar .ui.table.day {
  min-width: 30.85714286em;
}

.ui.massive.calendar .ui.table.day.andweek {
  min-width: 37.71428571em;
}

.ui.massive.calendar .ui.table.hour {
  min-width: 34.28571429em;
}

/*******************************
            States
*******************************/

/* --------------------
            Disabled
    --------------------- */

.ui.disabled.calendar {
  opacity: 0.45;
}

.ui.disabled.calendar > .input,
.ui.disabled.calendar .ui.table tr .link {
  pointer-events: none;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Checkbox
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           Checkbox
*******************************/

/* --------------
    Content
--------------- */

.ui.checkbox {
  position: relative;
  display: inline-block;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  outline: none;
  vertical-align: baseline;
  font-style: normal;
  min-height: 17px;
  font-size: 1em;
  line-height: 17px;
  min-width: 17px;
}

/* HTML Checkbox */

.ui.checkbox input[type="checkbox"],
.ui.checkbox input[type="radio"] {
  cursor: pointer;
  position: absolute;
  top: 0;
  left: 0;
  opacity: 0 !important;
  outline: none;
  z-index: 3;
  width: 17px;
  height: 17px;
}

.ui.checkbox label {
  cursor: auto;
  position: relative;
  display: block;
  outline: none;
  font-size: 1em;
}

.ui.checkbox:not(.invisible) label {
  padding-left: 1.85714em;
}

.ui.checkbox label::before {
  position: absolute;
  top: 0;
  left: 0;
  width: 17px;
  height: 17px;
  content: "";
  background: #fff;
  border-radius: 0.21428571rem;
  transition: border 0.1s ease, opacity 0.1s ease, transform 0.1s ease, box-shadow 0.1s ease;
  border: 1px solid #d4d4d5;
}

/* --------------
    Checkmark
--------------- */

.ui.checkbox label::after {
  position: absolute;
  font-size: 14px;
  top: 0;
  left: 0;
  width: 17px;
  height: 17px;
  text-align: center;
  opacity: 0;
  color: rgba(0, 0, 0, 0.87);
  transition: border 0.1s ease, opacity 0.1s ease, transform 0.1s ease, box-shadow 0.1s ease;
}

.ui.right.aligned.checkbox label {
  padding-left: 0;
  padding-right: 1.85714em;
}

.ui.right.aligned.checkbox label::after,
.ui.right.aligned.checkbox label::before {
  right: 0;
  left: auto;
}

/* --------------
      Label
--------------- */

/* Inside */

.ui.checkbox label,
.ui.checkbox + label {
  color: rgba(0, 0, 0, 0.87);
  transition: color 0.1s ease;
}

/* Outside */

.ui.checkbox + label {
  vertical-align: middle;
}

/*******************************
           States
*******************************/

/* --------------
      Hover
--------------- */

.ui.checkbox label:hover::before {
  background: #fff;
  border-color: rgba(34, 36, 38, 0.35);
}

.ui.checkbox:not(.invisible) label:hover,
.ui.checkbox:not(.invisible) + label:hover {
  color: rgba(0, 0, 0, 0.8);
}

/* --------------
      Down
--------------- */

.ui.checkbox label:active::before {
  background: #f9fafb;
  border-color: rgba(34, 36, 38, 0.35);
}

.ui.checkbox label:active::after {
  color: rgba(0, 0, 0, 0.95);
}

.ui.checkbox:not(.invisible) input:active ~ label {
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
     Focus
--------------- */

.ui.checkbox input:focus ~ label::before {
  background: #fff;
  border-color: #96c8da;
}

.ui.checkbox input:focus ~ label::after {
  color: rgba(0, 0, 0, 0.95);
}

.ui.checkbox:not(.invisible) input:focus ~ label {
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
     Active
--------------- */

.ui.checkbox input:checked ~ label::before {
  background: #fff;
  border-color: rgba(34, 36, 38, 0.35);
}

.ui.checkbox input:checked ~ label::after {
  opacity: 1;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
      Indeterminate
    --------------- */

.ui.checkbox input:not([type="radio"]):indeterminate ~ label::before {
  background: #fff;
  border-color: rgba(34, 36, 38, 0.35);
}

.ui.checkbox input:not([type="radio"]):indeterminate ~ label::after {
  opacity: 1;
  color: rgba(0, 0, 0, 0.95);
}

.ui.indeterminate.toggle.checkbox input:not([type="radio"]):indeterminate ~ label::before {
  background: rgba(0, 0, 0, 0.15);
}

.ui.indeterminate.toggle.checkbox input:not([type="radio"]) ~ label::after {
  left: 1.075rem;
}

.ui.right.aligned.indeterminate.toggle.checkbox input:not([type="radio"]) ~ label::after {
  left: auto;
  right: 1.075rem;
}

/* --------------
  Active Focus
--------------- */

.ui.checkbox input:not([type="radio"]):indeterminate:focus ~ label::before,
.ui.checkbox input:checked:focus ~ label::before {
  background: #fff;
  border-color: #96c8da;
}

.ui.checkbox input:not([type="radio"]):indeterminate:focus ~ label::after,
.ui.checkbox input:checked:focus ~ label::after {
  color: rgba(0, 0, 0, 0.95);
}

/* --------------
        Read-Only
    --------------- */

.ui.read-only.checkbox,
.ui.read-only.checkbox label {
  cursor: default;
  pointer-events: none;
}

/* --------------
         Disabled
    --------------- */

.ui.disabled.checkbox label,
.ui.checkbox input[disabled] ~ label {
  cursor: default;
  opacity: 0.5;
  color: #000000;
  pointer-events: none;
}

/* --------------
     Hidden
--------------- */

/* Initialized checkbox moves input below element
 to prevent manually triggering */

.ui.checkbox input.hidden {
  z-index: -1;
}

/* Selectable Label */

.ui.checkbox input + label[for],
.ui.checkbox input.hidden + label {
  cursor: pointer;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

/*******************************
             Types
*******************************/

/* --------------
         Radio
    --------------- */

.ui.radio.checkbox {
  min-height: 15px;
}

.ui.radio.checkbox label {
  padding-left: 1.85714em;
}

/* Box */

.ui.radio.checkbox label::before {
  content: "";
  transform: none;
  width: 15px;
  height: 15px;
  border-radius: 500rem;
  top: 1px;
  left: 0;
}

/* Bullet */

.ui.radio.checkbox label::after {
  border: none;
  content: "" !important;
  line-height: 15px;
  top: 1px;
  left: 0;
  width: 15px;
  height: 15px;
  border-radius: 500rem;
  transform: scale(0.46666667);
  background-color: rgba(0, 0, 0, 0.87);
}

/* Focus */

.ui.radio.checkbox input:focus ~ label::before {
  background-color: #fff;
}

.ui.radio.checkbox input:focus ~ label::after {
  background-color: rgba(0, 0, 0, 0.95);
}

/* Indeterminate */

.ui.radio.checkbox input:indeterminate ~ label::after {
  opacity: 0;
}

/* Active */

.ui.radio.checkbox input:checked ~ label::before {
  background-color: #fff;
}

.ui.radio.checkbox input:checked ~ label::after {
  background-color: rgba(0, 0, 0, 0.95);
}

/* Active Focus */

.ui.radio.checkbox input:focus:checked ~ label::before {
  background-color: #fff;
}

.ui.radio.checkbox input:focus:checked ~ label::after {
  background-color: rgba(0, 0, 0, 0.95);
}

/* --------------
         Slider
    --------------- */

.ui.slider.checkbox {
  min-height: 1.25rem;
}

/* Input */

.ui.slider.checkbox input {
  width: 3.5rem;
  height: 1.25rem;
}

/* Label */

.ui.slider.checkbox label {
  padding-left: 4.5rem;
  line-height: 1rem;
  color: rgba(0, 0, 0, 0.4);
}

/* Line */

.ui.slider.checkbox label::before {
  display: block;
  position: absolute;
  content: "";
  transform: none;
  border: none !important;
  left: 0;
  z-index: 1;
  top: 0.4rem;
  background-color: rgba(0, 0, 0, 0.05);
  width: 3.5rem;
  height: 0.21428571rem;
  border-radius: 500rem;
  transition: background 0.3s ease;
}

/* Handle */

.ui.slider.checkbox label::after {
  background: #fff linear-gradient(transparent, rgba(0, 0, 0, 0.05));
  position: absolute;
  content: "" !important;
  opacity: 1;
  z-index: 2;
  border: none;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
  width: 1.5rem;
  height: 1.5rem;
  top: -0.25rem;
  left: 0;
  transform: none;
  border-radius: 500rem;
  transition: left 0.3s ease;
}

/* Focus */

.ui.slider.checkbox input:focus ~ label::before {
  background-color: rgba(0, 0, 0, 0.15);
  border: none;
}

/* Hover */

.ui.slider.checkbox label:hover {
  color: rgba(0, 0, 0, 0.8);
}

.ui.slider.checkbox label:hover::before {
  background: rgba(0, 0, 0, 0.15);
}

/* Active */

.ui.slider.checkbox input:checked ~ label {
  color: rgba(0, 0, 0, 0.95);
}

.ui.slider.checkbox input:checked ~ label::before {
  background-color: #545454;
}

.ui.slider.checkbox input:checked ~ label::after {
  left: 2rem;
}

/* Active Focus */

.ui.slider.checkbox input:focus:checked ~ label {
  color: rgba(0, 0, 0, 0.95);
}

.ui.slider.checkbox input:focus:checked ~ label::before {
  background-color: #000000;
}

.ui.right.aligned.slider.checkbox label {
  padding-left: 0;
  padding-right: 4.5rem;
}

.ui.right.aligned.slider.checkbox label::after {
  left: auto;
  right: 2rem;
  transition: right 0.3s ease;
}

.ui.right.aligned.slider.checkbox input:checked ~ label::after {
  left: auto;
  right: 0;
}

/* --------------
         Toggle
    --------------- */

.ui.toggle.checkbox {
  min-height: 1.5rem;
}

/* Input */

.ui.toggle.checkbox input {
  width: 3.5rem;
  height: 1.5rem;
}

/* Label */

.ui.toggle.checkbox label {
  min-height: 1.5rem;
  padding-left: 4.5rem;
  color: rgba(0, 0, 0, 0.87);
}

.ui.toggle.checkbox label {
  padding-top: 0.15em;
}

/* Switch */

.ui.toggle.checkbox label::before {
  display: block;
  position: absolute;
  content: "";
  z-index: 1;
  transform: none;
  border: none;
  top: 0;
  background: rgba(0, 0, 0, 0.05);
  box-shadow: none;
  width: 3.5rem;
  height: 1.5rem;
  border-radius: 500rem;
}

/* Handle */

.ui.toggle.checkbox label::after {
  background: #fff linear-gradient(transparent, rgba(0, 0, 0, 0.05));
  position: absolute;
  content: "" !important;
  opacity: 1;
  z-index: 2;
  border: none;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
  width: 1.5rem;
  height: 1.5rem;
  top: 0;
  left: 0;
  border-radius: 500rem;
  transition: background 0.3s ease, left 0.3s ease;
}

.ui.toggle.checkbox input ~ label::after {
  left: -0.05rem;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
}

/* Focus */

.ui.toggle.checkbox input:focus ~ label::before {
  background-color: rgba(0, 0, 0, 0.15);
  border: none;
}

/* Hover */

.ui.toggle.checkbox label:hover::before {
  background-color: rgba(0, 0, 0, 0.15);
  border: none;
}

/* Active */

.ui.toggle.checkbox input:checked ~ label {
  color: rgba(0, 0, 0, 0.95);
}

.ui.toggle.checkbox input:checked ~ label::before {
  background-color: #2185d0;
}

.ui.toggle.checkbox input:checked ~ label::after {
  left: 2.15rem;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
}

/* Active Focus */

.ui.toggle.checkbox input:focus:checked ~ label {
  color: rgba(0, 0, 0, 0.95);
}

.ui.toggle.checkbox input:focus:checked ~ label::before {
  background-color: #0d71bb;
}

.ui.right.aligned.toggle.checkbox label {
  padding-left: 0;
  padding-right: 4.5rem;
}

.ui.right.aligned.toggle.checkbox input ~ label::after {
  left: auto;
  right: 2.15rem;
  transition: background 0.3s ease, right 0.3s ease;
}

.ui.right.aligned.toggle.checkbox input:checked ~ label::after {
  left: auto;
  right: -0.05rem;
}

/*******************************
            Variations
*******************************/

/* --------------
         Fitted
    --------------- */

.ui.ui.fitted.checkbox label {
  padding-left: 0;
}

.ui.fitted.toggle.checkbox {
  width: 3.5rem;
}

.ui.fitted.slider.checkbox {
  width: 3.5rem;
}

/* --------------
         Inverted
    --------------- */

.ui.ui.ui.inverted.checkbox label,
.ui.ui.ui.inverted.checkbox + label {
  color: rgba(255, 255, 255, 0.9);
}

/* Hover */

.ui.ui.inverted.checkbox label:hover {
  color: #ffffff;
}

.ui.inverted.checkbox label:hover::before {
  border-color: rgba(34, 36, 38, 0.5);
}

/* Slider Label */

.ui.inverted.slider.checkbox label {
  color: rgba(255, 255, 255, 0.5);
}

/* Slider Line */

.ui.ui.inverted.slider.checkbox label::before {
  background-color: rgba(255, 255, 255, 0.5);
}

/* Slider Hover */

.ui.ui.inverted.slider.checkbox label:hover::before {
  background: rgba(255, 255, 255, 0.7);
}

/* Slider Active */

.ui.ui.inverted.slider.checkbox input:checked ~ label {
  color: #ffffff;
}

.ui.ui.inverted.slider.checkbox input:checked ~ label::before {
  background-color: rgba(255, 255, 255, 0.8);
}

/* Slider Active Focus */

.ui.ui.inverted.slider.checkbox input:focus:checked ~ label {
  color: #ffffff;
}

.ui.ui.inverted.slider.checkbox input:focus:checked ~ label::before {
  background-color: rgba(255, 255, 255, 0.8);
}

/* Toggle Switch */

.ui.ui.inverted.toggle.checkbox label::before {
  background-color: rgba(255, 255, 255, 0.9);
}

/* Toggle Hover */

.ui.ui.inverted.toggle.checkbox label:hover::before {
  background: #ffffff;
}

/* Toggle Active */

.ui.ui.inverted.toggle.checkbox input:checked ~ label {
  color: #ffffff;
}

.ui.ui.inverted.toggle.checkbox input:checked ~ label::before {
  background-color: #2185d0;
}

/* Toggle Active Focus */

.ui.ui.inverted.toggle.checkbox input:focus:checked ~ label {
  color: #ffffff;
}

.ui.ui.inverted.toggle.checkbox input:focus:checked ~ label::before {
  background-color: #0d71bb;
}

/* --------------
         Invisible
    --------------- */

.ui.invisible.checkbox:not(.compact) {
  display: block;
}

.ui.invisible.checkbox input {
  left: -99999px;
  position: absolute;
}

.ui.invisible.checkbox label::before,
.ui.invisible.checkbox label::after {
  display: none;
}

.ui.invisible.checkbox label {
  transition: all 0.1s ease;
}

.ui.ui.ui.ui.ui.invisible.checkbox input:not(:checked) + label {
  background: transparent;
  border-color: transparent;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
  color: inherit;
}

.ui.ui.ui.ui.ui.invisible.checkbox input:not(:checked):not(:hover):focus + label:not(.image) {
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.35) inset;
}

.ui.ui.ui.ui.ui.invisible.checkbox input:not(:checked) + label.image,
.basic.ui.ui.ui.ui.ui.invisible.checkbox input:not(:checked) + label {
  box-shadow: none;
}

.ui.invisible.checkbox input:not(:checked) + label.image {
  opacity: 0.5;
  filter: grayscale(1);
}

.ui.invisible.checkbox input:not(:checked):not(:hover):focus + label.image {
  opacity: 0.75;
}

.ui.invisible.checkbox input:not(:checked)[disabled] + label.image,
.disabled.ui.invisible.checkbox input:not(:checked) + label.image {
  opacity: 0.25;
}

/* --------------------
        Size
--------------------- */

.ui.mini.checkbox {
  font-size: 0.78571429em;
}

.ui.tiny.checkbox {
  font-size: 0.85714286em;
}

.ui.small.checkbox {
  font-size: 0.92857143em;
}

.ui.large.checkbox {
  font-size: 1.14285714em;
}

.ui.large.form .checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.large.checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.large.form .checkbox:not(.slider):not(.toggle):not(.radio) label::before,
.ui.large.checkbox:not(.slider):not(.toggle):not(.radio) label::before {
  transform: scale(1.14285714);
  transform-origin: left;
}

.ui.large.form .checkbox.radio label::before,
.ui.large.checkbox.radio label::before {
  transform: scale(1.14285714);
  transform-origin: left;
}

.ui.large.form .checkbox.radio label::after,
.ui.large.checkbox.radio label::after {
  transform: scale(0.57142857);
  transform-origin: left;
  left: 0.33571429em;
}

.ui.big.checkbox {
  font-size: 1.28571429em;
}

.ui.big.form .checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.big.checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.big.form .checkbox:not(.slider):not(.toggle):not(.radio) label::before,
.ui.big.checkbox:not(.slider):not(.toggle):not(.radio) label::before {
  transform: scale(1.28571429);
  transform-origin: left;
}

.ui.big.form .checkbox.radio label::before,
.ui.big.checkbox.radio label::before {
  transform: scale(1.28571429);
  transform-origin: left;
}

.ui.big.form .checkbox.radio label::after,
.ui.big.checkbox.radio label::after {
  transform: scale(0.64285714);
  transform-origin: left;
  left: 0.37142857em;
}

.ui.huge.checkbox {
  font-size: 1.42857143em;
}

.ui.huge.form .checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.huge.checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.huge.form .checkbox:not(.slider):not(.toggle):not(.radio) label::before,
.ui.huge.checkbox:not(.slider):not(.toggle):not(.radio) label::before {
  transform: scale(1.42857143);
  transform-origin: left;
}

.ui.huge.form .checkbox.radio label::before,
.ui.huge.checkbox.radio label::before {
  transform: scale(1.42857143);
  transform-origin: left;
}

.ui.huge.form .checkbox.radio label::after,
.ui.huge.checkbox.radio label::after {
  transform: scale(0.71428571);
  transform-origin: left;
  left: 0.40714286em;
}

.ui.massive.checkbox {
  font-size: 1.71428571em;
}

.ui.massive.form .checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.massive.checkbox:not(.slider):not(.toggle):not(.radio) label::after,
.ui.massive.form .checkbox:not(.slider):not(.toggle):not(.radio) label::before,
.ui.massive.checkbox:not(.slider):not(.toggle):not(.radio) label::before {
  transform: scale(1.71428571);
  transform-origin: left;
}

.ui.massive.form .checkbox.radio label::before,
.ui.massive.checkbox.radio label::before {
  transform: scale(1.71428571);
  transform-origin: left;
}

.ui.massive.form .checkbox.radio label::after,
.ui.massive.checkbox.radio label::after {
  transform: scale(0.85714286);
  transform-origin: left;
  left: 0.47857143em;
}

/*******************************
         Theme Overrides
*******************************/

@font-face {
  font-family: Checkbox;
  src: url("data:application/font-woff;base64,d09GRgABAAAAAAR8AA0AAAAABtwAAgAAAAAAAAAAAAAAAAAAAAAAAAAAAABGRlRNAAAEYAAAABoAAAAcgbCWJ0dERUYAAARIAAAAFwAAABgAJQAAT1MvMgAAAZgAAABAAAAAYA8SBD9jbWFwAAAB9AAAAFMAAAFc71zo6Wdhc3AAAARAAAAACAAAAAgAAAAQZ2x5ZgAAAlwAAADJAAABVHshwGBoZWFkAAABMAAAAC4AAAA2Bj3tWWhoZWEAAAFgAAAAHQAAACQHfQPIaG10eAAAAdgAAAAZAAAAHg56AEVsb2NhAAACSAAAABIAAAASAQoAjG1heHAAAAGAAAAAGAAAACAACgAvbmFtZQAAAygAAADnAAABp5xOC4Fwb3N0AAAEEAAAADAAAABaHRtWW3icY2BkYGBgYmDgqbAVjee3+crAzcIAAhc3XilHppl3M+cBKQ6QWgYGAAHnCVsAAHicY2BkYGA+8P8AAwMLAwgw72ZgZEAF7ABaJgNOAAAAeJxjYGRgYOBg0GUA0QwMTAxoAAAFtQA6eJxjYGYWYpzAwMrAwDST6QwDA0M/hGZ8zWDMyMmAChgF0AQYHBgYXjAxH/h/gMGB+QCIx8CIJKvAwAgAIOcKW3icY2GAAMZQCM0ExCwMDK7MqiAeAAqXAM4AAAB4nN2NwQ2AMAwDL00pHYJH52AQ9n/w6BbFCS9WwJFlO3IUwHl5YAQuJctcOaVdU6TGmGWt7IxJuHXnvmcrnLPR2GWb6NnF4rrwhT5U/o0Hf5EJGAAAAAAAAAgACAAQABgASABsAKoAAHichY09CsJAEIVndiG2gyQ7IASJIbHQyvyIiFhZpBE8iNsIeiVvYOFx7FPbKE6yUcHGgZ23jzd8DxDeowAWCv77Dez0Vd1hAIB+D714NMa0yNdYZrMhum0CfaWnT4bpeSPaEjOd8djIltTBNBlhSGzES3Jqb5oW6UOrE1WBL7/A73lxNBqnK8znZRbNWMgJET8qJhlWF1HV3zvXSceBSifawrTjTLrd0uI3c4lFu4UcYvbh1/KsIaodUayx7Kxr+wb8Y+EFi+YsngAAAHicfc5BSsNAGMXxf9q0agURFy50M3shlOoNikIXddGFa5N0SEvLDKQp2KUH8AxuXXkkz+JL+NYJJPzmffNmAlzxS0L7JJxzYx5wxoN5qPzZnMrv5hGXfJrHyr/NE+74UytJL5Rcdye0Huiue/NQ+aM5lV/NI27ZmsfKv8wTnvhhzgZPyY6CyAfMN77cFVFaaVBxZE9OraWvjvtc6Kn0jN40qDnoVyIBx4yMqWJfH7YxuFk27a2/dLVGxUpbQndYrrVnrazgpO9C1chSb7s3UymGxlU++Dpv/NoVJ7co4zLGkPEPj0VARQB4nGNgYsAPOBgYGJkYmBiZGZgZWRhZGdnYS/MyDYAAShuCaFcLCB9Iw/hGAOPdCuMAAQAB//8AD3icY2BkYGDgAWIBIGYCYhYIDQACOwAmAHicY2BgYGQAgns+by1A9MWNV8phNABQ5wfrAAA=") format("woff");
}

/* Checkmark */

.ui.checkbox label::after {
  font-family: Checkbox;
}

/* Checked */

.ui.checkbox input:checked ~ label::after {
  content: "\e800";
}

/* Indeterminate */

.ui.checkbox input:indeterminate ~ label::after {
  font-size: 12px;
  content: "\e801";
}

/* UTF Reference
.check::before { content: '\e800'; }
.dash::before { content: '\e801'; }
.plus::before { content: '\e802'; }
*/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Dimmer
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Dimmer
*******************************/

.dimmable:not(body) {
  position: relative;
}

.ui.dimmer {
  display: none;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  text-align: center;
  vertical-align: middle;
  padding: 1em;
  background: rgba(0, 0, 0, 0.85);
  opacity: 0;
  line-height: 1;
  animation-fill-mode: both;
  animation-duration: 0.5s;
  transition: all 0.5s linear;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  will-change: opacity;
  z-index: 1000;
}

/* Dimmer Content */

.ui.dimmer > .content {
  -webkit-user-select: text;
  -ms-user-select: text;
  user-select: text;
  color: #fff;
}

/* Loose Coupling */

.ui.segment > .ui.dimmer:not(.page) {
  border-radius: inherit;
}

/* Scrollbars */

.ui.dimmer:not(.inverted)::-webkit-scrollbar-track {
  background: rgba(255, 255, 255, 0.1);
}

.ui.dimmer:not(.inverted)::-webkit-scrollbar-thumb {
  background: rgba(255, 255, 255, 0.25);
}

.ui.dimmer:not(.inverted)::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(255, 255, 255, 0.15);
}

.ui.dimmer:not(.inverted)::-webkit-scrollbar-thumb:hover {
  background: rgba(255, 255, 255, 0.35);
}

.ui.dimmer:not(.inverted) {
  /* IE11 */
  scrollbar-face-color: #656565;
  scrollbar-shadow-color: #656565;
  scrollbar-track-color: #323232;
  scrollbar-arrow-color: #323232;
  /* firefox: first color thumb, second track */
  scrollbar-color: rgba(255, 255, 255, 0.25) rgba(255, 255, 255, 0.1);
}

/*******************************
            States
*******************************/

/* Animating */

.animating.dimmable:not(body),
.dimmed.dimmable:not(body) {
  overflow: hidden;
}

/* Animating / Active / Visible */

.dimmed.dimmable > .ui.animating.dimmer,
.dimmed.dimmable > .ui.visible.dimmer,
.ui.active.dimmer {
  display: flex;
  opacity: 1;
}

/* Disabled */

.ui.disabled.dimmer {
  width: 0 !important;
  height: 0 !important;
}

/*******************************
           Variations
*******************************/

/* --------------
        Legacy
    --------------- */

/* Animating / Active / Visible */

.dimmed.dimmable > .ui.animating.legacy.dimmer,
.dimmed.dimmable > .ui.visible.legacy.dimmer,
.ui.active.legacy.dimmer {
  display: block;
}

/* --------------
        Alignment
    --------------- */

.ui[class*="top aligned"].dimmer {
  justify-content: flex-start;
}

.ui[class*="bottom aligned"].dimmer {
  justify-content: flex-end;
}

/* --------------
          Page
    --------------- */

.ui.page.dimmer {
  position: fixed;
  transform-style: "";
  perspective: 2000px;
  transform-origin: center center;
}

.ui.page.dimmer.modals {
  -moz-perspective: none;
}

body.animating.in.dimmable,
body.dimmed.dimmable {
  overflow: hidden;
}

body.dimmable > .dimmer {
  position: fixed;
}

/* --------------
        Blurring
    --------------- */

@supports (not (-webkit-backdrop-filter: none)) and (not ((-webkit-backdrop-filter: none) or (backdrop-filter: none))) {
  .blurring.dimmable > :not(.dimmer) {
    filter: initial;
    transition: 800ms filter ease;
  }

  .blurring.dimmed.dimmable > :not(.dimmer):not(.popup) {
    filter: blur(5px) grayscale(0.7);
  }
}

/* Dimmer Color */

.blurring.dimmable > .dimmer {
  background: rgba(0, 0, 0, 0.6);
  -webkit-backdrop-filter: blur(5px) grayscale(0.7);
  backdrop-filter: blur(5px) grayscale(0.7);
}

.blurring.dimmable > .inverted.dimmer {
  background: rgba(255, 255, 255, 0.6);
}

/* --------------
        Aligned
    --------------- */

.ui.dimmer > .top.aligned.content > * {
  vertical-align: top;
}

.ui.dimmer > .bottom.aligned.content > * {
  vertical-align: bottom;
}

/* --------------
        Shades
    --------------- */

.medium.medium.medium.medium.medium.dimmer {
  background: rgba(0, 0, 0, 0.65);
}

.light.light.light.light.light.dimmer {
  background: rgba(0, 0, 0, 0.45);
}

.very.very.very.very.light.dimmer {
  background: rgba(0, 0, 0, 0.25);
}

/* --------------
        Inverted
    --------------- */

.ui.inverted.dimmer {
  background: rgba(255, 255, 255, 0.85);
}

.ui.inverted.dimmer > .content,
.ui.inverted.dimmer > .content > * {
  color: #000;
}

/* --------------
         Inverted Shades
        --------------- */

.medium.medium.medium.medium.medium.inverted.dimmer {
  background: rgba(255, 255, 255, 0.65);
}

.light.light.light.light.light.inverted.dimmer {
  background: rgba(255, 255, 255, 0.45);
}

.very.very.very.very.light.inverted.dimmer {
  background: rgba(255, 255, 255, 0.25);
}

/* --------------
         Simple
    --------------- */

/* Displays without javascript */

.ui.simple.dimmer {
  display: block;
  overflow: hidden;
  opacity: 0;
  width: 0;
  height: 0;
  z-index: -100;
  background: rgba(0, 0, 0, 0);
}

.dimmed.dimmable > .ui.simple.dimmer {
  overflow: visible;
  opacity: 1;
  width: 100%;
  height: 100%;
  background: rgba(0, 0, 0, 0.85);
  z-index: 1;
}

.ui.simple.inverted.dimmer {
  background: rgba(255, 255, 255, 0);
}

.dimmed.dimmable > .ui.simple.inverted.dimmer {
  background: rgba(255, 255, 255, 0.85);
}

/* --------------
         Partially
    ---------------- */

.ui[class*="top dimmer"],
.ui[class*="center dimmer"],
.ui[class*="bottom dimmer"] {
  height: auto;
}

.ui[class*="bottom dimmer"] {
  top: auto;
  bottom: 0;
}

.ui[class*="center dimmer"] {
  top: 50%;
  transform: translateY(-50%);
}

.ui.segment > .ui.ui[class*="top dimmer"] {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.segment > .ui.ui[class*="center dimmer"] {
  border-radius: 0;
}

.ui.segment > .ui.ui[class*="bottom dimmer"] {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.ui[class*="center dimmer"].transition[class*="fade up"].in {
  animation-name: fadeInUpCenter;
}

.ui[class*="center dimmer"].transition[class*="fade down"].in {
  animation-name: fadeInDownCenter;
}

.ui[class*="center dimmer"].transition[class*="fade up"].out {
  animation-name: fadeOutUpCenter;
}

.ui[class*="center dimmer"].transition[class*="fade down"].out {
  animation-name: fadeOutDownCenter;
}

.ui[class*="center dimmer"].bounce.transition {
  animation-name: bounceCenter;
}

@keyframes fadeInUpCenter {
  0% {
    opacity: 0;
    transform: translateY(-40%);
  }

  100% {
    opacity: 1;
    transform: translateY(-50%);
  }
}

@keyframes fadeInDownCenter {
  0% {
    opacity: 0;
    transform: translateY(-60%);
  }

  100% {
    opacity: 1;
    transform: translateY(-50%);
  }
}

@keyframes fadeOutUpCenter {
  0% {
    opacity: 1;
    transform: translateY(-50%);
  }

  100% {
    opacity: 0;
    transform: translateY(-45%);
  }
}

@keyframes fadeOutDownCenter {
  0% {
    opacity: 1;
    transform: translateY(-50%);
  }

  100% {
    opacity: 0;
    transform: translateY(-55%);
  }
}

@keyframes bounceCenter {
  0%, 20%, 50%, 80%, 100% {
    transform: translateY(-50%);
  }

  40% {
    transform: translateY(calc(-50% - 30px));
  }

  60% {
    transform: translateY(calc(-50% - 15px));
  }
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Dropdown
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Dropdown
*******************************/

.ui.dropdown {
  cursor: pointer;
  position: relative;
  display: inline-block;
  outline: none;
  text-align: left;
  transition: box-shadow 0.1s ease, width 0.1s ease;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}

/*******************************
            Content
*******************************/

/* --------------
      Menu
--------------- */

.ui.dropdown .menu {
  cursor: auto;
  position: absolute;
  display: none;
  outline: none;
  top: 100%;
  min-width: -webkit-max-content;
  min-width: max-content;
  margin: 0;
  padding: 0;
  background: #fff;
  font-size: 1em;
  text-shadow: none;
  text-align: left;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
  border: 1px solid rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
  transition: opacity 0.1s ease;
  z-index: 11;
  will-change: transform, opacity;
}

.ui.dropdown .menu > * {
  white-space: nowrap;
}

/* --------------
  Hidden Input
--------------- */

.ui.dropdown > input:not(.search):first-child,
.ui.dropdown > select {
  display: none !important;
}

/* --------------
 Dropdown Icon
--------------- */

.ui.dropdown:not(.labeled) > .dropdown.icon {
  position: relative;
  width: auto;
  min-width: 1em;
  font-size: 0.85714286em;
  margin: 0 0 0 1em;
}

.ui.dropdown .menu > .item .dropdown.icon {
  width: auto;
  float: right;
  margin: 0em 0 0 1em;
}

.ui.dropdown .menu > .item .dropdown.icon + .text {
  margin-right: 1em;
}

/* --------------
      Text
--------------- */

.ui.dropdown > .text {
  display: inline-block;
  transition: none;
}

/* --------------
    Menu Item
--------------- */

.ui.dropdown .menu > .item {
  position: relative;
  cursor: pointer;
  display: block;
  border: none;
  height: auto;
  min-height: 2.57142857rem;
  text-align: left;
  border-top: none;
  line-height: 1em;
  font-size: 1rem;
  color: rgba(0, 0, 0, 0.87);
  padding: 0.78571429rem 1.14285714rem;
  text-transform: none;
  font-weight: normal;
  box-shadow: none;
  -webkit-touch-callout: none;
}

.ui.dropdown .menu > .item:first-child {
  border-top-width: 0;
}

.ui.dropdown .menu > .item.vertical {
  display: flex;
  flex-direction: column-reverse;
}

/* --------------
  Floated Content
--------------- */

.ui.dropdown > .text > [class*="right floated"],
.ui.dropdown .menu .item > [class*="right floated"] {
  float: right !important;
  margin-right: 0 !important;
  margin-left: 1em !important;
}

.ui.dropdown > .text > [class*="left floated"],
.ui.dropdown .menu .item > [class*="left floated"] {
  float: left !important;
  margin-left: 0 !important;
  margin-right: 1em !important;
}

.ui.dropdown .menu .item > i.icon.floated,
.ui.dropdown .menu .item > .flag.floated,
.ui.dropdown .menu .item > .image.floated,
.ui.dropdown .menu .item > img.floated {
  margin-top: 0em;
}

/* --------------
  Menu Divider
--------------- */

.ui.dropdown .menu > .header {
  margin: 1rem 0 0.75rem;
  padding: 0 1.14285714rem;
  font-weight: bold;
  text-transform: uppercase;
}

.ui.dropdown .menu > .header:not(.ui) {
  color: rgba(0, 0, 0, 0.85);
  font-size: 0.78571429em;
}

.ui.dropdown .menu > .divider {
  border-top: 1px solid rgba(34, 36, 38, 0.1);
  height: 0;
  margin: 0.5em 0;
}

.ui.dropdown .menu > .horizontal.divider {
  border-top: none;
}

.ui.ui.ui.dropdown .menu > .input {
  width: auto;
  display: flex;
  margin: 1.14285714rem 0.78571429rem;
  min-width: 10rem;
}

.ui.dropdown .menu > .header + .input {
  margin-top: 0;
}

.ui.dropdown .menu > .input:not(.transparent) input {
  padding: 0.5em 1em;
}

.ui.dropdown .menu > .input:not(.transparent) .button,
.ui.dropdown .menu > .input:not(.transparent) i.icon,
.ui.dropdown .menu > .input:not(.transparent) .label {
  padding-top: 0.5em;
  padding-bottom: 0.5em;
}

/* -----------------
  Item Description
------------------- */

.ui.dropdown > .text > .description,
.ui.dropdown .menu > .item > .description {
  float: right;
  margin: 0 0 0 1em;
  color: rgba(0, 0, 0, 0.4);
}

.ui.dropdown .menu > .item.vertical > .description {
  margin: 0;
}

/* -----------------
      Item Text
------------------- */

.ui.dropdown .menu > .item.vertical > .text {
  margin-bottom: 0.25em;
}

/* -----------------
       Message
------------------- */

.ui.dropdown .menu > .message {
  padding: 0.78571429rem 1.14285714rem;
  font-weight: normal;
}

.ui.dropdown .menu > .message:not(.ui) {
  color: rgba(0, 0, 0, 0.4);
}

/* --------------
    Sub Menu
--------------- */

.ui.dropdown .menu .menu {
  top: 0;
  left: 100%;
  right: auto;
  margin: 0 -0.5em !important;
  border-radius: 0.28571429rem !important;
  z-index: 21 !important;
}

/* Hide Arrow */

.ui.dropdown .menu .menu::after {
  display: none;
}

/* --------------
   Sub Elements
--------------- */

/* Icons / Flags / Labels / Image */

.ui.dropdown > .text > i.icons,
.ui.dropdown > .text > i.icon,
.ui.dropdown > .text > .label,
.ui.dropdown > .text > .flag,
.ui.dropdown > .text > img,
.ui.dropdown > .text > .image {
  margin-top: 0em;
}

.ui.dropdown .menu > .item > i.icons,
.ui.dropdown .menu > .item > i.icon,
.ui.dropdown .menu > .item > .label,
.ui.dropdown .menu > .item > .flag,
.ui.dropdown .menu > .item > .image,
.ui.dropdown .menu > .item > img {
  margin-top: 0em;
}

.ui.dropdown > .text > i.icons,
.ui.dropdown > .text > i.icon,
.ui.dropdown > .text > .label,
.ui.dropdown > .text > .flag,
.ui.dropdown > .text > img,
.ui.dropdown > .text > .image,
.ui.dropdown .menu > .item > i.icons,
.ui.dropdown .menu > .item > i.icon,
.ui.dropdown .menu > .item > .label,
.ui.dropdown .menu > .item > .flag,
.ui.dropdown .menu > .item > .image,
.ui.dropdown .menu > .item > img {
  margin-left: 0;
  float: none;
  margin-right: 0.78571429rem;
}

/* --------------
     Image
--------------- */

.ui.dropdown > .text > img,
.ui.dropdown > .text > .image:not(.icon),
.ui.dropdown .menu > .item > .image:not(.icon),
.ui.dropdown .menu > .item > img {
  display: inline-block;
  vertical-align: top;
  width: auto;
  margin-top: -0.5em;
  margin-bottom: -0.5em;
  max-height: 2em;
}

/*******************************
            Coupling
*******************************/

/* --------------
      Menu
--------------- */

/* Remove Menu Item Divider */

.ui.dropdown .ui.menu > .item::before,
.ui.menu .ui.dropdown .menu > .item::before {
  display: none;
}

/* Prevent Menu Item Border */

.ui.menu .ui.dropdown .menu .active.item {
  border-left: none;
}

/* Automatically float dropdown menu right on last menu item */

.ui.menu .right.menu .dropdown:last-child > .menu:not(.left),
.ui.menu .right.dropdown.item > .menu:not(.left),
.ui.buttons > .ui.dropdown:last-child > .menu:not(.left) {
  left: auto;
  right: 0;
}

/* --------------
          Label
    --------------- */

/* Dropdown Menu */

.ui.label.dropdown .menu {
  min-width: 100%;
}

/* --------------
         Button
    --------------- */

/* No Margin On Icon Button */

.ui.dropdown.icon.button > .dropdown.icon {
  margin: 0;
}

.ui.dropdown.button .menu {
  min-width: 100%;
}

.ui.dropdown.button:not(.pointing):not(.floating).active {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.dropdown.button:not(.pointing):not(.floating) > .menu {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/*******************************
              Types
*******************************/

select.ui.dropdown {
  height: 38px;
  padding: 0.5em;
  border: 1px solid rgba(34, 36, 38, 0.15);
  visibility: visible;
}

/* --------------
        Selection
    --------------- */

/* Displays like a select box */

.ui.selection.dropdown {
  cursor: pointer;
  word-wrap: break-word;
  line-height: 1em;
  white-space: normal;
  outline: 0;
  transform: rotateZ(0);
  min-width: 14em;
  min-height: 2.71428571em;
  background: #fff;
  display: inline-block;
  padding: 0.78571429em 3.2em 0.78571429em 1em;
  color: rgba(0, 0, 0, 0.87);
  box-shadow: none;
  border: 1px solid rgba(34, 36, 38, 0.15);
  border-radius: 0.28571429rem;
  transition: box-shadow 0.1s ease, width 0.1s ease;
}

.ui.selection.dropdown.visible,
.ui.selection.dropdown.active {
  z-index: 10;
}

.ui.selection.dropdown > .search.icon,
.ui.selection.dropdown > .delete.icon,
.ui.selection.dropdown > .dropdown.icon {
  cursor: pointer;
  position: absolute;
  width: auto;
  height: auto;
  line-height: 1.21428571em;
  top: 0.78571429em;
  right: 1em;
  z-index: 3;
  margin: -0.78571429em;
  padding: 0.91666667em;
  opacity: 0.8;
  transition: opacity 0.1s ease;
}

/* Compact */

.ui.compact.selection.dropdown {
  min-width: 0;
}

.ui.compact.selection.dropdown > .menu {
  width: auto;
}

.ui.compact.selection.dropdown > .menu:not(.multiline) {
  width: -webkit-max-content;
  width: max-content;
}

/* Selection Menu */

.ui.selection.dropdown:not(.unlimited) .menu {
  overflow-x: hidden;
  overflow-y: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

.ui.selection.dropdown .menu {
  outline: none;
  margin: 0 -1px;
  min-width: calc(100% + 2px);
  width: calc(100% + 2px);
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
  transition: opacity 0.1s ease;
}

.ui.selection.dropdown:not(.pointing):not(.floating) .menu {
  border-top-width: 0;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.selection.dropdown .menu::after,
.ui.selection.dropdown .menu::before {
  display: none;
}

/* --------------
        Message
    --------------- */

.ui.selection.dropdown .menu > .message {
  padding: 0.78571429rem 1.14285714rem;
}

@media only screen and (max-width: 767.98px) {
  .ui.selection.dropdown.short .menu {
    max-height: 6.01071429rem;
  }

  .ui.selection.dropdown[class*="very short"] .menu {
    max-height: 4.00714286rem;
  }

  .ui.selection.dropdown:not(.unlimited) .menu {
    max-height: 8.01428571rem;
  }

  .ui.selection.dropdown.long .menu {
    max-height: 16.02857143rem;
  }

  .ui.selection.dropdown[class*="very long"] .menu {
    max-height: 24.04285714rem;
  }
}

@media only screen and (min-width: 768px) {
  .ui.selection.dropdown.short .menu {
    max-height: 8.01428571rem;
  }

  .ui.selection.dropdown[class*="very short"] .menu {
    max-height: 5.34285714rem;
  }

  .ui.selection.dropdown:not(.unlimited) .menu {
    max-height: 10.68571429rem;
  }

  .ui.selection.dropdown.long .menu {
    max-height: 21.37142857rem;
  }

  .ui.selection.dropdown[class*="very long"] .menu {
    max-height: 32.05714286rem;
  }
}

@media only screen and (min-width: 992px) {
  .ui.selection.dropdown.short .menu {
    max-height: 12.02142857rem;
  }

  .ui.selection.dropdown[class*="very short"] .menu {
    max-height: 8.01428571rem;
  }

  .ui.selection.dropdown:not(.unlimited) .menu {
    max-height: 16.02857143rem;
  }

  .ui.selection.dropdown.long .menu {
    max-height: 32.05714286rem;
  }

  .ui.selection.dropdown[class*="very long"] .menu {
    max-height: 48.08571429rem;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.selection.dropdown.short .menu {
    max-height: 16.02857143rem;
  }

  .ui.selection.dropdown[class*="very short"] .menu {
    max-height: 10.68571429rem;
  }

  .ui.selection.dropdown:not(.unlimited) .menu {
    max-height: 21.37142857rem;
  }

  .ui.selection.dropdown.long .menu {
    max-height: 42.74285714rem;
  }

  .ui.selection.dropdown[class*="very long"] .menu {
    max-height: 64.11428571rem;
  }
}

/* Menu Item */

.ui.selection.dropdown .menu > .item {
  border-top: 1px solid #fafafa;
  padding: 0.78571429rem 1.14285714rem;
  white-space: normal;
  word-wrap: normal;
}

.ui.selection.pointing.dropdown .menu > .item:first-child,
.ui.selection.floating.dropdown .menu > .item:first-child {
  border-top: none;
}

/* User Item */

.ui.selection.dropdown .menu > .hidden.addition.item {
  display: none;
}

/* User addition item */

.ui.dropdown > .menu > .stuck.addition.item:first-child {
  position: sticky;
  top: 0;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
  background: #fff;
  z-index: 14;
}

/* Hover */

.ui.selection.dropdown:hover {
  border-color: rgba(34, 36, 38, 0.35);
  box-shadow: none;
}

/* Active */

.ui.selection.simple.dropdown:hover,
.ui.selection.active.dropdown {
  border-color: #96c8da;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
}

.ui.selection.simple.dropdown:hover .menu,
.ui.selection.active.dropdown .menu {
  border-color: #96c8da;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
}

/* Focus */

.ui.selection.dropdown:focus {
  border-color: #96c8da;
  box-shadow: none;
}

.ui.selection.dropdown:focus .menu {
  border-color: #96c8da;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
}

/* Visible */

.ui.selection.visible.dropdown > .text:not(.default) {
  font-weight: normal;
  color: rgba(0, 0, 0, 0.8);
}

/* Visible Hover */

.ui.selection.active.dropdown:hover {
  border-color: #96c8da;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
}

.ui.selection.active.dropdown:hover .menu {
  border-color: #96c8da;
  box-shadow: 0 2px 3px 0 rgba(34, 36, 38, 0.15);
}

/* Dropdown Icon */

.ui.active.selection.dropdown > .dropdown.icon,
.ui.visible.selection.dropdown > .dropdown.icon {
  opacity: "";
  z-index: 3;
}

/* Connecting Border */

.ui.active.selection.dropdown:not(.pointing):not(.floating) {
  border-bottom-left-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
}

/* Empty Connecting Border */

.ui.active.empty.selection.dropdown {
  border-radius: 0.28571429rem !important;
  box-shadow: none !important;
}

.ui.active.empty.selection.dropdown .menu {
  border: none !important;
  box-shadow: none !important;
}

/* CSS specific to iOS devices or firefox mobile only */

@supports (-webkit-touch-callout: none) or (-moz-appearance: none) {
@media (-moz-touch-enabled), (pointer: coarse) {
    .ui.dropdown .scrollhint.menu:not(.hidden)::before {
      animation: scrollhint 2s ease 2;
      content: "";
      z-index: 15;
      display: block;
      position: absolute;
      opacity: 0;
      right: 0.25em;
      top: 0;
      height: 100%;
      border-right: 0.25em solid;
      border-left: 0;
      border-image: linear-gradient(to bottom, rgba(0, 0, 0, 0.75), rgba(0, 0, 0, 0)) 1 100%;
    }

    .ui.inverted.dropdown .scrollhint.menu:not(.hidden)::before {
      border-image: linear-gradient(to bottom, rgba(255, 255, 255, 0.75), rgba(255, 255, 255, 0)) 1 100%;
    }

@keyframes scrollhint {
      0% {
        opacity: 1;
        top: 100%;
      }

      100% {
        opacity: 0;
        top: 0;
      }
}
}
}

/* --------------
       Searchable
    --------------- */

/* Search Selection */

.ui.search.dropdown {
  min-width: "";
}

/* Search Dropdown */

.ui.search.dropdown > input.search {
  background: none transparent !important;
  border: none !important;
  box-shadow: none !important;
  cursor: text;
  top: 0;
  left: 1px;
  width: 100%;
  outline: none;
  -webkit-tap-highlight-color: rgba(255, 255, 255, 0);
  padding: inherit;
}

/* Text Layering */

.ui.search.dropdown > input.search {
  position: absolute;
  z-index: 2;
}

.ui.search.dropdown > .text {
  cursor: text;
  position: relative;
  left: 1px;
  z-index: auto;
}

/* Search Selection */

.ui.search.selection.dropdown > input.search {
  line-height: 1.21428571em;
  padding: 0.67857143em 3.2em 0.67857143em 1em;
}

/* Used to size multi select input to character width */

.ui.search.selection.dropdown > span.sizer {
  line-height: 1.21428571em;
  padding: 0.67857143em 3.2em 0.67857143em 1em;
  display: none;
  white-space: pre;
}

/* Active/Visible Search */

.ui.search.dropdown.active > input.search,
.ui.search.dropdown.visible > input.search {
  cursor: auto;
}

.ui.search.dropdown.active > .text,
.ui.search.dropdown.visible > .text {
  pointer-events: none;
}

/* Filtered Text */

.ui.active.search.dropdown input.search:focus + .text i.icon,
.ui.active.search.dropdown input.search:focus + .text .flag {
  opacity: 0.45;
}

.ui.active.search.dropdown input.search:focus + .text {
  color: rgba(115, 115, 115, 0.87) !important;
}

.ui.search.dropdown.button > span.sizer {
  display: none;
}

/* Search Menu */

.ui.search.dropdown:not(.unlimited) .menu {
  overflow-x: hidden;
  overflow-y: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

@media only screen and (max-width: 767.98px) {
  .ui.search.dropdown:not(.unlimited) .menu {
    max-height: 8.01428571rem;
  }
}

@media only screen and (min-width: 768px) {
  .ui.search.dropdown:not(.unlimited) .menu {
    max-height: 10.68571429rem;
  }
}

@media only screen and (min-width: 992px) {
  .ui.search.dropdown:not(.unlimited) .menu {
    max-height: 16.02857143rem;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.search.dropdown:not(.unlimited) .menu {
    max-height: 21.37142857rem;
  }
}

/* Clearable Selection */

.ui.dropdown > .remove.icon {
  cursor: pointer;
  font-size: 0.85714286em;
  margin: -0.78571429em;
  padding: 0.91666667em;
  right: 2em;
  top: 0.78571429em;
  position: absolute;
  opacity: 0.6;
  z-index: 3;
}

.ui.selection.dropdown > .remove.icon {
  right: 3em;
}

.ui.inline.dropdown > .remove.icon {
  right: 2.2em;
}

.ui.clearable.dropdown .text,
.ui.clearable.dropdown a:last-of-type {
  margin-right: 1.5em;
}

.ui.dropdown select.noselection ~ .remove.icon,
.ui.dropdown input[value=""] ~ .remove.icon,
.ui.dropdown input:not([value]) ~ .remove.icon,
.ui.dropdown.loading > .remove.icon {
  display: none;
}

.ui.dropdown:not(.selection) > .remove.icon ~ .dropdown.icon {
  margin-left: 1.5em;
}

.ui.dropdown:not(.selection):not(.multiple) > .remove.icon {
  margin-top: -1.35em;
}

/* --------------
        Multiple
    --------------- */

/* Multiple Selection */

.ui.ui.multiple.dropdown {
  padding: 0.22619048em 3.2em 0.22619048em 0.35714286em;
}

.ui.multiple.dropdown .menu {
  cursor: auto;
}

/* Selection Label */

.ui.multiple.dropdown > .label {
  display: inline-block;
  white-space: normal;
  font-size: 1em;
  padding: 0.35714286em 0.78571429em;
  margin: 0.14285714rem 0.28571429rem 0.14285714rem 0;
  box-shadow: 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
}

/* Dropdown Icon */

.ui.multiple.dropdown .dropdown.icon {
  margin: "";
  padding: "";
}

/* Text */

.ui.multiple.dropdown > .text {
  position: static;
  padding: 0;
  max-width: 100%;
  margin: 0.45238095em 0 0.45238095em 0.64285714em;
  line-height: 1.21428571em;
}

.ui.multiple.dropdown > .text.default {
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}

.ui.multiple.dropdown > .label ~ input.search {
  margin-left: 0.14285714em !important;
}

.ui.multiple.dropdown > .label ~ .text {
  display: none;
}

.ui.multiple.dropdown > .label:not(.image) > img:not(.centered) {
  margin-right: 0.78571429rem;
}

.ui.multiple.dropdown > .label:not(.image) > img.ui:not(.avatar) {
  margin-bottom: 0.39285714rem;
}

.ui.multiple.dropdown > .image.label img {
  margin: -0.35714286em 0.78571429em -0.35714286em -0.78571429em;
  height: 1.71428571em;
}

.ui.multiple.dropdown:not(.selection):not(.labeled) > .dropdown.icon {
  right: -1.5em;
}

.ui.multiple.dropdown:not(.selection):not(.labeled) > .remove.icon {
  margin-right: -1.5em;
}

.ui.multiple.dropdown:not(.selection):not(.labeled):not(.search) > .remove.icon ~ .text.default,
.ui.multiple.dropdown:not(.selection):not(.labeled) > .text.default:first-child {
  overflow: inherit;
}

/* -----------------
          Multiple Search
        ----------------- */

/* Multiple Search Selection */

.ui.multiple.search.dropdown,
.ui.multiple.search.dropdown > input.search {
  cursor: text;
}

/* Prompt Text */

.ui.multiple.search.dropdown > .text {
  display: inline-block;
  position: absolute;
  top: 0;
  left: 0;
  padding: inherit;
  margin: 0.45238095em 0 0.45238095em 0.64285714em;
  line-height: 1.21428571em;
}

.ui.multiple.search.dropdown > .label ~ .text {
  display: none;
}

/* Search */

.ui.multiple.search.dropdown > input.search {
  position: static;
  padding: 0;
  max-width: 100%;
  margin: 0.45238095em 0 0.45238095em 0.64285714em;
  width: 2.2em;
  line-height: 1.21428571em;
}

.ui.multiple.search.dropdown.button {
  min-width: 14em;
}

.ui.multiple.search.dropdown > span.sizer {
  display: none;
}

.ui.multiple.search.dropdown:not(.selection) > .remove.icon + input.search,
.ui.multiple.search.dropdown:not(.selection) > input.search:first-child {
  min-width: 5.05em;
}

/* --------------
         Inline
    --------------- */

.ui.inline.dropdown {
  cursor: pointer;
  display: inline-block;
  color: inherit;
}

.ui.inline.dropdown .dropdown.icon {
  margin: 0 0.21428571em;
  vertical-align: baseline;
}

.ui.inline.dropdown > .text {
  font-weight: bold;
}

.ui.inline.dropdown .menu {
  cursor: auto;
  margin-top: 0.21428571em;
  border-radius: 0.28571429rem;
}

/*******************************
            States
*******************************/

/* --------------------
        Active
---------------------- */

/* Menu Item Active */

.ui.dropdown .menu .active.item {
  background: transparent;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.95);
  box-shadow: none;
  z-index: 12;
}

/* --------------------
        Hover
---------------------- */

/* Menu Item Hover */

.ui.dropdown .menu > .item:hover {
  background: #f2f2f2;
  color: rgba(0, 0, 0, 0.95);
  z-index: 13;
}

/* --------------------
     Default Text
---------------------- */

.ui.dropdown:not(.button) > .default.text,
.ui.default.dropdown:not(.button) > .text {
  color: rgba(191, 191, 191, 0.87);
}

.ui.dropdown:not(.button) > input:focus ~ .default.text,
.ui.default.dropdown:not(.button) > input:focus ~ .text {
  color: rgba(115, 115, 115, 0.87);
}

/* --------------------
           Loading
    --------------------- */

.ui.loading.dropdown > i.icon {
  height: 1em !important;
}

.ui.loading.selection.dropdown > i.icon {
  padding: 1.5em 1.28571429em !important;
}

.ui.loading.dropdown > i.icon::before {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.1);
}

.ui.loading.dropdown > i.icon::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  box-shadow: 0 0 0 1px transparent;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
}

/* Coupling */

.ui.loading.dropdown.button > i.icon::before,
.ui.loading.dropdown.button > i.icon::after {
  display: none;
}

.ui.loading.dropdown > .text {
  transition: none;
}

/* Used To Check Position */

.ui.dropdown .loading.menu {
  display: block;
  visibility: hidden;
  z-index: -1;
}

.ui.dropdown > .loading.menu {
  left: 0 !important;
  right: auto !important;
}

.ui.dropdown > .menu .loading.menu {
  left: 100% !important;
  right: auto !important;
}

/* --------------------
    Keyboard Select
---------------------- */

/* Selected Item */

.ui.dropdown.selected,
.ui.dropdown .menu .selected.item {
  background: #f7f7f7;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------------
    Search Filtered
---------------------- */

/* Filtered Item */

.ui.dropdown > .filtered.text {
  visibility: hidden;
}

.ui.dropdown .filtered.item {
  display: none !important;
}

/* --------------------
            States
    ---------------------- */

.ui.dropdown.error,
.ui.dropdown.error > .text,
.ui.dropdown.error > .default.text {
  color: #9f3a38;
}

.ui.selection.dropdown.error {
  background: #fff6f6;
  border-color: #e0b4b4;
}

.ui.selection.dropdown.error:hover {
  border-color: #e0b4b4;
}

.ui.multiple.selection.error.dropdown > .label {
  border-color: #e0b4b4;
}

.ui.dropdown.error > .menu,
.ui.dropdown.error > .menu .menu {
  border-color: #e0b4b4;
}

.ui.dropdown.error > .menu > .item {
  color: #9f3a38;
}

/* Item Hover */

.ui.dropdown.error > .menu > .item:hover {
  background-color: #fbe7e7;
}

/* Item Active */

.ui.dropdown.error > .menu .active.item {
  background-color: #fdcfcf;
}

.ui.dropdown.info,
.ui.dropdown.info > .text,
.ui.dropdown.info > .default.text {
  color: #276f86;
}

.ui.selection.dropdown.info {
  background: #f8ffff;
  border-color: #a9d5de;
}

.ui.selection.dropdown.info:hover {
  border-color: #a9d5de;
}

.ui.multiple.selection.info.dropdown > .label {
  border-color: #a9d5de;
}

.ui.dropdown.info > .menu,
.ui.dropdown.info > .menu .menu {
  border-color: #a9d5de;
}

.ui.dropdown.info > .menu > .item {
  color: #276f86;
}

/* Item Hover */

.ui.dropdown.info > .menu > .item:hover {
  background-color: #e9f2fb;
}

/* Item Active */

.ui.dropdown.info > .menu .active.item {
  background-color: #cef1fd;
}

.ui.dropdown.success,
.ui.dropdown.success > .text,
.ui.dropdown.success > .default.text {
  color: #2c662d;
}

.ui.selection.dropdown.success {
  background: #fcfff5;
  border-color: #a3c293;
}

.ui.selection.dropdown.success:hover {
  border-color: #a3c293;
}

.ui.multiple.selection.success.dropdown > .label {
  border-color: #a3c293;
}

.ui.dropdown.success > .menu,
.ui.dropdown.success > .menu .menu {
  border-color: #a3c293;
}

.ui.dropdown.success > .menu > .item {
  color: #2c662d;
}

/* Item Hover */

.ui.dropdown.success > .menu > .item:hover {
  background-color: #e9fbe9;
}

/* Item Active */

.ui.dropdown.success > .menu .active.item {
  background-color: #dafdce;
}

.ui.dropdown.warning,
.ui.dropdown.warning > .text,
.ui.dropdown.warning > .default.text {
  color: #573a08;
}

.ui.selection.dropdown.warning {
  background: #fffaf3;
  border-color: #c9ba9b;
}

.ui.selection.dropdown.warning:hover {
  border-color: #c9ba9b;
}

.ui.multiple.selection.warning.dropdown > .label {
  border-color: #c9ba9b;
}

.ui.dropdown.warning > .menu,
.ui.dropdown.warning > .menu .menu {
  border-color: #c9ba9b;
}

.ui.dropdown.warning > .menu > .item {
  color: #573a08;
}

/* Item Hover */

.ui.dropdown.warning > .menu > .item:hover {
  background-color: #fbfbe9;
}

/* Item Active */

.ui.dropdown.warning > .menu .active.item {
  background-color: #fdfdce;
}

/* --------------------
            Clear
    ---------------------- */

.ui.dropdown > .remove.icon,
.ui.dropdown > .clear.dropdown.icon {
  opacity: 0.6;
  transition: opacity 0.1s ease;
}

.ui.dropdown > .remove.icon:hover,
.ui.dropdown > .clear.dropdown.icon:hover {
  opacity: 1;
}

/* --------------------
            Disabled
    ---------------------- */

/* Disabled */

.ui.disabled.dropdown,
.ui.dropdown .menu > .disabled.item {
  cursor: default;
  pointer-events: none;
  opacity: 0.45;
}

/* --------------------
            Read-Only
    ---------------------- */

.ui.read-only.dropdown {
  cursor: default;
  pointer-events: none;
}

.ui.disabled.dropdown > .remove.icon,
.ui.read-only.dropdown > .remove.icon,
.ui.disabled.dropdown > .label > .delete.icon,
.ui.read-only.dropdown > .label > .delete.icon {
  display: none;
}

/*******************************
           Variations
*******************************/

/* --------------
    Direction
--------------- */

/* Flyout Direction */

.ui.dropdown .menu {
  left: 0;
}

/* Default Side (Right) */

.ui.dropdown .right.menu > .menu,
.ui.dropdown .menu .right.menu {
  left: 100% !important;
  right: auto !important;
  border-radius: 0.28571429rem !important;
}

/* Leftward Opening Menu */

.ui.dropdown > .left.menu {
  left: auto !important;
  right: 0 !important;
}

.ui.dropdown > .left.menu .menu,
.ui.dropdown .menu .left.menu {
  left: auto;
  right: 100%;
  margin: 0 -0.5em 0 0 !important;
  border-radius: 0.28571429rem !important;
}

.ui.dropdown .item .left.dropdown.icon,
.ui.dropdown .left.menu .item .dropdown.icon {
  width: auto;
  float: left;
  margin: 0em 0 0 0;
}

.ui.dropdown .item .left.dropdown.icon,
.ui.dropdown .left.menu .item .dropdown.icon {
  width: auto;
  float: left;
  margin: 0em 0 0 0;
}

.ui.dropdown .item .left.dropdown.icon + .text,
.ui.dropdown .left.menu .item .dropdown.icon + .text {
  margin-left: 1em;
  margin-right: 0;
}

/* --------------
         Upward
    --------------- */

/* Upward Main Menu */

.ui.upward.dropdown > .menu {
  top: auto;
  bottom: 100%;
  box-shadow: 0 0 3px 0 rgba(0, 0, 0, 0.08);
}

/* Upward Sub Menu */

.ui.dropdown .upward.menu {
  top: auto !important;
  bottom: 0 !important;
}

/* Active Upward */

.ui.simple.upward.active.dropdown,
.ui.simple.upward.dropdown:hover {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* Button */

.ui.upward.dropdown.button:not(.pointing):not(.floating).active {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.upward.dropdown.button:not(.pointing):not(.floating) > .menu {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

/* Selection */

.ui.ui.upward.selection.dropdown:not(.pointing):not(.floating) .menu {
  border-top-width: 1px;
  border-bottom-width: 0;
  box-shadow: 0 -2px 3px 0 rgba(0, 0, 0, 0.08);
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.upward.selection.dropdown:hover {
  box-shadow: 0 0 2px 0 rgba(0, 0, 0, 0.05);
}

/* Active Upward */

.ui.active.upward.selection.dropdown:not(.pointing):not(.floating) {
  border-radius: 0 0 0.28571429rem 0.28571429rem !important;
}

/* Visible Upward */

.ui.upward.selection.dropdown.visible {
  box-shadow: 0 0 3px 0 rgba(0, 0, 0, 0.08);
}

.ui.upward.selection.dropdown.visible:not(.pointing):not(.floating) {
  border-radius: 0 0 0.28571429rem 0.28571429rem !important;
}

/* Visible Hover Upward */

.ui.upward.active.selection.dropdown:hover {
  box-shadow: 0 0 3px 0 rgba(0, 0, 0, 0.05);
}

.ui.upward.active.selection.dropdown:hover:not(.pointing):not(.floating) .menu {
  box-shadow: 0 -2px 3px 0 rgba(0, 0, 0, 0.08);
}

/* --------------
         Scrolling
    --------------- */

/* Selection Menu */

.ui.scrolling.dropdown .menu,
.ui.dropdown .scrolling.menu {
  overflow-x: hidden;
  overflow-y: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  min-width: 100% !important;
  width: auto !important;
}

.ui.dropdown .scrolling.menu {
  position: static;
  border: none;
  box-shadow: none !important;
  border-radius: 0 0 0.28571429rem 0.28571429rem !important;
  margin: 0 !important;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.ui.ui.scrolling.dropdown .menu .item,
.ui.ui.ui.dropdown .scrolling.menu > .item {
  border-top: none;
}

.ui.scrolling.dropdown .menu .item:first-child,
.ui.dropdown .scrolling.menu .item:first-child {
  border-top: none;
}

.ui.dropdown > .animating.menu .scrolling.menu,
.ui.dropdown > .visible.menu .scrolling.menu {
  display: block;
}

/* Scrollbar in IE */

@media all and (-ms-high-contrast: none) {
  .ui.scrolling.dropdown .menu,
  .ui.dropdown .scrolling.menu {
    min-width: calc(100% - 17px);
  }
}

@media only screen and (max-width: 767.98px) {
  .ui.scrolling.dropdown .menu,
  .ui.dropdown .scrolling.menu {
    max-height: 10.28571429rem;
  }

  .ui.scrolling.resizable.dropdown .menu,
  .ui.dropdown .scrolling.resizable.menu {
    height: 10.28571429rem;
  }
}

@media only screen and (min-width: 768px) {
  .ui.scrolling.dropdown .menu,
  .ui.dropdown .scrolling.menu {
    max-height: 15.42857143rem;
  }

  .ui.scrolling.resizable.dropdown .menu,
  .ui.dropdown .scrolling.resizable.menu {
    height: 15.42857143rem;
  }
}

@media only screen and (min-width: 992px) {
  .ui.scrolling.dropdown .menu,
  .ui.dropdown .scrolling.menu {
    max-height: 20.57142857rem;
  }

  .ui.scrolling.resizable.dropdown .menu,
  .ui.dropdown .scrolling.resizable.menu {
    height: 20.57142857rem;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.scrolling.dropdown .menu,
  .ui.dropdown .scrolling.menu {
    max-height: 20.57142857rem;
  }

  .ui.scrolling.resizable.dropdown .menu,
  .ui.dropdown .scrolling.resizable.menu {
    height: 20.57142857rem;
  }
}

.ui.scrolling.resizable.dropdown .menu,
.ui.dropdown .scrolling.resizable.menu {
  resize: vertical;
  max-height: none;
}

/* --------------------
            Labeled
    --------------------- */

/* Regular Label on Left */

.ui.labeled.input:not([class*="corner labeled"]) .label:first-child + .dropdown {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-left-color: transparent;
}

/* Regular Label on Right */

.ui[class*="right labeled"].input > .dropdown:not(:last-child) {
  border-top-right-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
  border-right-color: transparent !important;
}

.ui[class*="right labeled"].input > .dropdown + .label {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

/* --------------
         Columnar
    --------------- */

.ui.column.dropdown > .menu {
  flex-wrap: wrap;
}

.ui.dropdown[class*="two column"] > .menu > .item {
  width: 50%;
}

.ui.dropdown[class*="three column"] > .menu > .item {
  width: 33%;
}

.ui.dropdown[class*="four column"] > .menu > .item {
  width: 25%;
}

.ui.dropdown[class*="five column"] > .menu > .item {
  width: 20%;
}

/* --------------
         Simple
    --------------- */

/* Displays without javascript */

.ui.simple.dropdown .menu::before,
.ui.simple.dropdown .menu::after {
  display: none;
}

.ui.simple.dropdown .menu {
  position: absolute;
  /* IE hack to make dropdown icons appear inline */
  display: -ms-inline-flexbox !important;
  display: block;
  overflow: hidden;
  top: -9999px;
  opacity: 0;
  width: 0;
  height: 0;
  transition: opacity 0.1s ease;
  margin-top: 0 !important;
}

.ui.simple.dropdown .upward.menu {
  margin-bottom: 1.5em;
}

.ui.simple.selection.dropdown .upward.menu {
  margin-bottom: 2.5em;
}

.ui.menu:not(.vertical) .ui.simple.dropdown.item .upward.menu {
  margin-bottom: 2.8em;
}

.ui.simple.active.dropdown,
.ui.simple.dropdown:hover {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.simple.active.dropdown > .menu,
.ui.simple.dropdown:hover > .menu {
  overflow: visible;
  width: auto;
  height: auto;
  top: 100%;
  opacity: 1;
}

.ui.simple.dropdown > .menu > .item:active > .menu,
.ui.simple.dropdown .menu .item:hover > .menu {
  overflow: visible;
  width: auto;
  height: auto;
  top: 0 !important;
  left: 100%;
  opacity: 1;
}

.ui.simple.selection.active.dropdown > .menu,
.ui.simple.selection.dropdown:hover > .menu,
.ui.simple.selection.dropdown > .menu > .item:active > .menu,
.ui.simple.selection.dropdown .menu .item:hover > .menu {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

.ui.simple.dropdown > .menu > .item:active > .left.menu,
.ui.simple.dropdown .menu .item:hover > .left.menu,
.right.menu .ui.simple.dropdown > .menu > .item:active > .menu:not(.right),
.right.menu .ui.simple.dropdown > .menu .item:hover > .menu:not(.right) {
  left: auto;
  right: 100%;
}

.ui.simple.disabled.dropdown:hover .menu {
  display: none;
  height: 0;
  width: 0;
  overflow: hidden;
}

/* Visible */

.ui.simple.visible.dropdown > .menu {
  display: block;
}

/* Scrolling */

.ui.simple.scrolling.active.dropdown > .menu,
.ui.simple.scrolling.dropdown:hover > .menu {
  overflow-x: hidden;
  overflow-y: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

/* --------------
          Fluid
    --------------- */

.ui.fluid.dropdown {
  display: block;
  width: 100% !important;
  min-width: 0;
}

.ui.fluid.dropdown > .dropdown.icon {
  float: right;
}

/* --------------
        Floating
    --------------- */

.ui.floating.dropdown > .menu {
  left: 0;
  right: auto;
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15) !important;
  border-radius: 0.28571429rem !important;
}

.ui:not(.upward).floating.dropdown > .menu {
  margin-top: 0.5em;
}

.ui.upward.floating.dropdown:not(.simple) > .menu {
  margin-bottom: 0.5em;
}

/* --------------
         Pointing
    --------------- */

.ui.pointing.dropdown > .menu {
  top: 100%;
  margin-top: 0.78571429rem;
  border-radius: 0.28571429rem;
}

.ui.pointing.dropdown > .menu:not(.hidden)::after {
  display: block;
  position: absolute;
  pointer-events: none;
  content: "";
  visibility: visible;
  transform: rotate(45deg);
  width: 0.5em;
  height: 0.5em;
  box-shadow: -1px -1px 0 0 rgba(34, 36, 38, 0.15);
  background: #fff;
  z-index: 2;
}

@supports selector(:has(.f)) {
  .ui.pointing.dropdown:not(.upward) .menu:has(:first-child:hover)::after,
  .ui.upward.pointing.dropdown .menu:has(:last-child:hover)::after {
    background: #f2f2f2;
  }

  .ui.pointing.dropdown:not(.upward) .menu:has(.selected.item:first-child)::after,
  .ui.upward.pointing.dropdown .menu:has(.selected.item:last-child)::after {
    background: #f7f7f7;
  }
}

.ui.selection.pointing.dropdown > .menu::after {
  box-shadow: -1px -1px 0 0 #96c8da;
}

.ui.pointing.dropdown > .menu:not(.hidden)::after {
  top: -0.25em;
  left: 50%;
  margin: 0 0 0 -0.25em;
}

/* Top Left Pointing */

.ui.top.left.pointing.dropdown > .menu {
  top: 100%;
  bottom: auto;
  left: 0;
  right: auto;
  margin: 1em 0 0;
}

.ui.top.left.pointing.dropdown > .menu {
  top: 100%;
  bottom: auto;
  left: 0;
  right: auto;
  margin: 1em 0 0;
}

.ui.top.left.pointing.dropdown > .menu::after {
  top: -0.25em;
  left: 1em;
  right: auto;
  margin: 0;
  transform: rotate(45deg);
}

/* Top Right Pointing */

.ui.top.right.pointing.dropdown > .menu {
  top: 100%;
  bottom: auto;
  right: 0;
  left: auto;
  margin: 1em 0 0;
}

.ui.top.pointing.dropdown > .left.menu::after,
.ui.top.right.pointing.dropdown > .menu::after {
  top: -0.25em;
  left: auto !important;
  right: 1em !important;
  margin: 0;
  transform: rotate(45deg);
}

/* Left Pointing */

.ui.left.pointing.dropdown > .menu {
  top: 0;
  left: 100%;
  right: auto;
  margin: 0 0 0 1em;
}

.ui.left.pointing.dropdown > .menu::after {
  top: 1em;
  left: -0.25em;
  margin: 0;
  transform: rotate(-45deg);
}

.ui.left:not(.top):not(.bottom).pointing.dropdown > .left.menu {
  left: auto !important;
  right: 100% !important;
  margin: 0 1em 0 0;
}

.ui.left:not(.top):not(.bottom).pointing.dropdown > .left.menu::after {
  top: 1em;
  left: auto;
  right: -0.25em;
  margin: 0;
  transform: rotate(135deg);
}

/* Right Pointing */

.ui.right.pointing.dropdown > .menu {
  top: 0;
  left: auto;
  right: 100%;
  margin: 0 1em 0 0;
}

.ui.right.pointing.dropdown > .menu::after {
  top: 1em;
  left: auto;
  right: -0.25em;
  margin: 0;
  transform: rotate(135deg);
}

/* Bottom Pointing */

.ui.bottom.pointing.dropdown > .menu {
  top: auto;
  bottom: 100%;
  left: 0;
  right: auto;
  margin: 0 0 1em;
}

.ui.bottom.pointing.dropdown > .menu::after {
  top: auto;
  bottom: -0.25em;
  right: auto;
  margin: 0;
  transform: rotate(-135deg);
}

/* Reverse Sub-Menu Direction */

.ui.bottom.pointing.dropdown > .menu .menu {
  top: auto !important;
  bottom: 0 !important;
}

/* Bottom Left */

.ui.bottom.left.pointing.dropdown > .menu {
  left: 0;
  right: auto;
}

.ui.bottom.left.pointing.dropdown > .menu::after {
  left: 1em;
  right: auto;
}

/* Bottom Right */

.ui.bottom.right.pointing.dropdown > .menu {
  right: 0;
  left: auto;
}

.ui.bottom.right.pointing.dropdown > .menu::after {
  left: auto;
  right: 1em;
}

/* Upward pointing */

.ui.pointing.upward.dropdown .menu,
.ui.top.pointing.upward.dropdown .menu {
  top: auto !important;
  bottom: 100% !important;
  margin: 0 0 0.78571429rem;
  border-radius: 0.28571429rem;
}

.ui.pointing.upward.dropdown .menu::after,
.ui.top.pointing.upward.dropdown .menu::after {
  top: 100% !important;
  bottom: auto !important;
  box-shadow: 1px 1px 0 0 rgba(34, 36, 38, 0.15);
  margin: -0.25em 0 0;
}

.ui.selection.pointing.upward.dropdown .menu::after {
  box-shadow: 1px 1px 0 0 #96c8da;
}

/* Right Pointing Upward */

.ui.right.pointing.upward.dropdown:not(.top):not(.bottom) .menu {
  top: auto !important;
  bottom: 0 !important;
  margin: 0 1em 0 0;
}

.ui.right.pointing.upward.dropdown:not(.top):not(.bottom) .menu::after {
  top: auto !important;
  bottom: 0 !important;
  margin: 0 0 1em 0;
  box-shadow: -1px -1px 0 0 rgba(34, 36, 38, 0.15);
}

/* Left Pointing Upward */

.ui.left.pointing.upward.dropdown:not(.top):not(.bottom) .menu {
  top: auto !important;
  bottom: 0 !important;
  margin: 0 0 0 1em;
}

.ui.left.pointing.upward.dropdown:not(.top):not(.bottom) .menu::after {
  top: auto !important;
  bottom: 0 !important;
  margin: 0 0 1em 0;
  box-shadow: -1px -1px 0 0 rgba(34, 36, 38, 0.15);
}

.ui.floating.dropdown .menu .item:first-child,
.ui.pointing.dropdown .menu .item:first-child {
  border-top-left-radius: 0.28571429rem;
  border-top-right-radius: 0.28571429rem;
}

.ui.floating.dropdown .menu .item:last-child,
.ui.pointing.dropdown .menu .item:last-child {
  border-bottom-left-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

/* --------------------
        Sizes
--------------------- */

.ui.dropdown,
.ui.dropdown .menu > .item {
  font-size: 1rem;
}

.ui.mini.dropdown,
.ui.mini.dropdown .menu > .item {
  font-size: 0.78571429rem;
}

.ui.tiny.dropdown,
.ui.tiny.dropdown .menu > .item {
  font-size: 0.85714286rem;
}

.ui.small.dropdown,
.ui.small.dropdown .menu > .item {
  font-size: 0.92857143rem;
}

.ui.large.dropdown,
.ui.large.dropdown .menu > .item {
  font-size: 1.14285714rem;
}

.ui.big.dropdown,
.ui.big.dropdown .menu > .item {
  font-size: 1.28571429rem;
}

.ui.huge.dropdown,
.ui.huge.dropdown .menu > .item {
  font-size: 1.42857143rem;
}

.ui.massive.dropdown,
.ui.massive.dropdown .menu > .item {
  font-size: 1.71428571rem;
}

/* --------------
        Inverted
    --------------- */

/* General rules and basic dropdowns */

.ui.dropdown .inverted.menu,
.ui.inverted.dropdown .menu {
  background: #1b1c1d;
  box-shadow: none;
  border: 1px solid rgba(255, 255, 255, 0.15);
}

.ui.dropdown .inverted.menu > .item,
.ui.inverted.dropdown .menu > .item {
  color: rgba(255, 255, 255, 0.8);
}

.ui.dropdown .inverted.menu .active.item,
.ui.inverted.dropdown .menu .active.item {
  background: transparent;
  color: rgba(255, 255, 255, 0.8);
  box-shadow: none;
}

.ui.dropdown .inverted.menu > .item:hover,
.ui.inverted.dropdown .menu > .item:hover {
  background: #2d2e2f;
  color: rgba(255, 255, 255, 0.8);
}

.ui.inverted.dropdown.selected,
.ui.dropdown .inverted.menu .selected.item,
.ui.inverted.dropdown .menu .selected.item {
  background: #3d3e3f;
  color: rgba(255, 255, 255, 0.8);
}

.ui.dropdown .inverted.menu > .header,
.ui.inverted.dropdown .menu > .header {
  color: #fff;
}

.ui.inverted.dropdown > .text > .description,
.ui.dropdown .inverted.menu > .item > .description,
.ui.inverted.dropdown .menu > .item > .description {
  color: rgba(255, 255, 255, 0.5);
}

.ui.dropdown .inverted.menu > .divider,
.ui.inverted.dropdown .menu > .divider {
  border-top: 1px solid rgba(255, 255, 255, 0.15);
}

.ui.dropdown .inverted.scrolling.menu,
.ui.inverted.dropdown .scrolling.menu {
  border: none;
  border-top: 1px solid rgba(255, 255, 255, 0.15);
}

/* Selection */

.ui.inverted.selection.dropdown {
  border: 1px solid rgba(255, 255, 255, 0.15);
  background: #1b1c1d;
  color: rgba(255, 255, 255, 0.8);
}

.ui.inverted.selection.dropdown:hover {
  border-color: rgba(255, 255, 255, 0.25);
  box-shadow: none;
}

.ui.inverted.selection.dropdown input {
  color: #fff;
}

.ui.inverted.selection.visible.dropdown > .text:not(.default) {
  color: rgba(255, 255, 255, 0.9);
}

.ui.selection.simple.dropdown:hover .inverted.menu,
.ui.inverted.selection.simple.dropdown:hover .menu,
.ui.selection.active.dropdown .inverted.menu,
.ui.inverted.selection.active.dropdown .menu,
.ui.inverted.selection.active.dropdown:hover {
  border-color: rgba(255, 255, 255, 0.15);
}

.ui.selection.dropdown .inverted.menu > .item,
.ui.inverted.selection.dropdown .menu > .item {
  border-top: 1px solid #242526;
}

.ui.inverted.dropdown:not(.button) > .default.text,
.ui.inverted.default.dropdown:not(.button) > .text {
  color: rgba(255, 255, 255, 0.5);
}

.ui.inverted.dropdown:not(.button) > input:focus ~ .default.text,
.ui.inverted.default.dropdown:not(.button) > input:focus ~ .text {
  color: rgba(255, 255, 255, 0.7);
}

.ui.inverted.active.search.dropdown input.search:focus + .text i.icon,
.ui.inverted.active.search.dropdown input.search:focus + .text .flag {
  opacity: 0.45;
}

.ui.inverted.active.search.dropdown input.search:focus + .text {
  color: rgba(255, 255, 255, 0.7) !important;
}

.ui.dropdown .inverted.menu > .message:not(.ui),
.ui.inverted.dropdown .menu > .message:not(.ui) {
  color: rgba(255, 255, 255, 0.5);
}

/* Fixing the border */

.ui.dropdown .inverted.menu > .item:first-child,
.ui.inverted.dropdown .menu > .item:first-child {
  border-top-width: 0;
}

/* Labels */

.ui.inverted.multiple.dropdown > .label {
  background-color: rgba(255, 255, 255, 0.7);
  background-image: none;
  color: #000000;
  box-shadow: 0 0 0 1px rgba(255, 255, 255, 0) inset;
}

.ui.inverted.multiple.dropdown > .label:hover {
  background-color: rgba(255, 255, 255, 0.9);
  border-color: rgba(255, 255, 255, 0.9);
  background-image: none;
  color: #000000;
}

.ui.inverted.multiple.dropdown > .label > .close.icon,
.ui.inverted.multiple.dropdown > .label > .delete.icon {
  opacity: 0.6;
}

.ui.inverted.multiple.dropdown > .label > .close.icon:hover,
.ui.inverted.multiple.dropdown > .label > .delete.icon:hover {
  opacity: 0.8;
}

/* Selection for form elements */

.ui.inverted.dropdown textarea::selection,
.ui.inverted.dropdown input::selection {
  background-color: rgba(255, 255, 255, 0.25);
  color: rgba(255, 255, 255, 0.8);
}

/* Scrollbars */

.ui.dropdown .inverted.menu::-webkit-scrollbar-track,
.ui.inverted.dropdown .menu::-webkit-scrollbar-track {
  background: rgba(255, 255, 255, 0.1);
}

.ui.dropdown .inverted.menu::-webkit-scrollbar-thumb,
.ui.inverted.dropdown .menu::-webkit-scrollbar-thumb {
  background: rgba(255, 255, 255, 0.25);
}

.ui.dropdown .inverted.menu::-webkit-scrollbar-thumb:window-inactive,
.ui.inverted.dropdown .menu::-webkit-scrollbar-thumb:window-inactive {
  background: rgba(255, 255, 255, 0.15);
}

.ui.dropdown .inverted.menu::-webkit-scrollbar-thumb:hover,
.ui.inverted.dropdown .menu::-webkit-scrollbar-thumb:hover {
  background: rgba(255, 255, 255, 0.35);
}

.ui.dropdown .inverted.menu,
.ui.inverted.dropdown .menu {
  /* IE11 */
  scrollbar-face-color: #656565;
  scrollbar-shadow-color: #656565;
  scrollbar-track-color: #323232;
  scrollbar-arrow-color: #323232;
  /* firefox: first color thumb, second track */
  scrollbar-color: rgba(255, 255, 255, 0.25) rgba(255, 255, 255, 0.1);
}

.ui.pointing.dropdown > .inverted.menu::after,
.ui.inverted.pointing.dropdown > .menu::after {
  background: #1b1c1d;
  box-shadow: -1px -1px 0 0 rgba(255, 255, 255, 0.15);
}

.ui.inverted.upward.pointing.dropdown > .menu::after {
  box-shadow: 1px 1px 0 0 rgba(255, 255, 255, 0.15);
}

@supports selector(:has(.f)) {
  .ui.inverted.pointing.dropdown:not(.upward) .menu:has(:first-child:hover)::after,
  .ui.inverted.upward.pointing.dropdown .menu:has(:last-child:hover)::after {
    background: #2d2e2f;
  }

  .ui.inverted.pointing.dropdown:not(.upward) .menu:has(.selected.item:first-child)::after,
  .ui.inverted.upward.pointing.dropdown .menu:has(.selected.item:last-child)::after {
    background: #3d3e3f;
  }
}

/*******************************
         Theme Overrides
*******************************/

/* Dropdown Carets */

@font-face {
  font-family: Dropdown;
  src: url("data:application/font-woff;base64,d09GRk9UVE8AAASEAAsAAAAABwQAAQAAAAAAAAAAAAAAAAAAAAAAAAAAAABDRkYgAAAC5AAAAU8AAAHrSr22AUZGVE0AAARQAAAAHAAAAByNrZ/TR0RFRgAABDQAAAAcAAAAIAAnABhPUy8yAAABYAAAAEIAAABgTadXCGNtYXAAAAKEAAAAVAAAAWQFcOKQaGVhZAAAAQgAAAAyAAAANg+j805oaGVhAAABPAAAABsAAAAkAwcBDWhtdHgAAARsAAAAFgAAAB4EAQASbWF4cAAAAVgAAAAGAAAABgAIUABuYW1lAAABpAAAAN8AAAG2rSbAM3Bvc3QAAALYAAAADAAAACAAAwAAeJxjYGRgYADiKjtG3nh+m68M3EwMIHCBwT0CRN9PyHYCUp6Mqox5QJqDASwNAMNOBrgAAHicY2BkYGB88P8BkFRlYACTjAyogB0AWWsDFgAAAFAAAAgAAHicY2Bm+Mg4gYGVgYHRhzGNgYHBHUp/ZZBkaGFABYwCSJyANNcUBgcGhg+3GB/8fwCUfMCgAFIDV6AAhIwAUbEMSQAAeJx1zTEKwkAUBNBJjIoKIggW2mwvBEVvIAgiNhY2VjEuQUh2YY2FeACPYOltvIH3cZJ8ywQS3p/d+QEwwAce/o8v9tDGUOzTU3GDXokD+ihuooeHuMX8Je5ijC9bXtCRP1T20Kcq+/RE3KAX4oDeipsYIRa3mD/FXSzxLtZeeGyR8bUwHGObWUvsoZHghhQRHEed3NLI1d+vyw/c43DleZErzBFixli768UaNQ9n9d11OedsJdxiyk0RZ40zsxPu/G7K7k66IUvW5CrRRrso12d1uqtNbHfcF+IHrnU+mwB4nGNgYGBmgGAZBkYGEIgD8hjBfBYGJyDNxcDBwASEjAwKH279//v/P1iVwofrYDbj/8dgPkQvGwMPkGRhYAWy2MHCbEDMxIAXMOOXHvIAAINhDt14nGNgZsALAAB9AAR4nE2Pv0rEQBCHZ3J7d4vGqEcOFaKC/zolZ6OWwmElNj6ClYWmyivcblyzE+RaK8EHsPMxBBtrwUdwA1fobpI7ZWBYfux8fD8ExgAR+fVVcpMkt4AeIAzLLSi3sdzxyt1WucqU31I+25iDtcvej1Kzh99V7+a19NoRPC1GAEsRvCxHwCP87MG8I7WgDRwWoAcrEMGmnT3YT+N0kB7G6Vk8PHLr2K0Tt04HB3FjMhUCQIESM7xDhfeYA3dcz5IZdOAcHuAD14MgeAu+zTNTRV5oTUR5obgZdwpVZFqQzEfKTiaE4JNwEralHGUuErnQNjE2oYxUwS0ldBTSxI3L7cOyKo60ib0lezSl1Rw+GXdq1pQiH03SdzKka4ymosLQTMcedSuIlEJUMqNcUi0jdIP5Ysb++leqLtSI/NWSTS0xq0WyYajQXPSVP/cLitO4NgB4nGNgZGBg4AFiAQYJBiYgzQLEIJoRggEFbQBAAAAAAQAAAADeTO04AAAAANAAR1gAAAAA32BrQnicY2BAAEYQVoVghu0MQgwMAAc/ARcAAA==") format("woff");
  font-weight: normal;
  font-style: normal;
}

.ui.dropdown > .dropdown.icon {
  font-family: Dropdown;
  line-height: 1;
  height: 1em;
  width: 1.23em;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  font-weight: normal;
  font-style: normal;
  text-align: center;
}

.ui.dropdown > .dropdown.icon {
  width: auto;
}

.ui.dropdown > .dropdown.icon::before {
  content: "\f0d7";
}

/* Sub Menu */

.ui.dropdown .menu .item .dropdown.icon::before {
  content: "\f0da"
;
}

.ui.dropdown .item .left.dropdown.icon::before,
.ui.dropdown .left.menu .item .dropdown.icon::before {
  content: "\f0d9" ;
}

/* Vertical Menu Dropdown */

.ui.vertical.menu .dropdown.item > .dropdown.icon::before {
  content: "\f0da"
;
}

/* Icons for Reference
.dropdown.down.icon {
  content: "\f0d7";
}
.dropdown.up.icon {
  content: "\f0d8";
}
.dropdown.left.icon {
  content: "\f0d9";
}
.dropdown.icon.icon {
  content: "\f0da";
}
*/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Embed
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Types
*******************************/

.ui.embed {
  position: relative;
  max-width: 100%;
  height: 0;
  overflow: hidden;
  background: #dcddde;
  padding-bottom: 56.25%;
}

/* -----------------
  Embedded Content
------------------ */

.ui.embed iframe,
.ui.embed embed,
.ui.embed object {
  position: absolute;
  border: none;
  width: 100%;
  height: 100%;
  top: 0;
  left: 0;
  margin: 0;
  padding: 0;
  overflow: hidden;
}

/* -----------------
      Embed
------------------ */

.ui.embed > .embed {
  display: none;
}

/* --------------
   Placeholder
--------------- */

.ui.embed > .placeholder {
  position: absolute;
  cursor: pointer;
  top: 0;
  left: 0;
  display: block;
  width: 100%;
  height: 100%;
  background: radial-gradient(transparent 45%, rgba(0, 0, 0, 0.3));
}

/* --------------
      Icon
--------------- */

.ui.embed > i.icon {
  cursor: pointer;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 2;
}

.ui.embed > i.icon::after {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 3;
  content: "";
  background: radial-gradient(transparent 45%, rgba(0, 0, 0, 0.3));
  opacity: 0.5;
  transition: opacity 0.5s ease;
}

.ui.embed > i.icon::before {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translateX(-50%) translateY(-50%);
  color: #fff;
  font-size: 6rem;
  text-shadow: 0 2px 10px rgba(34, 36, 38, 0.2);
  transition: opacity 0.5s ease, color 0.5s ease;
  z-index: 10;
}

/*******************************
            States
*******************************/

/* --------------
     Hover
--------------- */

.ui.embed i.icon:hover::after {
  background: radial-gradient(transparent 45%, rgba(0, 0, 0, 0.3));
  opacity: 1;
}

.ui.embed i.icon:hover::before {
  color: #fff;
}

/* --------------
     Active
--------------- */

.ui.active.embed > i.icon,
.ui.active.embed > .placeholder {
  display: none;
}

.ui.active.embed > .embed {
  display: block;
}

/*******************************
          Variations
*******************************/

.ui.square.embed {
  padding-bottom: 100%;
}

.ui[class*="4:3"].embed {
  padding-bottom: 75%;
}

.ui[class*="16:9"].embed {
  padding-bottom: 56.25%;
}

.ui[class*="21:9"].embed {
  padding-bottom: 42.85714286%;
}

/*******************************
        Video Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Flyout
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Flyout
*******************************/

/* Flyout Menu */

.ui.flyout {
  position: fixed;
  top: 0;
  left: 0;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  transition: none;
  will-change: transform;
  transform: translate3d(0, 0, 0);
  visibility: hidden;
  height: 100%;
  max-height: 100%;
  max-width: 100%;
  border-radius: 0;
  margin: 0;
  overflow-y: hidden;
  z-index: 102;
  background: #fff;
  outline: none;
}

/* GPU Layers for Child Elements */

.ui.flyout > * {
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

/* --------------
   Close
--------------- */

.ui.flyout > .close {
  cursor: pointer;
  position: absolute;
  top: 1.25rem;
  right: 1.5rem;
  z-index: 1;
  opacity: 0.8;
  font-size: 1.25em;
  color: rgba(0, 0, 0, 0.85);
  width: 2.25rem;
  height: 2.25rem;
  padding: 0;
  margin: 0 0 0 0.25rem;
  text-align: right;
}

.ui.flyout > .close:focus,
.ui.flyout > .close:hover {
  opacity: 1;
  outline: none;
}

/* --------------
   Header
--------------- */

.ui.flyout > .header {
  display: block;
  background: #fff;
  margin: 0;
  padding: 1.25rem 1.5rem;
  box-shadow: none;
  color: rgba(0, 0, 0, 0.85);
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

/* --------------
   Content
--------------- */

.ui.flyout > .content {
  display: block;
  width: 100%;
  font-size: 1em;
  line-height: 1.4;
  padding: 1.5rem;
  background: #fff;
}

.ui.flyout > .content.scrolling {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

.ui.flyout.left > .content,
.ui.flyout.right > .content {
  min-height: calc(100vh - 4.7rem);
}

.ui.flyout.left > .header + .content,
.ui.flyout.right > .header + .content {
  min-height: calc(100vh - 9.1rem);
}

.ui.flyout.left > .scrolling.content,
.ui.flyout.right > .scrolling.content {
  max-height: calc(100vh - 4.7rem);
}

.ui.flyout.left > .header + .scrolling.content,
.ui.flyout.right > .header + .scrolling.content {
  max-height: calc(100vh - 9.1rem);
}

.ui.flyout.top > .scrolling.content,
.ui.flyout.bottom > .scrolling.content {
  max-height: calc(80vh - 9.1rem);
}

/* --------------
       Actions
    --------------- */

.ui.flyout > .actions {
  background: #f9fafb;
  padding: 1rem;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  text-align: right;
}

.ui.flyout .actions > .button:not(.fluid) {
  margin-left: 0.75em;
}

.ui.ui.flyout > .basic.actions {
  border-top: none;
}

.ui.flyout > .left.actions {
  text-align: left;
}

.ui.flyout > .left.actions > .button:not(.fluid) {
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.ui.flyout > .centered,
.ui.flyout > .center.aligned {
  text-align: center;
}

.ui.flyout > .centered.actions > .button:not(.fluid),
.ui.flyout > .center.aligned.actions > .button:not(.fluid) {
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.ui.ui.flyout > .basic.header,
.ui.ui.flyout > .basic.actions {
  background-color: transparent;
}

.ui.flyout > .basic.header {
  border-bottom: none;
}

/* --------------
   Direction
--------------- */

.ui.left.flyout {
  right: auto;
  left: 0;
  transform: translate3d(-100%, 0, 0);
}

.ui.right.flyout {
  right: 0;
  left: auto;
  transform: translate3d(100%, 0, 0);
}

.ui.top.flyout,
.ui.bottom.flyout {
  width: 100%;
  height: auto;
}

.ui.top.flyout {
  top: 0;
  bottom: auto;
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.flyout {
  top: auto;
  bottom: 0;
  transform: translate3d(0, 100%, 0);
}

/* --------------
     Pushable
--------------- */

.pushable.pushable.pushable {
  height: 100%;
  overflow-x: hidden;
  padding: 0;
}

.pushable.pushable.pushable.locked {
  overflow-y: hidden;
  background: inherit;
}

/* Whole Page */

body.pushable {
  background: #545454;
}

body.pushable.dimmed {
  background: inherit;
}

/* Page Context */

.pushable:not(body) {
  transform: translate3d(0, 0, 0);
  overflow-y: hidden;
}

.pushable:not(body) > .ui.flyout,
.pushable:not(body) > .fixed,
.pushable:not(body) > .pusher::after {
  position: absolute;
}

/* --------------
     Fixed
--------------- */

.pushable > .fixed {
  position: fixed;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  transition: transform 500ms ease;
  will-change: transform;
  z-index: 101;
}

/* --------------
     Page
--------------- */

.pushable > .pusher {
  position: relative;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  min-height: 100%;
  transition: transform 500ms ease;
  z-index: 2;
  /* Pusher should inherit background from context */
  background: inherit;
}

.pushable > .pusher:not(.overflowing) {
  overflow: hidden;
}

body.pushable > .pusher {
  background: #fff;
}

/* --------------
     Dimmer
--------------- */

.pushable > .pusher::after {
  position: fixed;
  top: 0;
  right: 0;
  content: "";
  background: rgba(0, 0, 0, 0.4);
  overflow: hidden;
  opacity: 0;
  transition: all 500ms;
  will-change: opacity;
  z-index: 1000;
}

/* --------------
    Coupling
--------------- */

.ui.flyout.menu .item {
  border-radius: 0 !important;
}

/*******************************
            States
*******************************/

/* --------------
     Dimmed
--------------- */

.pushable > .pusher.dimmed::after {
  width: 100% !important;
  height: 100% !important;
  opacity: 1 !important;
}

.pushable > .pusher.dimmed.blurring:not(.closing)::after {
  background: rgba(0, 0, 0, 0.6);
  -webkit-backdrop-filter: blur(5px) grayscale(0.7);
  backdrop-filter: blur(5px) grayscale(0.7);
}

.pushable > .pusher.closing.dimmed::after {
  opacity: 0 !important;
}

/* --------------
    Animating
--------------- */

.ui.animating.flyout {
  visibility: visible;
}

/* --------------
     Visible
--------------- */

.ui.visible.flyout {
  visibility: visible;
  transform: translate3d(0, 0, 0);
}

/* Shadow Direction */

.ui.left.visible.flyout,
.ui.right.visible.flyout {
  box-shadow: 0 0 20px rgba(34, 36, 38, 0.15);
}

.ui.top.visible.flyout,
.ui.bottom.visible.flyout {
  box-shadow: 0 0 20px rgba(34, 36, 38, 0.15);
}

/* Visible On Load */

.ui.visible.left.flyout ~ .fixed,
.ui.visible.left.flyout ~ .pusher {
  transform: translate3d(400px, 0, 0);
}

.ui.visible.right.flyout ~ .fixed,
.ui.visible.right.flyout ~ .pusher {
  transform: translate3d(-400px, 0, 0);
}

.ui.visible.top.flyout ~ .fixed,
.ui.visible.top.flyout ~ .pusher {
  transform: translate3d(0, 36px, 0);
}

.ui.visible.bottom.flyout ~ .fixed,
.ui.visible.bottom.flyout ~ .pusher {
  transform: translate3d(0, -36px, 0);
}

/* opposite sides visible forces content overlay */

.ui.visible.left.flyout ~ .ui.visible.right.flyout ~ .fixed,
.ui.visible.left.flyout ~ .ui.visible.right.flyout ~ .pusher,
.ui.visible.right.flyout ~ .ui.visible.left.flyout ~ .fixed,
.ui.visible.right.flyout ~ .ui.visible.left.flyout ~ .pusher {
  transform: translate3d(0, 0, 0);
}

/* --------------
         Inverted
    --------------- */

.ui.flyout.inverted {
  background: #000;
}

.ui.flyout.inverted > .close {
  color: #fff;
}

.ui.flyout.inverted > .header,
.ui.flyout.inverted > .content {
  background: #000;
  color: #fff;
}

.ui.flyout.inverted > .actions {
  background: #191a1b;
  border-top: 1px solid rgba(34, 36, 38, 0.85);
  color: #fff;
}

/*******************************
          Variations
*******************************/

/* --------------
     Width
--------------- */

/* Left / Right */

.ui.left.flyout,
.ui.right.flyout {
  width: 400px;
}

.ui.thin.left.flyout,
.ui.thin.right.flyout {
  width: 200px;
}

.ui[class*="very thin"].left.flyout,
.ui[class*="very thin"].right.flyout {
  width: 120px;
}

.ui.wide.left.flyout,
.ui.wide.right.flyout {
  width: 600px;
}

.ui[class*="very wide"].left.flyout,
.ui[class*="very wide"].right.flyout {
  width: 800px;
}

/* Left Visible */

.ui.visible.thin.left.flyout ~ .fixed,
.ui.visible.thin.left.flyout ~ .pusher {
  transform: translate3d(200px, 0, 0);
}

.ui.visible[class*="very thin"].left.flyout ~ .fixed,
.ui.visible[class*="very thin"].left.flyout ~ .pusher {
  transform: translate3d(120px, 0, 0);
}

.ui.visible.wide.left.flyout ~ .fixed,
.ui.visible.wide.left.flyout ~ .pusher {
  transform: translate3d(600px, 0, 0);
}

.ui.visible[class*="very wide"].left.flyout ~ .fixed,
.ui.visible[class*="very wide"].left.flyout ~ .pusher {
  transform: translate3d(800px, 0, 0);
}

/* Right Visible */

.ui.visible.thin.right.flyout ~ .fixed,
.ui.visible.thin.right.flyout ~ .pusher {
  transform: translate3d(-200px, 0, 0);
}

.ui.visible[class*="very thin"].right.flyout ~ .fixed,
.ui.visible[class*="very thin"].right.flyout ~ .pusher {
  transform: translate3d(-120px, 0, 0);
}

.ui.visible.wide.right.flyout ~ .fixed,
.ui.visible.wide.right.flyout ~ .pusher {
  transform: translate3d(-600px, 0, 0);
}

.ui.visible[class*="very wide"].right.flyout ~ .fixed,
.ui.visible[class*="very wide"].right.flyout ~ .pusher {
  transform: translate3d(-800px, 0, 0);
}

/* Fullscreen */

.ui.fullscreen.flyout {
  width: 100%;
}

/* -------------------
      Column Width
    -------------------- */

/* Sizing Combinations */

.ui[class*="one wide"].flyout:not(.fullscreen) {
  width: 6.25%;
}

.ui[class*="two wide"].flyout:not(.fullscreen) {
  width: 12.5%;
}

.ui[class*="three wide"].flyout:not(.fullscreen) {
  width: 18.75%;
}

.ui[class*="four wide"].flyout:not(.fullscreen) {
  width: 25%;
}

.ui[class*="five wide"].flyout:not(.fullscreen) {
  width: 31.25%;
}

.ui[class*="six wide"].flyout:not(.fullscreen) {
  width: 37.5%;
}

.ui[class*="seven wide"].flyout:not(.fullscreen) {
  width: 43.75%;
}

.ui[class*="eight wide"].flyout:not(.fullscreen) {
  width: 50%;
}

.ui[class*="nine wide"].flyout:not(.fullscreen) {
  width: 56.25%;
}

.ui[class*="ten wide"].flyout:not(.fullscreen) {
  width: 62.5%;
}

.ui[class*="eleven wide"].flyout:not(.fullscreen) {
  width: 68.75%;
}

.ui[class*="twelve wide"].flyout:not(.fullscreen) {
  width: 75%;
}

.ui[class*="thirteen wide"].flyout:not(.fullscreen) {
  width: 81.25%;
}

.ui[class*="fourteen wide"].flyout:not(.fullscreen) {
  width: 87.5%;
}

.ui[class*="fifteen wide"].flyout:not(.fullscreen) {
  width: 93.75%;
}

.ui[class*="sixteen wide"].flyout:not(.fullscreen) {
  width: 100%;
}

/*******************************
          Animations
*******************************/

/* --------------
        Overlay
    --------------- */

/* Set-up */

.ui.overlay.flyout {
  z-index: 102;
}

/* Initial */

.ui.left.overlay.flyout {
  transform: translate3d(-100%, 0, 0);
}

.ui.right.overlay.flyout {
  transform: translate3d(100%, 0, 0);
}

.ui.top.overlay.flyout {
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.overlay.flyout {
  transform: translate3d(0, 100%, 0);
}

/* Animation */

.animating.ui.overlay.flyout,
.ui.visible.overlay.flyout {
  transition: transform 500ms ease;
}

/* End - flyout */

.ui.visible.overlay.flyout {
  transform: translate3d(0, 0, 0);
}

/* End - Pusher */

.ui.visible.overlay.flyout ~ .fixed,
.ui.visible.overlay.flyout ~ .pusher {
  transform: none !important;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Modal
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Modal
*******************************/

.ui.modal {
  position: absolute;
  display: none;
  z-index: 1001;
  text-align: left;
  background: #fff;
  border: none;
  box-shadow: 1px 3px 3px 0 rgba(0, 0, 0, 0.2), 1px 3px 15px 2px rgba(0, 0, 0, 0.2);
  transform-origin: 50% 25%;
  flex: 0 0 auto;
  border-radius: 0.28571429rem;
  -webkit-user-select: text;
  -ms-user-select: text;
  user-select: text;
  will-change: top, left, margin, transform, opacity;
  outline: none;
}

.ui.modal > :first-child:not(.close):not(.dimmer),
.ui.modal > i.icon:first-child + *,
.ui.modal > .dimmer:first-child + *:not(.close),
.ui.modal > .dimmer:first-child + i.icon + * {
  border-top-left-radius: 0.28571429rem;
  border-top-right-radius: 0.28571429rem;
}

.ui.modal > :last-child {
  border-bottom-left-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.modal > .ui.dimmer {
  border-radius: inherit;
}

/*******************************
            Content
*******************************/

/* --------------
     Close
--------------- */

.ui.modal > .close {
  cursor: pointer;
  position: absolute;
  top: -2.5rem;
  right: -2.5rem;
  z-index: 1;
  opacity: 0.8;
  font-size: 1.25em;
  color: #fff;
  width: 2.25rem;
  height: 2.25rem;
  padding: 0.625rem 0 0 0;
}

.ui.modal > .close:focus,
.ui.modal > .close:hover {
  opacity: 1;
  outline: none;
}

/* --------------
     Header
--------------- */

.ui.modal > .header {
  display: block;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  background: #fff;
  margin: 0;
  padding: 1.25rem 1.5rem;
  box-shadow: none;
  color: rgba(0, 0, 0, 0.85);
  border-bottom: 1px solid rgba(34, 36, 38, 0.15);
}

.ui.modal > .header:not(.ui) {
  font-size: 1.42857143rem;
  line-height: 1.28571429em;
  font-weight: bold;
}

/* --------------
     Content
--------------- */

.ui.modal > .content {
  display: block;
  width: 100%;
  font-size: 1em;
  line-height: 1.4;
  padding: 1.5rem;
  background: #fff;
}

.ui.modal > .image.content {
  display: flex;
  flex-direction: row;
}

/* Image */

.ui.modal > .content > .image {
  display: block;
  flex: 0 1 auto;
  width: "";
  align-self: start;
  max-width: 100%;
}

.ui.modal > [class*="top aligned"] {
  align-self: start;
}

.ui.modal > [class*="middle aligned"] {
  align-self: center;
}

.ui.modal > [class*="stretched"] {
  align-self: stretch;
}

/* Description */

.ui.modal > .content > .description {
  display: block;
  flex: 1 0 auto;
  min-width: 0;
  align-self: start;
}

.ui.modal > .content > i.icon + .description,
.ui.modal > .content > .image + .description {
  flex: 0 1 auto;
  min-width: "";
  width: auto;
  padding-left: 2em;
}

/* rtl:ignore */

.ui.modal > .content > .image > i.icon {
  margin: 0;
  opacity: 1;
  width: auto;
  line-height: 1;
  font-size: 8rem;
}

/* --------------
         Actions
    --------------- */

.ui.modal > .actions {
  background: #f9fafb;
  padding: 1rem;
  border-top: 1px solid rgba(34, 36, 38, 0.15);
  text-align: right;
}

.ui.modal .actions > .button:not(.fluid) {
  margin-left: 0.75em;
}

.ui.ui.modal > .basic.actions,
.ui.basic.modal > .actions {
  border-top: none;
}

.ui.modal > .left.actions {
  text-align: left;
}

.ui.modal > .left.actions > .button:not(.fluid) {
  margin-left: 0.5em;
  margin-right: 0.5em;
}

.ui.modal > .centered,
.ui.modal > .center.aligned {
  text-align: center;
}

.ui.modal > .centered.actions > .button:not(.fluid),
.ui.modal > .center.aligned.actions > .button:not(.fluid) {
  margin-left: 0.5em;
  margin-right: 0.5em;
}

/* -------------------
       Responsive
-------------------- */

/* Modal Width */

@media only screen and (max-width: 767.98px) {
  .ui.modal:not(.fullscreen) {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.modal:not(.fullscreen) {
    width: 88%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.modal:not(.fullscreen) {
    width: 850px;
    margin: 0;
  }

  .ui.modal:not(.fullscreen) > .active.dimmer + .close:not(.inside) {
    pointer-events: none;
    opacity: 0.1;
  }

  .ui.dimmer > .ui.modal:not(.fullscreen) > .close:not(.inside) {
    text-shadow: -1px -1px 2px rgba(0, 0, 0, 0.3), 1px -1px 2px rgba(0, 0, 0, 0.3), -1px 2px 2px rgba(0, 0, 0, 0.3), 1px 2px 2px rgba(0, 0, 0, 0.3);
  }

  .ui.inverted.dimmer > .ui.modal:not(.fullscreen) > .close:not(.inside) {
    text-shadow: -1px -1px 2px rgba(255, 255, 255, 0.3), 1px -1px 2px rgba(255, 255, 255, 0.3), -1px 2px 2px rgba(255, 255, 255, 0.3), 1px 2px 2px rgba(255, 255, 255, 0.3);
  }
}

@media only screen and (min-width: 1200px) {
  .ui.modal:not(.fullscreen) {
    width: 900px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.modal:not(.fullscreen) {
    width: 950px;
    margin: 0;
  }
}

/* Tablet and Mobile */

@media only screen and (max-width: 991.98px) {
  .ui.modal > .close + .header {
    padding-right: 2.25rem;
  }

  .ui.modal > .close {
    top: 1.0535rem;
    right: 1rem;
    color: rgba(0, 0, 0, 0.87);
  }
}

/* Mobile */

@media only screen and (max-width: 767.98px) {
  .ui.modal > .header {
    padding: 0.75rem 1rem;
  }

  .ui.modal > .close + .header {
    padding-right: 2.25rem;
  }

  .ui.overlay.fullscreen.modal > .content.content.content {
    min-height: calc(100vh - 8.1rem);
  }

  .ui.overlay.fullscreen.modal > .scrolling.content.content.content {
    max-height: calc(100vh - 8.1rem);
  }

  .ui.modal > .content {
    display: block;
    padding: 1rem !important;
  }

  .ui.modal > .close {
    top: 0.5rem !important;
    right: 0.5rem !important;
  }

  /* rtl:ignore */

  .ui.modal .image.content {
    flex-direction: column;
  }

  .ui.modal > .content > .image {
    display: block;
    max-width: 100%;
    margin: 0 auto !important;
    text-align: center;
    padding: 0 0 1rem !important;
  }

  .ui.modal > .content > .image > i.icon {
    font-size: 5rem;
    text-align: center;
  }

  /* rtl:ignore */

  .ui.modal > .content > .description {
    display: block;
    width: 100% !important;
    margin: 0 !important;
    padding: 1rem 0 !important;
    box-shadow: none;
  }

  /* Let Buttons Stack */

  .ui.modal > .actions {
    padding: 1rem 1rem 0rem !important;
  }

  .ui.modal .actions > .buttons,
  .ui.modal .actions > .button {
    margin-bottom: 1rem;
  }
}

/* --------------
    Coupling
--------------- */

.ui.inverted.dimmer > .ui.modal {
  box-shadow: 1px 3px 10px 2px rgba(0, 0, 0, 0.2);
}

/*******************************
             Types
*******************************/

.ui.basic.modal {
  background-color: transparent;
  border: none;
  border-radius: 0;
  box-shadow: none !important;
  color: #fff;
}

.ui.ui.modal > .basic.header,
.ui.ui.modal > .basic.actions,
.ui.basic.modal > .header,
.ui.basic.modal > .content,
.ui.basic.modal > .actions {
  background-color: transparent;
}

.ui.modal > .basic.header {
  border-bottom: none;
}

.ui.basic.modal > .header {
  color: #fff;
  border-bottom: none;
}

.ui.basic.modal > .close {
  top: 1rem;
  right: 1.5rem;
  color: #fff;
}

.ui.inverted.dimmer > .basic.modal {
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.dimmer > .ui.basic.modal > .header {
  color: rgba(0, 0, 0, 0.85);
}

/* Resort to margin positioning if legacy */

.ui.legacy.legacy.modal,
.ui.legacy.legacy.page.dimmer > .ui.modal {
  left: 50% !important;
}

.ui.legacy.legacy.modal:not(.aligned),
.ui.legacy.legacy.page.dimmer > .ui.modal:not(.aligned) {
  top: 50%;
}

.ui.legacy.legacy.page.dimmer > .ui.scrolling.modal:not(.aligned),
.ui.page.dimmer > .ui.scrolling.legacy.legacy.modal:not(.aligned),
.ui.top.aligned.legacy.legacy.page.dimmer > .ui.modal:not(.aligned),
.ui.top.aligned.dimmer > .ui.legacy.legacy.modal:not(.aligned) {
  top: auto;
}

.ui.legacy.overlay.fullscreen.modal {
  margin-top: -2rem !important;
}

/*******************************
             States
*******************************/

.ui.loading.modal {
  display: block;
  visibility: hidden;
  z-index: -1;
}

.ui.active.modal {
  display: block;
}

/*******************************
           Variations
*******************************/

/* --------------
       Aligned
    --------------- */

.modals.dimmer .ui.top.aligned.modal {
  top: 5vh;
}

.modals.dimmer .ui.bottom.aligned.modal {
  bottom: 5vh;
}

@media only screen and (max-width: 767.98px) {
  .modals.dimmer .ui.top.aligned.modal {
    top: 1rem;
  }

  .modals.dimmer .ui.bottom.aligned.modal {
    bottom: 1rem;
  }
}

/* --------------
        Scrolling
    --------------- */

/* Scrolling Dimmer */

.scrolling.dimmable.dimmed {
  overflow: hidden;
}

.scrolling.dimmable > .dimmer {
  justify-content: center;
  position: fixed;
}

.scrolling.dimmable:not(body) > .dimmer {
  position: absolute;
}

.scrolling.dimmable.dimmed > .dimmer {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

.modals.dimmer .ui.scrolling.modal.fullscreen {
  top: 0;
}

.modals.dimmer .ui.scrolling.modal:not(.fullscreen) {
  margin: 2rem auto;
  top: 1em;
}

/* Fix for Firefox, Edge, IE11 */

.modals.dimmer .ui.scrolling.modal:not([class*="overlay fullscreen"])::after {
  content: "\00A0";
  position: absolute;
  height: 2rem;
}

/* Undetached Scrolling */

.scrolling.undetached.dimmable.dimmed {
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

.scrolling.undetached.dimmable.dimmed > .dimmer {
  overflow: hidden;
}

.scrolling.undetached.dimmable .ui.scrolling.modal:not(.fullscreen) {
  position: absolute;
  left: 50%;
}

/* Scrolling Content */

.ui.modal > .scrolling.content {
  max-height: calc(80vh - 10rem);
  overflow: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
}

.ui.modal > .resizable.scrolling.content {
  resize: vertical;
  max-height: none;
}

.ui.overlay.fullscreen.modal > .content {
  min-height: calc(100vh - 9.1rem);
}

.ui.overlay.fullscreen.modal > .scrolling.content {
  max-height: calc(100vh - 9.1rem);
}

/* --------------
       Full Screen
    --------------- */

.ui.fullscreen.modal {
  width: 95%;
  left: 2.5%;
  margin: 1em auto;
}

.ui.overlay.fullscreen.modal {
  width: 100%;
  left: 0;
  margin: 0 auto;
  top: 0;
  border-radius: 0;
}

.ui.modal > .close.inside + .header:not(.centered):not(.center):not(.icon),
.ui.fullscreen.modal > .close + .header:not(.centered):not(.center):not(.icon) {
  padding-right: 2.25rem;
}

.ui.modal > .close.inside,
.ui.fullscreen.modal > .close {
  top: 1.0535rem;
  right: 1rem;
  color: rgba(0, 0, 0, 0.87);
}

.ui.basic.fullscreen.modal > .close {
  color: #fff;
}

/* --------------
      Size
--------------- */

.ui.modal {
  font-size: 1rem;
}

.ui.mini.modal > .header:not(.ui) {
  font-size: 1.3em;
}

@media only screen and (max-width: 767.98px) {
  .ui.mini.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.mini.modal {
    width: 35.2%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.mini.modal {
    width: 340px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.mini.modal {
    width: 360px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.mini.modal {
    width: 380px;
    margin: 0;
  }
}

.ui.tiny.modal > .header:not(.ui) {
  font-size: 1.3em;
}

@media only screen and (max-width: 767.98px) {
  .ui.tiny.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.tiny.modal {
    width: 52.8%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.tiny.modal {
    width: 510px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.tiny.modal {
    width: 540px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.tiny.modal {
    width: 570px;
    margin: 0;
  }
}

.ui.small.modal > .header:not(.ui) {
  font-size: 1.3em;
}

@media only screen and (max-width: 767.98px) {
  .ui.small.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.small.modal {
    width: 70.4%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.small.modal {
    width: 680px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.small.modal {
    width: 720px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.small.modal {
    width: 760px;
    margin: 0;
  }
}

.ui.large.modal > .header:not(.ui) {
  font-size: 1.6em;
}

@media only screen and (max-width: 767.98px) {
  .ui.large.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.large.modal {
    width: 88%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.large.modal {
    width: 1020px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.large.modal {
    width: 1080px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.large.modal {
    width: 1140px;
    margin: 0;
  }
}

.ui.big.modal > .header:not(.ui) {
  font-size: 1.6em;
}

@media only screen and (max-width: 767.98px) {
  .ui.big.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.big.modal {
    width: 88%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.big.modal {
    width: 1190px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.big.modal {
    width: 1260px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.big.modal {
    width: 1330px;
    margin: 0;
  }
}

.ui.huge.modal > .header:not(.ui) {
  font-size: 1.6em;
}

@media only screen and (max-width: 767.98px) {
  .ui.huge.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.huge.modal {
    width: 88%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.huge.modal {
    width: 1360px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.huge.modal {
    width: 1440px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.huge.modal {
    width: 1520px;
    margin: 0;
  }
}

.ui.massive.modal > .header:not(.ui) {
  font-size: 1.8em;
}

@media only screen and (max-width: 767.98px) {
  .ui.massive.modal {
    width: 95%;
    margin: 0;
  }
}

@media only screen and (min-width: 768px) {
  .ui.massive.modal {
    width: 88%;
    margin: 0;
  }
}

@media only screen and (min-width: 992px) {
  .ui.massive.modal {
    width: 1530px;
    margin: 0;
  }
}

@media only screen and (min-width: 1200px) {
  .ui.massive.modal {
    width: 1620px;
    margin: 0;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.massive.modal {
    width: 1710px;
    margin: 0;
  }
}

/*****************************
              Inverted
    *******************************/

.ui.inverted.modal {
  background: #000;
}

.ui.inverted.modal > .header,
.ui.inverted.modal > .content {
  background: #000;
  color: #fff;
}

.ui.inverted.modal > .actions {
  background: #191a1b;
  border-top: 1px solid rgba(34, 36, 38, 0.85);
  color: #fff;
}

.ui.inverted.dimmer > .modal > .close {
  color: rgba(0, 0, 0, 0.85);
}

@media only screen and (max-width: 991.98px) {
  .ui.dimmer .inverted.modal > .close {
    color: #fff;
  }
}

.ui.inverted.modal > .close.inside,
.ui.inverted.fullscreen.modal > .close {
  color: #fff;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Nag
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Nag
*******************************/

.ui.nag {
  display: none;
  opacity: 0.95;
  position: relative;
  top: 0;
  left: 0;
  z-index: 999;
  min-height: 0;
  width: 100%;
  margin: 0;
  padding: 0.75em 1em;
  background: #909090;
  box-shadow: 0 1px 2px 0 rgba(0, 0, 0, 0.2);
  font-size: 1rem;
  text-align: center;
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0 0 0.28571429rem 0.28571429rem;
  transition: 0.2s background ease;
}

a.ui.nag {
  cursor: pointer;
}

.ui.nag > .title {
  display: inline-block;
  margin: 0 0.5em;
  color: #fff;
}

.ui.nag > .close.icon {
  cursor: pointer;
  opacity: 0.4;
  position: absolute;
  top: 50%;
  right: 1em;
  font-size: 1em;
  margin: -0.5em 0 0;
  color: #fff;
  transition: opacity 0.2s ease;
}

.ui.nag:not(.overlay):not(.fixed) {
  border-radius: 0.28571429rem;
}

/*******************************
             States
*******************************/

/* Hover */

.ui.nag:hover {
  background: #909090;
  opacity: 1;
}

.ui.nag > .close:hover {
  opacity: 1;
}

/*******************************
           Variations
*******************************/

/* --------------
         Static
    --------------- */

.ui.overlay.nags,
.ui.overlay.nag {
  position: absolute;
}

/* --------------
         Fixed
    --------------- */

.ui.fixed.nags,
.ui.fixed.nag {
  position: fixed;
}

/* --------------
         Bottom
    --------------- */

.ui.bottom.nags,
.ui.bottom.nag {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
  top: auto;
  bottom: 0;
}

/* --------------
         Inverted
    --------------- */

.ui.inverted.nags .nag,
.ui.inverted.nag {
  background-color: #f3f4f5;
  color: rgba(0, 0, 0, 0.85);
}

.ui.inverted.nags .nag > .close,
.ui.inverted.nag > .close {
  color: rgba(0, 0, 0, 0.6);
}

.ui.inverted.nags .nag > .title,
.ui.inverted.nag > .title {
  color: rgba(0, 0, 0, 0.6);
}

/* -------------------
            Sizes
    -------------------- */

.ui.mini.nag,
.ui.mini.nags .nag {
  font-size: 0.78571429rem;
}

.ui.tiny.nag,
.ui.tiny.nags .nag {
  font-size: 0.85714286rem;
}

.ui.small.nag,
.ui.small.nags .nag {
  font-size: 0.92857143rem;
}

.ui.large.nag,
.ui.large.nags .nag {
  font-size: 1.14285714rem;
}

.ui.big.nag,
.ui.big.nags .nag {
  font-size: 1.28571429rem;
}

.ui.huge.nag,
.ui.huge.nags .nag {
  font-size: 1.42857143rem;
  line-height: 1;
}

.ui.massive.nag,
.ui.massive.nags .nag {
  font-size: 1.71428571rem;
  line-height: 1;
}

/* --------------
     Colors
-------------- */

.ui.primary.nag {
  background-color: #2185d0;
}

.ui.inverted.primary.nag {
  background-color: #54c8ff;
}

.ui.secondary.nag {
  background-color: #1b1c1d;
}

.ui.inverted.secondary.nag {
  background-color: #545454;
}

.ui.red.nag {
  background-color: #db2828;
}

.ui.inverted.red.nag {
  background-color: #ff695e;
}

.ui.orange.nag {
  background-color: #f2711c;
}

.ui.inverted.orange.nag {
  background-color: #ff851b;
}

.ui.yellow.nag {
  background-color: #fbbd08;
}

.ui.inverted.yellow.nag {
  background-color: #ffe21f;
}

.ui.olive.nag {
  background-color: #b5cc18;
}

.ui.inverted.olive.nag {
  background-color: #d9e778;
}

.ui.green.nag {
  background-color: #21ba45;
}

.ui.inverted.green.nag {
  background-color: #2ecc40;
}

.ui.teal.nag {
  background-color: #00b5ad;
}

.ui.inverted.teal.nag {
  background-color: #6dffff;
}

.ui.blue.nag {
  background-color: #2185d0;
}

.ui.inverted.blue.nag {
  background-color: #54c8ff;
}

.ui.violet.nag {
  background-color: #6435c9;
}

.ui.inverted.violet.nag {
  background-color: #a291fb;
}

.ui.purple.nag {
  background-color: #a333c8;
}

.ui.inverted.purple.nag {
  background-color: #dc73ff;
}

.ui.pink.nag {
  background-color: #e03997;
}

.ui.inverted.pink.nag {
  background-color: #ff8edf;
}

.ui.brown.nag {
  background-color: #a5673f;
}

.ui.inverted.brown.nag {
  background-color: #d67c1c;
}

.ui.grey.nag {
  background-color: #767676;
}

.ui.inverted.grey.nag {
  background-color: #dcddde;
}

.ui.black.nag {
  background-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.black.nag {
  background-color: #545454;
}

.ui.inverted.black.nag .title {
  color: #fff;
}

/*******************************
               Groups
    *******************************/

.ui.nags {
  top: 0;
  left: 0;
  width: 100%;
}

.ui.ui.nags .nag {
  border-radius: 0;
}

.ui.nags:not(.bottom) .nag:last-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.bottom.nags .nag:first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.nags:not(.fixed):not(.overlay) .nag:first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.nags:not(.fixed):not(.overlay) .nag:only-child {
  border-radius: 0.28571429rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Popup
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Popup
*******************************/

.ui.popup {
  display: none;
  position: absolute;
  top: 0;
  right: 0;
  /* Fixes content being squished when inline (moz only) */
  min-width: -webkit-min-content;
  min-width: min-content;
  z-index: 1900;
  border: 1px solid #d4d4d5;
  line-height: 1.4285em;
  max-width: 250px;
  background: #fff;
  padding: 0.833em 1em;
  font-weight: normal;
  font-style: normal;
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0.28571429rem;
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.popup > .header {
  padding: 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1.14285714em;
  line-height: 1.2;
  font-weight: bold;
}

.ui.popup > .header + .content {
  padding-top: 0.5em;
}

.ui.popup::before {
  position: absolute;
  content: "";
  width: 0.71428571em;
  height: 0.71428571em;
  background: #fff;
  transform: rotate(45deg);
  z-index: 1901;
  box-shadow: 1px 1px 0 0 #bababc;
}

/*******************************
            Types
*******************************/

/* --------------
        Tooltip
    --------------- */

/* Content */

[data-tooltip] {
  position: relative;
}

/* Arrow */

[data-tooltip]::before {
  pointer-events: none;
  position: absolute;
  content: "";
  font-size: 1rem;
  width: 0.71428571em;
  height: 0.71428571em;
  background: #fff;
  z-index: 1901;
  box-shadow: 1px 1px 0 0 #bababc;
}

/* Popup */

[data-tooltip]::after {
  pointer-events: none;
  content: attr(data-tooltip);
  position: absolute;
  text-transform: none;
  text-align: left;
  text-shadow: none;
  white-space: nowrap;
  font-size: 1rem;
  border: 1px solid #d4d4d5;
  line-height: 1.4285em;
  max-width: none;
  background: #fff;
  padding: 0.833em 1em;
  font-weight: normal;
  font-style: normal;
  color: rgba(0, 0, 0, 0.87);
  border-radius: 0.28571429rem;
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
  z-index: 1900;
}

/* Animation */

[data-tooltip]::before,
[data-tooltip]::after {
  pointer-events: none;
  opacity: 0;
  transition: transform 0.2s ease, opacity 0.2s ease;
}

[data-tooltip]::after,
[data-tooltip]:hover::before {
  transition-delay: 0.04s;
}

[data-tooltip]::before,
[data-tooltip]:hover::after {
  transition-delay: 0s;
}

[data-tooltip]::before {
  transform: rotate(45deg) scale(0.8);
  transform-origin: center top;
}

[data-tooltip]::after {
  transform-origin: center bottom;
}

[data-tooltip][data-variation~="visible"]::before,
[data-tooltip][data-variation~="visible"]::after,
[data-tooltip]:hover::before,
[data-tooltip]:hover::after {
  pointer-events: auto;
  opacity: 1;
}

[data-tooltip]:hover::before {
  transform: rotate(45deg) scale(1);
}

/* Animation Position */

[data-tooltip]:not([data-position])::after,
[data-tooltip][data-position="top center"]::after,
[data-tooltip][data-position="bottom center"]::after {
  transform: translateX(-50%) scale(0.8);
}

[data-tooltip]:not([data-position]):hover::after,
[data-tooltip][data-position="top center"]:hover::after,
[data-tooltip][data-position="bottom center"]:hover::after {
  transform: translateX(-50%) scale(1);
}

[data-tooltip][data-position="left center"]::after,
[data-tooltip][data-position="right center"]::after {
  transform: translateY(-50%) scale(0.8);
}

[data-tooltip][data-position="left center"]:hover::after,
[data-tooltip][data-position="right center"]:hover::after {
  transform: translateY(-50%) scale(1);
  -moz-transform: translateY(-50%) scale(1.0001) !important;
}

[data-tooltip][data-position="top left"]::after,
[data-tooltip][data-position="top right"]::after,
[data-tooltip][data-position="bottom left"]::after,
[data-tooltip][data-position="bottom right"]::after {
  transform: scale(0.8);
}

[data-tooltip][data-position="top left"]:hover::after,
[data-tooltip][data-position="top right"]:hover::after,
[data-tooltip][data-position="bottom left"]:hover::after,
[data-tooltip][data-position="bottom right"]:hover::after {
  transform: scale(1);
}

[data-tooltip][data-variation~="fixed"]::after {
  white-space: normal;
  width: 250px;
}

[data-tooltip][data-variation*="wide fixed"]::after {
  width: 350px;
}

[data-tooltip][data-variation*="very wide fixed"]::after {
  width: 550px;
}

@media only screen and (max-width: 767.98px) {
  [data-tooltip][data-variation~="fixed"]::after {
    width: 250px;
  }
}

/* --------------
            Inverted
        --------------- */

/* Arrow */

[data-tooltip][data-inverted]::before {
  box-shadow: none;
}

/* Arrow Position */

[data-tooltip]:not([data-position])[data-inverted]::before {
  background: #1b1c1d;
}

/* Popup */

[data-tooltip][data-inverted]::after {
  background: #1b1c1d;
  color: #fff;
  border: none;
  box-shadow: none;
}

/* --------------
            Position
        --------------- */

[data-position~="top"][data-tooltip]::before {
  background: #fff;
}

/* Top Center (default) */

[data-tooltip]:not([data-position])::after,
[data-position="top center"][data-tooltip]::after {
  top: auto;
  right: auto;
  left: 50%;
  bottom: 100%;
  margin-bottom: 0.5em;
}

[data-tooltip]:not([data-position])::before,
[data-position="top center"][data-tooltip]::before {
  top: auto;
  right: auto;
  bottom: 100%;
  left: 50%;
  background: #fff;
  margin-left: -0.07142857rem;
  margin-bottom: 0.14285714rem;
}

/* Top Left */

[data-position="top left"][data-tooltip]::after {
  top: auto;
  right: auto;
  left: 0;
  bottom: 100%;
  margin-bottom: 0.5em;
}

[data-position="top left"][data-tooltip]::before {
  top: auto;
  right: auto;
  bottom: 100%;
  left: 1em;
  margin-left: -0.07142857rem;
  margin-bottom: 0.14285714rem;
}

/* Top Right */

[data-position="top right"][data-tooltip]::after {
  top: auto;
  left: auto;
  right: 0;
  bottom: 100%;
  margin-bottom: 0.5em;
}

[data-position="top right"][data-tooltip]::before {
  top: auto;
  left: auto;
  bottom: 100%;
  right: 1em;
  margin-left: -0.07142857rem;
  margin-bottom: 0.14285714rem;
}

[data-position~="bottom"][data-tooltip]::before {
  background: #fff;
  box-shadow: -1px -1px 0 0 #bababc;
}

/* Bottom Center */

[data-position="bottom center"][data-tooltip]::after {
  bottom: auto;
  right: auto;
  left: 50%;
  top: 100%;
  margin-top: 0.5em;
}

[data-position="bottom center"][data-tooltip]::before {
  bottom: auto;
  right: auto;
  top: 100%;
  left: 50%;
  margin-left: -0.07142857rem;
  margin-top: 0.30714286em;
}

/* Bottom Left */

[data-position="bottom left"][data-tooltip]::after {
  left: 0;
  top: 100%;
  margin-top: 0.5em;
}

[data-position="bottom left"][data-tooltip]::before {
  bottom: auto;
  right: auto;
  top: 100%;
  left: 1em;
  margin-left: -0.07142857rem;
  margin-top: 0.14285714rem;
}

/* Bottom Right */

[data-position="bottom right"][data-tooltip]::after {
  right: 0;
  top: 100%;
  margin-top: 0.5em;
}

[data-position="bottom right"][data-tooltip]::before {
  bottom: auto;
  left: auto;
  top: 100%;
  right: 1em;
  margin-left: -0.14285714rem;
  margin-top: 0.07142857rem;
}

/* Left Center */

[data-position="left center"][data-tooltip]::after {
  right: 100%;
  top: 50%;
  margin-right: 0.5em;
}

[data-position="left center"][data-tooltip]::before {
  right: 100%;
  top: 50%;
  margin-top: -0.14285714rem;
  margin-right: -0.07142857rem;
  background: #fff;
  box-shadow: 1px -1px 0 0 #bababc;
}

/* Right Center */

[data-position="right center"][data-tooltip]::after {
  left: 100%;
  top: 50%;
  margin-left: 0.5em;
}

[data-position="right center"][data-tooltip]::before {
  left: 100%;
  top: 50%;
  margin-top: -0.07142857rem;
  margin-left: 0.14285714rem;
  background: #fff;
  box-shadow: -1px 1px 0 0 #bababc;
}

/* Inverted Arrow Color */

[data-inverted][data-position~="bottom"][data-tooltip]::before {
  background: #1b1c1d;
  box-shadow: -1px -1px 0 0 #bababc;
}

[data-inverted][data-position="left center"][data-tooltip]::before {
  background: #1b1c1d;
  box-shadow: 1px -1px 0 0 #bababc;
}

[data-inverted][data-position="right center"][data-tooltip]::before {
  background: #1b1c1d;
  box-shadow: -1px 1px 0 0 #bababc;
}

[data-inverted][data-position~="top"][data-tooltip]::before {
  background: #1b1c1d;
}

[data-position~="bottom"][data-tooltip]::before {
  transform-origin: center bottom;
}

[data-position~="bottom"][data-tooltip]::after {
  transform-origin: center top;
}

[data-position="bottom center"][data-tooltip]::before {
  transform-origin: center top;
}

[data-position="left center"][data-tooltip]::before {
  transform-origin: top center;
}

[data-position="left center"][data-tooltip]::after {
  transform-origin: right center;
}

[data-position="right center"][data-tooltip]::before {
  transform-origin: right center;
}

[data-position="right center"][data-tooltip]::after {
  transform-origin: left center;
}

[data-position="top left"][data-tooltip]::after {
  transform-origin: bottom left;
}

[data-position="top right"][data-tooltip]::after {
  transform-origin: bottom right;
}

[data-position="bottom left"][data-tooltip]::after {
  transform-origin: top left;
}

[data-position="bottom right"][data-tooltip]::after {
  transform-origin: top right;
}

/* --------------
            Basic
        --------------- */

[data-tooltip][data-variation~="basic"]::before {
  display: none;
}

[data-tooltip][data-variation~="multiline"]::after {
  white-space: pre-line;
}

/* --------------
     Spacing
--------------- */

.ui.popup {
  margin: 0;
}

/* Extending from Top */

.ui.top.popup {
  margin: 0 0 0.71428571em;
}

.ui.top.left.popup {
  transform-origin: left bottom;
}

.ui.top.center.popup {
  transform-origin: center bottom;
}

.ui.top.right.popup {
  transform-origin: right bottom;
}

/* Extending from Vertical Center */

.ui.left.center.popup {
  margin: 0 0.71428571em 0 0;
  transform-origin: right 50%;
}

.ui.right.center.popup {
  margin: 0 0 0 0.71428571em;
  transform-origin: left 50%;
}

/* Extending from Bottom */

.ui.bottom.popup {
  margin: 0.71428571em 0 0;
}

.ui.bottom.left.popup {
  transform-origin: left top;
}

.ui.bottom.center.popup {
  transform-origin: center top;
}

.ui.bottom.right.popup {
  transform-origin: right top;
}

/* --------------
         Pointer
    --------------- */

/* --- Below --- */

.ui.bottom.center.popup::before {
  margin-left: -0.30714286em;
  top: -0.30714286em;
  left: 50%;
  right: auto;
  bottom: auto;
  box-shadow: -1px -1px 0 0 #bababc;
}

.ui.bottom.left.popup {
  margin-left: 0;
}

/* rtl:rename */

.ui.bottom.left.popup::before {
  top: -0.30714286em;
  left: 1em;
  right: auto;
  bottom: auto;
  margin-left: 0;
  box-shadow: -1px -1px 0 0 #bababc;
}

.ui.bottom.right.popup {
  margin-right: 0;
}

/* rtl:rename */

.ui.bottom.right.popup::before {
  top: -0.30714286em;
  right: 1em;
  bottom: auto;
  left: auto;
  margin-left: 0;
  box-shadow: -1px -1px 0 0 #bababc;
}

/* --- Above --- */

.ui.top.center.popup::before {
  top: auto;
  right: auto;
  bottom: -0.30714286em;
  left: 50%;
  margin-left: -0.30714286em;
}

.ui.top.left.popup {
  margin-left: 0;
}

/* rtl:rename */

.ui.top.left.popup::before {
  bottom: -0.30714286em;
  left: 1em;
  top: auto;
  right: auto;
  margin-left: 0;
}

.ui.top.right.popup {
  margin-right: 0;
}

/* rtl:rename */

.ui.top.right.popup::before {
  bottom: -0.30714286em;
  right: 1em;
  top: auto;
  left: auto;
  margin-left: 0;
}

/* --- Left Center --- */

/* rtl:rename */

.ui.left.center.popup::before {
  top: 50%;
  right: -0.30714286em;
  bottom: auto;
  left: auto;
  margin-top: -0.30714286em;
  box-shadow: 1px -1px 0 0 #bababc;
}

/* --- Right Center --- */

/* rtl:rename */

.ui.right.center.popup::before {
  top: 50%;
  left: -0.30714286em;
  bottom: auto;
  right: auto;
  margin-top: -0.30714286em;
  box-shadow: -1px 1px 0 0 #bababc;
}

.ui.right.center.popup::before,
.ui.left.center.popup::before {
  background: #fff;
}

/* Arrow Color By Location */

.ui.bottom.popup::before {
  background: #fff;
}

.ui.top.popup::before {
  background: #fff;
}

/* Inverted Arrow Color */

.ui.inverted.bottom.popup::before {
  background: #1b1c1d;
}

.ui.inverted.right.center.popup::before,
.ui.inverted.left.center.popup::before {
  background: #1b1c1d;
}

.ui.inverted.top.popup::before {
  background: #1b1c1d;
}

/*******************************
            Coupling
*******************************/

/* Immediate Nested Grid */

.ui.ui.ui.popup > .ui.grid:not(.padded) {
  width: calc(100% + 1.75rem);
  margin: -0.7rem -0.875rem;
}

/*******************************
            States
*******************************/

.ui.loading.popup {
  cursor: default;
  pointer-events: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.ui.loading.popup * {
  opacity: 0.3;
}

.ui.loading.popup::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1em 0 0 -1em;
  width: 2em;
  height: 2em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
  visibility: visible;
  z-index: 101;
}

.ui.invisible.popup {
  display: block;
  visibility: hidden !important;
  z-index: -1;
}

.ui.animating.popup,
.ui.visible.popup {
  display: block;
}

.ui.visible.popup {
  transform: translateZ(0);
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

/*******************************
            Variations
*******************************/

/* --------------
         Basic
    --------------- */

.ui.basic.popup::before {
  display: none;
}

.ui.fixed.popup {
  width: 250px;
}

/* --------------
         Wide
    --------------- */

.ui.wide.popup {
  max-width: 350px;
}

.ui.wide.popup.fixed {
  width: 350px;
}

.ui[class*="very wide"].popup {
  max-width: 550px;
}

.ui[class*="very wide"].popup.fixed {
  width: 550px;
}

@media only screen and (max-width: 767.98px) {
  .ui.wide.popup,
  .ui[class*="very wide"].popup {
    max-width: 250px;
  }

  .ui.wide.popup.fixed,
  .ui[class*="very wide"].popup.fixed {
    width: 250px;
  }
}

/* --------------
         Fluid
    --------------- */

.ui.fluid.popup {
  width: 100%;
  max-width: none;
}

/* --------------
         Colors
    --------------- */

/* Inverted colors */

.ui.inverted.popup {
  background: #1b1c1d;
  color: #fff;
  border: none;
  box-shadow: none;
}

.ui.inverted.popup .header {
  background: none;
  color: #fff;
}

.ui.inverted.popup::before {
  background-color: #1b1c1d;
  box-shadow: none;
}

.ui.ui.ui.primary.popup::before,
.ui.primary.popup,
[data-tooltip][data-variation~="primary"]::after,
[data-tooltip][data-variation~="primary"]::before {
  background: #2185d0;
  color: #fff;
  border: none;
}

.ui.ui.ui.primary.loading.popup::before,
.ui.primary.loading.popup {
  background: #9bcbef;
}

.ui.ui.ui.primary.popup::before,
[data-tooltip][data-variation~="primary"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.primary.popup::before,
.ui.inverted.primary.popup,
[data-tooltip][data-inverted][data-variation~="primary"]::after,
[data-tooltip][data-inverted][data-variation~="primary"]::before {
  background: #54c8ff;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.primary.loading.popup::before,
.ui.inverted.primary.loading.popup {
  background: #007eba;
}

.ui.ui.ui.secondary.popup::before,
.ui.secondary.popup,
[data-tooltip][data-variation~="secondary"]::after,
[data-tooltip][data-variation~="secondary"]::before {
  background: #1b1c1d;
  color: #fff;
  border: none;
}

.ui.ui.ui.secondary.loading.popup::before,
.ui.secondary.loading.popup {
  background: #65696c;
}

.ui.ui.ui.secondary.popup::before,
[data-tooltip][data-variation~="secondary"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.secondary.popup::before,
.ui.inverted.secondary.popup,
[data-tooltip][data-inverted][data-variation~="secondary"]::after,
[data-tooltip][data-inverted][data-variation~="secondary"]::before {
  background: #545454;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.secondary.loading.popup::before,
.ui.inverted.secondary.loading.popup {
  background: #080808;
}

.ui.ui.ui.red.popup::before,
.ui.red.popup,
[data-tooltip][data-variation~="red"]::after,
[data-tooltip][data-variation~="red"]::before {
  background: #db2828;
  color: #fff;
  border: none;
}

.ui.ui.ui.red.loading.popup::before,
.ui.red.loading.popup {
  background: #f1abab;
}

.ui.ui.ui.red.popup::before,
[data-tooltip][data-variation~="red"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.red.popup::before,
.ui.inverted.red.popup,
[data-tooltip][data-inverted][data-variation~="red"]::after,
[data-tooltip][data-inverted][data-variation~="red"]::before {
  background: #ff695e;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.red.loading.popup::before,
.ui.inverted.red.loading.popup {
  background: #c40d00;
}

.ui.ui.ui.orange.popup::before,
.ui.orange.popup,
[data-tooltip][data-variation~="orange"]::after,
[data-tooltip][data-variation~="orange"]::before {
  background: #f2711c;
  color: #fff;
  border: none;
}

.ui.ui.ui.orange.loading.popup::before,
.ui.orange.loading.popup {
  background: #faccad;
}

.ui.ui.ui.orange.popup::before,
[data-tooltip][data-variation~="orange"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.orange.popup::before,
.ui.inverted.orange.popup,
[data-tooltip][data-inverted][data-variation~="orange"]::after,
[data-tooltip][data-inverted][data-variation~="orange"]::before {
  background: #ff851b;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.orange.loading.popup::before,
.ui.inverted.orange.loading.popup {
  background: #813c00;
}

.ui.ui.ui.yellow.popup::before,
.ui.yellow.popup,
[data-tooltip][data-variation~="yellow"]::after,
[data-tooltip][data-variation~="yellow"]::before {
  background: #fbbd08;
  color: #fff;
  border: none;
}

.ui.ui.ui.yellow.loading.popup::before,
.ui.yellow.loading.popup {
  background: #fde59f;
}

.ui.ui.ui.yellow.popup::before,
[data-tooltip][data-variation~="yellow"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.yellow.popup::before,
.ui.inverted.yellow.popup,
[data-tooltip][data-inverted][data-variation~="yellow"]::after,
[data-tooltip][data-inverted][data-variation~="yellow"]::before {
  background: #ffe21f;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.yellow.loading.popup::before,
.ui.inverted.yellow.loading.popup {
  background: #857400;
}

.ui.ui.ui.olive.popup::before,
.ui.olive.popup,
[data-tooltip][data-variation~="olive"]::after,
[data-tooltip][data-variation~="olive"]::before {
  background: #b5cc18;
  color: #fff;
  border: none;
}

.ui.ui.ui.olive.loading.popup::before,
.ui.olive.loading.popup {
  background: #e4f18c;
}

.ui.ui.ui.olive.popup::before,
[data-tooltip][data-variation~="olive"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.olive.popup::before,
.ui.inverted.olive.popup,
[data-tooltip][data-inverted][data-variation~="olive"]::after,
[data-tooltip][data-inverted][data-variation~="olive"]::before {
  background: #d9e778;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.olive.loading.popup::before,
.ui.inverted.olive.loading.popup {
  background: #97a81e;
}

.ui.ui.ui.green.popup::before,
.ui.green.popup,
[data-tooltip][data-variation~="green"]::after,
[data-tooltip][data-variation~="green"]::before {
  background: #21ba45;
  color: #fff;
  border: none;
}

.ui.ui.ui.green.loading.popup::before,
.ui.green.loading.popup {
  background: #8aeaa0;
}

.ui.ui.ui.green.popup::before,
[data-tooltip][data-variation~="green"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.green.popup::before,
.ui.inverted.green.popup,
[data-tooltip][data-inverted][data-variation~="green"]::after,
[data-tooltip][data-inverted][data-variation~="green"]::before {
  background: #2ecc40;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.green.loading.popup::before,
.ui.inverted.green.loading.popup {
  background: #124f19;
}

.ui.ui.ui.teal.popup::before,
.ui.teal.popup,
[data-tooltip][data-variation~="teal"]::after,
[data-tooltip][data-variation~="teal"]::before {
  background: #00b5ad;
  color: #fff;
  border: none;
}

.ui.ui.ui.teal.loading.popup::before,
.ui.teal.loading.popup {
  background: #4ffff7;
}

.ui.ui.ui.teal.popup::before,
[data-tooltip][data-variation~="teal"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.teal.popup::before,
.ui.inverted.teal.popup,
[data-tooltip][data-inverted][data-variation~="teal"]::after,
[data-tooltip][data-inverted][data-variation~="teal"]::before {
  background: #6dffff;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.teal.loading.popup::before,
.ui.inverted.teal.loading.popup {
  background: #00d3d3;
}

.ui.ui.ui.blue.popup::before,
.ui.blue.popup,
[data-tooltip][data-variation~="blue"]::after,
[data-tooltip][data-variation~="blue"]::before {
  background: #2185d0;
  color: #fff;
  border: none;
}

.ui.ui.ui.blue.loading.popup::before,
.ui.blue.loading.popup {
  background: #9bcbef;
}

.ui.ui.ui.blue.popup::before,
[data-tooltip][data-variation~="blue"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.blue.popup::before,
.ui.inverted.blue.popup,
[data-tooltip][data-inverted][data-variation~="blue"]::after,
[data-tooltip][data-inverted][data-variation~="blue"]::before {
  background: #54c8ff;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.blue.loading.popup::before,
.ui.inverted.blue.loading.popup {
  background: #007eba;
}

.ui.ui.ui.violet.popup::before,
.ui.violet.popup,
[data-tooltip][data-variation~="violet"]::after,
[data-tooltip][data-variation~="violet"]::before {
  background: #6435c9;
  color: #fff;
  border: none;
}

.ui.ui.ui.violet.loading.popup::before,
.ui.violet.loading.popup {
  background: #c1adea;
}

.ui.ui.ui.violet.popup::before,
[data-tooltip][data-variation~="violet"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.violet.popup::before,
.ui.inverted.violet.popup,
[data-tooltip][data-inverted][data-variation~="violet"]::after,
[data-tooltip][data-inverted][data-variation~="violet"]::before {
  background: #a291fb;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.violet.loading.popup::before,
.ui.inverted.violet.loading.popup {
  background: #2d09ea;
}

.ui.ui.ui.purple.popup::before,
.ui.purple.popup,
[data-tooltip][data-variation~="purple"]::after,
[data-tooltip][data-variation~="purple"]::before {
  background: #a333c8;
  color: #fff;
  border: none;
}

.ui.ui.ui.purple.loading.popup::before,
.ui.purple.loading.popup {
  background: #daabe9;
}

.ui.ui.ui.purple.popup::before,
[data-tooltip][data-variation~="purple"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.purple.popup::before,
.ui.inverted.purple.popup,
[data-tooltip][data-inverted][data-variation~="purple"]::after,
[data-tooltip][data-inverted][data-variation~="purple"]::before {
  background: #dc73ff;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.purple.loading.popup::before,
.ui.inverted.purple.loading.popup {
  background: #a300d9;
}

.ui.ui.ui.pink.popup::before,
.ui.pink.popup,
[data-tooltip][data-variation~="pink"]::after,
[data-tooltip][data-variation~="pink"]::before {
  background: #e03997;
  color: #fff;
  border: none;
}

.ui.ui.ui.pink.loading.popup::before,
.ui.pink.loading.popup {
  background: #f5bddc;
}

.ui.ui.ui.pink.popup::before,
[data-tooltip][data-variation~="pink"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.pink.popup::before,
.ui.inverted.pink.popup,
[data-tooltip][data-inverted][data-variation~="pink"]::after,
[data-tooltip][data-inverted][data-variation~="pink"]::before {
  background: #ff8edf;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.pink.loading.popup::before,
.ui.inverted.pink.loading.popup {
  background: #f400af;
}

.ui.ui.ui.brown.popup::before,
.ui.brown.popup,
[data-tooltip][data-variation~="brown"]::after,
[data-tooltip][data-variation~="brown"]::before {
  background: #a5673f;
  color: #fff;
  border: none;
}

.ui.ui.ui.brown.loading.popup::before,
.ui.brown.loading.popup {
  background: #dbb8a2;
}

.ui.ui.ui.brown.popup::before,
[data-tooltip][data-variation~="brown"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.brown.popup::before,
.ui.inverted.brown.popup,
[data-tooltip][data-inverted][data-variation~="brown"]::after,
[data-tooltip][data-inverted][data-variation~="brown"]::before {
  background: #d67c1c;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.brown.loading.popup::before,
.ui.inverted.brown.loading.popup {
  background: #4f2e0a;
}

.ui.ui.ui.grey.popup::before,
.ui.grey.popup,
[data-tooltip][data-variation~="grey"]::after,
[data-tooltip][data-variation~="grey"]::before {
  background: #767676;
  color: #fff;
  border: none;
}

.ui.ui.ui.grey.loading.popup::before,
.ui.grey.loading.popup {
  background: #c3c3c3;
}

.ui.ui.ui.grey.popup::before,
[data-tooltip][data-variation~="grey"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.grey.popup::before,
.ui.inverted.grey.popup,
[data-tooltip][data-inverted][data-variation~="grey"]::after,
[data-tooltip][data-inverted][data-variation~="grey"]::before {
  background: #dcddde;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.grey.loading.popup::before,
.ui.inverted.grey.loading.popup {
  background: #8d9194;
}

.ui.ui.ui.black.popup::before,
.ui.black.popup,
[data-tooltip][data-variation~="black"]::after,
[data-tooltip][data-variation~="black"]::before {
  background: #1b1c1d;
  color: #fff;
  border: none;
}

.ui.ui.ui.black.loading.popup::before,
.ui.black.loading.popup {
  background: #65696c;
}

.ui.ui.ui.black.popup::before,
[data-tooltip][data-variation~="black"]::before {
  box-shadow: none;
}

.ui.ui.ui.inverted.black.popup::before,
.ui.inverted.black.popup,
[data-tooltip][data-inverted][data-variation~="black"]::after,
[data-tooltip][data-inverted][data-variation~="black"]::before {
  background: #545454;
  color: #1b1c1d;
}

.ui.ui.ui.inverted.black.loading.popup::before,
.ui.inverted.black.loading.popup {
  background: #080808;
}

/* --------------
         Flowing
    --------------- */

.ui.flowing.popup {
  max-width: none;
}

/* --------------
     Sizes
--------------- */

.ui.popup {
  font-size: 1rem;
}

.ui.mini.popup {
  font-size: 0.78571429rem;
}

[data-tooltip][data-variation~="mini"]::before,
[data-tooltip][data-variation~="mini"]::after {
  font-size: 0.78571429rem;
}

.ui.tiny.popup {
  font-size: 0.85714286rem;
}

[data-tooltip][data-variation~="tiny"]::before,
[data-tooltip][data-variation~="tiny"]::after {
  font-size: 0.85714286rem;
}

.ui.small.popup {
  font-size: 0.92857143rem;
}

[data-tooltip][data-variation~="small"]::before,
[data-tooltip][data-variation~="small"]::after {
  font-size: 0.92857143rem;
}

.ui.large.popup {
  font-size: 1.14285714rem;
}

[data-tooltip][data-variation~="large"]::before,
[data-tooltip][data-variation~="large"]::after {
  font-size: 1.14285714rem;
}

.ui.big.popup {
  font-size: 1.28571429rem;
}

[data-tooltip][data-variation~="big"]::before,
[data-tooltip][data-variation~="big"]::after {
  font-size: 1.28571429rem;
}

.ui.huge.popup {
  font-size: 1.42857143rem;
}

[data-tooltip][data-variation~="huge"]::before,
[data-tooltip][data-variation~="huge"]::after {
  font-size: 1.42857143rem;
}

.ui.massive.popup {
  font-size: 1.71428571rem;
}

[data-tooltip][data-variation~="massive"]::before,
[data-tooltip][data-variation~="massive"]::after {
  font-size: 1.71428571rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Progress
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Progress
*******************************/

.ui.progress {
  position: relative;
  display: flex;
  max-width: 100%;
  border: none;
  margin: 1em 0 2.5em;
  box-shadow: none;
  background: rgba(0, 0, 0, 0.1);
  padding: 0;
  border-radius: 0.28571429rem;
}

.ui.progress:first-child {
  margin: 0 0 2.5em;
}

.ui.progress:last-child {
  margin: 0 0 1.5em;
}

/*******************************
            Content
*******************************/

/* Activity Bar */

.ui.progress .bar {
  display: block;
  line-height: 1;
  position: relative;
  width: 0;
  min-width: 2em;
  background: #888;
  border-radius: 0.28571429rem;
  transition: width 0.1s ease, background-color 0.1s ease;
  overflow: hidden;
}

.ui.ui.ui.progress:not([data-percent]):not(.indeterminate) .bar,
.ui.ui.ui.progress[data-percent="0"]:not(.indeterminate) .bar {
  background: transparent;
}

.ui.progress[data-percent="0"] .bar .progress {
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.progress[data-percent="0"] .bar .progress {
  color: rgba(255, 255, 255, 0.9);
}

/* Percent Complete */

.ui.progress .bar > .progress {
  white-space: nowrap;
  position: absolute;
  width: auto;
  font-size: 0.92857143em;
  top: 50%;
  right: 0.5em;
  left: auto;
  bottom: auto;
  color: rgba(255, 255, 255, 0.7);
  text-shadow: none;
  margin-top: -0.5em;
  font-weight: bold;
  text-align: left;
}

.ui.right.aligned.progress {
  justify-content: flex-end;
}

.ui.right.aligned.progress .bar > .progress {
  left: 0.5em;
  right: auto;
}

/* Label */

.ui.progress > .label {
  position: absolute;
  width: 100%;
  font-size: 1em;
  top: 100%;
  right: auto;
  left: 0;
  bottom: auto;
  color: rgba(0, 0, 0, 0.87);
  font-weight: bold;
  text-shadow: none;
  margin-top: 0.2em;
  text-align: center;
  transition: color 0.4s ease;
}

/*******************************
            Types
*******************************/

/* Indicating */

.ui.indicating.progress[data-percent^="1"] .bar,
.ui.indicating.progress[data-percent^="2"] .bar {
  background-color: #d95c5c;
}

.ui.indicating.progress[data-percent^="3"] .bar {
  background-color: #efbc72;
}

.ui.indicating.progress[data-percent^="4"] .bar,
.ui.indicating.progress[data-percent^="5"] .bar {
  background-color: #e6bb48;
}

.ui.indicating.progress[data-percent^="6"] .bar {
  background-color: #ddc928;
}

.ui.indicating.progress[data-percent^="7"] .bar,
.ui.indicating.progress[data-percent^="8"] .bar {
  background-color: #b4d95c;
}

.ui.indicating.progress[data-percent^="9"] .bar,
.ui.indicating.progress[data-percent^="100"] .bar {
  background-color: #66da81;
}

/* Indicating Label */

.ui.indicating.progress[data-percent^="1"] .label,
.ui.indicating.progress[data-percent^="2"] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.indicating.progress[data-percent^="3"] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.indicating.progress[data-percent^="4"] .label,
.ui.indicating.progress[data-percent^="5"] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.indicating.progress[data-percent^="6"] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.indicating.progress[data-percent^="7"] .label,
.ui.indicating.progress[data-percent^="8"] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.indicating.progress[data-percent^="9"] .label,
.ui.indicating.progress[data-percent^="100"] .label {
  color: rgba(0, 0, 0, 0.87);
}

/* Inverted Indicating Label */

.ui.inverted.indicating.progress[data-percent^="1"] .label,
.ui.inverted.indicating.progress[data-percent^="2"] .label {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.indicating.progress[data-percent^="3"] .label {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.indicating.progress[data-percent^="4"] .label,
.ui.inverted.indicating.progress[data-percent^="5"] .label {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.indicating.progress[data-percent^="6"] .label {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.indicating.progress[data-percent^="7"] .label,
.ui.inverted.indicating.progress[data-percent^="8"] .label {
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.indicating.progress[data-percent^="9"] .label,
.ui.inverted.indicating.progress[data-percent^="100"] .label {
  color: rgba(255, 255, 255, 0.9);
}

/* Single Digits */

.ui.indicating.progress[data-percent="1"] .bar,
.ui.indicating.progress[data-percent^="1."] .bar,
.ui.indicating.progress[data-percent="2"] .bar,
.ui.indicating.progress[data-percent^="2."] .bar,
.ui.indicating.progress[data-percent="3"] .bar,
.ui.indicating.progress[data-percent^="3."] .bar,
.ui.indicating.progress[data-percent="4"] .bar,
.ui.indicating.progress[data-percent^="4."] .bar,
.ui.indicating.progress[data-percent="5"] .bar,
.ui.indicating.progress[data-percent^="5."] .bar,
.ui.indicating.progress[data-percent="6"] .bar,
.ui.indicating.progress[data-percent^="6."] .bar,
.ui.indicating.progress[data-percent="7"] .bar,
.ui.indicating.progress[data-percent^="7."] .bar,
.ui.indicating.progress[data-percent="8"] .bar,
.ui.indicating.progress[data-percent^="8."] .bar,
.ui.indicating.progress[data-percent="9"] .bar,
.ui.indicating.progress[data-percent^="9."] .bar {
  background-color: #d95c5c;
}

.ui.indicating.progress[data-percent="0"] .label,
.ui.indicating.progress[data-percent^="0."] .label,
.ui.indicating.progress[data-percent="1"] .label,
.ui.indicating.progress[data-percent^="1."] .label,
.ui.indicating.progress[data-percent="2"] .label,
.ui.indicating.progress[data-percent^="2."] .label,
.ui.indicating.progress[data-percent="3"] .label,
.ui.indicating.progress[data-percent^="3."] .label,
.ui.indicating.progress[data-percent="4"] .label,
.ui.indicating.progress[data-percent^="4."] .label,
.ui.indicating.progress[data-percent="5"] .label,
.ui.indicating.progress[data-percent^="5."] .label,
.ui.indicating.progress[data-percent="6"] .label,
.ui.indicating.progress[data-percent^="6."] .label,
.ui.indicating.progress[data-percent="7"] .label,
.ui.indicating.progress[data-percent^="7."] .label,
.ui.indicating.progress[data-percent="8"] .label,
.ui.indicating.progress[data-percent^="8."] .label,
.ui.indicating.progress[data-percent="9"] .label,
.ui.indicating.progress[data-percent^="9."] .label {
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.indicating.progress[data-percent="0"] .label,
.ui.inverted.indicating.progress[data-percent^="0."] .label,
.ui.inverted.indicating.progress[data-percent="1"] .label,
.ui.inverted.indicating.progress[data-percent^="1."] .label,
.ui.inverted.indicating.progress[data-percent="2"] .label,
.ui.inverted.indicating.progress[data-percent^="2."] .label,
.ui.inverted.indicating.progress[data-percent="3"] .label,
.ui.inverted.indicating.progress[data-percent^="3."] .label,
.ui.inverted.indicating.progress[data-percent="4"] .label,
.ui.inverted.indicating.progress[data-percent^="4."] .label,
.ui.inverted.indicating.progress[data-percent="5"] .label,
.ui.inverted.indicating.progress[data-percent^="5."] .label,
.ui.inverted.indicating.progress[data-percent="6"] .label,
.ui.inverted.indicating.progress[data-percent^="6."] .label,
.ui.inverted.indicating.progress[data-percent="7"] .label,
.ui.inverted.indicating.progress[data-percent^="7."] .label,
.ui.inverted.indicating.progress[data-percent="8"] .label,
.ui.inverted.indicating.progress[data-percent^="8."] .label,
.ui.inverted.indicating.progress[data-percent="9"] .label,
.ui.inverted.indicating.progress[data-percent^="9."] .label {
  color: rgba(255, 255, 255, 0.9);
}

/* Indicating Success */

.ui.ui.indicating.progress.success .label {
  color: #1a531b;
}

/*******************************
             States
*******************************/

/* --------------
         Success
    --------------- */

.ui.ui.progress.success .bar {
  background-color: #21ba45;
}

.ui.ui.progress.success .bar,
.ui.ui.progress.success .bar::after {
  animation: none;
}

.ui.progress.success > .label {
  color: #1a531b;
}

/* --------------
         Warning
    --------------- */

.ui.ui.progress.warning .bar {
  background-color: #f2c037;
}

.ui.ui.progress.warning .bar,
.ui.ui.progress.warning .bar::after {
  animation: none;
}

.ui.progress.warning > .label {
  color: #794b02;
}

/* --------------
         Error
    --------------- */

.ui.ui.progress.error .bar {
  background-color: #db2828;
}

.ui.ui.progress.error .bar,
.ui.ui.progress.error .bar::after {
  animation: none;
}

.ui.progress.error > .label {
  color: #912d2b;
}

/* --------------
         Active
    --------------- */

.ui.active.progress .bar {
  position: relative;
  min-width: 2em;
}

.ui.active.progress .bar::after {
  content: "";
  opacity: 0;
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background: #fff;
  border-radius: 0.28571429rem;
  animation: progress-active 2s ease infinite;
  transform-origin: left;
}

.ui.active.right.aligned.progress .bar::after {
  transform-origin: right;
}

@keyframes progress-active {
  0% {
    opacity: 0.3;
    transform: scale(0, 1);
  }

  100% {
    opacity: 0;
    transform: scale(1);
  }
}

/* --------------
        Disabled
    --------------- */

.ui.disabled.progress {
  opacity: 0.35;
}

.ui.ui.disabled.progress .bar,
.ui.ui.disabled.progress .bar::after {
  animation: none;
}

/*******************************
           Variations
*******************************/

/* --------------
        Inverted
    --------------- */

.ui.inverted.progress {
  background: rgba(255, 255, 255, 0.08);
  border: none;
}

.ui.inverted.progress .bar {
  background: #888;
}

.ui.inverted.progress .bar > .progress {
  color: #1b1c1d;
}

.ui.inverted.progress > .label {
  color: #fff;
}

.ui.inverted.progress.success > .label {
  color: #21ba45;
}

.ui.inverted.progress.warning > .label {
  color: #f2c037;
}

.ui.inverted.progress.error > .label {
  color: #db2828;
}

/* --------------
        Attached
    --------------- */

/* bottom attached */

.ui.progress.attached {
  background: transparent;
  position: relative;
  border: none;
  margin: 0;
}

.ui.progress.attached,
.ui.progress.attached .bar {
  height: 0.2rem;
  padding: 0;
  overflow: hidden;
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

.ui.progress.attached .bar {
  border-radius: 0;
}

/* top attached */

.ui.progress.top.attached,
.ui.progress.top.attached .bar {
  top: 0;
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.progress.top.attached .bar {
  border-radius: 0;
}

/* Coupling */

.ui.segment > .ui.attached.progress,
.ui.card > .ui.attached.progress {
  position: absolute;
  top: auto;
  left: 0;
  bottom: 100%;
  width: 100%;
}

.ui.segment > .ui.bottom.attached.progress,
.ui.card > .ui.bottom.attached.progress {
  top: 100%;
  bottom: auto;
}

/* --------------
     Colors
--------------- */

.ui.indeterminate.primary.progress .bar::before,
.ui.primary.progress .bar,
.ui.progress .primary.bar {
  background-color: #2185d0;
}

.ui.inverted.indeterminate.primary.progress .bar::before,
.ui.primary.inverted.progress .bar,
.ui.inverted.progress .primary.bar {
  background-color: #54c8ff;
}

.ui.indeterminate.secondary.progress .bar::before,
.ui.secondary.progress .bar,
.ui.progress .secondary.bar {
  background-color: #1b1c1d;
}

.ui.inverted.indeterminate.secondary.progress .bar::before,
.ui.secondary.inverted.progress .bar,
.ui.inverted.progress .secondary.bar {
  background-color: #545454;
}

.ui.indeterminate.red.progress .bar::before,
.ui.red.progress .bar,
.ui.progress .red.bar {
  background-color: #db2828;
}

.ui.inverted.indeterminate.red.progress .bar::before,
.ui.red.inverted.progress .bar,
.ui.inverted.progress .red.bar {
  background-color: #ff695e;
}

.ui.indeterminate.orange.progress .bar::before,
.ui.orange.progress .bar,
.ui.progress .orange.bar {
  background-color: #f2711c;
}

.ui.inverted.indeterminate.orange.progress .bar::before,
.ui.orange.inverted.progress .bar,
.ui.inverted.progress .orange.bar {
  background-color: #ff851b;
}

.ui.indeterminate.yellow.progress .bar::before,
.ui.yellow.progress .bar,
.ui.progress .yellow.bar {
  background-color: #fbbd08;
}

.ui.inverted.indeterminate.yellow.progress .bar::before,
.ui.yellow.inverted.progress .bar,
.ui.inverted.progress .yellow.bar {
  background-color: #ffe21f;
}

.ui.indeterminate.olive.progress .bar::before,
.ui.olive.progress .bar,
.ui.progress .olive.bar {
  background-color: #b5cc18;
}

.ui.inverted.indeterminate.olive.progress .bar::before,
.ui.olive.inverted.progress .bar,
.ui.inverted.progress .olive.bar {
  background-color: #d9e778;
}

.ui.indeterminate.green.progress .bar::before,
.ui.green.progress .bar,
.ui.progress .green.bar {
  background-color: #21ba45;
}

.ui.inverted.indeterminate.green.progress .bar::before,
.ui.green.inverted.progress .bar,
.ui.inverted.progress .green.bar {
  background-color: #2ecc40;
}

.ui.indeterminate.teal.progress .bar::before,
.ui.teal.progress .bar,
.ui.progress .teal.bar {
  background-color: #00b5ad;
}

.ui.inverted.indeterminate.teal.progress .bar::before,
.ui.teal.inverted.progress .bar,
.ui.inverted.progress .teal.bar {
  background-color: #6dffff;
}

.ui.indeterminate.blue.progress .bar::before,
.ui.blue.progress .bar,
.ui.progress .blue.bar {
  background-color: #2185d0;
}

.ui.inverted.indeterminate.blue.progress .bar::before,
.ui.blue.inverted.progress .bar,
.ui.inverted.progress .blue.bar {
  background-color: #54c8ff;
}

.ui.indeterminate.violet.progress .bar::before,
.ui.violet.progress .bar,
.ui.progress .violet.bar {
  background-color: #6435c9;
}

.ui.inverted.indeterminate.violet.progress .bar::before,
.ui.violet.inverted.progress .bar,
.ui.inverted.progress .violet.bar {
  background-color: #a291fb;
}

.ui.indeterminate.purple.progress .bar::before,
.ui.purple.progress .bar,
.ui.progress .purple.bar {
  background-color: #a333c8;
}

.ui.inverted.indeterminate.purple.progress .bar::before,
.ui.purple.inverted.progress .bar,
.ui.inverted.progress .purple.bar {
  background-color: #dc73ff;
}

.ui.indeterminate.pink.progress .bar::before,
.ui.pink.progress .bar,
.ui.progress .pink.bar {
  background-color: #e03997;
}

.ui.inverted.indeterminate.pink.progress .bar::before,
.ui.pink.inverted.progress .bar,
.ui.inverted.progress .pink.bar {
  background-color: #ff8edf;
}

.ui.indeterminate.brown.progress .bar::before,
.ui.brown.progress .bar,
.ui.progress .brown.bar {
  background-color: #a5673f;
}

.ui.inverted.indeterminate.brown.progress .bar::before,
.ui.brown.inverted.progress .bar,
.ui.inverted.progress .brown.bar {
  background-color: #d67c1c;
}

.ui.indeterminate.grey.progress .bar::before,
.ui.grey.progress .bar,
.ui.progress .grey.bar {
  background-color: #767676;
}

.ui.inverted.indeterminate.grey.progress .bar::before,
.ui.grey.inverted.progress .bar,
.ui.inverted.progress .grey.bar {
  background-color: #dcddde;
}

.ui.indeterminate.black.progress .bar::before,
.ui.black.progress .bar,
.ui.progress .black.bar {
  background-color: #1b1c1d;
}

.ui.inverted.indeterminate.black.progress .bar::before,
.ui.black.inverted.progress .bar,
.ui.inverted.progress .black.bar {
  background-color: #545454;
}

/* --------------
     Sizes
--------------- */

.ui.progress {
  font-size: 1rem;
}

.ui.progress .bar {
  height: 1.75em;
}

.ui.mini.progress {
  font-size: 0.78571429rem;
}

.ui.mini.progress .bar {
  height: 0.3em;
}

.ui.tiny.progress {
  font-size: 0.85714286rem;
}

.ui.tiny.progress .bar {
  height: 0.5em;
}

.ui.small.progress {
  font-size: 0.92857143rem;
}

.ui.small.progress .bar {
  height: 1em;
}

.ui.large.progress {
  font-size: 1.14285714rem;
}

.ui.large.progress .bar {
  height: 2.5em;
}

.ui.big.progress {
  font-size: 1.28571429rem;
}

.ui.big.progress .bar {
  height: 3.5em;
}

.ui.huge.progress {
  font-size: 1.42857143rem;
}

.ui.huge.progress .bar {
  height: 4em;
}

.ui.massive.progress {
  font-size: 1.71428571rem;
}

.ui.massive.progress .bar {
  height: 5em;
}

/* ---------------
      Indeterminate
    ---------------- */

.ui.indeterminate.progress .bar {
  width: 100%;
}

.ui.indeterminate.progress .bar .progress,
.ui.progress .bar .centered.progress {
  text-align: center;
  position: relative;
  right: 0;
}

.ui.indeterminate.progress .bar::before {
  content: "";
  position: absolute;
  top: 0;
  bottom: 0;
  border-radius: 0.28571429rem;
  animation: progress-pulsating 2s ease infinite;
  transform-origin: center;
  width: 100%;
}

.ui.slow.indeterminate.progress .bar::before {
  animation-duration: 4s;
}

.ui.fast.indeterminate.progress .bar::before {
  animation-duration: 1s;
}

.ui.swinging.indeterminate.progress .bar::before {
  transform-origin: left;
  animation-name: progress-swinging;
}

.ui.sliding.indeterminate.progress .bar::before {
  transform-origin: left;
  animation-name: progress-sliding;
}

.ui.filling.indeterminate.progress .bar::before {
  animation-name: progress-filling;
}

.ui.indeterminate.progress:not(.sliding):not(.filling):not(.swinging) .bar::before {
  background: #fff;
}

.ui.sliding.indeterminate.progress .bar,
.ui.swinging.indeterminate.progress .bar,
.ui.filling.indeterminate.progress .bar {
  background: rgba(0, 0, 0, 0.1);
}

.ui.sliding.indeterminate.progress .bar .progress,
.ui.swinging.indeterminate.progress .bar .progress {
  color: #1b1c1d;
}

.ui.inverted.sliding.indeterminate.progress .bar,
.ui.inverted.swinging.indeterminate.progress .bar,
.ui.inverted.filling.indeterminate.progress .bar {
  background: rgba(255, 255, 255, 0.08);
}

.ui.inverted.sliding.indeterminate.progress .bar .progress,
.ui.inverted.swinging.indeterminate.progress .bar .progress {
  color: rgba(255, 255, 255, 0.7);
}

@keyframes progress-swinging {
  0%, 100% {
    width: 10%;
    left: -25%;
  }

  25%, 65% {
    width: 70%;
  }

  50% {
    width: 10%;
    left: 100%;
  }
}

@keyframes progress-sliding {
  0% {
    width: 10%;
    left: -25%;
  }

  50% {
    width: 70%;
  }

  100% {
    width: 10%;
    left: 100%;
  }
}

@keyframes progress-filling {
  0% {
    transform: scale(0, 1);
  }

  80% {
    transform: scale(1);
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
}

@keyframes progress-pulsating {
  0% {
    transform: scale(0, 1);
    opacity: 0.7;
  }

  100% {
    transform: scale(1);
    opacity: 0;
  }
}

/*******************************
            Progress
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Slider
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

.ui.slider:not(.vertical):not(.checkbox) {
  width: 100%;
  padding: 1em 0.5em;
}

.ui.slider:not(.checkbox) {
  position: relative;
}

.ui.slider:not(.checkbox):focus {
  outline: 0;
}

.ui.slider .inner {
  position: relative;
  z-index: 2;
}

.ui.slider:not(.vertical) .inner {
  height: 1.5em;
}

.ui.slider .inner:hover {
  cursor: auto;
}

.ui.slider .inner .track {
  position: absolute;
  border-radius: 4px;
  background-color: rgba(0, 0, 0, 0.05);
}

.ui.slider:not(.vertical) .inner .track {
  width: 100%;
  height: 0.4em;
  top: 0.55em;
  left: 0;
}

.ui.slider .inner .track-fill {
  position: absolute;
  border-radius: 4px;
  background-color: #1b1c1d;
}

.ui.slider:not(.vertical) .inner .track-fill {
  height: 0.4em;
  top: 0.55em;
  left: 0;
}

.ui.slider .inner .thumb {
  position: absolute;
  left: 0;
  top: 0;
  height: 1.5em;
  width: 1.5em;
  background: #fff linear-gradient(transparent, rgba(0, 0, 0, 0.05));
  border-radius: 100%;
  box-shadow: 0 1px 2px 0 rgba(34, 36, 38, 0.15), 0 0 0 1px rgba(34, 36, 38, 0.15) inset;
  transition: background 0.3s ease;
}

.ui.slider:not(.disabled) .inner .thumb:hover {
  cursor: pointer;
  background: #f2f2f2 linear-gradient(transparent, rgba(0, 0, 0, 0.05));
}

.ui.slider:not(.disabled):focus .inner .thumb {
  background: #f2f2f2 linear-gradient(transparent, rgba(0, 0, 0, 0.05));
}

/*******************************
            States
*******************************/

/* --------------
         Disabled
    --------------- */

.ui.disabled.slider:not(.checkbox) {
  opacity: 0.5;
}

.ui.disabled.slider .inner:hover {
  cursor: auto;
}

.ui.disabled.slider .inner .track-fill {
  background: #ccc;
}

/* --------------
        Reversed
    --------------- */

.ui.reversed.slider .inner .track-fill {
  left: auto;
  right: 0;
}

.ui.reversed.slider:not(.vertical) .inner .thumb {
  left: auto;
  right: 0;
}

.ui.reversed.vertical.slider .inner .thumb {
  left: 0.03em;
}

.ui.labeled.reversed.slider > .labels .label {
  transform: translate(-100%, -100%);
}

/*******************************
           Variations
*******************************/

/* --------------
        Vertical
    --------------- */

.ui.vertical.slider {
  height: 100%;
  width: 1.5em;
  padding: 0.5em 1em;
}

.ui.vertical.slider .inner {
  height: 100%;
}

.ui.vertical.slider .inner .track {
  height: 100%;
  width: 0.4em;
  left: 0.55em;
  top: 0;
}

.ui.vertical.slider .inner .track-fill {
  width: 0.4em;
  left: 0.55em;
  top: 0;
}

/* Vertical Reversed */

.ui.vertical.reversed.slider .inner .thumb {
  top: auto;
  bottom: 0;
}

.ui.vertical.reversed.slider .inner .track-fill {
  top: auto;
  bottom: 0;
}

/* --------------
        Labeled
    --------------- */

.ui.labeled.slider > .labels {
  height: 1.5em;
  width: auto;
  margin: 0;
  padding: 0;
  position: absolute;
  top: 50%;
  left: 0;
  right: 0;
}

.ui.labeled.slider:not(.vertical) > .labels {
  transform: translateY(-50%);
}

.ui.labeled.slider > .labels .label {
  display: inline-flex;
  padding: 0.2em 0;
  position: absolute;
  transform: translate(-50%, -100%);
  white-space: nowrap;
}

.ui.bottom.aligned.labeled.slider > .labels .label {
  bottom: 0;
  transform: translate(-50%, 100%);
}

.ui.labeled.ticked.slider > .labels .label::after {
  content: " ";
  height: 1.5em;
  width: 1px;
  background: #ccc;
  position: absolute;
  top: 100%;
  left: 50%;
}

.ui.bottom.aligned.labeled.ticked.slider > .labels .label::after {
  top: auto;
  bottom: 100%;
}

.ui.labeled.ticked.slider > .labels .halftick.label::after {
  height: 0.75em;
}

/* Vertical Labels */

.ui.labeled.vertical.slider > .labels {
  width: 1.5em;
  height: auto;
  left: 50%;
  top: 0;
  bottom: 0;
  transform: translateX(-50%);
}

.ui.labeled.vertical.slider > .labels .label {
  transform: translate(-100%, -50%);
}

.ui.labeled.vertical.slider > .labels .label::after {
  width: 1.5em;
  height: 1px;
  left: 100%;
  top: 50%;
}

.ui.labeled.vertical.slider > .labels .halftick.label::after {
  width: 0.75em;
  height: 1px;
}

/* Vertical Reversed Labels */

.ui.labeled.vertical.reversed.slider > .labels .label {
  transform: translate(-100%, 50%);
}

/* --------------
    Hover
--------------- */

.ui.hover.slider .inner .thumb {
  opacity: 0;
  transition: opacity 0.2s linear;
}

.ui.hover.slider:not(.disabled):hover .inner .thumb,
.ui.hover.slider:not(.disabled):focus .inner .thumb {
  opacity: 1;
}

/* --------------
        Inverted
    --------------- */

.ui.inverted.slider .inner .track-fill {
  background-color: #545454;
}

.ui.inverted.slider .inner .track {
  background-color: rgba(255, 255, 255, 0.08);
}

/* --------------
     Colors
--------------- */

/* Standard */

.ui.primary.slider .inner .track-fill {
  background-color: #2185d0;
}

.ui.primary.inverted.slider .inner .track-fill {
  background-color: #54c8ff;
}

/* Basic */

.ui.primary.slider.basic .inner .thumb {
  background-color: #2185d0;
}

.ui.primary.slider.basic .inner .thumb:hover,
.ui.primary.slider.basic:focus .inner .thumb {
  background-color: #1678c2;
}

/* Basic Inverted */

.ui.primary.inverted.slider.basic .inner .thumb {
  background-color: #54c8ff;
}

.ui.primary.inverted.slider.basic .inner .thumb:hover,
.ui.primary.inverted.slider.basic:focus .inner .thumb {
  background-color: #21b8ff;
}

/* Standard */

.ui.secondary.slider .inner .track-fill {
  background-color: #1b1c1d;
}

.ui.secondary.inverted.slider .inner .track-fill {
  background-color: #545454;
}

/* Basic */

.ui.secondary.slider.basic .inner .thumb {
  background-color: #1b1c1d;
}

.ui.secondary.slider.basic .inner .thumb:hover,
.ui.secondary.slider.basic:focus .inner .thumb {
  background-color: #27292a;
}

/* Basic Inverted */

.ui.secondary.inverted.slider.basic .inner .thumb {
  background-color: #545454;
}

.ui.secondary.inverted.slider.basic .inner .thumb:hover,
.ui.secondary.inverted.slider.basic:focus .inner .thumb {
  background-color: #6e6e6e;
}

/* Standard */

.ui.red.slider .inner .track-fill {
  background-color: #db2828;
}

.ui.red.inverted.slider .inner .track-fill {
  background-color: #ff695e;
}

/* Basic */

.ui.red.slider.basic .inner .thumb {
  background-color: #db2828;
}

.ui.red.slider.basic .inner .thumb:hover,
.ui.red.slider.basic:focus .inner .thumb {
  background-color: #d01919;
}

/* Basic Inverted */

.ui.red.inverted.slider.basic .inner .thumb {
  background-color: #ff695e;
}

.ui.red.inverted.slider.basic .inner .thumb:hover,
.ui.red.inverted.slider.basic:focus .inner .thumb {
  background-color: #ff392b;
}

/* Standard */

.ui.orange.slider .inner .track-fill {
  background-color: #f2711c;
}

.ui.orange.inverted.slider .inner .track-fill {
  background-color: #ff851b;
}

/* Basic */

.ui.orange.slider.basic .inner .thumb {
  background-color: #f2711c;
}

.ui.orange.slider.basic .inner .thumb:hover,
.ui.orange.slider.basic:focus .inner .thumb {
  background-color: #f26202;
}

/* Basic Inverted */

.ui.orange.inverted.slider.basic .inner .thumb {
  background-color: #ff851b;
}

.ui.orange.inverted.slider.basic .inner .thumb:hover,
.ui.orange.inverted.slider.basic:focus .inner .thumb {
  background-color: #e76b00;
}

/* Standard */

.ui.yellow.slider .inner .track-fill {
  background-color: #fbbd08;
}

.ui.yellow.inverted.slider .inner .track-fill {
  background-color: #ffe21f;
}

/* Basic */

.ui.yellow.slider.basic .inner .thumb {
  background-color: #fbbd08;
}

.ui.yellow.slider.basic .inner .thumb:hover,
.ui.yellow.slider.basic:focus .inner .thumb {
  background-color: #eaae00;
}

/* Basic Inverted */

.ui.yellow.inverted.slider.basic .inner .thumb {
  background-color: #ffe21f;
}

.ui.yellow.inverted.slider.basic .inner .thumb:hover,
.ui.yellow.inverted.slider.basic:focus .inner .thumb {
  background-color: #ebcd00;
}

/* Standard */

.ui.olive.slider .inner .track-fill {
  background-color: #b5cc18;
}

.ui.olive.inverted.slider .inner .track-fill {
  background-color: #d9e778;
}

/* Basic */

.ui.olive.slider.basic .inner .thumb {
  background-color: #b5cc18;
}

.ui.olive.slider.basic .inner .thumb:hover,
.ui.olive.slider.basic:focus .inner .thumb {
  background-color: #a7bd0d;
}

/* Basic Inverted */

.ui.olive.inverted.slider.basic .inner .thumb {
  background-color: #d9e778;
}

.ui.olive.inverted.slider.basic .inner .thumb:hover,
.ui.olive.inverted.slider.basic:focus .inner .thumb {
  background-color: #d2e745;
}

/* Standard */

.ui.green.slider .inner .track-fill {
  background-color: #21ba45;
}

.ui.green.inverted.slider .inner .track-fill {
  background-color: #2ecc40;
}

/* Basic */

.ui.green.slider.basic .inner .thumb {
  background-color: #21ba45;
}

.ui.green.slider.basic .inner .thumb:hover,
.ui.green.slider.basic:focus .inner .thumb {
  background-color: #16ab39;
}

/* Basic Inverted */

.ui.green.inverted.slider.basic .inner .thumb {
  background-color: #2ecc40;
}

.ui.green.inverted.slider.basic .inner .thumb:hover,
.ui.green.inverted.slider.basic:focus .inner .thumb {
  background-color: #1ea92e;
}

/* Standard */

.ui.teal.slider .inner .track-fill {
  background-color: #00b5ad;
}

.ui.teal.inverted.slider .inner .track-fill {
  background-color: #6dffff;
}

/* Basic */

.ui.teal.slider.basic .inner .thumb {
  background-color: #00b5ad;
}

.ui.teal.slider.basic .inner .thumb:hover,
.ui.teal.slider.basic:focus .inner .thumb {
  background-color: #009c95;
}

/* Basic Inverted */

.ui.teal.inverted.slider.basic .inner .thumb {
  background-color: #6dffff;
}

.ui.teal.inverted.slider.basic .inner .thumb:hover,
.ui.teal.inverted.slider.basic:focus .inner .thumb {
  background-color: #3affff;
}

/* Standard */

.ui.blue.slider .inner .track-fill {
  background-color: #2185d0;
}

.ui.blue.inverted.slider .inner .track-fill {
  background-color: #54c8ff;
}

/* Basic */

.ui.blue.slider.basic .inner .thumb {
  background-color: #2185d0;
}

.ui.blue.slider.basic .inner .thumb:hover,
.ui.blue.slider.basic:focus .inner .thumb {
  background-color: #1678c2;
}

/* Basic Inverted */

.ui.blue.inverted.slider.basic .inner .thumb {
  background-color: #54c8ff;
}

.ui.blue.inverted.slider.basic .inner .thumb:hover,
.ui.blue.inverted.slider.basic:focus .inner .thumb {
  background-color: #21b8ff;
}

/* Standard */

.ui.violet.slider .inner .track-fill {
  background-color: #6435c9;
}

.ui.violet.inverted.slider .inner .track-fill {
  background-color: #a291fb;
}

/* Basic */

.ui.violet.slider.basic .inner .thumb {
  background-color: #6435c9;
}

.ui.violet.slider.basic .inner .thumb:hover,
.ui.violet.slider.basic:focus .inner .thumb {
  background-color: #5829bb;
}

/* Basic Inverted */

.ui.violet.inverted.slider.basic .inner .thumb {
  background-color: #a291fb;
}

.ui.violet.inverted.slider.basic .inner .thumb:hover,
.ui.violet.inverted.slider.basic:focus .inner .thumb {
  background-color: #745aff;
}

/* Standard */

.ui.purple.slider .inner .track-fill {
  background-color: #a333c8;
}

.ui.purple.inverted.slider .inner .track-fill {
  background-color: #dc73ff;
}

/* Basic */

.ui.purple.slider.basic .inner .thumb {
  background-color: #a333c8;
}

.ui.purple.slider.basic .inner .thumb:hover,
.ui.purple.slider.basic:focus .inner .thumb {
  background-color: #9627ba;
}

/* Basic Inverted */

.ui.purple.inverted.slider.basic .inner .thumb {
  background-color: #dc73ff;
}

.ui.purple.inverted.slider.basic .inner .thumb:hover,
.ui.purple.inverted.slider.basic:focus .inner .thumb {
  background-color: #cf40ff;
}

/* Standard */

.ui.pink.slider .inner .track-fill {
  background-color: #e03997;
}

.ui.pink.inverted.slider .inner .track-fill {
  background-color: #ff8edf;
}

/* Basic */

.ui.pink.slider.basic .inner .thumb {
  background-color: #e03997;
}

.ui.pink.slider.basic .inner .thumb:hover,
.ui.pink.slider.basic:focus .inner .thumb {
  background-color: #e61a8d;
}

/* Basic Inverted */

.ui.pink.inverted.slider.basic .inner .thumb {
  background-color: #ff8edf;
}

.ui.pink.inverted.slider.basic .inner .thumb:hover,
.ui.pink.inverted.slider.basic:focus .inner .thumb {
  background-color: #ff5bd1;
}

/* Standard */

.ui.brown.slider .inner .track-fill {
  background-color: #a5673f;
}

.ui.brown.inverted.slider .inner .track-fill {
  background-color: #d67c1c;
}

/* Basic */

.ui.brown.slider.basic .inner .thumb {
  background-color: #a5673f;
}

.ui.brown.slider.basic .inner .thumb:hover,
.ui.brown.slider.basic:focus .inner .thumb {
  background-color: #975b33;
}

/* Basic Inverted */

.ui.brown.inverted.slider.basic .inner .thumb {
  background-color: #d67c1c;
}

.ui.brown.inverted.slider.basic .inner .thumb:hover,
.ui.brown.inverted.slider.basic:focus .inner .thumb {
  background-color: #b0620f;
}

/* Standard */

.ui.grey.slider .inner .track-fill {
  background-color: #767676;
}

.ui.grey.inverted.slider .inner .track-fill {
  background-color: #dcddde;
}

/* Basic */

.ui.grey.slider.basic .inner .thumb {
  background-color: #767676;
}

.ui.grey.slider.basic .inner .thumb:hover,
.ui.grey.slider.basic:focus .inner .thumb {
  background-color: #838383;
}

/* Basic Inverted */

.ui.grey.inverted.slider.basic .inner .thumb {
  background-color: #dcddde;
}

.ui.grey.inverted.slider.basic .inner .thumb:hover,
.ui.grey.inverted.slider.basic:focus .inner .thumb {
  background-color: #c2c4c5;
}

/* Standard */

.ui.black.slider .inner .track-fill {
  background-color: #1b1c1d;
}

.ui.black.inverted.slider .inner .track-fill {
  background-color: #545454;
}

/* Basic */

.ui.black.slider.basic .inner .thumb {
  background-color: #1b1c1d;
}

.ui.black.slider.basic .inner .thumb:hover,
.ui.black.slider.basic:focus .inner .thumb {
  background-color: #27292a;
}

/* Basic Inverted */

.ui.black.inverted.slider.basic .inner .thumb {
  background-color: #545454;
}

.ui.black.inverted.slider.basic .inner .thumb:hover,
.ui.black.inverted.slider.basic:focus .inner .thumb {
  background-color: #000000;
}

/* --------------
         Basic
    --------------- */

/* Standard */

.ui.slider.basic .inner .thumb {
  background-color: #1b1c1d;
}

.ui.slider.basic .inner .thumb:hover,
.ui.slider.basic:focus .inner .thumb {
  background-color: #27292a;
}

/* --------------
          Basic Inverted
        --------------- */

/* Standard */

.ui.inverted.slider.basic .inner .thumb {
  background-color: #545454;
}

.ui.inverted.slider.basic .inner .thumb:hover,
.ui.inverted.slider.basic:focus .inner .thumb {
  background-color: #000000;
}

/* --------------
     Sizing
--------------- */

.ui.slider.small .inner .thumb {
  height: 1em;
  width: 1em;
}

.ui.slider.small:not(.vertical) .inner {
  height: 1em;
}

.ui.slider.small:not(.vertical) .inner .track,
.ui.slider.small:not(.vertical) .inner .track-fill {
  height: 0.3em;
  top: 0.35em;
}

.ui.small.labeled.slider:not(.vertical) > .labels,
.ui.small.labeled.slider:not(.vertical) > .labels .label::after {
  height: 1em;
}

.ui.small.labeled.slider:not(.vertical) > .labels .halftick.label::after {
  height: 0.5em;
}

/* Small Vertical */

.ui.slider.small.vertical .inner {
  width: 1em;
}

.ui.slider.small.vertical .inner .track,
.ui.slider.small.vertical .inner .track-fill {
  width: 0.3em;
  left: 0.35em;
}

.ui.small.labeled.vertical.slider > .labels,
.ui.small.labeled.vertical.slider > .labels .label::after {
  width: 1em;
}

.ui.small.labeled.vertical.slider > .labels .halftick.label::after {
  width: 0.5em;
}

.ui.slider.large .inner .thumb {
  height: 2em;
  width: 2em;
}

.ui.slider.large:not(.vertical) .inner {
  height: 2em;
}

.ui.slider.large:not(.vertical) .inner .track,
.ui.slider.large:not(.vertical) .inner .track-fill {
  height: 0.5em;
  top: 0.75em;
}

.ui.large.labeled.slider:not(.vertical) > .labels,
.ui.large.labeled.slider:not(.vertical) > .labels .label::after {
  height: 2em;
}

.ui.large.labeled.slider:not(.vertical) > .labels .halftick.label::after {
  height: 1em;
}

/* Small Vertical */

.ui.slider.large.vertical .inner {
  width: 2em;
}

.ui.slider.large.vertical .inner .track,
.ui.slider.large.vertical .inner .track-fill {
  width: 0.5em;
  left: 0.75em;
}

.ui.large.labeled.vertical.slider > .labels,
.ui.large.labeled.vertical.slider > .labels .label::after {
  width: 2em;
}

.ui.large.labeled.vertical.slider > .labels .halftick.label::after {
  width: 1em;
}

.ui.slider.big .inner .thumb {
  height: 2.5em;
  width: 2.5em;
}

.ui.slider.big:not(.vertical) .inner {
  height: 2.5em;
}

.ui.slider.big:not(.vertical) .inner .track,
.ui.slider.big:not(.vertical) .inner .track-fill {
  height: 0.6em;
  top: 0.95em;
}

.ui.big.labeled.slider:not(.vertical) > .labels,
.ui.big.labeled.slider:not(.vertical) > .labels .label::after {
  height: 2.5em;
}

.ui.big.labeled.slider:not(.vertical) > .labels .halftick.label::after {
  height: 1.25em;
}

/* Small Vertical */

.ui.slider.big.vertical .inner {
  width: 2.5em;
}

.ui.slider.big.vertical .inner .track,
.ui.slider.big.vertical .inner .track-fill {
  width: 0.6em;
  left: 0.95em;
}

.ui.big.labeled.vertical.slider > .labels,
.ui.big.labeled.vertical.slider > .labels .label::after {
  width: 2.5em;
}

.ui.big.labeled.vertical.slider > .labels .halftick.label::after {
  width: 1.25em;
}

/*******************************
        Slider Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Rating
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           Rating
*******************************/

.ui.rating {
  display: inline-flex;
  white-space: nowrap;
  vertical-align: baseline;
}

.ui.rating:last-child {
  margin-right: 0;
}

/* Icon */

.ui.rating .icon {
  padding: 0;
  margin: 0;
  text-align: center;
  font-weight: normal;
  font-style: normal;
  flex: 1 0 auto;
  cursor: pointer;
  width: 1.25em;
  height: auto;
  transition: opacity 0.1s ease, background 0.1s ease, text-shadow 0.1s ease, color 0.1s ease;
  line-height: 1;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

/*******************************
             Types
*******************************/

/* -------------------
      Standard
-------------------- */

/* Inactive Icon */

.ui.rating .icon {
  background: transparent;
  color: rgba(0, 0, 0, 0.15);
}

/* Active Icon */

.ui.rating .active.icon {
  background: transparent;
  color: rgba(0, 0, 0, 0.85);
}

/* Partially Active Icon */

.ui.rating .icon.partial.active {
  background: linear-gradient(to right, rgba(0, 0, 0, 0.85) 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
}

/* central override for colors */

.ui.ui.ui.rating .icon.partial.active {
  -webkit-background-clip: text;
  background-clip: text;
  color: transparent;
  text-shadow: none;
}

/* Selected Icon */

.ui.rating .icon.selected,
.ui.rating .icon.selected.active,
.ui.rating .icon.selected.partial.active {
  background: transparent;
  color: rgba(0, 0, 0, 0.87);
  background-clip: unset;
}

/* --------------
     Colors
-------------- */

.ui.primary.rating .active.icon {
  color: #54c8ff;
  text-shadow: 0 -1px 0 #2185d0, -1px 0 0 #2185d0, 0 1px 0 #2185d0, 1px 0 0 #2185d0;
}

.ui.primary.rating .icon.selected,
.ui.primary.rating .icon.selected.active,
.ui.primary.rating .icon.selected.partial.active {
  background: inherit;
  color: #21b8ff;
  text-shadow: 0 -1px 0 #1678c2, -1px 0 0 #1678c2, 0 1px 0 #1678c2, 1px 0 0 #1678c2;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.primary.rating .icon.partial.active {
  background: linear-gradient(to right, #54c8ff 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #2185d0 0.78px;
}

.ui.secondary.rating .active.icon {
  color: #545454;
  text-shadow: 0 -1px 0 #1b1c1d, -1px 0 0 #1b1c1d, 0 1px 0 #1b1c1d, 1px 0 0 #1b1c1d;
}

.ui.secondary.rating .icon.selected,
.ui.secondary.rating .icon.selected.active,
.ui.secondary.rating .icon.selected.partial.active {
  background: inherit;
  color: #6e6e6e;
  text-shadow: 0 -1px 0 #27292a, -1px 0 0 #27292a, 0 1px 0 #27292a, 1px 0 0 #27292a;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.secondary.rating .icon.partial.active {
  background: linear-gradient(to right, #545454 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #1b1c1d 0.78px;
}

.ui.red.rating .active.icon {
  color: #ff695e;
  text-shadow: 0 -1px 0 #db2828, -1px 0 0 #db2828, 0 1px 0 #db2828, 1px 0 0 #db2828;
}

.ui.red.rating .icon.selected,
.ui.red.rating .icon.selected.active,
.ui.red.rating .icon.selected.partial.active {
  background: inherit;
  color: #ff392b;
  text-shadow: 0 -1px 0 #d01919, -1px 0 0 #d01919, 0 1px 0 #d01919, 1px 0 0 #d01919;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.red.rating .icon.partial.active {
  background: linear-gradient(to right, #ff695e 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #db2828 0.78px;
}

.ui.orange.rating .active.icon {
  color: #ff851b;
  text-shadow: 0 -1px 0 #f2711c, -1px 0 0 #f2711c, 0 1px 0 #f2711c, 1px 0 0 #f2711c;
}

.ui.orange.rating .icon.selected,
.ui.orange.rating .icon.selected.active,
.ui.orange.rating .icon.selected.partial.active {
  background: inherit;
  color: #e76b00;
  text-shadow: 0 -1px 0 #f26202, -1px 0 0 #f26202, 0 1px 0 #f26202, 1px 0 0 #f26202;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.orange.rating .icon.partial.active {
  background: linear-gradient(to right, #ff851b 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #f2711c 0.78px;
}

.ui.yellow.rating .active.icon {
  color: #ffe21f;
  text-shadow: 0 -1px 0 #fbbd08, -1px 0 0 #fbbd08, 0 1px 0 #fbbd08, 1px 0 0 #fbbd08;
}

.ui.yellow.rating .icon.selected,
.ui.yellow.rating .icon.selected.active,
.ui.yellow.rating .icon.selected.partial.active {
  background: inherit;
  color: #ebcd00;
  text-shadow: 0 -1px 0 #eaae00, -1px 0 0 #eaae00, 0 1px 0 #eaae00, 1px 0 0 #eaae00;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.yellow.rating .icon.partial.active {
  background: linear-gradient(to right, #ffe21f 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #fbbd08 0.78px;
}

.ui.olive.rating .active.icon {
  color: #d9e778;
  text-shadow: 0 -1px 0 #b5cc18, -1px 0 0 #b5cc18, 0 1px 0 #b5cc18, 1px 0 0 #b5cc18;
}

.ui.olive.rating .icon.selected,
.ui.olive.rating .icon.selected.active,
.ui.olive.rating .icon.selected.partial.active {
  background: inherit;
  color: #d2e745;
  text-shadow: 0 -1px 0 #a7bd0d, -1px 0 0 #a7bd0d, 0 1px 0 #a7bd0d, 1px 0 0 #a7bd0d;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.olive.rating .icon.partial.active {
  background: linear-gradient(to right, #d9e778 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #b5cc18 0.78px;
}

.ui.green.rating .active.icon {
  color: #2ecc40;
  text-shadow: 0 -1px 0 #21ba45, -1px 0 0 #21ba45, 0 1px 0 #21ba45, 1px 0 0 #21ba45;
}

.ui.green.rating .icon.selected,
.ui.green.rating .icon.selected.active,
.ui.green.rating .icon.selected.partial.active {
  background: inherit;
  color: #1ea92e;
  text-shadow: 0 -1px 0 #16ab39, -1px 0 0 #16ab39, 0 1px 0 #16ab39, 1px 0 0 #16ab39;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.green.rating .icon.partial.active {
  background: linear-gradient(to right, #2ecc40 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #21ba45 0.78px;
}

.ui.teal.rating .active.icon {
  color: #6dffff;
  text-shadow: 0 -1px 0 #00b5ad, -1px 0 0 #00b5ad, 0 1px 0 #00b5ad, 1px 0 0 #00b5ad;
}

.ui.teal.rating .icon.selected,
.ui.teal.rating .icon.selected.active,
.ui.teal.rating .icon.selected.partial.active {
  background: inherit;
  color: #3affff;
  text-shadow: 0 -1px 0 #009c95, -1px 0 0 #009c95, 0 1px 0 #009c95, 1px 0 0 #009c95;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.teal.rating .icon.partial.active {
  background: linear-gradient(to right, #6dffff 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #00b5ad 0.78px;
}

.ui.blue.rating .active.icon {
  color: #54c8ff;
  text-shadow: 0 -1px 0 #2185d0, -1px 0 0 #2185d0, 0 1px 0 #2185d0, 1px 0 0 #2185d0;
}

.ui.blue.rating .icon.selected,
.ui.blue.rating .icon.selected.active,
.ui.blue.rating .icon.selected.partial.active {
  background: inherit;
  color: #21b8ff;
  text-shadow: 0 -1px 0 #1678c2, -1px 0 0 #1678c2, 0 1px 0 #1678c2, 1px 0 0 #1678c2;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.blue.rating .icon.partial.active {
  background: linear-gradient(to right, #54c8ff 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #2185d0 0.78px;
}

.ui.violet.rating .active.icon {
  color: #a291fb;
  text-shadow: 0 -1px 0 #6435c9, -1px 0 0 #6435c9, 0 1px 0 #6435c9, 1px 0 0 #6435c9;
}

.ui.violet.rating .icon.selected,
.ui.violet.rating .icon.selected.active,
.ui.violet.rating .icon.selected.partial.active {
  background: inherit;
  color: #745aff;
  text-shadow: 0 -1px 0 #5829bb, -1px 0 0 #5829bb, 0 1px 0 #5829bb, 1px 0 0 #5829bb;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.violet.rating .icon.partial.active {
  background: linear-gradient(to right, #a291fb 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #6435c9 0.78px;
}

.ui.purple.rating .active.icon {
  color: #dc73ff;
  text-shadow: 0 -1px 0 #a333c8, -1px 0 0 #a333c8, 0 1px 0 #a333c8, 1px 0 0 #a333c8;
}

.ui.purple.rating .icon.selected,
.ui.purple.rating .icon.selected.active,
.ui.purple.rating .icon.selected.partial.active {
  background: inherit;
  color: #cf40ff;
  text-shadow: 0 -1px 0 #9627ba, -1px 0 0 #9627ba, 0 1px 0 #9627ba, 1px 0 0 #9627ba;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.purple.rating .icon.partial.active {
  background: linear-gradient(to right, #dc73ff 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #a333c8 0.78px;
}

.ui.pink.rating .active.icon {
  color: #ff8edf;
  text-shadow: 0 -1px 0 #e03997, -1px 0 0 #e03997, 0 1px 0 #e03997, 1px 0 0 #e03997;
}

.ui.pink.rating .icon.selected,
.ui.pink.rating .icon.selected.active,
.ui.pink.rating .icon.selected.partial.active {
  background: inherit;
  color: #ff5bd1;
  text-shadow: 0 -1px 0 #e61a8d, -1px 0 0 #e61a8d, 0 1px 0 #e61a8d, 1px 0 0 #e61a8d;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.pink.rating .icon.partial.active {
  background: linear-gradient(to right, #ff8edf 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #e03997 0.78px;
}

.ui.brown.rating .active.icon {
  color: #d67c1c;
  text-shadow: 0 -1px 0 #a5673f, -1px 0 0 #a5673f, 0 1px 0 #a5673f, 1px 0 0 #a5673f;
}

.ui.brown.rating .icon.selected,
.ui.brown.rating .icon.selected.active,
.ui.brown.rating .icon.selected.partial.active {
  background: inherit;
  color: #b0620f;
  text-shadow: 0 -1px 0 #975b33, -1px 0 0 #975b33, 0 1px 0 #975b33, 1px 0 0 #975b33;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.brown.rating .icon.partial.active {
  background: linear-gradient(to right, #d67c1c 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #a5673f 0.78px;
}

.ui.grey.rating .active.icon {
  color: #dcddde;
  text-shadow: 0 -1px 0 #767676, -1px 0 0 #767676, 0 1px 0 #767676, 1px 0 0 #767676;
}

.ui.grey.rating .icon.selected,
.ui.grey.rating .icon.selected.active,
.ui.grey.rating .icon.selected.partial.active {
  background: inherit;
  color: #c2c4c5;
  text-shadow: 0 -1px 0 #838383, -1px 0 0 #838383, 0 1px 0 #838383, 1px 0 0 #838383;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.grey.rating .icon.partial.active {
  background: linear-gradient(to right, #dcddde 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #767676 0.78px;
}

.ui.black.rating .active.icon {
  color: #545454;
  text-shadow: 0 -1px 0 #1b1c1d, -1px 0 0 #1b1c1d, 0 1px 0 #1b1c1d, 1px 0 0 #1b1c1d;
}

.ui.black.rating .icon.selected,
.ui.black.rating .icon.selected.active,
.ui.black.rating .icon.selected.partial.active {
  background: inherit;
  color: #000000;
  text-shadow: 0 -1px 0 #27292a, -1px 0 0 #27292a, 0 1px 0 #27292a, 1px 0 0 #27292a;
  -webkit-text-stroke: unset;
  background-clip: unset;
}

.ui.black.rating .icon.partial.active {
  background: linear-gradient(to right, #545454 0% var(--full), rgba(0, 0, 0, 0.15) var(--full) 100%);
  -webkit-text-stroke: #1b1c1d 0.78px;
}

/*******************************
             States
*******************************/

/* -------------------
           Disabled
    -------------------- */

/* disabled rating */

.ui.disabled.rating .icon {
  cursor: default;
  pointer-events: none;
}

/* -------------------
   User Interactive
-------------------- */

/* Selected Rating */

.ui.rating.selected .active.icon {
  opacity: 1;
}

.ui.rating.selected .icon.selected,
.ui.rating .icon.selected {
  opacity: 1;
}

/*******************************
          Variations
*******************************/

.ui.rating {
  font-size: 1rem;
}

.ui.mini.rating {
  font-size: 0.78571429rem;
}

.ui.tiny.rating {
  font-size: 0.85714286rem;
}

.ui.small.rating {
  font-size: 0.92857143rem;
}

.ui.large.rating {
  font-size: 1.14285714rem;
}

.ui.big.rating {
  font-size: 1.28571429rem;
}

.ui.huge.rating {
  font-size: 1.42857143rem;
}

.ui.massive.rating {
  font-size: 2rem;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Search
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
             Search
*******************************/

.ui.search {
  position: relative;
}

.ui.search > .prompt {
  margin: 0;
  outline: none;
  -webkit-appearance: none;
  -webkit-tap-highlight-color: rgba(255, 255, 255, 0);
  text-shadow: none;
  font-style: normal;
  font-weight: normal;
  line-height: 1.21428571em;
  padding: 0.67857143em 1em;
  font-size: 1em;
  background: #fff;
  border: 1px solid rgba(34, 36, 38, 0.15);
  color: rgba(0, 0, 0, 0.87);
  box-shadow: 0 0 0 0 transparent inset;
  transition: background-color 0.1s ease, color 0.1s ease, box-shadow 0.1s ease, border-color 0.1s ease;
}

.ui.search .prompt {
  border-radius: 500rem;
}

/* --------------
     Icon
--------------- */

.ui.search .prompt ~ .search.icon {
  cursor: pointer;
}

/* --------------
    Results
--------------- */

.ui.search > .results {
  display: none;
  position: absolute;
  top: 100%;
  left: 0;
  transform-origin: center top;
  white-space: normal;
  text-align: left;
  text-transform: none;
  background: #fff;
  margin-top: 0.5em;
  width: 18em;
  border-radius: 0.28571429rem;
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
  border: 1px solid #d4d4d5;
  z-index: 998;
}

.ui.search > .results > :first-child {
  border-radius: 0.28571429rem 0.28571429rem 0 0;
}

.ui.search > .results > :last-child {
  border-radius: 0 0 0.28571429rem 0.28571429rem;
}

/* --------------
    Result
--------------- */

.ui.search > .results .result {
  cursor: pointer;
  display: block;
  overflow: hidden;
  font-size: 1em;
  padding: 0.85714286em 1.14285714em;
  color: rgba(0, 0, 0, 0.87);
  line-height: 1.33;
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.search > .results .result:last-child {
  border-bottom: none !important;
}

/* Image */

.ui.search > .results .result .image {
  float: right;
  overflow: hidden;
  background: none;
  width: 5em;
  height: 3em;
  border-radius: 0.25em;
}

.ui.search > .results .result .image img {
  display: block;
  width: auto;
  height: 100%;
}

/* --------------
      Info
--------------- */

.ui.search > .results .result .image + .content {
  margin: 0 6em 0 0;
}

.ui.search > .results .result .title {
  margin: -0.14285714em 0 0;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-weight: bold;
  font-size: 1em;
  color: rgba(0, 0, 0, 0.85);
}

.ui.search > .results .result .description {
  margin-top: 0;
  font-size: 0.92857143em;
  color: rgba(0, 0, 0, 0.4);
}

.ui.search > .results .result .price {
  float: right;
  color: #21ba45;
}

/* --------------
    Message
--------------- */

.ui.search > .results > .message {
  padding: 1em 1em;
}

.ui.search > .results > .message .header {
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1rem;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.87);
}

.ui.search > .results > .message .description {
  margin-top: 0.25rem;
  font-size: 1em;
  color: rgba(0, 0, 0, 0.87);
}

/* View All Results */

.ui.search > .results > .action {
  display: block;
  border-top: none;
  background: #f3f4f5;
  padding: 0.92857143em 1em;
  color: rgba(0, 0, 0, 0.87);
  font-weight: bold;
  text-align: center;
}

/*******************************
            States
*******************************/

/* --------------------
       Focus
--------------------- */

.ui.search > .prompt:focus {
  border-color: rgba(34, 36, 38, 0.35);
  background: #fff;
  color: rgba(0, 0, 0, 0.95);
}

/* --------------------
           Loading
    --------------------- */

.ui.loading.search .input > i.icon::before {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.1);
}

.ui.loading.search .input > i.icon::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -0.64285714em 0 0 -0.64285714em;
  width: 1.28571429em;
  height: 1.28571429em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
}

/* --------------
      Hover
--------------- */

.ui.search > .results .result:hover,
.ui.category.search > .results .category .result:hover {
  background: #f9fafb;
}

.ui.search .action:hover:not(div) {
  background: #e0e0e0;
}

/* --------------
      Active
--------------- */

.ui.category.search > .results .category.active {
  background: #f3f4f5;
}

.ui.category.search > .results .category.active > .name {
  color: rgba(0, 0, 0, 0.87);
}

.ui.search > .results .result.active,
.ui.category.search > .results .category .result.active {
  position: relative;
  border-left-color: rgba(34, 36, 38, 0.1);
  background: #f3f4f5;
  box-shadow: none;
}

.ui.search > .results .result.active .title {
  color: rgba(0, 0, 0, 0.85);
}

.ui.search > .results .result.active .description {
  color: rgba(0, 0, 0, 0.85);
}

/* --------------------
            Disabled
    ---------------------- */

/* Disabled */

.ui.disabled.search {
  cursor: default;
  pointer-events: none;
  opacity: 0.45;
}

/*******************************
           Types
*******************************/

/* --------------
        Selection
    --------------- */

.ui.search.selection .prompt {
  border-radius: 0.28571429rem;
}

/* Remove input */

.ui.search.selection > .icon.input > .remove.icon {
  pointer-events: none;
  position: absolute;
  left: auto;
  opacity: 0;
  color: "";
  top: 0;
  right: 0;
  transition: color 0.1s ease, opacity 0.1s ease;
}

.ui.search.selection > .icon.input > .active.remove.icon {
  cursor: pointer;
  opacity: 0.8;
  pointer-events: auto;
}

.ui.search.selection > .icon.input:not([class*="left icon"]) > .icon ~ .remove.icon {
  right: 1.85714em;
}

.ui.search.selection > .icon.input > .remove.icon:hover {
  opacity: 1;
  color: #db2828;
}

/* --------------
        Category
    --------------- */

.ui.category.search .results {
  width: 28em;
}

.ui.category.search .results.animating,
.ui.category.search .results.visible {
  display: table;
}

/* Category */

.ui.category.search > .results .category {
  display: table-row;
  background: #f3f4f5;
  box-shadow: none;
  transition: background 0.1s ease, border-color 0.1s ease;
}

/* Last Category */

.ui.category.search > .results .category:last-child {
  border-bottom: none;
}

/* First / Last */

.ui.category.search > .results .category:first-child .name + .result {
  border-radius: 0 0.28571429rem 0 0;
}

.ui.category.search > .results .category:last-child .result:last-child {
  border-radius: 0 0 0.28571429rem 0;
}

/* Category Result Name */

.ui.category.search > .results .category > .name {
  display: table-cell;
  text-overflow: ellipsis;
  width: 100px;
  white-space: nowrap;
  background: transparent;
  font-family: "Lato", system-ui, -apple-system, "Segoe UI", Roboto, Oxygen, Ubuntu, Cantarell, "Helvetica Neue", Arial, "Noto Sans", "Liberation Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  font-size: 1em;
  padding: 0.4em 1em;
  font-weight: bold;
  color: rgba(0, 0, 0, 0.4);
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
}

/* Category Result */

.ui.category.search > .results .category .results {
  display: table-cell;
  background: #fff;
  border-left: 1px solid rgba(34, 36, 38, 0.15);
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
}

.ui.category.search > .results .category .result {
  border-bottom: 1px solid rgba(34, 36, 38, 0.1);
  transition: background 0.1s ease, border-color 0.1s ease;
  padding: 0.85714286em 1.14285714em;
}

.ui.horizontal.category.search > .results .category,
.ui.horizontal.category.search > .results .category > .name,
.ui.horizontal.category.search > .results .category > .results {
  display: block;
}

.ui.horizontal.category.search > .results .category > .results {
  width: 100%;
  border-left: 0;
}

/*******************************
           Variations
*******************************/

/* -------------------
         Scrolling
    -------------------- */

.ui.scrolling.search > .results,
.ui.search.long > .results,
.ui.search.short > .results {
  overflow-x: hidden;
  overflow-y: auto;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

@media only screen and (max-width: 767.98px) {
  .ui.scrolling.search > .results {
    max-height: 12.17714286em;
  }

  .resizable.ui.scrolling.search > .results {
    height: 12.17714286em;
  }
}

@media only screen and (min-width: 768px) {
  .ui.scrolling.search > .results {
    max-height: 18.26571429em;
  }

  .resizable.ui.scrolling.search > .results {
    height: 18.26571429em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.scrolling.search > .results {
    max-height: 24.35428571em;
  }

  .resizable.ui.scrolling.search > .results {
    height: 24.35428571em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.scrolling.search > .results {
    max-height: 36.53142857em;
  }

  .resizable.ui.scrolling.search > .results {
    height: 36.53142857em;
  }
}

.ui.scrolling.resizable.search > .results {
  resize: vertical;
  max-height: none;
}

@media only screen and (max-width: 767.98px) {
  .ui.search.short > .results {
    max-height: 12.17714286em;
  }

  .ui.search[class*="very short"] > .results {
    max-height: 9.13285714em;
  }

  .ui.search.long > .results {
    max-height: 24.35428571em;
  }

  .ui.search[class*="very long"] > .results {
    max-height: 36.53142857em;
  }
}

@media only screen and (min-width: 768px) {
  .ui.search.short > .results {
    max-height: 18.26571429em;
  }

  .ui.search[class*="very short"] > .results {
    max-height: 13.69928571em;
  }

  .ui.search.long > .results {
    max-height: 36.53142857em;
  }

  .ui.search[class*="very long"] > .results {
    max-height: 54.79714286em;
  }
}

@media only screen and (min-width: 992px) {
  .ui.search.short > .results {
    max-height: 24.35428571em;
  }

  .ui.search[class*="very short"] > .results {
    max-height: 18.26571429em;
  }

  .ui.search.long > .results {
    max-height: 48.70857143em;
  }

  .ui.search[class*="very long"] > .results {
    max-height: 73.06285714em;
  }
}

@media only screen and (min-width: 1920px) {
  .ui.search.short > .results {
    max-height: 36.53142857em;
  }

  .ui.search[class*="very short"] > .results {
    max-height: 27.39857143em;
  }

  .ui.search.long > .results {
    max-height: 73.06285714em;
  }

  .ui.search[class*="very long"] > .results {
    max-height: 109.59428571em;
  }
}

/* -------------------
         Left / Right
    -------------------- */

.ui[class*="left aligned"].search > .results {
  right: auto;
  left: 0;
}

.ui[class*="right aligned"].search > .results {
  right: 0;
  left: auto;
}

/* --------------
    Fluid
--------------- */

.ui.fluid.search .results {
  width: 100%;
}

/* --------------
      Sizes
--------------- */

.ui.search {
  font-size: 1em;
}

.ui.mini.search {
  font-size: 0.78571429em;
}

.ui.tiny.search {
  font-size: 0.85714286em;
}

.ui.small.search {
  font-size: 0.92857143em;
}

.ui.large.search {
  font-size: 1.14285714em;
}

.ui.big.search {
  font-size: 1.28571429em;
}

.ui.huge.search {
  font-size: 1.42857143em;
}

.ui.massive.search {
  font-size: 1.71428571em;
}

/* --------------
      Mobile
--------------- */

@media only screen and (max-width: 767.98px) {
  .ui.search .results {
    max-width: calc(100vw - 2rem);
  }
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Shape
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
              Shape
*******************************/

.ui.shape {
  position: relative;
  vertical-align: top;
  display: inline-block;
  perspective: 2000px;
  transition: transform 0.6s ease-in-out, left 0.6s ease-in-out, width 0.6s ease-in-out, height 0.6s ease-in-out;
}

.ui.shape .side,
.ui.shape .sides {
  transform-style: preserve-3d;
}

.ui.shape .side {
  display: none;
  opacity: 1;
  width: 100%;
  margin: 0 !important;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

.ui.shape .side * {
  -webkit-backface-visibility: visible !important;
  backface-visibility: visible !important;
}

/*******************************
             Types
*******************************/

.ui.cube.shape .side {
  min-width: 15em;
  height: 15em;
  padding: 2em;
  background-color: #e6e6e6;
  color: rgba(0, 0, 0, 0.87);
  box-shadow: 0 0 2px rgba(0, 0, 0, 0.3);
}

.ui.cube.shape .side > .content {
  width: 100%;
  height: 100%;
  display: table;
  text-align: center;
  -webkit-user-select: text;
  -ms-user-select: text;
  user-select: text;
}

.ui.cube.shape .side > .content > div {
  display: table-cell;
  vertical-align: middle;
  font-size: 2em;
}

/*******************************
          Variations
*******************************/

.ui.text.shape.animating .sides {
  position: static;
}

.ui.text.shape .side {
  white-space: nowrap;
}

.ui.text.shape .side > * {
  white-space: normal;
}

/*******************************
             States
*******************************/

/* --------------
        Loading
    --------------- */

.ui.loading.shape {
  position: absolute;
  top: -9999px;
  left: -9999px;
}

/* --------------
    Animating
--------------- */

.ui.shape .animating.side {
  position: absolute;
  top: 0;
  left: 0;
  display: block;
  z-index: 100;
}

.ui.shape .hidden.side {
  opacity: 0.6;
}

/* --------------
      CSS
--------------- */

.ui.shape.animating .sides {
  position: absolute;
  transition: transform 0.6s ease-in-out, left 0.6s ease-in-out, width 0.6s ease-in-out, height 0.6s ease-in-out;
}

.ui.shape.animating .side {
  transition: opacity 0.6s ease-in-out;
}

.ui.shape .animating.side *,
.ui.shape.animating .side * {
  transition: none;
}

/* --------------
     Active
--------------- */

.ui.shape .active.side {
  display: block;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Sidebar
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Sidebar
*******************************/

/* Sidebar Menu */

.ui.sidebar {
  position: fixed;
  top: 0;
  left: 0;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  transition: none;
  will-change: transform;
  transform: translate3d(0, 0, 0);
  visibility: hidden;
  height: 100% !important;
  max-height: 100%;
  border-radius: 0 !important;
  margin: 0 !important;
  overflow-y: auto !important;
  -ms-scroll-chaining: none;
  overscroll-behavior: none;
  z-index: 102;
}

/* GPU Layers for Child Elements */

.ui.sidebar > * {
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
}

/* --------------
   Direction
--------------- */

.ui.left.sidebar {
  right: auto;
  left: 0;
  transform: translate3d(-100%, 0, 0);
}

.ui.right.sidebar {
  right: 0 !important;
  left: auto !important;
  transform: translate3d(100%, 0, 0);
}

.ui.top.sidebar,
.ui.bottom.sidebar {
  width: 100% !important;
  height: auto !important;
}

.ui.top.sidebar {
  top: 0 !important;
  bottom: auto !important;
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.sidebar {
  top: auto !important;
  bottom: 0 !important;
  transform: translate3d(0, 100%, 0);
}

/* --------------
     Pushable
--------------- */

.pushable.pushable.pushable {
  height: 100%;
  overflow-x: hidden;
  padding: 0;
}

.pushable.pushable.pushable.locked {
  overflow-y: hidden;
  background: inherit;
}

/* Whole Page */

body.pushable {
  background: #545454;
}

body.pushable.dimmed {
  background: inherit;
}

/* Page Context */

.pushable:not(body) {
  transform: translate3d(0, 0, 0);
  overflow-y: hidden;
}

.pushable:not(body) > .ui.sidebar,
.pushable:not(body) > .fixed,
.pushable:not(body) > .pusher::after {
  position: absolute;
}

/* --------------
     Fixed
--------------- */

.pushable > .fixed {
  position: fixed;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  transition: transform 500ms ease;
  will-change: transform;
  z-index: 101;
}

/* --------------
     Page
--------------- */

.pushable > .pusher {
  position: relative;
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  min-height: 100%;
  transition: transform 500ms ease;
  z-index: 2;
  /* Pusher should inherit background from context */
  background: inherit;
}

.pushable > .pusher:not(.overflowing) {
  overflow: hidden;
}

body.pushable > .pusher {
  background: #fff;
}

/* --------------
     Dimmer
--------------- */

.pushable > .pusher::after {
  position: fixed;
  top: 0;
  right: 0;
  content: "";
  background: rgba(0, 0, 0, 0.4);
  overflow: hidden;
  opacity: 0;
  transition: all 500ms;
  will-change: opacity;
  z-index: 1000;
}

/* --------------
    Coupling
--------------- */

.ui.sidebar.menu .item {
  border-radius: 0 !important;
}

/*******************************
            States
*******************************/

/* --------------
     Dimmed
--------------- */

.pushable > .pusher.dimmed::after {
  width: 100% !important;
  height: 100% !important;
  opacity: 1 !important;
}

.pushable > .pusher.dimmed.blurring:not(.closing)::after {
  background: rgba(0, 0, 0, 0.6);
  -webkit-backdrop-filter: blur(5px) grayscale(0.7);
  backdrop-filter: blur(5px) grayscale(0.7);
}

.pushable > .pusher.closing.dimmed::after {
  opacity: 0 !important;
}

/* --------------
    Animating
--------------- */

.ui.animating.sidebar {
  visibility: visible;
}

/* --------------
     Visible
--------------- */

.ui.visible.sidebar {
  visibility: visible;
  transform: translate3d(0, 0, 0);
}

/* Shadow Direction */

.ui.left.visible.sidebar,
.ui.right.visible.sidebar {
  box-shadow: 0 0 20px rgba(34, 36, 38, 0.15);
}

.ui.top.visible.sidebar,
.ui.bottom.visible.sidebar {
  box-shadow: 0 0 20px rgba(34, 36, 38, 0.15);
}

/* Visible On Load */

.ui.visible.left.sidebar ~ .fixed,
.ui.visible.left.sidebar ~ .pusher {
  transform: translate3d(260px, 0, 0);
}

.ui.visible.right.sidebar ~ .fixed,
.ui.visible.right.sidebar ~ .pusher {
  transform: translate3d(-260px, 0, 0);
}

.ui.visible.top.sidebar ~ .fixed,
.ui.visible.top.sidebar ~ .pusher {
  transform: translate3d(0, 36px, 0);
}

.ui.visible.bottom.sidebar ~ .fixed,
.ui.visible.bottom.sidebar ~ .pusher {
  transform: translate3d(0, -36px, 0);
}

/* opposite sides visible forces content overlay */

.ui.visible.left.sidebar ~ .ui.visible.right.sidebar ~ .fixed,
.ui.visible.left.sidebar ~ .ui.visible.right.sidebar ~ .pusher,
.ui.visible.right.sidebar ~ .ui.visible.left.sidebar ~ .fixed,
.ui.visible.right.sidebar ~ .ui.visible.left.sidebar ~ .pusher {
  transform: translate3d(0, 0, 0);
}

/* --------------
       iOS
--------------- */

/*******************************
          Variations
*******************************/

/* --------------
     Width
--------------- */

/* Left / Right */

.ui.thin.left.sidebar,
.ui.thin.right.sidebar {
  width: 150px;
}

.ui[class*="very thin"].left.sidebar,
.ui[class*="very thin"].right.sidebar {
  width: 60px;
}

.ui.left.sidebar,
.ui.right.sidebar {
  width: 260px;
}

.ui.wide.left.sidebar,
.ui.wide.right.sidebar {
  width: 350px;
}

.ui[class*="very wide"].left.sidebar,
.ui[class*="very wide"].right.sidebar {
  width: 475px;
}

/* Left Visible */

.ui.visible.thin.left.sidebar ~ .fixed,
.ui.visible.thin.left.sidebar ~ .pusher {
  transform: translate3d(150px, 0, 0);
}

.ui.visible[class*="very thin"].left.sidebar ~ .fixed,
.ui.visible[class*="very thin"].left.sidebar ~ .pusher {
  transform: translate3d(60px, 0, 0);
}

.ui.visible.wide.left.sidebar ~ .fixed,
.ui.visible.wide.left.sidebar ~ .pusher {
  transform: translate3d(350px, 0, 0);
}

.ui.visible[class*="very wide"].left.sidebar ~ .fixed,
.ui.visible[class*="very wide"].left.sidebar ~ .pusher {
  transform: translate3d(475px, 0, 0);
}

/* Right Visible */

.ui.visible.thin.right.sidebar ~ .fixed,
.ui.visible.thin.right.sidebar ~ .pusher {
  transform: translate3d(-150px, 0, 0);
}

.ui.visible[class*="very thin"].right.sidebar ~ .fixed,
.ui.visible[class*="very thin"].right.sidebar ~ .pusher {
  transform: translate3d(-60px, 0, 0);
}

.ui.visible.wide.right.sidebar ~ .fixed,
.ui.visible.wide.right.sidebar ~ .pusher {
  transform: translate3d(-350px, 0, 0);
}

.ui.visible[class*="very wide"].right.sidebar ~ .fixed,
.ui.visible[class*="very wide"].right.sidebar ~ .pusher {
  transform: translate3d(-475px, 0, 0);
}

/*******************************
          Animations
*******************************/

/* --------------
        Overlay
    --------------- */

/* Set-up */

.ui.overlay.sidebar {
  z-index: 102;
}

/* Initial */

.ui.left.overlay.sidebar {
  transform: translate3d(-100%, 0, 0);
}

.ui.right.overlay.sidebar {
  transform: translate3d(100%, 0, 0);
}

.ui.top.overlay.sidebar {
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.overlay.sidebar {
  transform: translate3d(0, 100%, 0);
}

/* Animation */

.animating.ui.overlay.sidebar,
.ui.visible.overlay.sidebar {
  transition: transform 500ms ease;
}

/* End - Sidebar */

.ui.visible.left.overlay.sidebar {
  transform: translate3d(0, 0, 0);
}

.ui.visible.right.overlay.sidebar {
  transform: translate3d(0, 0, 0);
}

.ui.visible.top.overlay.sidebar {
  transform: translate3d(0, 0, 0);
}

.ui.visible.bottom.overlay.sidebar {
  transform: translate3d(0, 0, 0);
}

/* End - Pusher */

.ui.visible.overlay.sidebar ~ .fixed,
.ui.visible.overlay.sidebar ~ .pusher {
  transform: none !important;
}

/* --------------
          Push
    --------------- */

/* Initial */

.ui.push.sidebar {
  transition: transform 500ms ease;
  z-index: 102;
}

/* Sidebar - Initial */

.ui.left.push.sidebar {
  transform: translate3d(-100%, 0, 0);
}

.ui.right.push.sidebar {
  transform: translate3d(100%, 0, 0);
}

.ui.top.push.sidebar {
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.push.sidebar {
  transform: translate3d(0, 100%, 0);
}

/* End */

.ui.visible.push.sidebar {
  transform: translate3d(0, 0, 0);
}

/* --------------
        Uncover
    --------------- */

/* Initial */

.ui.uncover.sidebar {
  transform: translate3d(0, 0, 0);
  z-index: 1;
}

/* End */

.ui.visible.uncover.sidebar {
  transform: translate3d(0, 0, 0);
  transition: transform 500ms ease;
}

/* --------------
       Slide Along
    --------------- */

/* Initial */

.ui.slide.along.sidebar {
  z-index: 1;
}

/* Sidebar - Initial */

.ui.left.slide.along.sidebar {
  transform: translate3d(-50%, 0, 0);
}

.ui.right.slide.along.sidebar {
  transform: translate3d(50%, 0, 0);
}

.ui.top.slide.along.sidebar {
  transform: translate3d(0, -50%, 0);
}

.ui.bottom.slide.along.sidebar {
  transform: translate3d(0, 50%, 0);
}

/* Animation */

.ui.animating.slide.along.sidebar {
  transition: transform 500ms ease;
}

/* End */

.ui.visible.slide.along.sidebar {
  transform: translate3d(0, 0, 0);
}

/* --------------
       Slide Out
    --------------- */

/* Initial */

.ui.slide.out.sidebar {
  z-index: 1;
}

/* Sidebar - Initial */

.ui.left.slide.out.sidebar {
  transform: translate3d(50%, 0, 0);
}

.ui.right.slide.out.sidebar {
  transform: translate3d(-50%, 0, 0);
}

.ui.top.slide.out.sidebar {
  transform: translate3d(0, 50%, 0);
}

.ui.bottom.slide.out.sidebar {
  transform: translate3d(0, -50%, 0);
}

/* Animation */

.ui.animating.slide.out.sidebar {
  transition: transform 500ms ease;
}

/* End */

.ui.visible.slide.out.sidebar {
  transform: translate3d(0, 0, 0);
}

/* --------------
       Scale Down
    --------------- */

/* Initial */

.ui.scale.down.sidebar {
  transition: transform 500ms ease;
  z-index: 102;
}

.ui.left.scale.down.sidebar {
  transform: translate3d(-100%, 0, 0);
}

.ui.right.scale.down.sidebar {
  transform: translate3d(100%, 0, 0);
}

.ui.top.scale.down.sidebar {
  transform: translate3d(0, -100%, 0);
}

.ui.bottom.scale.down.sidebar {
  transform: translate3d(0, 100%, 0);
}

/* Pusher - Initial */

.ui.scale.down.left.sidebar ~ .pusher {
  transform-origin: 75% 50%;
}

.ui.scale.down.right.sidebar ~ .pusher {
  transform-origin: 25% 50%;
}

.ui.scale.down.top.sidebar ~ .pusher {
  transform-origin: 50% 75%;
}

.ui.scale.down.bottom.sidebar ~ .pusher {
  transform-origin: 50% 25%;
}

/* Animation */

.ui.animating.scale.down > .visible.ui.sidebar {
  transition: transform 500ms ease;
}

.ui.visible.scale.down.sidebar ~ .pusher,
.ui.animating.scale.down.sidebar ~ .pusher {
  display: block !important;
  width: 100%;
  height: 100%;
  overflow: hidden !important;
}

/* End */

.ui.visible.scale.down.sidebar {
  transform: translate3d(0, 0, 0);
}

.ui.visible.scale.down.sidebar ~ .pusher {
  transform: scale(0.75);
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Sticky
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
            Sticky
*******************************/

.ui.sticky {
  position: static;
  transition: none;
  z-index: 800;
}

/*******************************
            States
*******************************/

/* Bound */

.ui.sticky.bound {
  position: absolute;
  left: auto;
  right: auto;
}

/* Fixed */

.ui.sticky.fixed {
  position: fixed;
  left: auto;
  right: auto;
}

/* Bound/Fixed Position */

.ui.sticky.bound.top,
.ui.sticky.fixed.top {
  top: 0;
  bottom: auto;
}

.ui.sticky.bound.bottom,
.ui.sticky.fixed.bottom {
  top: auto;
  bottom: 0;
}

/*******************************
            Types
*******************************/

.ui.native.sticky {
  position: sticky;
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
         Site Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Tab
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
           UI Tabs
*******************************/

.ui.tab {
  display: none;
}

/*******************************
             States
*******************************/

/* --------------------
       Active
--------------------- */

.ui.tab.active,
.ui.tab.open {
  display: block;
}

/* --------------------
           Loading
    --------------------- */

.ui.tab.loading {
  position: relative;
  overflow: hidden;
  display: block;
  min-height: 250px;
}

.ui.tab.loading * {
  position: relative !important;
  left: -10000px !important;
}

.ui.tab.loading::before,
.ui.tab.loading.segment::before {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1.25em 0 0 -1.25em;
  width: 2.5em;
  height: 2.5em;
  border-radius: 500rem;
  border: 0.2em solid rgba(0, 0, 0, 0.1);
}

.ui.tab.loading::after,
.ui.tab.loading.segment::after {
  position: absolute;
  content: "";
  top: 50%;
  left: 50%;
  margin: -1.25em 0 0 -1.25em;
  width: 2.5em;
  height: 2.5em;
  animation: loader 0.6s infinite linear;
  border: 0.2em solid #767676;
  border-radius: 500rem;
  box-shadow: 0 0 0 1px transparent;
}

/*******************************
         Tab Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Toast
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
        Toast container
*******************************/

.ui.toast-container {
  position: fixed;
  z-index: 9999;
}

.ui.toast-container.absolute {
  position: absolute;
}

.ui.toast-container.ui.attached {
  width: 100%;
  left: 0;
}

.ui.toast-container.ui.attached .vertical.attached {
  border-radius: 0;
}

.ui.toast-container.ui.attached.ui.ui .attached.actions .button {
  border-radius: 0;
}

.ui.toast-container.ui.attached .toast-box {
  margin: 0;
  width: 100%;
  border-radius: 0;
}

.ui.toast-container.ui.attached .toast-box > .ui.toast,
.ui.toast-container.ui.attached .toast-box > .ui.message {
  margin-left: 0;
}

.ui.toast-container.ui.attached .toast-box.floating,
.ui.toast-container.ui.attached .toast-box.hoverfloating:hover {
  border: none;
}

.ui.toast-container.ui.attached .toast-box > .vertical > .content {
  flex: 1;
}

.ui.toast-container.ui.attached .toast-box > * {
  width: 100%;
  border-radius: 0;
}

.ui.toast-container.ui.attached .toast-box > * > .vertical:not(.actions) {
  flex: 1;
}

.ui.toast-container.ui.attached .toast-box > .attached.actions {
  margin-right: 1px;
}

.ui.toast-container.ui.attached.top {
  top: 0;
}

.ui.toast-container.ui.attached.bottom {
  bottom: 0;
}

.ui.toast-container.top.right {
  top: 0.85714286em;
  right: 0.85714286em;
  margin-left: 0.85714286em;
}

.ui.toast-container.top.left {
  top: 0.85714286em;
  left: 0.85714286em;
  margin-right: 0.85714286em;
}

.ui.toast-container.top.center {
  left: 50%;
  transform: translate(-50%, 0);
  top: 0.85714286em;
}

.ui.toast-container.bottom.right {
  bottom: 0.85714286em;
  right: 0.85714286em;
  margin-left: 0.85714286em;
}

.ui.toast-container.bottom.left {
  bottom: 0.85714286em;
  left: 0.85714286em;
  margin-right: 0.85714286em;
}

.ui.toast-container.bottom.center {
  left: 50%;
  transform: translate(-50%, 0);
  bottom: 0.85714286em;
}

.ui.toast-container.centered {
  transform: translate(-50%, -50%);
  top: 50%;
  left: 50%;
}

.ui.toast-container .visible.toast-box,
.ui.toast-container .animating.toast-box,
.ui.toast-container .toast-box {
  display: table !important;
}

.ui.toast-container.horizontal {
  display: flex;
  flex-direction: row;
}

.ui.toast-container.horizontal .toast-box {
  margin-right: 0.5em;
}

.ui.toast-container .toast-box {
  margin-bottom: 0.5em;
  border-radius: 0.28571429rem;
  cursor: default;
  will-change: transform, opacity;
}

.ui.toast-container .toast-box:hover {
  opacity: 1;
}

.ui.toast-container .toast-box:not(.unclickable):hover {
  cursor: pointer;
}

.ui.toast-container .toast-box.floating,
.ui.toast-container .toast-box.hoverfloating:hover {
  box-shadow: 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
  border: 1px solid rgba(34, 36, 38, 0.12);
}

.ui.toast-container .toast-box.inverted.floating,
.ui.toast-container .toast-box.inverted.hoverfloating:hover {
  box-shadow: 0 2px 4px 0 rgba(225, 225, 225, 0.1), 0 2px 10px 0 rgba(225, 225, 225, 0.5);
  border: 1px solid rgba(225, 225, 225, 0.5);
}

.ui.toast-container .toast-box.compact,
.ui.toast-container .toast-box > .compact {
  width: 350px;
}

.ui.toast-container .toast-box > .ui.toast,
.ui.toast-container .toast-box > .ui.message {
  margin: 0 -1px -0.01em;
  position: relative;
}

.ui.toast-container .toast-box > .attached.progress {
  z-index: 1;
}

.ui.toast-container .toast-box > .attached.progress.bottom {
  margin: -0.2em -1px -0.01em;
}

.ui.toast-container .toast-box > .attached.progress.top {
  margin: -0.01em -1px -0.2em;
}

.ui.toast-container .toast-box > .attached.progress .bar {
  min-width: 0;
}

.ui.toast-container .toast-box > .attached.progress.info .bar.bar.bar {
  background: #12a1bf;
}

.ui.toast-container .toast-box > .attached.progress.warning .bar.bar.bar {
  background: #cf9b0d;
}

.ui.toast-container .toast-box > .attached.progress.success .bar.bar.bar {
  background: #15792d;
}

.ui.toast-container .toast-box > .attached.progress.error .bar.bar.bar {
  background: #9c1a1a;
}

.ui.toast-container .toast-box > .attached.progress.neutral .bar.bar.bar {
  background: #d9d9d9;
}

.ui.toast-container .toast-box > .ui.message > .close.icon {
  top: 0.3em;
  right: 0.3em;
}

.ui.toast-container .toast-box > .ui.message > .actions:last-child {
  margin-bottom: -1em;
}

.ui.toast-container .toast-box > .ui.message.icon {
  align-items: inherit;
}

.ui.toast-container .toast-box > .ui.message.icon > :not(.icon):not(.actions) {
  padding-left: 5rem;
}

.ui.toast-container .toast-box > .ui.message.icon > i.icon:not(.close) {
  display: inline-block;
  position: absolute;
  width: 4rem;
  top: 50%;
  transform: translateY(-50%);
}

.ui.toast-container .toast-box > .ui.message.icon:not(.vertical).actions > i.icon:not(.close) {
  top: calc(50% - 1.2em);
  transform: none;
}

.ui.toast-container .toast-box > .ui.message.icon:not(.vertical).icon.icon.icon {
  display: block;
}

.ui.toast-container .toast-box .ui.toast > .close.icon {
  cursor: pointer;
  margin: 0;
  opacity: 0.7;
  transition: opacity 0.1s ease;
}

.ui.toast-container .toast-box .ui.toast > .close.icon:hover {
  opacity: 1;
}

.ui.toast-container .toast-box .ui.toast.vertical > .close.icon {
  margin-top: -0.3em;
  margin-right: -0.3em;
}

.ui.toast-container .toast-box .ui.toast:not(.vertical) > .close.icon {
  position: absolute;
  top: 0.3em;
}

.ui.toast-container .toast-box .ui.toast:not(.vertical) > .close.icon:not(.left) {
  right: 0.3em;
}

.ui.toast-container .toast-box .ui.toast:not(.vertical) > .close.icon.left {
  margin-left: -0.3em;
}

.ui.toast-container .toast-box .ui.card {
  margin: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).bottom {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).bottom.horizontal > .image > img {
  border-top-left-radius: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).bottom.horizontal > .image:last-child > img {
  border-top-right-radius: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).top {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).top.horizontal > .image > img {
  border-bottom-left-radius: 0;
}

.ui.toast-container .toast-box .ui.card.attached:not(.vertical).top.horizontal > .image:last-child > img {
  border-bottom-right-radius: 0;
}

.ui.toast-container .toast-box .ui.card.horizontal.actions > .image > img {
  border-bottom-left-radius: 0;
}

.ui.toast-container .toast-box .ui.card.horizontal.actions > .image:last-child > img {
  border-bottom-right-radius: 0;
}

.ui.toast-container .toast-box .progressing {
  animation-iteration-count: 1;
  animation-timing-function: linear;
}

.ui.toast-container .toast-box .progressing.up {
  animation-name: progressUp;
}

.ui.toast-container .toast-box .progressing.down {
  animation-name: progressDown;
}

.ui.toast-container .toast-box .progressing.wait {
  animation-name: progressWait;
}

.ui.toast-container .toast-box:hover .pausable.progressing {
  animation-play-state: paused;
}

.ui.toast-container .toast-box .ui.toast:not(.vertical) {
  display: block;
}

.ui.toast-container .toast-box :not(.comment):not(.card) .actions {
  margin: 0.5em -1em -1em -1em;
}

.ui.toast-container .toast-box :not(.comment) .actions {
  padding: 0.5em 0.5em 0.75em 0.5em;
  text-align: right;
}

.ui.toast-container .toast-box :not(.comment) .actions.attached:not(.vertical) {
  margin-right: 1px;
}

.ui.toast-container .toast-box :not(.comment) .actions:not(.basic):not(.attached) {
  background: rgba(255, 255, 255, 0.25);
  border-top: 1px solid rgba(0, 0, 0, 0.2);
}

.ui.toast-container .toast-box :not(.comment) .actions.left {
  text-align: left;
}

.ui.toast-container .toast-box .vertical.actions > .button,
.ui.toast-container .toast-box > .vertical > .vertical.vertical,
.ui.toast-container .toast-box > .vertical.vertical.vertical {
  display: flex;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions {
  flex-direction: column;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions > .button {
  justify-content: center;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions.attached > .button {
  align-items: center;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions:not(.attached) {
  border-top: 0;
  margin-top: -0.75em;
  margin-bottom: -0.75em;
  margin-left: 1em;
  justify-content: space-around;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions:not(.attached):not(.basic) {
  border-left: 1px solid rgba(0, 0, 0, 0.2);
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions:not(.attached) > .button:not(:last-child) {
  margin-bottom: 0.3em;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions:not(.attached).top {
  justify-content: flex-start;
}

.ui.toast-container .toast-box :not(.comment) .vertical.actions:not(.attached).bottom {
  justify-content: flex-end;
}

.ui.vertical.attached:not(.left).card > .image > img {
  border-top-right-radius: 0;
}

.ui.vertical.attached:not(.left).card.horizontal > .image:last-child > img {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.vertical.attached:not(.left).card,
.ui.vertical.attached:not(.left).toast {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.vertical.attached:not(.left).actions {
  border-top-right-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.vertical.attached:not(.left).actions .button:first-child,
.ui.vertical.attached:not(.left).actions .button:last-child {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui.vertical.attached:not(.left).message {
  border-top-right-radius: 0;
  border-bottom-left-radius: 0.28571429rem;
}

.ui.vertical.attached.left.card > .image > img {
  border-top-left-radius: 0;
}

.ui.vertical.attached.left.card.horizontal > .image > img {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui.vertical.attached.left.card,
.ui.vertical.attached.left.toast {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.ui.vertical.attached.left.actions {
  border-top-left-radius: 0.28571429rem;
  border-bottom-left-radius: 0.28571429rem;
}

.ui.vertical.attached.left.actions .button:first-child,
.ui.vertical.attached.left.actions .button:last-child {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.vertical.attached.left.message.message.message {
  border-top-right-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.attached:not(.vertical):not(.top).actions {
  border-bottom-left-radius: 0.28571429rem;
  border-bottom-right-radius: 0.28571429rem;
}

.ui.attached:not(.vertical):not(.top).actions .button:first-child {
  border-bottom-left-radius: 0.28571429rem;
}

.ui.attached:not(.vertical):not(.top).actions .button:last-child {
  border-bottom-right-radius: 0.28571429rem;
}

.ui.attached:not(.vertical).top.actions {
  border-top-left-radius: 0.28571429rem;
  border-top-right-radius: 0.28571429rem;
}

.ui.attached:not(.vertical).top.actions .button:first-child {
  border-top-left-radius: 0.28571429rem;
}

.ui.attached:not(.vertical).top.actions .button:last-child {
  border-top-right-radius: 0.28571429rem;
}

/*******************************
            Toast
*******************************/

.ui.toast {
  display: none;
  border-radius: 0.28571429rem;
  padding: 0.78571429em 1em;
  margin: 0 -1px -0.01em;
  color: rgba(0, 0, 0, 0.87);
  background-color: #fff;
}

.ui.toast > .content > .header {
  font-weight: bold;
  color: inherit;
  margin: 0;
}

.ui.toast.info {
  background-color: #31ccec;
  color: rgba(255, 255, 255, 0.9);
}

.ui.toast.warning {
  background-color: #f2c037;
  color: rgba(255, 255, 255, 0.9);
}

.ui.toast.success {
  background-color: #21ba45;
  color: rgba(255, 255, 255, 0.9);
}

.ui.toast.error {
  background-color: #db2828;
  color: rgba(255, 255, 255, 0.9);
}

.ui.toast.neutral {
  background-color: #fff;
  color: rgba(0, 0, 0, 0.87);
}

.ui.toast > i.icon:not(.close) {
  font-size: 1.5em;
}

.ui.toast:not(.vertical):not(.centered):not(.center) > i.icon:not(.close) {
  position: absolute;
}

.ui.toast:not(.vertical):not(.centered):not(.center) > .ui.image {
  position: absolute;
}

.ui.toast:not(.vertical) > i.icon:not(.close) + .content {
  padding-left: 3em;
}

.ui.toast:not(.vertical) > .close.icon + .content {
  padding-left: 1.5em;
}

.ui.toast:not(.vertical) > .ui.image.avatar + .content {
  padding-left: 3em;
  min-height: 2em;
}

.ui.toast:not(.vertical) > .ui.image.mini + .content {
  padding-left: 3.4em;
  min-height: 35px;
}

.ui.toast:not(.vertical) > .ui.image.tiny + .content {
  padding-left: 7em;
  min-height: 80px;
}

.ui.toast:not(.vertical) > .ui.image.small + .content {
  padding-left: 12em;
  min-height: 150px;
}

.ui.toast:not(.vertical):not(.centered):not(.center) > .centered.image,
.ui.toast:not(.vertical):not(.centered):not(.center) > .centered.icon {
  transform: translateY(-50%);
  top: 50%;
}

.ui.toast:not(.vertical).actions > .centered.image {
  top: calc(50% - 2em);
}

.ui.toast:not(.vertical).actions > .centered.icon {
  top: calc(50% - 1.2em);
}

.ui.toast.vertical > .content {
  flex-grow: 1;
}

.ui.toast.vertical.attached {
  flex-grow: 1;
}

.ui.toast.vertical > .close.icon + .content {
  padding-left: 1em;
}

.ui.toast.vertical > .ui.image + .content,
.ui.toast.vertical > i.icon:not(.close) + .content {
  padding-left: 1em;
}

.ui.toast.vertical > .ui.image {
  align-self: flex-start;
  flex-shrink: 0;
  /* IE11 fix */
}

.ui.toast.vertical > .centered.image,
.ui.toast.vertical > .centered.icon {
  align-self: center;
}

.ui.toast.attached.bottom {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.ui.toast.attached.top {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.ui.toast.ui.ui.ui.image {
  padding: 0;
}

.ui.toast.ui.ui.ui.image > .content {
  padding-top: 0.78571429em;
  padding-bottom: 0.78571429em;
  padding-right: 1em;
}

.ui.toast.ui.ui.ui.image > .actions {
  margin: 0;
}

.ui.toast.ui.ui.ui.image > .ui.image {
  border-top-left-radius: 0.28571429rem;
  border-bottom-left-radius: 0.28571429rem;
}

.ui.toast.ui.ui.ui.image > .ui.image.mini {
  min-width: calc(0.7857142857142857em + 35px);
}

.ui.toast.ui.ui.ui.image > .ui.image.mini + .content {
  min-height: calc(0.7857142857142857em + 35px);
  padding-left: 4.4em;
}

.ui.hoverfloating.message:hover {
  box-shadow: 0 0 0 1px inset, 0 2px 4px 0 rgba(34, 36, 38, 0.12), 0 2px 10px 0 rgba(34, 36, 38, 0.15);
}

.ui.center.toast-container .toast-box,
.ui.right.toast-container .toast-box {
  margin-left: auto;
}

.ui.center.toast-container .toast-box {
  margin-right: auto;
}

.ui.ui.toast-container .toast-box .centered.toast,
.ui.ui.toast-container .toast-box .center.aligned.toast {
  text-align: center;
  display: flex;
  justify-content: center;
}

.ui.ui.toast-container .toast-box .centered.toast > .content,
.ui.ui.toast-container .toast-box .center.aligned.toast > .content,
.ui.ui.toast-container .toast-box .centered.toast > .ui.image,
.ui.ui.toast-container .toast-box .center.aligned.toast > .ui.image,
.ui.ui.toast-container .toast-box .centered.toast > i.icon:not(.close),
.ui.ui.toast-container .toast-box .center.aligned.toast > i.icon:not(.close) {
  align-self: center;
}

.ui.toast-container .toast-box .toast .centered.content,
.ui.toast-container .toast-box .toast .center.aligned.content {
  text-align: center;
}

.ui.toast-container .toast-box .centered.actions,
.ui.toast-container .toast-box .center.aligned.actions {
  text-align: center;
}

.ui.toast-container .toast-box .centered.actions:not(.attached) > .button:not(.fluid),
.ui.toast-container .toast-box .center.aligned.actions:not(.attached) > .button:not(.fluid) {
  margin-left: 0.25em;
  margin-right: 0.25em;
}

/* --------------
     Colors
-------------- */

.ui.primary.toast {
  background-color: #2185d0;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.primary.toast,
.ui.toast-container .toast-box > .inverted.primary.attached.progress .bar {
  background-color: #54c8ff;
  color: rgba(0, 0, 0, 0.87);
}

.ui.secondary.toast {
  background-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.secondary.toast,
.ui.toast-container .toast-box > .inverted.secondary.attached.progress .bar {
  background-color: #545454;
  color: rgba(0, 0, 0, 0.87);
}

.ui.red.toast {
  background-color: #db2828;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.red.toast,
.ui.toast-container .toast-box > .inverted.red.attached.progress .bar {
  background-color: #ff695e;
  color: rgba(0, 0, 0, 0.87);
}

.ui.orange.toast {
  background-color: #f2711c;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.orange.toast,
.ui.toast-container .toast-box > .inverted.orange.attached.progress .bar {
  background-color: #ff851b;
  color: rgba(0, 0, 0, 0.87);
}

.ui.yellow.toast {
  background-color: #fbbd08;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.yellow.toast,
.ui.toast-container .toast-box > .inverted.yellow.attached.progress .bar {
  background-color: #ffe21f;
  color: rgba(0, 0, 0, 0.87);
}

.ui.olive.toast {
  background-color: #b5cc18;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.olive.toast,
.ui.toast-container .toast-box > .inverted.olive.attached.progress .bar {
  background-color: #d9e778;
  color: rgba(0, 0, 0, 0.87);
}

.ui.green.toast {
  background-color: #21ba45;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.green.toast,
.ui.toast-container .toast-box > .inverted.green.attached.progress .bar {
  background-color: #2ecc40;
  color: rgba(0, 0, 0, 0.87);
}

.ui.teal.toast {
  background-color: #00b5ad;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.teal.toast,
.ui.toast-container .toast-box > .inverted.teal.attached.progress .bar {
  background-color: #6dffff;
  color: rgba(0, 0, 0, 0.87);
}

.ui.blue.toast {
  background-color: #2185d0;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.blue.toast,
.ui.toast-container .toast-box > .inverted.blue.attached.progress .bar {
  background-color: #54c8ff;
  color: rgba(0, 0, 0, 0.87);
}

.ui.violet.toast {
  background-color: #6435c9;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.violet.toast,
.ui.toast-container .toast-box > .inverted.violet.attached.progress .bar {
  background-color: #a291fb;
  color: rgba(0, 0, 0, 0.87);
}

.ui.purple.toast {
  background-color: #a333c8;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.purple.toast,
.ui.toast-container .toast-box > .inverted.purple.attached.progress .bar {
  background-color: #dc73ff;
  color: rgba(0, 0, 0, 0.87);
}

.ui.pink.toast {
  background-color: #e03997;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.pink.toast,
.ui.toast-container .toast-box > .inverted.pink.attached.progress .bar {
  background-color: #ff8edf;
  color: rgba(0, 0, 0, 0.87);
}

.ui.brown.toast {
  background-color: #a5673f;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.brown.toast,
.ui.toast-container .toast-box > .inverted.brown.attached.progress .bar {
  background-color: #d67c1c;
  color: rgba(0, 0, 0, 0.87);
}

.ui.grey.toast {
  background-color: #767676;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.grey.toast,
.ui.toast-container .toast-box > .inverted.grey.attached.progress .bar {
  background-color: #dcddde;
  color: rgba(0, 0, 0, 0.87);
}

.ui.black.toast {
  background-color: #1b1c1d;
  color: rgba(255, 255, 255, 0.9);
}

.ui.inverted.black.toast,
.ui.toast-container .toast-box > .inverted.black.attached.progress .bar {
  background-color: #545454;
  color: rgba(0, 0, 0, 0.87);
}

.ui.inverted.toast {
  color: rgba(255, 255, 255, 0.9);
  background-color: #1b1c1d;
}

@media only screen and (max-width: 420px) {
  .ui.toast-container .toast-box.toast-box,
  .ui.toast-container .toast-box > .compact,
  .ui.toast-container .toast-box > .vertical > *,
  .ui.toast-container .toast-box > * {
    width: auto;
    max-width: 100%;
  }

  .ui.toast-container .toast-box > *:not(.vertical) {
    min-width: 280px;
  }

  .ui.toast-container .toast-box > .ui.card.horizontal,
  .ui.toast-container .toast-box > .vertical > .ui.horizontal.card {
    min-width: initial;
  }
}

/* ---------------
   Progress Bar
 ---------------- */

@keyframes progressDown {
  0% {
    width: 100%;
  }

  100% {
    width: 0;
  }
}

@keyframes progressUp {
  0% {
    width: 0;
  }

  100% {
    width: 100%;
  }
}

@keyframes progressWait {
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
}

/*******************************
         Theme Overrides
*******************************/

/*******************************
        User Overrides
*******************************/
/*!
 * # Fomantic-UI 2.9.3 - Transition
 * https://github.com/fomantic/Fomantic-UI/
 *
 *
 * Released under the MIT license
 * https://opensource.org/licenses/MIT
 *
 */

/*******************************
          Transitions
*******************************/

.transition {
  animation-iteration-count: 1;
  animation-duration: 300ms;
  animation-timing-function: ease;
  animation-fill-mode: both;
}

/*******************************
            States
*******************************/

/* Animating */

.animating.transition {
  -webkit-backface-visibility: hidden;
  backface-visibility: hidden;
  visibility: visible !important;
}

/* Loading */

.loading.transition {
  position: absolute;
  top: -99999px;
  left: -99999px;
}

/* Hidden */

.hidden.transition {
  display: none;
  visibility: hidden;
}

/* Visible */

.visible.transition {
  display: block !important;
  visibility: visible !important;
  /* backface-visibility: @backfaceVisibility;
      transform: @use3DAcceleration; */
}

/* Disabled */

.disabled.transition {
  animation-play-state: paused;
}

/*******************************
          Variations
*******************************/

.looping.transition {
  animation-iteration-count: infinite;
}

/* Pulsating */

.pulsating.transition {
  animation-name: pulsating;
  animation-duration: 2000ms;
  box-shadow: 0 0 0 0 rgba(128, 128, 128, 0.5);
}

.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(255, 255, 255, 0.7);
}

@keyframes pulsating {
  100% {
    box-shadow: 0 0 0 0.8rem rgba(255, 255, 255, 0);
  }
}

.primary.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(33, 133, 208, 0.5);
}

.primary.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(84, 200, 255, 0.7);
}

.secondary.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(27, 28, 29, 0.5);
}

.secondary.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(84, 84, 84, 0.7);
}

.red.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(219, 40, 40, 0.5);
}

.red.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(255, 105, 94, 0.7);
}

.orange.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(242, 113, 28, 0.5);
}

.orange.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(255, 133, 27, 0.7);
}

.yellow.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(251, 189, 8, 0.5);
}

.yellow.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(255, 226, 31, 0.7);
}

.olive.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(181, 204, 24, 0.5);
}

.olive.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(217, 231, 120, 0.7);
}

.green.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(33, 186, 69, 0.5);
}

.green.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(46, 204, 64, 0.7);
}

.teal.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(0, 181, 173, 0.5);
}

.teal.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(109, 255, 255, 0.7);
}

.blue.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(33, 133, 208, 0.5);
}

.blue.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(84, 200, 255, 0.7);
}

.violet.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(100, 53, 201, 0.5);
}

.violet.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(162, 145, 251, 0.7);
}

.purple.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(163, 51, 200, 0.5);
}

.purple.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(220, 115, 255, 0.7);
}

.pink.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(224, 57, 151, 0.5);
}

.pink.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(255, 142, 223, 0.7);
}

.brown.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(165, 103, 63, 0.5);
}

.brown.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(214, 124, 28, 0.7);
}

.grey.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(118, 118, 118, 0.5);
}

.grey.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(220, 221, 222, 0.7);
}

.black.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(27, 28, 29, 0.5);
}

.black.inverted.pulsating.transition {
  box-shadow: 0 0 0 0 rgba(84, 84, 84, 0.7);
}

/*******************************
          Transitions
*******************************/

/*
  Some transitions adapted from Animate CSS
  https://github.com/daneden/animate.css

  Additional transitions adapted from Glide
  by Nick Pettit - https://github.com/nickpettit/glide
*/

/* --------------
         Browse
    --------------- */

.transition.browse {
  animation-duration: 500ms;
}

.transition.browse.in {
  animation-name: browseIn;
}

.transition.browse.out,
.transition.browse.left.out {
  animation-name: browseOutLeft;
}

.transition.browse.right.out {
  animation-name: browseOutRight;
}

/* In */

@keyframes browseIn {
  0% {
    transform: scale(0.8) translateZ(0);
    z-index: -1;
  }

  10% {
    transform: scale(0.8) translateZ(0);
    z-index: -1;
    opacity: 0.7;
  }

  80% {
    transform: scale(1.05) translateZ(0);
    opacity: 1;
    z-index: 999;
  }

  100% {
    transform: scale(1) translateZ(0);
    z-index: 999;
  }
}

/* Out */

@keyframes browseOutLeft {
  0% {
    z-index: 999;
    transform: translateX(0) rotateY(0) rotateX(0);
  }

  50% {
    z-index: -1;
    transform: translateX(-105%) rotateY(35deg) rotateX(10deg) translateZ(-10px);
  }

  80% {
    opacity: 1;
  }

  100% {
    z-index: -1;
    transform: translateX(0) rotateY(0) rotateX(0) translateZ(-10px);
    opacity: 0;
  }
}

@keyframes browseOutRight {
  0% {
    z-index: 999;
    transform: translateX(0) rotateY(0) rotateX(0);
  }

  50% {
    z-index: 1;
    transform: translateX(105%) rotateY(35deg) rotateX(10deg) translateZ(-10px);
  }

  80% {
    opacity: 1;
  }

  100% {
    z-index: 1;
    transform: translateX(0) rotateY(0) rotateX(0) translateZ(-10px);
    opacity: 0;
  }
}

/* --------------
         Drop
    --------------- */

.drop.transition {
  transform-origin: top center;
  animation-duration: 400ms;
  animation-timing-function: cubic-bezier(0.34, 1.61, 0.7, 1);
}

.drop.transition.in {
  animation-name: dropIn;
}

.drop.transition.out {
  animation-name: dropOut;
}

/* Drop */

@keyframes dropIn {
  0% {
    opacity: 0;
    transform: scale(0);
  }

  100% {
    opacity: 1;
    transform: scale(1);
  }
}

@keyframes dropOut {
  0% {
    opacity: 1;
    transform: scale(1);
  }

  100% {
    opacity: 0;
    transform: scale(0);
  }
}

/* --------------
          Fade
    --------------- */

.transition.fade.in {
  animation-name: fadeIn;
}

.transition[class*="fade up"].in {
  animation-name: fadeInUp;
}

.transition[class*="fade down"].in {
  animation-name: fadeInDown;
}

.transition[class*="fade left"].in {
  animation-name: fadeInLeft;
}

.transition[class*="fade right"].in {
  animation-name: fadeInRight;
}

.transition.fade.out {
  animation-name: fadeOut;
}

.transition[class*="fade up"].out {
  animation-name: fadeOutUp;
}

.transition[class*="fade down"].out {
  animation-name: fadeOutDown;
}

.transition[class*="fade left"].out {
  animation-name: fadeOutLeft;
}

.transition[class*="fade right"].out {
  animation-name: fadeOutRight;
}

/* In */

@keyframes fadeIn {
  0% {
    opacity: 0;
  }

  100% {
    opacity: 1;
  }
}

@keyframes fadeInUp {
  0% {
    opacity: 0;
    transform: translateY(10%);
  }

  100% {
    opacity: 1;
    transform: translateY(0);
  }
}

@keyframes fadeInDown {
  0% {
    opacity: 0;
    transform: translateY(-10%);
  }

  100% {
    opacity: 1;
    transform: translateY(0);
  }
}

@keyframes fadeInLeft {
  0% {
    opacity: 0;
    transform: translateX(10%);
  }

  100% {
    opacity: 1;
    transform: translateX(0);
  }
}

@keyframes fadeInRight {
  0% {
    opacity: 0;
    transform: translateX(-10%);
  }

  100% {
    opacity: 1;
    transform: translateX(0);
  }
}

/* Out */

@keyframes fadeOut {
  0% {
    opacity: 1;
  }

  100% {
    opacity: 0;
  }
}

@keyframes fadeOutUp {
  0% {
    opacity: 1;
    transform: translateY(0);
  }

  100% {
    opacity: 0;
    transform: translateY(5%);
  }
}

@keyframes fadeOutDown {
  0% {
    opacity: 1;
    transform: translateY(0);
  }

  100% {
    opacity: 0;
    transform: translateY(-5%);
  }
}

@keyframes fadeOutLeft {
  0% {
    opacity: 1;
    transform: translateX(0);
  }

  100% {
    opacity: 0;
    transform: translateX(5%);
  }
}

@keyframes fadeOutRight {
  0% {
    opacity: 1;
    transform: translateX(0);
  }

  100% {
    opacity: 0;
    transform: translateX(-5%);
  }
}

/* --------------
         Flips
    --------------- */

.flip.transition.in,
.flip.transition.out {
  animation-duration: 600ms;
}

.horizontal.flip.transition.in {
  animation-name: horizontalFlipIn;
}

.horizontal.flip.transition.out {
  animation-name: horizontalFlipOut;
}

.vertical.flip.transition.in {
  animation-name: verticalFlipIn;
}

.vertical.flip.transition.out {
  animation-name: verticalFlipOut;
}

/* In */

@keyframes horizontalFlipIn {
  0% {
    transform: perspective(2000px) rotateY(-90deg);
    opacity: 0;
  }

  100% {
    transform: perspective(2000px) rotateY(0);
    opacity: 1;
  }
}

@keyframes verticalFlipIn {
  0% {
    transform: perspective(2000px) rotateX(-90deg);
    opacity: 0;
  }

  100% {
    transform: perspective(2000px) rotateX(0);
    opacity: 1;
  }
}

/* Out */

@keyframes horizontalFlipOut {
  0% {
    transform: perspective(2000px) rotateY(0);
    opacity: 1;
  }

  100% {
    transform: perspective(2000px) rotateY(90deg);
    opacity: 0;
  }
}

@keyframes verticalFlipOut {
  0% {
    transform: perspective(2000px) rotateX(0);
    opacity: 1;
  }

  100% {
    transform: perspective(2000px) rotateX(-90deg);
    opacity: 0;
  }
}

/* --------------
          Scale
    --------------- */

.scale.transition.in {
  animation-name: scaleIn;
}

.scale.transition.out {
  animation-name: scaleOut;
}

@keyframes scaleIn {
  0% {
    opacity: 0;
    transform: scale(0.8);
  }

  100% {
    opacity: 1;
    transform: scale(1);
  }
}

/* Out */

@keyframes scaleOut {
  0% {
    opacity: 1;
    transform: scale(1);
  }

  100% {
    opacity: 0;
    transform: scale(0.9);
  }
}

/* --------------
          Fly
    --------------- */

/* Inward */

.transition.fly {
  animation-duration: 0.6s;
  transition-timing-function: cubic-bezier(0.215, 0.61, 0.355, 1);
}

.transition.fly.in {
  animation-name: flyIn;
}

.transition[class*="fly up"].in {
  animation-name: flyInUp;
}

.transition[class*="fly down"].in {
  animation-name: flyInDown;
}

.transition[class*="fly left"].in {
  animation-name: flyInLeft;
}

.transition[class*="fly right"].in {
  animation-name: flyInRight;
}

/* Outward */

.transition.fly.out {
  animation-name: flyOut;
}

.transition[class*="fly up"].out {
  animation-name: flyOutUp;
}

.transition[class*="fly down"].out {
  animation-name: flyOutDown;
}

.transition[class*="fly left"].out {
  animation-name: flyOutLeft;
}

.transition[class*="fly right"].out {
  animation-name: flyOutRight;
}

/* In */

@keyframes flyIn {
  0% {
    opacity: 0;
    transform: scale3d(0.3, 0.3, 0.3);
  }

  20% {
    transform: scale3d(1.1, 1.1, 1.1);
  }

  40% {
    transform: scale3d(0.9, 0.9, 0.9);
  }

  60% {
    opacity: 1;
    transform: scale3d(1.03, 1.03, 1.03);
  }

  80% {
    transform: scale3d(0.97, 0.97, 0.97);
  }

  100% {
    opacity: 1;
    transform: scale3d(1, 1, 1);
  }
}

@keyframes flyInUp {
  0% {
    opacity: 0;
    transform: translate3d(0, 1500px, 0);
  }

  60% {
    opacity: 1;
    transform: translate3d(0, -20px, 0);
  }

  75% {
    transform: translate3d(0, 10px, 0);
  }

  90% {
    transform: translate3d(0, -5px, 0);
  }

  100% {
    transform: translate3d(0, 0, 0);
  }
}

@keyframes flyInDown {
  0% {
    opacity: 0;
    transform: translate3d(0, -1500px, 0);
  }

  60% {
    opacity: 1;
    transform: translate3d(0, 25px, 0);
  }

  75% {
    transform: translate3d(0, -10px, 0);
  }

  90% {
    transform: translate3d(0, 5px, 0);
  }

  100% {
    transform: none;
  }
}

@keyframes flyInLeft {
  0% {
    opacity: 0;
    transform: translate3d(1500px, 0, 0);
  }

  60% {
    opacity: 1;
    transform: translate3d(-25px, 0, 0);
  }

  75% {
    transform: translate3d(10px, 0, 0);
  }

  90% {
    transform: translate3d(-5px, 0, 0);
  }

  100% {
    transform: none;
  }
}

@keyframes flyInRight {
  0% {
    opacity: 0;
    transform: translate3d(-1500px, 0, 0);
  }

  60% {
    opacity: 1;
    transform: translate3d(25px, 0, 0);
  }

  75% {
    transform: translate3d(-10px, 0, 0);
  }

  90% {
    transform: translate3d(5px, 0, 0);
  }

  100% {
    transform: none;
  }
}

/* Out */

@keyframes flyOut {
  20% {
    transform: scale3d(0.9, 0.9, 0.9);
  }

  50%, 55% {
    opacity: 1;
    transform: scale3d(1.1, 1.1, 1.1);
  }

  100% {
    opacity: 0;
    transform: scale3d(0.3, 0.3, 0.3);
  }
}

@keyframes flyOutUp {
  20% {
    transform: translate3d(0, 10px, 0);
  }

  40%, 45% {
    opacity: 1;
    transform: translate3d(0, -20px, 0);
  }

  100% {
    opacity: 0;
    transform: translate3d(0, 2000px, 0);
  }
}

@keyframes flyOutDown {
  20% {
    transform: translate3d(0, -10px, 0);
  }

  40%, 45% {
    opacity: 1;
    transform: translate3d(0, 20px, 0);
  }

  100% {
    opacity: 0;
    transform: translate3d(0, -2000px, 0);
  }
}

@keyframes flyOutRight {
  20% {
    opacity: 1;
    transform: translate3d(20px, 0, 0);
  }

  100% {
    opacity: 0;
    transform: translate3d(-2000px, 0, 0);
  }
}

@keyframes flyOutLeft {
  20% {
    opacity: 1;
    transform: translate3d(-20px, 0, 0);
  }

  100% {
    opacity: 0;
    transform: translate3d(2000px, 0, 0);
  }
}

/* --------------
         Slide
    --------------- */

.transition.slide.in,
.transition[class*="slide down"].in {
  animation-name: slideInY;
  transform-origin: top center;
}

.transition[class*="slide up"].in {
  animation-name: slideInY;
  transform-origin: bottom center;
}

.transition[class*="slide left"].in {
  animation-name: slideInX;
  transform-origin: right center;
}

.transition[class*="slide right"].in {
  animation-name: slideInX;
  transform-origin: left center;
}

.transition.slide.out,
.transition[class*="slide down"].out {
  animation-name: slideOutY;
  transform-origin: top center;
}

.transition[class*="slide up"].out {
  animation-name: slideOutY;
  transform-origin: bottom center;
}

.transition[class*="slide left"].out {
  animation-name: slideOutX;
  transform-origin: right center;
}

.transition[class*="slide right"].out {
  animation-name: slideOutX;
  transform-origin: left center;
}

/* In */

@keyframes slideInY {
  0% {
    opacity: 0;
    transform: scaleY(0);
  }

  100% {
    opacity: 1;
    transform: scaleY(1);
  }
}

@keyframes slideInX {
  0% {
    opacity: 0;
    transform: scaleX(0);
  }

  100% {
    opacity: 1;
    transform: scaleX(1);
  }
}

/* Out */

@keyframes slideOutY {
  0% {
    opacity: 1;
    transform: scaleY(1);
  }

  100% {
    opacity: 0;
    transform: scaleY(0);
  }
}

@keyframes slideOutX {
  0% {
    opacity: 1;
    transform: scaleX(1);
  }

  100% {
    opacity: 0;
    transform: scaleX(0);
  }
}

/* --------------
         Swing
    --------------- */

.transition.swing {
  animation-duration: 800ms;
}

.transition[class*="swing down"].in {
  animation-name: swingInX;
  transform-origin: top center;
}

.transition[class*="swing up"].in {
  animation-name: swingInX;
  transform-origin: bottom center;
}

.transition[class*="swing left"].in {
  animation-name: swingInY;
  transform-origin: right center;
}

.transition[class*="swing right"].in {
  animation-name: swingInY;
  transform-origin: left center;
}

.transition.swing.out,
.transition[class*="swing down"].out {
  animation-name: swingOutX;
  transform-origin: top center;
}

.transition[class*="swing up"].out {
  animation-name: swingOutX;
  transform-origin: bottom center;
}

.transition[class*="swing left"].out {
  animation-name: swingOutY;
  transform-origin: right center;
}

.transition[class*="swing right"].out {
  animation-name: swingOutY;
  transform-origin: left center;
}

/* In */

@keyframes swingInX {
  0% {
    transform: perspective(1000px) rotateX(90deg);
    opacity: 0;
  }

  40% {
    transform: perspective(1000px) rotateX(-30deg);
    opacity: 1;
  }

  60% {
    transform: perspective(1000px) rotateX(15deg);
  }

  80% {
    transform: perspective(1000px) rotateX(-7.5deg);
  }

  100% {
    transform: perspective(1000px) rotateX(0);
  }
}

@keyframes swingInY {
  0% {
    transform: perspective(1000px) rotateY(-90deg);
    opacity: 0;
  }

  40% {
    transform: perspective(1000px) rotateY(30deg);
    opacity: 1;
  }

  60% {
    transform: perspective(1000px) rotateY(-17.5deg);
  }

  80% {
    transform: perspective(1000px) rotateY(7.5deg);
  }

  100% {
    transform: perspective(1000px) rotateY(0);
  }
}

/* Out */

@keyframes swingOutX {
  0% {
    transform: perspective(1000px) rotateX(0);
  }

  40% {
    transform: perspective(1000px) rotateX(-7.5deg);
  }

  60% {
    transform: perspective(1000px) rotateX(17.5deg);
  }

  80% {
    transform: perspective(1000px) rotateX(-30deg);
    opacity: 1;
  }

  100% {
    transform: perspective(1000px) rotateX(90deg);
    opacity: 0;
  }
}

@keyframes swingOutY {
  0% {
    transform: perspective(1000px) rotateY(0);
  }

  40% {
    transform: perspective(1000px) rotateY(7.5deg);
  }

  60% {
    transform: perspective(1000px) rotateY(-10deg);
  }

  80% {
    transform: perspective(1000px) rotateY(30deg);
    opacity: 1;
  }

  100% {
    transform: perspective(1000px) rotateY(-90deg);
    opacity: 0;
  }
}

/* --------------
          Zoom
    --------------- */

.transition.zoom.in {
  animation-name: zoomIn;
}

.transition.zoom.out {
  animation-name: zoomOut;
}

@keyframes zoomIn {
  0% {
    opacity: 1;
    transform: scale(0);
  }

  100% {
    opacity: 1;
    transform: scale(1);
  }
}

@keyframes zoomOut {
  0% {
    opacity: 1;
    transform: scale(1);
  }

  100% {
    opacity: 1;
    transform: scale(0);
  }
}

/*******************************
       Static Animations
*******************************/

/* --------------
    Emphasis
--------------- */

.flash.transition {
  animation-duration: 750ms;
  animation-name: flash;
}

.shake.transition {
  animation-duration: 750ms;
  animation-name: shake;
}

.bounce.transition {
  animation-duration: 750ms;
  animation-name: bounce;
}

.tada.transition {
  animation-duration: 750ms;
  animation-name: tada;
}

.pulse.transition {
  animation-duration: 500ms;
  animation-name: pulse;
}

.jiggle.transition {
  animation-duration: 750ms;
  animation-name: jiggle;
}

.transition.glow {
  animation-duration: 2000ms;
  animation-timing-function: cubic-bezier(0.19, 1, 0.22, 1);
}

.transition.glow {
  animation-name: glow;
}

/* Flash */

@keyframes flash {
  0%, 50%, 100% {
    opacity: 1;
  }

  25%, 75% {
    opacity: 0;
  }
}

/* Shake */

@keyframes shake {
  0%, 100% {
    transform: translateX(0);
  }

  10%, 30%, 50%, 70%, 90% {
    transform: translateX(-10px);
  }

  20%, 40%, 60%, 80% {
    transform: translateX(10px);
  }
}

/* Bounce */

@keyframes bounce {
  0%, 20%, 50%, 80%, 100% {
    transform: translateY(0);
  }

  40% {
    transform: translateY(-30px);
  }

  60% {
    transform: translateY(-15px);
  }
}

/* Tada */

@keyframes tada {
  0% {
    transform: scale(1);
  }

  10%, 20% {
    transform: scale(0.9) rotate(-3deg);
  }

  30%, 50%, 70%, 90% {
    transform: scale(1.1) rotate(3deg);
  }

  40%, 60%, 80% {
    transform: scale(1.1) rotate(-3deg);
  }

  100% {
    transform: scale(1) rotate(0);
  }
}

/* Pulse */

@keyframes pulse {
  0% {
    transform: scale(1);
    opacity: 1;
  }

  50% {
    transform: scale(0.9);
    opacity: 0.7;
  }

  100% {
    transform: scale(1);
    opacity: 1;
  }
}

/* Jiggle */

@keyframes jiggle {
  0% {
    transform: scale3d(1, 1, 1);
  }

  30% {
    transform: scale3d(1.25, 0.75, 1);
  }

  40% {
    transform: scale3d(0.75, 1.25, 1);
  }

  50% {
    transform: scale3d(1.15, 0.85, 1);
  }

  65% {
    transform: scale3d(0.95, 1.05, 1);
  }

  75% {
    transform: scale3d(1.05, 0.95, 1);
  }

  100% {
    transform: scale3d(1, 1, 1);
  }
}

/* Glow */

@keyframes glow {
  0% {
    background-color: #fcfcfd;
  }

  30% {
    background-color: #fff6cd;
  }

  100% {
    background-color: #fcfcfd;
  }
}

/*******************************
         Site Overrides
*******************************/
